local cv_placemode = CV_RegisterVar({"placemode", "Off", CV_NETVAR, {Off = 0, Manual = 1, Auto = 2}})
local cv_placedist = CV_RegisterVar({"placedist", "1024", CV_NETVAR, {MIN=128, MAX=4096}})
local spawnplacer = {}

local function placeSpawnpoint(p)
	if p.mo == nil
		print("You need a player object to place spawnpoints.")
		return
	end
	local newspawn = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_XMASPOLE)
	newspawn.order = p.starpostnum
	p.krplace.lastid = p.krplace.id
	newspawn.id = p.krplace.lastid
	p.krplace.lastx = newspawn.x
	p.krplace.lasty = newspawn.y
	p.krplace.lastz = newspawn.z
	p.krplace.id = $1 + 1
	newspawn.angle = p.mo.angle
	newspawn.flags = MF_NOGRAVITY|MF_NOCLIPHEIGHT|MF_SCENERY
	newspawn.sprite = SPR_POSS
	newspawn.frame = 0
	table.insert(spawnplacer.spawnpoints, newspawn)
end

local function dumpMyWaypoints(p)
	if #spawnplacer.spawnpoints < 16
		print("You need at least 16 spawn points. Don't be afraid to litter the map with them.")
		return
	end
	print("-- Leave this if statement in your lua file at least once.")
	print("if krespawn == nil")
	print("  rawset(_G, \"krespawn\", {})")
	print("  krespawn[\"points\"] = {}")
	print("end")
	print("-- Respawn definitions for " .. mapheaderinfo[gamemap].lvlttl .. " (" .. G_BuildMapName(gamemap) .. ")")
	print("krespawn.points[" .. tostring(gamemap) .. "] = {")
	print("  [0] = " .. tostring(#spawnplacer.spawnpoints) .. ",")
	for w = 1,#spawnplacer.spawnpoints
		local endstr = "  [" .. tostring(w) .. "] = {[\"x\"] = " .. tostring(spawnplacer.spawnpoints[w].x/FRACUNIT) ..", [\"y\"] = " .. tostring(spawnplacer.spawnpoints[w].y/FRACUNIT) .. ", [\"z\"] = " .. tostring(spawnplacer.spawnpoints[w].z/FRACUNIT) .. ", [\"order\"] = " .. tostring(spawnplacer.spawnpoints[w].order) .. ", [\"angle\"] = " .. tostring(spawnplacer.spawnpoints[w].angle) .. "}"
		if w ~= #spawnplacer.spawnpoints
			endstr = $1 .. ","
		end
		print(endstr)
	end
	print("}")
	print("===========EOF===========")
	print("Check log.txt for the full output. You can save this into a lua file as a definition for the course.")
end

COM_AddCommand("dumpwaypoints", dumpMyWaypoints, 1)

addHook("ThinkFrame", do
	for p in players.iterate do
		if p == players[0] --This is only usable by the local server.
			if p.mo ~= nil --And only if they are actually in the game.
				if p.krplace == nil
					p.krplace = {}
					p.krplace.lastx = 32767*FRACUNIT
					p.krplace.lasty = 32767*FRACUNIT
					p.krplace.lastz = 0
					p.krplace.lastid = 0
					p.krplace.id = 1
					p.krplace.canplace = false
				end
				if cv_placemode.value == 1
					if p.cmd.buttons & BT_ATTACK
						if p.krplace.canplace == true
							placeSpawnpoint(p)
							p.krplace.canplace = false
						end
					else
						p.krplace.canplace = true
					end
				elseif cv_placemode.value == 2
					if p.cmd.buttons & BT_ATTACK
						local a
					else
						local dist = P_AproxDistance(abs(p.mo.x - p.krplace.lastx), abs(p.mo.y - p.krplace.lasty))/FRACUNIT
						if dist > cv_placedist.value
						or p.krplace.lastid == 0
							placeSpawnpoint(p)
						end
					end
				end
			end
		end
	end
end)

addHook("MapLoad", do
	for sec in sectors.iterate do
		for mobj in sec.thinglist()
			if mobj.type == MT_RANDOMITEM
				mobj.flags = MF_SLIDEME|MF_NOGRAVITY|MF_NOCLIPHEIGHT|MF_DONTENCOREMAP --Remove MF_SPECIAL to stop players from popping it.
			end
		end
	end
	spawnplacer.spawnpoints = {}
	players[0].krplace.lastid = 0
	players[0].krplace.id = 1
end)
	

hud.add(function(v, p)
	local mode = cv_placemode.value
	local modestr
	if mode == 0
		modestr = "Off"
	elseif mode == 1
		modestr = "Manual"
	elseif mode == 2
		modestr = "Auto"
	else
		modestr = "Unknown"
	end
	if mode == 2
	and p.cmd.buttons & BT_ATTACK
		modestr = $1 .. " (Paused)"
	end
	v.drawString(0,128, "Mode: " .. modestr)
	if mode == 1
	or mode == 2
		if p.mo ~= nil
			v.drawString(0,136, "- Next placement - Checkpoint value: " .. p.starpostnum)
			v.drawString(0,144, "ID:" .. tostring(p.krplace.id) .. " x:" .. tostring(p.mo.x/FRACUNIT) .. " y:" .. tostring(p.mo.y/FRACUNIT) .. " z:" .. tostring(p.mo.z/FRACUNIT))
		end
		if p.krplace.lastid > 0
			v.drawString(0,152, "- Recent placement -")
			v.drawString(0,160, "ID:" .. tostring(p.krplace.lastid) .. " x:" .. tostring(p.krplace.lastx/FRACUNIT) .. " y:" .. tostring(p.krplace.lasty/FRACUNIT) .. " z:" .. tostring(p.krplace.lastz/FRACUNIT))
		end
	end
end, "game")