--Initialization--

if krespawn == nil
	rawset(_G, "krespawn", {})
	krespawn["points"] = {}
end
krespawn["racepoints"] = {}
krespawn["valid"] = false

--Functions--
local function rescueMe(p)
	if p.mo ~= nil
	and p.mo.health > 0
		local nextspot = p.lastrescue - 1
		if nextspot < 0
			nextspot = #krespawn.racepoints
			p.laps = $1 - 1
		end
		P_TeleportMove(p.mo, krespawn.racepoints[nextspot].x, krespawn.racepoints[nextspot].y, (krespawn.racepoints[nextspot].x + 96*FRACUNIT))
		p.mo.angle = closest.spawnangle
		p.starpostnum = krespawn.racepoints[nextspot].order
		--print(p.name .. " rescued back to respawn point " .. tostring(nextspot) .. ".")
	else
		CONS_Printf(p, "You need to have a living character to do this!")
	end
end

--Commands--
COM_AddCommand("rescue", rescueMe, 0)

--Variables--
local cv_restrictspawns = CV_RegisterVar({"br_restrictspawns", "On", CV_NETVAR, {Off=0, On=1}})
local cv_allowrescue = CV_RegisterVar({"br_allowrescue", "On", CV_NETVAR, {Off=0, On=1}})

--Handles Map-based definitions.
addHook("MapLoad", do
	krespawn.valid = false
	krespawn.racepoints = {}
	if krespawn.points[gamemap] == nil
		print("Using map-based respawn points.")
		local maprespawns = {}
		for i=0,(#mapthings - 1) do
			local thing = mapthings[i]
			if thing.mobj ~= nil
				local mobj = thing.mobj
				if mobj.type == MT_BOSS9GATHERPOINT --All this is to check for only objects we want.
					--print("Add thing ID " .. tostring(i))
					local g = thing.angle
					local ord = 0
					while g > 359 do
						g = $1 - 360
						ord = $1 + 1
					end
					mobj.order = ord
					mobj.spawnangle = FixedAngle(g*FRACUNIT)
					table.insert(krespawn.racepoints, mobj)
				end
			end
		end
	else
		print("Using lua-stored respawn points.")
		for a=1,krespawn.points[gamemap][0] do
			if krespawn.points[gamemap][a] == nil then return end --Failsafe for lack of data.
			local newobj = P_SpawnMobj(krespawn.points[gamemap][a].x*FRACUNIT, krespawn.points[gamemap][a].y*FRACUNIT, krespawn.points[gamemap][a].z*FRACUNIT, MT_BOSS9GATHERPOINT)
			newobj.order = krespawn.points[gamemap][a].order
			newobj.spawnangle = krespawn.points[gamemap][a].angle
			print(tostring(krespawn.points[gamemap][a].angle))
			table.insert(krespawn.racepoints, newobj)
		end
	end
	if #krespawn.racepoints < 1
		print("There are no respawn points.")
	else
		print("There are " .. tostring(#krespawn.racepoints) .. " respawn points.")
		krespawn.valid = true
	end
	for p in players.iterate do
		p.lastrescue = 0
	end
end)

--Handles respawning
addHook("ThinkFrame", do
	if krespawn.valid
		for p in players.iterate do
			if p.lastrescue == nil
				p.lastrescue = 0
			end
			if p.mo ~= nil
				if p.playerstate == PST_DEAD
					p.deadx = p.mo.x
					p.deady = p.mo.y
				end
			end
			if p.readytomove == nil
				p.readytomove = false
			end
			if p.playerstate == PST_REBORN
				p.readytomove = true
			else
				if p.readytomove == true
					if p.mo ~= nil
						local closest = krespawn.racepoints[1]
						local closedistance = 2147483647
						local closeid = 1
						for r=1,#krespawn.racepoints do
							if krespawn.racepoints[r].order == p.starpostnum
							and cv_restrictspawns.value == 1
								local dist = P_AproxDistance(abs(p.deadx - krespawn.racepoints[r].x), abs(p.deady - krespawn.racepoints[r].y))
								--print("Distance: " .. tostring(dist))
								if dist < closedistance
									closest = krespawn.racepoints[r]
									closedistance = dist
									closeid = r
								end
							end
						end
						--CONS_Printf(p, "Closest is " .. tostring(closeid))
						P_TeleportMove(p.mo, closest.x, closest.y, (closest.z + 96*FRACUNIT))
						p.mo.angle = closest.spawnangle
						p.readytomove = false
						p.lastrescue = closeid
					end
				end
			end
		end
	end
end)

-- Respawn definitions for GREEN HILLS (MAP01)
krespawn.points[1] = {
  [0] = 51,
  [1] = {["x"] = 493, ["y"] = 1044, ["z"] = 0, ["order"] = 0, ["angle"] = 1046020096},
  [2] = {["x"] = 566, ["y"] = 1753, ["z"] = 0, ["order"] = 0, ["angle"] = 876281856},
  [3] = {["x"] = 956, ["y"] = 2755, ["z"] = 0, ["order"] = 0, ["angle"] = 765788160},
  [4] = {["x"] = 1445, ["y"] = 3840, ["z"] = 0, ["order"] = 0, ["angle"] = 1096876032},
  [5] = {["x"] = 1681, ["y"] = 5121, ["z"] = 49, ["order"] = 0, ["angle"] = 1672675328},
  [6] = {["x"] = 1169, ["y"] = 6171, ["z"] = 39, ["order"] = 1, ["angle"] = 2055929856},
  [7] = {["x"] = -52, ["y"] = 6568, ["z"] = 6, ["order"] = 1, ["angle"] = -2035089408},
  [8] = {["x"] = -943, ["y"] = 6556, ["z"] = 0, ["order"] = 1, ["angle"] = -1999831040},
  [9] = {["x"] = -1702, ["y"] = 6291, ["z"] = 18, ["order"] = 1, ["angle"] = -1885536256},
  [10] = {["x"] = -2712, ["y"] = 5907, ["z"] = 252, ["order"] = 1, ["angle"] = -1959198720},
  [11] = {["x"] = -3711, ["y"] = 5702, ["z"] = 374, ["order"] = 1, ["angle"] = -2004549632},
  [12] = {["x"] = -4176, ["y"] = 5407, ["z"] = 400, ["order"] = 1, ["angle"] = -2029977600},
  [13] = {["x"] = -4850, ["y"] = 5273, ["z"] = 400, ["order"] = 1, ["angle"] = 2129854464},
  [14] = {["x"] = -5714, ["y"] = 5263, ["z"] = 438, ["order"] = 1, ["angle"] = 2129854464},
  [15] = {["x"] = -6673, ["y"] = 5220, ["z"] = 473, ["order"] = 1, ["angle"] = -1617231872},
  [16] = {["x"] = -7455, ["y"] = 4786, ["z"] = 466, ["order"] = 1, ["angle"] = -952172544},
  [17] = {["x"] = -7596, ["y"] = 3892, ["z"] = 486, ["order"] = 2, ["angle"] = -254214144},
  [18] = {["x"] = -6486, ["y"] = 3438, ["z"] = 460, ["order"] = 2, ["angle"] = -10813440},
  [19] = {["x"] = -5281, ["y"] = 3380, ["z"] = 386, ["order"] = 2, ["angle"] = -144637952},
  [20] = {["x"] = -4188, ["y"] = 3437, ["z"] = 300, ["order"] = 2, ["angle"] = -468779008},
  [21] = {["x"] = -3274, ["y"] = 2858, ["z"] = 300, ["order"] = 3, ["angle"] = -963837952},
  [22] = {["x"] = -2872, ["y"] = 1581, ["z"] = 250, ["order"] = 3, ["angle"] = -1030422528},
  [23] = {["x"] = -2860, ["y"] = 1, ["z"] = 24, ["order"] = 3, ["angle"] = -1054408704},
  [24] = {["x"] = -2842, ["y"] = -1481, ["z"] = -100, ["order"] = 3, ["angle"] = -1075904512},
  [25] = {["x"] = -2845, ["y"] = -3035, ["z"] = -100, ["order"] = 3, ["angle"] = -1075904512},
  [26] = {["x"] = -2849, ["y"] = -4194, ["z"] = -100, ["order"] = 3, ["angle"] = -1075904512},
  [27] = {["x"] = -2978, ["y"] = -5520, ["z"] = -100, ["order"] = 3, ["angle"] = -336527360},
  [28] = {["x"] = -2024, ["y"] = -6082, ["z"] = -83, ["order"] = 4, ["angle"] = 83165184},
  [29] = {["x"] = -908, ["y"] = -6029, ["z"] = 90, ["order"] = 4, ["angle"] = 87883776},
  [30] = {["x"] = 711, ["y"] = -5769, ["z"] = 100, ["order"] = 4, ["angle"] = 303628288},
  [31] = {["x"] = 1773, ["y"] = -5102, ["z"] = 100, ["order"] = 4, ["angle"] = 217907200},
  [32] = {["x"] = 2739, ["y"] = -4673, ["z"] = 100, ["order"] = 5, ["angle"] = -27459584},
  [33] = {["x"] = 4020, ["y"] = -4772, ["z"] = 100, ["order"] = 5, ["angle"] = 85393408},
  [34] = {["x"] = 5030, ["y"] = -4648, ["z"] = 100, ["order"] = 5, ["angle"] = 332201984},
  [35] = {["x"] = 6210, ["y"] = -3596, ["z"] = 211, ["order"] = 5, ["angle"] = -110297088},
  [36] = {["x"] = 7425, ["y"] = -3861, ["z"] = 400, ["order"] = 6, ["angle"] = -1119420416},
  [37] = {["x"] = 7017, ["y"] = -5201, ["z"] = 114, ["order"] = 6, ["angle"] = -1085603840},
  [38] = {["x"] = 6993, ["y"] = -6480, ["z"] = 100, ["order"] = 6, ["angle"] = -1093337088},
  [39] = {["x"] = 7075, ["y"] = -8063, ["z"] = 160, ["order"] = 6, ["angle"] = -1532821504},
  [40] = {["x"] = 6121, ["y"] = -9055, ["z"] = 275, ["order"] = 6, ["angle"] = 2139160576},
  [41] = {["x"] = 5142, ["y"] = -9055, ["z"] = 438, ["order"] = 7, ["angle"] = 2083979264},
  [42] = {["x"] = 3967, ["y"] = -8955, ["z"] = 622, ["order"] = 7, ["angle"] = 1686306816},
  [43] = {["x"] = 3156, ["y"] = -8861, ["z"] = 754, ["order"] = 8, ["angle"] = 1191116800},
  [44] = {["x"] = 2964, ["y"] = -7298, ["z"] = 700, ["order"] = 8, ["angle"] = 1070137344},
  [45] = {["x"] = 2952, ["y"] = -6232, ["z"] = 700, ["order"] = 8, ["angle"] = 1070137344},
  [46] = {["x"] = 2955, ["y"] = -5288, ["z"] = 700, ["order"] = 8, ["angle"] = 1070137344},
  [47] = {["x"] = 2959, ["y"] = -4445, ["z"] = 700, ["order"] = 8, ["angle"] = 1070137344},
  [48] = {["x"] = 2978, ["y"] = -3304, ["z"] = 699, ["order"] = 9, ["angle"] = 1519190016},
  [49] = {["x"] = 1275, ["y"] = -1701, ["z"] = 249, ["order"] = 9, ["angle"] = 1330839552},
  [50] = {["x"] = 538, ["y"] = -668, ["z"] = 33, ["order"] = 9, ["angle"] = 1085079552},
  [51] = {["x"] = 520, ["y"] = 297, ["z"] = -5, ["order"] = 9, ["angle"] = 1094123520}
}
-- Respawn definitions for DARK RACE (MAP02)
krespawn.points[2] = {
  [0] = 76,
  [1] = {["x"] = -11516, ["y"] = -367, ["z"] = 0, ["order"] = 0, ["angle"] = 1084293120},
  [2] = {["x"] = -11532, ["y"] = 1145, ["z"] = 237, ["order"] = 0, ["angle"] = 1084293120},
  [3] = {["x"] = -11532, ["y"] = 2967, ["z"] = 500, ["order"] = 0, ["angle"] = 1032388608},
  [4] = {["x"] = -11208, ["y"] = 4689, ["z"] = 500, ["order"] = 1, ["angle"] = 468647936},
  [5] = {["x"] = -10315, ["y"] = 5577, ["z"] = 500, ["order"] = 1, ["angle"] = -188284928},
  [6] = {["x"] = -8454, ["y"] = 5663, ["z"] = 500, ["order"] = 2, ["angle"] = -50659328},
  [7] = {["x"] = -6949, ["y"] = 5634, ["z"] = 500, ["order"] = 2, ["angle"] = 26411008},
  [8] = {["x"] = -5504, ["y"] = 5667, ["z"] = 500, ["order"] = 2, ["angle"] = 26411008},
  [9] = {["x"] = -3840, ["y"] = 5698, ["z"] = 482, ["order"] = 3, ["angle"] = -102301696},
  [10] = {["x"] = -2393, ["y"] = 5296, ["z"] = 346, ["order"] = 3, ["angle"] = -808255488},
  [11] = {["x"] = -1665, ["y"] = 3997, ["z"] = 86, ["order"] = 3, ["angle"] = -587399168},
  [12] = {["x"] = -924, ["y"] = 2867, ["z"] = 0, ["order"] = 3, ["angle"] = -8192000},
  [13] = {["x"] = 275, ["y"] = 2499, ["z"] = 0, ["order"] = 4, ["angle"] = 9895936},
  [14] = {["x"] = 1640, ["y"] = 2460, ["z"] = 0, ["order"] = 4, ["angle"] = 9895936},
  [15] = {["x"] = 2952, ["y"] = 2465, ["z"] = 0, ["order"] = 4, ["angle"] = 9895936},
  [16] = {["x"] = 4567, ["y"] = 2483, ["z"] = -300, ["order"] = 4, ["angle"] = 9895936},
  [17] = {["x"] = 6147, ["y"] = 2503, ["z"] = -300, ["order"] = 4, ["angle"] = 9895936},
  [18] = {["x"] = 7898, ["y"] = 2527, ["z"] = -300, ["order"] = 4, ["angle"] = 9895936},
  [19] = {["x"] = 9393, ["y"] = 2547, ["z"] = -200, ["order"] = 4, ["angle"] = 9895936},
  [20] = {["x"] = 10760, ["y"] = 2552, ["z"] = -200, ["order"] = 4, ["angle"] = -32309248},
  [21] = {["x"] = 12443, ["y"] = 2482, ["z"] = -200, ["order"] = 4, ["angle"] = -78839808},
  [22] = {["x"] = 13414, ["y"] = 2400, ["z"] = -200, ["order"] = 5, ["angle"] = -78839808},
  [23] = {["x"] = 15781, ["y"] = 2110, ["z"] = -1000, ["order"] = 5, ["angle"] = -43188224},
  [24] = {["x"] = 17477, ["y"] = 2125, ["z"] = -1000, ["order"] = 5, ["angle"] = -171114496},
  [25] = {["x"] = 19806, ["y"] = 1393, ["z"] = -1000, ["order"] = 5, ["angle"] = -774569984},
  [26] = {["x"] = 20631, ["y"] = -199, ["z"] = -1000, ["order"] = 6, ["angle"] = -1657733120},
  [27] = {["x"] = 19810, ["y"] = -886, ["z"] = -1000, ["order"] = 6, ["angle"] = -1863122944},
  [28] = {["x"] = 18712, ["y"] = -1241, ["z"] = -1000, ["order"] = 6, ["angle"] = -1509228544},
  [29] = {["x"] = 18029, ["y"] = -1973, ["z"] = -1000, ["order"] = 6, ["angle"] = -1259798528},
  [30] = {["x"] = 17884, ["y"] = -3173, ["z"] = -1000, ["order"] = 6, ["angle"] = -1549729792},
  [31] = {["x"] = 17280, ["y"] = -4109, ["z"] = -1000, ["order"] = 6, ["angle"] = -2099052544},
  [32] = {["x"] = 16160, ["y"] = -4283, ["z"] = -1000, ["order"] = 7, ["angle"] = -2077294592},
  [33] = {["x"] = 14146, ["y"] = -4493, ["z"] = -1000, ["order"] = 7, ["angle"] = 2144272384},
  [34] = {["x"] = 14120, ["y"] = -4189, ["z"] = -1000, ["order"] = 7, ["angle"] = -2140995584},
  [35] = {["x"] = 12717, ["y"] = -4364, ["z"] = -1000, ["order"] = 7, ["angle"] = -2129985536},
  [36] = {["x"] = 9595, ["y"] = -4388, ["z"] = -700, ["order"] = 7, ["angle"] = -2146893824},
  [37] = {["x"] = 8799, ["y"] = -4314, ["z"] = -500, ["order"] = 7, ["angle"] = -2143354880},
  [38] = {["x"] = 7170, ["y"] = -4272, ["z"] = -490, ["order"] = 7, ["angle"] = -2142175232},
  [39] = {["x"] = 6132, ["y"] = -4710, ["z"] = -490, ["order"] = 7, ["angle"] = -2065235968},
  [40] = {["x"] = 5179, ["y"] = -4866, ["z"] = -490, ["order"] = 7, ["angle"] = -2090008576},
  [41] = {["x"] = 3610, ["y"] = -4990, ["z"] = -480, ["order"] = 7, ["angle"] = -2127626240},
  [42] = {["x"] = 2285, ["y"] = -5036, ["z"] = -416, ["order"] = 8, ["angle"] = -2143223808},
  [43] = {["x"] = 1293, ["y"] = -5025, ["z"] = -200, ["order"] = 8, ["angle"] = -2143223808},
  [44] = {["x"] = -901, ["y"] = -5248, ["z"] = -142, ["order"] = 8, ["angle"] = -2074411008},
  [45] = {["x"] = -3047, ["y"] = -5263, ["z"] = -320, ["order"] = 9, ["angle"] = 2141650944},
  [46] = {["x"] = -4256, ["y"] = -5254, ["z"] = -300, ["order"] = 9, ["angle"] = 2141650944},
  [47] = {["x"] = -6096, ["y"] = -4756, ["z"] = 0, ["order"] = 9, ["angle"] = -2134966272},
  [48] = {["x"] = -8418, ["y"] = -4571, ["z"] = 0, ["order"] = 10, ["angle"] = 2142568448},
  [49] = {["x"] = -10430, ["y"] = -4476, ["z"] = 0, ["order"] = 10, ["angle"] = 1662582784},
  [50] = {["x"] = -11335, ["y"] = -3527, ["z"] = 0, ["order"] = 10, ["angle"] = 1105264640},
  [51] = {["x"] = -11507, ["y"] = -1809, ["z"] = 0, ["order"] = 10, ["angle"] = 1063190528},
  [52] = {["x"] = 7142, ["y"] = -3828, ["z"] = -490, ["order"] = 7, ["angle"] = 2092892160},
  [53] = {["x"] = 6329, ["y"] = -3735, ["z"] = -490, ["order"] = 7, ["angle"] = 2000879616},
  [54] = {["x"] = 5289, ["y"] = -3597, ["z"] = -490, ["order"] = 7, ["angle"] = 1932066816},
  [55] = {["x"] = 3622, ["y"] = -3301, ["z"] = -500, ["order"] = 7, ["angle"] = 2139291648},
  [56] = {["x"] = 2202, ["y"] = -3354, ["z"] = -400, ["order"] = 8, ["angle"] = 2127101952},
  [57] = {["x"] = 1268, ["y"] = -3322, ["z"] = -380, ["order"] = 8, ["angle"] = -2144534528},
  [58] = {["x"] = 105, ["y"] = -3364, ["z"] = -400, ["order"] = 8, ["angle"] = -2097086464},
  [59] = {["x"] = -1108, ["y"] = -3588, ["z"] = -420, ["order"] = 8, ["angle"] = -2003501056},
  [60] = {["x"] = -2165, ["y"] = -3883, ["z"] = -440, ["order"] = 9, ["angle"] = -2130378752},
  [61] = {["x"] = -3198, ["y"] = -3935, ["z"] = -310, ["order"] = 9, ["angle"] = -2043346944},
  [62] = {["x"] = -4259, ["y"] = -4036, ["z"] = -300, ["order"] = 9, ["angle"] = -2081357824},
  [63] = {["x"] = 1236, ["y"] = -4655, ["z"] = -200, ["order"] = 8, ["angle"] = 2126184448},
  [64] = {["x"] = -945, ["y"] = -4595, ["z"] = -138, ["order"] = 8, ["angle"] = 2071527424},
  [65] = {["x"] = -1982, ["y"] = -4769, ["z"] = -420, ["order"] = 9, ["angle"] = 2081095680},
  [66] = {["x"] = -2062, ["y"] = -4271, ["z"] = -440, ["order"] = 9, ["angle"] = -2139291648},
  [67] = {["x"] = -3136, ["y"] = -4533, ["z"] = -320, ["order"] = 9, ["angle"] = 2118189056},
  [68] = {["x"] = -4139, ["y"] = -4529, ["z"] = -300, ["order"] = 9, ["angle"] = -2087518208},
  [69] = {["x"] = 18953, ["y"] = 1966, ["z"] = -1000, ["order"] = 5, ["angle"] = -349241344},
  [70] = {["x"] = 20310, ["y"] = 792, ["z"] = -1000, ["order"] = 5, ["angle"] = -896335872},
  [71] = {["x"] = 1226, ["y"] = -3933, ["z"] = -340, ["order"] = 8, ["angle"] = -2081357824},
  [72] = {["x"] = 233, ["y"] = -4011, ["z"] = -330, ["order"] = 8, ["angle"] = -2142175232},
  [73] = {["x"] = -929, ["y"] = -4116, ["z"] = -460, ["order"] = 8, ["angle"] = -2143879168},
  [74] = {["x"] = -6254, ["y"] = -3944, ["z"] = 0, ["order"] = 9, ["angle"] = -2003238912},
  [75] = {["x"] = -7489, ["y"] = -4205, ["z"] = 0, ["order"] = 9, ["angle"] = -2065760256},
  [76] = {["x"] = -9288, ["y"] = -4537, ["z"] = 0, ["order"] = 10, ["angle"] = 2024079360}
}
-- Respawn definitions for NORTHERN DISTRICT (MAP03)
krespawn.points[3] = {
  [0] = 103,
  [1] = {["x"] = -6640, ["y"] = -5147, ["z"] = 1153, ["order"] = 0, ["angle"] = 1067646976},
  [2] = {["x"] = -6631, ["y"] = -4228, ["z"] = 1074, ["order"] = 0, ["angle"] = 1067646976},
  [3] = {["x"] = -6620, ["y"] = -3025, ["z"] = 1000, ["order"] = 0, ["angle"] = 1067646976},
  [4] = {["x"] = -6609, ["y"] = -1775, ["z"] = 1000, ["order"] = 0, ["angle"] = 1067646976},
  [5] = {["x"] = -6597, ["y"] = -517, ["z"] = 1000, ["order"] = 0, ["angle"] = 1067646976},
  [6] = {["x"] = -6584, ["y"] = 861, ["z"] = 1000, ["order"] = 0, ["angle"] = 1067646976},
  [7] = {["x"] = -6572, ["y"] = 2237, ["z"] = 1000, ["order"] = 1, ["angle"] = 314376192},
  [8] = {["x"] = -5904, ["y"] = 2707, ["z"] = 1000, ["order"] = 1, ["angle"] = -13697024},
  [9] = {["x"] = -4912, ["y"] = 2714, ["z"] = 1000, ["order"] = 1, ["angle"] = -37027840},
  [10] = {["x"] = -3846, ["y"] = 2663, ["z"] = 1000, ["order"] = 1, ["angle"] = -308609024},
  [11] = {["x"] = -2992, ["y"] = 1990, ["z"] = 1000, ["order"] = 1, ["angle"] = -380567552},
  [12] = {["x"] = -2421, ["y"] = 1601, ["z"] = 1000, ["order"] = 1, ["angle"] = 221839360},
  [13] = {["x"] = -1313, ["y"] = 2026, ["z"] = 1000, ["order"] = 1, ["angle"] = 181862400},
  [14] = {["x"] = -111, ["y"] = 2357, ["z"] = 1000, ["order"] = 1, ["angle"] = 100335616},
  [15] = {["x"] = 889, ["y"] = 2471, ["z"] = 1000, ["order"] = 1, ["angle"] = 77266944},
  [16] = {["x"] = 1828, ["y"] = 2497, ["z"] = 1000, ["order"] = 1, ["angle"] = 213975040},
  [17] = {["x"] = 2918, ["y"] = 2607, ["z"] = 1000, ["order"] = 2, ["angle"] = 929759232},
  [18] = {["x"] = 3181, ["y"] = 3750, ["z"] = 1000, ["order"] = 2, ["angle"] = 1071316992},
  [19] = {["x"] = 3211, ["y"] = 5036, ["z"] = 1000, ["order"] = 2, ["angle"] = 1071316992},
  [20] = {["x"] = 3220, ["y"] = 6501, ["z"] = 1000, ["order"] = 2, ["angle"] = 1071316992},
  [21] = {["x"] = 3251, ["y"] = 7842, ["z"] = 1000, ["order"] = 2, ["angle"] = 321191936},
  [22] = {["x"] = 4179, ["y"] = 8429, ["z"] = 1000, ["order"] = 3, ["angle"] = 5701632},
  [23] = {["x"] = 5151, ["y"] = 8375, ["z"] = 1000, ["order"] = 3, ["angle"] = -758448128},
  [24] = {["x"] = 5649, ["y"] = 7160, ["z"] = 1000, ["order"] = 3, ["angle"] = -1043791872},
  [25] = {["x"] = 5725, ["y"] = 6051, ["z"] = 1000, ["order"] = 3, ["angle"] = -1043791872},
  [26] = {["x"] = 5773, ["y"] = 5018, ["z"] = 1000, ["order"] = 3, ["angle"] = -1074593792},
  [27] = {["x"] = 5789, ["y"] = 3862, ["z"] = 1000, ["order"] = 3, ["angle"] = -1074593792},
  [28] = {["x"] = 5791, ["y"] = 2630, ["z"] = 1000, ["order"] = 3, ["angle"] = -1074593792},
  [29] = {["x"] = 5790, ["y"] = 1625, ["z"] = 1000, ["order"] = 4, ["angle"] = -1074593792},
  [30] = {["x"] = 5789, ["y"] = 693, ["z"] = 1000, ["order"] = 4, ["angle"] = -1074593792},
  [31] = {["x"] = 5787, ["y"] = -471, ["z"] = 1000, ["order"] = 4, ["angle"] = -1074593792},
  [32] = {["x"] = 5785, ["y"] = -1748, ["z"] = 1000, ["order"] = 4, ["angle"] = -1074593792},
  [33] = {["x"] = 5783, ["y"] = -3088, ["z"] = 1000, ["order"] = 4, ["angle"] = -1074593792},
  [34] = {["x"] = 5781, ["y"] = -4431, ["z"] = 1000, ["order"] = 4, ["angle"] = -406913024},
  [35] = {["x"] = 6410, ["y"] = -5275, ["z"] = 1000, ["order"] = 5, ["angle"] = 50135040},
  [36] = {["x"] = 7232, ["y"] = -5280, ["z"] = 1000, ["order"] = 5, ["angle"] = -9502720},
  [37] = {["x"] = 8178, ["y"] = -5288, ["z"] = 1000, ["order"] = 5, ["angle"] = -9502720},
  [38] = {["x"] = 9074, ["y"] = -5300, ["z"] = 1000, ["order"] = 5, ["angle"] = -9502720},
  [39] = {["x"] = 10303, ["y"] = -5317, ["z"] = 1000, ["order"] = 5, ["angle"] = -9502720},
  [40] = {["x"] = 11577, ["y"] = -5337, ["z"] = 1000, ["order"] = 5, ["angle"] = -830668800},
  [41] = {["x"] = 12041, ["y"] = -6452, ["z"] = 1000, ["order"] = 6, ["angle"] = -1073283072},
  [42] = {["x"] = 11994, ["y"] = -7578, ["z"] = 1000, ["order"] = 6, ["angle"] = -1543307264},
  [43] = {["x"] = 11074, ["y"] = -8936, ["z"] = 1000, ["order"] = 6, ["angle"] = -1845035008},
  [44] = {["x"] = 9986, ["y"] = -9622, ["z"] = 1000, ["order"] = 6, ["angle"] = -2104557568},
  [45] = {["x"] = 8415, ["y"] = -9780, ["z"] = 1000, ["order"] = 6, ["angle"] = 2141913088},
  [46] = {["x"] = 7228, ["y"] = -9781, ["z"] = 1000, ["order"] = 6, ["angle"] = 2141913088},
  [47] = {["x"] = 6096, ["y"] = -9773, ["z"] = 1000, ["order"] = 7, ["angle"] = 2141913088},
  [48] = {["x"] = 4526, ["y"] = -9759, ["z"] = 1250, ["order"] = 7, ["angle"] = 2141913088},
  [49] = {["x"] = -3002, ["y"] = -9272, ["z"] = 1000, ["order"] = 8, ["angle"] = 1156382720},
  [50] = {["x"] = -3014, ["y"] = -8150, ["z"] = 1000, ["order"] = 8, ["angle"] = 1379074048},
  [51] = {["x"] = -3640, ["y"] = -7243, ["z"] = 1000, ["order"] = 8, ["angle"] = -2145976320},
  [52] = {["x"] = -4265, ["y"] = -7322, ["z"] = 1000, ["order"] = 9, ["angle"] = -1416953856},
  [53] = {["x"] = -4561, ["y"] = -8582, ["z"] = 1056, ["order"] = 9, ["angle"] = -1243545600},
  [54] = {["x"] = -4823, ["y"] = -9800, ["z"] = 1202, ["order"] = 10, ["angle"] = -1881473024},
  [55] = {["x"] = -5768, ["y"] = -10265, ["z"] = 1257, ["order"] = 10, ["angle"] = -2133917696},
  [56] = {["x"] = -7139, ["y"] = -10446, ["z"] = 1279, ["order"] = 10, ["angle"] = 2031812608},
  [57] = {["x"] = -8309, ["y"] = -10412, ["z"] = 1275, ["order"] = 11, ["angle"] = 1275002880},
  [58] = {["x"] = -8743, ["y"] = -9555, ["z"] = 1172, ["order"] = 11, ["angle"] = 515571712},
  [59] = {["x"] = -7805, ["y"] = -8868, ["z"] = 1134, ["order"] = 11, ["angle"] = 631308288},
  [60] = {["x"] = -6815, ["y"] = -7999, ["z"] = 1134, ["order"] = 12, ["angle"] = 955318272},
  [61] = {["x"] = -6636, ["y"] = -6377, ["z"] = 1162, ["order"] = 12, ["angle"] = 1061748736},
  [62] = {["x"] = -6652, ["y"] = -5736, ["z"] = 1164, ["order"] = 12, ["angle"] = 1075511296},
  [63] = {["x"] = -7087, ["y"] = -3202, ["z"] = 1010, ["order"] = 0, ["angle"] = 1505558528},
  [64] = {["x"] = -7570, ["y"] = -1962, ["z"] = 820, ["order"] = 0, ["angle"] = 1058734080},
  [65] = {["x"] = -7531, ["y"] = -607, ["z"] = 810, ["order"] = 0, ["angle"] = 1058340864},
  [66] = {["x"] = -7507, ["y"] = 958, ["z"] = 810, ["order"] = 0, ["angle"] = 1058340864},
  [67] = {["x"] = -7580, ["y"] = 2268, ["z"] = 1010, ["order"] = 1, ["angle"] = 324075520},
  [68] = {["x"] = -6858, ["y"] = 2682, ["z"] = 1000, ["order"] = 1, ["angle"] = 19070976},
  [69] = {["x"] = -3834, ["y"] = 2829, ["z"] = 1000, ["order"] = 1, ["angle"] = 327614464},
  [70] = {["x"] = -3200, ["y"] = 3208, ["z"] = 1000, ["order"] = 1, ["angle"] = 191037440},
  [71] = {["x"] = -2396, ["y"] = 3741, ["z"] = 1000, ["order"] = 1, ["angle"] = -165609472},
  [72] = {["x"] = -1506, ["y"] = 3575, ["z"] = 1000, ["order"] = 1, ["angle"] = -519897088},
  [73] = {["x"] = -769, ["y"] = 3040, ["z"] = 1000, ["order"] = 1, ["angle"] = -179109888},
  [74] = {["x"] = 3, ["y"] = 2693, ["z"] = 1000, ["order"] = 1, ["angle"] = -38600704},
  [75] = {["x"] = 6524, ["y"] = 986, ["z"] = 1000, ["order"] = 4, ["angle"] = -119734272},
  [76] = {["x"] = 7819, ["y"] = 1019, ["z"] = 1000, ["order"] = 4, ["angle"] = 175439872},
  [77] = {["x"] = 9834, ["y"] = 1006, ["z"] = 1000, ["order"] = 4, ["angle"] = -7143424},
  [78] = {["x"] = 11635, ["y"] = 830, ["z"] = 1000, ["order"] = 4, ["angle"] = -677052416},
  [79] = {["x"] = 12480, ["y"] = -527, ["z"] = 1000, ["order"] = 5, ["angle"] = -1062535168},
  [80] = {["x"] = 11864, ["y"] = -454, ["z"] = 1000, ["order"] = 5, ["angle"] = -1022427136},
  [81] = {["x"] = 11883, ["y"] = -1881, ["z"] = 1000, ["order"] = 5, ["angle"] = -951648256},
  [82] = {["x"] = 12455, ["y"] = -1751, ["z"] = 1000, ["order"] = 5, ["angle"] = -1146159104},
  [83] = {["x"] = 12174, ["y"] = -3478, ["z"] = 1000, ["order"] = 5, ["angle"] = -1065549824},
  [84] = {["x"] = 12157, ["y"] = -5030, ["z"] = 1000, ["order"] = 6, ["angle"] = -1121648640},
  [85] = {["x"] = 11757, ["y"] = -8279, ["z"] = 1000, ["order"] = 6, ["angle"] = -1640955904},
  [86] = {["x"] = 7201, ["y"] = -10004, ["z"] = 1000, ["order"] = 6, ["angle"] = -2112815104},
  [87] = {["x"] = -328, ["y"] = -9892, ["z"] = 1372, ["order"] = 7, ["angle"] = 2119106560},
  [88] = {["x"] = -1252, ["y"] = -9881, ["z"] = 1166, ["order"] = 7, ["angle"] = -2132082688},
  [89] = {["x"] = -2338, ["y"] = -9910, ["z"] = 1000, ["order"] = 8, ["angle"] = 1670053888},
  [90] = {["x"] = -827, ["y"] = 2244, ["z"] = 1000, ["order"] = 1, ["angle"] = 111214592},
  [91] = {["x"] = 5632, ["y"] = 808, ["z"] = 1000, ["order"] = 4, ["angle"] = -1167392768},
  [92] = {["x"] = 5437, ["y"] = -662, ["z"] = 928, ["order"] = 4, ["angle"] = -1068695552},
  [93] = {["x"] = 5445, ["y"] = -1955, ["z"] = 600, ["order"] = 4, ["angle"] = -1024655360},
  [94] = {["x"] = 5531, ["y"] = -3260, ["z"] = 600, ["order"] = 4, ["angle"] = -1066336256},
  [95] = {["x"] = 5501, ["y"] = -4420, ["z"] = 600, ["order"] = 4, ["angle"] = -595263488},
  [96] = {["x"] = 6214, ["y"] = -5418, ["z"] = 600, ["order"] = 5, ["angle"] = -46596096},
  [97] = {["x"] = 7892, ["y"] = -5673, ["z"] = 600, ["order"] = 5, ["angle"] = 7536640},
  [98] = {["x"] = 8967, ["y"] = -5694, ["z"] = 877, ["order"] = 5, ["angle"] = 1507328},
  [99] = {["x"] = 10000, ["y"] = -5611, ["z"] = 1000, ["order"] = 5, ["angle"] = 146866176},
  [100] = {["x"] = 11989, ["y"] = -5828, ["z"] = 1000, ["order"] = 6, ["angle"] = -1051525120},
  [101] = {["x"] = 9223, ["y"] = -9829, ["z"] = 1000, ["order"] = 6, ["angle"] = 2141650944},
  [102] = {["x"] = 8359, ["y"] = -10013, ["z"] = 1000, ["order"] = 6, ["angle"] = -2136408064},
  [103] = {["x"] = 4671, ["y"] = -9999, ["z"] = 1217, ["order"] = 7, ["angle"] = -2146631680}
}
-- Respawn definitions for DARKVILE GARDEN (MAP04)
krespawn.points[4] = {
  [0] = 77,
  [1] = {["x"] = 733, ["y"] = -7744, ["z"] = 0, ["order"] = 0, ["angle"] = 65536},
  [2] = {["x"] = 1502, ["y"] = -7744, ["z"] = 0, ["order"] = 0, ["angle"] = 65536},
  [3] = {["x"] = 2695, ["y"] = -7744, ["z"] = 0, ["order"] = 0, ["angle"] = 65536},
  [4] = {["x"] = 3948, ["y"] = -7729, ["z"] = 0, ["order"] = 0, ["angle"] = 745865216},
  [5] = {["x"] = 4416, ["y"] = -6659, ["z"] = 0, ["order"] = 1, ["angle"] = 636813312},
  [6] = {["x"] = 5209, ["y"] = -5648, ["z"] = 0, ["order"] = 1, ["angle"] = 1064632320},
  [7] = {["x"] = 5259, ["y"] = -4739, ["z"] = 0, ["order"] = 1, ["angle"] = 1132396544},
  [8] = {["x"] = 4968, ["y"] = -3769, ["z"] = 0, ["order"] = 1, ["angle"] = 1490092032},
  [9] = {["x"] = 4231, ["y"] = -2654, ["z"] = 0, ["order"] = 1, ["angle"] = 1474363392},
  [10] = {["x"] = 3730, ["y"] = -1883, ["z"] = 0, ["order"] = 1, ["angle"] = 1337393152},
  [11] = {["x"] = 3421, ["y"] = -808, ["z"] = 0, ["order"] = 2, ["angle"] = 1018363904},
  [12] = {["x"] = 3685, ["y"] = 167, ["z"] = 0, ["order"] = 2, ["angle"] = 766312448},
  [13] = {["x"] = 4220, ["y"] = 1406, ["z"] = 0, ["order"] = 2, ["angle"] = 1039466496},
  [14] = {["x"] = 4318, ["y"] = 2690, ["z"] = 0, ["order"] = 2, ["angle"] = 1909784576},
  [15] = {["x"] = 3018, ["y"] = 3052, ["z"] = 0, ["order"] = 3, ["angle"] = 1546190848},
  [16] = {["x"] = 2259, ["y"] = 3956, ["z"] = 0, ["order"] = 3, ["angle"] = 1051394048},
  [17] = {["x"] = 2153, ["y"] = 5115, ["z"] = 0, ["order"] = 4, ["angle"] = 440598528},
  [18] = {["x"] = 3080, ["y"] = 5668, ["z"] = 0, ["order"] = 5, ["angle"] = 440860672},
  [19] = {["x"] = 4264, ["y"] = 5758, ["z"] = 0, ["order"] = 5, ["angle"] = 1041825792},
  [20] = {["x"] = 4291, ["y"] = 7269, ["z"] = 0, ["order"] = 6, ["angle"] = 2057240576},
  [21] = {["x"] = 3093, ["y"] = 7342, ["z"] = 0, ["order"] = 7, ["angle"] = -2137980928},
  [22] = {["x"] = 2106, ["y"] = 7332, ["z"] = 12, ["order"] = 7, ["angle"] = -2137980928},
  [23] = {["x"] = 1006, ["y"] = 7318, ["z"] = 99, ["order"] = 7, ["angle"] = -2137980928},
  [24] = {["x"] = 90, ["y"] = 7305, ["z"] = 128, ["order"] = 7, ["angle"] = -2137980928},
  [25] = {["x"] = -686, ["y"] = 7368, ["z"] = 128, ["order"] = 8, ["angle"] = -1753677824},
  [26] = {["x"] = -1363, ["y"] = 7203, ["z"] = 128, ["order"] = 8, ["angle"] = -1111949312},
  [27] = {["x"] = -1492, ["y"] = 6013, ["z"] = 128, ["order"] = 9, ["angle"] = -95354880},
  [28] = {["x"] = -387, ["y"] = 6012, ["z"] = 128, ["order"] = 10, ["angle"] = -517668864},
  [29] = {["x"] = 595, ["y"] = 5744, ["z"] = 128, ["order"] = 10, ["angle"] = -1065025536},
  [30] = {["x"] = 631, ["y"] = 4810, ["z"] = 128, ["order"] = 11, ["angle"] = -1065025536},
  [31] = {["x"] = 644, ["y"] = 3823, ["z"] = 128, ["order"] = 11, ["angle"] = -1065025536},
  [32] = {["x"] = 702, ["y"] = 2795, ["z"] = 128, ["order"] = 11, ["angle"] = -1919746048},
  [33] = {["x"] = -176, ["y"] = 2508, ["z"] = 128, ["order"] = 11, ["angle"] = -1884880896},
  [34] = {["x"] = -1061, ["y"] = 2473, ["z"] = 128, ["order"] = 12, ["angle"] = -1186529280},
  [35] = {["x"] = -1264, ["y"] = 1198, ["z"] = 95, ["order"] = 13, ["angle"] = -1050607616},
  [36] = {["x"] = -1172, ["y"] = 167, ["z"] = 0, ["order"] = 13, ["angle"] = -910753792},
  [37] = {["x"] = -895, ["y"] = -1010, ["z"] = 0, ["order"] = 13, ["angle"] = -1300037632},
  [38] = {["x"] = -961, ["y"] = -2275, ["z"] = -64, ["order"] = 13, ["angle"] = -1764294656},
  [39] = {["x"] = -1724, ["y"] = -2728, ["z"] = -64, ["order"] = 13, ["angle"] = -2102198272},
  [40] = {["x"] = -2848, ["y"] = -2701, ["z"] = -64, ["order"] = 13, ["angle"] = -2056585216},
  [41] = {["x"] = -4041, ["y"] = -2743, ["z"] = 0, ["order"] = 13, ["angle"] = -1939668992},
  [42] = {["x"] = -5035, ["y"] = -2994, ["z"] = 0, ["order"] = 13, ["angle"] = 2134966272},
  [43] = {["x"] = -6388, ["y"] = -2981, ["z"] = 128, ["order"] = 14, ["angle"] = -1861287936},
  [44] = {["x"] = -7369, ["y"] = -3402, ["z"] = 128, ["order"] = 14, ["angle"] = -1069219840},
  [45] = {["x"] = -7620, ["y"] = -4497, ["z"] = 25, ["order"] = 15, ["angle"] = -207552512},
  [46] = {["x"] = -7124, ["y"] = -5129, ["z"] = 0, ["order"] = 15, ["angle"] = -78446592},
  [47] = {["x"] = -6249, ["y"] = -5131, ["z"] = 0, ["order"] = 16, ["angle"] = -2818048},
  [48] = {["x"] = -5264, ["y"] = -5274, ["z"] = 0, ["order"] = 16, ["angle"] = -525008896},
  [49] = {["x"] = -4598, ["y"] = -5912, ["z"] = 0, ["order"] = 16, ["angle"] = -511246336},
  [50] = {["x"] = -3937, ["y"] = -6546, ["z"] = 0, ["order"] = 16, ["angle"] = -511246336},
  [51] = {["x"] = -3456, ["y"] = -6997, ["z"] = 0, ["order"] = 16, ["angle"] = -511246336},
  [52] = {["x"] = -2803, ["y"] = -7662, ["z"] = 0, ["order"] = 16, ["angle"] = -2293760},
  [53] = {["x"] = -2091, ["y"] = -7737, ["z"] = 0, ["order"] = 16, ["angle"] = -17235968},
  [54] = {["x"] = -1218, ["y"] = -7747, ["z"] = 0, ["order"] = 16, ["angle"] = 31129600},
  [55] = {["x"] = -623, ["y"] = -7734, ["z"] = 0, ["order"] = 16, ["angle"] = -12386304},
  [56] = {["x"] = 13, ["y"] = -7735, ["z"] = 0, ["order"] = 16, ["angle"] = -12386304},
  [57] = {["x"] = 2077, ["y"] = -7771, ["z"] = 0, ["order"] = 0, ["angle"] = -12386304},
  [58] = {["x"] = 3226, ["y"] = -7788, ["z"] = 0, ["order"] = 0, ["angle"] = 104398848},
  [59] = {["x"] = 4156, ["y"] = -6728, ["z"] = 0, ["order"] = 1, ["angle"] = 1434255360},
  [60] = {["x"] = 3502, ["y"] = -5848, ["z"] = 0, ["order"] = 1, ["angle"] = 1220345856},
  [61] = {["x"] = 3325, ["y"] = -5007, ["z"] = 0, ["order"] = 1, ["angle"] = 865009664},
  [62] = {["x"] = 3459, ["y"] = -4025, ["z"] = 0, ["order"] = 1, ["angle"] = 635633664},
  [63] = {["x"] = 4265, ["y"] = -3372, ["z"] = 0, ["order"] = 1, ["angle"] = 1068171264},
  [64] = {["x"] = 4364, ["y"] = -2692, ["z"] = 0, ["order"] = 1, ["angle"] = 723320832},
  [65] = {["x"] = 4954, ["y"] = -1993, ["z"] = 0, ["order"] = 1, ["angle"] = 829882368},
  [66] = {["x"] = 5239, ["y"] = -1159, ["z"] = 0, ["order"] = 2, ["angle"] = 1149042688},
  [67] = {["x"] = 5236, ["y"] = -333, ["z"] = 0, ["order"] = 2, ["angle"] = 1452867584},
  [68] = {["x"] = 5232, ["y"] = -291, ["z"] = 0, ["order"] = 2, ["angle"] = 1452867584},
  [69] = {["x"] = 4864, ["y"] = 304, ["z"] = 0, ["order"] = 2, ["angle"] = 1407254528},
  [70] = {["x"] = 4252, ["y"] = 169, ["z"] = 0, ["order"] = 2, ["angle"] = 1063321600},
  [71] = {["x"] = -1787, ["y"] = 1329, ["z"] = 135, ["order"] = 13, ["angle"] = -1118633984},
  [72] = {["x"] = -1991, ["y"] = -376, ["z"] = 192, ["order"] = 13, ["angle"] = -1109458944},
  [73] = {["x"] = -2070, ["y"] = -2042, ["z"] = 192, ["order"] = 13, ["angle"] = -1048641536},
  [74] = {["x"] = -2040, ["y"] = -3586, ["z"] = 192, ["order"] = 13, ["angle"] = -2095120384},
  [75] = {["x"] = -3147, ["y"] = -3742, ["z"] = 192, ["order"] = 13, ["angle"] = 2142437376},
  [76] = {["x"] = -4628, ["y"] = -3679, ["z"] = 192, ["order"] = 13, ["angle"] = 2012545024},
  [77] = {["x"] = -5939, ["y"] = -3431, ["z"] = 157, ["order"] = 14, ["angle"] = -2109014016}
}
-- Respawn definitions for DAYTONA SPEEDWAY (MAP05)
krespawn.points[5] = {
  [0] = 54,
  [1] = {["x"] = 2369, ["y"] = -5569, ["z"] = 0, ["order"] = 0, ["angle"] = 1074069504},
  [2] = {["x"] = 2369, ["y"] = -4457, ["z"] = 0, ["order"] = 0, ["angle"] = 1074069504},
  [3] = {["x"] = 2369, ["y"] = -3472, ["z"] = 0, ["order"] = 0, ["angle"] = 1074069504},
  [4] = {["x"] = 2365, ["y"] = -2357, ["z"] = 0, ["order"] = 0, ["angle"] = 1145765888},
  [5] = {["x"] = 2373, ["y"] = -1389, ["z"] = 53, ["order"] = 1, ["angle"] = 1539244032},
  [6] = {["x"] = 1655, ["y"] = -549, ["z"] = 36, ["order"] = 1, ["angle"] = -1955659776},
  [7] = {["x"] = 585, ["y"] = -1090, ["z"] = 35, ["order"] = 1, ["angle"] = -957808640},
  [8] = {["x"] = 765, ["y"] = -1952, ["z"] = 29, ["order"] = 1, ["angle"] = -428408832},
  [9] = {["x"] = 1966, ["y"] = -2343, ["z"] = 192, ["order"] = 1, ["angle"] = 14221312},
  [10] = {["x"] = 3419, ["y"] = -2076, ["z"] = 0, ["order"] = 1, ["angle"] = 4128768},
  [11] = {["x"] = 4833, ["y"] = -2282, ["z"] = 0, ["order"] = 2, ["angle"] = -8192000},
  [12] = {["x"] = 6425, ["y"] = -2101, ["z"] = 0, ["order"] = 2, ["angle"] = -651231232},
  [13] = {["x"] = 7046, ["y"] = -3517, ["z"] = 0, ["order"] = 2, ["angle"] = -1602682880},
  [14] = {["x"] = 6331, ["y"] = -4145, ["z"] = 27, ["order"] = 2, ["angle"] = -1509621760},
  [15] = {["x"] = 5446, ["y"] = -4971, ["z"] = 143, ["order"] = 2, ["angle"] = -1327693824},
  [16] = {["x"] = 5029, ["y"] = -5530, ["z"] = 144, ["order"] = 2, ["angle"] = -1336082432},
  [17] = {["x"] = 4397, ["y"] = -6664, ["z"] = 144, ["order"] = 2, ["angle"] = -998440960},
  [18] = {["x"] = 4318, ["y"] = -7760, ["z"] = 144, ["order"] = 2, ["angle"] = -1052311552},
  [19] = {["x"] = 4360, ["y"] = -8796, ["z"] = 91, ["order"] = 2, ["angle"] = -259325952},
  [20] = {["x"] = 5767, ["y"] = -9103, ["z"] = 0, ["order"] = 3, ["angle"] = 23658496},
  [21] = {["x"] = 7029, ["y"] = -9067, ["z"] = 0, ["order"] = 3, ["angle"] = 23658496},
  [22] = {["x"] = 8177, ["y"] = -9028, ["z"] = 0, ["order"] = 3, ["angle"] = 23658496},
  [23] = {["x"] = 8989, ["y"] = -9000, ["z"] = 0, ["order"] = 3, ["angle"] = 23658496},
  [24] = {["x"] = 9772, ["y"] = -8962, ["z"] = 0, ["order"] = 4, ["angle"] = -146997248},
  [25] = {["x"] = 10586, ["y"] = -8960, ["z"] = 0, ["order"] = 4, ["angle"] = -562757632},
  [26] = {["x"] = 11417, ["y"] = -9458, ["z"] = 52, ["order"] = 4, ["angle"] = -1193476096},
  [27] = {["x"] = 11443, ["y"] = -10369, ["z"] = 54, ["order"] = 4, ["angle"] = -1849622528},
  [28] = {["x"] = 10712, ["y"] = -10854, ["z"] = 0, ["order"] = 4, ["angle"] = 1953562624},
  [29] = {["x"] = 9882, ["y"] = -10536, ["z"] = 0, ["order"] = 4, ["angle"] = 1934950400},
  [30] = {["x"] = 8776, ["y"] = -10039, ["z"] = 0, ["order"] = 4, ["angle"] = -2100756480},
  [31] = {["x"] = 7639, ["y"] = -9978, ["z"] = 0, ["order"] = 4, ["angle"] = -1333723136},
  [32] = {["x"] = 7601, ["y"] = -10888, ["z"] = 0, ["order"] = 4, ["angle"] = -1332805632},
  [33] = {["x"] = 7436, ["y"] = -11779, ["z"] = 0, ["order"] = 4, ["angle"] = -1837039616},
  [34] = {["x"] = 6173, ["y"] = -12487, ["z"] = 0, ["order"] = 4, ["angle"] = -2082930688},
  [35] = {["x"] = 4875, ["y"] = -12577, ["z"] = 106, ["order"] = 5, ["angle"] = -2126053376},
  [36] = {["x"] = 3284, ["y"] = -12552, ["z"] = 256, ["order"] = 5, ["angle"] = 2135097344},
  [37] = {["x"] = 2197, ["y"] = -12468, ["z"] = 256, ["order"] = 5, ["angle"] = -1970339840},
  [38] = {["x"] = 1164, ["y"] = -12643, ["z"] = 256, ["order"] = 5, ["angle"] = -1544880128},
  [39] = {["x"] = 553, ["y"] = -13151, ["z"] = 256, ["order"] = 5, ["angle"] = -1544880128},
  [40] = {["x"] = -249, ["y"] = -13940, ["z"] = 274, ["order"] = 5, ["angle"] = 2029191168},
  [41] = {["x"] = -1397, ["y"] = -13781, ["z"] = 257, ["order"] = 5, ["angle"] = 1469120512},
  [42] = {["x"] = -2120, ["y"] = -12884, ["z"] = 257, ["order"] = 5, ["angle"] = 955318272},
  [43] = {["x"] = -1967, ["y"] = -11771, ["z"] = 263, ["order"] = 5, ["angle"] = 261423104},
  [44] = {["x"] = -490, ["y"] = -10982, ["z"] = 256, ["order"] = 6, ["angle"] = 313982976},
  [45] = {["x"] = 1509, ["y"] = -10181, ["z"] = 256, ["order"] = 6, ["angle"] = 547815424},
  [46] = {["x"] = 2521, ["y"] = -9040, ["z"] = 248, ["order"] = 7, ["angle"] = 1693908992},
  [47] = {["x"] = 1572, ["y"] = -8453, ["z"] = 181, ["order"] = 8, ["angle"] = -1449984000},
  [48] = {["x"] = 985, ["y"] = -9336, ["z"] = 112, ["order"] = 9, ["angle"] = -474152960},
  [49] = {["x"] = 1820, ["y"] = -9880, ["z"] = 42, ["order"] = 10, ["angle"] = 418840576},
  [50] = {["x"] = 2456, ["y"] = -8915, ["z"] = 0, ["order"] = 10, ["angle"] = 1112997888},
  [51] = {["x"] = 2441, ["y"] = -8009, ["z"] = 0, ["order"] = 11, ["angle"] = 1075249152},
  [52] = {["x"] = 2425, ["y"] = -7319, ["z"] = 0, ["order"] = 11, ["angle"] = 1115488256},
  [53] = {["x"] = 2387, ["y"] = -6513, ["z"] = 0, ["order"] = 11, ["angle"] = 1115488256},
  [54] = {["x"] = 2365, ["y"] = -5961, ["z"] = 0, ["order"] = 11, ["angle"] = 1098973184}
}
-- Respawn definitions for EGG ZEPPELIN (MAP06)
krespawn.points[6] = {
  [0] = 72,
  [1] = {["x"] = 383, ["y"] = 4965, ["z"] = 0, ["order"] = 0, ["angle"] = 1095565312},
  [2] = {["x"] = 397, ["y"] = 5833, ["z"] = 0, ["order"] = 0, ["angle"] = 1065156608},
  [3] = {["x"] = 438, ["y"] = 6787, ["z"] = 0, ["order"] = 0, ["angle"] = 1189150720},
  [4] = {["x"] = 404, ["y"] = 7814, ["z"] = 7, ["order"] = 1, ["angle"] = 1360199680},
  [5] = {["x"] = 192, ["y"] = 8719, ["z"] = 82, ["order"] = 1, ["angle"] = 1572143104},
  [6] = {["x"] = -337, ["y"] = 9474, ["z"] = 145, ["order"] = 1, ["angle"] = 1953169408},
  [7] = {["x"] = -1118, ["y"] = 9978, ["z"] = 187, ["order"] = 1, ["angle"] = 2119106560},
  [8] = {["x"] = -2144, ["y"] = 10117, ["z"] = 199, ["order"] = 1, ["angle"] = -1774256128},
  [9] = {["x"] = -3026, ["y"] = 9874, ["z"] = 178, ["order"] = 1, ["angle"] = -1463615488},
  [10] = {["x"] = -3680, ["y"] = 9377, ["z"] = 137, ["order"] = 1, ["angle"] = -1195048960},
  [11] = {["x"] = -4064, ["y"] = 8655, ["z"] = 77, ["order"] = 1, ["angle"] = -1102774272},
  [12] = {["x"] = -4244, ["y"] = 7482, ["z"] = 0, ["order"] = 2, ["angle"] = -1066991616},
  [13] = {["x"] = -4252, ["y"] = 6512, ["z"] = 0, ["order"] = 2, ["angle"] = -1066991616},
  [14] = {["x"] = -4245, ["y"] = 5390, ["z"] = 0, ["order"] = 2, ["angle"] = -1066991616},
  [15] = {["x"] = -4304, ["y"] = 4201, ["z"] = 0, ["order"] = 3, ["angle"] = -486604800},
  [16] = {["x"] = -3878, ["y"] = 3115, ["z"] = 0, ["order"] = 3, ["angle"] = -108855296},
  [17] = {["x"] = -2605, ["y"] = 2794, ["z"] = 0, ["order"] = 4, ["angle"] = -64028672},
  [18] = {["x"] = -1226, ["y"] = 2748, ["z"] = 128, ["order"] = 4, ["angle"] = -15269888},
  [19] = {["x"] = 105, ["y"] = 2851, ["z"] = 275, ["order"] = 4, ["angle"] = -79101952},
  [20] = {["x"] = 1732, ["y"] = 2835, ["z"] = 441, ["order"] = 4, ["angle"] = -265093120},
  [21] = {["x"] = 3268, ["y"] = 2527, ["z"] = 599, ["order"] = 4, ["angle"] = -524746752},
  [22] = {["x"] = 4132, ["y"] = 1840, ["z"] = 688, ["order"] = 4, ["angle"] = -658833408},
  [23] = {["x"] = 4717, ["y"] = 998, ["z"] = 748, ["order"] = 5, ["angle"] = -1809252352},
  [24] = {["x"] = 3694, ["y"] = 287, ["z"] = 643, ["order"] = 5, ["angle"] = -1797718016},
  [25] = {["x"] = 2370, ["y"] = -289, ["z"] = 507, ["order"] = 5, ["angle"] = -2076246016},
  [26] = {["x"] = 1325, ["y"] = -275, ["z"] = 400, ["order"] = 5, ["angle"] = -2113601536},
  [27] = {["x"] = -91, ["y"] = -339, ["z"] = 256, ["order"] = 6, ["angle"] = -1891827712},
  [28] = {["x"] = -1279, ["y"] = -516, ["z"] = 256, ["order"] = 6, ["angle"] = -1518272512},
  [29] = {["x"] = -2178, ["y"] = -1228, ["z"] = 256, ["order"] = 6, ["angle"] = -1054539776},
  [30] = {["x"] = -2307, ["y"] = -5029, ["z"] = 0, ["order"] = 8, ["angle"] = -1508835328},
  [31] = {["x"] = -3285, ["y"] = -5831, ["z"] = 0, ["order"] = 8, ["angle"] = -2073755648},
  [32] = {["x"] = -4785, ["y"] = -5993, ["z"] = 0, ["order"] = 9, ["angle"] = -2133262336},
  [33] = {["x"] = -5836, ["y"] = -6024, ["z"] = 137, ["order"] = 9, ["angle"] = -2133262336},
  [34] = {["x"] = -7074, ["y"] = -5995, ["z"] = 369, ["order"] = 10, ["angle"] = -1442512896},
  [35] = {["x"] = -7779, ["y"] = -6010, ["z"] = 501, ["order"] = 10, ["angle"] = -1025966080},
  [36] = {["x"] = -7693, ["y"] = -6741, ["z"] = 485, ["order"] = 10, ["angle"] = -1009451008},
  [37] = {["x"] = -7376, ["y"] = -7869, ["z"] = 426, ["order"] = 10, ["angle"] = -922812416},
  [38] = {["x"] = -7246, ["y"] = -9383, ["z"] = 401, ["order"] = 10, ["angle"] = -893190144},
  [39] = {["x"] = -6770, ["y"] = -10401, ["z"] = 312, ["order"] = 11, ["angle"] = -53149696},
  [40] = {["x"] = -5856, ["y"] = -10392, ["z"] = 141, ["order"] = 11, ["angle"] = -43843584},
  [41] = {["x"] = -4756, ["y"] = -10400, ["z"] = 0, ["order"] = 12, ["angle"] = 122748928},
  [42] = {["x"] = -3160, ["y"] = -10112, ["z"] = 84, ["order"] = 12, ["angle"] = 270467072},
  [43] = {["x"] = -1373, ["y"] = -9274, ["z"] = 173, ["order"] = 12, ["angle"] = 515964928},
  [44] = {["x"] = -385, ["y"] = -8304, ["z"] = 174, ["order"] = 12, ["angle"] = 867368960},
  [45] = {["x"] = 149, ["y"] = -6920, ["z"] = 95, ["order"] = 12, ["angle"] = 1035665408},
  [46] = {["x"] = 382, ["y"] = -4833, ["z"] = 0, ["order"] = 13, ["angle"] = 1079574528},
  [47] = {["x"] = 377, ["y"] = -2475, ["z"] = 0, ["order"] = 13, ["angle"] = 1080885248},
  [48] = {["x"] = 364, ["y"] = -43, ["z"] = 0, ["order"] = 13, ["angle"] = 1099104256},
  [49] = {["x"] = 380, ["y"] = 1794, ["z"] = 0, ["order"] = 13, ["angle"] = 1047461888},
  [50] = {["x"] = 398, ["y"] = 3612, ["z"] = 0, ["order"] = 13, ["angle"] = 1092419584},
  [51] = {["x"] = 4231, ["y"] = 2187, ["z"] = 698, ["order"] = 4, ["angle"] = -346095616},
  [52] = {["x"] = 5822, ["y"] = 1303, ["z"] = 861, ["order"] = 5, ["angle"] = -1388249088},
  [53] = {["x"] = 4995, ["y"] = 571, ["z"] = 776, ["order"] = 5, ["angle"] = -1897070592},
  [54] = {["x"] = -8382, ["y"] = -6672, ["z"] = 614, ["order"] = 10, ["angle"] = -984416256},
  [55] = {["x"] = -8275, ["y"] = -7755, ["z"] = 594, ["order"] = 10, ["angle"] = -995819520},
  [56] = {["x"] = -8267, ["y"] = -9277, ["z"] = 593, ["order"] = 10, ["angle"] = -971177984},
  [57] = {["x"] = -8197, ["y"] = -10225, ["z"] = 580, ["order"] = 11, ["angle"] = -109510656},
  [58] = {["x"] = -7493, ["y"] = -10481, ["z"] = 447, ["order"] = 11, ["angle"] = 71237632},
  [59] = {["x"] = -3907, ["y"] = -10296, ["z"] = 31, ["order"] = 12, ["angle"] = 169148416},
  [60] = {["x"] = -2179, ["y"] = -9982, ["z"] = 164, ["order"] = 12, ["angle"] = 415301632},
  [61] = {["x"] = -754, ["y"] = -8958, ["z"] = 201, ["order"] = 12, ["angle"] = 712310784},
  [62] = {["x"] = 26, ["y"] = -7699, ["z"] = 156, ["order"] = 12, ["angle"] = 990183424},
  [63] = {["x"] = 645, ["y"] = -4853, ["z"] = 0, ["order"] = 13, ["angle"] = 1084817408},
  [64] = {["x"] = 653, ["y"] = -2730, ["z"] = 0, ["order"] = 13, ["angle"] = 1072496640},
  [65] = {["x"] = 659, ["y"] = -134, ["z"] = 0, ["order"] = 13, ["angle"] = 1072496640},
  [66] = {["x"] = 418, ["y"] = 2786, ["z"] = 0, ["order"] = 13, ["angle"] = 1047068672},
  [67] = {["x"] = -7682, ["y"] = -9211, ["z"] = 483, ["order"] = 10, ["angle"] = -1025835008},
  [68] = {["x"] = -7100, ["y"] = -8383, ["z"] = 374, ["order"] = 10, ["angle"] = -1022427136},
  [69] = {["x"] = -8297, ["y"] = -8620, ["z"] = 598, ["order"] = 10, ["angle"] = -1040515072},
  [70] = {["x"] = 123, ["y"] = -4813, ["z"] = 0, ["order"] = 13, ["angle"] = 1065287680},
  [71] = {["x"] = 119, ["y"] = -2411, ["z"] = 0, ["order"] = 13, ["angle"] = 1066598400},
  [72] = {["x"] = 128, ["y"] = -34, ["z"] = 0, ["order"] = 13, ["angle"] = 1078788096}
}
-- Respawn definitions for SONIC SPEEDWAY (MAP07)
krespawn.points[7] = {
  [0] = 75,
  [1] = {["x"] = 8467, ["y"] = -2590, ["z"] = 0, ["order"] = 0, ["angle"] = 1120600064},
  [2] = {["x"] = 8519, ["y"] = -1635, ["z"] = 0, ["order"] = 0, ["angle"] = 1242628096},
  [3] = {["x"] = 8420, ["y"] = -439, ["z"] = 0, ["order"] = 0, ["angle"] = 1518534656},
  [4] = {["x"] = 7918, ["y"] = 529, ["z"] = 0, ["order"] = 0, ["angle"] = 1882652672},
  [5] = {["x"] = 7091, ["y"] = 1119, ["z"] = 0, ["order"] = 0, ["angle"] = -2055667712},
  [6] = {["x"] = 6196, ["y"] = 1169, ["z"] = 0, ["order"] = 1, ["angle"] = -1724841984},
  [7] = {["x"] = 5316, ["y"] = 682, ["z"] = 0, ["order"] = 1, ["angle"] = -1428094976},
  [8] = {["x"] = 4934, ["y"] = -11, ["z"] = 0, ["order"] = 1, ["angle"] = -1413545984},
  [9] = {["x"] = 4609, ["y"] = -1129, ["z"] = 0, ["order"] = 1, ["angle"] = -1728905216},
  [10] = {["x"] = 3830, ["y"] = -1735, ["z"] = 0, ["order"] = 1, ["angle"] = -2072444928},
  [11] = {["x"] = 2381, ["y"] = -1918, ["z"] = 0, ["order"] = 1, ["angle"] = -2124611584},
  [12] = {["x"] = 911, ["y"] = -1942, ["z"] = 0, ["order"] = 1, ["angle"] = 2143617024},
  [13] = {["x"] = -900, ["y"] = -1733, ["z"] = 144, ["order"] = 2, ["angle"] = 2134179840},
  [14] = {["x"] = -905, ["y"] = -2125, ["z"] = 145, ["order"] = 2, ["angle"] = 2098790400},
  [15] = {["x"] = -2614, ["y"] = -1909, ["z"] = 256, ["order"] = 2, ["angle"] = 2147155968},
  [16] = {["x"] = -3722, ["y"] = -1904, ["z"] = 256, ["order"] = 2, ["angle"] = 2147155968},
  [17] = {["x"] = -4609, ["y"] = -1902, ["z"] = 214, ["order"] = 2, ["angle"] = 2147155968},
  [18] = {["x"] = -5573, ["y"] = -1902, ["z"] = 107, ["order"] = 2, ["angle"] = 2147155968},
  [19] = {["x"] = -6643, ["y"] = -1936, ["z"] = 0, ["order"] = 2, ["angle"] = 1701249024},
  [20] = {["x"] = -7827, ["y"] = -1356, ["z"] = 0, ["order"] = 2, ["angle"] = 1348403200},
  [21] = {["x"] = -8233, ["y"] = -699, ["z"] = 0, ["order"] = 2, ["angle"] = 1131741184},
  [22] = {["x"] = -8403, ["y"] = 267, ["z"] = 0, ["order"] = 3, ["angle"] = 1052442624},
  [23] = {["x"] = -8350, ["y"] = 1152, ["z"] = 0, ["order"] = 3, ["angle"] = 944570368},
  [24] = {["x"] = -8104, ["y"] = 2041, ["z"] = 0, ["order"] = 3, ["angle"] = 991363072},
  [25] = {["x"] = -7870, ["y"] = 3061, ["z"] = 0, ["order"] = 3, ["angle"] = 1196359680},
  [26] = {["x"] = -8044, ["y"] = 3883, ["z"] = 0, ["order"] = 3, ["angle"] = 1269497856},
  [27] = {["x"] = -8341, ["y"] = 4548, ["z"] = 0, ["order"] = 3, ["angle"] = 1034485760},
  [28] = {["x"] = -8419, ["y"] = 5116, ["z"] = 0, ["order"] = 3, ["angle"] = 778108928},
  [29] = {["x"] = -8230, ["y"] = 6271, ["z"] = 0, ["order"] = 3, ["angle"] = 466288640},
  [30] = {["x"] = -7511, ["y"] = 6956, ["z"] = 0, ["order"] = 3, ["angle"] = 82903040},
  [31] = {["x"] = -6268, ["y"] = 7365, ["z"] = 0, ["order"] = 4, ["angle"] = -46858240},
  [32] = {["x"] = -4771, ["y"] = 7476, ["z"] = 0, ["order"] = 4, ["angle"] = -149094400},
  [33] = {["x"] = -3659, ["y"] = 7354, ["z"] = 0, ["order"] = 4, ["angle"] = -745603072},
  [34] = {["x"] = -3126, ["y"] = 6705, ["z"] = 0, ["order"] = 4, ["angle"] = -1070137344},
  [35] = {["x"] = -3044, ["y"] = 5636, ["z"] = 0, ["order"] = 5, ["angle"] = -1082720256},
  [36] = {["x"] = -2932, ["y"] = 4451, ["z"] = -151, ["order"] = 5, ["angle"] = -1342767104},
  [37] = {["x"] = -3169, ["y"] = 3509, ["z"] = -256, ["order"] = 5, ["angle"] = -1710424064},
  [38] = {["x"] = -3816, ["y"] = 3091, ["z"] = -256, ["order"] = 5, ["angle"] = -1749090304},
  [39] = {["x"] = -4739, ["y"] = 2856, ["z"] = -256, ["order"] = 6, ["angle"] = -1595342848},
  [40] = {["x"] = -5687, ["y"] = 2305, ["z"] = -256, ["order"] = 6, ["angle"] = -796196864},
  [41] = {["x"] = -5159, ["y"] = 1401, ["z"] = -256, ["order"] = 6, ["angle"] = -265093120},
  [42] = {["x"] = -3992, ["y"] = 1220, ["z"] = -256, ["order"] = 7, ["angle"] = -649265152},
  [43] = {["x"] = -3350, ["y"] = 482, ["z"] = -256, ["order"] = 7, ["angle"] = -1030422528},
  [44] = {["x"] = -3214, ["y"] = -232, ["z"] = -256, ["order"] = 7, ["angle"] = -1030422528},
  [45] = {["x"] = -3134, ["y"] = -1202, ["z"] = -256, ["order"] = 7, ["angle"] = -1030422528},
  [46] = {["x"] = -3119, ["y"] = -1893, ["z"] = -256, ["order"] = 7, ["angle"] = -1075380224},
  [47] = {["x"] = -3120, ["y"] = -2520, ["z"] = -256, ["order"] = 7, ["angle"] = -1075380224},
  [48] = {["x"] = -3114, ["y"] = -3217, ["z"] = -256, ["order"] = 7, ["angle"] = -1023213568},
  [49] = {["x"] = -3090, ["y"] = -3968, ["z"] = -256, ["order"] = 7, ["angle"] = -1041432576},
  [50] = {["x"] = -3079, ["y"] = -4914, ["z"] = -256, ["order"] = 7, ["angle"] = -1071316992},
  [51] = {["x"] = -3066, ["y"] = -5874, ["z"] = -256, ["order"] = 8, ["angle"] = -786104320},
  [52] = {["x"] = -3058, ["y"] = -6651, ["z"] = -256, ["order"] = 8, ["angle"] = -333250560},
  [53] = {["x"] = -2698, ["y"] = -7108, ["z"] = -256, ["order"] = 8, ["angle"] = 110034944},
  [54] = {["x"] = -2318, ["y"] = -7158, ["z"] = -256, ["order"] = 8, ["angle"] = -25755648},
  [55] = {["x"] = -1722, ["y"] = -7181, ["z"] = -256, ["order"] = 8, ["angle"] = -25755648},
  [56] = {["x"] = -985, ["y"] = -7210, ["z"] = -256, ["order"] = 8, ["angle"] = 158138368},
  [57] = {["x"] = -259, ["y"] = -7224, ["z"] = -256, ["order"] = 8, ["angle"] = 483065856},
  [58] = {["x"] = 170, ["y"] = -6988, ["z"] = -256, ["order"] = 8, ["angle"] = 883490816},
  [59] = {["x"] = 168, ["y"] = -6169, ["z"] = -256, ["order"] = 9, ["angle"] = 829095936},
  [60] = {["x"] = 372, ["y"] = -4874, ["z"] = -198, ["order"] = 9, ["angle"] = 414646272},
  [61] = {["x"] = 1011, ["y"] = -4108, ["z"] = -96, ["order"] = 10, ["angle"] = 6356992},
  [62] = {["x"] = 1935, ["y"] = -3872, ["z"] = -96, ["order"] = 10, ["angle"] = -456327168},
  [63] = {["x"] = 2906, ["y"] = -4252, ["z"] = -90, ["order"] = 10, ["angle"] = -741408768},
  [64] = {["x"] = 3292, ["y"] = -4971, ["z"] = -25, ["order"] = 10, ["angle"] = -949944320},
  [65] = {["x"] = 3229, ["y"] = -5905, ["z"] = 0, ["order"] = 10, ["angle"] = -935395328},
  [66] = {["x"] = 3212, ["y"] = -6046, ["z"] = 0, ["order"] = 10, ["angle"] = -839581696},
  [67] = {["x"] = 3365, ["y"] = -6634, ["z"] = 0, ["order"] = 10, ["angle"] = -423821312},
  [68] = {["x"] = 3880, ["y"] = -7182, ["z"] = 0, ["order"] = 10, ["angle"] = 10944512},
  [69] = {["x"] = 5047, ["y"] = -7407, ["z"] = 0, ["order"] = 11, ["angle"] = 335347712},
  [70] = {["x"] = 6172, ["y"] = -7096, ["z"] = 0, ["order"] = 11, ["angle"] = 580190208},
  [71] = {["x"] = 7213, ["y"] = -6471, ["z"] = 0, ["order"] = 11, ["angle"] = 602210304},
  [72] = {["x"] = 7910, ["y"] = -5952, ["z"] = 0, ["order"] = 11, ["angle"] = 909049856},
  [73] = {["x"] = 8302, ["y"] = -4920, ["z"] = 0, ["order"] = 11, ["angle"] = 1213267968},
  [74] = {["x"] = 8375, ["y"] = -3623, ["z"] = 0, ["order"] = 11, ["angle"] = 996212736},
  [75] = {["x"] = 8430, ["y"] = -3093, ["z"] = 0, ["order"] = 11, ["angle"] = 996212736}
}
-- Respawn definitions for HILL TOP (MAP08)
krespawn.points[8] = {
  [0] = 62,
  [1] = {["x"] = -65, ["y"] = -2667, ["z"] = 0, ["order"] = 0, ["angle"] = 1067646976},
  [2] = {["x"] = -70, ["y"] = 82, ["z"] = 0, ["order"] = 0, ["angle"] = 1078001664},
  [3] = {["x"] = -211, ["y"] = 1007, ["z"] = 0, ["order"] = 0, ["angle"] = 1273561088},
  [4] = {["x"] = 192, ["y"] = 932, ["z"] = 0, ["order"] = 0, ["angle"] = 850198528},
  [5] = {["x"] = 701, ["y"] = 2091, ["z"] = 0, ["order"] = 0, ["angle"] = 1051394048},
  [6] = {["x"] = 746, ["y"] = 3132, ["z"] = 0, ["order"] = 0, ["angle"] = 1184038912},
  [7] = {["x"] = 696, ["y"] = 4186, ["z"] = 0, ["order"] = 0, ["angle"] = 1341063168},
  [8] = {["x"] = -102, ["y"] = 4913, ["z"] = 0, ["order"] = 0, ["angle"] = 1084424192},
  [9] = {["x"] = -558, ["y"] = 2010, ["z"] = 0, ["order"] = 0, ["angle"] = 1115095040},
  [10] = {["x"] = -591, ["y"] = 2822, ["z"] = 0, ["order"] = 0, ["angle"] = 1079836672},
  [11] = {["x"] = -581, ["y"] = 3539, ["z"] = 0, ["order"] = 0, ["angle"] = 1004732416},
  [12] = {["x"] = -528, ["y"] = 4453, ["z"] = 0, ["order"] = 0, ["angle"] = 842072064},
  [13] = {["x"] = -31, ["y"] = 5894, ["z"] = 20, ["order"] = 1, ["angle"] = 1076166656},
  [14] = {["x"] = 141, ["y"] = 6965, ["z"] = 73, ["order"] = 1, ["angle"] = 1485766656},
  [15] = {["x"] = -95, ["y"] = 7910, ["z"] = 121, ["order"] = 1, ["angle"] = 1878720512},
  [16] = {["x"] = -1208, ["y"] = 8382, ["z"] = 144, ["order"] = 1, ["angle"] = -2124087296},
  [17] = {["x"] = -2616, ["y"] = 8404, ["z"] = 168, ["order"] = 2, ["angle"] = -2143879168},
  [18] = {["x"] = -3791, ["y"] = 8569, ["z"] = 168, ["order"] = 2, ["angle"] = -1625096192},
  [19] = {["x"] = -4678, ["y"] = 7999, ["z"] = 168, ["order"] = 2, ["angle"] = -1180631040},
  [20] = {["x"] = -4808, ["y"] = 6546, ["z"] = 168, ["order"] = 2, ["angle"] = -1061486592},
  [21] = {["x"] = -4848, ["y"] = 5464, ["z"] = 168, ["order"] = 2, ["angle"] = -777191424},
  [22] = {["x"] = -4787, ["y"] = 4565, ["z"] = 168, ["order"] = 2, ["angle"] = -400621568},
  [23] = {["x"] = -3686, ["y"] = 3772, ["z"] = 168, ["order"] = 3, ["angle"] = -11337728},
  [24] = {["x"] = -1838, ["y"] = 3747, ["z"] = 507, ["order"] = 3, ["angle"] = 17235968},
  [25] = {["x"] = -2618, ["y"] = 3734, ["z"] = 345, ["order"] = 3, ["angle"] = 29425664},
  [26] = {["x"] = 1566, ["y"] = 3780, ["z"] = 384, ["order"] = 3, ["angle"] = 12255232},
  [27] = {["x"] = 2927, ["y"] = 3797, ["z"] = 195, ["order"] = 3, ["angle"] = 20905984},
  [28] = {["x"] = 4160, ["y"] = 3839, ["z"] = 0, ["order"] = 3, ["angle"] = -151977984},
  [29] = {["x"] = 5410, ["y"] = 3805, ["z"] = 0, ["order"] = 3, ["angle"] = -593559552},
  [30] = {["x"] = 6322, ["y"] = 3149, ["z"] = 0, ["order"] = 3, ["angle"] = -1348796416},
  [31] = {["x"] = 6264, ["y"] = 2284, ["z"] = 0, ["order"] = 3, ["angle"] = -1868890112},
  [32] = {["x"] = 5215, ["y"] = 2026, ["z"] = 0, ["order"] = 4, ["angle"] = -1709375488},
  [33] = {["x"] = 4362, ["y"] = 1654, ["z"] = 75, ["order"] = 4, ["angle"] = -1374879744},
  [34] = {["x"] = 3806, ["y"] = 857, ["z"] = 236, ["order"] = 4, ["angle"] = -554631168},
  [35] = {["x"] = 4375, ["y"] = 280, ["z"] = 71, ["order"] = 4, ["angle"] = -221052928},
  [36] = {["x"] = 5415, ["y"] = 152, ["z"] = 50, ["order"] = 4, ["angle"] = -507576320},
  [37] = {["x"] = 6088, ["y"] = -389, ["z"] = 207, ["order"] = 4, ["angle"] = -1068040192},
  [38] = {["x"] = 6322, ["y"] = -1603, ["z"] = 261, ["order"] = 4, ["angle"] = -1794572288},
  [39] = {["x"] = 5720, ["y"] = -2304, ["z"] = 121, ["order"] = 4, ["angle"] = -2073624576},
  [40] = {["x"] = 4464, ["y"] = -2426, ["z"] = 0, ["order"] = 5, ["angle"] = -2128019456},
  [41] = {["x"] = 2815, ["y"] = -2500, ["z"] = -461, ["order"] = 5, ["angle"] = -2141782016},
  [42] = {["x"] = 1950, ["y"] = -2510, ["z"] = -512, ["order"] = 5, ["angle"] = -2141782016},
  [43] = {["x"] = 1044, ["y"] = -2517, ["z"] = -512, ["order"] = 5, ["angle"] = -2141782016},
  [44] = {["x"] = 2, ["y"] = -2525, ["z"] = -512, ["order"] = 5, ["angle"] = -2141782016},
  [45] = {["x"] = -1226, ["y"] = -2534, ["z"] = -512, ["order"] = 5, ["angle"] = -2141782016},
  [46] = {["x"] = -2024, ["y"] = -2541, ["z"] = -512, ["order"] = 5, ["angle"] = -2141782016},
  [47] = {["x"] = -2825, ["y"] = -2574, ["z"] = -415, ["order"] = 5, ["angle"] = 2120286208},
  [48] = {["x"] = -3474, ["y"] = -2494, ["z"] = -320, ["order"] = 5, ["angle"] = -2110849024},
  [49] = {["x"] = -4146, ["y"] = -2418, ["z"] = -223, ["order"] = 6, ["angle"] = -1897463808},
  [50] = {["x"] = -4719, ["y"] = -2604, ["z"] = -139, ["order"] = 6, ["angle"] = -1511194624},
  [51] = {["x"] = -5064, ["y"] = -3026, ["z"] = -89, ["order"] = 6, ["angle"] = -1150877696},
  [52] = {["x"] = -5204, ["y"] = -3789, ["z"] = -69, ["order"] = 6, ["angle"] = -794230784},
  [53] = {["x"] = -5224, ["y"] = -4434, ["z"] = 0, ["order"] = 6, ["angle"] = -616103936},
  [54] = {["x"] = -4875, ["y"] = -5014, ["z"] = 0, ["order"] = 6, ["angle"] = -148176896},
  [55] = {["x"] = -4320, ["y"] = -5324, ["z"] = 0, ["order"] = 6, ["angle"] = 25624576},
  [56] = {["x"] = -3511, ["y"] = -5378, ["z"] = 0, ["order"] = 6, ["angle"] = -4521984},
  [57] = {["x"] = -2705, ["y"] = -5417, ["z"] = 0, ["order"] = 7, ["angle"] = 121044992},
  [58] = {["x"] = -1914, ["y"] = -5460, ["z"] = 0, ["order"] = 7, ["angle"] = 326172672},
  [59] = {["x"] = -1146, ["y"] = -5313, ["z"] = 0, ["order"] = 7, ["angle"] = 629342208},
  [60] = {["x"] = -522, ["y"] = -4881, ["z"] = 0, ["order"] = 7, ["angle"] = 823984128},
  [61] = {["x"] = -81, ["y"] = -3679, ["z"] = 0, ["order"] = 7, ["angle"] = 1060962304},
  [62] = {["x"] = -74, ["y"] = -3244, ["z"] = 0, ["order"] = 7, ["angle"] = 1068040192}
}
-- Respawn definitions for MISTY MAZE (MAP09)
krespawn.points[9] = {
  [0] = 94,
  [1] = {["x"] = 80, ["y"] = 806, ["z"] = 0, ["order"] = 0, ["angle"] = 1073020928},
  [2] = {["x"] = 81, ["y"] = 1412, ["z"] = 0, ["order"] = 0, ["angle"] = 1073020928},
  [3] = {["x"] = 82, ["y"] = 2411, ["z"] = -32, ["order"] = 0, ["angle"] = 1073020928},
  [4] = {["x"] = 84, ["y"] = 3648, ["z"] = -32, ["order"] = 0, ["angle"] = 1073020928},
  [5] = {["x"] = 162, ["y"] = 4893, ["z"] = -32, ["order"] = 1, ["angle"] = 1710161920},
  [6] = {["x"] = -365, ["y"] = 5580, ["z"] = -32, ["order"] = 1, ["angle"] = -2125398016},
  [7] = {["x"] = -1254, ["y"] = 5624, ["z"] = -32, ["order"] = 1, ["angle"] = 2038759424},
  [8] = {["x"] = -2292, ["y"] = 5563, ["z"] = -32, ["order"] = 1, ["angle"] = 1796407296},
  [9] = {["x"] = -3173, ["y"] = 5833, ["z"] = -32, ["order"] = 1, ["angle"] = 1336868864},
  [10] = {["x"] = -3580, ["y"] = 6680, ["z"] = -32, ["order"] = 1, ["angle"] = 1003552768},
  [11] = {["x"] = -3651, ["y"] = 7734, ["z"] = -32, ["order"] = 2, ["angle"] = 801177600},
  [12] = {["x"] = -3256, ["y"] = 8808, ["z"] = -32, ["order"] = 2, ["angle"] = 6750208},
  [13] = {["x"] = -2317, ["y"] = 8919, ["z"] = -288, ["order"] = 2, ["angle"] = -38862848},
  [14] = {["x"] = -1536, ["y"] = 8833, ["z"] = -288, ["order"] = 2, ["angle"] = 33226752},
  [15] = {["x"] = -525, ["y"] = 8736, ["z"] = -288, ["order"] = 2, ["angle"] = 151453696},
  [16] = {["x"] = 599, ["y"] = 8824, ["z"] = -288, ["order"] = 2, ["angle"] = 657260544},
  [17] = {["x"] = 1311, ["y"] = 9339, ["z"] = -288, ["order"] = 3, ["angle"] = 1048903680},
  [18] = {["x"] = 1571, ["y"] = 10238, ["z"] = -288, ["order"] = 3, ["angle"] = 1498218496},
  [19] = {["x"] = 1356, ["y"] = 11077, ["z"] = -288, ["order"] = 4, ["angle"] = 1862991872},
  [20] = {["x"] = 638, ["y"] = 11585, ["z"] = -288, ["order"] = 4, ["angle"] = -2101018624},
  [21] = {["x"] = -1409, ["y"] = 11514, ["z"] = -31, ["order"] = 4, ["angle"] = -2144534528},
  [22] = {["x"] = -2418, ["y"] = 11127, ["z"] = 110, ["order"] = 4, ["angle"] = -2142961664},
  [23] = {["x"] = -2458, ["y"] = 11904, ["z"] = 115, ["order"] = 4, ["angle"] = 2140340224},
  [24] = {["x"] = -3685, ["y"] = 11437, ["z"] = 287, ["order"] = 4, ["angle"] = 2127364096},
  [25] = {["x"] = -4700, ["y"] = 11485, ["z"] = 425, ["order"] = 4, ["angle"] = 2146762752},
  [26] = {["x"] = -5770, ["y"] = 11508, ["z"] = 480, ["order"] = 5, ["angle"] = -2141519872},
  [27] = {["x"] = -6563, ["y"] = 11551, ["z"] = 480, ["order"] = 5, ["angle"] = -2128674816},
  [28] = {["x"] = -7357, ["y"] = 11593, ["z"] = 480, ["order"] = 5, ["angle"] = -1792606208},
  [29] = {["x"] = -8256, ["y"] = 11385, ["z"] = 480, ["order"] = 5, ["angle"] = -1269891072},
  [30] = {["x"] = -8506, ["y"] = 10580, ["z"] = 480, ["order"] = 5, ["angle"] = -1095565312},
  [31] = {["x"] = -8522, ["y"] = 9425, ["z"] = 428, ["order"] = 5, ["angle"] = -1061093376},
  [32] = {["x"] = -8527, ["y"] = 8165, ["z"] = 224, ["order"] = 5, ["angle"] = -1085472768},
  [33] = {["x"] = -8539, ["y"] = 7142, ["z"] = 147, ["order"] = 5, ["angle"] = -1085472768},
  [34] = {["x"] = -8485, ["y"] = 6041, ["z"] = 96, ["order"] = 5, ["angle"] = -1076035584},
  [35] = {["x"] = -8656, ["y"] = 5133, ["z"] = 96, ["order"] = 5, ["angle"] = -1930625024},
  [36] = {["x"] = -9359, ["y"] = 4590, ["z"] = 96, ["order"] = 6, ["angle"] = -1987379200},
  [37] = {["x"] = -10232, ["y"] = 4482, ["z"] = 96, ["order"] = 6, ["angle"] = -1678573568},
  [38] = {["x"] = -10967, ["y"] = 4031, ["z"] = 96, ["order"] = 6, ["angle"] = -1200029696},
  [39] = {["x"] = -11032, ["y"] = 3055, ["z"] = 96, ["order"] = 6, ["angle"] = -1073938432},
  [40] = {["x"] = -11132, ["y"] = 2038, ["z"] = 96, ["order"] = 6, ["angle"] = -1205796864},
  [41] = {["x"] = -11379, ["y"] = 1093, ["z"] = 96, ["order"] = 6, ["angle"] = -881393664},
  [42] = {["x"] = -11304, ["y"] = 85, ["z"] = 96, ["order"] = 6, ["angle"] = -659226624},
  [43] = {["x"] = -11030, ["y"] = -789, ["z"] = 96, ["order"] = 6, ["angle"] = -335347712},
  [44] = {["x"] = -10405, ["y"] = -1002, ["z"] = 96, ["order"] = 6, ["angle"] = -264962048},
  [45] = {["x"] = -9320, ["y"] = -1202, ["z"] = 96, ["order"] = 6, ["angle"] = -612564992},
  [46] = {["x"] = -8987, ["y"] = -1919, ["z"] = 66, ["order"] = 6, ["angle"] = -865009664},
  [47] = {["x"] = -8521, ["y"] = -3095, ["z"] = -169, ["order"] = 7, ["angle"] = -1054015488},
  [48] = {["x"] = -8515, ["y"] = -4039, ["z"] = -392, ["order"] = 7, ["angle"] = -1082458112},
  [49] = {["x"] = -11030, ["y"] = -11192, ["z"] = -456, ["order"] = 7, ["angle"] = -2109014016},
  [50] = {["x"] = -12940, ["y"] = -11087, ["z"] = -456, ["order"] = 7, ["angle"] = -2114125824},
  [51] = {["x"] = -14494, ["y"] = -11138, ["z"] = -406, ["order"] = 8, ["angle"] = -1396244480},
  [52] = {["x"] = -15128, ["y"] = -11667, ["z"] = -302, ["order"] = 8, ["angle"] = -425656320},
  [53] = {["x"] = -14309, ["y"] = -12311, ["z"] = -436, ["order"] = 8, ["angle"] = -43974656},
  [54] = {["x"] = -13411, ["y"] = -12471, ["z"] = -456, ["order"] = 8, ["angle"] = 26017792},
  [55] = {["x"] = -10105, ["y"] = -12562, ["z"] = 480, ["order"] = 9, ["angle"] = 4259840},
  [56] = {["x"] = -9204, ["y"] = -12539, ["z"] = 480, ["order"] = 9, ["angle"] = 4259840},
  [57] = {["x"] = -8236, ["y"] = -12532, ["z"] = 480, ["order"] = 9, ["angle"] = 4259840},
  [58] = {["x"] = -6993, ["y"] = -12524, ["z"] = 480, ["order"] = 9, ["angle"] = 4259840},
  [59] = {["x"] = -5703, ["y"] = -12516, ["z"] = 480, ["order"] = 9, ["angle"] = 4259840},
  [60] = {["x"] = -4368, ["y"] = -12508, ["z"] = 480, ["order"] = 9, ["angle"] = 228655104},
  [61] = {["x"] = -3447, ["y"] = -12341, ["z"] = 480, ["order"] = 9, ["angle"] = 719126528},
  [62] = {["x"] = -2981, ["y"] = -11570, ["z"] = 480, ["order"] = 9, ["angle"] = 1002504192},
  [63] = {["x"] = -2932, ["y"] = -10312, ["z"] = 480, ["order"] = 9, ["angle"] = 1075904512},
  [64] = {["x"] = -3014, ["y"] = -9369, ["z"] = 480, ["order"] = 9, ["angle"] = 639303680},
  [65] = {["x"] = -2577, ["y"] = -8728, ["z"] = 480, ["order"] = 9, ["angle"] = 205586432},
  [66] = {["x"] = -1753, ["y"] = -8552, ["z"] = 480, ["order"] = 9, ["angle"] = 171769856},
  [67] = {["x"] = -729, ["y"] = -8492, ["z"] = 480, ["order"] = 9, ["angle"] = 554106880},
  [68] = {["x"] = -119, ["y"] = -8060, ["z"] = 480, ["order"] = 9, ["angle"] = 1071054848},
  [69] = {["x"] = 91, ["y"] = -7209, ["z"] = 480, ["order"] = 9, ["angle"] = 1070661632},
  [70] = {["x"] = 102, ["y"] = -5792, ["z"] = 480, ["order"] = 9, ["angle"] = 1073807360},
  [71] = {["x"] = 59, ["y"] = -4049, ["z"] = -32, ["order"] = 9, ["angle"] = 1078788096},
  [72] = {["x"] = 61, ["y"] = -3224, ["z"] = -32, ["order"] = 9, ["angle"] = 1077739520},
  [73] = {["x"] = 58, ["y"] = -2418, ["z"] = -32, ["order"] = 9, ["angle"] = 1077739520},
  [74] = {["x"] = 53, ["y"] = -1285, ["z"] = 0, ["order"] = 9, ["angle"] = 1077739520},
  [75] = {["x"] = 75, ["y"] = -132, ["z"] = 0, ["order"] = 9, ["angle"] = 1126498304},
  [76] = {["x"] = -2469, ["y"] = 11483, ["z"] = 117, ["order"] = 4, ["angle"] = -2131296256},
  [77] = {["x"] = -8296, ["y"] = 5325, ["z"] = 96, ["order"] = 5, ["angle"] = -537853952},
  [78] = {["x"] = -7753, ["y"] = 4684, ["z"] = 96, ["order"] = 6, ["angle"] = -264699904},
  [79] = {["x"] = -6663, ["y"] = 4559, ["z"] = 96, ["order"] = 6, ["angle"] = -516489216},
  [80] = {["x"] = -6089, ["y"] = 4161, ["z"] = 96, ["order"] = 6, ["angle"] = -904855552},
  [81] = {["x"] = -5978, ["y"] = 3438, ["z"] = 96, ["order"] = 6, ["angle"] = -1069088768},
  [82] = {["x"] = -6019, ["y"] = 2561, ["z"] = 96, ["order"] = 6, ["angle"] = -770244608},
  [83] = {["x"] = -5631, ["y"] = 1544, ["z"] = 96, ["order"] = 6, ["angle"] = -1135542272},
  [84] = {["x"] = -5609, ["y"] = 404, ["z"] = 96, ["order"] = 6, ["angle"] = -1446182912},
  [85] = {["x"] = -5810, ["y"] = -608, ["z"] = 96, ["order"] = 6, ["angle"] = -1706491904},
  [86] = {["x"] = -6417, ["y"] = -897, ["z"] = 96, ["order"] = 6, ["angle"] = -1817509888},
  [87] = {["x"] = -7212, ["y"] = -963, ["z"] = 96, ["order"] = 6, ["angle"] = -1808728064},
  [88] = {["x"] = -7900, ["y"] = -1160, ["z"] = 96, ["order"] = 6, ["angle"] = -1248788480},
  [89] = {["x"] = -8108, ["y"] = -1931, ["z"] = 64, ["order"] = 6, ["angle"] = -1120600064},
  [90] = {["x"] = -8345, ["y"] = -3087, ["z"] = -167, ["order"] = 7, ["angle"] = -1163591680},
  [91] = {["x"] = -11543, ["y"] = -11225, ["z"] = -456, ["order"] = 7, ["angle"] = 2027356160},
  [92] = {["x"] = -12669, ["y"] = -11167, ["z"] = -456, ["order"] = 7, ["angle"] = 2035089408},
  [93] = {["x"] = -13487, ["y"] = -11099, ["z"] = -456, ["order"] = 8, ["angle"] = -2044788736},
  [94] = {["x"] = -14967, ["y"] = -12087, ["z"] = -328, ["order"] = 8, ["angle"] = -148832256}
}
-- Respawn definitions for GRAND METROPOLIS (MAP10)
krespawn.points[10] = {
  [0] = 115,
  [1] = {["x"] = 588, ["y"] = -2883, ["z"] = 0, ["order"] = 0, ["angle"] = 1087963136},
  [2] = {["x"] = 567, ["y"] = -1959, ["z"] = 0, ["order"] = 0, ["angle"] = 1064763392},
  [3] = {["x"] = 574, ["y"] = -729, ["z"] = 0, ["order"] = 0, ["angle"] = 1064763392},
  [4] = {["x"] = 551, ["y"] = 483, ["z"] = -8, ["order"] = 0, ["angle"] = 1021378560},
  [5] = {["x"] = 530, ["y"] = 2472, ["z"] = -8, ["order"] = 0, ["angle"] = 615972864},
  [6] = {["x"] = 1028, ["y"] = 3805, ["z"] = -29, ["order"] = 1, ["angle"] = 118030336},
  [7] = {["x"] = 2415, ["y"] = 4232, ["z"] = -67, ["order"] = 1, ["angle"] = -661061632},
  [8] = {["x"] = 3276, ["y"] = 3633, ["z"] = -91, ["order"] = 1, ["angle"] = -1268056064},
  [9] = {["x"] = 3656, ["y"] = 2621, ["z"] = -116, ["order"] = 2, ["angle"] = -1576468480},
  [10] = {["x"] = 3300, ["y"] = 1582, ["z"] = -147, ["order"] = 2, ["angle"] = -2083717120},
  [11] = {["x"] = 2242, ["y"] = 997, ["z"] = -173, ["order"] = 2, ["angle"] = 2048458752},
  [12] = {["x"] = 355, ["y"] = 1034, ["z"] = -315, ["order"] = 2, ["angle"] = 2112028672},
  [13] = {["x"] = -2235, ["y"] = 1177, ["z"] = 920, ["order"] = 2, ["angle"] = -2086600704},
  [14] = {["x"] = -3281, ["y"] = 1221, ["z"] = 1044, ["order"] = 2, ["angle"] = -1982005248},
  [15] = {["x"] = -4953, ["y"] = 804, ["z"] = 1056, ["order"] = 3, ["angle"] = -1194524672},
  [16] = {["x"] = -5173, ["y"] = -558, ["z"] = 739, ["order"] = 3, ["angle"] = -1010761728},
  [17] = {["x"] = -4976, ["y"] = -1721, ["z"] = 544, ["order"] = 3, ["angle"] = -546766848},
  [18] = {["x"] = -4002, ["y"] = -2713, ["z"] = 544, ["order"] = 3, ["angle"] = -903544832},
  [19] = {["x"] = -3639, ["y"] = -3623, ["z"] = 544, ["order"] = 3, ["angle"] = -1335689216},
  [20] = {["x"] = -4016, ["y"] = -4381, ["z"] = 544, ["order"] = 3, ["angle"] = -1403060224},
  [21] = {["x"] = -4683, ["y"] = -5263, ["z"] = 544, ["order"] = 3, ["angle"] = -1216544768},
  [22] = {["x"] = -5006, ["y"] = -6252, ["z"] = 544, ["order"] = 3, ["angle"] = -1062141952},
  [23] = {["x"] = -5020, ["y"] = -7105, ["z"] = 544, ["order"] = 3, ["angle"] = -1062141952},
  [24] = {["x"] = -5006, ["y"] = -8221, ["z"] = 544, ["order"] = 3, ["angle"] = -1062141952},
  [25] = {["x"] = -4997, ["y"] = -9313, ["z"] = 594, ["order"] = 4, ["angle"] = -1312358400},
  [26] = {["x"] = -5498, ["y"] = -10344, ["z"] = 820, ["order"] = 4, ["angle"] = -2110455808},
  [27] = {["x"] = -6553, ["y"] = -10398, ["z"] = 1018, ["order"] = 4, ["angle"] = 1338048512},
  [28] = {["x"] = -7148, ["y"] = -9495, ["z"] = 1211, ["order"] = 5, ["angle"] = 629997568},
  [29] = {["x"] = -6901, ["y"] = -8720, ["z"] = 1355, ["order"] = 5, ["angle"] = 171900928},
  [30] = {["x"] = -6147, ["y"] = -8445, ["z"] = 1509, ["order"] = 5, ["angle"] = -54984704},
  [31] = {["x"] = -5057, ["y"] = -8419, ["z"] = 1568, ["order"] = 5, ["angle"] = 2949120},
  [32] = {["x"] = -4022, ["y"] = -8351, ["z"] = 1568, ["order"] = 5, ["angle"] = -55508992},
  [33] = {["x"] = -3088, ["y"] = -8407, ["z"] = 1568, ["order"] = 5, ["angle"] = -723976192},
  [34] = {["x"] = -2558, ["y"] = -9071, ["z"] = 1568, ["order"] = 6, ["angle"] = -1097662464},
  [35] = {["x"] = -2579, ["y"] = -9944, ["z"] = 1568, ["order"] = 6, ["angle"] = -742850560},
  [36] = {["x"] = -2396, ["y"] = -10830, ["z"] = 1568, ["order"] = 6, ["angle"] = -170065920},
  [37] = {["x"] = -1823, ["y"] = -11177, ["z"] = 1568, ["order"] = 6, ["angle"] = 15269888},
  [38] = {["x"] = 818, ["y"] = -11243, ["z"] = 540, ["order"] = 6, ["angle"] = 983040},
  [39] = {["x"] = 1922, ["y"] = -11236, ["z"] = 512, ["order"] = 6, ["angle"] = -725942272},
  [40] = {["x"] = 2482, ["y"] = -14118, ["z"] = 512, ["order"] = 7, ["angle"] = -455802880},
  [41] = {["x"] = 2886, ["y"] = -14570, ["z"] = 512, ["order"] = 7, ["angle"] = 31784960},
  [42] = {["x"] = 6047, ["y"] = -14312, ["z"] = 512, ["order"] = 8, ["angle"] = 1065156608},
  [43] = {["x"] = 6052, ["y"] = -12883, ["z"] = 520, ["order"] = 8, ["angle"] = 1064108032},
  [44] = {["x"] = 6053, ["y"] = -11191, ["z"] = 520, ["order"] = 8, ["angle"] = 1061879808},
  [45] = {["x"] = 6063, ["y"] = -9576, ["z"] = 520, ["order"] = 8, ["angle"] = 1061879808},
  [46] = {["x"] = 6140, ["y"] = -8547, ["z"] = 512, ["order"] = 8, ["angle"] = 28508160},
  [47] = {["x"] = 7617, ["y"] = -8374, ["z"] = 520, ["order"] = 9, ["angle"] = 4915200},
  [48] = {["x"] = 9695, ["y"] = -8384, ["z"] = 520, ["order"] = 9, ["angle"] = 34013184},
  [49] = {["x"] = 11384, ["y"] = -8353, ["z"] = 1398, ["order"] = 9, ["angle"] = -17104896},
  [50] = {["x"] = 13024, ["y"] = -8375, ["z"] = 2266, ["order"] = 9, ["angle"] = -17104896},
  [51] = {["x"] = 14766, ["y"] = -8487, ["z"] = 3189, ["order"] = 9, ["angle"] = -709033984},
  [52] = {["x"] = 15663, ["y"] = -9752, ["z"] = 3665, ["order"] = 9, ["angle"] = -1510801408},
  [53] = {["x"] = 15176, ["y"] = -11032, ["z"] = 3406, ["order"] = 10, ["angle"] = -1902968832},
  [54] = {["x"] = 14139, ["y"] = -11550, ["z"] = 3072, ["order"] = 10, ["angle"] = -2141257728},
  [55] = {["x"] = 13351, ["y"] = -11562, ["z"] = 3072, ["order"] = 10, ["angle"] = -2141257728},
  [56] = {["x"] = 12214, ["y"] = -11602, ["z"] = 3072, ["order"] = 10, ["angle"] = 1558118400},
  [57] = {["x"] = 11528, ["y"] = -10833, ["z"] = 3072, ["order"] = 11, ["angle"] = 1501757440},
  [58] = {["x"] = 10879, ["y"] = -9842, ["z"] = 3072, ["order"] = 11, ["angle"] = 1821704192},
  [59] = {["x"] = 10228, ["y"] = -9390, ["z"] = 3072, ["order"] = 11, ["angle"] = -2102722560},
  [60] = {["x"] = 9122, ["y"] = -9312, ["z"] = 3072, ["order"] = 11, ["angle"] = -1923416064},
  [61] = {["x"] = 8467, ["y"] = -9353, ["z"] = 3072, ["order"] = 11, ["angle"] = -1531510784},
  [62] = {["x"] = 8065, ["y"] = -9762, ["z"] = 3072, ["order"] = 12, ["angle"] = -1191510016},
  [63] = {["x"] = 7858, ["y"] = -10500, ["z"] = 3072, ["order"] = 12, ["angle"] = -1117847552},
  [64] = {["x"] = 7851, ["y"] = -11630, ["z"] = 3072, ["order"] = 12, ["angle"] = -1343553536},
  [65] = {["x"] = 7559, ["y"] = -12380, ["z"] = 3072, ["order"] = 12, ["angle"] = -1714356224},
  [66] = {["x"] = 6950, ["y"] = -12780, ["z"] = 3072, ["order"] = 13, ["angle"] = -2113994752},
  [67] = {["x"] = 5922, ["y"] = -12852, ["z"] = 3072, ["order"] = 13, ["angle"] = -2145452032},
  [68] = {["x"] = 5120, ["y"] = -12877, ["z"] = 3072, ["order"] = 13, ["angle"] = -1805582336},
  [69] = {["x"] = 4427, ["y"] = -13299, ["z"] = 3072, ["order"] = 14, ["angle"] = -1301610496},
  [70] = {["x"] = 4096, ["y"] = -14159, ["z"] = 3072, ["order"] = 14, ["angle"] = -1222574080},
  [71] = {["x"] = 3900, ["y"] = -15095, ["z"] = 3072, ["order"] = 14, ["angle"] = -1484455936},
  [72] = {["x"] = 3492, ["y"] = -15878, ["z"] = 3072, ["order"] = 14, ["angle"] = -1904672768},
  [73] = {["x"] = 2881, ["y"] = -16203, ["z"] = 3072, ["order"] = 15, ["angle"] = -2057633792},
  [74] = {["x"] = 1944, ["y"] = -16343, ["z"] = 3072, ["order"] = 15, ["angle"] = 1995898880},
  [75] = {["x"] = 1282, ["y"] = -16194, ["z"] = 3072, ["order"] = 15, ["angle"] = 1560477696},
  [76] = {["x"] = 769, ["y"] = -15741, ["z"] = 3072, ["order"] = 16, ["angle"] = 1189675008},
  [77] = {["x"] = 554, ["y"] = -15078, ["z"] = 3072, ["order"] = 16, ["angle"] = 1067909120},
  [78] = {["x"] = 515, ["y"] = -13729, ["z"] = 3072, ["order"] = 16, ["angle"] = 1050738688},
  [79] = {["x"] = 539, ["y"] = -12828, ["z"] = 3072, ["order"] = 16, ["angle"] = 1050738688},
  [80] = {["x"] = 564, ["y"] = -12069, ["z"] = 3072, ["order"] = 16, ["angle"] = 1050738688},
  [81] = {["x"] = 643, ["y"] = -9693, ["z"] = 3072, ["order"] = 16, ["angle"] = 1060438016},
  [82] = {["x"] = 628, ["y"] = -8435, ["z"] = 3072, ["order"] = 16, ["angle"] = 1073545216},
  [83] = {["x"] = 570, ["y"] = 1658, ["z"] = -8, ["order"] = 0, ["angle"] = 1043398656},
  [84] = {["x"] = 1834, ["y"] = 4252, ["z"] = -33, ["order"] = 1, ["angle"] = -529072128},
  [85] = {["x"] = 1089, ["y"] = 1056, ["z"] = -198, ["order"] = 2, ["angle"] = 2111635456},
  [86] = {["x"] = -4080, ["y"] = 1106, ["z"] = 1056, ["order"] = 2, ["angle"] = -1839792128},
  [87] = {["x"] = -5186, ["y"] = -1783, ["z"] = 544, ["order"] = 3, ["angle"] = -1537146880},
  [88] = {["x"] = -5681, ["y"] = -2296, ["z"] = 544, ["order"] = 3, ["angle"] = -1557331968},
  [89] = {["x"] = -6291, ["y"] = -2952, ["z"] = 544, ["order"] = 3, ["angle"] = -1279590400},
  [90] = {["x"] = -6517, ["y"] = -3747, ["z"] = 544, ["order"] = 3, ["angle"] = -786235392},
  [91] = {["x"] = -6286, ["y"] = -4286, ["z"] = 544, ["order"] = 3, ["angle"] = -527368192},
  [92] = {["x"] = -5639, ["y"] = -5001, ["z"] = 544, ["order"] = 3, ["angle"] = -784400384},
  [93] = {["x"] = -5232, ["y"] = -6022, ["z"] = 544, ["order"] = 3, ["angle"] = -995164160},
  [94] = {["x"] = -5080, ["y"] = -9904, ["z"] = 724, ["order"] = 4, ["angle"] = -1790115840},
  [95] = {["x"] = -5922, ["y"] = -10514, ["z"] = 912, ["order"] = 4, ["angle"] = 1908604928},
  [96] = {["x"] = -6962, ["y"] = -10042, ["z"] = 1102, ["order"] = 4, ["angle"] = 1257963520},
  [97] = {["x"] = -7188, ["y"] = -9131, ["z"] = 1256, ["order"] = 5, ["angle"] = 566689792},
  [98] = {["x"] = -939, ["y"] = -11323, ["z"] = 1350, ["order"] = 6, ["angle"] = 7012352},
  [99] = {["x"] = 70, ["y"] = -11349, ["z"] = 868, ["order"] = 6, ["angle"] = 47513600},
  [100] = {["x"] = 2442, ["y"] = -12310, ["z"] = 520, ["order"] = 7, ["angle"] = -1059520512},
  [101] = {["x"] = 2432, ["y"] = -13276, ["z"] = 520, ["order"] = 7, ["angle"] = -835125248},
  [102] = {["x"] = 3513, ["y"] = -14570, ["z"] = 520, ["order"] = 7, ["angle"] = -40173568},
  [103] = {["x"] = 4937, ["y"] = -14668, ["z"] = 520, ["order"] = 8, ["angle"] = 388694016},
  [104] = {["x"] = 6066, ["y"] = -13727, ["z"] = 520, ["order"] = 8, ["angle"] = 1088356352},
  [105] = {["x"] = 6068, ["y"] = -12078, ["z"] = 520, ["order"] = 8, ["angle"] = 1089404928},
  [106] = {["x"] = 6054, ["y"] = -10324, ["z"] = 520, ["order"] = 8, ["angle"] = 1055195136},
  [107] = {["x"] = 6799, ["y"] = -8382, ["z"] = 520, ["order"] = 8, ["angle"] = -4784128},
  [108] = {["x"] = 8638, ["y"] = -8362, ["z"] = 520, ["order"] = 9, ["angle"] = -41484288},
  [109] = {["x"] = 10519, ["y"] = -8370, ["z"] = 939, ["order"] = 9, ["angle"] = -3604480},
  [110] = {["x"] = 12117, ["y"] = -8347, ["z"] = 1786, ["order"] = 9, ["angle"] = -26935296},
  [111] = {["x"] = 13925, ["y"] = -8278, ["z"] = 2744, ["order"] = 9, ["angle"] = -228524032},
  [112] = {["x"] = 15224, ["y"] = -9005, ["z"] = 3432, ["order"] = 9, ["angle"] = -831979520},
  [113] = {["x"] = 15597, ["y"] = -10230, ["z"] = 3629, ["order"] = 10, ["angle"] = -1496645632},
  [114] = {["x"] = 14775, ["y"] = -11453, ["z"] = 3194, ["order"] = 10, ["angle"] = -2073886720},
  [115] = {["x"] = 11749, ["y"] = -11411, ["z"] = 3072, ["order"] = 10, ["angle"] = 1508835328}
}
-- Respawn definitions for SUNBEAM PARADISE (MAP11)
krespawn.points[11] = {
  [0] = 81,
  [1] = {["x"] = 6475, ["y"] = -8083, ["z"] = 300, ["order"] = 0, ["angle"] = 1006698496},
  [2] = {["x"] = 6620, ["y"] = -7270, ["z"] = 300, ["order"] = 0, ["angle"] = 990183424},
  [3] = {["x"] = 6813, ["y"] = -6284, ["z"] = 300, ["order"] = 0, ["angle"] = 1178009600},
  [4] = {["x"] = 6874, ["y"] = -5181, ["z"] = 242, ["order"] = 0, ["angle"] = 1368064000},
  [5] = {["x"] = 6511, ["y"] = -4246, ["z"] = 200, ["order"] = 1, ["angle"] = 2079916032},
  [6] = {["x"] = 5862, ["y"] = -4040, ["z"] = 152, ["order"] = 1, ["angle"] = -2022637568},
  [7] = {["x"] = 5080, ["y"] = -4179, ["z"] = 103, ["order"] = 1, ["angle"] = -2025127936},
  [8] = {["x"] = 4166, ["y"] = -4487, ["z"] = 100, ["order"] = 1, ["angle"] = -2061041664},
  [9] = {["x"] = 3200, ["y"] = -4655, ["z"] = 64, ["order"] = 1, ["angle"] = 1844117504},
  [10] = {["x"] = 2368, ["y"] = -4295, ["z"] = 71, ["order"] = 1, ["angle"] = 1273954304},
  [11] = {["x"] = 1983, ["y"] = -3558, ["z"] = 88, ["order"] = 1, ["angle"] = 1006436352},
  [12] = {["x"] = 2158, ["y"] = -2603, ["z"] = 86, ["order"] = 2, ["angle"] = 963969024},
  [13] = {["x"] = 2317, ["y"] = -1713, ["z"] = 86, ["order"] = 2, ["angle"] = 1106706432},
  [14] = {["x"] = 2172, ["y"] = -807, ["z"] = 116, ["order"] = 2, ["angle"] = 959643648},
  [15] = {["x"] = 2195, ["y"] = 108, ["z"] = 111, ["order"] = 2, ["angle"] = 658046976},
  [16] = {["x"] = 2668, ["y"] = 1019, ["z"] = 66, ["order"] = 2, ["angle"] = 290127872},
  [17] = {["x"] = 3378, ["y"] = 1522, ["z"] = 60, ["order"] = 2, ["angle"] = 66912256},
  [18] = {["x"] = 4110, ["y"] = 1603, ["z"] = -1, ["order"] = 3, ["angle"] = 19595264},
  [19] = {["x"] = 4935, ["y"] = 1513, ["z"] = -157, ["order"] = 3, ["angle"] = 249757696},
  [20] = {["x"] = 5832, ["y"] = 1604, ["z"] = -200, ["order"] = 4, ["angle"] = 707461120},
  [21] = {["x"] = 6343, ["y"] = 2146, ["z"] = -200, ["order"] = 4, ["angle"] = 1030684672},
  [22] = {["x"] = 6459, ["y"] = 2999, ["z"] = -200, ["order"] = 4, ["angle"] = 1347092480},
  [23] = {["x"] = 6286, ["y"] = 3736, ["z"] = -200, ["order"] = 4, ["angle"] = 1643577344},
  [24] = {["x"] = 5746, ["y"] = 4384, ["z"] = -166, ["order"] = 5, ["angle"] = 1908736000},
  [25] = {["x"] = 5081, ["y"] = 4639, ["z"] = -19, ["order"] = 5, ["angle"] = -2147024896},
  [26] = {["x"] = 4307, ["y"] = 4594, ["z"] = 140, ["order"] = 5, ["angle"] = -1970733056},
  [27] = {["x"] = 3399, ["y"] = 4293, ["z"] = 140, ["order"] = 5, ["angle"] = -1947271168},
  [28] = {["x"] = 2202, ["y"] = 3960, ["z"] = 114, ["order"] = 5, ["angle"] = 2047934464},
  [29] = {["x"] = 1321, ["y"] = 4101, ["z"] = 87, ["order"] = 5, ["angle"] = 2032205824},
  [30] = {["x"] = 320, ["y"] = 4185, ["z"] = 68, ["order"] = 5, ["angle"] = -2004418560},
  [31] = {["x"] = -609, ["y"] = 4046, ["z"] = 78, ["order"] = 5, ["angle"] = -2144141312},
  [32] = {["x"] = -1640, ["y"] = 3971, ["z"] = 138, ["order"] = 6, ["angle"] = 2105081856},
  [33] = {["x"] = -2628, ["y"] = 4037, ["z"] = 140, ["order"] = 6, ["angle"] = 2125922304},
  [34] = {["x"] = -3740, ["y"] = 4077, ["z"] = 286, ["order"] = 6, ["angle"] = -2092105728},
  [35] = {["x"] = -4787, ["y"] = 4110, ["z"] = 420, ["order"] = 6, ["angle"] = -2018836480},
  [36] = {["x"] = -5728, ["y"] = 4037, ["z"] = 420, ["order"] = 6, ["angle"] = -1604780032},
  [37] = {["x"] = -6382, ["y"] = 3693, ["z"] = 420, ["order"] = 7, ["angle"] = -1092681728},
  [38] = {["x"] = -6560, ["y"] = 3067, ["z"] = 395, ["order"] = 7, ["angle"] = -1041956864},
  [39] = {["x"] = -6585, ["y"] = 1843, ["z"] = 193, ["order"] = 7, ["angle"] = -1148780544},
  [40] = {["x"] = -6586, ["y"] = -803, ["z"] = 120, ["order"] = 8, ["angle"] = -1219297280},
  [41] = {["x"] = -6774, ["y"] = -2192, ["z"] = 50, ["order"] = 8, ["angle"] = -863043584},
  [42] = {["x"] = -6780, ["y"] = -3213, ["z"] = 50, ["order"] = 8, ["angle"] = -604438528},
  [43] = {["x"] = -6261, ["y"] = -3939, ["z"] = 50, ["order"] = 9, ["angle"] = -199819264},
  [44] = {["x"] = -5248, ["y"] = -4106, ["z"] = 50, ["order"] = 9, ["angle"] = -50003968},
  [45] = {["x"] = -4071, ["y"] = -4285, ["z"] = 50, ["order"] = 9, ["angle"] = -667090944},
  [46] = {["x"] = -3606, ["y"] = -5144, ["z"] = 123, ["order"] = 10, ["angle"] = -1070006272},
  [47] = {["x"] = -3594, ["y"] = -5826, ["z"] = 140, ["order"] = 10, ["angle"] = -1098842112},
  [48] = {["x"] = -3673, ["y"] = -6769, ["z"] = 140, ["order"] = 10, ["angle"] = -1027932160},
  [49] = {["x"] = -3698, ["y"] = -7836, ["z"] = 140, ["order"] = 10, ["angle"] = -848756736},
  [50] = {["x"] = -3647, ["y"] = -8858, ["z"] = 140, ["order"] = 11, ["angle"] = -407568384},
  [51] = {["x"] = -3167, ["y"] = -9231, ["z"] = 157, ["order"] = 11, ["angle"] = -72941568},
  [52] = {["x"] = -2392, ["y"] = -9412, ["z"] = 278, ["order"] = 11, ["angle"] = -52756480},
  [53] = {["x"] = -1092, ["y"] = -9352, ["z"] = 420, ["order"] = 11, ["angle"] = 172032000},
  [54] = {["x"] = -330, ["y"] = -9108, ["z"] = 409, ["order"] = 11, ["angle"] = 270336000},
  [55] = {["x"] = 649, ["y"] = -8612, ["z"] = 320, ["order"] = 11, ["angle"] = -40566784},
  [56] = {["x"] = 1599, ["y"] = -8471, ["z"] = 320, ["order"] = 11, ["angle"] = -287375360},
  [57] = {["x"] = 2385, ["y"] = -8702, ["z"] = 351, ["order"] = 11, ["angle"] = -509149184},
  [58] = {["x"] = 3014, ["y"] = -9227, ["z"] = 417, ["order"] = 11, ["angle"] = -415956992},
  [59] = {["x"] = 3736, ["y"] = -9876, ["z"] = 420, ["order"] = 12, ["angle"] = -361431040},
  [60] = {["x"] = 4490, ["y"] = -10451, ["z"] = 420, ["order"] = 12, ["angle"] = -186187776},
  [61] = {["x"] = 5221, ["y"] = -10845, ["z"] = 420, ["order"] = 12, ["angle"] = 25624576},
  [62] = {["x"] = 5883, ["y"] = -10972, ["z"] = 420, ["order"] = 12, ["angle"] = 508755968},
  [63] = {["x"] = 6472, ["y"] = -10605, ["z"] = 420, ["order"] = 13, ["angle"] = 1090977792},
  [64] = {["x"] = 6562, ["y"] = -9812, ["z"] = 351, ["order"] = 13, ["angle"] = 1062404096},
  [65] = {["x"] = 6485, ["y"] = -8886, ["z"] = 300, ["order"] = 13, ["angle"] = 1100021760},
  [66] = {["x"] = 3291, ["y"] = 4610, ["z"] = 84, ["order"] = 5, ["angle"] = -2120155136},
  [67] = {["x"] = 2109, ["y"] = 4571, ["z"] = 0, ["order"] = 5, ["angle"] = -2117009408},
  [68] = {["x"] = 902, ["y"] = 4634, ["z"] = -10, ["order"] = 5, ["angle"] = -2062745600},
  [69] = {["x"] = 78, ["y"] = 4455, ["z"] = 13, ["order"] = 5, ["angle"] = -1941372928},
  [70] = {["x"] = -1326, ["y"] = 3951, ["z"] = 92, ["order"] = 6, ["angle"] = -1589837824},
  [71] = {["x"] = -1969, ["y"] = 3514, ["z"] = 140, ["order"] = 6, ["angle"] = -1376321536},
  [72] = {["x"] = -2232, ["y"] = 2822, ["z"] = 121, ["order"] = 6, ["angle"] = -1226637312},
  [73] = {["x"] = -2244, ["y"] = 1834, ["z"] = 122, ["order"] = 6, ["angle"] = -1231355904},
  [74] = {["x"] = -2291, ["y"] = 934, ["z"] = 113, ["order"] = 6, ["angle"] = -1425997824},
  [75] = {["x"] = -2567, ["y"] = -77, ["z"] = 56, ["order"] = 6, ["angle"] = -1678573568},
  [76] = {["x"] = -3106, ["y"] = -694, ["z"] = 60, ["order"] = 7, ["angle"] = -1949761536},
  [77] = {["x"] = -3868, ["y"] = -999, ["z"] = 88, ["order"] = 7, ["angle"] = 1997471744},
  [78] = {["x"] = -4652, ["y"] = -875, ["z"] = 64, ["order"] = 7, ["angle"] = 2085289984},
  [79] = {["x"] = -5468, ["y"] = -707, ["z"] = 83, ["order"] = 7, ["angle"] = -1962606592},
  [80] = {["x"] = -6192, ["y"] = -962, ["z"] = 121, ["order"] = 7, ["angle"] = -1375272960},
  [81] = {["x"] = -6748, ["y"] = -1677, ["z"] = 97, ["order"] = 8, ["angle"] = -1025703936}
}
-- Respawn definitions for DIAMOND SQUARE (MAP12)
krespawn.points[12] = {
  [0] = 79,
  [1] = {["x"] = -10354, ["y"] = 11, ["z"] = 0, ["order"] = 0, ["angle"] = 458752},
  [2] = {["x"] = -9407, ["y"] = -6, ["z"] = 0, ["order"] = 0, ["angle"] = -17104896},
  [3] = {["x"] = -8502, ["y"] = -23, ["z"] = 0, ["order"] = 0, ["angle"] = 27066368},
  [4] = {["x"] = -7269, ["y"] = 4, ["z"] = 0, ["order"] = 0, ["angle"] = 27066368},
  [5] = {["x"] = -6160, ["y"] = 22, ["z"] = 0, ["order"] = 0, ["angle"] = -3997696},
  [6] = {["x"] = -5123, ["y"] = -44, ["z"] = 0, ["order"] = 0, ["angle"] = -370081792},
  [7] = {["x"] = -3979, ["y"] = -648, ["z"] = 0, ["order"] = 0, ["angle"] = -480313344},
  [8] = {["x"] = -3243, ["y"] = -1050, ["z"] = 0, ["order"] = 0, ["angle"] = 134676480},
  [9] = {["x"] = -2667, ["y"] = -1010, ["z"] = -10, ["order"] = 0, ["angle"] = 461045760},
  [10] = {["x"] = -2094, ["y"] = -553, ["z"] = -10, ["order"] = 0, ["angle"] = 375848960},
  [11] = {["x"] = -1418, ["y"] = -212, ["z"] = 0, ["order"] = 0, ["angle"] = 113967104},
  [12] = {["x"] = -480, ["y"] = -31, ["z"] = 0, ["order"] = 1, ["angle"] = 120127488},
  [13] = {["x"] = 580, ["y"] = 185, ["z"] = 45, ["order"] = 1, ["angle"] = -42926080},
  [14] = {["x"] = 1563, ["y"] = 238, ["z"] = 106, ["order"] = 1, ["angle"] = -623706112},
  [15] = {["x"] = 2153, ["y"] = -390, ["z"] = 174, ["order"] = 1, ["angle"] = -1012203520},
  [16] = {["x"] = 2280, ["y"] = -1491, ["z"] = 560, ["order"] = 1, ["angle"] = -1242234880},
  [17] = {["x"] = 2321, ["y"] = -2433, ["z"] = 400, ["order"] = 2, ["angle"] = -1637023744},
  [18] = {["x"] = 2310, ["y"] = -3107, ["z"] = 400, ["order"] = 2, ["angle"] = -1591803904},
  [19] = {["x"] = 1217, ["y"] = -3662, ["z"] = 400, ["order"] = 2, ["angle"] = -2062483456},
  [20] = {["x"] = 503, ["y"] = -3841, ["z"] = 454, ["order"] = 2, ["angle"] = -2098266112},
  [21] = {["x"] = -879, ["y"] = -3990, ["z"] = 406, ["order"] = 2, ["angle"] = -2098266112},
  [22] = {["x"] = -3124, ["y"] = -4126, ["z"] = 400, ["order"] = 2, ["angle"] = -2098266112},
  [23] = {["x"] = -4292, ["y"] = -4127, ["z"] = 400, ["order"] = 3, ["angle"] = 1898250240},
  [24] = {["x"] = -5568, ["y"] = -3747, ["z"] = 400, ["order"] = 3, ["angle"] = 1804402688},
  [25] = {["x"] = -6771, ["y"] = -3351, ["z"] = 400, ["order"] = 3, ["angle"] = 2099314688},
  [26] = {["x"] = -7610, ["y"] = -3253, ["z"] = 400, ["order"] = 3, ["angle"] = -2132082688},
  [27] = {["x"] = -10674, ["y"] = -3227, ["z"] = 408, ["order"] = 3, ["angle"] = -2132082688},
  [28] = {["x"] = -12223, ["y"] = -3192, ["z"] = 944, ["order"] = 3, ["angle"] = -2128805888},
  [29] = {["x"] = -13035, ["y"] = -3206, ["z"] = 944, ["order"] = 3, ["angle"] = -2062221312},
  [30] = {["x"] = -14844, ["y"] = -3222, ["z"] = 1124, ["order"] = 4, ["angle"] = -1944256512},
  [31] = {["x"] = -16254, ["y"] = -3931, ["z"] = 1451, ["order"] = 4, ["angle"] = -678887424},
  [32] = {["x"] = -16059, ["y"] = -4539, ["z"] = 1406, ["order"] = 4, ["angle"] = -310575104},
  [33] = {["x"] = -15344, ["y"] = -4760, ["z"] = 1240, ["order"] = 4, ["angle"] = -143327232},
  [34] = {["x"] = -14560, ["y"] = -4801, ["z"] = 1059, ["order"] = 4, ["angle"] = -289996800},
  [35] = {["x"] = -13692, ["y"] = -5137, ["z"] = 944, ["order"] = 5, ["angle"] = -1079574528},
  [36] = {["x"] = -13485, ["y"] = -5993, ["z"] = 944, ["order"] = 6, ["angle"] = -1706098688},
  [37] = {["x"] = -14041, ["y"] = -6461, ["z"] = 944, ["order"] = 7, ["angle"] = -2108751872},
  [38] = {["x"] = -14821, ["y"] = -6573, ["z"] = 807, ["order"] = 7, ["angle"] = -2106916864},
  [39] = {["x"] = -15728, ["y"] = -6603, ["z"] = 637, ["order"] = 7, ["angle"] = 1945042944},
  [40] = {["x"] = -16949, ["y"] = -6573, ["z"] = 408, ["order"] = 7, ["angle"] = 1610416128},
  [41] = {["x"] = -17628, ["y"] = -6220, ["z"] = 392, ["order"] = 7, ["angle"] = 1267531776},
  [42] = {["x"] = -17912, ["y"] = -5603, ["z"] = 392, ["order"] = 7, ["angle"] = 1081278464},
  [43] = {["x"] = -17983, ["y"] = -4720, ["z"] = 392, ["order"] = 8, ["angle"] = 1088356352},
  [44] = {["x"] = -17983, ["y"] = -1214, ["z"] = 328, ["order"] = 9, ["angle"] = 1004994560},
  [45] = {["x"] = -18065, ["y"] = -462, ["z"] = 328, ["order"] = 9, ["angle"] = 657129472},
  [46] = {["x"] = -17679, ["y"] = 405, ["z"] = 328, ["order"] = 9, ["angle"] = 243990528},
  [47] = {["x"] = -16770, ["y"] = 890, ["z"] = 328, ["order"] = 9, ["angle"] = 17760256},
  [48] = {["x"] = -15702, ["y"] = 1038, ["z"] = 49, ["order"] = 9, ["angle"] = -152371200},
  [49] = {["x"] = -14474, ["y"] = 964, ["z"] = 0, ["order"] = 9, ["angle"] = -586481664},
  [50] = {["x"] = -13897, ["y"] = 338, ["z"] = 0, ["order"] = 9, ["angle"] = -522649600},
  [51] = {["x"] = -13246, ["y"] = -357, ["z"] = 0, ["order"] = 9, ["angle"] = 43712512},
  [52] = {["x"] = -12368, ["y"] = -344, ["z"] = 0, ["order"] = 9, ["angle"] = 137691136},
  [53] = {["x"] = -11615, ["y"] = -67, ["z"] = 0, ["order"] = 9, ["angle"] = 129826816},
  [54] = {["x"] = -10901, ["y"] = 33, ["z"] = 0, ["order"] = 9, ["angle"] = 3080192},
  [55] = {["x"] = -9925, ["y"] = 14, ["z"] = 0, ["order"] = 9, ["angle"] = -13959168},
  [56] = {["x"] = -5206, ["y"] = 85, ["z"] = 0, ["order"] = 0, ["angle"] = 301268992},
  [57] = {["x"] = -4584, ["y"] = 347, ["z"] = 0, ["order"] = 0, ["angle"] = 296681472},
  [58] = {["x"] = -3810, ["y"] = 859, ["z"] = 0, ["order"] = 0, ["angle"] = 120258560},
  [59] = {["x"] = -3017, ["y"] = 1051, ["z"] = 0, ["order"] = 0, ["angle"] = -276627456},
  [60] = {["x"] = -2300, ["y"] = 694, ["z"] = -10, ["order"] = 0, ["angle"] = -258015232},
  [61] = {["x"] = -1604, ["y"] = 329, ["z"] = -10, ["order"] = 0, ["angle"] = -137953280},
  [62] = {["x"] = -587, ["y"] = 205, ["z"] = 0, ["order"] = 0, ["angle"] = 111214592},
  [63] = {["x"] = 737, ["y"] = 565, ["z"] = 56, ["order"] = 1, ["angle"] = 243204096},
  [64] = {["x"] = 1648, ["y"] = 963, ["z"] = 112, ["order"] = 1, ["angle"] = 33488896},
  [65] = {["x"] = 2507, ["y"] = 1102, ["z"] = 120, ["order"] = 1, ["angle"] = -70582272},
  [66] = {["x"] = 3306, ["y"] = 545, ["z"] = 512, ["order"] = 1, ["angle"] = -1078919168},
  [67] = {["x"] = 3357, ["y"] = -308, ["z"] = 284, ["order"] = 1, ["angle"] = -1048248320},
  [68] = {["x"] = 3390, ["y"] = -1361, ["z"] = 684, ["order"] = 1, ["angle"] = -1170145280},
  [69] = {["x"] = 3384, ["y"] = -1796, ["z"] = 696, ["order"] = 2, ["angle"] = -956104704},
  [70] = {["x"] = 3856, ["y"] = -2515, ["z"] = 720, ["order"] = 2, ["angle"] = -1154154496},
  [71] = {["x"] = 3830, ["y"] = -3379, ["z"] = 720, ["order"] = 2, ["angle"] = -1500839936},
  [72] = {["x"] = 2186, ["y"] = -4159, ["z"] = 400, ["order"] = 2, ["angle"] = -1780285440},
  [73] = {["x"] = 1478, ["y"] = -4603, ["z"] = 400, ["order"] = 2, ["angle"] = -1838219264},
  [74] = {["x"] = 652, ["y"] = -4837, ["z"] = 438, ["order"] = 2, ["angle"] = -2131820544},
  [75] = {["x"] = -984, ["y"] = -4931, ["z"] = 416, ["order"] = 2, ["angle"] = 1938489344},
  [76] = {["x"] = -3153, ["y"] = -4732, ["z"] = 400, ["order"] = 2, ["angle"] = 1930756096},
  [77] = {["x"] = -4967, ["y"] = -4285, ["z"] = 400, ["order"] = 3, ["angle"] = 1812004864},
  [78] = {["x"] = -6175, ["y"] = -3838, ["z"] = 400, ["order"] = 3, ["angle"] = 1620377600},
  [79] = {["x"] = -13967, ["y"] = -3236, ["z"] = 944, ["order"] = 4, ["angle"] = -2137325568}
}
-- Respawn definitions for MIDNIGHT MEADOW (MAP13)
krespawn.points[13] = {
  [0] = 77,
  [1] = {["x"] = 810, ["y"] = 6597, ["z"] = 700, ["order"] = 0, ["angle"] = -41615360},
  [2] = {["x"] = 1650, ["y"] = 6531, ["z"] = 892, ["order"] = 0, ["angle"] = -41615360},
  [3] = {["x"] = 2424, ["y"] = 6495, ["z"] = 1084, ["order"] = 0, ["angle"] = 91422720},
  [4] = {["x"] = 3238, ["y"] = 6635, ["z"] = 1084, ["order"] = 1, ["angle"] = 286588928},
  [5] = {["x"] = 4099, ["y"] = 6849, ["z"] = 1084, ["order"] = 1, ["angle"] = -62193664},
  [6] = {["x"] = 5037, ["y"] = 6854, ["z"] = 1084, ["order"] = 1, ["angle"] = -158269440},
  [7] = {["x"] = 6102, ["y"] = 6742, ["z"] = 1092, ["order"] = 1, ["angle"] = -399704064},
  [8] = {["x"] = 6941, ["y"] = 6433, ["z"] = 1010, ["order"] = 1, ["angle"] = -544800768},
  [9] = {["x"] = 7897, ["y"] = 5750, ["z"] = 952, ["order"] = 1, ["angle"] = -875233280},
  [10] = {["x"] = 8475, ["y"] = 4204, ["z"] = 840, ["order"] = 2, ["angle"] = -1266352128},
  [11] = {["x"] = 8131, ["y"] = 3145, ["z"] = 721, ["order"] = 2, ["angle"] = -1657470976},
  [12] = {["x"] = 7313, ["y"] = 2315, ["z"] = 606, ["order"] = 2, ["angle"] = -1701904384},
  [13] = {["x"] = 6623, ["y"] = 1714, ["z"] = 539, ["order"] = 2, ["angle"] = -1384972288},
  [14] = {["x"] = 6023, ["y"] = 923, ["z"] = 450, ["order"] = 2, ["angle"] = -1086914560},
  [15] = {["x"] = 5822, ["y"] = -46, ["z"] = 391, ["order"] = 3, ["angle"] = -1078394880},
  [16] = {["x"] = 5823, ["y"] = -1293, ["z"] = 325, ["order"] = 3, ["angle"] = -1084686336},
  [17] = {["x"] = 5726, ["y"] = -2290, ["z"] = 300, ["order"] = 3, ["angle"] = -1272643584},
  [18] = {["x"] = 5341, ["y"] = -3380, ["z"] = 300, ["order"] = 3, ["angle"] = -1225981952},
  [19] = {["x"] = 5349, ["y"] = -4573, ["z"] = 316, ["order"] = 3, ["angle"] = -928710656},
  [20] = {["x"] = 5603, ["y"] = -5547, ["z"] = 300, ["order"] = 3, ["angle"] = -1049559040},
  [21] = {["x"] = 5720, ["y"] = -6565, ["z"] = 312, ["order"] = 3, ["angle"] = -1049952256},
  [22] = {["x"] = 5771, ["y"] = -7585, ["z"] = 300, ["order"] = 3, ["angle"] = -984416256},
  [23] = {["x"] = 5801, ["y"] = -8850, ["z"] = 400, ["order"] = 4, ["angle"] = -751239168},
  [24] = {["x"] = 6212, ["y"] = -10032, ["z"] = 400, ["order"] = 4, ["angle"] = -263258112},
  [25] = {["x"] = 7201, ["y"] = -10537, ["z"] = 424, ["order"] = 5, ["angle"] = 74514432},
  [26] = {["x"] = 8536, ["y"] = -10527, ["z"] = 472, ["order"] = 5, ["angle"] = 556466176},
  [27] = {["x"] = 9347, ["y"] = -9951, ["z"] = 500, ["order"] = 5, ["angle"] = 881262592},
  [28] = {["x"] = 9742, ["y"] = -8870, ["z"] = 548, ["order"] = 6, ["angle"] = 1054146560},
  [29] = {["x"] = 9695, ["y"] = -7193, ["z"] = 700, ["order"] = 6, ["angle"] = 806682624},
  [30] = {["x"] = 9814, ["y"] = -6138, ["z"] = 700, ["order"] = 6, ["angle"] = 894369792},
  [31] = {["x"] = 10142, ["y"] = -5090, ["z"] = 700, ["order"] = 6, ["angle"] = 778895360},
  [32] = {["x"] = 10680, ["y"] = -4107, ["z"] = 700, ["order"] = 6, ["angle"] = 1158086656},
  [33] = {["x"] = 10679, ["y"] = -3280, ["z"] = 703, ["order"] = 7, ["angle"] = 1719468032},
  [34] = {["x"] = 9953, ["y"] = -2820, ["z"] = 750, ["order"] = 7, ["angle"] = -1900609536},
  [35] = {["x"] = 9105, ["y"] = -3097, ["z"] = 807, ["order"] = 7, ["angle"] = -1760493568},
  [36] = {["x"] = 8179, ["y"] = -3639, ["z"] = 872, ["order"] = 7, ["angle"] = -2122252288},
  [37] = {["x"] = 7191, ["y"] = -3755, ["z"] = 900, ["order"] = 7, ["angle"] = -2113732608},
  [38] = {["x"] = 5865, ["y"] = -3748, ["z"] = 900, ["order"] = 7, ["angle"] = -2136670208},
  [39] = {["x"] = 4543, ["y"] = -3754, ["z"] = 900, ["order"] = 7, ["angle"] = 2126053376},
  [40] = {["x"] = 3242, ["y"] = -3809, ["z"] = 858, ["order"] = 8, ["angle"] = 2078081024},
  [41] = {["x"] = 2316, ["y"] = -3845, ["z"] = 685, ["order"] = 8, ["angle"] = 1998651392},
  [42] = {["x"] = 1033, ["y"] = -3835, ["z"] = 455, ["order"] = 8, ["angle"] = 1770455040},
  [43] = {["x"] = 260, ["y"] = -3574, ["z"] = 400, ["order"] = 9, ["angle"] = 966983680},
  [44] = {["x"] = 154, ["y"] = -2629, ["z"] = 400, ["order"] = 9, ["angle"] = 569573376},
  [45] = {["x"] = 718, ["y"] = -1808, ["z"] = 400, ["order"] = 9, ["angle"] = 525795328},
  [46] = {["x"] = 1580, ["y"] = -1147, ["z"] = 495, ["order"] = 9, ["angle"] = 725942272},
  [47] = {["x"] = 2195, ["y"] = -208, ["z"] = 600, ["order"] = 9, ["angle"] = 1350369280},
  [48] = {["x"] = 2140, ["y"] = 895, ["z"] = 600, ["order"] = 9, ["angle"] = 1739784192},
  [49] = {["x"] = 1528, ["y"] = 1443, ["z"] = 600, ["order"] = 10, ["angle"] = 1804009472},
  [50] = {["x"] = 564, ["y"] = 1811, ["z"] = 677, ["order"] = 10, ["angle"] = 1848180736},
  [51] = {["x"] = -424, ["y"] = 2030, ["z"] = 837, ["order"] = 10, ["angle"] = 1733754880},
  [52] = {["x"] = -1401, ["y"] = 2393, ["z"] = 936, ["order"] = 10, ["angle"] = 1289158656},
  [53] = {["x"] = -1878, ["y"] = 3070, ["z"] = 987, ["order"] = 11, ["angle"] = 1232928768},
  [54] = {["x"] = -2176, ["y"] = 3839, ["z"] = 894, ["order"] = 11, ["angle"] = 1147469824},
  [55] = {["x"] = -2336, ["y"] = 4658, ["z"] = 850, ["order"] = 11, ["angle"] = 1104609280},
  [56] = {["x"] = -2553, ["y"] = 5524, ["z"] = 840, ["order"] = 11, ["angle"] = 1053884416},
  [57] = {["x"] = -2694, ["y"] = 6383, ["z"] = 870, ["order"] = 11, ["angle"] = 770506752},
  [58] = {["x"] = -2587, ["y"] = 7206, ["z"] = 870, ["order"] = 11, ["angle"] = 405078016},
  [59] = {["x"] = -2142, ["y"] = 7767, ["z"] = 870, ["order"] = 11, ["angle"] = 182648832},
  [60] = {["x"] = -1425, ["y"] = 8051, ["z"] = 835, ["order"] = 12, ["angle"] = -110952448},
  [61] = {["x"] = -500, ["y"] = 8108, ["z"] = 723, ["order"] = 12, ["angle"] = -81199104},
  [62] = {["x"] = 265, ["y"] = 8021, ["z"] = 700, ["order"] = 12, ["angle"] = -106758144},
  [63] = {["x"] = 1093, ["y"] = 7826, ["z"] = 700, ["order"] = 0, ["angle"] = -234160128},
  [64] = {["x"] = 1903, ["y"] = 7515, ["z"] = 873, ["order"] = 0, ["angle"] = -264962048},
  [65] = {["x"] = 2497, ["y"] = 7216, ["z"] = 1056, ["order"] = 0, ["angle"] = -175308800},
  [66] = {["x"] = 3215, ["y"] = 7007, ["z"] = 1084, ["order"] = 1, ["angle"] = -95092736},
  [67] = {["x"] = 6076, ["y"] = -2277, ["z"] = 300, ["order"] = 3, ["angle"] = -891879424},
  [68] = {["x"] = 6379, ["y"] = -3133, ["z"] = 300, ["order"] = 3, ["angle"] = -840892416},
  [69] = {["x"] = 6499, ["y"] = -3758, ["z"] = 300, ["order"] = 3, ["angle"] = -1101201408},
  [70] = {["x"] = 6291, ["y"] = -4396, ["z"] = 300, ["order"] = 3, ["angle"] = -1363869696},
  [71] = {["x"] = 6025, ["y"] = -5149, ["z"] = 300, ["order"] = 3, ["angle"] = -1143537664},
  [72] = {["x"] = 6890, ["y"] = -4215, ["z"] = 300, ["order"] = 3, ["angle"] = -1126498304},
  [73] = {["x"] = 6839, ["y"] = -5048, ["z"] = 300, ["order"] = 3, ["angle"] = -1540947968},
  [74] = {["x"] = 6244, ["y"] = -5839, ["z"] = 300, ["order"] = 3, ["angle"] = -1220345856},
  [75] = {["x"] = 10053, ["y"] = -6289, ["z"] = 700, ["order"] = 6, ["angle"] = 680853504},
  [76] = {["x"] = 10636, ["y"] = -5302, ["z"] = 700, ["order"] = 6, ["angle"] = 781910016},
  [77] = {["x"] = 11089, ["y"] = -4359, ["z"] = 700, ["order"] = 6, ["angle"] = 1258618880}
}
-- Respawn definitions for TWINKLE CART (MAP14)
krespawn.points[14] = {
  [0] = 211,
  [1] = {["x"] = 23, ["y"] = -1709, ["z"] = 0, ["order"] = 0, ["angle"] = 1091502080},
  [2] = {["x"] = -10, ["y"] = -950, ["z"] = 0, ["order"] = 0, ["angle"] = 1052573696},
  [3] = {["x"] = -8, ["y"] = -7, ["z"] = 0, ["order"] = 0, ["angle"] = 1091371008},
  [4] = {["x"] = -15, ["y"] = 826, ["z"] = 0, ["order"] = 0, ["angle"] = 1065287680},
  [5] = {["x"] = -299, ["y"] = 1591, ["z"] = 0, ["order"] = 0, ["angle"] = 1035927552},
  [6] = {["x"] = 247, ["y"] = 1562, ["z"] = 0, ["order"] = 0, ["angle"] = 1110638592},
  [7] = {["x"] = -15, ["y"] = 2859, ["z"] = 0, ["order"] = 0, ["angle"] = 1047724032},
  [8] = {["x"] = 3, ["y"] = 3934, ["z"] = 0, ["order"] = 0, ["angle"] = 1071972352},
  [9] = {["x"] = 13, ["y"] = 5054, ["z"] = -91, ["order"] = 0, ["angle"] = 1071972352},
  [10] = {["x"] = -53, ["y"] = 6181, ["z"] = -229, ["order"] = 0, ["angle"] = 1026228224},
  [11] = {["x"] = -31, ["y"] = 7383, ["z"] = -272, ["order"] = 1, ["angle"] = 604831744},
  [12] = {["x"] = 415, ["y"] = 8104, ["z"] = -272, ["order"] = 1, ["angle"] = 190906368},
  [13] = {["x"] = 1168, ["y"] = 8281, ["z"] = -272, ["order"] = 1, ["angle"] = 161415168},
  [14] = {["x"] = 2117, ["y"] = 8370, ["z"] = -272, ["order"] = 2, ["angle"] = 304545792},
  [15] = {["x"] = 2979, ["y"] = 8570, ["z"] = -272, ["order"] = 2, ["angle"] = 581107712},
  [16] = {["x"] = 3547, ["y"] = 9187, ["z"] = -272, ["order"] = 2, ["angle"] = 914423808},
  [17] = {["x"] = 3673, ["y"] = 9781, ["z"] = -272, ["order"] = 2, ["angle"] = 1046937600},
  [18] = {["x"] = 3784, ["y"] = 10618, ["z"] = -272, ["order"] = 3, ["angle"] = 1234632704},
  [19] = {["x"] = 3670, ["y"] = 11289, ["z"] = -272, ["order"] = 3, ["angle"] = 1720254464},
  [20] = {["x"] = 3265, ["y"] = 11661, ["z"] = -272, ["order"] = 4, ["angle"] = 2137718784},
  [21] = {["x"] = 2517, ["y"] = 11772, ["z"] = -272, ["order"] = 4, ["angle"] = -2083323904},
  [22] = {["x"] = 1302, ["y"] = 11793, ["z"] = -784, ["order"] = 4, ["angle"] = 2147155968},
  [23] = {["x"] = 707, ["y"] = 11768, ["z"] = -784, ["order"] = 4, ["angle"] = -2103246848},
  [24] = {["x"] = -229, ["y"] = 11725, ["z"] = -784, ["order"] = 4, ["angle"] = 2124611584},
  [25] = {["x"] = -1173, ["y"] = 11801, ["z"] = -784, ["order"] = 4, ["angle"] = 2033385472},
  [26] = {["x"] = -2083, ["y"] = 12021, ["z"] = -784, ["order"] = 5, ["angle"] = 1991049216},
  [27] = {["x"] = -2969, ["y"] = 12318, ["z"] = -784, ["order"] = 5, ["angle"] = -1824849920},
  [28] = {["x"] = -3944, ["y"] = 12055, ["z"] = -784, ["order"] = 5, ["angle"] = -1382350848},
  [29] = {["x"] = -4373, ["y"] = 11497, ["z"] = -784, ["order"] = 5, ["angle"] = -1161625600},
  [30] = {["x"] = -4534, ["y"] = 10903, ["z"] = -784, ["order"] = 5, ["angle"] = -889257984},
  [31] = {["x"] = -4348, ["y"] = 10273, ["z"] = -784, ["order"] = 5, ["angle"] = -867368960},
  [32] = {["x"] = -4059, ["y"] = 9588, ["z"] = -784, ["order"] = 5, ["angle"] = -1059651584},
  [33] = {["x"] = -3955, ["y"] = 8763, ["z"] = -784, ["order"] = 5, ["angle"] = -1594556416},
  [34] = {["x"] = -4442, ["y"] = 8225, ["z"] = -784, ["order"] = 6, ["angle"] = -1926955008},
  [35] = {["x"] = -5326, ["y"] = 7996, ["z"] = -784, ["order"] = 6, ["angle"] = -1953038336},
  [36] = {["x"] = -6111, ["y"] = 7930, ["z"] = -784, ["order"] = 6, ["angle"] = -1679753216},
  [37] = {["x"] = -6729, ["y"] = 7639, ["z"] = -784, ["order"] = 6, ["angle"] = -1473708032},
  [38] = {["x"] = -7070, ["y"] = 7075, ["z"] = -784, ["order"] = 6, ["angle"] = -1227554816},
  [39] = {["x"] = -7211, ["y"] = 6079, ["z"] = -784, ["order"] = 6, ["angle"] = -1090584576},
  [40] = {["x"] = -7243, ["y"] = 3229, ["z"] = -664, ["order"] = 7, ["angle"] = -1073676288},
  [41] = {["x"] = -7243, ["y"] = 2391, ["z"] = -664, ["order"] = 7, ["angle"] = -1073676288},
  [42] = {["x"] = -7243, ["y"] = 1606, ["z"] = -664, ["order"] = 7, ["angle"] = -1073676288},
  [43] = {["x"] = -7243, ["y"] = 871, ["z"] = -664, ["order"] = 7, ["angle"] = -1073676288},
  [44] = {["x"] = -7243, ["y"] = 121, ["z"] = -556, ["order"] = 7, ["angle"] = -1073676288},
  [45] = {["x"] = -7243, ["y"] = -693, ["z"] = -356, ["order"] = 7, ["angle"] = -1073676288},
  [46] = {["x"] = -7243, ["y"] = -1718, ["z"] = -216, ["order"] = 7, ["angle"] = -1073676288},
  [47] = {["x"] = -7243, ["y"] = -2730, ["z"] = -216, ["order"] = 7, ["angle"] = -1073676288},
  [48] = {["x"] = -7264, ["y"] = -3581, ["z"] = -59, ["order"] = 8, ["angle"] = -1447493632},
  [49] = {["x"] = -7797, ["y"] = -4517, ["z"] = 188, ["order"] = 8, ["angle"] = -1928396800},
  [50] = {["x"] = -8493, ["y"] = -4784, ["z"] = 259, ["order"] = 8, ["angle"] = 1847918592},
  [51] = {["x"] = -9392, ["y"] = -4577, ["z"] = 205, ["order"] = 8, ["angle"] = 1371340800},
  [52] = {["x"] = -9855, ["y"] = -4072, ["z"] = 71, ["order"] = 8, ["angle"] = 1061093376},
  [53] = {["x"] = -10147, ["y"] = -3144, ["z"] = -175, ["order"] = 8, ["angle"] = 1058340864},
  [54] = {["x"] = -9740, ["y"] = -2060, ["z"] = -216, ["order"] = 9, ["angle"] = 1079181312},
  [55] = {["x"] = -10120, ["y"] = -747, ["z"] = -216, ["order"] = 9, ["angle"] = 986120192},
  [56] = {["x"] = -9725, ["y"] = 479, ["z"] = -216, ["order"] = 9, ["angle"] = 1069481984},
  [57] = {["x"] = -9922, ["y"] = 1643, ["z"] = -216, ["order"] = 9, ["angle"] = 1079443456},
  [58] = {["x"] = -9914, ["y"] = 2347, ["z"] = -216, ["order"] = 9, ["angle"] = 1079443456},
  [59] = {["x"] = -9787, ["y"] = 3262, ["z"] = -215, ["order"] = 0, ["angle"] = 1320484864},
  [60] = {["x"] = -9894, ["y"] = 4033, ["z"] = 0, ["order"] = 0, ["angle"] = 1527578624},
  [61] = {["x"] = -10339, ["y"] = 4718, ["z"] = 216, ["order"] = 0, ["angle"] = 2045313024},
  [62] = {["x"] = -11039, ["y"] = 4990, ["z"] = 302, ["order"] = 0, ["angle"] = -1865089024},
  [63] = {["x"] = -11778, ["y"] = 4911, ["z"] = 277, ["order"] = 0, ["angle"] = -1506607104},
  [64] = {["x"] = -12235, ["y"] = 4466, ["z"] = 136, ["order"] = 0, ["angle"] = -1219428352},
  [65] = {["x"] = -12427, ["y"] = 3838, ["z"] = 32, ["order"] = 0, ["angle"] = -1043267584},
  [66] = {["x"] = -12487, ["y"] = 3127, ["z"] = 200, ["order"] = 1, ["angle"] = -900530176},
  [67] = {["x"] = -12356, ["y"] = 2231, ["z"] = 200, ["order"] = 1, ["angle"] = -398393344},
  [68] = {["x"] = -11695, ["y"] = 1703, ["z"] = 200, ["order"] = 1, ["angle"] = 34537472},
  [69] = {["x"] = -10895, ["y"] = 1628, ["z"] = 296, ["order"] = 1, ["angle"] = -23658496},
  [70] = {["x"] = -10005, ["y"] = 1598, ["z"] = 296, ["order"] = 2, ["angle"] = -23658496},
  [71] = {["x"] = -8935, ["y"] = 1626, ["z"] = 296, ["order"] = 2, ["angle"] = 3735552},
  [72] = {["x"] = -6025, ["y"] = 1633, ["z"] = 345, ["order"] = 2, ["angle"] = 65536},
  [73] = {["x"] = -5372, ["y"] = 1722, ["z"] = 333, ["order"] = 2, ["angle"] = -202178560},
  [74] = {["x"] = -4434, ["y"] = 1523, ["z"] = 360, ["order"] = 3, ["angle"] = -430768128},
  [75] = {["x"] = -3711, ["y"] = 918, ["z"] = 442, ["order"] = 3, ["angle"] = -1191903232},
  [76] = {["x"] = -3734, ["y"] = 283, ["z"] = 529, ["order"] = 3, ["angle"] = -1881997312},
  [77] = {["x"] = -4476, ["y"] = -46, ["z"] = 573, ["order"] = 3, ["angle"] = -1797455872},
  [78] = {["x"] = -5233, ["y"] = -310, ["z"] = 609, ["order"] = 3, ["angle"] = -1425866752},
  [79] = {["x"] = -5763, ["y"] = -924, ["z"] = 693, ["order"] = 3, ["angle"] = -583598080},
  [80] = {["x"] = -5283, ["y"] = -1504, ["z"] = 771, ["order"] = 3, ["angle"] = -278069248},
  [81] = {["x"] = -4410, ["y"] = -1726, ["z"] = 801, ["order"] = 3, ["angle"] = -483852288},
  [82] = {["x"] = -3803, ["y"] = -2196, ["z"] = 865, ["order"] = 3, ["angle"] = -816250880},
  [83] = {["x"] = -3391, ["y"] = -2957, ["z"] = 969, ["order"] = 3, ["angle"] = -1003552768},
  [84] = {["x"] = -3251, ["y"] = -4116, ["z"] = 992, ["order"] = 3, ["angle"] = -1344864256},
  [85] = {["x"] = -3358, ["y"] = -5075, ["z"] = 1048, ["order"] = 4, ["angle"] = -1470693376},
  [86] = {["x"] = -3654, ["y"] = -5781, ["z"] = 1111, ["order"] = 4, ["angle"] = -1934032896},
  [87] = {["x"] = -4355, ["y"] = -6266, ["z"] = 1155, ["order"] = 4, ["angle"] = -2071003136},
  [88] = {["x"] = -5379, ["y"] = -6392, ["z"] = 1186, ["order"] = 4, ["angle"] = -2122121216},
  [89] = {["x"] = -6512, ["y"] = -6378, ["z"] = 1200, ["order"] = 4, ["angle"] = -2130378752},
  [90] = {["x"] = -7344, ["y"] = -6374, ["z"] = 1200, ["order"] = 4, ["angle"] = -1919483904},
  [91] = {["x"] = -8117, ["y"] = -6512, ["z"] = 1200, ["order"] = 4, ["angle"] = -1518272512},
  [92] = {["x"] = -8670, ["y"] = -7191, ["z"] = 1200, ["order"] = 4, ["angle"] = -1748041728},
  [93] = {["x"] = -9537, ["y"] = -7745, ["z"] = 1200, ["order"] = 4, ["angle"] = -2135228416},
  [94] = {["x"] = -10571, ["y"] = -8030, ["z"] = 1200, ["order"] = 4, ["angle"] = -2118975488},
  [95] = {["x"] = -11789, ["y"] = -8111, ["z"] = 1200, ["order"] = 4, ["angle"] = 1918697472},
  [96] = {["x"] = -12846, ["y"] = -7849, ["z"] = 1200, ["order"] = 5, ["angle"] = 1503592448},
  [97] = {["x"] = -13378, ["y"] = -7326, ["z"] = 1200, ["order"] = 5, ["angle"] = 1207238656},
  [98] = {["x"] = -13638, ["y"] = -6584, ["z"] = 1200, ["order"] = 6, ["angle"] = 1385889792},
  [99] = {["x"] = -14099, ["y"] = -6108, ["z"] = 1200, ["order"] = 6, ["angle"] = 1458110464},
  [100] = {["x"] = -14909, ["y"] = -5653, ["z"] = 1200, ["order"] = 6, ["angle"] = 1102381056},
  [101] = {["x"] = -14487, ["y"] = -5335, ["z"] = 1200, ["order"] = 6, ["angle"] = 1077215232},
  [102] = {["x"] = -15002, ["y"] = -5040, ["z"] = 1200, ["order"] = 6, ["angle"] = 1092812800},
  [103] = {["x"] = -14710, ["y"] = -4366, ["z"] = 1200, ["order"] = 6, ["angle"] = 1072496640},
  [104] = {["x"] = -14721, ["y"] = -3606, ["z"] = 1200, ["order"] = 6, ["angle"] = 1072496640},
  [105] = {["x"] = -14721, ["y"] = -2767, ["z"] = 1200, ["order"] = 6, ["angle"] = 1072496640},
  [106] = {["x"] = -14720, ["y"] = -2019, ["z"] = 1200, ["order"] = 6, ["angle"] = 1072496640},
  [107] = {["x"] = -14718, ["y"] = -1061, ["z"] = 1200, ["order"] = 6, ["angle"] = 1072496640},
  [108] = {["x"] = -14716, ["y"] = -210, ["z"] = 1200, ["order"] = 7, ["angle"] = 1072496640},
  [109] = {["x"] = -14714, ["y"] = 501, ["z"] = 1258, ["order"] = 7, ["angle"] = 1072496640},
  [110] = {["x"] = -14643, ["y"] = 1370, ["z"] = 1346, ["order"] = 7, ["angle"] = 1341456384},
  [111] = {["x"] = -14766, ["y"] = 1989, ["z"] = 1409, ["order"] = 7, ["angle"] = 1654456320},
  [112] = {["x"] = -15186, ["y"] = 2450, ["z"] = 1455, ["order"] = 7, ["angle"] = -2145845248},
  [113] = {["x"] = -15967, ["y"] = 2606, ["z"] = 1504, ["order"] = 8, ["angle"] = -1800994816},
  [114] = {["x"] = -16526, ["y"] = 2410, ["z"] = 1504, ["order"] = 8, ["angle"] = -1397424128},
  [115] = {["x"] = -16860, ["y"] = 1996, ["z"] = 1504, ["order"] = 8, ["angle"] = -1128595456},
  [116] = {["x"] = -17013, ["y"] = 1334, ["z"] = 1504, ["order"] = 8, ["angle"] = -983367680},
  [117] = {["x"] = -17067, ["y"] = 628, ["z"] = 1504, ["order"] = 8, ["angle"] = -900923392},
  [118] = {["x"] = -17063, ["y"] = -9, ["z"] = 1504, ["order"] = 8, ["angle"] = -572981248},
  [119] = {["x"] = -16593, ["y"] = -400, ["z"] = 1504, ["order"] = 8, ["angle"] = -545849344},
  [120] = {["x"] = -16034, ["y"] = -847, ["z"] = 1504, ["order"] = 8, ["angle"] = -1063452672},
  [121] = {["x"] = -15788, ["y"] = -1879, ["z"] = 1504, ["order"] = 0, ["angle"] = -1061093376},
  [122] = {["x"] = -13453, ["y"] = -6158, ["z"] = 1200, ["order"] = 6, ["angle"] = 337051648},
  [123] = {["x"] = -12209, ["y"] = -5904, ["z"] = 1200, ["order"] = 6, ["angle"] = 1039335424},
  [124] = {["x"] = -12715, ["y"] = -5146, ["z"] = 1200, ["order"] = 6, ["angle"] = 1101594624},
  [125] = {["x"] = -12428, ["y"] = -4340, ["z"] = 1200, ["order"] = 6, ["angle"] = 1107230720},
  [126] = {["x"] = -12406, ["y"] = -3652, ["z"] = 1200, ["order"] = 6, ["angle"] = 1098579968},
  [127] = {["x"] = -12343, ["y"] = -2807, ["z"] = 1200, ["order"] = 6, ["angle"] = 1157955584},
  [128] = {["x"] = -12307, ["y"] = -1901, ["z"] = 1200, ["order"] = 6, ["angle"] = 1398603776},
  [129] = {["x"] = -12593, ["y"] = -1200, ["z"] = 1200, ["order"] = 6, ["angle"] = 1771372544},
  [130] = {["x"] = -13168, ["y"] = -891, ["z"] = 1200, ["order"] = 6, ["angle"] = 2036006912},
  [131] = {["x"] = -14126, ["y"] = -771, ["z"] = 1200, ["order"] = 6, ["angle"] = 1629159424},
  [132] = {["x"] = -15748, ["y"] = -2485, ["z"] = 1433, ["order"] = 0, ["angle"] = -1079705600},
  [133] = {["x"] = -15761, ["y"] = -3801, ["z"] = 886, ["order"] = 0, ["angle"] = -1079705600},
  [134] = {["x"] = -15769, ["y"] = -4907, ["z"] = 1191, ["order"] = 0, ["angle"] = -1079705600},
  [135] = {["x"] = -15776, ["y"] = -5792, ["z"] = 1496, ["order"] = 0, ["angle"] = -1079705600},
  [136] = {["x"] = -15784, ["y"] = -6633, ["z"] = 1299, ["order"] = 0, ["angle"] = -1079705600},
  [137] = {["x"] = -15776, ["y"] = -7727, ["z"] = 880, ["order"] = 0, ["angle"] = -1032912896},
  [138] = {["x"] = -15736, ["y"] = -8708, ["z"] = 1173, ["order"] = 0, ["angle"] = -1079443456},
  [139] = {["x"] = -15725, ["y"] = -9486, ["z"] = 1483, ["order"] = 0, ["angle"] = -1079443456},
  [140] = {["x"] = -15726, ["y"] = -10382, ["z"] = 1504, ["order"] = 0, ["angle"] = -1079443456},
  [141] = {["x"] = -15823, ["y"] = -11283, ["z"] = 1516, ["order"] = 0, ["angle"] = -915341312},
  [142] = {["x"] = -15683, ["y"] = -12303, ["z"] = 1743, ["order"] = 1, ["angle"] = -314376192},
  [143] = {["x"] = -14926, ["y"] = -12925, ["z"] = 1885, ["order"] = 1, ["angle"] = -7667712},
  [144] = {["x"] = -13952, ["y"] = -13111, ["z"] = 1928, ["order"] = 1, ["angle"] = 106627072},
  [145] = {["x"] = -12728, ["y"] = -13084, ["z"] = 2000, ["order"] = 1, ["angle"] = 22609920},
  [146] = {["x"] = -11514, ["y"] = -13133, ["z"] = 1992, ["order"] = 1, ["angle"] = 211353600},
  [147] = {["x"] = -10538, ["y"] = -13089, ["z"] = 2000, ["order"] = 2, ["angle"] = 440860672},
  [148] = {["x"] = -9718, ["y"] = -12663, ["z"] = 2000, ["order"] = 2, ["angle"] = 959512576},
  [149] = {["x"] = -9429, ["y"] = -11796, ["z"] = 2000, ["order"] = 3, ["angle"] = 1805058048},
  [150] = {["x"] = -10033, ["y"] = -11281, ["z"] = 2000, ["order"] = 3, ["angle"] = -2091712512},
  [151] = {["x"] = -11857, ["y"] = -11232, ["z"] = 2034, ["order"] = 3, ["angle"] = -2146762752},
  [152] = {["x"] = -12680, ["y"] = -11337, ["z"] = 2161, ["order"] = 3, ["angle"] = 1973354496},
  [153] = {["x"] = -13692, ["y"] = -11340, ["z"] = 2256, ["order"] = 4, ["angle"] = 1668612096},
  [154] = {["x"] = -14630, ["y"] = -10852, ["z"] = 2256, ["order"] = 4, ["angle"] = 914161664},
  [155] = {["x"] = -14682, ["y"] = -9941, ["z"] = 2256, ["order"] = 5, ["angle"] = 302710784},
  [156] = {["x"] = -13952, ["y"] = -9416, ["z"] = 2256, ["order"] = 5, ["angle"] = -24182784},
  [157] = {["x"] = -12981, ["y"] = -9328, ["z"] = 2256, ["order"] = 5, ["angle"] = -78970880},
  [158] = {["x"] = -11083, ["y"] = -9362, ["z"] = 2286, ["order"] = 5, ["angle"] = 4915200},
  [159] = {["x"] = -10330, ["y"] = -9360, ["z"] = 2402, ["order"] = 5, ["angle"] = 4915200},
  [160] = {["x"] = -9509, ["y"] = -9355, ["z"] = 2512, ["order"] = 5, ["angle"] = 4915200},
  [161] = {["x"] = -8218, ["y"] = -9346, ["z"] = 2512, ["order"] = 5, ["angle"] = 4915200},
  [162] = {["x"] = -7904, ["y"] = -8162, ["z"] = 2512, ["order"] = 5, ["angle"] = 58785792},
  [163] = {["x"] = -7812, ["y"] = -10564, ["z"] = 2512, ["order"] = 5, ["angle"] = -44105728},
  [164] = {["x"] = -4828, ["y"] = -9368, ["z"] = 2512, ["order"] = 5, ["angle"] = 65536},
  [165] = {["x"] = -3258, ["y"] = -9368, ["z"] = 2512, ["order"] = 5, ["angle"] = 65536},
  [166] = {["x"] = -1922, ["y"] = -9368, ["z"] = 2512, ["order"] = 5, ["angle"] = 65536},
  [167] = {["x"] = -1080, ["y"] = -9368, ["z"] = 2512, ["order"] = 5, ["angle"] = 65536},
  [168] = {["x"] = -199, ["y"] = -9460, ["z"] = 2512, ["order"] = 5, ["angle"] = -362741760},
  [169] = {["x"] = 221, ["y"] = -9804, ["z"] = 2512, ["order"] = 5, ["angle"] = -22085632},
  [170] = {["x"] = 275, ["y"] = -10271, ["z"] = 2512, ["order"] = 5, ["angle"] = 24838144},
  [171] = {["x"] = 191, ["y"] = -8864, ["z"] = 2512, ["order"] = 5, ["angle"] = -17235968},
  [172] = {["x"] = 250, ["y"] = -8418, ["z"] = 2512, ["order"] = 5, ["angle"] = 33488896},
  [173] = {["x"] = 661, ["y"] = -7853, ["z"] = 2512, ["order"] = 5, ["angle"] = -10289152},
  [174] = {["x"] = 834, ["y"] = -10741, ["z"] = 2512, ["order"] = 5, ["angle"] = 22085632},
  [175] = {["x"] = -6523, ["y"] = -10599, ["z"] = 2512, ["order"] = 5, ["angle"] = -26804224},
  [176] = {["x"] = -5789, ["y"] = -10602, ["z"] = 2512, ["order"] = 5, ["angle"] = -10027008},
  [177] = {["x"] = -4836, ["y"] = -10604, ["z"] = 2512, ["order"] = 5, ["angle"] = -10027008},
  [178] = {["x"] = -3484, ["y"] = -10657, ["z"] = 2512, ["order"] = 5, ["angle"] = 205979648},
  [179] = {["x"] = -3649, ["y"] = -10175, ["z"] = 2512, ["order"] = 5, ["angle"] = 456589312},
  [180] = {["x"] = -3688, ["y"] = -8832, ["z"] = 2512, ["order"] = 5, ["angle"] = -259719168},
  [181] = {["x"] = -3342, ["y"] = -8021, ["z"] = 2512, ["order"] = 5, ["angle"] = 10158080},
  [182] = {["x"] = -6898, ["y"] = -8013, ["z"] = 2512, ["order"] = 5, ["angle"] = -851968},
  [183] = {["x"] = -6358, ["y"] = -8023, ["z"] = 2512, ["order"] = 5, ["angle"] = -851968},
  [184] = {["x"] = -5566, ["y"] = -8026, ["z"] = 2512, ["order"] = 5, ["angle"] = -851968},
  [185] = {["x"] = -4779, ["y"] = -8027, ["z"] = 2512, ["order"] = 5, ["angle"] = -851968},
  [186] = {["x"] = -4003, ["y"] = -8029, ["z"] = 2512, ["order"] = 5, ["angle"] = -20119552},
  [187] = {["x"] = -2586, ["y"] = -9322, ["z"] = 2512, ["order"] = 5, ["angle"] = -10289152},
  [188] = {["x"] = 1577, ["y"] = -10793, ["z"] = 2512, ["order"] = 5, ["angle"] = 27328512},
  [189] = {["x"] = 2256, ["y"] = -10785, ["z"] = 2512, ["order"] = 5, ["angle"] = -14614528},
  [190] = {["x"] = 3420, ["y"] = -10766, ["z"] = 2512, ["order"] = 5, ["angle"] = 459735040},
  [191] = {["x"] = 3119, ["y"] = -10125, ["z"] = 2512, ["order"] = 5, ["angle"] = 9240576},
  [192] = {["x"] = 3967, ["y"] = -9900, ["z"] = 2512, ["order"] = 5, ["angle"] = 206503936},
  [193] = {["x"] = 3870, ["y"] = -9126, ["z"] = 2512, ["order"] = 5, ["angle"] = -274137088},
  [194] = {["x"] = 3024, ["y"] = -8676, ["z"] = 2512, ["order"] = 5, ["angle"] = -12648448},
  [195] = {["x"] = 3508, ["y"] = -7898, ["z"] = 2512, ["order"] = 5, ["angle"] = -405340160},
  [196] = {["x"] = 1563, ["y"] = -7908, ["z"] = 2512, ["order"] = 5, ["angle"] = -7929856},
  [197] = {["x"] = 2408, ["y"] = -7890, ["z"] = 2512, ["order"] = 5, ["angle"] = 9764864},
  [198] = {["x"] = 4673, ["y"] = -9348, ["z"] = 2512, ["order"] = 5, ["angle"] = 13434880},
  [199] = {["x"] = 5666, ["y"] = -9348, ["z"] = 2512, ["order"] = 5, ["angle"] = 10682368},
  [200] = {["x"] = 6646, ["y"] = -9466, ["z"] = 2512, ["order"] = 6, ["angle"] = 167575552},
  [201] = {["x"] = 7724, ["y"] = -9328, ["z"] = 2512, ["order"] = 6, ["angle"] = 548864000},
  [202] = {["x"] = 8336, ["y"] = -8796, ["z"] = 2512, ["order"] = 6, ["angle"] = 924647424},
  [203] = {["x"] = 8511, ["y"] = -8015, ["z"] = 2512, ["order"] = 6, ["angle"] = 1090191360},
  [204] = {["x"] = 8538, ["y"] = -7055, ["z"] = 2505, ["order"] = 6, ["angle"] = 1062010880},
  [205] = {["x"] = 8544, ["y"] = -5910, ["z"] = 1981, ["order"] = 6, ["angle"] = 1091502080},
  [206] = {["x"] = 8535, ["y"] = -4657, ["z"] = 1346, ["order"] = 6, ["angle"] = 1070792704},
  [207] = {["x"] = 8536, ["y"] = -3487, ["z"] = 753, ["order"] = 6, ["angle"] = 1070792704},
  [208] = {["x"] = 8559, ["y"] = -2200, ["z"] = 552, ["order"] = 6, ["angle"] = 1058865152},
  [209] = {["x"] = 8572, ["y"] = 2363, ["z"] = 1640, ["order"] = 6, ["angle"] = 1073807360},
  [210] = {["x"] = 8572, ["y"] = 3526, ["z"] = 1640, ["order"] = 6, ["angle"] = 1073807360},
  [211] = {["x"] = 8572, ["y"] = 4828, ["z"] = 1640, ["order"] = 6, ["angle"] = 1073807360}
}
-- Respawn definitions for PLEASURE CASTLE (MAP15)
krespawn.points[15] = {
  [0] = 94,
  [1] = {["x"] = -1079, ["y"] = -6628, ["z"] = 512, ["order"] = 0, ["angle"] = 2140471296},
  [2] = {["x"] = -1926, ["y"] = -6625, ["z"] = 512, ["order"] = 0, ["angle"] = 2140471296},
  [3] = {["x"] = -3032, ["y"] = -6870, ["z"] = 512, ["order"] = 0, ["angle"] = 2128019456},
  [4] = {["x"] = -3756, ["y"] = -6456, ["z"] = 520, ["order"] = 0, ["angle"] = -2135228416},
  [5] = {["x"] = -4179, ["y"] = -6674, ["z"] = 520, ["order"] = 0, ["angle"] = 2146107392},
  [6] = {["x"] = -5223, ["y"] = -6892, ["z"] = 520, ["order"] = 0, ["angle"] = -2142961664},
  [7] = {["x"] = -5149, ["y"] = -6454, ["z"] = 520, ["order"] = 0, ["angle"] = 2117664768},
  [8] = {["x"] = -6351, ["y"] = -6726, ["z"] = 520, ["order"] = 1, ["angle"] = 1863385088},
  [9] = {["x"] = -7045, ["y"] = -6331, ["z"] = 512, ["order"] = 1, ["angle"] = 1299906560},
  [10] = {["x"] = -7330, ["y"] = -5471, ["z"] = 502, ["order"] = 1, ["angle"] = 1039073280},
  [11] = {["x"] = -7440, ["y"] = -4337, ["z"] = 384, ["order"] = 2, ["angle"] = 485687296},
  [12] = {["x"] = -6867, ["y"] = -3879, ["z"] = 384, ["order"] = 3, ["angle"] = -718995456},
  [13] = {["x"] = -6283, ["y"] = -4909, ["z"] = 384, ["order"] = 4, ["angle"] = -122355712},
  [14] = {["x"] = -5320, ["y"] = -4703, ["z"] = 384, ["order"] = 4, ["angle"] = -21430272},
  [15] = {["x"] = -5324, ["y"] = -5493, ["z"] = 384, ["order"] = 4, ["angle"] = 150405120},
  [16] = {["x"] = -4195, ["y"] = -5064, ["z"] = 384, ["order"] = 4, ["angle"] = 1638400},
  [17] = {["x"] = -3116, ["y"] = -5114, ["z"] = 512, ["order"] = 5, ["angle"] = 389480448},
  [18] = {["x"] = -1737, ["y"] = -4186, ["z"] = 512, ["order"] = 5, ["angle"] = 975503360},
  [19] = {["x"] = -1582, ["y"] = -3441, ["z"] = 512, ["order"] = 5, ["angle"] = 1052442624},
  [20] = {["x"] = -1539, ["y"] = -2488, ["z"] = 512, ["order"] = 5, ["angle"] = 1052442624},
  [21] = {["x"] = -1537, ["y"] = -1541, ["z"] = 520, ["order"] = 5, ["angle"] = 1095565312},
  [22] = {["x"] = -1268, ["y"] = -1567, ["z"] = 520, ["order"] = 5, ["angle"] = 1088880640},
  [23] = {["x"] = -1790, ["y"] = -1562, ["z"] = 520, ["order"] = 5, ["angle"] = 1087700992},
  [24] = {["x"] = -1283, ["y"] = -44, ["z"] = 520, ["order"] = 5, ["angle"] = 1165164544},
  [25] = {["x"] = -1758, ["y"] = -65, ["z"] = 520, ["order"] = 5, ["angle"] = 1136721920},
  [26] = {["x"] = -1437, ["y"] = 825, ["z"] = 512, ["order"] = 6, ["angle"] = 1375797248},
  [27] = {["x"] = -1466, ["y"] = 1555, ["z"] = 512, ["order"] = 6, ["angle"] = 1734803456},
  [28] = {["x"] = -2157, ["y"] = 2060, ["z"] = 512, ["order"] = 6, ["angle"] = 2143485952},
  [29] = {["x"] = -3065, ["y"] = 2195, ["z"] = 561, ["order"] = 6, ["angle"] = -2068119552},
  [30] = {["x"] = -3811, ["y"] = 2169, ["z"] = 701, ["order"] = 6, ["angle"] = -1917386752},
  [31] = {["x"] = -5677, ["y"] = 2109, ["z"] = 704, ["order"] = 7, ["angle"] = 1152188416},
  [32] = {["x"] = -5885, ["y"] = 3277, ["z"] = 694, ["order"] = 8, ["angle"] = 973930496},
  [33] = {["x"] = -5878, ["y"] = 4056, ["z"] = 676, ["order"] = 8, ["angle"] = 619118592},
  [34] = {["x"] = -5355, ["y"] = 4843, ["z"] = 600, ["order"] = 8, ["angle"] = 109379584},
  [35] = {["x"] = -4470, ["y"] = 5121, ["z"] = 494, ["order"] = 8, ["angle"] = -19070976},
  [36] = {["x"] = -3520, ["y"] = 5164, ["z"] = 386, ["order"] = 8, ["angle"] = -114491392},
  [37] = {["x"] = -2980, ["y"] = 5098, ["z"] = 384, ["order"] = 8, ["angle"] = -33882112},
  [38] = {["x"] = -1658, ["y"] = 5404, ["z"] = 384, ["order"] = 8, ["angle"] = -52101120},
  [39] = {["x"] = -805, ["y"] = 5177, ["z"] = 384, ["order"] = 8, ["angle"] = 22478848},
  [40] = {["x"] = 100, ["y"] = 5139, ["z"] = 384, ["order"] = 8, ["angle"] = -35717120},
  [41] = {["x"] = 1499, ["y"] = 5024, ["z"] = 384, ["order"] = 9, ["angle"] = -86573056},
  [42] = {["x"] = 1980, ["y"] = 5273, ["z"] = 128, ["order"] = 9, ["angle"] = -209125376},
  [43] = {["x"] = 2553, ["y"] = 5031, ["z"] = 128, ["order"] = 9, ["angle"] = -119472128},
  [44] = {["x"] = 3456, ["y"] = 4892, ["z"] = 128, ["order"] = 9, ["angle"] = -355008512},
  [45] = {["x"] = 4774, ["y"] = 4372, ["z"] = 128, ["order"] = 9, ["angle"] = -638386176},
  [46] = {["x"] = 5367, ["y"] = 4002, ["z"] = 128, ["order"] = 9, ["angle"] = -955711488},
  [47] = {["x"] = 5505, ["y"] = 3172, ["z"] = 128, ["order"] = 9, ["angle"] = -1033961472},
  [48] = {["x"] = 5565, ["y"] = 2355, ["z"] = 241, ["order"] = 10, ["angle"] = -1033961472},
  [49] = {["x"] = 5611, ["y"] = 1652, ["z"] = 373, ["order"] = 10, ["angle"] = -1033961472},
  [50] = {["x"] = 5601, ["y"] = 1077, ["z"] = 481, ["order"] = 10, ["angle"] = -789118976},
  [51] = {["x"] = 5842, ["y"] = 276, ["z"] = 570, ["order"] = 10, ["angle"] = -281083904},
  [52] = {["x"] = 6526, ["y"] = -38, ["z"] = 621, ["order"] = 10, ["angle"] = -400883712},
  [53] = {["x"] = 7243, ["y"] = -529, ["z"] = 699, ["order"] = 10, ["angle"] = -1089142784},
  [54] = {["x"] = 7406, ["y"] = -1081, ["z"] = 788, ["order"] = 10, ["angle"] = -1837826048},
  [55] = {["x"] = 6748, ["y"] = -1384, ["z"] = 837, ["order"] = 10, ["angle"] = -1604386816},
  [56] = {["x"] = 6118, ["y"] = -1769, ["z"] = 898, ["order"] = 10, ["angle"] = -1316683776},
  [57] = {["x"] = 5775, ["y"] = -2696, ["z"] = 1047, ["order"] = 10, ["angle"] = -1280901120},
  [58] = {["x"] = 5722, ["y"] = -3598, ["z"] = 1152, ["order"] = 11, ["angle"] = -1436614656},
  [59] = {["x"] = 5459, ["y"] = -4638, ["z"] = 1152, ["order"] = 11, ["angle"] = -1762721792},
  [60] = {["x"] = 4779, ["y"] = -5417, ["z"] = 1152, ["order"] = 11, ["angle"] = -1981087744},
  [61] = {["x"] = 3888, ["y"] = -5805, ["z"] = 1152, ["order"] = 11, ["angle"] = -2020540416},
  [62] = {["x"] = 2902, ["y"] = -6049, ["z"] = 1068, ["order"] = 11, ["angle"] = -1959854080},
  [63] = {["x"] = 1590, ["y"] = -6474, ["z"] = 694, ["order"] = 11, ["angle"] = -2113077248},
  [64] = {["x"] = 595, ["y"] = -6620, ["z"] = 512, ["order"] = 11, ["angle"] = -2061697024},
  [65] = {["x"] = -372, ["y"] = -6685, ["z"] = 512, ["order"] = 11, ["angle"] = 2141126656},
  [66] = {["x"] = -6081, ["y"] = -3481, ["z"] = 388, ["order"] = 4, ["angle"] = 6750208},
  [67] = {["x"] = -5073, ["y"] = -3471, ["z"] = 447, ["order"] = 4, ["angle"] = 6750208},
  [68] = {["x"] = -4053, ["y"] = -3462, ["z"] = 507, ["order"] = 4, ["angle"] = 6750208},
  [69] = {["x"] = -2087, ["y"] = 4624, ["z"] = 384, ["order"] = 8, ["angle"] = -1245184},
  [70] = {["x"] = -1199, ["y"] = 4733, ["z"] = 384, ["order"] = 8, ["angle"] = 229703680},
  [71] = {["x"] = 1316, ["y"] = 5675, ["z"] = 384, ["order"] = 9, ["angle"] = 63897600},
  [72] = {["x"] = 2195, ["y"] = 5805, ["z"] = 128, ["order"] = 9, ["angle"] = 25493504},
  [73] = {["x"] = 2928, ["y"] = 5856, ["z"] = 128, ["order"] = 9, ["angle"] = -19070976},
  [74] = {["x"] = 3865, ["y"] = 5679, ["z"] = 128, ["order"] = 9, ["angle"] = -596574208},
  [75] = {["x"] = 3927, ["y"] = 5188, ["z"] = 128, ["order"] = 9, ["angle"] = 558301184},
  [76] = {["x"] = 4381, ["y"] = 5902, ["z"] = 128, ["order"] = 9, ["angle"] = -62455808},
  [77] = {["x"] = 5358, ["y"] = 5738, ["z"] = 128, ["order"] = 9, ["angle"] = -966590464},
  [78] = {["x"] = 5712, ["y"] = 4804, ["z"] = 128, ["order"] = 9, ["angle"] = -1036976128},
  [79] = {["x"] = 5669, ["y"] = 4172, ["z"] = 128, ["order"] = 9, ["angle"] = -1097007104},
  [80] = {["x"] = 3390, ["y"] = 5408, ["z"] = 640, ["order"] = 9, ["angle"] = -29687808},
  [81] = {["x"] = 4877, ["y"] = 5393, ["z"] = 904, ["order"] = 9, ["angle"] = -1039073280},
  [82] = {["x"] = 4911, ["y"] = 4409, ["z"] = 928, ["order"] = 9, ["angle"] = -1149960192},
  [83] = {["x"] = 4877, ["y"] = 3981, ["z"] = 128, ["order"] = 9, ["angle"] = -334430208},
  [84] = {["x"] = 4115, ["y"] = 5363, ["z"] = 781, ["order"] = 9, ["angle"] = 23527424},
  [85] = {["x"] = 4960, ["y"] = 3288, ["z"] = 1024, ["order"] = 10, ["angle"] = -1360855040},
  [86] = {["x"] = 3800, ["y"] = 2996, ["z"] = 1256, ["order"] = 10, ["angle"] = -1172242432},
  [87] = {["x"] = 3709, ["y"] = 2341, ["z"] = 1256, ["order"] = 10, ["angle"] = -1084948480},
  [88] = {["x"] = 3681, ["y"] = 1358, ["z"] = 1256, ["order"] = 10, ["angle"] = -1131610112},
  [89] = {["x"] = 3605, ["y"] = -234, ["z"] = 704, ["order"] = 10, ["angle"] = -1096220672},
  [90] = {["x"] = 3702, ["y"] = -2154, ["z"] = 1152, ["order"] = 10, ["angle"] = -941293568},
  [91] = {["x"] = 3924, ["y"] = -2841, ["z"] = 1152, ["order"] = 10, ["angle"] = -1016266752},
  [92] = {["x"] = 4081, ["y"] = -3824, ["z"] = 1152, ["order"] = 11, ["angle"] = -1189412864},
  [93] = {["x"] = 4075, ["y"] = -4318, ["z"] = 1152, ["order"] = 11, ["angle"] = -1311703040},
  [94] = {["x"] = 3779, ["y"] = -5128, ["z"] = 1152, ["order"] = 11, ["angle"] = -1592066048}
}
-- Respawn definitions for PARADISE HILL (MAP16)
krespawn.points[16] = {
  [0] = 120,
  [1] = {["x"] = -141, ["y"] = 557, ["z"] = 0, ["order"] = 0, ["angle"] = 827654144},
  [2] = {["x"] = 127, ["y"] = 1417, ["z"] = 0, ["order"] = 0, ["angle"] = 432734208},
  [3] = {["x"] = 826, ["y"] = 2019, ["z"] = 0, ["order"] = 0, ["angle"] = 48693248},
  [4] = {["x"] = 1920, ["y"] = 2242, ["z"] = -26, ["order"] = 0, ["angle"] = -107282432},
  [5] = {["x"] = 2929, ["y"] = 2253, ["z"] = -120, ["order"] = 0, ["angle"] = -47382528},
  [6] = {["x"] = 3967, ["y"] = 2249, ["z"] = -120, ["order"] = 0, ["angle"] = -387383296},
  [7] = {["x"] = 4744, ["y"] = 1836, ["z"] = -120, ["order"] = 1, ["angle"] = -801308672},
  [8] = {["x"] = 4991, ["y"] = 1053, ["z"] = -120, ["order"] = 1, ["angle"] = -1121255424},
  [9] = {["x"] = 4876, ["y"] = -48, ["z"] = -92, ["order"] = 1, ["angle"] = -1019543552},
  [10] = {["x"] = 4804, ["y"] = -1185, ["z"] = -72, ["order"] = 2, ["angle"] = -805371904},
  [11] = {["x"] = 4938, ["y"] = -2502, ["z"] = -72, ["order"] = 2, ["angle"] = -475987968},
  [12] = {["x"] = 5510, ["y"] = -3217, ["z"] = -72, ["order"] = 2, ["angle"] = 71368704},
  [13] = {["x"] = 6442, ["y"] = -3500, ["z"] = -136, ["order"] = 2, ["angle"] = 167968768},
  [14] = {["x"] = 7672, ["y"] = -3536, ["z"] = -328, ["order"] = 2, ["angle"] = 390529024},
  [15] = {["x"] = 8389, ["y"] = -3176, ["z"] = -392, ["order"] = 2, ["angle"] = 867762176},
  [16] = {["x"] = 8747, ["y"] = -2500, ["z"] = -392, ["order"] = 2, ["angle"] = 1135017984},
  [17] = {["x"] = 8784, ["y"] = -19, ["z"] = -383, ["order"] = 3, ["angle"] = 1016135680},
  [18] = {["x"] = 8852, ["y"] = 1176, ["z"] = -364, ["order"] = 4, ["angle"] = 960299008},
  [19] = {["x"] = 8984, ["y"] = 2196, ["z"] = -358, ["order"] = 4, ["angle"] = 628686848},
  [20] = {["x"] = 10151, ["y"] = 2917, ["z"] = -178, ["order"] = 4, ["angle"] = -284098560},
  [21] = {["x"] = 10904, ["y"] = 2380, ["z"] = -312, ["order"] = 4, ["angle"] = -899612672},
  [22] = {["x"] = 11243, ["y"] = 1223, ["z"] = -352, ["order"] = 5, ["angle"] = -399572992},
  [23] = {["x"] = 11953, ["y"] = 608, ["z"] = -352, ["order"] = 5, ["angle"] = 138477568},
  [24] = {["x"] = 12990, ["y"] = 529, ["z"] = -352, ["order"] = 5, ["angle"] = 503382016},
  [25] = {["x"] = 13976, ["y"] = 1078, ["z"] = -352, ["order"] = 5, ["angle"] = 1068040192},
  [26] = {["x"] = 14137, ["y"] = 2145, ["z"] = -352, ["order"] = 6, ["angle"] = 1458372608},
  [27] = {["x"] = 13416, ["y"] = 3053, ["z"] = -352, ["order"] = 6, ["angle"] = 1479081984},
  [28] = {["x"] = 12680, ["y"] = 4003, ["z"] = -352, ["order"] = 6, ["angle"] = 836304896},
  [29] = {["x"] = 13074, ["y"] = 4635, ["z"] = -352, ["order"] = 6, ["angle"] = -151060480},
  [30] = {["x"] = 13982, ["y"] = 4441, ["z"] = -352, ["order"] = 6, ["angle"] = -281083904},
  [31] = {["x"] = 14898, ["y"] = 4414, ["z"] = -352, ["order"] = 7, ["angle"] = -327090176},
  [32] = {["x"] = 15922, ["y"] = 4096, ["z"] = -352, ["order"] = 7, ["angle"] = -779288576},
  [33] = {["x"] = 16226, ["y"] = 3204, ["z"] = -352, ["order"] = 7, ["angle"] = -980090880},
  [34] = {["x"] = 16239, ["y"] = 2217, ["z"] = -352, ["order"] = 7, ["angle"] = -1003159552},
  [35] = {["x"] = 16148, ["y"] = 961, ["z"] = -352, ["order"] = 7, ["angle"] = -867631104},
  [36] = {["x"] = 16353, ["y"] = -90, ["z"] = -352, ["order"] = 8, ["angle"] = -330760192},
  [37] = {["x"] = 17122, ["y"] = -574, ["z"] = -352, ["order"] = 8, ["angle"] = 42795008},
  [38] = {["x"] = 17890, ["y"] = -676, ["z"] = -305, ["order"] = 8, ["angle"] = 117506048},
  [39] = {["x"] = 18756, ["y"] = -700, ["z"] = -205, ["order"] = 8, ["angle"] = 82116608},
  [40] = {["x"] = 19640, ["y"] = -653, ["z"] = -120, ["order"] = 8, ["angle"] = 534315008},
  [41] = {["x"] = 20307, ["y"] = -283, ["z"] = -120, ["order"] = 8, ["angle"] = 918487040},
  [42] = {["x"] = 20624, ["y"] = 313, ["z"] = -120, ["order"] = 9, ["angle"] = 1368719360},
  [43] = {["x"] = 20557, ["y"] = 1025, ["z"] = -120, ["order"] = 9, ["angle"] = 1818820608},
  [44] = {["x"] = 20011, ["y"] = 1472, ["z"] = -120, ["order"] = 9, ["angle"] = -2141782016},
  [45] = {["x"] = 19409, ["y"] = 1538, ["z"] = -120, ["order"] = 9, ["angle"] = -2075721728},
  [46] = {["x"] = 18668, ["y"] = 1529, ["z"] = -120, ["order"] = 9, ["angle"] = -2122776576},
  [47] = {["x"] = 17803, ["y"] = 1586, ["z"] = -120, ["order"] = 9, ["angle"] = -2104033280},
  [48] = {["x"] = 16729, ["y"] = 1656, ["z"] = -120, ["order"] = 9, ["angle"] = -1961426944},
  [49] = {["x"] = 15811, ["y"] = 1405, ["z"] = -120, ["order"] = 9, ["angle"] = -1630863360},
  [50] = {["x"] = 15043, ["y"] = 786, ["z"] = -120, ["order"] = 9, ["angle"] = -1275002880},
  [51] = {["x"] = 14762, ["y"] = -401, ["z"] = -120, ["order"] = 9, ["angle"] = -1058471936},
  [52] = {["x"] = 14774, ["y"] = -1553, ["z"] = -216, ["order"] = 10, ["angle"] = -1105657856},
  [53] = {["x"] = 14769, ["y"] = -2719, ["z"] = -324, ["order"] = 10, ["angle"] = -1075380224},
  [54] = {["x"] = 14799, ["y"] = -3543, ["z"] = -422, ["order"] = 10, ["angle"] = -1246822400},
  [55] = {["x"] = 14640, ["y"] = -4667, ["z"] = -464, ["order"] = 11, ["angle"] = -1566375936},
  [56] = {["x"] = 14185, ["y"] = -5214, ["z"] = -464, ["order"] = 11, ["angle"] = -1605435392},
  [57] = {["x"] = 13466, ["y"] = -6318, ["z"] = -768, ["order"] = 11, ["angle"] = 1873477632},
  [58] = {["x"] = 12156, ["y"] = -6207, ["z"] = -768, ["order"] = 11, ["angle"] = -2098397184},
  [59] = {["x"] = 10921, ["y"] = -6155, ["z"] = -768, ["order"] = 11, ["angle"] = -2018705408},
  [60] = {["x"] = 9753, ["y"] = -6361, ["z"] = -768, ["order"] = 11, ["angle"] = -1813970944},
  [61] = {["x"] = 8604, ["y"] = -7026, ["z"] = -768, ["order"] = 12, ["angle"] = -1948581888},
  [62] = {["x"] = 7415, ["y"] = -7472, ["z"] = -768, ["order"] = 12, ["angle"] = 2097610752},
  [63] = {["x"] = 5967, ["y"] = -7495, ["z"] = -768, ["order"] = 12, ["angle"] = 2145714176},
  [64] = {["x"] = 4833, ["y"] = -7490, ["z"] = -768, ["order"] = 12, ["angle"] = 2145714176},
  [65] = {["x"] = 3056, ["y"] = -7533, ["z"] = -1208, ["order"] = 12, ["angle"] = 2083454976},
  [66] = {["x"] = 2256, ["y"] = -7524, ["z"] = -1208, ["order"] = 12, ["angle"] = 2130771968},
  [67] = {["x"] = 1222, ["y"] = -7436, ["z"] = -1208, ["order"] = 12, ["angle"] = 2135883776},
  [68] = {["x"] = 5, ["y"] = -7402, ["z"] = -1208, ["order"] = 12, ["angle"] = -2105606144},
  [69] = {["x"] = -1395, ["y"] = -7524, ["z"] = -1208, ["order"] = 12, ["angle"] = -2129854464},
  [70] = {["x"] = -2328, ["y"] = -7645, ["z"] = -868, ["order"] = 12, ["angle"] = 2002845696},
  [71] = {["x"] = -3546, ["y"] = -7746, ["z"] = -912, ["order"] = 13, ["angle"] = 1577123840},
  [72] = {["x"] = -4268, ["y"] = -7206, ["z"] = -912, ["order"] = 13, ["angle"] = 1314586624},
  [73] = {["x"] = -4529, ["y"] = -6362, ["z"] = -912, ["order"] = 13, ["angle"] = 1473708032},
  [74] = {["x"] = -4838, ["y"] = -5315, ["z"] = -912, ["order"] = 13, ["angle"] = 1766391808},
  [75] = {["x"] = -5658, ["y"] = -4726, ["z"] = -903, ["order"] = 13, ["angle"] = -2084765696},
  [76] = {["x"] = -6674, ["y"] = -4635, ["z"] = -792, ["order"] = 13, ["angle"] = -1931149312},
  [77] = {["x"] = -7779, ["y"] = -4635, ["z"] = -662, ["order"] = 13, ["angle"] = -1391263744},
  [78] = {["x"] = -8391, ["y"] = -5285, ["z"] = -648, ["order"] = 13, ["angle"] = -1097793536},
  [79] = {["x"] = -8627, ["y"] = -6118, ["z"] = -648, ["order"] = 13, ["angle"] = -1156120576},
  [80] = {["x"] = -8856, ["y"] = -7145, ["z"] = -648, ["order"] = 14, ["angle"] = -982056960},
  [81] = {["x"] = -9018, ["y"] = -8490, ["z"] = -648, ["order"] = 14, ["angle"] = -671809536},
  [82] = {["x"] = -8742, ["y"] = -9540, ["z"] = -648, ["order"] = 14, ["angle"] = 36241408},
  [83] = {["x"] = -7927, ["y"] = -9860, ["z"] = -648, ["order"] = 14, ["angle"] = 285933568},
  [84] = {["x"] = -6951, ["y"] = -9521, ["z"] = -648, ["order"] = 14, ["angle"] = 64290816},
  [85] = {["x"] = -5804, ["y"] = -9304, ["z"] = -648, ["order"] = 14, ["angle"] = -7012352},
  [86] = {["x"] = -4659, ["y"] = -9175, ["z"] = -648, ["order"] = 14, ["angle"] = 152371200},
  [87] = {["x"] = -3622, ["y"] = -9078, ["z"] = -605, ["order"] = 14, ["angle"] = 210960384},
  [88] = {["x"] = -2337, ["y"] = -8960, ["z"] = -512, ["order"] = 14, ["angle"] = 288161792},
  [89] = {["x"] = -1235, ["y"] = -8680, ["z"] = -504, ["order"] = 15, ["angle"] = 793575424},
  [90] = {["x"] = -648, ["y"] = -7831, ["z"] = -504, ["order"] = 15, ["angle"] = 1160445952},
  [91] = {["x"] = -565, ["y"] = -6850, ["z"] = -504, ["order"] = 15, ["angle"] = 1374748672},
  [92] = {["x"] = -907, ["y"] = -5653, ["z"] = -504, ["order"] = 15, ["angle"] = 1851981824},
  [93] = {["x"] = -1632, ["y"] = -5287, ["z"] = -504, ["order"] = 15, ["angle"] = 1984102400},
  [94] = {["x"] = -2678, ["y"] = -5165, ["z"] = -504, ["order"] = 15, ["angle"] = 1674772480},
  [95] = {["x"] = -3427, ["y"] = -4762, ["z"] = -504, ["order"] = 15, ["angle"] = 1058603008},
  [96] = {["x"] = -3686, ["y"] = -4012, ["z"] = -504, ["order"] = 15, ["angle"] = 1077477376},
  [97] = {["x"] = -3742, ["y"] = -3168, ["z"] = -504, ["order"] = 17, ["angle"] = 999751680},
  [98] = {["x"] = -3812, ["y"] = -2004, ["z"] = -504, ["order"] = 17, ["angle"] = 1046413312},
  [99] = {["x"] = -3798, ["y"] = -962, ["z"] = -504, ["order"] = 17, ["angle"] = 616759296},
  [100] = {["x"] = -3314, ["y"] = -139, ["z"] = -504, ["order"] = 18, ["angle"] = 200081408},
  [101] = {["x"] = -2540, ["y"] = 192, ["z"] = -504, ["order"] = 18, ["angle"] = -529596416},
  [102] = {["x"] = -1866, ["y"] = -428, ["z"] = -504, ["order"] = 18, ["angle"] = -1003814912},
  [103] = {["x"] = -1533, ["y"] = -1806, ["z"] = -344, ["order"] = 19, ["angle"] = -1066729472},
  [104] = {["x"] = -1632, ["y"] = -2680, ["z"] = -199, ["order"] = 19, ["angle"] = -814153728},
  [105] = {["x"] = -1330, ["y"] = -3748, ["z"] = -33, ["order"] = 19, ["angle"] = -144244736},
  [106] = {["x"] = -563, ["y"] = -4316, ["z"] = 83, ["order"] = 19, ["angle"] = 546897920},
  [107] = {["x"] = 97, ["y"] = -4044, ["z"] = 27, ["order"] = 19, ["angle"] = 861470720},
  [108] = {["x"] = 406, ["y"] = -3306, ["z"] = -104, ["order"] = 20, ["angle"] = 1084162048},
  [109] = {["x"] = 425, ["y"] = -2426, ["z"] = -102, ["order"] = 20, ["angle"] = 1257832448},
  [110] = {["x"] = 155, ["y"] = -1306, ["z"] = -84, ["order"] = 20, ["angle"] = 1156644864},
  [111] = {["x"] = -21, ["y"] = -249, ["z"] = 0, ["order"] = 20, ["angle"] = 1045233664},
  [112] = {["x"] = 14470, ["y"] = -400, ["z"] = -120, ["order"] = 9, ["angle"] = -1128071168},
  [113] = {["x"] = 14389, ["y"] = -1379, ["z"] = -216, ["order"] = 10, ["angle"] = -1082064896},
  [114] = {["x"] = 14381, ["y"] = -2006, ["z"] = -239, ["order"] = 10, ["angle"] = -1082064896},
  [115] = {["x"] = 14373, ["y"] = -2613, ["z"] = -312, ["order"] = 10, ["angle"] = -1082064896},
  [116] = {["x"] = 14363, ["y"] = -3430, ["z"] = -409, ["order"] = 10, ["angle"] = -1082064896},
  [117] = {["x"] = 14343, ["y"] = -4445, ["z"] = -464, ["order"] = 11, ["angle"] = -1298071552},
  [118] = {["x"] = 13694, ["y"] = -5194, ["z"] = -464, ["order"] = 11, ["angle"] = -1838743552},
  [119] = {["x"] = 13160, ["y"] = -5793, ["z"] = -699, ["order"] = 11, ["angle"] = -1987117056},
  [120] = {["x"] = 13045, ["y"] = -6617, ["z"] = -696, ["order"] = 11, ["angle"] = -2113470464}
}
-- Respawn definitions for SUB-ZERO PEAK (MAP17)
krespawn.points[17] = {
  [0] = 103,
  [1] = {["x"] = -3234, ["y"] = 501, ["z"] = 2000, ["order"] = 0, ["angle"] = -196608},
  [2] = {["x"] = -2669, ["y"] = 450, ["z"] = 2000, ["order"] = 0, ["angle"] = -170328064},
  [3] = {["x"] = -2000, ["y"] = 244, ["z"] = 2000, ["order"] = 0, ["angle"] = -152371200},
  [4] = {["x"] = -1099, ["y"] = -74, ["z"] = 2000, ["order"] = 0, ["angle"] = -29032448},
  [5] = {["x"] = -296, ["y"] = -219, ["z"] = 2000, ["order"] = 0, ["angle"] = 201654272},
  [6] = {["x"] = 429, ["y"] = -47, ["z"] = 2000, ["order"] = 0, ["angle"] = 92602368},
  [7] = {["x"] = 1056, ["y"] = -22, ["z"] = 2000, ["order"] = 0, ["angle"] = -21692416},
  [8] = {["x"] = 1906, ["y"] = -168, ["z"] = 2000, ["order"] = 0, ["angle"] = 179765248},
  [9] = {["x"] = 2849, ["y"] = -135, ["z"] = 2000, ["order"] = 0, ["angle"] = 465764352},
  [10] = {["x"] = 3738, ["y"] = 203, ["z"] = 2000, ["order"] = 1, ["angle"] = 629866496},
  [11] = {["x"] = 4324, ["y"] = 734, ["z"] = 2000, ["order"] = 1, ["angle"] = 1108279296},
  [12] = {["x"] = 4419, ["y"] = 1480, ["z"] = 2000, ["order"] = 1, ["angle"] = 1583808512},
  [13] = {["x"] = 4148, ["y"] = 1902, ["z"] = 2000, ["order"] = 1, ["angle"] = 1760362496},
  [14] = {["x"] = 3562, ["y"] = 2227, ["z"] = 2000, ["order"] = 1, ["angle"] = 1589968896},
  [15] = {["x"] = 2881, ["y"] = 2746, ["z"] = 2000, ["order"] = 1, ["angle"] = 1093599232},
  [16] = {["x"] = 2647, ["y"] = 3353, ["z"] = 2000, ["order"] = 2, ["angle"] = 336003072},
  [17] = {["x"] = 3103, ["y"] = 3771, ["z"] = 2000, ["order"] = 2, ["angle"] = 17629184},
  [18] = {["x"] = 3996, ["y"] = 3821, ["z"] = 1997, ["order"] = 2, ["angle"] = -851968},
  [19] = {["x"] = 5117, ["y"] = 3871, ["z"] = 1822, ["order"] = 2, ["angle"] = -189333504},
  [20] = {["x"] = 6224, ["y"] = 3694, ["z"] = 1740, ["order"] = 3, ["angle"] = -484245504},
  [21] = {["x"] = 6936, ["y"] = 3134, ["z"] = 1740, ["order"] = 3, ["angle"] = -900661248},
  [22] = {["x"] = 7168, ["y"] = 2159, ["z"] = 1740, ["order"] = 3, ["angle"] = -1135149056},
  [23] = {["x"] = 7201, ["y"] = 1193, ["z"] = 1740, ["order"] = 3, ["angle"] = -1242365952},
  [24] = {["x"] = 7088, ["y"] = 99, ["z"] = 1740, ["order"] = 4, ["angle"] = -1458241536},
  [25] = {["x"] = 6714, ["y"] = -784, ["z"] = 1740, ["order"] = 4, ["angle"] = -1901133824},
  [26] = {["x"] = 5916, ["y"] = -1066, ["z"] = 1740, ["order"] = 4, ["angle"] = -1913192448},
  [27] = {["x"] = 4995, ["y"] = -1248, ["z"] = 1740, ["order"] = 4, ["angle"] = -1645019136},
  [28] = {["x"] = 4104, ["y"] = -1788, ["z"] = 1740, ["order"] = 4, ["angle"] = -1270022144},
  [29] = {["x"] = 3766, ["y"] = -2446, ["z"] = 1740, ["order"] = 4, ["angle"] = -888209408},
  [30] = {["x"] = 4021, ["y"] = -3317, ["z"] = 1740, ["order"] = 5, ["angle"] = -781123584},
  [31] = {["x"] = 4452, ["y"] = -4396, ["z"] = 1740, ["order"] = 5, ["angle"] = -1089798144},
  [32] = {["x"] = 4474, ["y"] = -5426, ["z"] = 1740, ["order"] = 5, ["angle"] = -1667432448},
  [33] = {["x"] = 4079, ["y"] = -6000, ["z"] = 1740, ["order"] = 5, ["angle"] = 1993015296},
  [34] = {["x"] = 3171, ["y"] = -5808, ["z"] = 1740, ["order"] = 6, ["angle"] = 1982660608},
  [35] = {["x"] = 2263, ["y"] = -5578, ["z"] = 1740, ["order"] = 6, ["angle"] = 2043346944},
  [36] = {["x"] = 1440, ["y"] = -5402, ["z"] = 1740, ["order"] = 6, ["angle"] = 2084241408},
  [37] = {["x"] = 418, ["y"] = -5311, ["z"] = 1740, ["order"] = 6, ["angle"] = -2093154304},
  [38] = {["x"] = -545, ["y"] = -5416, ["z"] = 1869, ["order"] = 6, ["angle"] = -2026176512},
  [39] = {["x"] = -2118, ["y"] = -5539, ["z"] = 2106, ["order"] = 7, ["angle"] = -1376583680},
  [40] = {["x"] = -2708, ["y"] = -6309, ["z"] = 2174, ["order"] = 7, ["angle"] = -601292800},
  [41] = {["x"] = -2223, ["y"] = -7047, ["z"] = 2251, ["order"] = 7, ["angle"] = -238092288},
  [42] = {["x"] = -1443, ["y"] = -7566, ["z"] = 2299, ["order"] = 7, ["angle"] = 193789952},
  [43] = {["x"] = -596, ["y"] = -7685, ["z"] = 2329, ["order"] = 7, ["angle"] = 937492480},
  [44] = {["x"] = -210, ["y"] = -6817, ["z"] = 2396, ["order"] = 8, ["angle"] = 1153499136},
  [45] = {["x"] = -200, ["y"] = -5857, ["z"] = 2500, ["order"] = 8, ["angle"] = 1231749120},
  [46] = {["x"] = -402, ["y"] = -4963, ["z"] = 2500, ["order"] = 8, ["angle"] = 1304625152},
  [47] = {["x"] = -874, ["y"] = -3975, ["z"] = 2500, ["order"] = 8, ["angle"] = 1212612608},
  [48] = {["x"] = -1164, ["y"] = -2730, ["z"] = 2520, ["order"] = 8, ["angle"] = 1093992448},
  [49] = {["x"] = -1275, ["y"] = -1304, ["z"] = 2520, ["order"] = 9, ["angle"] = 1093992448},
  [50] = {["x"] = -1315, ["y"] = 39, ["z"] = 2500, ["order"] = 9, ["angle"] = 1100152832},
  [51] = {["x"] = -1334, ["y"] = 1678, ["z"] = 2500, ["order"] = 9, ["angle"] = 1079574528},
  [52] = {["x"] = -1251, ["y"] = 3157, ["z"] = 2500, ["order"] = 10, ["angle"] = 1273036800},
  [53] = {["x"] = -1596, ["y"] = 4523, ["z"] = 2510, ["order"] = 10, ["angle"] = -2043346944},
  [54] = {["x"] = -2561, ["y"] = 4343, ["z"] = 2510, ["order"] = 10, ["angle"] = -1870987264},
  [55] = {["x"] = -3435, ["y"] = 3844, ["z"] = 2510, ["order"] = 10, ["angle"] = -2123431936},
  [56] = {["x"] = -4773, ["y"] = 3326, ["z"] = 2510, ["order"] = 10, ["angle"] = 1754071040},
  [57] = {["x"] = -5712, ["y"] = 3773, ["z"] = 2510, ["order"] = 10, ["angle"] = 1037893632},
  [58] = {["x"] = -5976, ["y"] = 4835, ["z"] = 2530, ["order"] = 10, ["angle"] = 1094909952},
  [59] = {["x"] = -6213, ["y"] = 5916, ["z"] = 2500, ["order"] = 11, ["angle"] = 965410816},
  [60] = {["x"] = -6152, ["y"] = 7033, ["z"] = 2552, ["order"] = 11, ["angle"] = 271777792},
  [61] = {["x"] = -5362, ["y"] = 7574, ["z"] = 2618, ["order"] = 12, ["angle"] = -509542400},
  [62] = {["x"] = -4599, ["y"] = 7094, ["z"] = 2675, ["order"] = 12, ["angle"] = -1403977728},
  [63] = {["x"] = -4753, ["y"] = 6353, ["z"] = 2755, ["order"] = 13, ["angle"] = -1915420672},
  [64] = {["x"] = -5914, ["y"] = 6109, ["z"] = 2800, ["order"] = 13, ["angle"] = -2081751040},
  [65] = {["x"] = -6549, ["y"] = 6322, ["z"] = 2800, ["order"] = 13, ["angle"] = -2087649280},
  [66] = {["x"] = -8083, ["y"] = 6490, ["z"] = 2655, ["order"] = 13, ["angle"] = -1992359936},
  [67] = {["x"] = -8986, ["y"] = 6467, ["z"] = 2424, ["order"] = 13, ["angle"] = -1653407744},
  [68] = {["x"] = -9733, ["y"] = 6114, ["z"] = 2176, ["order"] = 13, ["angle"] = -1435303936},
  [69] = {["x"] = -10110, ["y"] = 5538, ["z"] = 2000, ["order"] = 13, ["angle"] = -1018757120},
  [70] = {["x"] = -10091, ["y"] = 4639, ["z"] = 2020, ["order"] = 13, ["angle"] = -1018888192},
  [71] = {["x"] = -9886, ["y"] = 3695, ["z"] = 2000, ["order"] = 14, ["angle"] = -919535616},
  [72] = {["x"] = -9684, ["y"] = 2646, ["z"] = 2000, ["order"] = 14, ["angle"] = -1095303168},
  [73] = {["x"] = -9690, ["y"] = 1633, ["z"] = 2010, ["order"] = 14, ["angle"] = -1142882304},
  [74] = {["x"] = -10042, ["y"] = 267, ["z"] = 1990, ["order"] = 14, ["angle"] = -761069568},
  [75] = {["x"] = -9887, ["y"] = -800, ["z"] = 2010, ["order"] = 14, ["angle"] = -69795840},
  [76] = {["x"] = -9156, ["y"] = -1046, ["z"] = 2010, ["order"] = 14, ["angle"] = 450297856},
  [77] = {["x"] = -8603, ["y"] = -684, ["z"] = 2000, ["order"] = 15, ["angle"] = 918487040},
  [78] = {["x"] = -8453, ["y"] = 236, ["z"] = 2000, ["order"] = 15, ["angle"] = 771555328},
  [79] = {["x"] = -8244, ["y"] = 1144, ["z"] = 2000, ["order"] = 15, ["angle"] = 497221632},
  [80] = {["x"] = -7596, ["y"] = 1646, ["z"] = 2000, ["order"] = 16, ["angle"] = -589824},
  [81] = {["x"] = -6620, ["y"] = 1420, ["z"] = 2000, ["order"] = 16, ["angle"] = -292618240},
  [82] = {["x"] = -5850, ["y"] = 960, ["z"] = 2000, ["order"] = 16, ["angle"] = -208732160},
  [83] = {["x"] = -5010, ["y"] = 635, ["z"] = 2000, ["order"] = 16, ["angle"] = -26935296},
  [84] = {["x"] = -3956, ["y"] = 499, ["z"] = 2000, ["order"] = 16, ["angle"] = -26935296},
  [85] = {["x"] = -2128, ["y"] = 624, ["z"] = 2000, ["order"] = 0, ["angle"] = 47120384},
  [86] = {["x"] = -1280, ["y"] = 763, ["z"] = 2000, ["order"] = 0, ["angle"] = 118947840},
  [87] = {["x"] = -230, ["y"] = 920, ["z"] = 2000, ["order"] = 0, ["angle"] = -86441984},
  [88] = {["x"] = 518, ["y"] = 824, ["z"] = 2000, ["order"] = 0, ["angle"] = -308346880},
  [89] = {["x"] = 1272, ["y"] = 458, ["z"] = 2000, ["order"] = 0, ["angle"] = -230752256},
  [90] = {["x"] = 5140, ["y"] = -4030, ["z"] = 1740, ["order"] = 5, ["angle"] = -1493368832},
  [91] = {["x"] = 4759, ["y"] = -4915, ["z"] = 1740, ["order"] = 5, ["angle"] = -1559560192},
  [92] = {["x"] = -228, ["y"] = -4938, ["z"] = 1819, ["order"] = 6, ["angle"] = -2131820544},
  [93] = {["x"] = -1125, ["y"] = -4839, ["z"] = 1966, ["order"] = 6, ["angle"] = -1861812224},
  [94] = {["x"] = -2032, ["y"] = -5024, ["z"] = 2102, ["order"] = 6, ["angle"] = -1365573632},
  [95] = {["x"] = -2563, ["y"] = -5812, ["z"] = 2140, ["order"] = 7, ["angle"] = -687144960},
  [96] = {["x"] = -12, ["y"] = -5097, ["z"] = 2500, ["order"] = 8, ["angle"] = 1076690944},
  [97] = {["x"] = -19, ["y"] = -3795, ["z"] = 2510, ["order"] = 8, ["angle"] = 1414332416},
  [98] = {["x"] = -723, ["y"] = -2739, ["z"] = 2530, ["order"] = 8, ["angle"] = 1294401536},
  [99] = {["x"] = -10423, ["y"] = 3897, ["z"] = 2000, ["order"] = 14, ["angle"] = -1071054848},
  [100] = {["x"] = -10508, ["y"] = 2707, ["z"] = 1990, ["order"] = 14, ["angle"] = -926089216},
  [101] = {["x"] = -10514, ["y"] = 1590, ["z"] = 2010, ["order"] = 14, ["angle"] = -893452288},
  [102] = {["x"] = -10510, ["y"] = 391, ["z"] = 2010, ["order"] = 14, ["angle"] = -537985024},
  [103] = {["x"] = -10138, ["y"] = -495, ["z"] = 2010, ["order"] = 14, ["angle"] = -218955776}
}
-- Respawn definitions for SAPPHIRE COAST (MAP18)
krespawn.points[18] = {
  [0] = 62,
  [1] = {["x"] = 5417, ["y"] = -8922, ["z"] = 0, ["order"] = 0, ["angle"] = 30736384},
  [2] = {["x"] = 6238, ["y"] = -8952, ["z"] = 0, ["order"] = 0, ["angle"] = 144900096},
  [3] = {["x"] = 7078, ["y"] = -8875, ["z"] = 0, ["order"] = 0, ["angle"] = 419889152},
  [4] = {["x"] = 7824, ["y"] = -8531, ["z"] = 0, ["order"] = 0, ["angle"] = 767623168},
  [5] = {["x"] = 8356, ["y"] = -7917, ["z"] = 0, ["order"] = 1, ["angle"] = 1139736576},
  [6] = {["x"] = 8478, ["y"] = -7155, ["z"] = 0, ["order"] = 1, ["angle"] = 1437663232},
  [7] = {["x"] = 8212, ["y"] = -6445, ["z"] = 0, ["order"] = 1, ["angle"] = 1589444608},
  [8] = {["x"] = 7564, ["y"] = -5749, ["z"] = 0, ["order"] = 1, ["angle"] = 1631518720},
  [9] = {["x"] = 6744, ["y"] = -5032, ["z"] = 0, ["order"] = 1, ["angle"] = 1541603328},
  [10] = {["x"] = 5999, ["y"] = -4258, ["z"] = 0, ["order"] = 2, ["angle"] = 1525481472},
  [11] = {["x"] = 5277, ["y"] = -3492, ["z"] = 0, ["order"] = 2, ["angle"] = 1411842048},
  [12] = {["x"] = 4718, ["y"] = -2704, ["z"] = 63, ["order"] = 2, ["angle"] = 1103167488},
  [13] = {["x"] = 4520, ["y"] = -1940, ["z"] = 120, ["order"] = 2, ["angle"] = 622919680},
  [14] = {["x"] = 4866, ["y"] = -1008, ["z"] = 120, ["order"] = 3, ["angle"] = 18284544},
  [15] = {["x"] = 5764, ["y"] = -555, ["z"] = 120, ["order"] = 3, ["angle"] = -461438976},
  [16] = {["x"] = 6611, ["y"] = -854, ["z"] = 172, ["order"] = 3, ["angle"] = -845873152},
  [17] = {["x"] = 7038, ["y"] = -1530, ["z"] = 233, ["order"] = 3, ["angle"] = -1237778432},
  [18] = {["x"] = 6938, ["y"] = -2433, ["z"] = 256, ["order"] = 3, ["angle"] = -1575813120},
  [19] = {["x"] = 6380, ["y"] = -3260, ["z"] = 256, ["order"] = 4, ["angle"] = -1724973056},
  [20] = {["x"] = 5549, ["y"] = -3854, ["z"] = 256, ["order"] = 4, ["angle"] = -1802960896},
  [21] = {["x"] = 4116, ["y"] = -4552, ["z"] = 0, ["order"] = 4, ["angle"] = -1687748608},
  [22] = {["x"] = 3376, ["y"] = -4931, ["z"] = 0, ["order"] = 4, ["angle"] = -1416429568},
  [23] = {["x"] = 2904, ["y"] = -5695, ["z"] = 0, ["order"] = 4, ["angle"] = -1473314816},
  [24] = {["x"] = 2581, ["y"] = -6565, ["z"] = 0, ["order"] = 4, ["angle"] = -1702297600},
  [25] = {["x"] = 1920, ["y"] = -7254, ["z"] = 0, ["order"] = 4, ["angle"] = 1889599488},
  [26] = {["x"] = 788, ["y"] = -6951, ["z"] = 0, ["order"] = 5, ["angle"] = 1176174592},
  [27] = {["x"] = 536, ["y"] = -5980, ["z"] = 32, ["order"] = 5, ["angle"] = 670367744},
  [28] = {["x"] = 1162, ["y"] = -5241, ["z"] = 84, ["order"] = 5, ["angle"] = 734986240},
  [29] = {["x"] = 1639, ["y"] = -4320, ["z"] = 150, ["order"] = 5, ["angle"] = 1210384384},
  [30] = {["x"] = 1422, ["y"] = -3289, ["z"] = 225, ["order"] = 5, ["angle"] = 856227840},
  [31] = {["x"] = 1817, ["y"] = -2495, ["z"] = 256, ["order"] = 5, ["angle"] = 648609792},
  [32] = {["x"] = 2613, ["y"] = -1807, ["z"] = 256, ["order"] = 5, ["angle"] = 995819520},
  [33] = {["x"] = 2818, ["y"] = -749, ["z"] = 256, ["order"] = 5, ["angle"] = 1755906048},
  [34] = {["x"] = 2148, ["y"] = 9, ["z"] = 256, ["order"] = 5, ["angle"] = -2085027840},
  [35] = {["x"] = 1142, ["y"] = 73, ["z"] = 256, ["order"] = 6, ["angle"] = -1675034624},
  [36] = {["x"] = 178, ["y"] = -262, ["z"] = 256, ["order"] = 6, ["angle"] = -1837957120},
  [37] = {["x"] = -778, ["y"] = -637, ["z"] = 256, ["order"] = 6, ["angle"] = -1616052224},
  [38] = {["x"] = -1758, ["y"] = -1232, ["z"] = 256, ["order"] = 6, ["angle"] = -1425211392},
  [39] = {["x"] = -2421, ["y"] = -2129, ["z"] = 264, ["order"] = 7, ["angle"] = -1124532224},
  [40] = {["x"] = -2551, ["y"] = -2999, ["z"] = 408, ["order"] = 7, ["angle"] = -1007616000},
  [41] = {["x"] = -2439, ["y"] = -3804, ["z"] = 512, ["order"] = 7, ["angle"] = -979566592},
  [42] = {["x"] = -2244, ["y"] = -4546, ["z"] = 512, ["order"] = 8, ["angle"] = -856752128},
  [43] = {["x"] = -1882, ["y"] = -5705, ["z"] = 0, ["order"] = 8, ["angle"] = -906297344},
  [44] = {["x"] = -1634, ["y"] = -6630, ["z"] = 0, ["order"] = 8, ["angle"] = -887160832},
  [45] = {["x"] = -1302, ["y"] = -7797, ["z"] = 0, ["order"] = 8, ["angle"] = -919797760},
  [46] = {["x"] = -1059, ["y"] = -8660, ["z"] = 0, ["order"] = 8, ["angle"] = -841285632},
  [47] = {["x"] = -803, ["y"] = -9508, ["z"] = 0, ["order"] = 8, ["angle"] = -1019805696},
  [48] = {["x"] = -709, ["y"] = -10584, ["z"] = 0, ["order"] = 9, ["angle"] = -1082982400},
  [49] = {["x"] = -679, ["y"] = -11838, ["z"] = 0, ["order"] = 9, ["angle"] = -1036582912},
  [50] = {["x"] = -709, ["y"] = -12807, ["z"] = 0, ["order"] = 9, ["angle"] = -1125842944},
  [51] = {["x"] = -719, ["y"] = -13905, ["z"] = 0, ["order"] = 9, ["angle"] = -725549056},
  [52] = {["x"] = -227, ["y"] = -14798, ["z"] = 0, ["order"] = 9, ["angle"] = -74383360},
  [53] = {["x"] = 587, ["y"] = -15071, ["z"] = 0, ["order"] = 10, ["angle"] = 97583104},
  [54] = {["x"] = 1536, ["y"] = -15118, ["z"] = 0, ["order"] = 10, ["angle"] = 692387840},
  [55] = {["x"] = 2056, ["y"] = -14421, ["z"] = 0, ["order"] = 10, ["angle"] = 1069481984},
  [56] = {["x"] = 2122, ["y"] = -13138, ["z"] = 0, ["order"] = 11, ["angle"] = 1067384832},
  [57] = {["x"] = 2028, ["y"] = -12278, ["z"] = 0, ["order"] = 11, ["angle"] = 882704384},
  [58] = {["x"] = 2098, ["y"] = -11203, ["z"] = 0, ["order"] = 11, ["angle"] = 646643712},
  [59] = {["x"] = 2499, ["y"] = -10336, ["z"] = 0, ["order"] = 11, ["angle"] = 442695680},
  [60] = {["x"] = 3051, ["y"] = -9746, ["z"] = 0, ["order"] = 11, ["angle"] = 223412224},
  [61] = {["x"] = 3731, ["y"] = -9342, ["z"] = 0, ["order"] = 11, ["angle"] = 116588544},
  [62] = {["x"] = 4641, ["y"] = -8983, ["z"] = 0, ["order"] = 11, ["angle"] = 28639232}
}
-- Respawn definitions for SAND VALLEY (MAP19)
krespawn.points[19] = {
  [0] = 87,
  [1] = {["x"] = 223, ["y"] = 1004, ["z"] = 0, ["order"] = 0, ["angle"] = 1067909120},
  [2] = {["x"] = 231, ["y"] = 1642, ["z"] = 0, ["order"] = 0, ["angle"] = 1067909120},
  [3] = {["x"] = 239, ["y"] = 2553, ["z"] = 0, ["order"] = 0, ["angle"] = 1067909120},
  [4] = {["x"] = 261, ["y"] = 3650, ["z"] = 0, ["order"] = 0, ["angle"] = 1109196800},
  [5] = {["x"] = 305, ["y"] = 4750, ["z"] = 101, ["order"] = 0, ["angle"] = 1131216896},
  [6] = {["x"] = 309, ["y"] = 5831, ["z"] = 21, ["order"] = 0, ["angle"] = 1129644032},
  [7] = {["x"] = 302, ["y"] = 6727, ["z"] = 76, ["order"] = 0, ["angle"] = 1130561536},
  [8] = {["x"] = 303, ["y"] = 7516, ["z"] = 14, ["order"] = 0, ["angle"] = 1122566144},
  [9] = {["x"] = 284, ["y"] = 8268, ["z"] = 98, ["order"] = 0, ["angle"] = 1257832448},
  [10] = {["x"] = 54, ["y"] = 9139, ["z"] = 13, ["order"] = 0, ["angle"] = 1307115520},
  [11] = {["x"] = -301, ["y"] = 9916, ["z"] = 174, ["order"] = 0, ["angle"] = 1327562752},
  [12] = {["x"] = -739, ["y"] = 10652, ["z"] = 325, ["order"] = 0, ["angle"] = 651231232},
  [13] = {["x"] = -480, ["y"] = 11258, ["z"] = 426, ["order"] = 1, ["angle"] = 42926080},
  [14] = {["x"] = 328, ["y"] = 11301, ["z"] = 350, ["order"] = 1, ["angle"] = 2424832},
  [15] = {["x"] = 1177, ["y"] = 11278, ["z"] = 210, ["order"] = 1, ["angle"] = 2424832},
  [16] = {["x"] = 1843, ["y"] = 11240, ["z"] = 99, ["order"] = 1, ["angle"] = 37552128},
  [17] = {["x"] = 2523, ["y"] = 11196, ["z"] = 20, ["order"] = 1, ["angle"] = 15400960},
  [18] = {["x"] = 3414, ["y"] = 11149, ["z"] = 7, ["order"] = 1, ["angle"] = 54722560},
  [19] = {["x"] = 4101, ["y"] = 11128, ["z"] = 32, ["order"] = 1, ["angle"] = 111083520},
  [20] = {["x"] = 4778, ["y"] = 11164, ["z"] = 32, ["order"] = 1, ["angle"] = 362741760},
  [21] = {["x"] = 5286, ["y"] = 11404, ["z"] = 32, ["order"] = 1, ["angle"] = 716767232},
  [22] = {["x"] = 5505, ["y"] = 11879, ["z"] = 32, ["order"] = 1, ["angle"] = 937754624},
  [23] = {["x"] = 5517, ["y"] = 12579, ["z"] = 32, ["order"] = 1, ["angle"] = 947060736},
  [24] = {["x"] = 5513, ["y"] = 13249, ["z"] = 32, ["order"] = 1, ["angle"] = 432078848},
  [25] = {["x"] = 5882, ["y"] = 13673, ["z"] = 32, ["order"] = 1, ["angle"] = 203489280},
  [26] = {["x"] = 6425, ["y"] = 13947, ["z"] = 32, ["order"] = 3, ["angle"] = -267059200},
  [27] = {["x"] = 7016, ["y"] = 13888, ["z"] = 32, ["order"] = 3, ["angle"] = -795148288},
  [28] = {["x"] = 7373, ["y"] = 13387, ["z"] = 32, ["order"] = 3, ["angle"] = -1021771776},
  [29] = {["x"] = 7587, ["y"] = 12415, ["z"] = 32, ["order"] = 3, ["angle"] = -1066991616},
  [30] = {["x"] = 7517, ["y"] = 11545, ["z"] = 32, ["order"] = 3, ["angle"] = -1029242880},
  [31] = {["x"] = 7473, ["y"] = 10570, ["z"] = 32, ["order"] = 3, ["angle"] = -542703616},
  [32] = {["x"] = 7960, ["y"] = 9926, ["z"] = 32, ["order"] = 4, ["angle"] = 150929408},
  [33] = {["x"] = 8723, ["y"] = 9898, ["z"] = 32, ["order"] = 4, ["angle"] = 709033984},
  [34] = {["x"] = 9331, ["y"] = 10559, ["z"] = 32, ["order"] = 4, ["angle"] = 1020067840},
  [35] = {["x"] = 9512, ["y"] = 11488, ["z"] = 32, ["order"] = 4, ["angle"] = 1379729408},
  [36] = {["x"] = 9154, ["y"] = 12387, ["z"] = 32, ["order"] = 4, ["angle"] = 1284964352},
  [37] = {["x"] = 8896, ["y"] = 13204, ["z"] = 32, ["order"] = 4, ["angle"] = 865140736},
  [38] = {["x"] = 9279, ["y"] = 13897, ["z"] = 32, ["order"] = 4, ["angle"] = 946929664},
  [39] = {["x"] = 9435, ["y"] = 14558, ["z"] = 32, ["order"] = 4, ["angle"] = 1317208064},
  [40] = {["x"] = 9318, ["y"] = 15415, ["z"] = 32, ["order"] = 4, ["angle"] = 1396899840},
  [41] = {["x"] = 9094, ["y"] = 16077, ["z"] = 32, ["order"] = 4, ["angle"] = 1733885952},
  [42] = {["x"] = 8624, ["y"] = 16568, ["z"] = 32, ["order"] = 5, ["angle"] = -1953038336},
  [43] = {["x"] = 7945, ["y"] = 16602, ["z"] = 32, ["order"] = 5, ["angle"] = -1463746560},
  [44] = {["x"] = 7436, ["y"] = 16062, ["z"] = 32, ["order"] = 5, ["angle"] = -1096613888},
  [45] = {["x"] = 7350, ["y"] = 15109, ["z"] = 126, ["order"] = 5, ["angle"] = -1053753344},
  [46] = {["x"] = 7362, ["y"] = 14271, ["z"] = 280, ["order"] = 5, ["angle"] = -1053753344},
  [47] = {["x"] = 7363, ["y"] = 13325, ["z"] = 280, ["order"] = 5, ["angle"] = -1103691776},
  [48] = {["x"] = 7351, ["y"] = 12364, ["z"] = 280, ["order"] = 5, ["angle"] = -1046020096},
  [49] = {["x"] = 7365, ["y"] = 11134, ["z"] = 280, ["order"] = 5, ["angle"] = -1059258368},
  [50] = {["x"] = 7367, ["y"] = 9621, ["z"] = 280, ["order"] = 5, ["angle"] = -1082327040},
  [51] = {["x"] = 7336, ["y"] = 5240, ["z"] = -2032, ["order"] = 6, ["angle"] = -927137792},
  [52] = {["x"] = 7346, ["y"] = 4565, ["z"] = -2032, ["order"] = 6, ["angle"] = -558301184},
  [53] = {["x"] = 8067, ["y"] = 4111, ["z"] = -2032, ["order"] = 6, ["angle"] = -113967104},
  [54] = {["x"] = 9262, ["y"] = 4015, ["z"] = -1712, ["order"] = 6, ["angle"] = -290127872},
  [55] = {["x"] = 10149, ["y"] = 3432, ["z"] = -1466, ["order"] = 6, ["angle"] = -1092943872},
  [56] = {["x"] = 9796, ["y"] = 2263, ["z"] = -1564, ["order"] = 6, ["angle"] = 2090795008},
  [57] = {["x"] = 5912, ["y"] = 2302, ["z"] = -2000, ["order"] = 8, ["angle"] = -2077294592},
  [58] = {["x"] = 4955, ["y"] = 2272, ["z"] = -1836, ["order"] = 8, ["angle"] = -1613561856},
  [59] = {["x"] = 4049, ["y"] = 1021, ["z"] = -1436, ["order"] = 8, ["angle"] = -679673856},
  [60] = {["x"] = 4687, ["y"] = 231, ["z"] = -1718, ["order"] = 8, ["angle"] = -258670592},
  [61] = {["x"] = 5983, ["y"] = 243, ["z"] = -2000, ["order"] = 9, ["angle"] = -548995072},
  [62] = {["x"] = 6521, ["y"] = -436, ["z"] = -2000, ["order"] = 9, ["angle"] = -956760064},
  [63] = {["x"] = 6636, ["y"] = -1395, ["z"] = -2000, ["order"] = 9, ["angle"] = -898957312},
  [64] = {["x"] = 6782, ["y"] = -2376, ["z"] = -1977, ["order"] = 9, ["angle"] = -1050607616},
  [65] = {["x"] = 6913, ["y"] = -4046, ["z"] = -1344, ["order"] = 9, ["angle"] = -1050607616},
  [66] = {["x"] = 7015, ["y"] = -5199, ["z"] = -853, ["order"] = 9, ["angle"] = -1050607616},
  [67] = {["x"] = 7129, ["y"] = -6538, ["z"] = -300, ["order"] = 9, ["angle"] = -1317994496},
  [68] = {["x"] = 7051, ["y"] = -8084, ["z"] = -31, ["order"] = 10, ["angle"] = -1540947968},
  [69] = {["x"] = 6469, ["y"] = -8673, ["z"] = 71, ["order"] = 10, ["angle"] = -1850146816},
  [70] = {["x"] = 4987, ["y"] = -8583, ["z"] = 48, ["order"] = 10, ["angle"] = 1728774144},
  [71] = {["x"] = 4077, ["y"] = -7680, ["z"] = 172, ["order"] = 11, ["angle"] = -2040463360},
  [72] = {["x"] = 3165, ["y"] = -7748, ["z"] = 156, ["order"] = 11, ["angle"] = -1639251968},
  [73] = {["x"] = 2313, ["y"] = -8718, ["z"] = 84, ["order"] = 11, ["angle"] = -1959329792},
  [74] = {["x"] = 917, ["y"] = -8765, ["z"] = 96, ["order"] = 11, ["angle"] = 1540554752},
  [75] = {["x"] = 597, ["y"] = -7294, ["z"] = 53, ["order"] = 11, ["angle"] = 1220476928},
  [76] = {["x"] = 270, ["y"] = -5546, ["z"] = 4, ["order"] = 11, ["angle"] = 1131347968},
  [77] = {["x"] = -55, ["y"] = -3974, ["z"] = 15, ["order"] = 11, ["angle"] = 1026752512},
  [78] = {["x"] = 240, ["y"] = -1962, ["z"] = 0, ["order"] = 11, ["angle"] = 1085210624},
  [79] = {["x"] = 230, ["y"] = -1206, ["z"] = 0, ["order"] = 11, ["angle"] = 1085210624},
  [80] = {["x"] = 209, ["y"] = 105, ["z"] = 0, ["order"] = 11, ["angle"] = 1085210624},
  [81] = {["x"] = 5592, ["y"] = 11232, ["z"] = 32, ["order"] = 2, ["angle"] = 1080754176},
  [82] = {["x"] = 5650, ["y"] = 11905, ["z"] = 32, ["order"] = 2, ["angle"] = 1139867648},
  [83] = {["x"] = 5572, ["y"] = 12567, ["z"] = 32, ["order"] = 2, ["angle"] = 1011679232},
  [84] = {["x"] = 5556, ["y"] = 13208, ["z"] = 32, ["order"] = 2, ["angle"] = 498663424},
  [85] = {["x"] = 5861, ["y"] = 13573, ["z"] = 32, ["order"] = 2, ["angle"] = 304414720},
  [86] = {["x"] = 6603, ["y"] = 2227, ["z"] = -2520, ["order"] = 7, ["angle"] = -1002504192},
  [87] = {["x"] = 6620, ["y"] = 353, ["z"] = -2000, ["order"] = 7, ["angle"] = -1089404928}
}
-- Respawn definitions for MEGABLOCK CASTLE (MAP20)
krespawn.points[20] = {
  [0] = 130,
  [1] = {["x"] = 30, ["y"] = -7269, ["z"] = 1005, ["order"] = 0, ["angle"] = 1070661632},
  [2] = {["x"] = 26, ["y"] = -6517, ["z"] = 1005, ["order"] = 0, ["angle"] = 1070661632},
  [3] = {["x"] = 30, ["y"] = -5489, ["z"] = 1005, ["order"] = 0, ["angle"] = 1070661632},
  [4] = {["x"] = 91, ["y"] = -4619, ["z"] = 1005, ["order"] = 0, ["angle"] = 1321795584},
  [5] = {["x"] = -110, ["y"] = -3760, ["z"] = 1005, ["order"] = 1, ["angle"] = 1979121664},
  [6] = {["x"] = -746, ["y"] = -3501, ["z"] = 1005, ["order"] = 1, ["angle"] = -2075328512},
  [7] = {["x"] = -1675, ["y"] = -3590, ["z"] = 1005, ["order"] = 1, ["angle"] = 2108620800},
  [8] = {["x"] = -2751, ["y"] = -3667, ["z"] = 1005, ["order"] = 2, ["angle"] = 2070740992},
  [9] = {["x"] = -3695, ["y"] = -3687, ["z"] = 1005, ["order"] = 2, ["angle"] = 1791033344},
  [10] = {["x"] = -4548, ["y"] = -3431, ["z"] = 1005, ["order"] = 2, ["angle"] = 1235419136},
  [11] = {["x"] = -4837, ["y"] = -2753, ["z"] = 1005, ["order"] = 3, ["angle"] = 761069568},
  [12] = {["x"] = -4406, ["y"] = -1911, ["z"] = 1005, ["order"] = 3, ["angle"] = 875888640},
  [13] = {["x"] = -4066, ["y"] = -968, ["z"] = 1005, ["order"] = 3, ["angle"] = 1008664576},
  [14] = {["x"] = -3917, ["y"] = 46, ["z"] = 1005, ["order"] = 3, ["angle"] = 1119420416},
  [15] = {["x"] = -3840, ["y"] = 1289, ["z"] = 1075, ["order"] = 3, ["angle"] = 1299775488},
  [16] = {["x"] = -3914, ["y"] = 2179, ["z"] = 1217, ["order"] = 3, ["angle"] = 1704394752},
  [17] = {["x"] = -4306, ["y"] = 2658, ["z"] = 1256, ["order"] = 4, ["angle"] = 2054225920},
  [18] = {["x"] = -5146, ["y"] = 2828, ["z"] = 1256, ["order"] = 4, ["angle"] = 2146369536},
  [19] = {["x"] = -6073, ["y"] = 2954, ["z"] = 1256, ["order"] = 5, ["angle"] = 2141388800},
  [20] = {["x"] = -7068, ["y"] = 3021, ["z"] = 1256, ["order"] = 5, ["angle"] = -2069037056},
  [21] = {["x"] = -8123, ["y"] = 2947, ["z"] = 1256, ["order"] = 5, ["angle"] = -1920925696},
  [22] = {["x"] = -8840, ["y"] = 2747, ["z"] = 1256, ["order"] = 5, ["angle"] = -1448411136},
  [23] = {["x"] = -9408, ["y"] = 2082, ["z"] = 1256, ["order"] = 6, ["angle"] = -1100546048},
  [24] = {["x"] = -9656, ["y"] = 1144, ["z"] = 1256, ["order"] = 6, ["angle"] = -1141440512},
  [25] = {["x"] = -9768, ["y"] = -50, ["z"] = 1256, ["order"] = 6, ["angle"] = -1037238272},
  [26] = {["x"] = -9804, ["y"] = -1247, ["z"] = 1378, ["order"] = 7, ["angle"] = -1044054016},
  [27] = {["x"] = -9676, ["y"] = -2628, ["z"] = 1606, ["order"] = 7, ["angle"] = -623443968},
  [28] = {["x"] = -9044, ["y"] = -3518, ["z"] = 1899, ["order"] = 7, ["angle"] = 603521024},
  [29] = {["x"] = -8472, ["y"] = -2800, ["z"] = 1663, ["order"] = 8, ["angle"] = 962920448},
  [30] = {["x"] = -8389, ["y"] = -1971, ["z"] = 1663, ["order"] = 8, ["angle"] = 1031471104},
  [31] = {["x"] = -8278, ["y"] = -874, ["z"] = 1911, ["order"] = 8, ["angle"] = 670892032},
  [32] = {["x"] = -7477, ["y"] = 29, ["z"] = 2194, ["order"] = 8, ["angle"] = -464453632},
  [33] = {["x"] = -6718, ["y"] = -519, ["z"] = 2022, ["order"] = 9, ["angle"] = -953483264},
  [34] = {["x"] = -6617, ["y"] = -1286, ["z"] = 1940, ["order"] = 9, ["angle"] = -918880256},
  [35] = {["x"] = -6599, ["y"] = -2354, ["z"] = 1940, ["order"] = 9, ["angle"] = -720437248},
  [36] = {["x"] = -5928, ["y"] = -3099, ["z"] = 1956, ["order"] = 10, ["angle"] = -45547520},
  [37] = {["x"] = -5302, ["y"] = -3456, ["z"] = 2000, ["order"] = 10, ["angle"] = 4915200},
  [38] = {["x"] = -4382, ["y"] = -3488, ["z"] = 2000, ["order"] = 10, ["angle"] = 38076416},
  [39] = {["x"] = -2923, ["y"] = -3443, ["z"] = 2000, ["order"] = 10, ["angle"] = 252510208},
  [40] = {["x"] = -1954, ["y"] = -2979, ["z"] = 2000, ["order"] = 10, ["angle"] = 153419776},
  [41] = {["x"] = -1170, ["y"] = -2660, ["z"] = 2000, ["order"] = 10, ["angle"] = 6619136},
  [42] = {["x"] = -301, ["y"] = -2507, ["z"] = 2000, ["order"] = 10, ["angle"] = -174653440},
  [43] = {["x"] = 592, ["y"] = -2976, ["z"] = 2000, ["order"] = 10, ["angle"] = -258670592},
  [44] = {["x"] = 1444, ["y"] = -3377, ["z"] = 2000, ["order"] = 10, ["angle"] = 161021952},
  [45] = {["x"] = 2472, ["y"] = -3454, ["z"] = 2000, ["order"] = 11, ["angle"] = 787152896},
  [46] = {["x"] = 2862, ["y"] = -2403, ["z"] = 2000, ["order"] = 11, ["angle"] = 1084555264},
  [47] = {["x"] = 2953, ["y"] = -1117, ["z"] = 2000, ["order"] = 11, ["angle"] = 1090846720},
  [48] = {["x"] = 3010, ["y"] = 159, ["z"] = 2000, ["order"] = 11, ["angle"] = 1095958528},
  [49] = {["x"] = 3052, ["y"] = 1515, ["z"] = 2000, ["order"] = 11, ["angle"] = 1072234496},
  [50] = {["x"] = 3022, ["y"] = 2472, ["z"] = 2000, ["order"] = 12, ["angle"] = 1084948480},
  [51] = {["x"] = 2811, ["y"] = 4228, ["z"] = 1512, ["order"] = 12, ["angle"] = 1041301504},
  [52] = {["x"] = 2628, ["y"] = 5133, ["z"] = 1512, ["order"] = 12, ["angle"] = 1249050624},
  [53] = {["x"] = 2441, ["y"] = 6289, ["z"] = 1512, ["order"] = 12, ["angle"] = 1044185088},
  [54] = {["x"] = 2412, ["y"] = 7497, ["z"] = 1512, ["order"] = 12, ["angle"] = 1124663296},
  [55] = {["x"] = 2309, ["y"] = 8879, ["z"] = 1512, ["order"] = 13, ["angle"] = 685047808},
  [56] = {["x"] = 2781, ["y"] = 9623, ["z"] = 1512, ["order"] = 13, ["angle"] = -16973824},
  [57] = {["x"] = 3722, ["y"] = 9925, ["z"] = 1512, ["order"] = 13, ["angle"] = -519897088},
  [58] = {["x"] = 4392, ["y"] = 9384, ["z"] = 1512, ["order"] = 14, ["angle"] = -1033437184},
  [59] = {["x"] = 4425, ["y"] = 8168, ["z"] = 1400, ["order"] = 14, ["angle"] = -1083768832},
  [60] = {["x"] = 4436, ["y"] = 7013, ["z"] = 1000, ["order"] = 14, ["angle"] = -1082327040},
  [61] = {["x"] = 4529, ["y"] = 6088, ["z"] = 1000, ["order"] = 14, ["angle"] = -1454833664},
  [62] = {["x"] = 4070, ["y"] = 5508, ["z"] = 1000, ["order"] = 15, ["angle"] = -2039414784},
  [63] = {["x"] = 3165, ["y"] = 5255, ["z"] = 1000, ["order"] = 15, ["angle"] = 1967587328},
  [64] = {["x"] = 2153, ["y"] = 5330, ["z"] = 1000, ["order"] = 15, ["angle"] = 1643184128},
  [65] = {["x"] = 1657, ["y"] = 5713, ["z"] = 1000, ["order"] = 16, ["angle"] = 1149566976},
  [66] = {["x"] = 1579, ["y"] = 6548, ["z"] = 1000, ["order"] = 16, ["angle"] = 1084424192},
  [67] = {["x"] = 1651, ["y"] = 7498, ["z"] = 1000, ["order"] = 16, ["angle"] = 1261109248},
  [68] = {["x"] = 1394, ["y"] = 8403, ["z"] = 1000, ["order"] = 17, ["angle"] = 1875705856},
  [69] = {["x"] = 915, ["y"] = 8689, ["z"] = 1000, ["order"] = 17, ["angle"] = 2132869120},
  [70] = {["x"] = -72, ["y"] = 8627, ["z"] = 1116, ["order"] = 17, ["angle"] = 2066284544},
  [71] = {["x"] = -1134, ["y"] = 8769, ["z"] = 1200, ["order"] = 17, ["angle"] = 2127495168},
  [72] = {["x"] = -2033, ["y"] = 8832, ["z"] = 1200, ["order"] = 17, ["angle"] = 2106392576},
  [73] = {["x"] = -3258, ["y"] = 9070, ["z"] = 1200, ["order"] = 17, ["angle"] = -1832845312},
  [74] = {["x"] = -4171, ["y"] = 8737, ["z"] = 1200, ["order"] = 17, ["angle"] = -1379598336},
  [75] = {["x"] = -4709, ["y"] = 7908, ["z"] = 1200, ["order"] = 17, ["angle"] = -818741248},
  [76] = {["x"] = -4456, ["y"] = 6918, ["z"] = 1200, ["order"] = 18, ["angle"] = -459866112},
  [77] = {["x"] = -3994, ["y"] = 6254, ["z"] = 1200, ["order"] = 18, ["angle"] = -178061312},
  [78] = {["x"] = -3431, ["y"] = 6078, ["z"] = 1200, ["order"] = 18, ["angle"] = 82509824},
  [79] = {["x"] = -2677, ["y"] = 6237, ["z"] = 1200, ["order"] = 18, ["angle"] = 166395904},
  [80] = {["x"] = -1827, ["y"] = 6532, ["z"] = 1200, ["order"] = 18, ["angle"] = -110559232},
  [81] = {["x"] = -867, ["y"] = 6567, ["z"] = 1200, ["order"] = 19, ["angle"] = -651886592},
  [82] = {["x"] = -593, ["y"] = 5792, ["z"] = 1140, ["order"] = 19, ["angle"] = -1046544384},
  [83] = {["x"] = -709, ["y"] = 4850, ["z"] = 1000, ["order"] = 19, ["angle"] = -957808640},
  [84] = {["x"] = -674, ["y"] = 3963, ["z"] = 1000, ["order"] = 19, ["angle"] = -509280256},
  [85] = {["x"] = -248, ["y"] = 3366, ["z"] = 1000, ["order"] = 20, ["angle"] = -213975040},
  [86] = {["x"] = 507, ["y"] = 3142, ["z"] = 1030, ["order"] = 20, ["angle"] = 19857408},
  [87] = {["x"] = 1427, ["y"] = 3103, ["z"] = 1228, ["order"] = 20, ["angle"] = 22216704},
  [88] = {["x"] = 2562, ["y"] = 3129, ["z"] = 1205, ["order"] = 20, ["angle"] = -72286208},
  [89] = {["x"] = 3469, ["y"] = 3116, ["z"] = 1274, ["order"] = 20, ["angle"] = -54984704},
  [90] = {["x"] = 4388, ["y"] = 3212, ["z"] = 1305, ["order"] = 21, ["angle"] = -35323904},
  [91] = {["x"] = 5290, ["y"] = 3277, ["z"] = 1305, ["order"] = 21, ["angle"] = -142409728},
  [92] = {["x"] = 6313, ["y"] = 3156, ["z"] = 1305, ["order"] = 21, ["angle"] = -590282752},
  [93] = {["x"] = 6720, ["y"] = 2724, ["z"] = 1305, ["order"] = 22, ["angle"] = -707854336},
  [94] = {["x"] = 7013, ["y"] = 2079, ["z"] = 1305, ["order"] = 22, ["angle"] = -1194786816},
  [95] = {["x"] = 6958, ["y"] = 1329, ["z"] = 1305, ["order"] = 22, ["angle"] = -1492844544},
  [96] = {["x"] = 6315, ["y"] = 636, ["z"] = 1305, ["order"] = 22, ["angle"] = -1565720576},
  [97] = {["x"] = 5585, ["y"] = -441, ["z"] = 1305, ["order"] = 22, ["angle"] = -1361510400},
  [98] = {["x"] = 5217, ["y"] = -1644, ["z"] = 1305, ["order"] = 22, ["angle"] = -1224802304},
  [99] = {["x"] = 5184, ["y"] = -2297, ["z"] = 1305, ["order"] = 23, ["angle"] = -1338179584},
  [100] = {["x"] = 4958, ["y"] = -2957, ["z"] = 1305, ["order"] = 23, ["angle"] = -1282736128},
  [101] = {["x"] = 4610, ["y"] = -3782, ["z"] = 1305, ["order"] = 23, ["angle"] = -1251147776},
  [102] = {["x"] = 4319, ["y"] = -4838, ["z"] = 1305, ["order"] = 23, ["angle"] = -1115619328},
  [103] = {["x"] = 4306, ["y"] = -5819, ["z"] = 1330, ["order"] = 23, ["angle"] = -1105920000},
  [104] = {["x"] = 4340, ["y"] = -6979, ["z"] = 1420, ["order"] = 23, ["angle"] = -1338179584},
  [105] = {["x"] = 4095, ["y"] = -8008, ["z"] = 1500, ["order"] = 23, ["angle"] = -2064056320},
  [106] = {["x"] = 3460, ["y"] = -8212, ["z"] = 1500, ["order"] = 24, ["angle"] = 1355743232},
  [107] = {["x"] = 3018, ["y"] = -7559, ["z"] = 1500, ["order"] = 24, ["angle"] = 926875648},
  [108] = {["x"] = 3127, ["y"] = -6597, ["z"] = 1500, ["order"] = 24, ["angle"] = 1034878976},
  [109] = {["x"] = 3172, ["y"] = -5491, ["z"] = 1380, ["order"] = 24, ["angle"] = 1226506240},
  [110] = {["x"] = 3077, ["y"] = -4389, ["z"] = 1040, ["order"] = 25, ["angle"] = 1439236096},
  [111] = {["x"] = 2722, ["y"] = -3791, ["z"] = 1005, ["order"] = 25, ["angle"] = 2048720896},
  [112] = {["x"] = 2031, ["y"] = -3596, ["z"] = 1005, ["order"] = 25, ["angle"] = -2133655552},
  [113] = {["x"] = 1025, ["y"] = -3568, ["z"] = 1005, ["order"] = 25, ["angle"] = 2120548352},
  [114] = {["x"] = 363, ["y"] = -3542, ["z"] = 1005, ["order"] = 1, ["angle"] = 2120548352},
  [115] = {["x"] = 3490, ["y"] = -2663, ["z"] = 2000, ["order"] = 11, ["angle"] = 1139736576},
  [116] = {["x"] = 3402, ["y"] = -1583, ["z"] = 2228, ["order"] = 11, ["angle"] = 1072758784},
  [117] = {["x"] = 2593, ["y"] = -411, ["z"] = 2000, ["order"] = 11, ["angle"] = 1091502080},
  [118] = {["x"] = 2555, ["y"] = 332, ["z"] = 2178, ["order"] = 11, ["angle"] = 1043922944},
  [119] = {["x"] = 2928, ["y"] = 5075, ["z"] = 1512, ["order"] = 12, ["angle"] = 1044185088},
  [120] = {["x"] = 2988, ["y"] = 6110, ["z"] = 1433, ["order"] = 12, ["angle"] = 1044185088},
  [121] = {["x"] = 3012, ["y"] = 7215, ["z"] = 1012, ["order"] = 12, ["angle"] = 1006436352},
  [122] = {["x"] = 3002, ["y"] = 8168, ["z"] = 1000, ["order"] = 12, ["angle"] = 554500096},
  [123] = {["x"] = 3498, ["y"] = 8878, ["z"] = 1000, ["order"] = 13, ["angle"] = 89718784},
  [124] = {["x"] = 4464, ["y"] = 9101, ["z"] = 1000, ["order"] = 13, ["angle"] = -470089728},
  [125] = {["x"] = 5300, ["y"] = 8804, ["z"] = 1000, ["order"] = 14, ["angle"] = -944046080},
  [126] = {["x"] = 5608, ["y"] = 7980, ["z"] = 1000, ["order"] = 14, ["angle"] = -1104478208},
  [127] = {["x"] = 5682, ["y"] = 7109, ["z"] = 1000, ["order"] = 14, ["angle"] = -1263206400},
  [128] = {["x"] = 5550, ["y"] = 6110, ["z"] = 1000, ["order"] = 14, ["angle"] = -1630339072},
  [129] = {["x"] = 4852, ["y"] = 5412, ["z"] = 1000, ["order"] = 15, ["angle"] = -2109669376},
  [130] = {["x"] = 4167, ["y"] = 5239, ["z"] = 1000, ["order"] = 15, ["angle"] = -2131558400}
}
-- Respawn definitions for CANYON RUSH (MAP21)
krespawn.points[21] = {
  [0] = 202,
  [1] = {["x"] = 4157, ["y"] = 182, ["z"] = 2000, ["order"] = 0, ["angle"] = -2104295424},
  [2] = {["x"] = 3413, ["y"] = 65, ["z"] = 2000, ["order"] = 0, ["angle"] = -2098921472},
  [3] = {["x"] = 2298, ["y"] = -49, ["z"] = 2000, ["order"] = 0, ["angle"] = 1952907264},
  [4] = {["x"] = 1358, ["y"] = 170, ["z"] = 2000, ["order"] = 0, ["angle"] = 1473052672},
  [5] = {["x"] = 616, ["y"] = 831, ["z"] = 2000, ["order"] = 0, ["angle"] = 1371078656},
  [6] = {["x"] = 0, ["y"] = 1719, ["z"] = 2000, ["order"] = 0, ["angle"] = 964493312},
  [7] = {["x"] = 263, ["y"] = 2422, ["z"] = 2000, ["order"] = 0, ["angle"] = 936968192},
  [8] = {["x"] = 565, ["y"] = 3195, ["z"] = 2000, ["order"] = 0, ["angle"] = 943390720},
  [9] = {["x"] = 981, ["y"] = 4202, ["z"] = 2000, ["order"] = 0, ["angle"] = 1132920832},
  [10] = {["x"] = 1112, ["y"] = 5208, ["z"] = 2000, ["order"] = 0, ["angle"] = 1490616320},
  [11] = {["x"] = 844, ["y"] = 6150, ["z"] = 2000, ["order"] = 0, ["angle"] = 2089877504},
  [12] = {["x"] = 359, ["y"] = 6137, ["z"] = 2000, ["order"] = 0, ["angle"] = -2099314688},
  [13] = {["x"] = -882, ["y"] = 6044, ["z"] = 2000, ["order"] = 2, ["angle"] = 2144534528},
  [14] = {["x"] = -1836, ["y"] = 6023, ["z"] = 1440, ["order"] = 2, ["angle"] = 2144534528},
  [15] = {["x"] = -2836, ["y"] = 6023, ["z"] = 1437, ["order"] = 2, ["angle"] = 2075983872},
  [16] = {["x"] = -4064, ["y"] = 6100, ["z"] = 1437, ["order"] = 2, ["angle"] = 2145189888},
  [17] = {["x"] = -5026, ["y"] = 6135, ["z"] = 1517, ["order"] = 2, ["angle"] = 2145189888},
  [18] = {["x"] = -6039, ["y"] = 6161, ["z"] = 1500, ["order"] = 2, ["angle"] = -2062745600},
  [19] = {["x"] = -7180, ["y"] = 6109, ["z"] = 1500, ["order"] = 3, ["angle"] = -1812135936},
  [20] = {["x"] = -8220, ["y"] = 5764, ["z"] = 1500, ["order"] = 3, ["angle"] = -1506476032},
  [21] = {["x"] = -8677, ["y"] = 5201, ["z"] = 1500, ["order"] = 3, ["angle"] = -954531840},
  [22] = {["x"] = -8386, ["y"] = 4332, ["z"] = 1500, ["order"] = 4, ["angle"] = -696582144},
  [23] = {["x"] = -7868, ["y"] = 3379, ["z"] = 1500, ["order"] = 4, ["angle"] = -886767616},
  [24] = {["x"] = -7513, ["y"] = 2067, ["z"] = 1500, ["order"] = 4, ["angle"] = -1528233984},
  [25] = {["x"] = -8266, ["y"] = 1092, ["z"] = 1497, ["order"] = 4, ["angle"] = -1810694144},
  [26] = {["x"] = -9143, ["y"] = 637, ["z"] = 1500, ["order"] = 5, ["angle"] = -1498349568},
  [27] = {["x"] = -9626, ["y"] = -70, ["z"] = 1500, ["order"] = 5, ["angle"] = -878903296},
  [28] = {["x"] = -9280, ["y"] = -1205, ["z"] = 1500, ["order"] = 5, ["angle"] = -1041956864},
  [29] = {["x"] = -9088, ["y"] = -3104, ["z"] = 1500, ["order"] = 6, ["angle"] = -1136984064},
  [30] = {["x"] = -9154, ["y"] = -4084, ["z"] = 1500, ["order"] = 6, ["angle"] = -1649082368},
  [31] = {["x"] = -9824, ["y"] = -4781, ["z"] = 1515, ["order"] = 6, ["angle"] = 2001534976},
  [32] = {["x"] = -10968, ["y"] = -4340, ["z"] = 1517, ["order"] = 6, ["angle"] = 1116143616},
  [33] = {["x"] = -11267, ["y"] = -3403, ["z"] = 1646, ["order"] = 6, ["angle"] = 487522304},
  [34] = {["x"] = -10529, ["y"] = -2653, ["z"] = 1749, ["order"] = 6, ["angle"] = -40435712},
  [35] = {["x"] = -7955, ["y"] = -2592, ["z"] = 2000, ["order"] = 6, ["angle"] = -130744320},
  [36] = {["x"] = -7197, ["y"] = -2666, ["z"] = 2000, ["order"] = 6, ["angle"] = -19857408},
  [37] = {["x"] = -6171, ["y"] = -2650, ["z"] = 2000, ["order"] = 6, ["angle"] = 131923968},
  [38] = {["x"] = -4691, ["y"] = -2178, ["z"] = 2000, ["order"] = 6, ["angle"] = 329318400},
  [39] = {["x"] = -3652, ["y"] = -1646, ["z"] = 2000, ["order"] = 6, ["angle"] = 179503104},
  [40] = {["x"] = -2319, ["y"] = -1247, ["z"] = 2000, ["order"] = 6, ["angle"] = -37814272},
  [41] = {["x"] = -1174, ["y"] = -1288, ["z"] = 2000, ["order"] = 6, ["angle"] = -385286144},
  [42] = {["x"] = -575, ["y"] = -1701, ["z"] = 2000, ["order"] = 6, ["angle"] = -437714944},
  [43] = {["x"] = -246, ["y"] = -2523, ["z"] = 2000, ["order"] = 6, ["angle"] = 21954560},
  [44] = {["x"] = 517, ["y"] = -2360, ["z"] = 2000, ["order"] = 6, ["angle"] = -103350272},
  [45] = {["x"] = 1507, ["y"] = -2497, ["z"] = 2000, ["order"] = 1, ["angle"] = -435355648},
  [46] = {["x"] = 2968, ["y"] = -3139, ["z"] = 2000, ["order"] = 1, ["angle"] = -800522240},
  [47] = {["x"] = 3488, ["y"] = -3959, ["z"] = 2000, ["order"] = 1, ["angle"] = -1199505408},
  [48] = {["x"] = 3501, ["y"] = -5162, ["z"] = 2000, ["order"] = 2, ["angle"] = -1845166080},
  [49] = {["x"] = 2634, ["y"] = -5583, ["z"] = 2000, ["order"] = 2, ["angle"] = 1980825600},
  [50] = {["x"] = 2196, ["y"] = -5399, ["z"] = 2000, ["order"] = 2, ["angle"] = 1887371264},
  [51] = {["x"] = 1655, ["y"] = -5113, ["z"] = 2000, ["order"] = 2, ["angle"] = 1956446208},
  [52] = {["x"] = 803, ["y"] = -4782, ["z"] = 2000, ["order"] = 2, ["angle"] = -1829175296},
  [53] = {["x"] = 245, ["y"] = -5120, ["z"] = 2000, ["order"] = 2, ["angle"] = -1284440064},
  [54] = {["x"] = -109, ["y"] = -6429, ["z"] = 2250, ["order"] = 2, ["angle"] = -1571749888},
  [55] = {["x"] = -628, ["y"] = -7146, ["z"] = 2250, ["order"] = 2, ["angle"] = -1876623360},
  [56] = {["x"] = -1455, ["y"] = -7572, ["z"] = 2250, ["order"] = 2, ["angle"] = 2115174400},
  [57] = {["x"] = -2741, ["y"] = -7685, ["z"] = 2250, ["order"] = 3, ["angle"] = -2133393408},
  [58] = {["x"] = -3915, ["y"] = -7727, ["z"] = 2250, ["order"] = 3, ["angle"] = 1911750656},
  [59] = {["x"] = -5088, ["y"] = -7497, ["z"] = 2250, ["order"] = 3, ["angle"] = 1674379264},
  [60] = {["x"] = -5914, ["y"] = -6881, ["z"] = 2250, ["order"] = 3, ["angle"] = 1180237824},
  [61] = {["x"] = -6225, ["y"] = -5984, ["z"] = 2250, ["order"] = 3, ["angle"] = 964755456},
  [62] = {["x"] = -6180, ["y"] = -4960, ["z"] = 2317, ["order"] = 3, ["angle"] = 896598016},
  [63] = {["x"] = -5952, ["y"] = -902, ["z"] = 3300, ["order"] = 4, ["angle"] = 1016659968},
  [64] = {["x"] = -5904, ["y"] = 36, ["z"] = 3300, ["order"] = 5, ["angle"] = 1016659968},
  [65] = {["x"] = -5829, ["y"] = 921, ["z"] = 3300, ["order"] = 5, ["angle"] = 868417536},
  [66] = {["x"] = -5783, ["y"] = 2086, ["z"] = 3300, ["order"] = 5, ["angle"] = 422641664},
  [67] = {["x"] = -5409, ["y"] = 2975, ["z"] = 3300, ["order"] = 5, ["angle"] = 105447424},
  [68] = {["x"] = -4589, ["y"] = 2946, ["z"] = 3300, ["order"] = 6, ["angle"] = 202178560},
  [69] = {["x"] = -3948, ["y"] = 3167, ["z"] = 3300, ["order"] = 6, ["angle"] = 141361152},
  [70] = {["x"] = -2947, ["y"] = 3550, ["z"] = 3300, ["order"] = 6, ["angle"] = -94568448},
  [71] = {["x"] = -1869, ["y"] = 3613, ["z"] = 3200, ["order"] = 6, ["angle"] = -691077120},
  [72] = {["x"] = -1165, ["y"] = 2916, ["z"] = 3104, ["order"] = 6, ["angle"] = -1445265408},
  [73] = {["x"] = -1547, ["y"] = 2073, ["z"] = 2984, ["order"] = 6, ["angle"] = -2012807168},
  [74] = {["x"] = -2907, ["y"] = 1770, ["z"] = 2970, ["order"] = 7, ["angle"] = 2062090240},
  [75] = {["x"] = -4034, ["y"] = 1910, ["z"] = 2960, ["order"] = 7, ["angle"] = -1731264512},
  [76] = {["x"] = -4718, ["y"] = 1527, ["z"] = 2888, ["order"] = 7, ["angle"] = -1023737856},
  [77] = {["x"] = -4730, ["y"] = 658, ["z"] = 2860, ["order"] = 7, ["angle"] = -385286144},
  [78] = {["x"] = -4070, ["y"] = 133, ["z"] = 2775, ["order"] = 7, ["angle"] = 104267776},
  [79] = {["x"] = -3423, ["y"] = 129, ["z"] = 2750, ["order"] = 8, ["angle"] = 294322176},
  [80] = {["x"] = -3093, ["y"] = 858, ["z"] = 2750, ["order"] = 8, ["angle"] = 37289984},
  [81] = {["x"] = -2202, ["y"] = 571, ["z"] = 2750, ["order"] = 8, ["angle"] = 172687360},
  [82] = {["x"] = -1052, ["y"] = 953, ["z"] = 2750, ["order"] = 8, ["angle"] = 347406336},
  [83] = {["x"] = 63, ["y"] = 1509, ["z"] = 2750, ["order"] = 8, ["angle"] = 249757696},
  [84] = {["x"] = 1069, ["y"] = 2021, ["z"] = 2750, ["order"] = 9, ["angle"] = 131006464},
  [85] = {["x"] = 2659, ["y"] = 2323, ["z"] = 2750, ["order"] = 9, ["angle"] = 85262336},
  [86] = {["x"] = 1432, ["y"] = -5307, ["z"] = 2000, ["order"] = 2, ["angle"] = -1513160704},
  [87] = {["x"] = 965, ["y"] = -5844, ["z"] = 2000, ["order"] = 2, ["angle"] = -1360592896},
  [88] = {["x"] = 711, ["y"] = -6928, ["z"] = 2000, ["order"] = 2, ["angle"] = -1570177024},
  [89] = {["x"] = 73, ["y"] = -7670, ["z"] = 2000, ["order"] = 2, ["angle"] = 2069430272},
  [90] = {["x"] = -955, ["y"] = -7459, ["z"] = 2000, ["order"] = 2, ["angle"] = 1635450880},
  [91] = {["x"] = -1547, ["y"] = -6878, ["z"] = 2000, ["order"] = 2, ["angle"] = 1932460032},
  [92] = {["x"] = -2389, ["y"] = -6749, ["z"] = 2000, ["order"] = 3, ["angle"] = -2094071808},
  [93] = {["x"] = -3237, ["y"] = -6879, ["z"] = 2000, ["order"] = 3, ["angle"] = 2021982208},
  [94] = {["x"] = -4558, ["y"] = -6861, ["z"] = 2000, ["order"] = 3, ["angle"] = 1570832384},
  [95] = {["x"] = -5364, ["y"] = -6277, ["z"] = 2004, ["order"] = 3, ["angle"] = 1176698880},
  [96] = {["x"] = -5582, ["y"] = -5127, ["z"] = 2291, ["order"] = 3, ["angle"] = 1090322432},
  [97] = {["x"] = -2132, ["y"] = 1971, ["z"] = 2970, ["order"] = 7, ["angle"] = -2102460416},
  [98] = {["x"] = 3491, ["y"] = 2379, ["z"] = 2750, ["order"] = 1, ["angle"] = -35192832},
  [99] = {["x"] = 4533, ["y"] = 2265, ["z"] = 2750, ["order"] = 1, ["angle"] = 134414336},
  [100] = {["x"] = 5618, ["y"] = 2372, ["z"] = 2750, ["order"] = 1, ["angle"] = 583073792},
  [101] = {["x"] = 6290, ["y"] = 3223, ["z"] = 2792, ["order"] = 1, ["angle"] = 553975808},
  [102] = {["x"] = 6808, ["y"] = 4193, ["z"] = 2880, ["order"] = 1, ["angle"] = 450691072},
  [103] = {["x"] = 7570, ["y"] = 4872, ["z"] = 2880, ["order"] = 1, ["angle"] = -259194880},
  [104] = {["x"] = 8494, ["y"] = 4816, ["z"] = 2839, ["order"] = 1, ["angle"] = -762642432},
  [105] = {["x"] = 9139, ["y"] = 3979, ["z"] = 2820, ["order"] = 1, ["angle"] = -1642790912},
  [106] = {["x"] = 8353, ["y"] = 3378, ["z"] = 2750, ["order"] = 1, ["angle"] = -1573584896},
  [107] = {["x"] = 7498, ["y"] = 2761, ["z"] = 2709, ["order"] = 1, ["angle"] = -974716928},
  [108] = {["x"] = 7327, ["y"] = 1671, ["z"] = 2623, ["order"] = 1, ["angle"] = 123273216},
  [109] = {["x"] = 8056, ["y"] = 1719, ["z"] = 2615, ["order"] = 3, ["angle"] = 265748480},
  [110] = {["x"] = 8826, ["y"] = 2262, ["z"] = 2615, ["order"] = 3, ["angle"] = -580321280},
  [111] = {["x"] = 9368, ["y"] = 1960, ["z"] = 2615, ["order"] = 3, ["angle"] = -1008402432},
  [112] = {["x"] = 9384, ["y"] = 637, ["z"] = 2600, ["order"] = 3, ["angle"] = -1243545600},
  [113] = {["x"] = 10227, ["y"] = -291, ["z"] = 2600, ["order"] = 3, ["angle"] = -1095696384},
  [114] = {["x"] = 8992, ["y"] = -242, ["z"] = 2600, ["order"] = 3, ["angle"] = -1082720256},
  [115] = {["x"] = 8589, ["y"] = -1077, ["z"] = 2600, ["order"] = 3, ["angle"] = -1172111360},
  [116] = {["x"] = 8814, ["y"] = -1978, ["z"] = 2600, ["order"] = 3, ["angle"] = -1105920000},
  [117] = {["x"] = 9398, ["y"] = -2005, ["z"] = 2600, ["order"] = 3, ["angle"] = -855441408},
  [118] = {["x"] = 10240, ["y"] = -1435, ["z"] = 2600, ["order"] = 3, ["angle"] = -1281425408},
  [119] = {["x"] = 10138, ["y"] = -2530, ["z"] = 2600, ["order"] = 3, ["angle"] = -1280245760},
  [120] = {["x"] = 9969, ["y"] = -3499, ["z"] = 2600, ["order"] = 4, ["angle"] = -1779236864},
  [121] = {["x"] = 9310, ["y"] = -4339, ["z"] = 2600, ["order"] = 4, ["angle"] = -2018705408},
  [122] = {["x"] = 8338, ["y"] = -4741, ["z"] = 2600, ["order"] = 4, ["angle"] = 1695612928},
  [123] = {["x"] = 6969, ["y"] = -4490, ["z"] = 2600, ["order"] = 4, ["angle"] = 962527232},
  [124] = {["x"] = 7297, ["y"] = -3692, ["z"] = 2564, ["order"] = 5, ["angle"] = 1046151168},
  [125] = {["x"] = 7605, ["y"] = -2435, ["z"] = 2410, ["order"] = 5, ["angle"] = 1029767168},
  [126] = {["x"] = 7164, ["y"] = -2501, ["z"] = 2410, ["order"] = 5, ["angle"] = 1175519232},
  [127] = {["x"] = 7399, ["y"] = -1688, ["z"] = 2410, ["order"] = 5, ["angle"] = 941948928},
  [128] = {["x"] = 7733, ["y"] = -866, ["z"] = 2410, ["order"] = 6, ["angle"] = 1214316544},
  [129] = {["x"] = 7721, ["y"] = -107, ["z"] = 2410, ["order"] = 6, ["angle"] = 1821966336},
  [130] = {["x"] = 7033, ["y"] = 300, ["z"] = 2410, ["order"] = 6, ["angle"] = -1808334848},
  [131] = {["x"] = 6249, ["y"] = -139, ["z"] = 2410, ["order"] = 6, ["angle"] = -1687355392},
  [132] = {["x"] = 5478, ["y"] = -513, ["z"] = 2410, ["order"] = 6, ["angle"] = -1660354560},
  [133] = {["x"] = 4392, ["y"] = -1315, ["z"] = 2410, ["order"] = 6, ["angle"] = -945225728},
  [134] = {["x"] = 4342, ["y"] = -2321, ["z"] = 1572, ["order"] = 7, ["angle"] = -989659136},
  [135] = {["x"] = 4482, ["y"] = -3037, ["z"] = 1570, ["order"] = 7, ["angle"] = -886898688},
  [136] = {["x"] = 4398, ["y"] = -3982, ["z"] = 1590, ["order"] = 7, ["angle"] = -688848896},
  [137] = {["x"] = 4635, ["y"] = -4812, ["z"] = 1580, ["order"] = 7, ["angle"] = -444268544},
  [138] = {["x"] = 5374, ["y"] = -5603, ["z"] = 1600, ["order"] = 7, ["angle"] = -380043264},
  [139] = {["x"] = 6318, ["y"] = -6062, ["z"] = 1600, ["order"] = 7, ["angle"] = -446103552},
  [140] = {["x"] = 7317, ["y"] = -6548, ["z"] = 1600, ["order"] = 8, ["angle"] = -660144128},
  [141] = {["x"] = 8188, ["y"] = -7286, ["z"] = 1600, ["order"] = 8, ["angle"] = -1166606336},
  [142] = {["x"] = 8365, ["y"] = -8133, ["z"] = 1600, ["order"] = 8, ["angle"] = -1652490240},
  [143] = {["x"] = 7815, ["y"] = -8842, ["z"] = 1600, ["order"] = 9, ["angle"] = -2086993920},
  [144] = {["x"] = 7311, ["y"] = -9517, ["z"] = 1600, ["order"] = 9, ["angle"] = 1794179072},
  [145] = {["x"] = 6516, ["y"] = -8706, ["z"] = 1600, ["order"] = 9, ["angle"] = 1724448768},
  [146] = {["x"] = 5431, ["y"] = -7729, ["z"] = 1600, ["order"] = 9, ["angle"] = -1754071040},
  [147] = {["x"] = 4626, ["y"] = -7837, ["z"] = 1600, ["order"] = 9, ["angle"] = -1760362496},
  [148] = {["x"] = 4048, ["y"] = -8159, ["z"] = 1600, ["order"] = 1, ["angle"] = -1760362496},
  [149] = {["x"] = 3195, ["y"] = -8807, ["z"] = 1600, ["order"] = 1, ["angle"] = -1811349504},
  [150] = {["x"] = 2352, ["y"] = -9361, ["z"] = 1600, ["order"] = 1, ["angle"] = -2108751872},
  [151] = {["x"] = 1360, ["y"] = -9464, ["z"] = 1620, ["order"] = 1, ["angle"] = 2134441984},
  [152] = {["x"] = 269, ["y"] = -9411, ["z"] = 1600, ["order"] = 1, ["angle"] = -2001928192},
  [153] = {["x"] = -784, ["y"] = -9613, ["z"] = 1600, ["order"] = 1, ["angle"] = -1784217600},
  [154] = {["x"] = -3072, ["y"] = -10902, ["z"] = 1900, ["order"] = 2, ["angle"] = -1733230592},
  [155] = {["x"] = -3525, ["y"] = -11537, ["z"] = 1900, ["order"] = 2, ["angle"] = -1850933248},
  [156] = {["x"] = -4381, ["y"] = -11930, ["z"] = 1870, ["order"] = 2, ["angle"] = -2043609088},
  [157] = {["x"] = -5284, ["y"] = -12119, ["z"] = 1900, ["order"] = 2, ["angle"] = -2013855744},
  [158] = {["x"] = -6296, ["y"] = -12512, ["z"] = 1900, ["order"] = 2, ["angle"] = 1898512384},
  [159] = {["x"] = -7186, ["y"] = -12146, ["z"] = 1900, ["order"] = 2, ["angle"] = 1521156096},
  [160] = {["x"] = -7579, ["y"] = -11106, ["z"] = 1900, ["order"] = 2, ["angle"] = 2107310080},
  [161] = {["x"] = -8437, ["y"] = -10845, ["z"] = 1900, ["order"] = 2, ["angle"] = -1285750784},
  [162] = {["x"] = -8859, ["y"] = -11559, ["z"] = 1900, ["order"] = 3, ["angle"] = -1059651584},
  [163] = {["x"] = -8979, ["y"] = -12562, ["z"] = 1900, ["order"] = 3, ["angle"] = -961478656},
  [164] = {["x"] = -8951, ["y"] = -13848, ["z"] = 1900, ["order"] = 3, ["angle"] = -876412928},
  [165] = {["x"] = -8853, ["y"] = -14962, ["z"] = 1900, ["order"] = 3, ["angle"] = -609026048},
  [166] = {["x"] = -8524, ["y"] = -15863, ["z"] = 1900, ["order"] = 4, ["angle"] = -30605312},
  [167] = {["x"] = -7890, ["y"] = -16424, ["z"] = 1900, ["order"] = 4, ["angle"] = 345702400},
  [168] = {["x"] = -7379, ["y"] = -15758, ["z"] = 1913, ["order"] = 4, ["angle"] = 174915584},
  [169] = {["x"] = -6406, ["y"] = -15291, ["z"] = 2065, ["order"] = 4, ["angle"] = -4915200},
  [170] = {["x"] = -5141, ["y"] = -15080, ["z"] = 2100, ["order"] = 4, ["angle"] = -373227520},
  [171] = {["x"] = -3798, ["y"] = -15450, ["z"] = 2100, ["order"] = 5, ["angle"] = -864354304},
  [172] = {["x"] = -3230, ["y"] = -16251, ["z"] = 2100, ["order"] = 5, ["angle"] = -1127677952},
  [173] = {["x"] = -3105, ["y"] = -17161, ["z"] = 2100, ["order"] = 5, ["angle"] = -1125318656},
  [174] = {["x"] = -2970, ["y"] = -18404, ["z"] = 2100, ["order"] = 5, ["angle"] = -1251934208},
  [175] = {["x"] = -3094, ["y"] = -19797, ["z"] = 2100, ["order"] = 5, ["angle"] = -1685389312},
  [176] = {["x"] = -3722, ["y"] = -20748, ["z"] = 2100, ["order"] = 5, ["angle"] = -1868496896},
  [177] = {["x"] = -4735, ["y"] = -21373, ["z"] = 2100, ["order"] = 5, ["angle"] = 2035089408},
  [178] = {["x"] = -5844, ["y"] = -21561, ["z"] = 2100, ["order"] = 5, ["angle"] = 1794310144},
  [179] = {["x"] = -6664, ["y"] = -21285, ["z"] = 2100, ["order"] = 5, ["angle"] = 1111949312},
  [180] = {["x"] = -7077, ["y"] = -20478, ["z"] = 2100, ["order"] = 6, ["angle"] = 566165504},
  [181] = {["x"] = -6451, ["y"] = -19729, ["z"] = 2100, ["order"] = 6, ["angle"] = 693043200},
  [182] = {["x"] = -5721, ["y"] = -19047, ["z"] = 2100, ["order"] = 6, ["angle"] = 1023475712},
  [183] = {["x"] = -5447, ["y"] = -18086, ["z"] = 2100, ["order"] = 6, ["angle"] = 1459159040},
  [184] = {["x"] = -5821, ["y"] = -17213, ["z"] = 2100, ["order"] = 6, ["angle"] = -2073624576},
  [185] = {["x"] = -6977, ["y"] = -16890, ["z"] = 2258, ["order"] = 7, ["angle"] = -2052128768},
  [186] = {["x"] = -7873, ["y"] = -16869, ["z"] = 2390, ["order"] = 7, ["angle"] = -1807286272},
  [187] = {["x"] = -8880, ["y"] = -17072, ["z"] = 2442, ["order"] = 7, ["angle"] = -1336213504},
  [188] = {["x"] = -9383, ["y"] = -17773, ["z"] = 2485, ["order"] = 8, ["angle"] = -390660096},
  [189] = {["x"] = -8802, ["y"] = -18294, ["z"] = 2520, ["order"] = 8, ["angle"] = 168886272},
  [190] = {["x"] = -8218, ["y"] = -17892, ["z"] = 2520, ["order"] = 8, ["angle"] = -10813440},
  [191] = {["x"] = -7501, ["y"] = -18113, ["z"] = 2550, ["order"] = 8, ["angle"] = -67174400},
  [192] = {["x"] = -6082, ["y"] = -18092, ["z"] = 2640, ["order"] = 8, ["angle"] = 983040},
  [193] = {["x"] = -5109, ["y"] = -18081, ["z"] = 2640, ["order"] = 8, ["angle"] = 93257728},
  [194] = {["x"] = -3957, ["y"] = -17872, ["z"] = 2720, ["order"] = 8, ["angle"] = -18677760},
  [195] = {["x"] = -3984, ["y"] = -18366, ["z"] = 2719, ["order"] = 8, ["angle"] = 28901376},
  [196] = {["x"] = -3137, ["y"] = -18053, ["z"] = 2720, ["order"] = 8, ["angle"] = -54198272},
  [197] = {["x"] = 3091, ["y"] = -18000, ["z"] = 1364, ["order"] = 8, ["angle"] = -21299200},
  [198] = {["x"] = 4499, ["y"] = -18023, ["z"] = 1254, ["order"] = 8, ["angle"] = -21299200},
  [199] = {["x"] = 5951, ["y"] = -18063, ["z"] = 1141, ["order"] = 8, ["angle"] = -21299200},
  [200] = {["x"] = 7313, ["y"] = -18104, ["z"] = 1034, ["order"] = 8, ["angle"] = -21299200},
  [201] = {["x"] = 8522, ["y"] = -18142, ["z"] = 1000, ["order"] = 9, ["angle"] = -21299200},
  [202] = {["x"] = 9385, ["y"] = -18169, ["z"] = 1000, ["order"] = 9, ["angle"] = -21299200}
}
-- Respawn definitions for CASINO RESORT (MAP22)
krespawn.points[22] = {
  [0] = 114,
  [1] = {["x"] = -969, ["y"] = -7544, ["z"] = 510, ["order"] = 0, ["angle"] = -263258112},
  [2] = {["x"] = 0, ["y"] = -7703, ["z"] = 510, ["order"] = 0, ["angle"] = -798162944},
  [3] = {["x"] = 275, ["y"] = -8512, ["z"] = 510, ["order"] = 1, ["angle"] = -819396608},
  [4] = {["x"] = 496, ["y"] = -9598, ["z"] = 510, ["order"] = 1, ["angle"] = -490274816},
  [5] = {["x"] = 1158, ["y"] = -10162, ["z"] = 510, ["order"] = 1, ["angle"] = -3866624},
  [6] = {["x"] = 2401, ["y"] = -10317, ["z"] = 510, ["order"] = 1, ["angle"] = 92733440},
  [7] = {["x"] = 3594, ["y"] = -10334, ["z"] = 510, ["order"] = 1, ["angle"] = 224067584},
  [8] = {["x"] = 4771, ["y"] = -10313, ["z"] = 510, ["order"] = 1, ["angle"] = 272039936},
  [9] = {["x"] = 5855, ["y"] = -10083, ["z"] = 510, ["order"] = 1, ["angle"] = 692781056},
  [10] = {["x"] = 6415, ["y"] = -9470, ["z"] = 510, ["order"] = 1, ["angle"] = 1163198464},
  [11] = {["x"] = 6428, ["y"] = -8596, ["z"] = 510, ["order"] = 2, ["angle"] = 1726545920},
  [12] = {["x"] = 5980, ["y"] = -8116, ["z"] = 510, ["order"] = 2, ["angle"] = 2083979264},
  [13] = {["x"] = 5194, ["y"] = -8104, ["z"] = 515, ["order"] = 2, ["angle"] = 1967456256},
  [14] = {["x"] = 4103, ["y"] = -7995, ["z"] = 510, ["order"] = 2, ["angle"] = 1692467200},
  [15] = {["x"] = 3266, ["y"] = -7418, ["z"] = 500, ["order"] = 3, ["angle"] = 1372127232},
  [16] = {["x"] = 2818, ["y"] = -6597, ["z"] = 510, ["order"] = 3, ["angle"] = 1106575360},
  [17] = {["x"] = 2723, ["y"] = -5453, ["z"] = 510, ["order"] = 3, ["angle"] = 1089536000},
  [18] = {["x"] = 2703, ["y"] = -4268, ["z"] = 521, ["order"] = 3, ["angle"] = 1089536000},
  [19] = {["x"] = 2677, ["y"] = -3048, ["z"] = 510, ["order"] = 3, ["angle"] = 1089536000},
  [20] = {["x"] = 2782, ["y"] = -1858, ["z"] = 618, ["order"] = 3, ["angle"] = 1328611328},
  [21] = {["x"] = 2677, ["y"] = -981, ["z"] = 670, ["order"] = 4, ["angle"] = 1766129664},
  [22] = {["x"] = 2107, ["y"] = -561, ["z"] = 670, ["order"] = 4, ["angle"] = 2059075584},
  [23] = {["x"] = 1017, ["y"] = -546, ["z"] = 675, ["order"] = 4, ["angle"] = 1833500672},
  [24] = {["x"] = -94, ["y"] = -306, ["z"] = 670, ["order"] = 4, ["angle"] = 1574371328},
  [25] = {["x"] = -708, ["y"] = 299, ["z"] = 670, ["order"] = 4, ["angle"] = 1111687168},
  [26] = {["x"] = -912, ["y"] = 1282, ["z"] = 670, ["order"] = 5, ["angle"] = 1087045632},
  [27] = {["x"] = -939, ["y"] = 2401, ["z"] = 688, ["order"] = 5, ["angle"] = 1036845056},
  [28] = {["x"] = -909, ["y"] = 3305, ["z"] = 759, ["order"] = 5, ["angle"] = 1036845056},
  [29] = {["x"] = -894, ["y"] = 6214, ["z"] = 1060, ["order"] = 5, ["angle"] = 932511744},
  [30] = {["x"] = -841, ["y"] = 7285, ["z"] = 1050, ["order"] = 5, ["angle"] = 594739200},
  [31] = {["x"] = -320, ["y"] = 8172, ["z"] = 1050, ["order"] = 5, ["angle"] = 107806720},
  [32] = {["x"] = 780, ["y"] = 8618, ["z"] = 1050, ["order"] = 6, ["angle"] = -541786112},
  [33] = {["x"] = 1797, ["y"] = 8201, ["z"] = 1050, ["order"] = 6, ["angle"] = -1063452672},
  [34] = {["x"] = 2279, ["y"] = 7238, ["z"] = 1050, ["order"] = 6, ["angle"] = -1273954304},
  [35] = {["x"] = 2133, ["y"] = 6087, ["z"] = 1060, ["order"] = 6, ["angle"] = -1087963136},
  [36] = {["x"] = 2076, ["y"] = 5065, ["z"] = 1160, ["order"] = 6, ["angle"] = -1111425024},
  [37] = {["x"] = 2264, ["y"] = 3784, ["z"] = 1311, ["order"] = 6, ["angle"] = -1482883072},
  [38] = {["x"] = 1724, ["y"] = 2959, ["z"] = 1410, ["order"] = 7, ["angle"] = -2003238912},
  [39] = {["x"] = 505, ["y"] = 2795, ["z"] = 1428, ["order"] = 7, ["angle"] = -1883832320},
  [40] = {["x"] = -832, ["y"] = 2450, ["z"] = 1428, ["order"] = 7, ["angle"] = -990838784},
  [41] = {["x"] = -765, ["y"] = 1745, ["z"] = 1428, ["order"] = 7, ["angle"] = -198901760},
  [42] = {["x"] = 299, ["y"] = 1472, ["z"] = 1428, ["order"] = 8, ["angle"] = -103088128},
  [43] = {["x"] = 1462, ["y"] = 1455, ["z"] = 1428, ["order"] = 8, ["angle"] = -262209536},
  [44] = {["x"] = 2769, ["y"] = 1278, ["z"] = 1428, ["order"] = 8, ["angle"] = -658571264},
  [45] = {["x"] = 2669, ["y"] = 1913, ["z"] = 1428, ["order"] = 8, ["angle"] = -471007232},
  [46] = {["x"] = 3242, ["y"] = 1272, ["z"] = 1428, ["order"] = 8, ["angle"] = -870645760},
  [47] = {["x"] = 3464, ["y"] = 359, ["z"] = 1428, ["order"] = 9, ["angle"] = -1724317696},
  [48] = {["x"] = 2830, ["y"] = -276, ["z"] = 1428, ["order"] = 9, ["angle"] = -2114256896},
  [49] = {["x"] = 1982, ["y"] = -287, ["z"] = 1428, ["order"] = 9, ["angle"] = 2121596928},
  [50] = {["x"] = 946, ["y"] = -252, ["z"] = 1428, ["order"] = 9, ["angle"] = 2121596928},
  [51] = {["x"] = -222, ["y"] = -209, ["z"] = 1428, ["order"] = 9, ["angle"] = 2121596928},
  [52] = {["x"] = -1429, ["y"] = -163, ["z"] = 1428, ["order"] = 10, ["angle"] = 2145714176},
  [53] = {["x"] = -1615, ["y"] = -809, ["z"] = 1420, ["order"] = 10, ["angle"] = 1933508608},
  [54] = {["x"] = -2512, ["y"] = -261, ["z"] = 1420, ["order"] = 10, ["angle"] = 2077425664},
  [55] = {["x"] = -3456, ["y"] = -112, ["z"] = 1420, ["order"] = 10, ["angle"] = 2077425664},
  [56] = {["x"] = -4647, ["y"] = 12, ["z"] = 1420, ["order"] = 10, ["angle"] = -2056192000},
  [57] = {["x"] = -5698, ["y"] = 43, ["z"] = 1420, ["order"] = 10, ["angle"] = -1457848320},
  [58] = {["x"] = -6253, ["y"] = -498, ["z"] = 1420, ["order"] = 11, ["angle"] = -889651200},
  [59] = {["x"] = -6297, ["y"] = -1368, ["z"] = 1280, ["order"] = 11, ["angle"] = -387252224},
  [60] = {["x"] = -5723, ["y"] = -1979, ["z"] = 1250, ["order"] = 11, ["angle"] = 165871616},
  [61] = {["x"] = -4725, ["y"] = -2039, ["z"] = 1250, ["order"] = 12, ["angle"] = 650313728},
  [62] = {["x"] = -4168, ["y"] = -1437, ["z"] = 1236, ["order"] = 12, ["angle"] = 1356267520},
  [63] = {["x"] = -4216, ["y"] = -531, ["z"] = 1085, ["order"] = 12, ["angle"] = 1902444544},
  [64] = {["x"] = -5002, ["y"] = -30, ["z"] = 1080, ["order"] = 12, ["angle"] = -1127153664},
  [65] = {["x"] = -5276, ["y"] = -890, ["z"] = 1017, ["order"] = 12, ["angle"] = -1012203520},
  [66] = {["x"] = -5256, ["y"] = -2300, ["z"] = 910, ["order"] = 12, ["angle"] = -1062666240},
  [67] = {["x"] = -5116, ["y"] = -2273, ["z"] = 910, ["order"] = 12, ["angle"] = -731578368},
  [68] = {["x"] = -5338, ["y"] = -2305, ["z"] = 910, ["order"] = 12, ["angle"] = -1444741120},
  [69] = {["x"] = -5881, ["y"] = -3040, ["z"] = 910, ["order"] = 13, ["angle"] = -1569259520},
  [70] = {["x"] = -6435, ["y"] = -3817, ["z"] = 867, ["order"] = 13, ["angle"] = -1062666240},
  [71] = {["x"] = -6538, ["y"] = -4802, ["z"] = 788, ["order"] = 13, ["angle"] = -1138950144},
  [72] = {["x"] = -6688, ["y"] = -5922, ["z"] = 698, ["order"] = 13, ["angle"] = -858849280},
  [73] = {["x"] = -6280, ["y"] = -6797, ["z"] = 670, ["order"] = 13, ["angle"] = -737738752},
  [74] = {["x"] = -5769, ["y"] = -7916, ["z"] = 600, ["order"] = 13, ["angle"] = -1362558976},
  [75] = {["x"] = -6104, ["y"] = -9073, ["z"] = 600, ["order"] = 14, ["angle"] = -1883701248},
  [76] = {["x"] = -7242, ["y"] = -9508, ["z"] = 661, ["order"] = 14, ["angle"] = 1891434496},
  [77] = {["x"] = -8259, ["y"] = -9412, ["z"] = 746, ["order"] = 14, ["angle"] = 1332674560},
  [78] = {["x"] = -8766, ["y"] = -8717, ["z"] = 783, ["order"] = 14, ["angle"] = 766443520},
  [79] = {["x"] = -8444, ["y"] = -8005, ["z"] = 840, ["order"] = 15, ["angle"] = 65732608},
  [80] = {["x"] = -8016, ["y"] = -7535, ["z"] = 864, ["order"] = 15, ["angle"] = 138084352},
  [81] = {["x"] = -7213, ["y"] = -7539, ["z"] = 927, ["order"] = 15, ["angle"] = -91684864},
  [82] = {["x"] = -6295, ["y"] = -7623, ["z"] = 950, ["order"] = 15, ["angle"] = 50397184},
  [83] = {["x"] = -5112, ["y"] = -7629, ["z"] = 950, ["order"] = 15, ["angle"] = -30081024},
  [84] = {["x"] = -4071, ["y"] = -7632, ["z"] = 940, ["order"] = 15, ["angle"] = -25231360},
  [85] = {["x"] = -2766, ["y"] = -7647, ["z"] = 500, ["order"] = 15, ["angle"] = -25231360},
  [86] = {["x"] = -1873, ["y"] = -7669, ["z"] = 500, ["order"] = 15, ["angle"] = -25231360},
  [87] = {["x"] = 4171, ["y"] = -8167, ["z"] = 510, ["order"] = 2, ["angle"] = -1722613760},
  [88] = {["x"] = 3058, ["y"] = -8954, ["z"] = 510, ["order"] = 3, ["angle"] = 2124349440},
  [89] = {["x"] = 2246, ["y"] = -9022, ["z"] = 510, ["order"] = 3, ["angle"] = 1424556032},
  [90] = {["x"] = 1840, ["y"] = -8689, ["z"] = 510, ["order"] = 3, ["angle"] = 1071316992},
  [91] = {["x"] = 1746, ["y"] = -7900, ["z"] = 510, ["order"] = 3, ["angle"] = 541917184},
  [92] = {["x"] = 2363, ["y"] = -7216, ["z"] = 510, ["order"] = 3, ["angle"] = 915865600},
  [93] = {["x"] = -5207, ["y"] = -2850, ["z"] = 910, ["order"] = 13, ["angle"] = -1196883968},
  [94] = {["x"] = -5524, ["y"] = -3802, ["z"] = 868, ["order"] = 13, ["angle"] = -1286406144},
  [95] = {["x"] = -5784, ["y"] = -4787, ["z"] = 789, ["order"] = 13, ["angle"] = -1008926720},
  [96] = {["x"] = -5581, ["y"] = -5754, ["z"] = 712, ["order"] = 13, ["angle"] = -811925504},
  [97] = {["x"] = -5219, ["y"] = -6487, ["z"] = 670, ["order"] = 13, ["angle"] = -1038286848},
  [98] = {["x"] = -4534, ["y"] = -2972, ["z"] = 910, ["order"] = 13, ["angle"] = -728956928},
  [99] = {["x"] = -4177, ["y"] = -3622, ["z"] = 882, ["order"] = 13, ["angle"] = -1034354688},
  [100] = {["x"] = -4151, ["y"] = -4594, ["z"] = 805, ["order"] = 13, ["angle"] = -1000800256},
  [101] = {["x"] = -3851, ["y"] = -5341, ["z"] = 745, ["order"] = 13, ["angle"] = -1050738688},
  [102] = {["x"] = -3811, ["y"] = -5984, ["z"] = 693, ["order"] = 13, ["angle"] = -1315504128},
  [103] = {["x"] = -4196, ["y"] = -6866, ["z"] = 670, ["order"] = 13, ["angle"] = -1458503680},
  [104] = {["x"] = -5152, ["y"] = -7135, ["z"] = 670, ["order"] = 13, ["angle"] = -1231749120},
  [105] = {["x"] = -4614, ["y"] = -4354, ["z"] = 824, ["order"] = 13, ["angle"] = -1094385664},
  [106] = {["x"] = -4784, ["y"] = -5346, ["z"] = 745, ["order"] = 13, ["angle"] = -1259012096},
  [107] = {["x"] = -5588, ["y"] = -4204, ["z"] = 836, ["order"] = 13, ["angle"] = -1034616832},
  [108] = {["x"] = -5045, ["y"] = -3765, ["z"] = 871, ["order"] = 13, ["angle"] = -868286464},
  [109] = {["x"] = -4297, ["y"] = -7880, ["z"] = 600, ["order"] = 13, ["angle"] = -1317208064},
  [110] = {["x"] = -5156, ["y"] = -7848, ["z"] = 600, ["order"] = 13, ["angle"] = -1252589568},
  [111] = {["x"] = -4533, ["y"] = -8639, ["z"] = 600, ["order"] = 13, ["angle"] = -1830879232},
  [112] = {["x"] = -5234, ["y"] = -8374, ["z"] = 610, ["order"] = 13, ["angle"] = -1802960896},
  [113] = {["x"] = -5222, ["y"] = -8978, ["z"] = 600, ["order"] = 14, ["angle"] = -1760362496},
  [114] = {["x"] = -6341, ["y"] = -9577, ["z"] = 600, ["order"] = 14, ["angle"] = -2134441984}
}
-- Respawn definitions for SILVERCLOUD ISLAND (MAP23)
krespawn.points[23] = {
  [0] = 89,
  [1] = {["x"] = -5788, ["y"] = -7665, ["z"] = 1200, ["order"] = 0, ["angle"] = -10420224},
  [2] = {["x"] = -4907, ["y"] = -7779, ["z"] = 1200, ["order"] = 0, ["angle"] = 128253952},
  [3] = {["x"] = -3781, ["y"] = -7774, ["z"] = 1200, ["order"] = 0, ["angle"] = 184614912},
  [4] = {["x"] = -2584, ["y"] = -7580, ["z"] = 1200, ["order"] = 0, ["angle"] = 726597632},
  [5] = {["x"] = -2160, ["y"] = -6824, ["z"] = 1200, ["order"] = 1, ["angle"] = 954793984},
  [6] = {["x"] = -2078, ["y"] = -5801, ["z"] = 1200, ["order"] = 1, ["angle"] = 923074560},
  [7] = {["x"] = -1635, ["y"] = -4781, ["z"] = 1200, ["order"] = 1, ["angle"] = 738394112},
  [8] = {["x"] = -1050, ["y"] = -3707, ["z"] = 1200, ["order"] = 1, ["angle"] = 1239744512},
  [9] = {["x"] = -1119, ["y"] = -2948, ["z"] = 1200, ["order"] = 1, ["angle"] = 1383923712},
  [10] = {["x"] = -1647, ["y"] = -1969, ["z"] = 1200, ["order"] = 2, ["angle"] = 1328611328},
  [11] = {["x"] = -2006, ["y"] = -887, ["z"] = 1200, ["order"] = 2, ["angle"] = 971440128},
  [12] = {["x"] = -2065, ["y"] = 199, ["z"] = 1200, ["order"] = 2, ["angle"] = 740622336},
  [13] = {["x"] = -1864, ["y"] = 1164, ["z"] = 1200, ["order"] = 2, ["angle"] = 346619904},
  [14] = {["x"] = -1318, ["y"] = 1618, ["z"] = 1200, ["order"] = 2, ["angle"] = -265224192},
  [15] = {["x"] = -633, ["y"] = 1469, ["z"] = 1200, ["order"] = 3, ["angle"] = -807731200},
  [16] = {["x"] = -424, ["y"] = 763, ["z"] = 1200, ["order"] = 3, ["angle"] = -1120993280},
  [17] = {["x"] = -503, ["y"] = -122, ["z"] = 1200, ["order"] = 3, ["angle"] = -719781888},
  [18] = {["x"] = -18, ["y"] = -1074, ["z"] = 1200, ["order"] = 4, ["angle"] = 3080192},
  [19] = {["x"] = 1383, ["y"] = -1349, ["z"] = 1000, ["order"] = 4, ["angle"] = 155910144},
  [20] = {["x"] = 2220, ["y"] = -1378, ["z"] = 1000, ["order"] = 4, ["angle"] = 510459904},
  [21] = {["x"] = 2897, ["y"] = -868, ["z"] = 1000, ["order"] = 4, ["angle"] = 1105526784},
  [22] = {["x"] = 3126, ["y"] = 35, ["z"] = 1000, ["order"] = 5, ["angle"] = 1393885184},
  [23] = {["x"] = 2774, ["y"] = 851, ["z"] = 1000, ["order"] = 5, ["angle"] = 1650655232},
  [24] = {["x"] = 1998, ["y"] = 1698, ["z"] = 1000, ["order"] = 5, ["angle"] = 1717895168},
  [25] = {["x"] = 1212, ["y"] = 2332, ["z"] = 600, ["order"] = 5, ["angle"] = 1616969728},
  [26] = {["x"] = 452, ["y"] = 2981, ["z"] = 400, ["order"] = 5, ["angle"] = 1764425728},
  [27] = {["x"] = -214, ["y"] = 3518, ["z"] = 400, ["order"] = 5, ["angle"] = 1934557184},
  [28] = {["x"] = -999, ["y"] = 4026, ["z"] = 400, ["order"] = 6, ["angle"] = -2114519040},
  [29] = {["x"] = -1735, ["y"] = 3952, ["z"] = 410, ["order"] = 6, ["angle"] = -2009923584},
  [30] = {["x"] = -2680, ["y"] = 3677, ["z"] = 410, ["order"] = 6, ["angle"] = -2143485952},
  [31] = {["x"] = -3803, ["y"] = 3584, ["z"] = 410, ["order"] = 6, ["angle"] = 1981743104},
  [32] = {["x"] = -4785, ["y"] = 3823, ["z"] = 420, ["order"] = 6, ["angle"] = 1903230976},
  [33] = {["x"] = -5551, ["y"] = 4177, ["z"] = 410, ["order"] = 6, ["angle"] = 2083061760},
  [34] = {["x"] = -6516, ["y"] = 4540, ["z"] = 400, ["order"] = 6, ["angle"] = -1747779584},
  [35] = {["x"] = -7261, ["y"] = 4308, ["z"] = 400, ["order"] = 7, ["angle"] = -1138556928},
  [36] = {["x"] = -7472, ["y"] = 3948, ["z"] = 400, ["order"] = 7, ["angle"] = -1021771776},
  [37] = {["x"] = -7439, ["y"] = 3243, ["z"] = 400, ["order"] = 7, ["angle"] = -1022689280},
  [38] = {["x"] = -7389, ["y"] = 2068, ["z"] = 250, ["order"] = 7, ["angle"] = -1025703936},
  [39] = {["x"] = -7364, ["y"] = 997, ["z"] = 250, ["order"] = 7, ["angle"] = -1079312384},
  [40] = {["x"] = -7380, ["y"] = -313, ["z"] = 250, ["order"] = 8, ["angle"] = -1079312384},
  [41] = {["x"] = -7217, ["y"] = -57, ["z"] = 250, ["order"] = 8, ["angle"] = 4521984},
  [42] = {["x"] = -6420, ["y"] = -21, ["z"] = 250, ["order"] = 8, ["angle"] = 4521984},
  [43] = {["x"] = -5608, ["y"] = 3, ["z"] = 250, ["order"] = 8, ["angle"] = -195624960},
  [44] = {["x"] = -4500, ["y"] = -50, ["z"] = 250, ["order"] = 8, ["angle"] = -696451072},
  [45] = {["x"] = -3963, ["y"] = -656, ["z"] = 250, ["order"] = 9, ["angle"] = -1060175872},
  [46] = {["x"] = -3933, ["y"] = -1469, ["z"] = 250, ["order"] = 9, ["angle"] = -1057423360},
  [47] = {["x"] = -3955, ["y"] = -2447, ["z"] = 250, ["order"] = 9, ["angle"] = -881524736},
  [48] = {["x"] = -3763, ["y"] = -3101, ["z"] = 250, ["order"] = 10, ["angle"] = -631701504},
  [49] = {["x"] = -3221, ["y"] = -3715, ["z"] = 250, ["order"] = 10, ["angle"] = -424083456},
  [50] = {["x"] = -2560, ["y"] = -4430, ["z"] = 250, ["order"] = 10, ["angle"] = -191823872},
  [51] = {["x"] = -1921, ["y"] = -4784, ["z"] = 250, ["order"] = 11, ["angle"] = 176226304},
  [52] = {["x"] = -1319, ["y"] = -4668, ["z"] = 250, ["order"] = 11, ["angle"] = 371392512},
  [53] = {["x"] = -663, ["y"] = -4083, ["z"] = 250, ["order"] = 11, ["angle"] = 196411392},
  [54] = {["x"] = -14, ["y"] = -3730, ["z"] = 250, ["order"] = 11, ["angle"] = -336265216},
  [55] = {["x"] = 769, ["y"] = -3830, ["z"] = 250, ["order"] = 12, ["angle"] = -773914624},
  [56] = {["x"] = 1181, ["y"] = -4471, ["z"] = 262, ["order"] = 12, ["angle"] = -1284440064},
  [57] = {["x"] = 1107, ["y"] = -5360, ["z"] = 400, ["order"] = 12, ["angle"] = -1613430784},
  [58] = {["x"] = 675, ["y"] = -6121, ["z"] = 400, ["order"] = 12, ["angle"] = -1861025792},
  [59] = {["x"] = -107, ["y"] = -6782, ["z"] = 400, ["order"] = 12, ["angle"] = -1997340672},
  [60] = {["x"] = -961, ["y"] = -7169, ["z"] = 400, ["order"] = 13, ["angle"] = -2082013184},
  [61] = {["x"] = -1883, ["y"] = -7384, ["z"] = 230, ["order"] = 13, ["angle"] = -2082013184},
  [62] = {["x"] = -2552, ["y"] = -7505, ["z"] = 210, ["order"] = 13, ["angle"] = -2063794176},
  [63] = {["x"] = -3473, ["y"] = -7687, ["z"] = 210, ["order"] = 13, ["angle"] = -2071527424},
  [64] = {["x"] = -4332, ["y"] = -7846, ["z"] = 210, ["order"] = 13, ["angle"] = 2002976768},
  [65] = {["x"] = -5412, ["y"] = -7851, ["z"] = 230, ["order"] = 13, ["angle"] = 1860108288},
  [66] = {["x"] = -6375, ["y"] = -7727, ["z"] = 210, ["order"] = 13, ["angle"] = 1603469312},
  [67] = {["x"] = -6978, ["y"] = -7291, ["z"] = 210, ["order"] = 13, ["angle"] = 1122566144},
  [68] = {["x"] = -7076, ["y"] = -6446, ["z"] = 225, ["order"] = 14, ["angle"] = 1216937984},
  [69] = {["x"] = -7152, ["y"] = -5522, ["z"] = 250, ["order"] = 14, ["angle"] = 1876230144},
  [70] = {["x"] = -7782, ["y"] = -5047, ["z"] = 270, ["order"] = 14, ["angle"] = -2115436544},
  [71] = {["x"] = -8636, ["y"] = -4909, ["z"] = 410, ["order"] = 14, ["angle"] = -2099576832},
  [72] = {["x"] = -9645, ["y"] = -4873, ["z"] = 500, ["order"] = 15, ["angle"] = -1915551744},
  [73] = {["x"] = -10742, ["y"] = -5039, ["z"] = 630, ["order"] = 15, ["angle"] = -1432551424},
  [74] = {["x"] = -11368, ["y"] = -5661, ["z"] = 660, ["order"] = 15, ["angle"] = -1071054848},
  [75] = {["x"] = -11526, ["y"] = -6416, ["z"] = 685, ["order"] = 15, ["angle"] = -693305344},
  [76] = {["x"] = -11212, ["y"] = -7182, ["z"] = 700, ["order"] = 16, ["angle"] = -240189440},
  [77] = {["x"] = -10457, ["y"] = -7539, ["z"] = 760, ["order"] = 16, ["angle"] = 2031616},
  [78] = {["x"] = -9438, ["y"] = -7615, ["z"] = 926, ["order"] = 16, ["angle"] = -9633792},
  [79] = {["x"] = -8220, ["y"] = -7630, ["z"] = 1125, ["order"] = 16, ["angle"] = 59965440},
  [80] = {["x"] = -7232, ["y"] = -7603, ["z"] = 1200, ["order"] = 16, ["angle"] = -109117440},
  [81] = {["x"] = -6355, ["y"] = -7658, ["z"] = 1200, ["order"] = 16, ["angle"] = -54853632},
  [82] = {["x"] = -7389, ["y"] = -774, ["z"] = 250, ["order"] = 8, ["angle"] = -1064632320},
  [83] = {["x"] = -7435, ["y"] = -1517, ["z"] = 250, ["order"] = 8, ["angle"] = -958464000},
  [84] = {["x"] = -7469, ["y"] = -2238, ["z"] = 250, ["order"] = 8, ["angle"] = -785580032},
  [85] = {["x"] = -7326, ["y"] = -2931, ["z"] = 250, ["order"] = 9, ["angle"] = -181075968},
  [86] = {["x"] = -6559, ["y"] = -3217, ["z"] = 250, ["order"] = 9, ["angle"] = -83034112},
  [87] = {["x"] = -5666, ["y"] = -3201, ["z"] = 250, ["order"] = 9, ["angle"] = -10944512},
  [88] = {["x"] = -4937, ["y"] = -3175, ["z"] = 250, ["order"] = 9, ["angle"] = -208994304},
  [89] = {["x"] = -4118, ["y"] = -3341, ["z"] = 250, ["order"] = 10, ["angle"] = -360775680}
}
-- Respawn definitions for BLUE MOUNTAIN (MAP24)
krespawn.points[24] = {
  [0] = 93,
  [1] = {["x"] = 35, ["y"] = 4, ["z"] = 0, ["order"] = 0, ["angle"] = 1069875200},
  [2] = {["x"] = 32, ["y"] = 780, ["z"] = 0, ["order"] = 0, ["angle"] = 1069875200},
  [3] = {["x"] = 36, ["y"] = 1810, ["z"] = 0, ["order"] = 0, ["angle"] = 1069875200},
  [4] = {["x"] = 41, ["y"] = 2757, ["z"] = 0, ["order"] = 0, ["angle"] = 1069875200},
  [5] = {["x"] = 45, ["y"] = 3441, ["z"] = -145, ["order"] = 0, ["angle"] = 1069875200},
  [6] = {["x"] = 64, ["y"] = 7442, ["z"] = 2808, ["order"] = 0, ["angle"] = 1083113472},
  [7] = {["x"] = 72, ["y"] = 6799, ["z"] = 2796, ["order"] = 0, ["angle"] = 1082064896},
  [8] = {["x"] = 205, ["y"] = 8042, ["z"] = 2808, ["order"] = 0, ["angle"] = 1205534720},
  [9] = {["x"] = 249, ["y"] = 8894, ["z"] = 2808, ["order"] = 0, ["angle"] = 1302921216},
  [10] = {["x"] = 67, ["y"] = 9945, ["z"] = 2808, ["order"] = 0, ["angle"] = 1984888832},
  [11] = {["x"] = -1118, ["y"] = 10263, ["z"] = 2808, ["order"] = 0, ["angle"] = 2090795008},
  [12] = {["x"] = -2424, ["y"] = 10220, ["z"] = 2808, ["order"] = 0, ["angle"] = 1546321920},
  [13] = {["x"] = -3049, ["y"] = 10877, ["z"] = 2844, ["order"] = 1, ["angle"] = 803012608},
  [14] = {["x"] = -2616, ["y"] = 11557, ["z"] = 2811, ["order"] = 1, ["angle"] = 118947840},
  [15] = {["x"] = -1655, ["y"] = 11795, ["z"] = 2808, ["order"] = 1, ["angle"] = -79364096},
  [16] = {["x"] = -519, ["y"] = 11716, ["z"] = 2808, ["order"] = 1, ["angle"] = -147652608},
  [17] = {["x"] = 814, ["y"] = 11491, ["z"] = 2827, ["order"] = 2, ["angle"] = 313982976},
  [18] = {["x"] = 1707, ["y"] = 11571, ["z"] = 3098, ["order"] = 2, ["angle"] = 911802368},
  [19] = {["x"] = 2277, ["y"] = 12148, ["z"] = 3250, ["order"] = 2, ["angle"] = 1646329856},
  [20] = {["x"] = 2230, ["y"] = 12679, ["z"] = 3237, ["order"] = 2, ["angle"] = -2145058816},
  [21] = {["x"] = 1499, ["y"] = 12984, ["z"] = 3041, ["order"] = 2, ["angle"] = -2132344832},
  [22] = {["x"] = 592, ["y"] = 13017, ["z"] = 3048, ["order"] = 2, ["angle"] = 2142568448},
  [23] = {["x"] = 16, ["y"] = 12948, ["z"] = 3048, ["order"] = 2, ["angle"] = -2115698688},
  [24] = {["x"] = -578, ["y"] = 12924, ["z"] = 3048, ["order"] = 3, ["angle"] = -2145976320},
  [25] = {["x"] = -1427, ["y"] = 12875, ["z"] = 3048, ["order"] = 3, ["angle"] = -2141782016},
  [26] = {["x"] = -2430, ["y"] = 12846, ["z"] = 3048, ["order"] = 3, ["angle"] = -2141782016},
  [27] = {["x"] = -3907, ["y"] = 12898, ["z"] = 3048, ["order"] = 3, ["angle"] = -2102460416},
  [28] = {["x"] = -4675, ["y"] = 12749, ["z"] = 3048, ["order"] = 3, ["angle"] = 2001534976},
  [29] = {["x"] = -5367, ["y"] = 12866, ["z"] = 3048, ["order"] = 3, ["angle"] = 1646723072},
  [30] = {["x"] = -6260, ["y"] = 13350, ["z"] = 3073, ["order"] = 3, ["angle"] = 1139998720},
  [31] = {["x"] = -6527, ["y"] = 14097, ["z"] = 3066, ["order"] = 4, ["angle"] = 753729536},
  [32] = {["x"] = -6356, ["y"] = 15004, ["z"] = 3069, ["order"] = 4, ["angle"] = 449380352},
  [33] = {["x"] = -5822, ["y"] = 15691, ["z"] = 3072, ["order"] = 4, ["angle"] = -5570560},
  [34] = {["x"] = -4939, ["y"] = 15970, ["z"] = 3048, ["order"] = 4, ["angle"] = -72155136},
  [35] = {["x"] = -3783, ["y"] = 16009, ["z"] = 3048, ["order"] = 4, ["angle"] = -30343168},
  [36] = {["x"] = -2480, ["y"] = 15989, ["z"] = 3048, ["order"] = 5, ["angle"] = -3473408},
  [37] = {["x"] = -624, ["y"] = 15966, ["z"] = 2645, ["order"] = 5, ["angle"] = -3473408},
  [38] = {["x"] = 922, ["y"] = 15956, ["z"] = 2416, ["order"] = 5, ["angle"] = -3473408},
  [39] = {["x"] = 2053, ["y"] = 16030, ["z"] = 2198, ["order"] = 6, ["angle"] = 13959168},
  [40] = {["x"] = 3062, ["y"] = 15962, ["z"] = 2024, ["order"] = 6, ["angle"] = 255787008},
  [41] = {["x"] = 4130, ["y"] = 15898, ["z"] = 2147, ["order"] = 6, ["angle"] = 971964416},
  [42] = {["x"] = 4825, ["y"] = 16487, ["z"] = 2285, ["order"] = 6, ["angle"] = 1589313536},
  [43] = {["x"] = 4830, ["y"] = 17158, ["z"] = 2286, ["order"] = 6, ["angle"] = 2022375424},
  [44] = {["x"] = 4201, ["y"] = 17318, ["z"] = 2161, ["order"] = 6, ["angle"] = 2009530368},
  [45] = {["x"] = 3405, ["y"] = 17220, ["z"] = 2024, ["order"] = 7, ["angle"] = 2012676096},
  [46] = {["x"] = 2469, ["y"] = 17178, ["z"] = 2024, ["order"] = 7, ["angle"] = 1900478464},
  [47] = {["x"] = 1390, ["y"] = 17191, ["z"] = 2114, ["order"] = 7, ["angle"] = 1657733120},
  [48] = {["x"] = 649, ["y"] = 17794, ["z"] = 2104, ["order"] = 7, ["angle"] = 1259012096},
  [49] = {["x"] = 607, ["y"] = 18583, ["z"] = 2068, ["order"] = 8, ["angle"] = 669188096},
  [50] = {["x"] = 1335, ["y"] = 18877, ["z"] = 2024, ["order"] = 8, ["angle"] = -62717952},
  [51] = {["x"] = 2301, ["y"] = 18874, ["z"] = 2024, ["order"] = 8, ["angle"] = -1507328},
  [52] = {["x"] = 3161, ["y"] = 18852, ["z"] = 2024, ["order"] = 8, ["angle"] = -1507328},
  [53] = {["x"] = 3577, ["y"] = 18844, ["z"] = 2014, ["order"] = 8, ["angle"] = -1507328},
  [54] = {["x"] = 4205, ["y"] = 18872, ["z"] = 1877, ["order"] = 8, ["angle"] = 27852800},
  [55] = {["x"] = 5047, ["y"] = 18970, ["z"] = 1711, ["order"] = 8, ["angle"] = -144637952},
  [56] = {["x"] = 5934, ["y"] = 18908, ["z"] = 1600, ["order"] = 8, ["angle"] = -436011008},
  [57] = {["x"] = 6751, ["y"] = 18482, ["z"] = 1600, ["order"] = 8, ["angle"] = -725942272},
  [58] = {["x"] = 7139, ["y"] = 17626, ["z"] = 1600, ["order"] = 9, ["angle"] = -1105526784},
  [59] = {["x"] = 7175, ["y"] = 15950, ["z"] = 1488, ["order"] = 9, ["angle"] = -1082195968},
  [60] = {["x"] = 7182, ["y"] = 15344, ["z"] = 1488, ["order"] = 9, ["angle"] = -1082195968},
  [61] = {["x"] = 7189, ["y"] = 14211, ["z"] = 1376, ["order"] = 9, ["angle"] = -1082195968},
  [62] = {["x"] = 7183, ["y"] = 12814, ["z"] = 1272, ["order"] = 9, ["angle"] = -1082195968},
  [63] = {["x"] = 7201, ["y"] = 11656, ["z"] = 1250, ["order"] = 9, ["angle"] = -1195048960},
  [64] = {["x"] = 7165, ["y"] = 10332, ["z"] = 1160, ["order"] = 9, ["angle"] = -1599799296},
  [65] = {["x"] = 6595, ["y"] = 9480, ["z"] = 1160, ["order"] = 9, ["angle"] = -1980039168},
  [66] = {["x"] = 5523, ["y"] = 9372, ["z"] = 1160, ["order"] = 9, ["angle"] = -2041643008},
  [67] = {["x"] = 4255, ["y"] = 9511, ["z"] = 1007, ["order"] = 10, ["angle"] = -1865613312},
  [68] = {["x"] = 3279, ["y"] = 9424, ["z"] = 974, ["order"] = 10, ["angle"] = -1329266688},
  [69] = {["x"] = 2834, ["y"] = 8517, ["z"] = 960, ["order"] = 10, ["angle"] = -1113522176},
  [70] = {["x"] = 2999, ["y"] = 6915, ["z"] = 960, ["order"] = 10, ["angle"] = -1071316992},
  [71] = {["x"] = 3101, ["y"] = 4993, ["z"] = 952, ["order"] = 10, ["angle"] = -1071316992},
  [72] = {["x"] = 3117, ["y"] = 4198, ["z"] = 952, ["order"] = 10, ["angle"] = -1071316992},
  [73] = {["x"] = 3129, ["y"] = 3302, ["z"] = 952, ["order"] = 10, ["angle"] = -1071316992},
  [74] = {["x"] = 3121, ["y"] = 1792, ["z"] = 1002, ["order"] = 11, ["angle"] = -1178271744},
  [75] = {["x"] = 3019, ["y"] = 1035, ["z"] = 853, ["order"] = 11, ["angle"] = -1164378112},
  [76] = {["x"] = 2874, ["y"] = -156, ["z"] = 631, ["order"] = 11, ["angle"] = -1211301888},
  [77] = {["x"] = 2752, ["y"] = -1728, ["z"] = 125, ["order"] = 11, ["angle"] = -1128333312},
  [78] = {["x"] = 2740, ["y"] = -3277, ["z"] = -64, ["order"] = 11, ["angle"] = -1152450560},
  [79] = {["x"] = 2510, ["y"] = -4544, ["z"] = 237, ["order"] = 12, ["angle"] = -1613561856},
  [80] = {["x"] = 1483, ["y"] = -5368, ["z"] = 514, ["order"] = 12, ["angle"] = 2037055488},
  [81] = {["x"] = 431, ["y"] = -5121, ["z"] = 431, ["order"] = 12, ["angle"] = 1444216832},
  [82] = {["x"] = -47, ["y"] = -3996, ["z"] = 53, ["order"] = 12, ["angle"] = 1031602176},
  [83] = {["x"] = -20, ["y"] = -2827, ["z"] = 0, ["order"] = 12, ["angle"] = 1031602176},
  [84] = {["x"] = 20, ["y"] = -2145, ["z"] = 0, ["order"] = 12, ["angle"] = 1031602176},
  [85] = {["x"] = 38, ["y"] = -1406, ["z"] = 0, ["order"] = 12, ["angle"] = 1076953088},
  [86] = {["x"] = 38, ["y"] = -488, ["z"] = 0, ["order"] = 12, ["angle"] = 1076953088},
  [87] = {["x"] = 2336, ["y"] = 7103, ["z"] = 960, ["order"] = 10, ["angle"] = -1074331648},
  [88] = {["x"] = 2236, ["y"] = 4872, ["z"] = 952, ["order"] = 10, ["angle"] = -1086783488},
  [89] = {["x"] = 2248, ["y"] = 3194, ["z"] = 952, ["order"] = 10, ["angle"] = -1030553600},
  [90] = {["x"] = 2293, ["y"] = 1609, ["z"] = 949, ["order"] = 11, ["angle"] = -1076822016},
  [91] = {["x"] = 2313, ["y"] = 858, ["z"] = 798, ["order"] = 11, ["angle"] = -1029505024},
  [92] = {["x"] = 2356, ["y"] = -216, ["z"] = 623, ["order"] = 11, ["angle"] = -1000931328},
  [93] = {["x"] = 2451, ["y"] = -1985, ["z"] = 37, ["order"] = 11, ["angle"] = -1012858880}
}
-- Respawn definitions for PETROLEUM REFINERY (MAP25)
krespawn.points[25] = {
  [0] = 116,
  [1] = {["x"] = 3060, ["y"] = -2031, ["z"] = 0, ["order"] = 0, ["angle"] = -20250624},
  [2] = {["x"] = 4227, ["y"] = -1987, ["z"] = 0, ["order"] = 0, ["angle"] = -52756480},
  [3] = {["x"] = 5328, ["y"] = -1962, ["z"] = 0, ["order"] = 0, ["angle"] = -225509376},
  [4] = {["x"] = 6446, ["y"] = -2177, ["z"] = 0, ["order"] = 0, ["angle"] = -590675968},
  [5] = {["x"] = 7315, ["y"] = -2938, ["z"] = 0, ["order"] = 1, ["angle"] = -859766784},
  [6] = {["x"] = 7639, ["y"] = -3827, ["z"] = 0, ["order"] = 1, ["angle"] = -1112342528},
  [7] = {["x"] = 7610, ["y"] = -4886, ["z"] = 0, ["order"] = 1, ["angle"] = -1071316992},
  [8] = {["x"] = 7510, ["y"] = -6195, ["z"] = 0, ["order"] = 1, ["angle"] = -1034616832},
  [9] = {["x"] = 7505, ["y"] = -7491, ["z"] = 0, ["order"] = 1, ["angle"] = -927137792},
  [10] = {["x"] = 7486, ["y"] = -8689, ["z"] = 0, ["order"] = 1, ["angle"] = -798687232},
  [11] = {["x"] = 7734, ["y"] = -9879, ["z"] = 0, ["order"] = 2, ["angle"] = -521076736},
  [12] = {["x"] = 8320, ["y"] = -10590, ["z"] = 0, ["order"] = 2, ["angle"] = -201392128},
  [13] = {["x"] = 9369, ["y"] = -10915, ["z"] = 26, ["order"] = 2, ["angle"] = 47906816},
  [14] = {["x"] = 10310, ["y"] = -11030, ["z"] = 183, ["order"] = 3, ["angle"] = 285409280},
  [15] = {["x"] = 11258, ["y"] = -10864, ["z"] = 256, ["order"] = 3, ["angle"] = 671547392},
  [16] = {["x"] = 11869, ["y"] = -10247, ["z"] = 256, ["order"] = 3, ["angle"] = 995688448},
  [17] = {["x"] = 12162, ["y"] = -9296, ["z"] = 256, ["order"] = 4, ["angle"] = 1116274688},
  [18] = {["x"] = 12189, ["y"] = -8171, ["z"] = 365, ["order"] = 4, ["angle"] = 1086652416},
  [19] = {["x"] = 12188, ["y"] = -6942, ["z"] = 609, ["order"] = 4, ["angle"] = 1022951424},
  [20] = {["x"] = 12234, ["y"] = -5602, ["z"] = 875, ["order"] = 4, ["angle"] = 1312358400},
  [21] = {["x"] = 12267, ["y"] = -4649, ["z"] = 1024, ["order"] = 4, ["angle"] = 1543831552},
  [22] = {["x"] = 11563, ["y"] = -3789, ["z"] = 1024, ["order"] = 5, ["angle"] = 1819869184},
  [23] = {["x"] = 10244, ["y"] = -3459, ["z"] = 1024, ["order"] = 5, ["angle"] = 1659699200},
  [24] = {["x"] = 9021, ["y"] = -2825, ["z"] = 1082, ["order"] = 5, ["angle"] = 1598750720},
  [25] = {["x"] = 8192, ["y"] = -1950, ["z"] = 1119, ["order"] = 6, ["angle"] = 1441988608},
  [26] = {["x"] = 7570, ["y"] = -640, ["z"] = 1024, ["order"] = 6, ["angle"] = 1447231488},
  [27] = {["x"] = 7036, ["y"] = 480, ["z"] = 1024, ["order"] = 6, ["angle"] = 1855258624},
  [28] = {["x"] = 6146, ["y"] = 981, ["z"] = 1024, ["order"] = 6, ["angle"] = 2100756480},
  [29] = {["x"] = 5118, ["y"] = 1138, ["z"] = 768, ["order"] = 7, ["angle"] = -2145714176},
  [30] = {["x"] = 4404, ["y"] = 1166, ["z"] = 768, ["order"] = 7, ["angle"] = 2131558400},
  [31] = {["x"] = 2973, ["y"] = 1166, ["z"] = 768, ["order"] = 7, ["angle"] = -2138243072},
  [32] = {["x"] = 1678, ["y"] = 1140, ["z"] = 768, ["order"] = 7, ["angle"] = -2138243072},
  [33] = {["x"] = 243, ["y"] = 1051, ["z"] = 768, ["order"] = 7, ["angle"] = 2027880448},
  [34] = {["x"] = -608, ["y"] = 1086, ["z"] = 768, ["order"] = 7, ["angle"] = 1099759616},
  [35] = {["x"] = -654, ["y"] = 2308, ["z"] = 768, ["order"] = 8, ["angle"] = 1099759616},
  [36] = {["x"] = -794, ["y"] = 4074, ["z"] = 768, ["order"] = 9, ["angle"] = 1452474368},
  [37] = {["x"] = -1448, ["y"] = 5089, ["z"] = 768, ["order"] = 9, ["angle"] = 1765867520},
  [38] = {["x"] = -2576, ["y"] = 5703, ["z"] = 768, ["order"] = 9, ["angle"] = -2044395520},
  [39] = {["x"] = -3763, ["y"] = 5662, ["z"] = 768, ["order"] = 10, ["angle"] = -1731657728},
  [40] = {["x"] = -4824, ["y"] = 4953, ["z"] = 768, ["order"] = 10, ["angle"] = -1316683776},
  [41] = {["x"] = -5332, ["y"] = 4091, ["z"] = 768, ["order"] = 10, ["angle"] = -1096351744},
  [42] = {["x"] = -5384, ["y"] = 3163, ["z"] = 768, ["order"] = 10, ["angle"] = -1105920000},
  [43] = {["x"] = -5544, ["y"] = 1783, ["z"] = 768, ["order"] = 10, ["angle"] = -1067909120},
  [44] = {["x"] = -5232, ["y"] = 1801, ["z"] = 768, ["order"] = 10, ["angle"] = -1020067840},
  [45] = {["x"] = -5274, ["y"] = -2367, ["z"] = 768, ["order"] = 11, ["angle"] = -1110245376},
  [46] = {["x"] = -5289, ["y"] = -3299, ["z"] = 768, ["order"] = 11, ["angle"] = -1127022592},
  [47] = {["x"] = -5247, ["y"] = -4408, ["z"] = 768, ["order"] = 11, ["angle"] = -1202257920},
  [48] = {["x"] = -5339, ["y"] = -5727, ["z"] = 768, ["order"] = 11, ["angle"] = -1424818176},
  [49] = {["x"] = -5866, ["y"] = -6753, ["z"] = 768, ["order"] = 11, ["angle"] = -1930887168},
  [50] = {["x"] = -6908, ["y"] = -7247, ["z"] = 768, ["order"] = 11, ["angle"] = 2116222976},
  [51] = {["x"] = -8004, ["y"] = -7367, ["z"] = 768, ["order"] = 11, ["angle"] = 1999306752},
  [52] = {["x"] = -9275, ["y"] = -7294, ["z"] = 768, ["order"] = 12, ["angle"] = 1740832768},
  [53] = {["x"] = -10244, ["y"] = -6607, ["z"] = 768, ["order"] = 12, ["angle"] = 1614610432},
  [54] = {["x"] = -11145, ["y"] = -5550, ["z"] = 768, ["order"] = 12, ["angle"] = 1664417792},
  [55] = {["x"] = -12026, ["y"] = -4612, ["z"] = 768, ["order"] = 12, ["angle"] = 1886191616},
  [56] = {["x"] = -12730, ["y"] = -4226, ["z"] = 768, ["order"] = 12, ["angle"] = -2107441152},
  [57] = {["x"] = -13989, ["y"] = -4205, ["z"] = 768, ["order"] = 12, ["angle"] = 2135359488},
  [58] = {["x"] = -15883, ["y"] = -4246, ["z"] = 317, ["order"] = 12, ["angle"] = 2137063424},
  [59] = {["x"] = -16798, ["y"] = -4165, ["z"] = 384, ["order"] = 12, ["angle"] = -1926430720},
  [60] = {["x"] = -17811, ["y"] = -4477, ["z"] = 384, ["order"] = 13, ["angle"] = -1528496128},
  [61] = {["x"] = -18379, ["y"] = -5277, ["z"] = 384, ["order"] = 13, ["angle"] = -1219559424},
  [62] = {["x"] = -18536, ["y"] = -6416, ["z"] = 384, ["order"] = 13, ["angle"] = -1024917504},
  [63] = {["x"] = -18513, ["y"] = -7671, ["z"] = 384, ["order"] = 13, ["angle"] = -1068171264},
  [64] = {["x"] = -18490, ["y"] = -8921, ["z"] = 384, ["order"] = 13, ["angle"] = -635502592},
  [65] = {["x"] = -17872, ["y"] = -10056, ["z"] = 384, ["order"] = 13, ["angle"] = -229441536},
  [66] = {["x"] = -17019, ["y"] = -10409, ["z"] = 414, ["order"] = 13, ["angle"] = -47382528},
  [67] = {["x"] = -15980, ["y"] = -10415, ["z"] = 686, ["order"] = 13, ["angle"] = -61538304},
  [68] = {["x"] = -14612, ["y"] = -10337, ["z"] = 768, ["order"] = 13, ["angle"] = -64946176},
  [69] = {["x"] = -13468, ["y"] = -10370, ["z"] = 768, ["order"] = 15, ["angle"] = -207814656},
  [70] = {["x"] = -12250, ["y"] = -10414, ["z"] = 768, ["order"] = 15, ["angle"] = -432472064},
  [71] = {["x"] = -11160, ["y"] = -10764, ["z"] = 653, ["order"] = 15, ["angle"] = -763035648},
  [72] = {["x"] = -10690, ["y"] = -11755, ["z"] = 591, ["order"] = 15, ["angle"] = -637992960},
  [73] = {["x"] = -10381, ["y"] = -12748, ["z"] = 550, ["order"] = 15, ["angle"] = -341114880},
  [74] = {["x"] = -9813, ["y"] = -13434, ["z"] = 474, ["order"] = 15, ["angle"] = -95092736},
  [75] = {["x"] = -9060, ["y"] = -13747, ["z"] = 373, ["order"] = 15, ["angle"] = 36372480},
  [76] = {["x"] = -7809, ["y"] = -13870, ["z"] = 207, ["order"] = 15, ["angle"] = 34144256},
  [77] = {["x"] = -6477, ["y"] = -13847, ["z"] = 29, ["order"] = 15, ["angle"] = 34144256},
  [78] = {["x"] = -5396, ["y"] = -13802, ["z"] = 0, ["order"] = 16, ["angle"] = 34144256},
  [79] = {["x"] = -4072, ["y"] = -13854, ["z"] = 0, ["order"] = 16, ["angle"] = 247136256},
  [80] = {["x"] = -3194, ["y"] = -13864, ["z"] = 0, ["order"] = 16, ["angle"] = 564854784},
  [81] = {["x"] = -2217, ["y"] = -13417, ["z"] = 0, ["order"] = 16, ["angle"] = 906559488},
  [82] = {["x"] = -1687, ["y"] = -12499, ["z"] = 0, ["order"] = 16, ["angle"] = 1107099648},
  [83] = {["x"] = -1641, ["y"] = -8816, ["z"] = 0, ["order"] = 16, ["angle"] = 1078001664},
  [84] = {["x"] = -1644, ["y"] = -8065, ["z"] = 0, ["order"] = 16, ["angle"] = 1078001664},
  [85] = {["x"] = -1650, ["y"] = -7129, ["z"] = 0, ["order"] = 16, ["angle"] = 1078001664},
  [86] = {["x"] = -1656, ["y"] = -6096, ["z"] = 0, ["order"] = 16, ["angle"] = 1078001664},
  [87] = {["x"] = -1662, ["y"] = -5073, ["z"] = 0, ["order"] = 16, ["angle"] = 1078001664},
  [88] = {["x"] = -1696, ["y"] = -3907, ["z"] = 27, ["order"] = 17, ["angle"] = 953352192},
  [89] = {["x"] = -1201, ["y"] = -2738, ["z"] = 82, ["order"] = 17, ["angle"] = 373751808},
  [90] = {["x"] = -223, ["y"] = -1994, ["z"] = 125, ["order"] = 17, ["angle"] = -47120384},
  [91] = {["x"] = 557, ["y"] = -2175, ["z"] = 0, ["order"] = 17, ["angle"] = 139657216},
  [92] = {["x"] = 1586, ["y"] = -2055, ["z"] = 0, ["order"] = 17, ["angle"] = -10813440},
  [93] = {["x"] = 2521, ["y"] = -2026, ["z"] = 0, ["order"] = 17, ["angle"] = -10813440},
  [94] = {["x"] = -5359, ["y"] = -147, ["z"] = 0, ["order"] = 10, ["angle"] = -1094647808},
  [95] = {["x"] = -5381, ["y"] = -876, ["z"] = 0, ["order"] = 10, ["angle"] = -1094647808},
  [96] = {["x"] = -5414, ["y"] = -1954, ["z"] = 0, ["order"] = 10, ["angle"] = -1094647808},
  [97] = {["x"] = -5392, ["y"] = -2958, ["z"] = 0, ["order"] = 10, ["angle"] = -1029242880},
  [98] = {["x"] = -5299, ["y"] = -4204, ["z"] = 0, ["order"] = 10, ["angle"] = -1201340416},
  [99] = {["x"] = -5367, ["y"] = -5604, ["z"] = 0, ["order"] = 10, ["angle"] = -1377370112},
  [100] = {["x"] = -5774, ["y"] = -6628, ["z"] = 0, ["order"] = 10, ["angle"] = -1728905216},
  [101] = {["x"] = -6429, ["y"] = -7385, ["z"] = 0, ["order"] = 10, ["angle"] = -1748041728},
  [102] = {["x"] = -7267, ["y"] = -8164, ["z"] = 0, ["order"] = 10, ["angle"] = -1782775808},
  [103] = {["x"] = -8347, ["y"] = -8611, ["z"] = 0, ["order"] = 12, ["angle"] = 2141126656},
  [104] = {["x"] = -10018, ["y"] = -8644, ["z"] = 29, ["order"] = 12, ["angle"] = 2123825152},
  [105] = {["x"] = -12080, ["y"] = -8577, ["z"] = 43, ["order"] = 12, ["angle"] = 2123825152},
  [106] = {["x"] = -13091, ["y"] = -8660, ["z"] = 0, ["order"] = 12, ["angle"] = 1900871680},
  [107] = {["x"] = -14063, ["y"] = -8243, ["z"] = 39, ["order"] = 12, ["angle"] = 1439891456},
  [108] = {["x"] = -14714, ["y"] = -7312, ["z"] = 138, ["order"] = 12, ["angle"] = 1146028032},
  [109] = {["x"] = -14846, ["y"] = -5975, ["z"] = 158, ["order"] = 12, ["angle"] = 1334902784},
  [110] = {["x"] = -15282, ["y"] = -4882, ["z"] = 225, ["order"] = 12, ["angle"] = 1770586112},
  [111] = {["x"] = -18732, ["y"] = -4967, ["z"] = 384, ["order"] = 13, ["angle"] = -1117585408},
  [112] = {["x"] = -18843, ["y"] = -5802, ["z"] = 384, ["order"] = 13, ["angle"] = -1093206016},
  [113] = {["x"] = -18833, ["y"] = -6676, ["z"] = 384, ["order"] = 13, ["angle"] = -1060700160},
  [114] = {["x"] = -18817, ["y"] = -7762, ["z"] = 384, ["order"] = 14, ["angle"] = -1060700160},
  [115] = {["x"] = -18896, ["y"] = -9238, ["z"] = 384, ["order"] = 14, ["angle"] = -674168832},
  [116] = {["x"] = -18280, ["y"] = -10199, ["z"] = 384, ["order"] = 14, ["angle"] = -203096064}
}
-- Respawn definitions for DESERT PALACE (MAP26)
krespawn.points[26] = {
  [0] = 39,
  [1] = {["x"] = -3210, ["y"] = 25, ["z"] = 0, ["order"] = 0, ["angle"] = 1048903680},
  [2] = {["x"] = -3208, ["y"] = 964, ["z"] = 10, ["order"] = 0, ["angle"] = 1109721088},
  [3] = {["x"] = -3287, ["y"] = 1769, ["z"] = 69, ["order"] = 0, ["angle"] = 1196490752},
  [4] = {["x"] = -3539, ["y"] = 2918, ["z"] = 144, ["order"] = 1, ["angle"] = 458555392},
  [5] = {["x"] = -3005, ["y"] = 3509, ["z"] = 144, ["order"] = 1, ["angle"] = 1114112},
  [6] = {["x"] = -1844, ["y"] = 3604, ["z"] = 144, ["order"] = 2, ["angle"] = -272957440},
  [7] = {["x"] = -431, ["y"] = 2988, ["z"] = -210, ["order"] = 2, ["angle"] = -613089280},
  [8] = {["x"] = 335, ["y"] = 2125, ["z"] = -384, ["order"] = 3, ["angle"] = -592904192},
  [9] = {["x"] = 1050, ["y"] = 1254, ["z"] = -384, ["order"] = 3, ["angle"] = -108855296},
  [10] = {["x"] = 1975, ["y"] = 976, ["z"] = -384, ["order"] = 3, ["angle"] = 334168064},
  [11] = {["x"] = 2937, ["y"] = 1235, ["z"] = -384, ["order"] = 4, ["angle"] = 830275584},
  [12] = {["x"] = 3426, ["y"] = 2014, ["z"] = -295, ["order"] = 4, ["angle"] = 1412759552},
  [13] = {["x"] = 3092, ["y"] = 2837, ["z"] = -165, ["order"] = 4, ["angle"] = 1971650560},
  [14] = {["x"] = 2146, ["y"] = 2975, ["z"] = -144, ["order"] = 4, ["angle"] = -1251016704},
  [15] = {["x"] = 1873, ["y"] = 2310, ["z"] = -192, ["order"] = 5, ["angle"] = -1074069504},
  [16] = {["x"] = 1826, ["y"] = 1305, ["z"] = -192, ["order"] = 5, ["angle"] = -1074069504},
  [17] = {["x"] = 1856, ["y"] = 160, ["z"] = -192, ["order"] = 6, ["angle"] = -954793984},
  [18] = {["x"] = 2108, ["y"] = -735, ["z"] = -192, ["order"] = 6, ["angle"] = -773390336},
  [19] = {["x"] = 2470, ["y"] = -1574, ["z"] = -192, ["order"] = 6, ["angle"] = -1130168320},
  [20] = {["x"] = 2566, ["y"] = -2478, ["z"] = -192, ["order"] = 6, ["angle"] = -1639251968},
  [21] = {["x"] = 2010, ["y"] = -3259, ["z"] = -192, ["order"] = 6, ["angle"] = -2091057152},
  [22] = {["x"] = 1062, ["y"] = -3524, ["z"] = -109, ["order"] = 6, ["angle"] = 1812267008},
  [23] = {["x"] = 181, ["y"] = -3126, ["z"] = -64, ["order"] = 6, ["angle"] = 1138294784},
  [24] = {["x"] = 67, ["y"] = -2288, ["z"] = -131, ["order"] = 6, ["angle"] = 871170048},
  [25] = {["x"] = 501, ["y"] = -1376, ["z"] = -192, ["order"] = 6, ["angle"] = 1183776768},
  [26] = {["x"] = 467, ["y"] = -647, ["z"] = -192, ["order"] = 7, ["angle"] = 1817378816},
  [27] = {["x"] = -154, ["y"] = -213, ["z"] = -233, ["order"] = 7, ["angle"] = -1846214656},
  [28] = {["x"] = -1092, ["y"] = -287, ["z"] = -384, ["order"] = 7, ["angle"] = -1086914560},
  [29] = {["x"] = -1284, ["y"] = -1085, ["z"] = -384, ["order"] = 8, ["angle"] = -1047986176},
  [30] = {["x"] = -1178, ["y"] = -1961, ["z"] = -179, ["order"] = 8, ["angle"] = -1133707264},
  [31] = {["x"] = -1299, ["y"] = -2766, ["z"] = 61, ["order"] = 8, ["angle"] = -1576075264},
  [32] = {["x"] = -1754, ["y"] = -3297, ["z"] = 221, ["order"] = 8, ["angle"] = 2034302976},
  [33] = {["x"] = -2699, ["y"] = -3464, ["z"] = 271, ["order"] = 8, ["angle"] = 1380515840},
  [34] = {["x"] = -3116, ["y"] = -2730, ["z"] = 72, ["order"] = 9, ["angle"] = 1047592960},
  [35] = {["x"] = -3171, ["y"] = -1830, ["z"] = 121, ["order"] = 9, ["angle"] = 1047592960},
  [36] = {["x"] = -3182, ["y"] = -733, ["z"] = 0, ["order"] = 9, ["angle"] = 1086259200},
  [37] = {["x"] = 3346, ["y"] = -2052, ["z"] = -128, ["order"] = 6, ["angle"] = -937361408},
  [38] = {["x"] = 3572, ["y"] = -2861, ["z"] = -128, ["order"] = 6, ["angle"] = -1593901056},
  [39] = {["x"] = 2773, ["y"] = -3396, ["z"] = -128, ["order"] = 6, ["angle"] = 2119630848}
}
-- Respawn definitions for AURORA ATOLL (MAP27)
krespawn.points[27] = {
  [0] = 95,
  [1] = {["x"] = -1390, ["y"] = 1097, ["z"] = 256, ["order"] = 0, ["angle"] = 1076559872},
  [2] = {["x"] = -1372, ["y"] = 2177, ["z"] = 256, ["order"] = 0, ["angle"] = 1338179584},
  [3] = {["x"] = -1750, ["y"] = 3182, ["z"] = 256, ["order"] = 0, ["angle"] = 1478950912},
  [4] = {["x"] = -2290, ["y"] = 3932, ["z"] = 256, ["order"] = 1, ["angle"] = 1449459712},
  [5] = {["x"] = -3092, ["y"] = 4909, ["z"] = 256, ["order"] = 1, ["angle"] = 866451456},
  [6] = {["x"] = -3053, ["y"] = 5650, ["z"] = 256, ["order"] = 1, ["angle"] = 254869504},
  [7] = {["x"] = -2345, ["y"] = 5887, ["z"] = 256, ["order"] = 1, ["angle"] = 53149696},
  [8] = {["x"] = -1320, ["y"] = 5857, ["z"] = 256, ["order"] = 1, ["angle"] = 211877888},
  [9] = {["x"] = -486, ["y"] = 6160, ["z"] = 215, ["order"] = 1, ["angle"] = 764739584},
  [10] = {["x"] = -210, ["y"] = 6962, ["z"] = 198, ["order"] = 1, ["angle"] = 1009844224},
  [11] = {["x"] = -228, ["y"] = 7911, ["z"] = 199, ["order"] = 1, ["angle"] = 609550336},
  [12] = {["x"] = 322, ["y"] = 8684, ["z"] = 164, ["order"] = 1, ["angle"] = 193527808},
  [13] = {["x"] = 1265, ["y"] = 8831, ["z"] = 128, ["order"] = 1, ["angle"] = 185008128},
  [14] = {["x"] = 2175, ["y"] = 8909, ["z"] = 176, ["order"] = 1, ["angle"] = 669581312},
  [15] = {["x"] = 2770, ["y"] = 9515, ["z"] = 214, ["order"] = 1, ["angle"] = 1082064896},
  [16] = {["x"] = 2855, ["y"] = 10554, ["z"] = 256, ["order"] = 2, ["angle"] = 1065287680},
  [17] = {["x"] = 2774, ["y"] = 11608, ["z"] = 256, ["order"] = 2, ["angle"] = 877985792},
  [18] = {["x"] = 2957, ["y"] = 12734, ["z"] = 256, ["order"] = 2, ["angle"] = 568918016},
  [19] = {["x"] = 3486, ["y"] = 13516, ["z"] = 256, ["order"] = 2, ["angle"] = 54329344},
  [20] = {["x"] = 4436, ["y"] = 13774, ["z"] = 256, ["order"] = 2, ["angle"] = -131399680},
  [21] = {["x"] = 5567, ["y"] = 13726, ["z"] = 256, ["order"] = 2, ["angle"] = -164560896},
  [22] = {["x"] = 6737, ["y"] = 13348, ["z"] = 256, ["order"] = 3, ["angle"] = -297992192},
  [23] = {["x"] = 7689, ["y"] = 13043, ["z"] = 256, ["order"] = 3, ["angle"] = -52625408},
  [24] = {["x"] = 8853, ["y"] = 12963, ["z"] = 256, ["order"] = 3, ["angle"] = -87097344},
  [25] = {["x"] = 9971, ["y"] = 12739, ["z"] = 256, ["order"] = 4, ["angle"] = -348192768},
  [26] = {["x"] = 10966, ["y"] = 12257, ["z"] = 256, ["order"] = 4, ["angle"] = -38469632},
  [27] = {["x"] = 12144, ["y"] = 12187, ["z"] = 295, ["order"] = 4, ["angle"] = 18284544},
  [28] = {["x"] = 15893, ["y"] = 12141, ["z"] = 368, ["order"] = 4, ["angle"] = 312541184},
  [29] = {["x"] = 16974, ["y"] = 12254, ["z"] = 362, ["order"] = 4, ["angle"] = 498925568},
  [30] = {["x"] = 17495, ["y"] = 12807, ["z"] = 337, ["order"] = 4, ["angle"] = 1005780992},
  [31] = {["x"] = 17757, ["y"] = 13673, ["z"] = 298, ["order"] = 4, ["angle"] = 1342636032},
  [32] = {["x"] = 17390, ["y"] = 14627, ["z"] = 256, ["order"] = 5, ["angle"] = 2012282880},
  [33] = {["x"] = 16442, ["y"] = 15067, ["z"] = 256, ["order"] = 5, ["angle"] = -1833369600},
  [34] = {["x"] = 15492, ["y"] = 14840, ["z"] = 256, ["order"] = 5, ["angle"] = -1320747008},
  [35] = {["x"] = 15039, ["y"] = 13819, ["z"] = 256, ["order"] = 6, ["angle"] = -1122566144},
  [36] = {["x"] = 15021, ["y"] = 12703, ["z"] = 256, ["order"] = 6, ["angle"] = -1077870592},
  [37] = {["x"] = 14981, ["y"] = 11140, ["z"] = 256, ["order"] = 6, ["angle"] = -1194131456},
  [38] = {["x"] = 14766, ["y"] = 10027, ["z"] = 256, ["order"] = 6, ["angle"] = -1279328256},
  [39] = {["x"] = 14352, ["y"] = 8822, ["z"] = 256, ["order"] = 6, ["angle"] = -1269497856},
  [40] = {["x"] = 14121, ["y"] = 7347, ["z"] = 256, ["order"] = 6, ["angle"] = -1031340032},
  [41] = {["x"] = 14204, ["y"] = 6090, ["z"] = 256, ["order"] = 7, ["angle"] = -1064370176},
  [42] = {["x"] = 14312, ["y"] = 4839, ["z"] = 256, ["order"] = 7, ["angle"] = -1167392768},
  [43] = {["x"] = 14275, ["y"] = 3337, ["z"] = 256, ["order"] = 7, ["angle"] = -1213267968},
  [44] = {["x"] = 14099, ["y"] = 2146, ["z"] = 256, ["order"] = 7, ["angle"] = -1182466048},
  [45] = {["x"] = 13898, ["y"] = 1064, ["z"] = 316, ["order"] = 8, ["angle"] = -1600454656},
  [46] = {["x"] = 13352, ["y"] = 315, ["z"] = 410, ["order"] = 8, ["angle"] = -2109800448},
  [47] = {["x"] = 12364, ["y"] = 258, ["z"] = 408, ["order"] = 8, ["angle"] = -2056847360},
  [48] = {["x"] = 11361, ["y"] = 319, ["z"] = 384, ["order"] = 8, ["angle"] = -1770586112},
  [49] = {["x"] = 10470, ["y"] = 18, ["z"] = 384, ["order"] = 8, ["angle"] = -1257308160},
  [50] = {["x"] = 10196, ["y"] = -741, ["z"] = 384, ["order"] = 8, ["angle"] = -1113784320},
  [51] = {["x"] = 10205, ["y"] = -1581, ["z"] = 384, ["order"] = 8, ["angle"] = -1148649472},
  [52] = {["x"] = 10232, ["y"] = -2517, ["z"] = 384, ["order"] = 8, ["angle"] = -1336737792},
  [53] = {["x"] = 10019, ["y"] = -3511, ["z"] = 384, ["order"] = 9, ["angle"] = -1810563072},
  [54] = {["x"] = 9224, ["y"] = -3840, ["z"] = 384, ["order"] = 9, ["angle"] = -2049245184},
  [55] = {["x"] = 8291, ["y"] = -4071, ["z"] = 384, ["order"] = 9, ["angle"] = -1812660224},
  [56] = {["x"] = 7311, ["y"] = -4666, ["z"] = 384, ["order"] = 9, ["angle"] = -1836908544},
  [57] = {["x"] = 6125, ["y"] = -5122, ["z"] = 384, ["order"] = 9, ["angle"] = -2087256064},
  [58] = {["x"] = 4856, ["y"] = -5288, ["z"] = 208, ["order"] = 9, ["angle"] = 2107179008},
  [59] = {["x"] = 3807, ["y"] = -5243, ["z"] = -16, ["order"] = 9, ["angle"] = -2144534528},
  [60] = {["x"] = 3054, ["y"] = -5244, ["z"] = -16, ["order"] = 9, ["angle"] = -2147024896},
  [61] = {["x"] = 2138, ["y"] = -5331, ["z"] = -16, ["order"] = 9, ["angle"] = 2134573056},
  [62] = {["x"] = 1151, ["y"] = -5370, ["z"] = -16, ["order"] = 9, ["angle"] = 1639120896},
  [63] = {["x"] = 258, ["y"] = -4881, ["z"] = -16, ["order"] = 9, ["angle"] = 1265434624},
  [64] = {["x"] = -19, ["y"] = -3927, ["z"] = -16, ["order"] = 9, ["angle"] = 1014431744},
  [65] = {["x"] = -48, ["y"] = -2853, ["z"] = -16, ["order"] = 9, ["angle"] = 995295232},
  [66] = {["x"] = -41, ["y"] = -1775, ["z"] = -16, ["order"] = 9, ["angle"] = 1005649920},
  [67] = {["x"] = -110, ["y"] = -577, ["z"] = -16, ["order"] = 9, ["angle"] = 913375232},
  [68] = {["x"] = -147, ["y"] = 463, ["z"] = -16, ["order"] = 9, ["angle"] = 656605184},
  [69] = {["x"] = 238, ["y"] = 1375, ["z"] = -16, ["order"] = 9, ["angle"] = 308740096},
  [70] = {["x"] = 1067, ["y"] = 1849, ["z"] = 55, ["order"] = 10, ["angle"] = -517931008},
  [71] = {["x"] = 1883, ["y"] = 1530, ["z"] = 128, ["order"] = 10, ["angle"] = -930283520},
  [72] = {["x"] = 2164, ["y"] = 805, ["z"] = 151, ["order"] = 10, ["angle"] = -1150615552},
  [73] = {["x"] = 2292, ["y"] = -257, ["z"] = 256, ["order"] = 10, ["angle"] = -1776877568},
  [74] = {["x"] = 1677, ["y"] = -851, ["z"] = 256, ["order"] = 10, ["angle"] = 2120679424},
  [75] = {["x"] = 774, ["y"] = -955, ["z"] = 256, ["order"] = 10, ["angle"] = 2009006080},
  [76] = {["x"] = -379, ["y"] = -838, ["z"] = 256, ["order"] = 10, ["angle"] = 1707671552},
  [77] = {["x"] = -1300, ["y"] = -287, ["z"] = 256, ["order"] = 10, ["angle"] = 1130954752},
  [78] = {["x"] = -1505, ["y"] = 568, ["z"] = 256, ["order"] = 10, ["angle"] = 1062273024},
  [79] = {["x"] = 13190, ["y"] = 12088, ["z"] = 403, ["order"] = 4, ["angle"] = 65339392},
  [80] = {["x"] = 15215, ["y"] = 10455, ["z"] = 256, ["order"] = 6, ["angle"] = -666566656},
  [81] = {["x"] = 15962, ["y"] = 9318, ["z"] = 256, ["order"] = 6, ["angle"] = -738787328},
  [82] = {["x"] = 16556, ["y"] = 8407, ["z"] = 256, ["order"] = 6, ["angle"] = -746651648},
  [83] = {["x"] = 17840, ["y"] = 6769, ["z"] = 256, ["order"] = 6, ["angle"] = -950861824},
  [84] = {["x"] = 18234, ["y"] = 5782, ["z"] = 256, ["order"] = 7, ["angle"] = -1376190464},
  [85] = {["x"] = 17905, ["y"] = 4342, ["z"] = 256, ["order"] = 7, ["angle"] = -1873215488},
  [86] = {["x"] = 17259, ["y"] = 3737, ["z"] = 256, ["order"] = 7, ["angle"] = 2137587712},
  [87] = {["x"] = 15647, ["y"] = 3364, ["z"] = 256, ["order"] = 7, ["angle"] = -1788805120},
  [88] = {["x"] = 14352, ["y"] = 3125, ["z"] = 256, ["order"] = 7, ["angle"] = -1275002880},
  [89] = {["x"] = 15074, ["y"] = 11881, ["z"] = 256, ["order"] = 6, ["angle"] = -1099628544},
  [90] = {["x"] = 15547, ["y"] = 9946, ["z"] = 256, ["order"] = 6, ["angle"] = -694222848},
  [91] = {["x"] = 17334, ["y"] = 7481, ["z"] = 256, ["order"] = 6, ["angle"] = -852033536},
  [92] = {["x"] = 18408, ["y"] = 4932, ["z"] = 256, ["order"] = 7, ["angle"] = -1627062272},
  [93] = {["x"] = 16519, ["y"] = 3782, ["z"] = 256, ["order"] = 7, ["angle"] = -1835859968},
  [94] = {["x"] = 15042, ["y"] = 3835, ["z"] = 256, ["order"] = 7, ["angle"] = -1388773376},
  [95] = {["x"] = 14616, ["y"] = 2874, ["z"] = 256, ["order"] = 7, ["angle"] = -1396637696}
}
-- Respawn definitions for BARREN BADLANDS (MAP28)
krespawn.points[28] = {
  [0] = 97,
  [1] = {["x"] = 7414, ["y"] = -2887, ["z"] = 140, ["order"] = 0, ["angle"] = 1091633152},
  [2] = {["x"] = 7510, ["y"] = -1980, ["z"] = 140, ["order"] = 0, ["angle"] = 1303707648},
  [3] = {["x"] = 7374, ["y"] = -922, ["z"] = 140, ["order"] = 0, ["angle"] = 1533739008},
  [4] = {["x"] = 6763, ["y"] = -192, ["z"] = 140, ["order"] = 0, ["angle"] = 1956577280},
  [5] = {["x"] = 5811, ["y"] = 122, ["z"] = 135, ["order"] = 0, ["angle"] = 2117140480},
  [6] = {["x"] = 4725, ["y"] = 53, ["z"] = 46, ["order"] = 0, ["angle"] = 2068905984},
  [7] = {["x"] = 3717, ["y"] = 114, ["z"] = 4, ["order"] = 1, ["angle"] = 1682636800},
  [8] = {["x"] = 3032, ["y"] = 649, ["z"] = 4, ["order"] = 1, ["angle"] = 1352204288},
  [9] = {["x"] = 2696, ["y"] = 1537, ["z"] = 4, ["order"] = 1, ["angle"] = 1412497408},
  [10] = {["x"] = 2374, ["y"] = 2257, ["z"] = 4, ["order"] = 1, ["angle"] = 1999962112},
  [11] = {["x"] = 1724, ["y"] = 2584, ["z"] = 20, ["order"] = 1, ["angle"] = 2134835200},
  [12] = {["x"] = 877, ["y"] = 2700, ["z"] = 84, ["order"] = 2, ["angle"] = -1765998592},
  [13] = {["x"] = 83, ["y"] = 2348, ["z"] = 84, ["order"] = 2, ["angle"] = -1222443008},
  [14] = {["x"] = -216, ["y"] = 1665, ["z"] = 130, ["order"] = 2, ["angle"] = -1109065728},
  [15] = {["x"] = -294, ["y"] = 810, ["z"] = 324, ["order"] = 2, ["angle"] = -1017839616},
  [16] = {["x"] = -333, ["y"] = -137, ["z"] = 388, ["order"] = 2, ["angle"] = -566034432},
  [17] = {["x"] = 69, ["y"] = -678, ["z"] = 508, ["order"] = 2, ["angle"] = -68091904},
  [18] = {["x"] = 1055, ["y"] = -755, ["z"] = 530, ["order"] = 2, ["angle"] = 1019543552},
  [19] = {["x"] = 1085, ["y"] = -93, ["z"] = 388, ["order"] = 2, ["angle"] = 1073020928},
  [20] = {["x"] = 1106, ["y"] = 909, ["z"] = 486, ["order"] = 4, ["angle"] = 1073020928},
  [21] = {["x"] = 1115, ["y"] = 4151, ["z"] = 644, ["order"] = 4, ["angle"] = 1073020928},
  [22] = {["x"] = 1152, ["y"] = 3539, ["z"] = 644, ["order"] = 5, ["angle"] = 1147207680},
  [23] = {["x"] = 1110, ["y"] = 4842, ["z"] = 644, ["order"] = 5, ["angle"] = 1065287680},
  [24] = {["x"] = 1093, ["y"] = 5835, ["z"] = 644, ["order"] = 5, ["angle"] = 1074855936},
  [25] = {["x"] = 1057, ["y"] = 6828, ["z"] = 644, ["order"] = 5, ["angle"] = 1016791040},
  [26] = {["x"] = 1063, ["y"] = 7833, ["z"] = 644, ["order"] = 5, ["angle"] = 924254208},
  [27] = {["x"] = 1294, ["y"] = 8767, ["z"] = 573, ["order"] = 5, ["angle"] = 223805440},
  [28] = {["x"] = 2212, ["y"] = 9231, ["z"] = 521, ["order"] = 5, ["angle"] = 79888384},
  [29] = {["x"] = 3218, ["y"] = 9366, ["z"] = 506, ["order"] = 5, ["angle"] = -120389632},
  [30] = {["x"] = 4127, ["y"] = 9303, ["z"] = 513, ["order"] = 5, ["angle"] = -402325504},
  [31] = {["x"] = 4992, ["y"] = 8952, ["z"] = 553, ["order"] = 5, ["angle"] = -965279744},
  [32] = {["x"] = 5348, ["y"] = 8079, ["z"] = 651, ["order"] = 5, ["angle"] = -1644363776},
  [33] = {["x"] = 4845, ["y"] = 7520, ["z"] = 714, ["order"] = 5, ["angle"] = -1958936576},
  [34] = {["x"] = 3967, ["y"] = 7334, ["z"] = 734, ["order"] = 5, ["angle"] = -1914372096},
  [35] = {["x"] = 3084, ["y"] = 7207, ["z"] = 749, ["order"] = 5, ["angle"] = -1360986112},
  [36] = {["x"] = 2578, ["y"] = 6496, ["z"] = 829, ["order"] = 5, ["angle"] = -1160708096},
  [37] = {["x"] = 2437, ["y"] = 5643, ["z"] = 925, ["order"] = 5, ["angle"] = -585302016},
  [38] = {["x"] = 2970, ["y"] = 4878, ["z"] = 1011, ["order"] = 5, ["angle"] = -200736768},
  [39] = {["x"] = 3883, ["y"] = 4792, ["z"] = 1020, ["order"] = 5, ["angle"] = -154599424},
  [40] = {["x"] = 4580, ["y"] = 4595, ["z"] = 1043, ["order"] = 5, ["angle"] = -139919360},
  [41] = {["x"] = 5402, ["y"] = 4114, ["z"] = 1097, ["order"] = 5, ["angle"] = -62193664},
  [42] = {["x"] = 6355, ["y"] = 3968, ["z"] = 1113, ["order"] = 5, ["angle"] = 318832640},
  [43] = {["x"] = 7444, ["y"] = 4280, ["z"] = 1078, ["order"] = 5, ["angle"] = 610992128},
  [44] = {["x"] = 8118, ["y"] = 4882, ["z"] = 1010, ["order"] = 5, ["angle"] = 905248768},
  [45] = {["x"] = 8551, ["y"] = 5734, ["z"] = 914, ["order"] = 5, ["angle"] = 1115488256},
  [46] = {["x"] = 8583, ["y"] = 6629, ["z"] = 814, ["order"] = 5, ["angle"] = 1269104640},
  [47] = {["x"] = 7903, ["y"] = 7839, ["z"] = 678, ["order"] = 5, ["angle"] = 978124800},
  [48] = {["x"] = 7899, ["y"] = 8620, ["z"] = 703, ["order"] = 8, ["angle"] = 510459904},
  [49] = {["x"] = 8302, ["y"] = 9357, ["z"] = 875, ["order"] = 8, ["angle"] = 415039488},
  [50] = {["x"] = 9064, ["y"] = 9999, ["z"] = 1025, ["order"] = 8, ["angle"] = 20643840},
  [51] = {["x"] = 9969, ["y"] = 10045, ["z"] = 1036, ["order"] = 8, ["angle"] = -582156288},
  [52] = {["x"] = 10744, ["y"] = 9457, ["z"] = 898, ["order"] = 8, ["angle"] = -898826240},
  [53] = {["x"] = 11134, ["y"] = 8636, ["z"] = 706, ["order"] = 8, ["angle"] = -1076166656},
  [54] = {["x"] = 11121, ["y"] = 7254, ["z"] = 383, ["order"] = 8, ["angle"] = -1101987840},
  [55] = {["x"] = 11171, ["y"] = 5445, ["z"] = -39, ["order"] = 8, ["angle"] = -1061093376},
  [56] = {["x"] = 11295, ["y"] = 4458, ["z"] = -270, ["order"] = 8, ["angle"] = -1126629376},
  [57] = {["x"] = 11372, ["y"] = 3355, ["z"] = -508, ["order"] = 9, ["angle"] = -1192427520},
  [58] = {["x"] = 11375, ["y"] = 2364, ["z"] = -508, ["order"] = 9, ["angle"] = -1343553536},
  [59] = {["x"] = 11101, ["y"] = 1597, ["z"] = -508, ["order"] = 9, ["angle"] = -1736900608},
  [60] = {["x"] = 10284, ["y"] = 1189, ["z"] = -508, ["order"] = 9, ["angle"] = -1851981824},
  [61] = {["x"] = 9398, ["y"] = 838, ["z"] = -405, ["order"] = 9, ["angle"] = -1499529216},
  [62] = {["x"] = 8825, ["y"] = 66, ["z"] = -331, ["order"] = 9, ["angle"] = -772341760},
  [63] = {["x"] = 8788, ["y"] = -716, ["z"] = -326, ["order"] = 9, ["angle"] = -347668480},
  [64] = {["x"] = 9512, ["y"] = -951, ["z"] = -420, ["order"] = 9, ["angle"] = -394199040},
  [65] = {["x"] = 10366, ["y"] = -1243, ["z"] = -508, ["order"] = 9, ["angle"] = -616497152},
  [66] = {["x"] = 11099, ["y"] = -1860, ["z"] = -508, ["order"] = 9, ["angle"] = -978386944},
  [67] = {["x"] = 11304, ["y"] = -2645, ["z"] = -508, ["order"] = 10, ["angle"] = -1445265408},
  [68] = {["x"] = 10988, ["y"] = -3622, ["z"] = -508, ["order"] = 10, ["angle"] = -1887895552},
  [69] = {["x"] = 10168, ["y"] = -4034, ["z"] = -508, ["order"] = 10, ["angle"] = 2132344832},
  [70] = {["x"] = 9369, ["y"] = -3867, ["z"] = -508, ["order"] = 10, ["angle"] = 1815543808},
  [71] = {["x"] = 8384, ["y"] = -3420, ["z"] = -508, ["order"] = 10, ["angle"] = 2131820544},
  [72] = {["x"] = 7289, ["y"] = -3380, ["z"] = -508, ["order"] = 10, ["angle"] = -2120417280},
  [73] = {["x"] = 6045, ["y"] = -3356, ["z"] = -508, ["order"] = 10, ["angle"] = -1941110784},
  [74] = {["x"] = 5208, ["y"] = -3568, ["z"] = -508, ["order"] = 10, ["angle"] = -1896677376},
  [75] = {["x"] = 4479, ["y"] = -3721, ["z"] = -508, ["order"] = 10, ["angle"] = -1930625024},
  [76] = {["x"] = 3481, ["y"] = -3968, ["z"] = -490, ["order"] = 10, ["angle"] = -1586429952},
  [77] = {["x"] = 2983, ["y"] = -4548, ["z"] = -296, ["order"] = 11, ["angle"] = -1402273792},
  [78] = {["x"] = 2726, ["y"] = -5597, ["z"] = -31, ["order"] = 11, ["angle"] = -1067515904},
  [79] = {["x"] = 2643, ["y"] = -6560, ["z"] = 136, ["order"] = 11, ["angle"] = -512032768},
  [80] = {["x"] = 2932, ["y"] = -7545, ["z"] = 217, ["order"] = 11, ["angle"] = -318177280},
  [81] = {["x"] = 3685, ["y"] = -8289, ["z"] = 254, ["order"] = 11, ["angle"] = -198246400},
  [82] = {["x"] = 4616, ["y"] = -8537, ["z"] = 271, ["order"] = 11, ["angle"] = 89456640},
  [83] = {["x"] = 6037, ["y"] = -8511, ["z"] = 319, ["order"] = 11, ["angle"] = 377552896},
  [84] = {["x"] = 7095, ["y"] = -7950, ["z"] = 329, ["order"] = 11, ["angle"] = 914948096},
  [85] = {["x"] = 7470, ["y"] = -5708, ["z"] = 140, ["order"] = 12, ["angle"] = 1116798976},
  [86] = {["x"] = 7459, ["y"] = -5096, ["z"] = 140, ["order"] = 12, ["angle"] = 1103036416},
  [87] = {["x"] = 7436, ["y"] = -4422, ["z"] = 140, ["order"] = 12, ["angle"] = 1103036416},
  [88] = {["x"] = 7408, ["y"] = -3382, ["z"] = 140, ["order"] = 12, ["angle"] = 1085865984},
  [89] = {["x"] = 11223, ["y"] = 7836, ["z"] = 519, ["order"] = 8, ["angle"] = -1108934656},
  [90] = {["x"] = 9480, ["y"] = -4220, ["z"] = -508, ["order"] = 10, ["angle"] = -1970470912},
  [91] = {["x"] = 8180, ["y"] = -4675, ["z"] = -508, ["order"] = 10, ["angle"] = 2125004800},
  [92] = {["x"] = 7081, ["y"] = -4713, ["z"] = -508, ["order"] = 10, ["angle"] = 2124873728},
  [93] = {["x"] = 5984, ["y"] = -4615, ["z"] = -508, ["order"] = 10, ["angle"] = 1985150976},
  [94] = {["x"] = 4661, ["y"] = -4284, ["z"] = -508, ["order"] = 10, ["angle"] = -2045444096},
  [95] = {["x"] = 3554, ["y"] = -4528, ["z"] = -402, ["order"] = 11, ["angle"] = -1595211776},
  [96] = {["x"] = 2850, ["y"] = -4986, ["z"] = -186, ["order"] = 11, ["angle"] = -1227423744},
  [97] = {["x"] = 4985, ["y"] = -8658, ["z"] = 299, ["order"] = 11, ["angle"] = 137428992}
}
-- Respawn definitions for RED BARRAGE (MAP29)
krespawn.points[29] = {
  [0] = 116,
  [1] = {["x"] = -2427, ["y"] = -3220, ["z"] = 1500, ["order"] = 0, ["angle"] = 1066991616},
  [2] = {["x"] = -2522, ["y"] = -2104, ["z"] = 1500, ["order"] = 0, ["angle"] = 974716928},
  [3] = {["x"] = -2556, ["y"] = -1050, ["z"] = 1500, ["order"] = 0, ["angle"] = 636551168},
  [4] = {["x"] = -2080, ["y"] = -209, ["z"] = 1500, ["order"] = 0, ["angle"] = 201916416},
  [5] = {["x"] = -1176, ["y"] = -30, ["z"] = 1500, ["order"] = 1, ["angle"] = 128385024},
  [6] = {["x"] = -84, ["y"] = 20, ["z"] = 1500, ["order"] = 1, ["angle"] = 772866048},
  [7] = {["x"] = 465, ["y"] = 580, ["z"] = 1500, ["order"] = 1, ["angle"] = 1312096256},
  [8] = {["x"] = 378, ["y"] = 1159, ["z"] = 1500, ["order"] = 2, ["angle"] = 1854078976},
  [9] = {["x"] = -506, ["y"] = 1370, ["z"] = 1500, ["order"] = 2, ["angle"] = 1310785536},
  [10] = {["x"] = -775, ["y"] = 2075, ["z"] = 1500, ["order"] = 2, ["angle"] = 999227392},
  [11] = {["x"] = -815, ["y"] = 2964, ["z"] = 0, ["order"] = 2, ["angle"] = 1068957696},
  [12] = {["x"] = -811, ["y"] = 3775, ["z"] = 0, ["order"] = 2, ["angle"] = 1068957696},
  [13] = {["x"] = -805, ["y"] = 4506, ["z"] = 0, ["order"] = 2, ["angle"] = 1068957696},
  [14] = {["x"] = -779, ["y"] = 5468, ["z"] = 0, ["order"] = 3, ["angle"] = 1101463552},
  [15] = {["x"] = -870, ["y"] = 6398, ["z"] = -69, ["order"] = 3, ["angle"] = 1080754176},
  [16] = {["x"] = -990, ["y"] = 7366, ["z"] = -253, ["order"] = 3, ["angle"] = 804716544},
  [17] = {["x"] = -827, ["y"] = 8181, ["z"] = -256, ["order"] = 3, ["angle"] = 451608576},
  [18] = {["x"] = -351, ["y"] = 8757, ["z"] = -256, ["order"] = 3, ["angle"] = -151060480},
  [19] = {["x"] = 820, ["y"] = 8858, ["z"] = -256, ["order"] = 4, ["angle"] = -348061696},
  [20] = {["x"] = 1519, ["y"] = 8603, ["z"] = -256, ["order"] = 4, ["angle"] = -786366464},
  [21] = {["x"] = 1939, ["y"] = 7915, ["z"] = -256, ["order"] = 4, ["angle"] = -1042481152},
  [22] = {["x"] = 1962, ["y"] = 6962, ["z"] = -256, ["order"] = 4, ["angle"] = -1151664128},
  [23] = {["x"] = 1806, ["y"] = 6012, ["z"] = -158, ["order"] = 4, ["angle"] = -1208942592},
  [24] = {["x"] = 1784, ["y"] = 5037, ["z"] = 94, ["order"] = 5, ["angle"] = -1033568256},
  [25] = {["x"] = 1731, ["y"] = 4075, ["z"] = 91, ["order"] = 5, ["angle"] = -930283520},
  [26] = {["x"] = 1796, ["y"] = 3032, ["z"] = -48, ["order"] = 5, ["angle"] = -600113152},
  [27] = {["x"] = 2228, ["y"] = 2557, ["z"] = -48, ["order"] = 5, ["angle"] = -243859456},
  [28] = {["x"] = 4179, ["y"] = 2238, ["z"] = -48, ["order"] = 5, ["angle"] = -47775744},
  [29] = {["x"] = 5314, ["y"] = 2331, ["z"] = -48, ["order"] = 5, ["angle"] = -43319296},
  [30] = {["x"] = 6389, ["y"] = 2376, ["z"] = -48, ["order"] = 6, ["angle"] = -190906368},
  [31] = {["x"] = 7143, ["y"] = 2340, ["z"] = -48, ["order"] = 6, ["angle"] = -264830976},
  [32] = {["x"] = 8273, ["y"] = 2528, ["z"] = -48, ["order"] = 6, ["angle"] = -1090715648},
  [33] = {["x"] = 8293, ["y"] = 1601, ["z"] = -48, ["order"] = 6, ["angle"] = -1058209792},
  [34] = {["x"] = 8342, ["y"] = 792, ["z"] = -48, ["order"] = 6, ["angle"] = -1116536832},
  [35] = {["x"] = 8327, ["y"] = -377, ["z"] = -48, ["order"] = 6, ["angle"] = -1062797312},
  [36] = {["x"] = 8345, ["y"] = -1398, ["z"] = -162, ["order"] = 7, ["angle"] = -1026883584},
  [37] = {["x"] = 8355, ["y"] = -2434, ["z"] = -416, ["order"] = 7, ["angle"] = -1100283904},
  [38] = {["x"] = 8337, ["y"] = -3130, ["z"] = -416, ["order"] = 7, ["angle"] = -1100283904},
  [39] = {["x"] = 8283, ["y"] = -4777, ["z"] = 162, ["order"] = 7, ["angle"] = -1069481984},
  [40] = {["x"] = 8263, ["y"] = -5542, ["z"] = 712, ["order"] = 7, ["angle"] = -1047986176},
  [41] = {["x"] = 8282, ["y"] = -6392, ["z"] = 712, ["order"] = 7, ["angle"] = -978255872},
  [42] = {["x"] = 8194, ["y"] = -7497, ["z"] = 434, ["order"] = 7, ["angle"] = -905248768},
  [43] = {["x"] = 8319, ["y"] = -8656, ["z"] = 368, ["order"] = 7, ["angle"] = -475856896},
  [44] = {["x"] = 8978, ["y"] = -9481, ["z"] = 368, ["order"] = 7, ["angle"] = -156565504},
  [45] = {["x"] = 9864, ["y"] = -9676, ["z"] = 368, ["order"] = 8, ["angle"] = 211484672},
  [46] = {["x"] = 10843, ["y"] = -9447, ["z"] = 368, ["order"] = 8, ["angle"] = 633405440},
  [47] = {["x"] = 11357, ["y"] = -8789, ["z"] = 368, ["order"] = 8, ["angle"] = 989003776},
  [48] = {["x"] = 11524, ["y"] = -7941, ["z"] = 368, ["order"] = 8, ["angle"] = 1140129792},
  [49] = {["x"] = 11319, ["y"] = -6559, ["z"] = 0, ["order"] = 8, ["angle"] = 984416256},
  [50] = {["x"] = 11544, ["y"] = -5349, ["z"] = 0, ["order"] = 8, ["angle"] = 942342144},
  [51] = {["x"] = 11655, ["y"] = -4782, ["z"] = 1, ["order"] = 8, ["angle"] = 1308688384},
  [52] = {["x"] = 11573, ["y"] = -3941, ["z"] = 0, ["order"] = 9, ["angle"] = 1463222272},
  [53] = {["x"] = 11346, ["y"] = -3347, ["z"] = 0, ["order"] = 9, ["angle"] = 1723924480},
  [54] = {["x"] = 10704, ["y"] = -2869, ["z"] = 0, ["order"] = 9, ["angle"] = 2108620800},
  [55] = {["x"] = 7491, ["y"] = -2837, ["z"] = 1024, ["order"] = 9, ["angle"] = 2133524480},
  [56] = {["x"] = 6995, ["y"] = -2827, ["z"] = 1024, ["order"] = 9, ["angle"] = 2133524480},
  [57] = {["x"] = 6088, ["y"] = -2808, ["z"] = 1025, ["order"] = 9, ["angle"] = 2133524480},
  [58] = {["x"] = 4673, ["y"] = -2806, ["z"] = 1024, ["order"] = 10, ["angle"] = -1754857472},
  [59] = {["x"] = 3796, ["y"] = -2648, ["z"] = 1024, ["order"] = 10, ["angle"] = -1069219840},
  [60] = {["x"] = 3640, ["y"] = -3341, ["z"] = 1024, ["order"] = 10, ["angle"] = -1107755008},
  [61] = {["x"] = 3249, ["y"] = -5035, ["z"] = 16, ["order"] = 10, ["angle"] = -1090453504},
  [62] = {["x"] = 3249, ["y"] = -5702, ["z"] = 16, ["order"] = 10, ["angle"] = -1115619328},
  [63] = {["x"] = 3213, ["y"] = -6521, ["z"] = 16, ["order"] = 10, ["angle"] = -1101463552},
  [64] = {["x"] = 3209, ["y"] = -7352, ["z"] = 16, ["order"] = 10, ["angle"] = -1096220672},
  [65] = {["x"] = 3234, ["y"] = -8304, ["z"] = 16, ["order"] = 10, ["angle"] = -1107492864},
  [66] = {["x"] = 3266, ["y"] = -9127, ["z"] = 16, ["order"] = 10, ["angle"] = -1102381056},
  [67] = {["x"] = 3187, ["y"] = -10000, ["z"] = 16, ["order"] = 11, ["angle"] = -2018181120},
  [68] = {["x"] = 2290, ["y"] = -10116, ["z"] = 16, ["order"] = 11, ["angle"] = 2146369536},
  [69] = {["x"] = 1429, ["y"] = -10121, ["z"] = 16, ["order"] = 11, ["angle"] = 2146369536},
  [70] = {["x"] = 539, ["y"] = -10120, ["z"] = 16, ["order"] = 11, ["angle"] = 2146369536},
  [71] = {["x"] = -522, ["y"] = -10119, ["z"] = 16, ["order"] = 11, ["angle"] = 2146369536},
  [72] = {["x"] = -1630, ["y"] = -10116, ["z"] = 135, ["order"] = 11, ["angle"] = 2146369536},
  [73] = {["x"] = -2905, ["y"] = -10113, ["z"] = 484, ["order"] = 11, ["angle"] = 2146369536},
  [74] = {["x"] = -3707, ["y"] = -10111, ["z"] = 560, ["order"] = 11, ["angle"] = 2146369536},
  [75] = {["x"] = -4496, ["y"] = -10109, ["z"] = 560, ["order"] = 11, ["angle"] = 2146369536},
  [76] = {["x"] = -5339, ["y"] = -10100, ["z"] = 560, ["order"] = 11, ["angle"] = -1941897216},
  [77] = {["x"] = -6541, ["y"] = -10395, ["z"] = 560, ["order"] = 12, ["angle"] = 1937702912},
  [78] = {["x"] = -7294, ["y"] = -10023, ["z"] = 560, ["order"] = 12, ["angle"] = 1463484416},
  [79] = {["x"] = -7033, ["y"] = -9504, ["z"] = 560, ["order"] = 12, ["angle"] = 697106432},
  [80] = {["x"] = -7246, ["y"] = -8699, ["z"] = 560, ["order"] = 13, ["angle"] = 544407552},
  [81] = {["x"] = -6726, ["y"] = -8410, ["z"] = 560, ["order"] = 13, ["angle"] = 160235520},
  [82] = {["x"] = -5754, ["y"] = -8317, ["z"] = 560, ["order"] = 14, ["angle"] = 126287872},
  [83] = {["x"] = -4819, ["y"] = -8248, ["z"] = 560, ["order"] = 14, ["angle"] = 14876672},
  [84] = {["x"] = -3961, ["y"] = -8350, ["z"] = 560, ["order"] = 14, ["angle"] = 46596096},
  [85] = {["x"] = -2999, ["y"] = -8314, ["z"] = 560, ["order"] = 15, ["angle"] = 491061248},
  [86] = {["x"] = -2498, ["y"] = -7541, ["z"] = 560, ["order"] = 15, ["angle"] = 1065156608},
  [87] = {["x"] = -2400, ["y"] = -4100, ["z"] = 1500, ["order"] = 15, ["angle"] = 1073807360},
  [88] = {["x"] = -2400, ["y"] = -3606, ["z"] = 1500, ["order"] = 15, ["angle"] = 1129906176},
  [89] = {["x"] = -847, ["y"] = 4909, ["z"] = 0, ["order"] = 2, ["angle"] = 1042612224},
  [90] = {["x"] = 3767, ["y"] = 1988, ["z"] = -40, ["order"] = 5, ["angle"] = 52887552},
  [91] = {["x"] = 10083, ["y"] = -2876, ["z"] = 81, ["order"] = 9, ["angle"] = 2122907648},
  [92] = {["x"] = 4103, ["y"] = -4899, ["z"] = 16, ["order"] = 10, ["angle"] = -1070923776},
  [93] = {["x"] = 4099, ["y"] = -5604, ["z"] = 16, ["order"] = 10, ["angle"] = -1070923776},
  [94] = {["x"] = 4102, ["y"] = -6454, ["z"] = 16, ["order"] = 10, ["angle"] = -1070923776},
  [95] = {["x"] = 4105, ["y"] = -7440, ["z"] = 16, ["order"] = 10, ["angle"] = -1070923776},
  [96] = {["x"] = 4108, ["y"] = -8126, ["z"] = 16, ["order"] = 10, ["angle"] = -1070923776},
  [97] = {["x"] = 4209, ["y"] = -10016, ["z"] = 16, ["order"] = 11, ["angle"] = -1328611328},
  [98] = {["x"] = 4004, ["y"] = -10964, ["z"] = 16, ["order"] = 11, ["angle"] = -2075459584},
  [99] = {["x"] = 3310, ["y"] = -11061, ["z"] = 16, ["order"] = 11, ["angle"] = 2090008576},
  [100] = {["x"] = 2314, ["y"] = -11006, ["z"] = 16, ["order"] = 11, ["angle"] = 2132869120},
  [101] = {["x"] = 1541, ["y"] = -11002, ["z"] = 16, ["order"] = 11, ["angle"] = -2144141312},
  [102] = {["x"] = 643, ["y"] = -11008, ["z"] = 16, ["order"] = 11, ["angle"] = -2144141312},
  [103] = {["x"] = -317, ["y"] = -11013, ["z"] = 16, ["order"] = 11, ["angle"] = -2144141312},
  [104] = {["x"] = -1543, ["y"] = -11019, ["z"] = 111, ["order"] = 11, ["angle"] = -2144141312},
  [105] = {["x"] = -2298, ["y"] = -11022, ["z"] = 318, ["order"] = 11, ["angle"] = -2144141312},
  [106] = {["x"] = -3390, ["y"] = -11027, ["z"] = 560, ["order"] = 11, ["angle"] = -2144141312},
  [107] = {["x"] = -4402, ["y"] = -11030, ["z"] = 560, ["order"] = 11, ["angle"] = 2090926080},
  [108] = {["x"] = -5303, ["y"] = -11023, ["z"] = 560, ["order"] = 11, ["angle"] = 2099183616},
  [109] = {["x"] = -6164, ["y"] = -10911, ["z"] = 560, ["order"] = 12, ["angle"] = 1625227264},
  [110] = {["x"] = -6625, ["y"] = -9813, ["z"] = 560, ["order"] = 12, ["angle"] = 1197015040},
  [111] = {["x"] = -6946, ["y"] = -8970, ["z"] = 560, ["order"] = 13, ["angle"] = 608370688},
  [112] = {["x"] = -6563, ["y"] = -7797, ["z"] = 560, ["order"] = 13, ["angle"] = 217907200},
  [113] = {["x"] = -6017, ["y"] = -7441, ["z"] = 560, ["order"] = 14, ["angle"] = -91160576},
  [114] = {["x"] = -4669, ["y"] = -7514, ["z"] = 560, ["order"] = 14, ["angle"] = 94175232},
  [115] = {["x"] = -3876, ["y"] = -7487, ["z"] = 560, ["order"] = 14, ["angle"] = 95092736},
  [116] = {["x"] = -2903, ["y"] = -7463, ["z"] = 560, ["order"] = 15, ["angle"] = 834469888}
}
-- Respawn definitions for MIDNIGHT CHANNEL (MAP30)
krespawn.points[30] = {
  [0] = 117,
  [1] = {["x"] = 1254, ["y"] = 1064, ["z"] = 0, ["order"] = 0, ["angle"] = 1059258368},
  [2] = {["x"] = 1304, ["y"] = 1813, ["z"] = 0, ["order"] = 0, ["angle"] = 1088880640},
  [3] = {["x"] = 1415, ["y"] = 2741, ["z"] = 0, ["order"] = 0, ["angle"] = 1149435904},
  [4] = {["x"] = 1421, ["y"] = 3764, ["z"] = 237, ["order"] = 1, ["angle"] = 1671233536},
  [5] = {["x"] = 1016, ["y"] = 4426, ["z"] = 458, ["order"] = 1, ["angle"] = -2103902208},
  [6] = {["x"] = 168, ["y"] = 4465, ["z"] = 472, ["order"] = 1, ["angle"] = -1992097792},
  [7] = {["x"] = -613, ["y"] = 4200, ["z"] = 314, ["order"] = 1, ["angle"] = -2101936128},
  [8] = {["x"] = -1665, ["y"] = 3959, ["z"] = 256, ["order"] = 1, ["angle"] = 1769144320},
  [9] = {["x"] = -2645, ["y"] = 4121, ["z"] = 256, ["order"] = 1, ["angle"] = 1544093696},
  [10] = {["x"] = -3157, ["y"] = 4662, ["z"] = 256, ["order"] = 1, ["angle"] = 1092943872},
  [11] = {["x"] = -3108, ["y"] = 5783, ["z"] = 256, ["order"] = 2, ["angle"] = 1131610112},
  [12] = {["x"] = -3568, ["y"] = 5793, ["z"] = 256, ["order"] = 2, ["angle"] = 936050688},
  [13] = {["x"] = -3516, ["y"] = 6911, ["z"] = 256, ["order"] = 2, ["angle"] = 1044054016},
  [14] = {["x"] = -3168, ["y"] = 6809, ["z"] = 256, ["order"] = 2, ["angle"] = 972357632},
  [15] = {["x"] = -3394, ["y"] = 7627, ["z"] = 312, ["order"] = 3, ["angle"] = 712048640},
  [16] = {["x"] = -2863, ["y"] = 8505, ["z"] = 635, ["order"] = 3, ["angle"] = 229179392},
  [17] = {["x"] = -1882, ["y"] = 8606, ["z"] = 672, ["order"] = 3, ["angle"] = -512950272},
  [18] = {["x"] = -1324, ["y"] = 7779, ["z"] = 368, ["order"] = 3, ["angle"] = -661979136},
  [19] = {["x"] = -968, ["y"] = 6736, ["z"] = 111, ["order"] = 3, ["angle"] = -154206208},
  [20] = {["x"] = 202, ["y"] = 6314, ["z"] = 24, ["order"] = 3, ["angle"] = 656343040},
  [21] = {["x"] = 1173, ["y"] = 7036, ["z"] = 172, ["order"] = 3, ["angle"] = 1159528448},
  [22] = {["x"] = 1119, ["y"] = 8001, ["z"] = 264, ["order"] = 4, ["angle"] = 1131741184},
  [23] = {["x"] = 1205, ["y"] = 8811, ["z"] = 264, ["order"] = 4, ["angle"] = 1118371840},
  [24] = {["x"] = 1276, ["y"] = 9638, ["z"] = 264, ["order"] = 4, ["angle"] = 1290600448},
  [25] = {["x"] = 1274, ["y"] = 10514, ["z"] = 223, ["order"] = 4, ["angle"] = 1581711360},
  [26] = {["x"] = 882, ["y"] = 11213, ["z"] = 126, ["order"] = 4, ["angle"] = 1786970112},
  [27] = {["x"] = 254, ["y"] = 11696, ["z"] = 58, ["order"] = 4, ["angle"] = 2003501056},
  [28] = {["x"] = -437, ["y"] = 11907, ["z"] = 29, ["order"] = 4, ["angle"] = 2091319296},
  [29] = {["x"] = -1634, ["y"] = 11945, ["z"] = -40, ["order"] = 4, ["angle"] = -2110324736},
  [30] = {["x"] = -4731, ["y"] = 11967, ["z"] = 344, ["order"] = 5, ["angle"] = 2104688640},
  [31] = {["x"] = -5498, ["y"] = 12009, ["z"] = 344, ["order"] = 5, ["angle"] = -2137980928},
  [32] = {["x"] = -6304, ["y"] = 12013, ["z"] = 344, ["order"] = 5, ["angle"] = -2137980928},
  [33] = {["x"] = -7039, ["y"] = 12005, ["z"] = 344, ["order"] = 5, ["angle"] = -2137980928},
  [34] = {["x"] = -7882, ["y"] = 11994, ["z"] = 344, ["order"] = 5, ["angle"] = -2137980928},
  [35] = {["x"] = -8748, ["y"] = 11982, ["z"] = 344, ["order"] = 5, ["angle"] = -2137980928},
  [36] = {["x"] = -9795, ["y"] = 11981, ["z"] = 344, ["order"] = 5, ["angle"] = 2116354048},
  [37] = {["x"] = -10737, ["y"] = 12003, ["z"] = 344, ["order"] = 6, ["angle"] = 2113077248},
  [38] = {["x"] = -11754, ["y"] = 11976, ["z"] = 344, ["order"] = 6, ["angle"] = 2019229696},
  [39] = {["x"] = -12707, ["y"] = 12042, ["z"] = 401, ["order"] = 6, ["angle"] = 1839398912},
  [40] = {["x"] = -13259, ["y"] = 12269, ["z"] = 576, ["order"] = 6, ["angle"] = 1800994816},
  [41] = {["x"] = -13940, ["y"] = 12783, ["z"] = 792, ["order"] = 6, ["angle"] = 1313538048},
  [42] = {["x"] = -14310, ["y"] = 13682, ["z"] = 909, ["order"] = 6, ["angle"] = 1271332864},
  [43] = {["x"] = -14336, ["y"] = 14846, ["z"] = 918, ["order"] = 6, ["angle"] = 560136192},
  [44] = {["x"] = -13413, ["y"] = 15559, ["z"] = 625, ["order"] = 6, ["angle"] = 306118656},
  [45] = {["x"] = -12368, ["y"] = 15984, ["z"] = 344, ["order"] = 6, ["angle"] = 90243072},
  [46] = {["x"] = -11718, ["y"] = 16059, ["z"] = 344, ["order"] = 6, ["angle"] = 34144256},
  [47] = {["x"] = -10861, ["y"] = 16021, ["z"] = 344, ["order"] = 7, ["angle"] = 17498112},
  [48] = {["x"] = -9832, ["y"] = 16021, ["z"] = 144, ["order"] = 7, ["angle"] = -32571392},
  [49] = {["x"] = -7004, ["y"] = 16024, ["z"] = 504, ["order"] = 7, ["angle"] = -37814272},
  [50] = {["x"] = -6533, ["y"] = 16017, ["z"] = 504, ["order"] = 7, ["angle"] = -11206656},
  [51] = {["x"] = -5513, ["y"] = 15995, ["z"] = 504, ["order"] = 7, ["angle"] = -11206656},
  [52] = {["x"] = -3892, ["y"] = 16032, ["z"] = 504, ["order"] = 7, ["angle"] = -8323072},
  [53] = {["x"] = -3282, ["y"] = 15608, ["z"] = 504, ["order"] = 7, ["angle"] = -219217920},
  [54] = {["x"] = -2431, ["y"] = 15296, ["z"] = 504, ["order"] = 7, ["angle"] = -25493504},
  [55] = {["x"] = -449, ["y"] = 15240, ["z"] = 504, ["order"] = 8, ["angle"] = 77135872},
  [56] = {["x"] = 152, ["y"] = 15359, ["z"] = 504, ["order"] = 8, ["angle"] = 226689024},
  [57] = {["x"] = 799, ["y"] = 15673, ["z"] = 504, ["order"] = 8, ["angle"] = 301006848},
  [58] = {["x"] = 1471, ["y"] = 15935, ["z"] = 504, ["order"] = 8, ["angle"] = -17367040},
  [59] = {["x"] = 2356, ["y"] = 15992, ["z"] = 504, ["order"] = 9, ["angle"] = -35717120},
  [60] = {["x"] = 3353, ["y"] = 16064, ["z"] = 504, ["order"] = 9, ["angle"] = -211222528},
  [61] = {["x"] = 4370, ["y"] = 15902, ["z"] = 461, ["order"] = 9, ["angle"] = -453312512},
  [62] = {["x"] = 5162, ["y"] = 15371, ["z"] = 373, ["order"] = 9, ["angle"] = -883359744},
  [63] = {["x"] = 5626, ["y"] = 14661, ["z"] = 321, ["order"] = 9, ["angle"] = -1045495808},
  [64] = {["x"] = 5718, ["y"] = 14306, ["z"] = 311, ["order"] = 9, ["angle"] = -1066860544},
  [65] = {["x"] = 5720, ["y"] = 13757, ["z"] = 303, ["order"] = 9, ["angle"] = -1003552768},
  [66] = {["x"] = 5722, ["y"] = 12987, ["z"] = 256, ["order"] = 9, ["angle"] = -1044971520},
  [67] = {["x"] = 5683, ["y"] = 12035, ["z"] = 256, ["order"] = 10, ["angle"] = -987168768},
  [68] = {["x"] = 5690, ["y"] = 11031, ["z"] = 256, ["order"] = 10, ["angle"] = -821493760},
  [69] = {["x"] = 5906, ["y"] = 9886, ["z"] = 256, ["order"] = 10, ["angle"] = -643629056},
  [70] = {["x"] = 6388, ["y"] = 9117, ["z"] = 256, ["order"] = 10, ["angle"] = -219217920},
  [71] = {["x"] = 7124, ["y"] = 8745, ["z"] = 256, ["order"] = 10, ["angle"] = -64028672},
  [72] = {["x"] = 8962, ["y"] = 8590, ["z"] = 609, ["order"] = 10, ["angle"] = -587923456},
  [73] = {["x"] = 9631, ["y"] = 7043, ["z"] = 835, ["order"] = 10, ["angle"] = -814284800},
  [74] = {["x"] = 9586, ["y"] = 5881, ["z"] = 733, ["order"] = 10, ["angle"] = -1091371008},
  [75] = {["x"] = 9700, ["y"] = 4992, ["z"] = 733, ["order"] = 10, ["angle"] = -1084030976},
  [76] = {["x"] = 9775, ["y"] = 4522, ["z"] = 733, ["order"] = 10, ["angle"] = -759496704},
  [77] = {["x"] = 9598, ["y"] = 4505, ["z"] = 733, ["order"] = 11, ["angle"] = -1295974400},
  [78] = {["x"] = 9463, ["y"] = 4086, ["z"] = 733, ["order"] = 11, ["angle"] = -1475674112},
  [79] = {["x"] = 9124, ["y"] = 3720, ["z"] = 733, ["order"] = 11, ["angle"] = -1455357952},
  [80] = {["x"] = 8761, ["y"] = 3166, ["z"] = 733, ["order"] = 11, ["angle"] = -1199636480},
  [81] = {["x"] = 8601, ["y"] = 2481, ["z"] = 733, ["order"] = 11, ["angle"] = -1059389440},
  [82] = {["x"] = 8555, ["y"] = 1679, ["z"] = 733, ["order"] = 11, ["angle"] = -775749632},
  [83] = {["x"] = 8782, ["y"] = 1020, ["z"] = 733, ["order"] = 11, ["angle"] = -707592192},
  [84] = {["x"] = 9251, ["y"] = 401, ["z"] = 733, ["order"] = 11, ["angle"] = -820707328},
  [85] = {["x"] = 9564, ["y"] = -291, ["z"] = 733, ["order"] = 11, ["angle"] = -898957312},
  [86] = {["x"] = 9773, ["y"] = -1093, ["z"] = 679, ["order"] = 11, ["angle"] = -1100283904},
  [87] = {["x"] = 9820, ["y"] = -1932, ["z"] = 599, ["order"] = 11, ["angle"] = -1211039744},
  [88] = {["x"] = 9695, ["y"] = -2753, ["z"] = 521, ["order"] = 11, ["angle"] = -1438056448},
  [89] = {["x"] = 9367, ["y"] = -3428, ["z"] = 490, ["order"] = 11, ["angle"] = -1638203392},
  [90] = {["x"] = 8706, ["y"] = -4118, ["z"] = 490, ["order"] = 12, ["angle"] = -1933508608},
  [91] = {["x"] = 7893, ["y"] = -4528, ["z"] = 490, ["order"] = 12, ["angle"] = -1923547136},
  [92] = {["x"] = 6721, ["y"] = -5028, ["z"] = 490, ["order"] = 12, ["angle"] = -1955790848},
  [93] = {["x"] = 6041, ["y"] = -5254, ["z"] = 490, ["order"] = 12, ["angle"] = -2007564288},
  [94] = {["x"] = 5358, ["y"] = -5424, ["z"] = 490, ["order"] = 12, ["angle"] = -1955659776},
  [95] = {["x"] = 3802, ["y"] = -6136, ["z"] = 490, ["order"] = 13, ["angle"] = -1984888832},
  [96] = {["x"] = 3001, ["y"] = -6372, ["z"] = 490, ["order"] = 13, ["angle"] = 2006908928},
  [97] = {["x"] = 2206, ["y"] = -6158, ["z"] = 418, ["order"] = 13, ["angle"] = 1542520832},
  [98] = {["x"] = 1689, ["y"] = -5596, ["z"] = 355, ["order"] = 13, ["angle"] = 1208156160},
  [99] = {["x"] = 1457, ["y"] = -5064, ["z"] = 327, ["order"] = 13, ["angle"] = 1119289344},
  [100] = {["x"] = 1285, ["y"] = -3969, ["z"] = 261, ["order"] = 13, ["angle"] = 1105264640},
  [101] = {["x"] = 1225, ["y"] = -1506, ["z"] = 0, ["order"] = 14, ["angle"] = 1071448064},
  [102] = {["x"] = 1221, ["y"] = -1113, ["z"] = 0, ["order"] = 14, ["angle"] = 1071448064},
  [103] = {["x"] = 1222, ["y"] = -575, ["z"] = 0, ["order"] = 14, ["angle"] = 1071448064},
  [104] = {["x"] = 1233, ["y"] = 36, ["z"] = 0, ["order"] = 14, ["angle"] = 1032126464},
  [105] = {["x"] = 1246, ["y"] = 690, ["z"] = 0, ["order"] = 14, ["angle"] = 1085210624},
  [106] = {["x"] = -3502, ["y"] = 16288, ["z"] = 504, ["order"] = 7, ["angle"] = 280428544},
  [107] = {["x"] = -2503, ["y"] = 16801, ["z"] = 504, ["order"] = 7, ["angle"] = -34275328},
  [108] = {["x"] = -468, ["y"] = 16888, ["z"] = 504, ["order"] = 8, ["angle"] = 31784960},
  [109] = {["x"] = 181, ["y"] = 16927, ["z"] = 504, ["order"] = 8, ["angle"] = -349765632},
  [110] = {["x"] = 762, ["y"] = 16578, ["z"] = 504, ["order"] = 8, ["angle"] = -269811712},
  [111] = {["x"] = 10054, ["y"] = 3998, ["z"] = 733, ["order"] = 11, ["angle"] = -663027712},
  [112] = {["x"] = 10488, ["y"] = 3467, ["z"] = 733, ["order"] = 11, ["angle"] = -820707328},
  [113] = {["x"] = 10829, ["y"] = 2637, ["z"] = 733, ["order"] = 11, ["angle"] = -1031340032},
  [114] = {["x"] = 10914, ["y"] = 1805, ["z"] = 733, ["order"] = 11, ["angle"] = -1203044352},
  [115] = {["x"] = 10806, ["y"] = 1071, ["z"] = 733, ["order"] = 11, ["angle"] = -1484718080},
  [116] = {["x"] = 10521, ["y"] = 524, ["z"] = 733, ["order"] = 11, ["angle"] = -1543831552},
  [117] = {["x"] = 9978, ["y"] = -240, ["z"] = 733, ["order"] = 11, ["angle"] = -1207762944}
}
-- Respawn definitions for VANILLA HOTEL (MAP31)
krespawn.points[31] = {
  [0] = 108,
  [1] = {["x"] = 2127, ["y"] = -6316, ["z"] = 0, ["order"] = 0, ["angle"] = 1029242880},
  [2] = {["x"] = 2106, ["y"] = -5453, ["z"] = 0, ["order"] = 0, ["angle"] = 1647902720},
  [3] = {["x"] = 1723, ["y"] = -4927, ["z"] = 0, ["order"] = 1, ["angle"] = 2080571392},
  [4] = {["x"] = 1009, ["y"] = -4840, ["z"] = 0, ["order"] = 1, ["angle"] = -2125004800},
  [5] = {["x"] = 105, ["y"] = -4868, ["z"] = 0, ["order"] = 1, ["angle"] = 1913323520},
  [6] = {["x"] = -911, ["y"] = -4758, ["z"] = 0, ["order"] = 2, ["angle"] = 1761673216},
  [7] = {["x"] = -1131, ["y"] = -4033, ["z"] = 0, ["order"] = 2, ["angle"] = 1516830720},
  [8] = {["x"] = -1764, ["y"] = -4383, ["z"] = 0, ["order"] = 2, ["angle"] = 1456013312},
  [9] = {["x"] = -1975, ["y"] = -3691, ["z"] = 0, ["order"] = 2, ["angle"] = 1485373440},
  [10] = {["x"] = -2261, ["y"] = -3016, ["z"] = 0, ["order"] = 2, ["angle"] = 1579614208},
  [11] = {["x"] = -2887, ["y"] = -2017, ["z"] = 36, ["order"] = 2, ["angle"] = 1976762368},
  [12] = {["x"] = -3716, ["y"] = -1528, ["z"] = 100, ["order"] = 2, ["angle"] = -1899298816},
  [13] = {["x"] = -4596, ["y"] = -1680, ["z"] = 159, ["order"] = 3, ["angle"] = -1256914944},
  [14] = {["x"] = -5228, ["y"] = -2367, ["z"] = 200, ["order"] = 3, ["angle"] = -812187648},
  [15] = {["x"] = -4956, ["y"] = -3274, ["z"] = 260, ["order"] = 4, ["angle"] = -143982592},
  [16] = {["x"] = -3995, ["y"] = -3565, ["z"] = 368, ["order"] = 4, ["angle"] = -3735552},
  [17] = {["x"] = -2966, ["y"] = -3569, ["z"] = 512, ["order"] = 4, ["angle"] = -16842752},
  [18] = {["x"] = -1964, ["y"] = -3572, ["z"] = 512, ["order"] = 4, ["angle"] = 8454144},
  [19] = {["x"] = -864, ["y"] = -3571, ["z"] = 512, ["order"] = 4, ["angle"] = 8454144},
  [20] = {["x"] = 248, ["y"] = -3692, ["z"] = 512, ["order"] = 4, ["angle"] = 412811264},
  [21] = {["x"] = 1038, ["y"] = -3337, ["z"] = 512, ["order"] = 4, ["angle"] = 1035665408},
  [22] = {["x"] = 1042, ["y"] = -2697, ["z"] = 512, ["order"] = 5, ["angle"] = 1180762112},
  [23] = {["x"] = 852, ["y"] = -1933, ["z"] = 512, ["order"] = 5, ["angle"] = 1119027200},
  [24] = {["x"] = 730, ["y"] = -1114, ["z"] = 512, ["order"] = 5, ["angle"] = 767098880},
  [25] = {["x"] = 955, ["y"] = -216, ["z"] = 512, ["order"] = 5, ["angle"] = 334299136},
  [26] = {["x"] = 1615, ["y"] = 331, ["z"] = 512, ["order"] = 5, ["angle"] = -389349376},
  [27] = {["x"] = 2338, ["y"] = -88, ["z"] = 512, ["order"] = 6, ["angle"] = -979697664},
  [28] = {["x"] = 2403, ["y"] = -913, ["z"] = 512, ["order"] = 6, ["angle"] = -936050688},
  [29] = {["x"] = 2391, ["y"] = -1717, ["z"] = 512, ["order"] = 6, ["angle"] = -456982528},
  [30] = {["x"] = 2788, ["y"] = -2473, ["z"] = 512, ["order"] = 6, ["angle"] = 64028672},
  [31] = {["x"] = 3667, ["y"] = -2711, ["z"] = 512, ["order"] = 6, ["angle"] = 612433920},
  [32] = {["x"] = 4044, ["y"] = -2075, ["z"] = 512, ["order"] = 6, ["angle"] = 1076690944},
  [33] = {["x"] = 4296, ["y"] = -1324, ["z"] = 512, ["order"] = 7, ["angle"] = 1096613888},
  [34] = {["x"] = 4337, ["y"] = -306, ["z"] = 512, ["order"] = 7, ["angle"] = 1096613888},
  [35] = {["x"] = 4408, ["y"] = 498, ["z"] = 512, ["order"] = 7, ["angle"] = 1155858432},
  [36] = {["x"] = 4345, ["y"] = 1455, ["z"] = 512, ["order"] = 8, ["angle"] = 1333854208},
  [37] = {["x"] = 4023, ["y"] = 2157, ["z"] = 512, ["order"] = 8, ["angle"] = 1437007872},
  [38] = {["x"] = 3408, ["y"] = 2906, ["z"] = 512, ["order"] = 8, ["angle"] = 1142226944},
  [39] = {["x"] = 3052, ["y"] = 4032, ["z"] = 769, ["order"] = 8, ["angle"] = -77791232},
  [40] = {["x"] = 4026, ["y"] = 3593, ["z"] = 482, ["order"] = 9, ["angle"] = -470220800},
  [41] = {["x"] = 4922, ["y"] = 2722, ["z"] = 124, ["order"] = 9, ["angle"] = -279379968},
  [42] = {["x"] = 5569, ["y"] = 2288, ["z"] = 29, ["order"] = 9, ["angle"] = 159449088},
  [43] = {["x"] = 6418, ["y"] = 2315, ["z"] = -10, ["order"] = 10, ["angle"] = 184352768},
  [44] = {["x"] = 7104, ["y"] = 2542, ["z"] = -10, ["order"] = 10, ["angle"] = 340197376},
  [45] = {["x"] = 7841, ["y"] = 2942, ["z"] = -10, ["order"] = 10, ["angle"] = -181075968},
  [46] = {["x"] = 8497, ["y"] = 2679, ["z"] = -10, ["order"] = 10, ["angle"] = -280297472},
  [47] = {["x"] = 9231, ["y"] = 2125, ["z"] = -10, ["order"] = 10, ["angle"] = 54591488},
  [48] = {["x"] = 9963, ["y"] = 2073, ["z"] = -10, ["order"] = 10, ["angle"] = 585039872},
  [49] = {["x"] = 10365, ["y"] = 2589, ["z"] = -10, ["order"] = 11, ["angle"] = 994246656},
  [50] = {["x"] = 10375, ["y"] = 3402, ["z"] = -10, ["order"] = 11, ["angle"] = 816381952},
  [51] = {["x"] = 10609, ["y"] = 4207, ["z"] = -10, ["order"] = 11, ["angle"] = 114491392},
  [52] = {["x"] = 11336, ["y"] = 4429, ["z"] = -10, ["order"] = 12, ["angle"] = -767754240},
  [53] = {["x"] = 11679, ["y"] = 3826, ["z"] = 15, ["order"] = 12, ["angle"] = -1152712704},
  [54] = {["x"] = 11690, ["y"] = 2937, ["z"] = 161, ["order"] = 12, ["angle"] = -1082589184},
  [55] = {["x"] = 11654, ["y"] = 1696, ["z"] = 335, ["order"] = 12, ["angle"] = -1154285568},
  [56] = {["x"] = 11091, ["y"] = -395, ["z"] = 268, ["order"] = 13, ["angle"] = -1120731136},
  [57] = {["x"] = 10427, ["y"] = -1151, ["z"] = 256, ["order"] = 13, ["angle"] = -1030160384},
  [58] = {["x"] = 10418, ["y"] = -2269, ["z"] = 256, ["order"] = 13, ["angle"] = -1040646144},
  [59] = {["x"] = 10404, ["y"] = -3156, ["z"] = 256, ["order"] = 13, ["angle"] = -855834624},
  [60] = {["x"] = 10729, ["y"] = -3872, ["z"] = 256, ["order"] = 13, ["angle"] = -723582976},
  [61] = {["x"] = 11526, ["y"] = -4740, ["z"] = 256, ["order"] = 13, ["angle"] = -1043791872},
  [62] = {["x"] = 11706, ["y"] = -5737, ["z"] = 256, ["order"] = 13, ["angle"] = -1801125888},
  [63] = {["x"] = 10950, ["y"] = -6406, ["z"] = 256, ["order"] = 14, ["angle"] = -2125266944},
  [64] = {["x"] = 10100, ["y"] = -6374, ["z"] = 256, ["order"] = 14, ["angle"] = 2130771968},
  [65] = {["x"] = 8892, ["y"] = -6326, ["z"] = 256, ["order"] = 14, ["angle"] = -2096693248},
  [66] = {["x"] = 7676, ["y"] = -6236, ["z"] = 256, ["order"] = 14, ["angle"] = -2037710848},
  [67] = {["x"] = 6231, ["y"] = -6305, ["z"] = 355, ["order"] = 14, ["angle"] = -1743454208},
  [68] = {["x"] = 5137, ["y"] = -6800, ["z"] = 625, ["order"] = 15, ["angle"] = -774832128},
  [69] = {["x"] = 5482, ["y"] = -7693, ["z"] = 496, ["order"] = 15, ["angle"] = -359202816},
  [70] = {["x"] = 6076, ["y"] = -8128, ["z"] = 502, ["order"] = 15, ["angle"] = -68616192},
  [71] = {["x"] = 6856, ["y"] = -8167, ["z"] = 661, ["order"] = 15, ["angle"] = -294453248},
  [72] = {["x"] = 7729, ["y"] = -8346, ["z"] = 666, ["order"] = 15, ["angle"] = -786497536},
  [73] = {["x"] = 8061, ["y"] = -9289, ["z"] = 666, ["order"] = 16, ["angle"] = -993853440},
  [74] = {["x"] = 8211, ["y"] = -10242, ["z"] = 666, ["order"] = 16, ["angle"] = -1280507904},
  [75] = {["x"] = 8160, ["y"] = -11325, ["z"] = 666, ["order"] = 16, ["angle"] = -1232404480},
  [76] = {["x"] = 7930, ["y"] = -12048, ["z"] = 666, ["order"] = 16, ["angle"] = -1916076032},
  [77] = {["x"] = 7343, ["y"] = -12317, ["z"] = 666, ["order"] = 17, ["angle"] = 2010578944},
  [78] = {["x"] = 6599, ["y"] = -12301, ["z"] = 666, ["order"] = 17, ["angle"] = 2131427328},
  [79] = {["x"] = 4702, ["y"] = -12263, ["z"] = 0, ["order"] = 17, ["angle"] = 2111897600},
  [80] = {["x"] = 3969, ["y"] = -12283, ["z"] = 0, ["order"] = 17, ["angle"] = 2108358656},
  [81] = {["x"] = 3172, ["y"] = -12236, ["z"] = -10, ["order"] = 17, ["angle"] = -2021851136},
  [82] = {["x"] = 2411, ["y"] = -12463, ["z"] = -10, ["order"] = 17, ["angle"] = -2110062592},
  [83] = {["x"] = 1505, ["y"] = -12667, ["z"] = -10, ["order"] = 17, ["angle"] = 1658519552},
  [84] = {["x"] = 788, ["y"] = -12212, ["z"] = -10, ["order"] = 18, ["angle"] = 1126629376},
  [85] = {["x"] = 828, ["y"] = -11523, ["z"] = -10, ["order"] = 18, ["angle"] = 574160896},
  [86] = {["x"] = 1402, ["y"] = -10920, ["z"] = -10, ["order"] = 18, ["angle"] = 550961152},
  [87] = {["x"] = 1987, ["y"] = -10340, ["z"] = -10, ["order"] = 18, ["angle"] = 947322880},
  [88] = {["x"] = 2610, ["y"] = -9705, ["z"] = -10, ["order"] = 18, ["angle"] = 656080896},
  [89] = {["x"] = 3071, ["y"] = -9049, ["z"] = -10, ["order"] = 18, ["angle"] = 1206321152},
  [90] = {["x"] = 3090, ["y"] = -8195, ["z"] = -10, ["order"] = 18, ["angle"] = 1561001984},
  [91] = {["x"] = 2267, ["y"] = -7201, ["z"] = -10, ["order"] = 18, ["angle"] = 1352073216},
  [92] = {["x"] = 2079, ["y"] = -6685, ["z"] = 0, ["order"] = 18, ["angle"] = 1138425856},
  [93] = {["x"] = 7895, ["y"] = 3141, ["z"] = -10, ["order"] = 10, ["angle"] = 628555776},
  [94] = {["x"] = 8402, ["y"] = 3736, ["z"] = -10, ["order"] = 10, ["angle"] = 537460736},
  [95] = {["x"] = 9074, ["y"] = 4423, ["z"] = -10, ["order"] = 10, ["angle"] = 690290688},
  [96] = {["x"] = 9395, ["y"] = 5101, ["z"] = -10, ["order"] = 10, ["angle"] = 742850560},
  [97] = {["x"] = 9828, ["y"] = 5907, ["z"] = -10, ["order"] = 10, ["angle"] = 203096064},
  [98] = {["x"] = 10658, ["y"] = 6281, ["z"] = -10, ["order"] = 11, ["angle"] = -78053376},
  [99] = {["x"] = 11517, ["y"] = 6345, ["z"] = -10, ["order"] = 11, ["angle"] = -237568000},
  [100] = {["x"] = 12358, ["y"] = 6138, ["z"] = -10, ["order"] = 11, ["angle"] = -681639936},
  [101] = {["x"] = 12743, ["y"] = 5647, ["z"] = -10, ["order"] = 11, ["angle"] = -1293352960},
  [102] = {["x"] = 12633, ["y"] = 5019, ["z"] = -10, ["order"] = 11, ["angle"] = -1652621312},
  [103] = {["x"] = 11992, ["y"] = 4079, ["z"] = -10, ["order"] = 12, ["angle"] = -1198981120},
  [104] = {["x"] = 5301, ["y"] = -12326, ["z"] = 0, ["order"] = 17, ["angle"] = 2097741824},
  [105] = {["x"] = 1752, ["y"] = -9868, ["z"] = -10, ["order"] = 18, ["angle"] = 1542127616},
  [106] = {["x"] = 1134, ["y"] = -9041, ["z"] = -10, ["order"] = 18, ["angle"] = 1147469824},
  [107] = {["x"] = 1074, ["y"] = -8198, ["z"] = -10, ["order"] = 18, ["angle"] = 660013056},
  [108] = {["x"] = 1613, ["y"] = -7398, ["z"] = -10, ["order"] = 18, ["angle"] = 811663360}
}
-- Respawn definitions for TOXIC PALACE (MAP32)
krespawn.points[32] = {
  [0] = 112,
  [1] = {["x"] = -7513, ["y"] = -2883, ["z"] = 410, ["order"] = 0, ["angle"] = 1060306944},
  [2] = {["x"] = -7532, ["y"] = -2095, ["z"] = 410, ["order"] = 0, ["angle"] = 1034485760},
  [3] = {["x"] = -7562, ["y"] = -898, ["z"] = 510, ["order"] = 0, ["angle"] = 986251264},
  [4] = {["x"] = -7471, ["y"] = 307, ["z"] = 520, ["order"] = 0, ["angle"] = 861732864},
  [5] = {["x"] = -7069, ["y"] = 1339, ["z"] = 520, ["order"] = 1, ["angle"] = 811008000},
  [6] = {["x"] = -6807, ["y"] = 2094, ["z"] = 520, ["order"] = 1, ["angle"] = 935264256},
  [7] = {["x"] = -6795, ["y"] = 3289, ["z"] = 520, ["order"] = 1, ["angle"] = 609812480},
  [8] = {["x"] = -6300, ["y"] = 3893, ["z"] = 520, ["order"] = 2, ["angle"] = 52756480},
  [9] = {["x"] = -5409, ["y"] = 4100, ["z"] = 520, ["order"] = 2, ["angle"] = -32178176},
  [10] = {["x"] = -4304, ["y"] = 4108, ["z"] = 520, ["order"] = 2, ["angle"] = 8585216},
  [11] = {["x"] = -3015, ["y"] = 4118, ["z"] = 520, ["order"] = 2, ["angle"] = 8585216},
  [12] = {["x"] = -2132, ["y"] = 4227, ["z"] = 590, ["order"] = 3, ["angle"] = -202047488},
  [13] = {["x"] = -1298, ["y"] = 4265, ["z"] = 850, ["order"] = 3, ["angle"] = -556597248},
  [14] = {["x"] = -751, ["y"] = 3701, ["z"] = 900, ["order"] = 3, ["angle"] = -1028849664},
  [15] = {["x"] = -611, ["y"] = 2902, ["z"] = 900, ["order"] = 3, ["angle"] = -1284702208},
  [16] = {["x"] = -719, ["y"] = 1994, ["z"] = 900, ["order"] = 3, ["angle"] = -1788149760},
  [17] = {["x"] = -1649, ["y"] = 1545, ["z"] = 900, ["order"] = 4, ["angle"] = -2099707904},
  [18] = {["x"] = -2704, ["y"] = 1665, ["z"] = 900, ["order"] = 4, ["angle"] = -2013855744},
  [19] = {["x"] = -3818, ["y"] = 1576, ["z"] = 900, ["order"] = 4, ["angle"] = -1382875136},
  [20] = {["x"] = -4347, ["y"] = 857, ["z"] = 900, ["order"] = 4, ["angle"] = -788987904},
  [21] = {["x"] = -4293, ["y"] = -432, ["z"] = 900, ["order"] = 5, ["angle"] = -13303808},
  [22] = {["x"] = -3802, ["y"] = 56, ["z"] = 900, ["order"] = 5, ["angle"] = -118423552},
  [23] = {["x"] = -2866, ["y"] = -144, ["z"] = 900, ["order"] = 5, ["angle"] = 47513600},
  [24] = {["x"] = -1870, ["y"] = 87, ["z"] = 900, ["order"] = 5, ["angle"] = -62193664},
  [25] = {["x"] = -1745, ["y"] = 520, ["z"] = 900, ["order"] = 5, ["angle"] = 138870784},
  [26] = {["x"] = -891, ["y"] = 725, ["z"] = 900, ["order"] = 5, ["angle"] = -316473344},
  [27] = {["x"] = -899, ["y"] = 71, ["z"] = 900, ["order"] = 5, ["angle"] = -18808832},
  [28] = {["x"] = -170, ["y"] = 273, ["z"] = 900, ["order"] = 5, ["angle"] = -222887936},
  [29] = {["x"] = 734, ["y"] = 68, ["z"] = 900, ["order"] = 5, ["angle"] = -102563840},
  [30] = {["x"] = 1551, ["y"] = -8, ["z"] = 950, ["order"] = 5, ["angle"] = -14221312},
  [31] = {["x"] = 2659, ["y"] = 139, ["z"] = 1035, ["order"] = 6, ["angle"] = -254476288},
  [32] = {["x"] = 3346, ["y"] = -124, ["z"] = 1030, ["order"] = 6, ["angle"] = -1054277632},
  [33] = {["x"] = 3623, ["y"] = -752, ["z"] = 1030, ["order"] = 6, ["angle"] = -1543438336},
  [34] = {["x"] = 3214, ["y"] = -1293, ["z"] = 1030, ["order"] = 6, ["angle"] = -2120024064},
  [35] = {["x"] = 2502, ["y"] = -1316, ["z"] = 1030, ["order"] = 7, ["angle"] = 2083192832},
  [36] = {["x"] = 196, ["y"] = -1058, ["z"] = 790, ["order"] = 7, ["angle"] = -1890254848},
  [37] = {["x"] = -1244, ["y"] = -1003, ["z"] = 790, ["order"] = 8, ["angle"] = -1096220672},
  [38] = {["x"] = -627, ["y"] = -1251, ["z"] = 790, ["order"] = 8, ["angle"] = -1036189696},
  [39] = {["x"] = -718, ["y"] = -2349, ["z"] = 790, ["order"] = 8, ["angle"] = -1362558976},
  [40] = {["x"] = -1116, ["y"] = -3209, ["z"] = 790, ["order"] = 8, ["angle"] = -1168572416},
  [41] = {["x"] = -1202, ["y"] = -4089, ["z"] = 790, ["order"] = 8, ["angle"] = -1477246976},
  [42] = {["x"] = -1766, ["y"] = -4933, ["z"] = 790, ["order"] = 8, ["angle"] = -1995636736},
  [43] = {["x"] = -2445, ["y"] = -5035, ["z"] = 800, ["order"] = 9, ["angle"] = -2058158080},
  [44] = {["x"] = -3260, ["y"] = -5004, ["z"] = 820, ["order"] = 9, ["angle"] = -1978335232},
  [45] = {["x"] = -4150, ["y"] = -4979, ["z"] = 825, ["order"] = 9, ["angle"] = -1654718464},
  [46] = {["x"] = -4910, ["y"] = -5384, ["z"] = 840, ["order"] = 9, ["angle"] = -1213136896},
  [47] = {["x"] = -5096, ["y"] = -6137, ["z"] = 870, ["order"] = 9, ["angle"] = -289341440},
  [48] = {["x"] = -4285, ["y"] = -6542, ["z"] = 895, ["order"] = 10, ["angle"] = 52887552},
  [49] = {["x"] = -3400, ["y"] = -6397, ["z"] = 890, ["order"] = 10, ["angle"] = -130482176},
  [50] = {["x"] = -2497, ["y"] = -6381, ["z"] = 920, ["order"] = 10, ["angle"] = -404291584},
  [51] = {["x"] = -1690, ["y"] = -6689, ["z"] = 930, ["order"] = 10, ["angle"] = -666042368},
  [52] = {["x"] = -1032, ["y"] = -7382, ["z"] = 960, ["order"] = 10, ["angle"] = -947847168},
  [53] = {["x"] = -725, ["y"] = -8199, ["z"] = 1020, ["order"] = 10, ["angle"] = -1040908288},
  [54] = {["x"] = -662, ["y"] = -8975, ["z"] = 1020, ["order"] = 10, ["angle"] = -1033043968},
  [55] = {["x"] = -806, ["y"] = -9874, ["z"] = 1020, ["order"] = 11, ["angle"] = -293797888},
  [56] = {["x"] = 5, ["y"] = -10235, ["z"] = 1020, ["order"] = 11, ["angle"] = 28114944},
  [57] = {["x"] = 1034, ["y"] = -10254, ["z"] = 1020, ["order"] = 11, ["angle"] = 28114944},
  [58] = {["x"] = 1869, ["y"] = -10232, ["z"] = 1020, ["order"] = 11, ["angle"] = 28114944},
  [59] = {["x"] = 2742, ["y"] = -10267, ["z"] = 1020, ["order"] = 11, ["angle"] = -224460800},
  [60] = {["x"] = 2755, ["y"] = -10198, ["z"] = 1020, ["order"] = 11, ["angle"] = -5832704},
  [61] = {["x"] = 3521, ["y"] = -10168, ["z"] = 1020, ["order"] = 11, ["angle"] = 815988736},
  [62] = {["x"] = 3461, ["y"] = -10299, ["z"] = 1020, ["order"] = 11, ["angle"] = -744030208},
  [63] = {["x"] = 3739, ["y"] = -9778, ["z"] = 1010, ["order"] = 11, ["angle"] = 957415424},
  [64] = {["x"] = 3725, ["y"] = -8905, ["z"] = 870, ["order"] = 11, ["angle"] = 679673856},
  [65] = {["x"] = 4105, ["y"] = -8183, ["z"] = 870, ["order"] = 11, ["angle"] = 248315904},
  [66] = {["x"] = 4917, ["y"] = -7850, ["z"] = 780, ["order"] = 11, ["angle"] = -438501376},
  [67] = {["x"] = 5549, ["y"] = -8270, ["z"] = 720, ["order"] = 12, ["angle"] = -887422976},
  [68] = {["x"] = 5785, ["y"] = -9251, ["z"] = 660, ["order"] = 12, ["angle"] = -1464008704},
  [69] = {["x"] = 5444, ["y"] = -10015, ["z"] = 560, ["order"] = 12, ["angle"] = -2007040000},
  [70] = {["x"] = 4424, ["y"] = -10158, ["z"] = 500, ["order"] = 12, ["angle"] = 2137194496},
  [71] = {["x"] = 3332, ["y"] = -10280, ["z"] = 500, ["order"] = 13, ["angle"] = -2144272384},
  [72] = {["x"] = 1293, ["y"] = -10192, ["z"] = 500, ["order"] = 13, ["angle"] = -2005991424},
  [73] = {["x"] = 528, ["y"] = -10167, ["z"] = 500, ["order"] = 13, ["angle"] = -1511849984},
  [74] = {["x"] = 145, ["y"] = -10687, ["z"] = 500, ["order"] = 14, ["angle"] = -1057816576},
  [75] = {["x"] = 190, ["y"] = -11518, ["z"] = 500, ["order"] = 14, ["angle"] = -1245118464},
  [76] = {["x"] = 134, ["y"] = -12301, ["z"] = 500, ["order"] = 14, ["angle"] = -1637679104},
  [77] = {["x"] = -342, ["y"] = -12796, ["z"] = 500, ["order"] = 14, ["angle"] = 2056847360},
  [78] = {["x"] = -1173, ["y"] = -12859, ["z"] = 500, ["order"] = 14, ["angle"] = 2039021568},
  [79] = {["x"] = -2162, ["y"] = -12845, ["z"] = 500, ["order"] = 14, ["angle"] = 2026045440},
  [80] = {["x"] = -2009, ["y"] = -12215, ["z"] = 500, ["order"] = 15, ["angle"] = 1973616640},
  [81] = {["x"] = -2918, ["y"] = -11667, ["z"] = 500, ["order"] = 15, ["angle"] = -1951596544},
  [82] = {["x"] = -2771, ["y"] = -12892, ["z"] = 500, ["order"] = 15, ["angle"] = 2131820544},
  [83] = {["x"] = -3313, ["y"] = -12909, ["z"] = 500, ["order"] = 15, ["angle"] = -2086338560},
  [84] = {["x"] = -3453, ["y"] = -11739, ["z"] = 500, ["order"] = 15, ["angle"] = 2091843584},
  [85] = {["x"] = -5082, ["y"] = -12356, ["z"] = 500, ["order"] = 15, ["angle"] = 2142699520},
  [86] = {["x"] = -5826, ["y"] = -12476, ["z"] = 500, ["order"] = 15, ["angle"] = 2058682368},
  [87] = {["x"] = -6874, ["y"] = -12437, ["z"] = 500, ["order"] = 15, ["angle"] = 1654718464},
  [88] = {["x"] = -7544, ["y"] = -11819, ["z"] = 500, ["order"] = 16, ["angle"] = 1069088768},
  [89] = {["x"] = -7855, ["y"] = -10520, ["z"] = 360, ["order"] = 16, ["angle"] = 1065287680},
  [90] = {["x"] = -7532, ["y"] = -10577, ["z"] = 370, ["order"] = 16, ["angle"] = 1065549824},
  [91] = {["x"] = -7557, ["y"] = -8117, ["z"] = 170, ["order"] = 16, ["angle"] = 1101594624},
  [92] = {["x"] = -7562, ["y"] = -7376, ["z"] = 100, ["order"] = 16, ["angle"] = 1038942208},
  [93] = {["x"] = -7532, ["y"] = -6452, ["z"] = 100, ["order"] = 16, ["angle"] = 1104871424},
  [94] = {["x"] = -7535, ["y"] = -5191, ["z"] = 140, ["order"] = 16, ["angle"] = 1065156608},
  [95] = {["x"] = -7531, ["y"] = -4251, ["z"] = 340, ["order"] = 16, ["angle"] = 1065156608},
  [96] = {["x"] = -7524, ["y"] = -3384, ["z"] = 410, ["order"] = 16, ["angle"] = 1065156608},
  [97] = {["x"] = -7626, ["y"] = 730, ["z"] = 520, ["order"] = 1, ["angle"] = 1328611328},
  [98] = {["x"] = -8038, ["y"] = 1559, ["z"] = 520, ["order"] = 1, ["angle"] = 1291255808},
  [99] = {["x"] = -8353, ["y"] = 2478, ["z"] = 520, ["order"] = 1, ["angle"] = 914161664},
  [100] = {["x"] = -8438, ["y"] = 3400, ["z"] = 520, ["order"] = 1, ["angle"] = 663420928},
  [101] = {["x"] = -8041, ["y"] = 4140, ["z"] = 520, ["order"] = 1, ["angle"] = -94437376},
  [102] = {["x"] = -7516, ["y"] = 4091, ["z"] = 520, ["order"] = 1, ["angle"] = 2424832},
  [103] = {["x"] = -6643, ["y"] = 4126, ["z"] = 520, ["order"] = 2, ["angle"] = 2424832},
  [104] = {["x"] = 3624, ["y"] = -10583, ["z"] = 1020, ["order"] = 11, ["angle"] = -864747520},
  [105] = {["x"] = 3708, ["y"] = -11261, ["z"] = 920, ["order"] = 11, ["angle"] = -812056576},
  [106] = {["x"] = 3919, ["y"] = -12077, ["z"] = 870, ["order"] = 11, ["angle"] = -385810432},
  [107] = {["x"] = 4523, ["y"] = -12648, ["z"] = 840, ["order"] = 11, ["angle"] = 108462080},
  [108] = {["x"] = 5168, ["y"] = -12688, ["z"] = 740, ["order"] = 11, ["angle"] = 550043648},
  [109] = {["x"] = 5670, ["y"] = -12319, ["z"] = 720, ["order"] = 12, ["angle"] = 1054539776},
  [110] = {["x"] = 5909, ["y"] = -11611, ["z"] = 710, ["order"] = 12, ["angle"] = 1300299776},
  [111] = {["x"] = 5898, ["y"] = -10808, ["z"] = 600, ["order"] = 12, ["angle"] = 1643839488},
  [112] = {["x"] = 5359, ["y"] = -10325, ["z"] = 560, ["order"] = 12, ["angle"] = 2043084800}
}
-- Respawn definitions for ANCIENT TOMB (MAP33)
krespawn.points[33] = {
  [0] = 218,
  [1] = {["x"] = -8319, ["y"] = -11222, ["z"] = 2000, ["order"] = 0, ["angle"] = 1071448064},
  [2] = {["x"] = -8299, ["y"] = -10305, ["z"] = 2000, ["order"] = 0, ["angle"] = 1071448064},
  [3] = {["x"] = -8244, ["y"] = -9389, ["z"] = 2000, ["order"] = 0, ["angle"] = 1104347136},
  [4] = {["x"] = -8147, ["y"] = -8300, ["z"] = 2000, ["order"] = 0, ["angle"] = 1208549376},
  [5] = {["x"] = -8128, ["y"] = -7125, ["z"] = 2000, ["order"] = 1, ["angle"] = 1235419136},
  [6] = {["x"] = -8143, ["y"] = -5778, ["z"] = 2005, ["order"] = 1, ["angle"] = 1397161984},
  [7] = {["x"] = -8413, ["y"] = -4904, ["z"] = 2000, ["order"] = 2, ["angle"] = 1970995200},
  [8] = {["x"] = -9172, ["y"] = -4588, ["z"] = 1978, ["order"] = 2, ["angle"] = -2091843584},
  [9] = {["x"] = -10194, ["y"] = -4663, ["z"] = 1814, ["order"] = 2, ["angle"] = 2010578944},
  [10] = {["x"] = -11329, ["y"] = -4670, ["z"] = 1632, ["order"] = 2, ["angle"] = 1799028736},
  [11] = {["x"] = -12265, ["y"] = -4318, ["z"] = 1635, ["order"] = 3, ["angle"] = 1317076992},
  [12] = {["x"] = -12653, ["y"] = -3722, ["z"] = 1630, ["order"] = 3, ["angle"] = 1131216896},
  [13] = {["x"] = -12754, ["y"] = -2798, ["z"] = 1630, ["order"] = 3, ["angle"] = 896729088},
  [14] = {["x"] = -12606, ["y"] = -1720, ["z"] = 1630, ["order"] = 3, ["angle"] = 958070784},
  [15] = {["x"] = -12588, ["y"] = -829, ["z"] = 1630, ["order"] = 3, ["angle"] = 888864768},
  [16] = {["x"] = -12330, ["y"] = 30, ["z"] = 1630, ["order"] = 3, ["angle"] = 421724160},
  [17] = {["x"] = -11778, ["y"] = 474, ["z"] = 1630, ["order"] = 3, ["angle"] = -297336832},
  [18] = {["x"] = -11053, ["y"] = 245, ["z"] = 1630, ["order"] = 4, ["angle"] = -672595968},
  [19] = {["x"] = -10708, ["y"] = -506, ["z"] = 1630, ["order"] = 4, ["angle"] = -1013514240},
  [20] = {["x"] = -10718, ["y"] = -1355, ["z"] = 1630, ["order"] = 4, ["angle"] = -869859328},
  [21] = {["x"] = -10668, ["y"] = -2269, ["z"] = 1450, ["order"] = 4, ["angle"] = -545062912},
  [22] = {["x"] = -10195, ["y"] = -2959, ["z"] = 1350, ["order"] = 5, ["angle"] = -35848192},
  [23] = {["x"] = -9266, ["y"] = -3299, ["z"] = 1350, ["order"] = 5, ["angle"] = -10551296},
  [24] = {["x"] = -9071, ["y"] = -2497, ["z"] = 1350, ["order"] = 5, ["angle"] = 183173120},
  [25] = {["x"] = -8084, ["y"] = -2171, ["z"] = 1350, ["order"] = 5, ["angle"] = -175964160},
  [26] = {["x"] = -8012, ["y"] = -3380, ["z"] = 1350, ["order"] = 5, ["angle"] = -125894656},
  [27] = {["x"] = -6965, ["y"] = -2905, ["z"] = 1350, ["order"] = 5, ["angle"] = 213712896},
  [28] = {["x"] = -6216, ["y"] = -2571, ["z"] = 1355, ["order"] = 5, ["angle"] = 82116608},
  [29] = {["x"] = -4729, ["y"] = -2335, ["z"] = 1350, ["order"] = 5, ["angle"] = 463667200},
  [30] = {["x"] = -4089, ["y"] = -1907, ["z"] = 1360, ["order"] = 6, ["angle"] = 1132134400},
  [31] = {["x"] = -4059, ["y"] = -937, ["z"] = 1400, ["order"] = 6, ["angle"] = 1683161088},
  [32] = {["x"] = -4416, ["y"] = -1232, ["z"] = 1411, ["order"] = 6, ["angle"] = 1738735616},
  [33] = {["x"] = -5128, ["y"] = -691, ["z"] = 1495, ["order"] = 6, ["angle"] = 1507786752},
  [34] = {["x"] = -4483, ["y"] = -251, ["z"] = 1486, ["order"] = 6, ["angle"] = 1619197952},
  [35] = {["x"] = -5281, ["y"] = 150, ["z"] = 1570, ["order"] = 6, ["angle"] = 1526923264},
  [36] = {["x"] = -6027, ["y"] = 755, ["z"] = 1570, ["order"] = 6, ["angle"] = 677576704},
  [37] = {["x"] = -6107, ["y"] = 2007, ["z"] = 1560, ["order"] = 7, ["angle"] = 31129600},
  [38] = {["x"] = -5915, ["y"] = 1594, ["z"] = 1570, ["order"] = 7, ["angle"] = -3342336},
  [39] = {["x"] = -5122, ["y"] = 1773, ["z"] = 1560, ["order"] = 7, ["angle"] = -27066368},
  [40] = {["x"] = -4182, ["y"] = 1830, ["z"] = 1560, ["order"] = 7, ["angle"] = 79757312},
  [41] = {["x"] = -3225, ["y"] = 1616, ["z"] = 1560, ["order"] = 7, ["angle"] = -19333120},
  [42] = {["x"] = -3208, ["y"] = 2011, ["z"] = 1560, ["order"] = 7, ["angle"] = -21692416},
  [43] = {["x"] = -2121, ["y"] = 1813, ["z"] = 1560, ["order"] = 7, ["angle"] = -13697024},
  [44] = {["x"] = -1423, ["y"] = 1854, ["z"] = 1560, ["order"] = 7, ["angle"] = -143720448},
  [45] = {["x"] = -482, ["y"] = 1797, ["z"] = 1560, ["order"] = 7, ["angle"] = -555941888},
  [46] = {["x"] = 96, ["y"] = 1241, ["z"] = 1560, ["order"] = 8, ["angle"] = -949944320},
  [47] = {["x"] = 325, ["y"] = 329, ["z"] = 1560, ["order"] = 8, ["angle"] = -1088356352},
  [48] = {["x"] = 289, ["y"] = -625, ["z"] = 1560, ["order"] = 8, ["angle"] = -1091108864},
  [49] = {["x"] = 192, ["y"] = -1882, ["z"] = 1376, ["order"] = 8, ["angle"] = -903806976},
  [50] = {["x"] = 160, ["y"] = -2859, ["z"] = 1350, ["order"] = 9, ["angle"] = -911671296},
  [51] = {["x"] = 203, ["y"] = -3876, ["z"] = 1350, ["order"] = 9, ["angle"] = -686751744},
  [52] = {["x"] = 420, ["y"] = -4868, ["z"] = 1355, ["order"] = 9, ["angle"] = -229572608},
  [53] = {["x"] = 1021, ["y"] = -5424, ["z"] = 1350, ["order"] = 9, ["angle"] = 152764416},
  [54] = {["x"] = 1967, ["y"] = -5553, ["z"] = 1350, ["order"] = 10, ["angle"] = 315817984},
  [55] = {["x"] = 2713, ["y"] = -5133, ["z"] = 1350, ["order"] = 10, ["angle"] = 992804864},
  [56] = {["x"] = 2876, ["y"] = -4297, ["z"] = 1350, ["order"] = 10, ["angle"] = 1012596736},
  [57] = {["x"] = 2885, ["y"] = -3201, ["z"] = 1350, ["order"] = 10, ["angle"] = 660668416},
  [58] = {["x"] = 3325, ["y"] = -2429, ["z"] = 1350, ["order"] = 11, ["angle"] = 96403456},
  [59] = {["x"] = 4134, ["y"] = -2244, ["z"] = 1190, ["order"] = 11, ["angle"] = 290783232},
  [60] = {["x"] = 5068, ["y"] = -1957, ["z"] = 1010, ["order"] = 12, ["angle"] = 594608128},
  [61] = {["x"] = 5699, ["y"] = -1370, ["z"] = 1000, ["order"] = 12, ["angle"] = 1070923776},
  [62] = {["x"] = 5654, ["y"] = -652, ["z"] = 1000, ["order"] = 12, ["angle"] = 1550385152},
  [63] = {["x"] = 4975, ["y"] = 163, ["z"] = 1000, ["order"] = 12, ["angle"] = 1553006592},
  [64] = {["x"] = 4263, ["y"] = 850, ["z"] = 1000, ["order"] = 13, ["angle"] = 1520762880},
  [65] = {["x"] = 3633, ["y"] = 1645, ["z"] = 1000, ["order"] = 13, ["angle"] = 1389821952},
  [66] = {["x"] = 3265, ["y"] = 2658, ["z"] = 1000, ["order"] = 13, ["angle"] = 1220870144},
  [67] = {["x"] = 3018, ["y"] = 3907, ["z"] = 1000, ["order"] = 13, ["angle"] = 1697054720},
  [68] = {["x"] = 2480, ["y"] = 4610, ["z"] = 1000, ["order"] = 14, ["angle"] = -2048196608},
  [69] = {["x"] = 1554, ["y"] = 4713, ["z"] = 1005, ["order"] = 14, ["angle"] = -2098921472},
  [70] = {["x"] = 836, ["y"] = 4713, ["z"] = 1000, ["order"] = 14, ["angle"] = -2089746432},
  [71] = {["x"] = -249, ["y"] = 4805, ["z"] = 1000, ["order"] = 15, ["angle"] = -1911095296},
  [72] = {["x"] = -1197, ["y"] = 4597, ["z"] = 1000, ["order"] = 16, ["angle"] = -1683554304},
  [73] = {["x"] = -2016, ["y"] = 3888, ["z"] = 1005, ["order"] = 17, ["angle"] = -1830092800},
  [74] = {["x"] = -2851, ["y"] = 3229, ["z"] = 1005, ["order"] = 17, ["angle"] = 2075721728},
  [75] = {["x"] = -4014, ["y"] = 2981, ["z"] = 1005, ["order"] = 17, ["angle"] = 1812135936},
  [76] = {["x"] = -5004, ["y"] = 3308, ["z"] = 1005, ["order"] = 17, ["angle"] = 1688666112},
  [77] = {["x"] = -5843, ["y"] = 3982, ["z"] = 1005, ["order"] = 17, ["angle"] = 1807155200},
  [78] = {["x"] = -6776, ["y"] = 4473, ["z"] = 1005, ["order"] = 17, ["angle"] = 2050031616},
  [79] = {["x"] = -7734, ["y"] = 4669, ["z"] = 1000, ["order"] = 1, ["angle"] = -2137849856},
  [80] = {["x"] = -8790, ["y"] = 4697, ["z"] = 1000, ["order"] = 1, ["angle"] = -2124742656},
  [81] = {["x"] = -10002, ["y"] = 4650, ["z"] = 1000, ["order"] = 1, ["angle"] = 1834156032},
  [82] = {["x"] = -10936, ["y"] = 4976, ["z"] = 990, ["order"] = 2, ["angle"] = 1268318208},
  [83] = {["x"] = -11587, ["y"] = 4865, ["z"] = 1000, ["order"] = 2, ["angle"] = 1117192192},
  [84] = {["x"] = -11421, ["y"] = 5670, ["z"] = 1005, ["order"] = 2, ["angle"] = 911933440},
  [85] = {["x"] = -11053, ["y"] = 6978, ["z"] = 1000, ["order"] = 2, ["angle"] = 1145765888},
  [86] = {["x"] = -11637, ["y"] = 6973, ["z"] = 1000, ["order"] = 2, ["angle"] = 1179713536},
  [87] = {["x"] = -11358, ["y"] = 7793, ["z"] = 1000, ["order"] = 2, ["angle"] = 1044185088},
  [88] = {["x"] = -11132, ["y"] = 8455, ["z"] = 1000, ["order"] = 2, ["angle"] = 1299120128},
  [89] = {["x"] = -11459, ["y"] = 9177, ["z"] = 1000, ["order"] = 3, ["angle"] = 1908473856},
  [90] = {["x"] = -12061, ["y"] = 9470, ["z"] = 1000, ["order"] = 3, ["angle"] = -2145714176},
  [91] = {["x"] = -12830, ["y"] = 9455, ["z"] = 1005, ["order"] = 3, ["angle"] = -2059468800},
  [92] = {["x"] = -13540, ["y"] = 9310, ["z"] = 1000, ["order"] = 3, ["angle"] = -2136276992},
  [93] = {["x"] = -14501, ["y"] = 9233, ["z"] = 1000, ["order"] = 3, ["angle"] = 1748172800},
  [94] = {["x"] = -15388, ["y"] = 9640, ["z"] = 1000, ["order"] = 3, ["angle"] = 1439498240},
  [95] = {["x"] = -15780, ["y"] = 10276, ["z"] = 1000, ["order"] = 3, ["angle"] = 1297285120},
  [96] = {["x"] = -16002, ["y"] = 11065, ["z"] = 1000, ["order"] = 3, ["angle"] = 753598464},
  [97] = {["x"] = -15697, ["y"] = 11817, ["z"] = 1000, ["order"] = 4, ["angle"] = 417660928},
  [98] = {["x"] = -15072, ["y"] = 12372, ["z"] = 1000, ["order"] = 4, ["angle"] = -208601088},
  [99] = {["x"] = -14367, ["y"] = 12350, ["z"] = 1000, ["order"] = 4, ["angle"] = -313720832},
  [100] = {["x"] = -13694, ["y"] = 11915, ["z"] = 1005, ["order"] = 4, ["angle"] = -373096448},
  [101] = {["x"] = -13000, ["y"] = 11531, ["z"] = 1000, ["order"] = 4, ["angle"] = -84738048},
  [102] = {["x"] = -12195, ["y"] = 11501, ["z"] = 1005, ["order"] = 4, ["angle"] = 143327232},
  [103] = {["x"] = -11145, ["y"] = 11768, ["z"] = 1000, ["order"] = 4, ["angle"] = -9371648},
  [104] = {["x"] = -10370, ["y"] = 11616, ["z"] = 1005, ["order"] = 5, ["angle"] = -894894080},
  [105] = {["x"] = -10119, ["y"] = 11028, ["z"] = 1000, ["order"] = 5, ["angle"] = -1072758784},
  [106] = {["x"] = -10201, ["y"] = 10268, ["z"] = 1120, ["order"] = 5, ["angle"] = -1047855104},
  [107] = {["x"] = -10253, ["y"] = 9361, ["z"] = 1260, ["order"] = 5, ["angle"] = -739573760},
  [108] = {["x"] = -9805, ["y"] = 8642, ["z"] = 1305, ["order"] = 6, ["angle"] = -72155136},
  [109] = {["x"] = -9070, ["y"] = 8498, ["z"] = 1305, ["order"] = 6, ["angle"] = 63373312},
  [110] = {["x"] = -8156, ["y"] = 8448, ["z"] = 1305, ["order"] = 6, ["angle"] = 214499328},
  [111] = {["x"] = -7265, ["y"] = 8690, ["z"] = 1310, ["order"] = 7, ["angle"] = 700121088},
  [112] = {["x"] = -6903, ["y"] = 9281, ["z"] = 1350, ["order"] = 7, ["angle"] = 926220288},
  [113] = {["x"] = -6807, ["y"] = 9926, ["z"] = 1450, ["order"] = 7, ["angle"] = 1602027520},
  [114] = {["x"] = -7305, ["y"] = 10403, ["z"] = 1450, ["order"] = 8, ["angle"] = 2092498944},
  [115] = {["x"] = -8069, ["y"] = 10506, ["z"] = 1550, ["order"] = 8, ["angle"] = -2135097344},
  [116] = {["x"] = -8976, ["y"] = 10445, ["z"] = 1690, ["order"] = 8, ["angle"] = -2112159744},
  [117] = {["x"] = -9988, ["y"] = 10357, ["z"] = 1740, ["order"] = 8, ["angle"] = 2060386304},
  [118] = {["x"] = -10990, ["y"] = 10377, ["z"] = 1745, ["order"] = 8, ["angle"] = 2082668544},
  [119] = {["x"] = -12015, ["y"] = 10362, ["z"] = 1740, ["order"] = 8, ["angle"] = 2117271552},
  [120] = {["x"] = -13152, ["y"] = 10338, ["z"] = 1740, ["order"] = 8, ["angle"] = 1721434112},
  [121] = {["x"] = -13943, ["y"] = 10762, ["z"] = 1745, ["order"] = 10, ["angle"] = 1237254144},
  [122] = {["x"] = -14137, ["y"] = 11439, ["z"] = 1740, ["order"] = 10, ["angle"] = 1165426688},
  [123] = {["x"] = -14200, ["y"] = 12574, ["z"] = 1740, ["order"] = 10, ["angle"] = 1043791872},
  [124] = {["x"] = -14149, ["y"] = 13396, ["z"] = 1740, ["order"] = 11, ["angle"] = 1044316160},
  [125] = {["x"] = -14081, ["y"] = 14583, ["z"] = 1740, ["order"] = 11, ["angle"] = 1322582016},
  [126] = {["x"] = -14418, ["y"] = 15376, ["z"] = 1740, ["order"] = 12, ["angle"] = 1982660608},
  [127] = {["x"] = -15218, ["y"] = 15491, ["z"] = 1780, ["order"] = 12, ["angle"] = -2002321408},
  [128] = {["x"] = -16168, ["y"] = 15261, ["z"] = 1850, ["order"] = 12, ["angle"] = 1882390528},
  [129] = {["x"] = -16980, ["y"] = 15587, ["z"] = 1890, ["order"] = 13, ["angle"] = 1229914112},
  [130] = {["x"] = -17315, ["y"] = 16269, ["z"] = 1930, ["order"] = 13, ["angle"] = 1016528896},
  [131] = {["x"] = -17322, ["y"] = 17051, ["z"] = 1990, ["order"] = 13, ["angle"] = 611516416},
  [132] = {["x"] = -16666, ["y"] = 17652, ["z"] = 2000, ["order"] = 14, ["angle"] = 121307136},
  [133] = {["x"] = -15905, ["y"] = 17668, ["z"] = 2060, ["order"] = 14, ["angle"] = -7274496},
  [134] = {["x"] = -15031, ["y"] = 17691, ["z"] = 2130, ["order"] = 15, ["angle"] = 92340224},
  [135] = {["x"] = -13953, ["y"] = 17800, ["z"] = 2130, ["order"] = 16, ["angle"] = -21168128},
  [136] = {["x"] = -12910, ["y"] = 17849, ["z"] = 2090, ["order"] = 16, ["angle"] = 170328064},
  [137] = {["x"] = -12914, ["y"] = 17749, ["z"] = 2090, ["order"] = 16, ["angle"] = -182779904},
  [138] = {["x"] = -11971, ["y"] = 17463, ["z"] = 2050, ["order"] = 16, ["angle"] = 90374144},
  [139] = {["x"] = -11913, ["y"] = 18136, ["z"] = 2050, ["order"] = 16, ["angle"] = -81461248},
  [140] = {["x"] = -10958, ["y"] = 17856, ["z"] = 2010, ["order"] = 16, ["angle"] = -17104896},
  [141] = {["x"] = -9973, ["y"] = 17819, ["z"] = 1990, ["order"] = 16, ["angle"] = 149094400},
  [142] = {["x"] = -9988, ["y"] = 17757, ["z"] = 1990, ["order"] = 16, ["angle"] = -140312576},
  [143] = {["x"] = -8959, ["y"] = 17565, ["z"] = 1950, ["order"] = 16, ["angle"] = -22478848},
  [144] = {["x"] = -8956, ["y"] = 17958, ["z"] = 1950, ["order"] = 16, ["angle"] = 64159744},
  [145] = {["x"] = -7226, ["y"] = 17751, ["z"] = 1520, ["order"] = 17, ["angle"] = 3211264},
  [146] = {["x"] = -6216, ["y"] = 17739, ["z"] = 1520, ["order"] = 18, ["angle"] = 3211264},
  [147] = {["x"] = -2352, ["y"] = 17751, ["z"] = 500, ["order"] = 1, ["angle"] = 3211264},
  [148] = {["x"] = -1407, ["y"] = 17755, ["z"] = 500, ["order"] = 1, ["angle"] = 3211264},
  [149] = {["x"] = -474, ["y"] = 17760, ["z"] = 500, ["order"] = 1, ["angle"] = 3211264},
  [150] = {["x"] = 656, ["y"] = 17765, ["z"] = 500, ["order"] = 1, ["angle"] = 3211264},
  [151] = {["x"] = 1746, ["y"] = 17841, ["z"] = 500, ["order"] = 1, ["angle"] = -69140480},
  [152] = {["x"] = 2924, ["y"] = 17827, ["z"] = 500, ["order"] = 1, ["angle"] = -435486720},
  [153] = {["x"] = 3652, ["y"] = 17482, ["z"] = 500, ["order"] = 2, ["angle"] = -926482432},
  [154] = {["x"] = 3801, ["y"] = 16285, ["z"] = 500, ["order"] = 2, ["angle"] = -1296105472},
  [155] = {["x"] = 3478, ["y"] = 15506, ["z"] = 500, ["order"] = 2, ["angle"] = -1891434496},
  [156] = {["x"] = 2502, ["y"] = 15244, ["z"] = 500, ["order"] = 2, ["angle"] = -1994588160},
  [157] = {["x"] = 1414, ["y"] = 14922, ["z"] = 500, ["order"] = 2, ["angle"] = -1845166080},
  [158] = {["x"] = 272, ["y"] = 14589, ["z"] = 500, ["order"] = 2, ["angle"] = -1904934912},
  [159] = {["x"] = -783, ["y"] = 14210, ["z"] = 500, ["order"] = 2, ["angle"] = -1486422016},
  [160] = {["x"] = -1489, ["y"] = 13218, ["z"] = 500, ["order"] = 3, ["angle"] = -1018363904},
  [161] = {["x"] = -1517, ["y"] = 11953, ["z"] = 450, ["order"] = 3, ["angle"] = -1029636096},
  [162] = {["x"] = -1385, ["y"] = 11136, ["z"] = 450, ["order"] = 3, ["angle"] = -1139343360},
  [163] = {["x"] = -1276, ["y"] = 10148, ["z"] = 455, ["order"] = 3, ["angle"] = -1223491584},
  [164] = {["x"] = -1234, ["y"] = 8937, ["z"] = 450, ["order"] = 3, ["angle"] = -1382875136},
  [165] = {["x"] = -1595, ["y"] = 7838, ["z"] = 450, ["order"] = 3, ["angle"] = -1779761152},
  [166] = {["x"] = -2205, ["y"] = 7330, ["z"] = 485, ["order"] = 4, ["angle"] = -2106785792},
  [167] = {["x"] = -3111, ["y"] = 7123, ["z"] = 625, ["order"] = 4, ["angle"] = 1911357440},
  [168] = {["x"] = -4034, ["y"] = 7310, ["z"] = 775, ["order"] = 4, ["angle"] = 1580924928},
  [169] = {["x"] = -4547, ["y"] = 7996, ["z"] = 810, ["order"] = 4, ["angle"] = 1065549824},
  [170] = {["x"] = -4704, ["y"] = 8887, ["z"] = 950, ["order"] = 4, ["angle"] = 890437632},
  [171] = {["x"] = -4582, ["y"] = 9672, ["z"] = 1070, ["order"] = 4, ["angle"] = 468910080},
  [172] = {["x"] = -3981, ["y"] = 10231, ["z"] = 1100, ["order"] = 5, ["angle"] = 28114944},
  [173] = {["x"] = -2989, ["y"] = 10332, ["z"] = 1100, ["order"] = 6, ["angle"] = 6225920},
  [174] = {["x"] = -1907, ["y"] = 10352, ["z"] = 1105, ["order"] = 6, ["angle"] = 6225920},
  [175] = {["x"] = -807, ["y"] = 10364, ["z"] = 1105, ["order"] = 6, ["angle"] = 6225920},
  [176] = {["x"] = 13, ["y"] = 10346, ["z"] = 1100, ["order"] = 6, ["angle"] = 22478848},
  [177] = {["x"] = 1754, ["y"] = 10372, ["z"] = 505, ["order"] = 7, ["angle"] = 22478848},
  [178] = {["x"] = 2633, ["y"] = 10441, ["z"] = 505, ["order"] = 7, ["angle"] = -127467520},
  [179] = {["x"] = 3745, ["y"] = 10555, ["z"] = 500, ["order"] = 7, ["angle"] = -253952000},
  [180] = {["x"] = 4731, ["y"] = 10363, ["z"] = 500, ["order"] = 7, ["angle"] = -517668864},
  [181] = {["x"] = 5236, ["y"] = 9629, ["z"] = 510, ["order"] = 7, ["angle"] = -649527296},
  [182] = {["x"] = 5640, ["y"] = 8704, ["z"] = 515, ["order"] = 7, ["angle"] = -393936896},
  [183] = {["x"] = 6210, ["y"] = 8168, ["z"] = 510, ["order"] = 7, ["angle"] = 180158464},
  [184] = {["x"] = 7093, ["y"] = 8304, ["z"] = 515, ["order"] = 8, ["angle"] = 981532672},
  [185] = {["x"] = 7254, ["y"] = 9100, ["z"] = 600, ["order"] = 8, ["angle"] = 1018626048},
  [186] = {["x"] = 7164, ["y"] = 10046, ["z"] = 750, ["order"] = 8, ["angle"] = 985595904},
  [187] = {["x"] = 7045, ["y"] = 11087, ["z"] = 910, ["order"] = 8, ["angle"] = 836042752},
  [188] = {["x"] = 7039, ["y"] = 11842, ["z"] = 1000, ["order"] = 8, ["angle"] = 548208640},
  [189] = {["x"] = 7396, ["y"] = 12578, ["z"] = 1010, ["order"] = 9, ["angle"] = 128385024},
  [190] = {["x"] = 8298, ["y"] = 12752, ["z"] = 1010, ["order"] = 9, ["angle"] = 70057984},
  [191] = {["x"] = 11801, ["y"] = 12816, ["z"] = 1900, ["order"] = 9, ["angle"] = 65536},
  [192] = {["x"] = 12558, ["y"] = 12816, ["z"] = 1900, ["order"] = 9, ["angle"] = 65536},
  [193] = {["x"] = 13423, ["y"] = 12908, ["z"] = 1905, ["order"] = 10, ["angle"] = -95092736},
  [194] = {["x"] = 14471, ["y"] = 12925, ["z"] = 1905, ["order"] = 10, ["angle"] = -178585600},
  [195] = {["x"] = 15915, ["y"] = 12783, ["z"] = 1900, ["order"] = 10, ["angle"] = -264568832},
  [196] = {["x"] = 16897, ["y"] = 12409, ["z"] = 1905, ["order"] = 10, ["angle"] = -576520192},
  [197] = {["x"] = 17822, ["y"] = 11601, ["z"] = 1710, ["order"] = 11, ["angle"] = -583860224},
  [198] = {["x"] = 18709, ["y"] = 10423, ["z"] = 1525, ["order"] = 12, ["angle"] = -828047360},
  [199] = {["x"] = 19103, ["y"] = 9581, ["z"] = 1525, ["order"] = 12, ["angle"] = -1016135680},
  [200] = {["x"] = 19239, ["y"] = 8613, ["z"] = 1520, ["order"] = 12, ["angle"] = -1279459328},
  [201] = {["x"] = 19018, ["y"] = 7588, ["z"] = 1525, ["order"] = 12, ["angle"] = -1684078592},
  [202] = {["x"] = 18308, ["y"] = 7050, ["z"] = 1520, ["order"] = 13, ["angle"] = 2024341504},
  [203] = {["x"] = 17527, ["y"] = 7234, ["z"] = 1520, ["order"] = 13, ["angle"] = 1626800128},
  [204] = {["x"] = 16775, ["y"] = 8097, ["z"] = 1520, ["order"] = 13, ["angle"] = 1788280832},
  [205] = {["x"] = 15962, ["y"] = 8591, ["z"] = 1520, ["order"] = 13, ["angle"] = -2018181120},
  [206] = {["x"] = 15168, ["y"] = 8294, ["z"] = 1520, ["order"] = 14, ["angle"] = -1419182080},
  [207] = {["x"] = 14659, ["y"] = 7483, ["z"] = 1520, ["order"] = 14, ["angle"] = -1898643456},
  [208] = {["x"] = 13999, ["y"] = 6995, ["z"] = 1520, ["order"] = 14, ["angle"] = -2121072640},
  [209] = {["x"] = 13159, ["y"] = 6903, ["z"] = 1525, ["order"] = 14, ["angle"] = 1689845760},
  [210] = {["x"] = 12460, ["y"] = 7466, ["z"] = 1525, ["order"] = 15, ["angle"] = 1102643200},
  [211] = {["x"] = 12258, ["y"] = 8424, ["z"] = 1505, ["order"] = 17, ["angle"] = 1058996224},
  [212] = {["x"] = 12214, ["y"] = 9265, ["z"] = 1400, ["order"] = 17, ["angle"] = 1094516736},
  [213] = {["x"] = 12176, ["y"] = 10335, ["z"] = 1420, ["order"] = 17, ["angle"] = 1094516736},
  [214] = {["x"] = 12149, ["y"] = 11633, ["z"] = 1505, ["order"] = 17, ["angle"] = 1055981568},
  [215] = {["x"] = 12158, ["y"] = 12500, ["z"] = 1400, ["order"] = 17, ["angle"] = 1055981568},
  [216] = {["x"] = 12175, ["y"] = 13358, ["z"] = 1410, ["order"] = 17, ["angle"] = 1055981568},
  [217] = {["x"] = 12191, ["y"] = 14750, ["z"] = 1500, ["order"] = 18, ["angle"] = 1070006272},
  [218] = {["x"] = 12185, ["y"] = 16079, ["z"] = 1505, ["order"] = 18, ["angle"] = 1093992448}
}
-- Respawn definitions for CLOUD CRADLE (MAP34)
krespawn.points[34] = {
  [0] = 69,
  [1] = {["x"] = 3568, ["y"] = 2443, ["z"] = 632, ["order"] = 0, ["angle"] = 3866624},
  [2] = {["x"] = 4278, ["y"] = 2422, ["z"] = 632, ["order"] = 0, ["angle"] = 20512768},
  [3] = {["x"] = 5039, ["y"] = 2359, ["z"] = 660, ["order"] = 0, ["angle"] = 61931520},
  [4] = {["x"] = 5887, ["y"] = 2298, ["z"] = 768, ["order"] = 0, ["angle"] = 155385856},
  [5] = {["x"] = 6697, ["y"] = 2402, ["z"] = 776, ["order"] = 0, ["angle"] = 616103936},
  [6] = {["x"] = 7160, ["y"] = 2917, ["z"] = 776, ["order"] = 1, ["angle"] = 1011679232},
  [7] = {["x"] = 7191, ["y"] = 3721, ["z"] = 776, ["order"] = 1, ["angle"] = 1065549824},
  [8] = {["x"] = 7101, ["y"] = 4622, ["z"] = 768, ["order"] = 1, ["angle"] = 957939712},
  [9] = {["x"] = 7100, ["y"] = 5507, ["z"] = 760, ["order"] = 1, ["angle"] = 672595968},
  [10] = {["x"] = 7458, ["y"] = 6076, ["z"] = 760, ["order"] = 2, ["angle"] = 41353216},
  [11] = {["x"] = 8101, ["y"] = 6250, ["z"] = 750, ["order"] = 2, ["angle"] = -89587712},
  [12] = {["x"] = 9115, ["y"] = 6200, ["z"] = 668, ["order"] = 2, ["angle"] = -28901376},
  [13] = {["x"] = 10336, ["y"] = 6294, ["z"] = 678, ["order"] = 2, ["angle"] = -108462080},
  [14] = {["x"] = 11161, ["y"] = 6266, ["z"] = 760, ["order"] = 2, ["angle"] = -297467904},
  [15] = {["x"] = 11877, ["y"] = 5861, ["z"] = 760, ["order"] = 3, ["angle"] = -936443904},
  [16] = {["x"] = 12036, ["y"] = 5149, ["z"] = 760, ["order"] = 3, ["angle"] = -1105788928},
  [17] = {["x"] = 11945, ["y"] = 4139, ["z"] = 760, ["order"] = 3, ["angle"] = -1154678784},
  [18] = {["x"] = 11906, ["y"] = 3166, ["z"] = 770, ["order"] = 3, ["angle"] = -1126891520},
  [19] = {["x"] = 11913, ["y"] = 2097, ["z"] = 880, ["order"] = 3, ["angle"] = -1485504512},
  [20] = {["x"] = 11529, ["y"] = 1290, ["z"] = 880, ["order"] = 4, ["angle"] = -2055798784},
  [21] = {["x"] = 10796, ["y"] = 1010, ["z"] = 880, ["order"] = 4, ["angle"] = 2092892160},
  [22] = {["x"] = 9779, ["y"] = 1008, ["z"] = 880, ["order"] = 4, ["angle"] = 1533739008},
  [23] = {["x"] = 9120, ["y"] = 1525, ["z"] = 880, ["order"] = 4, ["angle"] = 890699776},
  [24] = {["x"] = 9433, ["y"] = 2574, ["z"] = 880, ["order"] = 4, ["angle"] = 1033961472},
  [25] = {["x"] = 9541, ["y"] = 3155, ["z"] = 880, ["order"] = 5, ["angle"] = 1096744960},
  [26] = {["x"] = 9497, ["y"] = 7010, ["z"] = 880, ["order"] = 6, ["angle"] = 1108672512},
  [27] = {["x"] = 9500, ["y"] = 7800, ["z"] = 880, ["order"] = 6, ["angle"] = 1231880192},
  [28] = {["x"] = 9288, ["y"] = 8826, ["z"] = 880, ["order"] = 6, ["angle"] = 1602682880},
  [29] = {["x"] = 8835, ["y"] = 9406, ["z"] = 872, ["order"] = 6, ["angle"] = 2006122496},
  [30] = {["x"] = 8043, ["y"] = 9604, ["z"] = 880, ["order"] = 6, ["angle"] = -1896022016},
  [31] = {["x"] = 7373, ["y"] = 9308, ["z"] = 872, ["order"] = 6, ["angle"] = -1448280064},
  [32] = {["x"] = 6926, ["y"] = 8378, ["z"] = 880, ["order"] = 7, ["angle"] = -1665335296},
  [33] = {["x"] = 6234, ["y"] = 7793, ["z"] = 880, ["order"] = 7, ["angle"] = -2096300032},
  [34] = {["x"] = 5396, ["y"] = 7595, ["z"] = 880, ["order"] = 7, ["angle"] = 2005729280},
  [35] = {["x"] = 4172, ["y"] = 7504, ["z"] = 880, ["order"] = 7, ["angle"] = 1832714240},
  [36] = {["x"] = 3185, ["y"] = 7824, ["z"] = 880, ["order"] = 7, ["angle"] = 1354563584},
  [37] = {["x"] = 2949, ["y"] = 8488, ["z"] = 880, ["order"] = 8, ["angle"] = 1148649472},
  [38] = {["x"] = 2956, ["y"] = 9349, ["z"] = 880, ["order"] = 8, ["angle"] = 1197539328},
  [39] = {["x"] = 2967, ["y"] = 10331, ["z"] = 880, ["order"] = 8, ["angle"] = 1494548480},
  [40] = {["x"] = 2539, ["y"] = 10952, ["z"] = 880, ["order"] = 9, ["angle"] = 2076508160},
  [41] = {["x"] = 1766, ["y"] = 11106, ["z"] = 880, ["order"] = 9, ["angle"] = 2128936960},
  [42] = {["x"] = 548, ["y"] = 11190, ["z"] = 880, ["order"] = 9, ["angle"] = -1883308032},
  [43] = {["x"] = -657, ["y"] = 11041, ["z"] = 880, ["order"] = 9, ["angle"] = -1519321088},
  [44] = {["x"] = -1136, ["y"] = 10461, ["z"] = 880, ["order"] = 9, ["angle"] = -706281472},
  [45] = {["x"] = -513, ["y"] = 9736, ["z"] = 880, ["order"] = 10, ["angle"] = -466550784},
  [46] = {["x"] = 221, ["y"] = 9133, ["z"] = 880, ["order"] = 10, ["angle"] = -927268864},
  [47] = {["x"] = 449, ["y"] = 8275, ["z"] = 880, ["order"] = 10, ["angle"] = -1408040960},
  [48] = {["x"] = 73, ["y"] = 7690, ["z"] = 880, ["order"] = 10, ["angle"] = -1137377280},
  [49] = {["x"] = 129, ["y"] = 6669, ["z"] = 880, ["order"] = 11, ["angle"] = -620167168},
  [50] = {["x"] = 848, ["y"] = 5353, ["z"] = 880, ["order"] = 11, ["angle"] = -289865728},
  [51] = {["x"] = -265, ["y"] = 5789, ["z"] = 880, ["order"] = 11, ["angle"] = -758972416},
  [52] = {["x"] = 347, ["y"] = 5144, ["z"] = 880, ["order"] = 11, ["angle"] = -78053376},
  [53] = {["x"] = 1505, ["y"] = 4999, ["z"] = 880, ["order"] = 11, ["angle"] = -35979264},
  [54] = {["x"] = 2551, ["y"] = 5054, ["z"] = 880, ["order"] = 11, ["angle"] = 13041664},
  [55] = {["x"] = 3455, ["y"] = 5170, ["z"] = 880, ["order"] = 11, ["angle"] = -232849408},
  [56] = {["x"] = 4196, ["y"] = 4909, ["z"] = 880, ["order"] = 12, ["angle"] = -944308224},
  [57] = {["x"] = 4339, ["y"] = 4131, ["z"] = 880, ["order"] = 12, ["angle"] = -1068433408},
  [58] = {["x"] = 4358, ["y"] = 3132, ["z"] = 880, ["order"] = 12, ["angle"] = -1068433408},
  [59] = {["x"] = 4412, ["y"] = 2180, ["z"] = 880, ["order"] = 12, ["angle"] = -1197539328},
  [60] = {["x"] = 4441, ["y"] = 934, ["z"] = 880, ["order"] = 12, ["angle"] = -1428619264},
  [61] = {["x"] = 4065, ["y"] = 0, ["z"] = 880, ["order"] = 13, ["angle"] = -2132606976},
  [62] = {["x"] = 3374, ["y"] = -120, ["z"] = 880, ["order"] = 13, ["angle"] = -2132606976},
  [63] = {["x"] = 2160, ["y"] = -250, ["z"] = 888, ["order"] = 13, ["angle"] = 1935474688},
  [64] = {["x"] = 944, ["y"] = 110, ["z"] = 854, ["order"] = 13, ["angle"] = 1426915328},
  [65] = {["x"] = 364, ["y"] = 1104, ["z"] = 761, ["order"] = 14, ["angle"] = 645595136},
  [66] = {["x"] = 1087, ["y"] = 2074, ["z"] = 640, ["order"] = 14, ["angle"] = 153681920},
  [67] = {["x"] = 1782, ["y"] = 2385, ["z"] = 640, ["order"] = 14, ["angle"] = -4915200},
  [68] = {["x"] = 2634, ["y"] = 2458, ["z"] = 632, ["order"] = 14, ["angle"] = -18546688},
  [69] = {["x"] = 3123, ["y"] = 2459, ["z"] = 632, ["order"] = 14, ["angle"] = -18546688}
}
-- Respawn definitions for VOLCANIC VALLEY (MAP35)
krespawn.points[35] = {
  [0] = 112,
  [1] = {["x"] = -9778, ["y"] = -1959, ["z"] = 600, ["order"] = 0, ["angle"] = 1093861376},
  [2] = {["x"] = -9795, ["y"] = -938, ["z"] = 609, ["order"] = 0, ["angle"] = 1031602176},
  [3] = {["x"] = -9766, ["y"] = 148, ["z"] = 560, ["order"] = 0, ["angle"] = 1071841280},
  [4] = {["x"] = -9758, ["y"] = 910, ["z"] = 600, ["order"] = 0, ["angle"] = 1071841280},
  [5] = {["x"] = -9771, ["y"] = 1795, ["z"] = 600, ["order"] = 0, ["angle"] = 1020723200},
  [6] = {["x"] = -9810, ["y"] = 2785, ["z"] = 662, ["order"] = 1, ["angle"] = 925171712},
  [7] = {["x"] = -9760, ["y"] = 3824, ["z"] = 804, ["order"] = 1, ["angle"] = 619249664},
  [8] = {["x"] = -9327, ["y"] = 4427, ["z"] = 867, ["order"] = 1, ["angle"] = 213843968},
  [9] = {["x"] = -8335, ["y"] = 4618, ["z"] = 825, ["order"] = 2, ["angle"] = 106233856},
  [10] = {["x"] = -7165, ["y"] = 4497, ["z"] = 800, ["order"] = 2, ["angle"] = -222625792},
  [11] = {["x"] = -6013, ["y"] = 3916, ["z"] = 864, ["order"] = 2, ["angle"] = -286064640},
  [12] = {["x"] = -4942, ["y"] = 3535, ["z"] = 939, ["order"] = 2, ["angle"] = -89980928},
  [13] = {["x"] = -3773, ["y"] = 3442, ["z"] = 1063, ["order"] = 3, ["angle"] = 200605696},
  [14] = {["x"] = -2671, ["y"] = 3817, ["z"] = 1101, ["order"] = 3, ["angle"] = 464846848},
  [15] = {["x"] = -1913, ["y"] = 4578, ["z"] = 1070, ["order"] = 3, ["angle"] = 540213248},
  [16] = {["x"] = -1130, ["y"] = 5523, ["z"] = 1239, ["order"] = 3, ["angle"] = 225640448},
  [17] = {["x"] = -109, ["y"] = 6106, ["z"] = 1466, ["order"] = 3, ["angle"] = -94306304},
  [18] = {["x"] = 883, ["y"] = 6066, ["z"] = 1565, ["order"] = 3, ["angle"] = -382140416},
  [19] = {["x"] = 1696, ["y"] = 5447, ["z"] = 1590, ["order"] = 3, ["angle"] = -277282816},
  [20] = {["x"] = 2328, ["y"] = 5006, ["z"] = 1605, ["order"] = 4, ["angle"] = -265879552},
  [21] = {["x"] = 3506, ["y"] = 4634, ["z"] = 1625, ["order"] = 4, ["angle"] = 74645504},
  [22] = {["x"] = 4444, ["y"] = 4686, ["z"] = 1753, ["order"] = 4, ["angle"] = -529989632},
  [23] = {["x"] = 5191, ["y"] = 4072, ["z"] = 2009, ["order"] = 4, ["angle"] = -1522597888},
  [24] = {["x"] = 4386, ["y"] = 3432, ["z"] = 1757, ["order"] = 5, ["angle"] = -1845952512},
  [25] = {["x"] = 3157, ["y"] = 3055, ["z"] = 1757, ["order"] = 5, ["angle"] = -1543831552},
  [26] = {["x"] = 2552, ["y"] = 2319, ["z"] = 1757, ["order"] = 5, ["angle"] = -1116405760},
  [27] = {["x"] = 2484, ["y"] = 1393, ["z"] = 1863, ["order"] = 6, ["angle"] = -1031471104},
  [28] = {["x"] = 2482, ["y"] = 396, ["z"] = 1995, ["order"] = 6, ["angle"] = -452395008},
  [29] = {["x"] = 3106, ["y"] = -108, ["z"] = 1995, ["order"] = 7, ["angle"] = -82116608},
  [30] = {["x"] = 4096, ["y"] = -125, ["z"] = 1995, ["order"] = 7, ["angle"] = -204931072},
  [31] = {["x"] = 5114, ["y"] = -272, ["z"] = 1995, ["order"] = 7, ["angle"] = -480444416},
  [32] = {["x"] = 6157, ["y"] = -1118, ["z"] = 2413, ["order"] = 8, ["angle"] = -1580662784},
  [33] = {["x"] = 5659, ["y"] = -2285, ["z"] = 1995, ["order"] = 8, ["angle"] = 2086600704},
  [34] = {["x"] = 4491, ["y"] = -1996, ["z"] = 1995, ["order"] = 8, ["angle"] = -2047148032},
  [35] = {["x"] = 3467, ["y"] = -2042, ["z"] = 2124, ["order"] = 8, ["angle"] = -2031550464},
  [36] = {["x"] = 2378, ["y"] = -2006, ["z"] = 2275, ["order"] = 8, ["angle"] = -1919614976},
  [37] = {["x"] = 1103, ["y"] = -2213, ["z"] = 2190, ["order"] = 8, ["angle"] = -1494810624},
  [38] = {["x"] = 316, ["y"] = -3157, ["z"] = 2238, ["order"] = 9, ["angle"] = -1046937600},
  [39] = {["x"] = 642, ["y"] = -4507, ["z"] = 2351, ["order"] = 9, ["angle"] = -139132928},
  [40] = {["x"] = 1564, ["y"] = -4793, ["z"] = 2526, ["order"] = 10, ["angle"] = 8192000},
  [41] = {["x"] = 2506, ["y"] = -4882, ["z"] = 2692, ["order"] = 10, ["angle"] = 63635456},
  [42] = {["x"] = 3615, ["y"] = -4803, ["z"] = 2844, ["order"] = 10, ["angle"] = 624099328},
  [43] = {["x"] = 4352, ["y"] = -4250, ["z"] = 2968, ["order"] = 10, ["angle"] = 1472921600},
  [44] = {["x"] = 4059, ["y"] = -3564, ["z"] = 3015, ["order"] = 10, ["angle"] = 2000617472},
  [45] = {["x"] = 3190, ["y"] = -3398, ["z"] = 3087, ["order"] = 11, ["angle"] = -2054619136},
  [46] = {["x"] = 2212, ["y"] = -3457, ["z"] = 3145, ["order"] = 11, ["angle"] = 2145452032},
  [47] = {["x"] = 879, ["y"] = -3415, ["z"] = 3145, ["order"] = 11, ["angle"] = -2117009408},
  [48] = {["x"] = -2, ["y"] = -3452, ["z"] = 3145, ["order"] = 11, ["angle"] = -1966931968},
  [49] = {["x"] = -1073, ["y"] = -3753, ["z"] = 3181, ["order"] = 11, ["angle"] = 1858535424},
  [50] = {["x"] = -1903, ["y"] = -3436, ["z"] = 3188, ["order"] = 12, ["angle"] = 1026752512},
  [51] = {["x"] = -2017, ["y"] = -2206, ["z"] = 2949, ["order"] = 12, ["angle"] = 439943168},
  [52] = {["x"] = -1104, ["y"] = -1622, ["z"] = 2840, ["order"] = 12, ["angle"] = 545062912},
  [53] = {["x"] = -257, ["y"] = -1004, ["z"] = 2724, ["order"] = 12, ["angle"] = 1186136064},
  [54] = {["x"] = -447, ["y"] = -80, ["z"] = 2551, ["order"] = 12, ["angle"] = 1511587840},
  [55] = {["x"] = -1117, ["y"] = 712, ["z"] = 2685, ["order"] = 12, ["angle"] = 1758003200},
  [56] = {["x"] = -2180, ["y"] = 1133, ["z"] = 2812, ["order"] = 13, ["angle"] = 2137063424},
  [57] = {["x"] = -3495, ["y"] = 901, ["z"] = 2697, ["order"] = 13, ["angle"] = -2024603648},
  [58] = {["x"] = -4483, ["y"] = 580, ["z"] = 2479, ["order"] = 13, ["angle"] = -2036793344},
  [59] = {["x"] = -5541, ["y"] = 499, ["z"] = 2287, ["order"] = 13, ["angle"] = -1831010304},
  [60] = {["x"] = -7189, ["y"] = -482, ["z"] = 2796, ["order"] = 13, ["angle"] = -586088448},
  [61] = {["x"] = -6498, ["y"] = -1520, ["z"] = 2300, ["order"] = 14, ["angle"] = -285147136},
  [62] = {["x"] = -5319, ["y"] = -1700, ["z"] = 2317, ["order"] = 14, ["angle"] = -189595648},
  [63] = {["x"] = -4408, ["y"] = -2797, ["z"] = 2277, ["order"] = 15, ["angle"] = -1056243712},
  [64] = {["x"] = -4328, ["y"] = -4039, ["z"] = 2171, ["order"] = 15, ["angle"] = -1067253760},
  [65] = {["x"] = -4339, ["y"] = -5157, ["z"] = 1891, ["order"] = 15, ["angle"] = -1083375616},
  [66] = {["x"] = -4368, ["y"] = -6439, ["z"] = 1571, ["order"] = 16, ["angle"] = -1083375616},
  [67] = {["x"] = -4467, ["y"] = -10298, ["z"] = 1402, ["order"] = 17, ["angle"] = -1135804416},
  [68] = {["x"] = -4462, ["y"] = -11001, ["z"] = 1593, ["order"] = 17, ["angle"] = -1205010432},
  [69] = {["x"] = -4522, ["y"] = -11959, ["z"] = 1853, ["order"] = 17, ["angle"] = -1645150208},
  [70] = {["x"] = -5163, ["y"] = -12539, ["z"] = 2011, ["order"] = 17, ["angle"] = 2081095680},
  [71] = {["x"] = -6067, ["y"] = -12666, ["z"] = 2045, ["order"] = 18, ["angle"] = -2047803392},
  [72] = {["x"] = -7247, ["y"] = -12882, ["z"] = 2053, ["order"] = 18, ["angle"] = 2144141312},
  [73] = {["x"] = -8686, ["y"] = -13031, ["z"] = 2372, ["order"] = 18, ["angle"] = 1538981888},
  [74] = {["x"] = -9236, ["y"] = -12471, ["z"] = 2335, ["order"] = 18, ["angle"] = 1278410752},
  [75] = {["x"] = -9544, ["y"] = -11277, ["z"] = 2088, ["order"] = 19, ["angle"] = 817692672},
  [76] = {["x"] = -8708, ["y"] = -9671, ["z"] = 1515, ["order"] = 20, ["angle"] = 771162112},
  [77] = {["x"] = -8239, ["y"] = -8938, ["z"] = 1741, ["order"] = 20, ["angle"] = 983105536},
  [78] = {["x"] = -8167, ["y"] = -8006, ["z"] = 1929, ["order"] = 20, ["angle"] = 1705181184},
  [79] = {["x"] = -8776, ["y"] = -7436, ["z"] = 1922, ["order"] = 21, ["angle"] = 1690370048},
  [80] = {["x"] = -9397, ["y"] = -6912, ["z"] = 1903, ["order"] = 21, ["angle"] = 1868496896},
  [81] = {["x"] = -10362, ["y"] = -6547, ["z"] = 1634, ["order"] = 22, ["angle"] = 1559822336},
  [82] = {["x"] = -11086, ["y"] = -5564, ["z"] = 1723, ["order"] = 22, ["angle"] = 999882752},
  [83] = {["x"] = -11152, ["y"] = -4199, ["z"] = 1400, ["order"] = 22, ["angle"] = 749273088},
  [84] = {["x"] = -10486, ["y"] = -3396, ["z"] = 920, ["order"] = 22, ["angle"] = 819396608},
  [85] = {["x"] = -9951, ["y"] = -2533, ["z"] = 630, ["order"] = 22, ["angle"] = 1016791040},
  [86] = {["x"] = -7330, ["y"] = 4761, ["z"] = 800, ["order"] = 2, ["angle"] = 275316736},
  [87] = {["x"] = -6174, ["y"] = 5273, ["z"] = 813, ["order"] = 2, ["angle"] = 279773184},
  [88] = {["x"] = -5010, ["y"] = 5862, ["z"] = 947, ["order"] = 2, ["angle"] = -3997696},
  [89] = {["x"] = -3659, ["y"] = 5925, ["z"] = 1083, ["order"] = 3, ["angle"] = -198377472},
  [90] = {["x"] = -2527, ["y"] = 5458, ["z"] = 1120, ["order"] = 3, ["angle"] = -609026048},
  [91] = {["x"] = -2002, ["y"] = 4826, ["z"] = 1070, ["order"] = 3, ["angle"] = -589234176},
  [92] = {["x"] = -1541, ["y"] = 4189, ["z"] = 1129, ["order"] = 3, ["angle"] = -405995520},
  [93] = {["x"] = -793, ["y"] = 3594, ["z"] = 1304, ["order"] = 3, ["angle"] = -176619520},
  [94] = {["x"] = 328, ["y"] = 3220, ["z"] = 1499, ["order"] = 3, ["angle"] = 220921856},
  [95] = {["x"] = 1272, ["y"] = 3407, ["z"] = 1586, ["order"] = 3, ["angle"] = 434831360},
  [96] = {["x"] = 2192, ["y"] = 3962, ["z"] = 1629, ["order"] = 3, ["angle"] = 379256832},
  [97] = {["x"] = 5158, ["y"] = -1910, ["z"] = 1995, ["order"] = 8, ["angle"] = -1986330624},
  [98] = {["x"] = -9069, ["y"] = -4358, ["z"] = 600, ["order"] = 22, ["angle"] = 1172635648},
  [99] = {["x"] = -9175, ["y"] = -3774, ["z"] = 600, ["order"] = 22, ["angle"] = 1205403648},
  [100] = {["x"] = -9344, ["y"] = -3148, ["z"] = 600, ["order"] = 22, ["angle"] = 1305411584},
  [101] = {["x"] = -9584, ["y"] = -2419, ["z"] = 600, ["order"] = 22, ["angle"] = 1204748288},
  [102] = {["x"] = -6849, ["y"] = 5672, ["z"] = 800, ["order"] = 2, ["angle"] = 310706176},
  [103] = {["x"] = -5286, ["y"] = 6472, ["z"] = 1427, ["order"] = 2, ["angle"] = 125632512},
  [104] = {["x"] = -4068, ["y"] = 6429, ["z"] = 1464, ["order"] = 3, ["angle"] = 67436544},
  [105] = {["x"] = -2824, ["y"] = 6179, ["z"] = 1400, ["order"] = 3, ["angle"] = -15269888},
  [106] = {["x"] = 456, ["y"] = 4675, ["z"] = 1406, ["order"] = 4, ["angle"] = -3342336},
  [107] = {["x"] = 1333, ["y"] = 4688, ["z"] = 1561, ["order"] = 4, ["angle"] = -3342336},
  [108] = {["x"] = -6988, ["y"] = 3715, ["z"] = 800, ["order"] = 2, ["angle"] = -265093120},
  [109] = {["x"] = -5229, ["y"] = 2881, ["z"] = 1435, ["order"] = 2, ["angle"] = -121962496},
  [110] = {["x"] = -4009, ["y"] = 2832, ["z"] = 1489, ["order"] = 3, ["angle"] = 34537472},
  [111] = {["x"] = -2879, ["y"] = 3184, ["z"] = 1400, ["order"] = 3, ["angle"] = -82903040},
  [112] = {["x"] = 2268, ["y"] = 4714, ["z"] = 1605, ["order"] = 4, ["angle"] = 10289152}
}
-- Respawn definitions for KODACHROME VOID (MAP36)
krespawn.points[36] = {
  [0] = 60,
  [1] = {["x"] = 7048, ["y"] = -3421, ["z"] = 0, ["order"] = 0, ["angle"] = 1350762496},
  [2] = {["x"] = 6459, ["y"] = -1998, ["z"] = 128, ["order"] = 0, ["angle"] = 1367408640},
  [3] = {["x"] = 5789, ["y"] = -1029, ["z"] = 128, ["order"] = 0, ["angle"] = 838139904},
  [4] = {["x"] = 6045, ["y"] = -134, ["z"] = 128, ["order"] = 0, ["angle"] = 695795712},
  [5] = {["x"] = 6721, ["y"] = 683, ["z"] = 128, ["order"] = 0, ["angle"] = 896073728},
  [6] = {["x"] = 6964, ["y"] = 1592, ["z"] = 98, ["order"] = 0, ["angle"] = 990576640},
  [7] = {["x"] = 7079, ["y"] = 2349, ["z"] = -118, ["order"] = 0, ["angle"] = 980877312},
  [8] = {["x"] = 7086, ["y"] = 3244, ["z"] = -373, ["order"] = 0, ["angle"] = 976027648},
  [9] = {["x"] = 7121, ["y"] = 4300, ["z"] = -384, ["order"] = 1, ["angle"] = 685572096},
  [10] = {["x"] = 7617, ["y"] = 5327, ["z"] = -384, ["order"] = 1, ["angle"] = 211746816},
  [11] = {["x"] = 8391, ["y"] = 5821, ["z"] = -384, ["order"] = 1, ["angle"] = -374800384},
  [12] = {["x"] = 9123, ["y"] = 5460, ["z"] = -384, ["order"] = 1, ["angle"] = -648347648},
  [13] = {["x"] = 9742, ["y"] = 4635, ["z"] = -384, ["order"] = 1, ["angle"] = -563937280},
  [14] = {["x"] = 10647, ["y"] = 3525, ["z"] = -384, ["order"] = 1, ["angle"] = -425132032},
  [15] = {["x"] = 12048, ["y"] = 2778, ["z"] = -384, ["order"] = 1, ["angle"] = -496041984},
  [16] = {["x"] = 12966, ["y"] = 1211, ["z"] = -384, ["order"] = 1, ["angle"] = 14614528},
  [17] = {["x"] = 14256, ["y"] = 1174, ["z"] = -384, ["order"] = 1, ["angle"] = 14614528},
  [18] = {["x"] = 15760, ["y"] = 1202, ["z"] = -384, ["order"] = 1, ["angle"] = 3735552},
  [19] = {["x"] = 18194, ["y"] = 1109, ["z"] = -384, ["order"] = 2, ["angle"] = -5963776},
  [20] = {["x"] = 19085, ["y"] = 1157, ["z"] = -384, ["order"] = 2, ["angle"] = -18808832},
  [21] = {["x"] = 20208, ["y"] = 1281, ["z"] = -384, ["order"] = 2, ["angle"] = -286851072},
  [22] = {["x"] = 21285, ["y"] = 1148, ["z"] = -597, ["order"] = 2, ["angle"] = -528416768},
  [23] = {["x"] = 21916, ["y"] = 751, ["z"] = -640, ["order"] = 2, ["angle"] = -999227392},
  [24] = {["x"] = 22008, ["y"] = -540, ["z"] = -640, ["order"] = 2, ["angle"] = -1046937600},
  [25] = {["x"] = 21910, ["y"] = -3242, ["z"] = -464, ["order"] = 3, ["angle"] = -965935104},
  [26] = {["x"] = 22032, ["y"] = -4102, ["z"] = -464, ["order"] = 3, ["angle"] = -983105536},
  [27] = {["x"] = 22171, ["y"] = -5167, ["z"] = -464, ["order"] = 3, ["angle"] = -1285750784},
  [28] = {["x"] = 21901, ["y"] = -5990, ["z"] = -464, ["order"] = 3, ["angle"] = -1836908544},
  [29] = {["x"] = 21143, ["y"] = -6228, ["z"] = -464, ["order"] = 3, ["angle"] = 1980694528},
  [30] = {["x"] = 20395, ["y"] = -6153, ["z"] = -464, ["order"] = 3, ["angle"] = 2139553792},
  [31] = {["x"] = 19615, ["y"] = -6103, ["z"] = -464, ["order"] = 4, ["angle"] = 2139553792},
  [32] = {["x"] = 18849, ["y"] = -5856, ["z"] = -413, ["order"] = 4, ["angle"] = 1830092800},
  [33] = {["x"] = 17783, ["y"] = -5196, ["z"] = -464, ["order"] = 4, ["angle"] = 2089222144},
  [34] = {["x"] = 16455, ["y"] = -5154, ["z"] = -464, ["order"] = 4, ["angle"] = -1792868352},
  [35] = {["x"] = 15071, ["y"] = -5613, ["z"] = -461, ["order"] = 4, ["angle"] = -1798111232},
  [36] = {["x"] = 13938, ["y"] = -5940, ["z"] = -464, ["order"] = 4, ["angle"] = -1929314304},
  [37] = {["x"] = 12849, ["y"] = -6228, ["z"] = -464, ["order"] = 4, ["angle"] = -1649082368},
  [38] = {["x"] = 12292, ["y"] = -6882, ["z"] = -464, ["order"] = 5, ["angle"] = -1276837888},
  [39] = {["x"] = 12109, ["y"] = -8033, ["z"] = -464, ["order"] = 5, ["angle"] = -1379860480},
  [40] = {["x"] = 11753, ["y"] = -9108, ["z"] = -464, ["order"] = 5, ["angle"] = -1841758208},
  [41] = {["x"] = 11146, ["y"] = -9539, ["z"] = -464, ["order"] = 6, ["angle"] = -2083061760},
  [42] = {["x"] = 10039, ["y"] = -9645, ["z"] = -318, ["order"] = 6, ["angle"] = 2110062592},
  [43] = {["x"] = 9013, ["y"] = -9715, ["z"] = -82, ["order"] = 6, ["angle"] = 1931935744},
  [44] = {["x"] = 8135, ["y"] = -9428, ["z"] = 0, ["order"] = 6, ["angle"] = 1500839936},
  [45] = {["x"] = 7430, ["y"] = -8704, ["z"] = 0, ["order"] = 6, ["angle"] = 1121517568},
  [46] = {["x"] = 7295, ["y"] = -7624, ["z"] = 0, ["order"] = 6, ["angle"] = 1024786432},
  [47] = {["x"] = 7298, ["y"] = -6426, ["z"] = 0, ["order"] = 7, ["angle"] = 1073545216},
  [48] = {["x"] = 7255, ["y"] = -5120, ["z"] = 0, ["order"] = 7, ["angle"] = 1074462720},
  [49] = {["x"] = 7496, ["y"] = -3599, ["z"] = 0, ["order"] = 0, ["angle"] = 808910848},
  [50] = {["x"] = 8113, ["y"] = -2133, ["z"] = 128, ["order"] = 0, ["angle"] = 785711104},
  [51] = {["x"] = 8773, ["y"] = -1074, ["z"] = 128, ["order"] = 0, ["angle"] = 1327562752},
  [52] = {["x"] = 8517, ["y"] = -104, ["z"] = 128, ["order"] = 0, ["angle"] = 1465188352},
  [53] = {["x"] = 7998, ["y"] = 574, ["z"] = 128, ["order"] = 0, ["angle"] = 1368719360},
  [54] = {["x"] = 7647, ["y"] = 1348, ["z"] = 128, ["order"] = 0, ["angle"] = 1238171648},
  [55] = {["x"] = 7413, ["y"] = 2310, ["z"] = -107, ["order"] = 0, ["angle"] = 1252851712},
  [56] = {["x"] = 21994, ["y"] = 169, ["z"] = -640, ["order"] = 2, ["angle"] = -1085997056},
  [57] = {["x"] = 18363, ["y"] = -6534, ["z"] = -463, ["order"] = 4, ["angle"] = -2008875008},
  [58] = {["x"] = 17308, ["y"] = -6954, ["z"] = -457, ["order"] = 4, ["angle"] = 2108620800},
  [59] = {["x"] = 15900, ["y"] = -6850, ["z"] = -464, ["order"] = 4, ["angle"] = 1866661888},
  [60] = {["x"] = 15120, ["y"] = -6449, ["z"] = -445, ["order"] = 4, ["angle"] = 1973878784}
}
-- Respawn definitions for BOILING BEDROCK (MAP37)
krespawn.points[37] = {
  [0] = 69,
  [1] = {["x"] = -7553, ["y"] = -8204, ["z"] = 0, ["order"] = 0, ["angle"] = 1084424192},
  [2] = {["x"] = -7545, ["y"] = -7534, ["z"] = 0, ["order"] = 0, ["angle"] = 1084424192},
  [3] = {["x"] = -7514, ["y"] = -6545, ["z"] = 0, ["order"] = 0, ["angle"] = 1218510848},
  [4] = {["x"] = -7556, ["y"] = -6011, ["z"] = -16, ["order"] = 0, ["angle"] = 1218510848},
  [5] = {["x"] = -7661, ["y"] = -5435, ["z"] = 0, ["order"] = 0, ["angle"] = 1296367616},
  [6] = {["x"] = -7951, ["y"] = -4741, ["z"] = 0, ["order"] = 1, ["angle"] = 1360723968},
  [7] = {["x"] = -8575, ["y"] = -3816, ["z"] = 76, ["order"] = 1, ["angle"] = 1257439232},
  [8] = {["x"] = -9017, ["y"] = -2704, ["z"] = 414, ["order"] = 1, ["angle"] = 980090880},
  [9] = {["x"] = -9053, ["y"] = -1668, ["z"] = 647, ["order"] = 1, ["angle"] = 720699392},
  [10] = {["x"] = -8743, ["y"] = -663, ["z"] = 798, ["order"] = 1, ["angle"] = 462094336},
  [11] = {["x"] = -8135, ["y"] = 107, ["z"] = 834, ["order"] = 1, ["angle"] = 126550016},
  [12] = {["x"] = -7457, ["y"] = 397, ["z"] = 749, ["order"] = 1, ["angle"] = -115015680},
  [13] = {["x"] = -6618, ["y"] = 384, ["z"] = 694, ["order"] = 2, ["angle"] = -152633344},
  [14] = {["x"] = -5853, ["y"] = 162, ["z"] = 586, ["order"] = 2, ["angle"] = -458555392},
  [15] = {["x"] = -5129, ["y"] = -688, ["z"] = 475, ["order"] = 2, ["angle"] = -285933568},
  [16] = {["x"] = -4302, ["y"] = -1241, ["z"] = 348, ["order"] = 2, ["angle"] = 89980928},
  [17] = {["x"] = -3265, ["y"] = -1343, ["z"] = 189, ["order"] = 2, ["angle"] = 587399168},
  [18] = {["x"] = -2474, ["y"] = -772, ["z"] = 67, ["order"] = 2, ["angle"] = 1140785152},
  [19] = {["x"] = -2375, ["y"] = 24, ["z"] = 52, ["order"] = 3, ["angle"] = 1531248640},
  [20] = {["x"] = -2851, ["y"] = 641, ["z"] = 125, ["order"] = 3, ["angle"] = 1362296832},
  [21] = {["x"] = -3512, ["y"] = 1255, ["z"] = 227, ["order"] = 3, ["angle"] = 911409152},
  [22] = {["x"] = -3519, ["y"] = 1906, ["z"] = 228, ["order"] = 3, ["angle"] = 614006784},
  [23] = {["x"] = -3064, ["y"] = 2450, ["z"] = 158, ["order"] = 3, ["angle"] = 358285312},
  [24] = {["x"] = -2390, ["y"] = 2729, ["z"] = 55, ["order"] = 3, ["angle"] = 247398400},
  [25] = {["x"] = -1708, ["y"] = 2876, ["z"] = 35, ["order"] = 4, ["angle"] = 424869888},
  [26] = {["x"] = -898, ["y"] = 3158, ["z"] = 190, ["order"] = 4, ["angle"] = 858193920},
  [27] = {["x"] = -401, ["y"] = 3788, ["z"] = 285, ["order"] = 4, ["angle"] = 1197408256},
  [28] = {["x"] = -329, ["y"] = 4513, ["z"] = 298, ["order"] = 4, ["angle"] = 1664679936},
  [29] = {["x"] = -810, ["y"] = 4930, ["z"] = 206, ["order"] = 4, ["angle"] = 1728512000},
  [30] = {["x"] = -1457, ["y"] = 5243, ["z"] = 83, ["order"] = 4, ["angle"] = 1281687552},
  [31] = {["x"] = -1855, ["y"] = 5868, ["z"] = 8, ["order"] = 4, ["angle"] = 985202688},
  [32] = {["x"] = -1944, ["y"] = 6740, ["z"] = 0, ["order"] = 5, ["angle"] = 965804032},
  [33] = {["x"] = -1930, ["y"] = 7893, ["z"] = 0, ["order"] = 5, ["angle"] = -11993088},
  [34] = {["x"] = -960, ["y"] = 7381, ["z"] = 24, ["order"] = 5, ["angle"] = -22740992},
  [35] = {["x"] = -324, ["y"] = 7705, ["z"] = 0, ["order"] = 5, ["angle"] = 42926080},
  [36] = {["x"] = 387, ["y"] = 7781, ["z"] = 0, ["order"] = 5, ["angle"] = -53149696},
  [37] = {["x"] = 2190, ["y"] = 7762, ["z"] = 0, ["order"] = 5, ["angle"] = 64159744},
  [38] = {["x"] = 3113, ["y"] = 7870, ["z"] = 0, ["order"] = 5, ["angle"] = 21299200},
  [39] = {["x"] = 3768, ["y"] = 8026, ["z"] = 0, ["order"] = 5, ["angle"] = -54853632},
  [40] = {["x"] = 4547, ["y"] = 8091, ["z"] = 0, ["order"] = 5, ["angle"] = -151977984},
  [41] = {["x"] = 5608, ["y"] = 8052, ["z"] = 0, ["order"] = 6, ["angle"] = -815595520},
  [42] = {["x"] = 5960, ["y"] = 7369, ["z"] = 0, ["order"] = 6, ["angle"] = -1624571904},
  [43] = {["x"] = 5458, ["y"] = 6633, ["z"] = 0, ["order"] = 6, ["angle"] = -1949892608},
  [44] = {["x"] = 4576, ["y"] = 6372, ["z"] = 0, ["order"] = 6, ["angle"] = -1919221760},
  [45] = {["x"] = 3481, ["y"] = 6156, ["z"] = 0, ["order"] = 6, ["angle"] = -1475805184},
  [46] = {["x"] = 3120, ["y"] = 5382, ["z"] = -7, ["order"] = 7, ["angle"] = -1047855104},
  [47] = {["x"] = 3166, ["y"] = 4545, ["z"] = -115, ["order"] = 7, ["angle"] = -1047855104},
  [48] = {["x"] = 3185, ["y"] = 3324, ["z"] = -476, ["order"] = 7, ["angle"] = -1090715648},
  [49] = {["x"] = 3176, ["y"] = 2149, ["z"] = -834, ["order"] = 7, ["angle"] = -1090715648},
  [50] = {["x"] = 3166, ["y"] = 1114, ["z"] = -1129, ["order"] = 8, ["angle"] = -1090715648},
  [51] = {["x"] = 3139, ["y"] = -1764, ["z"] = -279, ["order"] = 8, ["angle"] = -1085865984},
  [52] = {["x"] = 3207, ["y"] = -2458, ["z"] = -114, ["order"] = 8, ["angle"] = -1166344192},
  [53] = {["x"] = 3054, ["y"] = -3315, ["z"] = 8, ["order"] = 8, ["angle"] = -1630208000},
  [54] = {["x"] = 2489, ["y"] = -3999, ["z"] = 116, ["order"] = 9, ["angle"] = -1893269504},
  [55] = {["x"] = 1578, ["y"] = -4268, ["z"] = 160, ["order"] = 9, ["angle"] = -2136801280},
  [56] = {["x"] = 329, ["y"] = -4217, ["z"] = 256, ["order"] = 10, ["angle"] = -2070740992},
  [57] = {["x"] = -654, ["y"] = -4194, ["z"] = 234, ["order"] = 10, ["angle"] = -2004680704},
  [58] = {["x"] = -1717, ["y"] = -4335, ["z"] = 202, ["order"] = 10, ["angle"] = -1640562688},
  [59] = {["x"] = -2287, ["y"] = -4942, ["z"] = 111, ["order"] = 10, ["angle"] = -1121517568},
  [60] = {["x"] = -2564, ["y"] = -5721, ["z"] = 3, ["order"] = 10, ["angle"] = -1196359680},
  [61] = {["x"] = -2718, ["y"] = -6722, ["z"] = 0, ["order"] = 10, ["angle"] = -1063845888},
  [62] = {["x"] = -2724, ["y"] = -7721, ["z"] = 0, ["order"] = 10, ["angle"] = -1368850432},
  [63] = {["x"] = -3157, ["y"] = -8559, ["z"] = 71, ["order"] = 11, ["angle"] = -1591279616},
  [64] = {["x"] = -3452, ["y"] = -9696, ["z"] = 23, ["order"] = 11, ["angle"] = -2042429440},
  [65] = {["x"] = -4441, ["y"] = -9902, ["z"] = 0, ["order"] = 11, ["angle"] = -2096300032},
  [66] = {["x"] = -5354, ["y"] = -10078, ["z"] = 0, ["order"] = 11, ["angle"] = 2058944512},
  [67] = {["x"] = -6287, ["y"] = -10073, ["z"] = 0, ["order"] = 12, ["angle"] = 1690763264},
  [68] = {["x"] = -7187, ["y"] = -9399, ["z"] = 0, ["order"] = 12, ["angle"] = 1241186304},
  [69] = {["x"] = -7468, ["y"] = -8724, ["z"] = 0, ["order"] = 12, ["angle"] = 1169883136}
}
-- Respawn definitions for EGG QUARTERS (MAP38)
krespawn.points[38] = {
  [0] = 119,
  [1] = {["x"] = -2570, ["y"] = -177, ["z"] = 800, ["order"] = 0, ["angle"] = -1056243712},
  [2] = {["x"] = -2493, ["y"] = -1074, ["z"] = 800, ["order"] = 0, ["angle"] = -1226244096},
  [3] = {["x"] = -2167, ["y"] = -2123, ["z"] = 756, ["order"] = 0, ["angle"] = -477691904},
  [4] = {["x"] = -2912, ["y"] = -2145, ["z"] = 756, ["order"] = 0, ["angle"] = -1673723904},
  [5] = {["x"] = -3235, ["y"] = -3108, ["z"] = 500, ["order"] = 0, ["angle"] = -1006567424},
  [6] = {["x"] = -3239, ["y"] = -4017, ["z"] = 500, ["order"] = 0, ["angle"] = -786497536},
  [7] = {["x"] = -3335, ["y"] = -4784, ["z"] = 500, ["order"] = 0, ["angle"] = -896729088},
  [8] = {["x"] = -3306, ["y"] = -5468, ["z"] = 500, ["order"] = 0, ["angle"] = -314638336},
  [9] = {["x"] = -2382, ["y"] = -5739, ["z"] = 500, ["order"] = 0, ["angle"] = 95092736},
  [10] = {["x"] = -1754, ["y"] = -5938, ["z"] = 500, ["order"] = 0, ["angle"] = 571277312},
  [11] = {["x"] = -1675, ["y"] = -5415, ["z"] = 500, ["order"] = 0, ["angle"] = 794230784},
  [12] = {["x"] = -1415, ["y"] = -4678, ["z"] = 500, ["order"] = 0, ["angle"] = 762773504},
  [13] = {["x"] = -1166, ["y"] = -3819, ["z"] = 500, ["order"] = 0, ["angle"] = 358547456},
  [14] = {["x"] = -412, ["y"] = -3344, ["z"] = 500, ["order"] = 1, ["angle"] = -56557568},
  [15] = {["x"] = 613, ["y"] = -3397, ["z"] = 500, ["order"] = 1, ["angle"] = 84082688},
  [16] = {["x"] = 1392, ["y"] = -3375, ["z"] = 500, ["order"] = 1, ["angle"] = -42008576},
  [17] = {["x"] = 2055, ["y"] = -3363, ["z"] = 250, ["order"] = 1, ["angle"] = -42008576},
  [18] = {["x"] = 2788, ["y"] = -3396, ["z"] = 0, ["order"] = 1, ["angle"] = -42008576},
  [19] = {["x"] = 3560, ["y"] = -3441, ["z"] = 0, ["order"] = 1, ["angle"] = -42008576},
  [20] = {["x"] = 4419, ["y"] = -3402, ["z"] = 0, ["order"] = 2, ["angle"] = -91815936},
  [21] = {["x"] = 5394, ["y"] = -3391, ["z"] = 0, ["order"] = 2, ["angle"] = -511508480},
  [22] = {["x"] = 6218, ["y"] = -3915, ["z"] = 0, ["order"] = 2, ["angle"] = -1084555264},
  [23] = {["x"] = 6462, ["y"] = -4752, ["z"] = 0, ["order"] = 2, ["angle"] = -1575419904},
  [24] = {["x"] = 5985, ["y"] = -5547, ["z"] = 0, ["order"] = 3, ["angle"] = -1906376704},
  [25] = {["x"] = 5167, ["y"] = -5838, ["z"] = 0, ["order"] = 3, ["angle"] = -1954480128},
  [26] = {["x"] = 4234, ["y"] = -6072, ["z"] = 0, ["order"] = 3, ["angle"] = -1954480128},
  [27] = {["x"] = 3873, ["y"] = -5917, ["z"] = 0, ["order"] = 3, ["angle"] = 1745158144},
  [28] = {["x"] = 3323, ["y"] = -5380, ["z"] = 0, ["order"] = 3, ["angle"] = 2061434880},
  [29] = {["x"] = 1880, ["y"] = -4986, ["z"] = 0, ["order"] = 3, ["angle"] = -1775435776},
  [30] = {["x"] = 984, ["y"] = -5461, ["z"] = 0, ["order"] = 3, ["angle"] = -1303576576},
  [31] = {["x"] = 802, ["y"] = -6263, ["z"] = 0, ["order"] = 3, ["angle"] = -1325989888},
  [32] = {["x"] = 425, ["y"] = -7099, ["z"] = 0, ["order"] = 4, ["angle"] = -1540685824},
  [33] = {["x"] = -429, ["y"] = -7816, ["z"] = 0, ["order"] = 4, ["angle"] = -1389559808},
  [34] = {["x"] = -1258, ["y"] = -8687, ["z"] = 0, ["order"] = 4, ["angle"] = -678494208},
  [35] = {["x"] = -1077, ["y"] = -9680, ["z"] = 0, ["order"] = 5, ["angle"] = -92078080},
  [36] = {["x"] = -209, ["y"] = -9984, ["z"] = 0, ["order"] = 5, ["angle"] = -12255232},
  [37] = {["x"] = 981, ["y"] = -9961, ["z"] = 0, ["order"] = 5, ["angle"] = -73596928},
  [38] = {["x"] = 2265, ["y"] = -9944, ["z"] = 0, ["order"] = 6, ["angle"] = -48037888},
  [39] = {["x"] = 3523, ["y"] = -9959, ["z"] = 73, ["order"] = 6, ["angle"] = -19726336},
  [40] = {["x"] = 4659, ["y"] = -9982, ["z"] = 256, ["order"] = 6, ["angle"] = -19726336},
  [41] = {["x"] = 5773, ["y"] = -9996, ["z"] = 256, ["order"] = 6, ["angle"] = 31916032},
  [42] = {["x"] = 6846, ["y"] = -9984, ["z"] = 256, ["order"] = 7, ["angle"] = -10944512},
  [43] = {["x"] = 7905, ["y"] = -10080, ["z"] = 256, ["order"] = 7, ["angle"] = 194314240},
  [44] = {["x"] = 8899, ["y"] = -9830, ["z"] = 256, ["order"] = 7, ["angle"] = 980877312},
  [45] = {["x"] = 8891, ["y"] = -8998, ["z"] = 256, ["order"] = 8, ["angle"] = 1070137344},
  [46] = {["x"] = 9435, ["y"] = -7945, ["z"] = 256, ["order"] = 8, ["angle"] = 1101070336},
  [47] = {["x"] = 9510, ["y"] = -6880, ["z"] = 256, ["order"] = 9, ["angle"] = 1105133568},
  [48] = {["x"] = 9344, ["y"] = -4248, ["z"] = 256, ["order"] = 10, ["angle"] = 1092812800},
  [49] = {["x"] = 8560, ["y"] = -4254, ["z"] = 256, ["order"] = 10, ["angle"] = 1125580800},
  [50] = {["x"] = 7817, ["y"] = -4263, ["z"] = 256, ["order"] = 10, ["angle"] = 1170538496},
  [51] = {["x"] = 7737, ["y"] = -2335, ["z"] = 256, ["order"] = 10, ["angle"] = 1047986176},
  [52] = {["x"] = 9306, ["y"] = -2200, ["z"] = 256, ["order"] = 10, ["angle"] = 1085734912},
  [53] = {["x"] = 9446, ["y"] = -1389, ["z"] = 256, ["order"] = 10, ["angle"] = 1074462720},
  [54] = {["x"] = 9461, ["y"] = -525, ["z"] = 256, ["order"] = 10, ["angle"] = 1074462720},
  [55] = {["x"] = 9445, ["y"] = 478, ["z"] = 256, ["order"] = 11, ["angle"] = 1013645312},
  [56] = {["x"] = 9544, ["y"] = 1328, ["z"] = 256, ["order"] = 11, ["angle"] = 875364352},
  [57] = {["x"] = 9978, ["y"] = 2365, ["z"] = 256, ["order"] = 11, ["angle"] = 1133314048},
  [58] = {["x"] = 9979, ["y"] = 3449, ["z"] = 256, ["order"] = 11, ["angle"] = 1946091520},
  [59] = {["x"] = 9662, ["y"] = 4147, ["z"] = 256, ["order"] = 11, ["angle"] = -2085683200},
  [60] = {["x"] = 9007, ["y"] = 3930, ["z"] = 256, ["order"] = 11, ["angle"] = 2098003968},
  [61] = {["x"] = 8173, ["y"] = 3969, ["z"] = 256, ["order"] = 11, ["angle"] = -2118057984},
  [62] = {["x"] = 7230, ["y"] = 3981, ["z"] = 256, ["order"] = 12, ["angle"] = -2099838976},
  [63] = {["x"] = 6520, ["y"] = 4009, ["z"] = 256, ["order"] = 12, ["angle"] = 1884225536},
  [64] = {["x"] = 5978, ["y"] = 4304, ["z"] = 256, ["order"] = 12, ["angle"] = 1871118336},
  [65] = {["x"] = 5371, ["y"] = 4649, ["z"] = 256, ["order"] = 12, ["angle"] = -1935474688},
  [66] = {["x"] = 4736, ["y"] = 4393, ["z"] = 256, ["order"] = 12, ["angle"] = -1928527872},
  [67] = {["x"] = 3929, ["y"] = 4091, ["z"] = 256, ["order"] = 12, ["angle"] = -2120024064},
  [68] = {["x"] = 3021, ["y"] = 4053, ["z"] = 256, ["order"] = 12, ["angle"] = -2093547520},
  [69] = {["x"] = 2105, ["y"] = 4068, ["z"] = 256, ["order"] = 12, ["angle"] = -2094727168},
  [70] = {["x"] = 1137, ["y"] = 4084, ["z"] = 256, ["order"] = 13, ["angle"] = -2069430272},
  [71] = {["x"] = 152, ["y"] = 4108, ["z"] = 256, ["order"] = 13, ["angle"] = -1531510784},
  [72] = {["x"] = -273, ["y"] = 3727, ["z"] = 256, ["order"] = 13, ["angle"] = -1006829568},
  [73] = {["x"] = -273, ["y"] = 3082, ["z"] = 256, ["order"] = 13, ["angle"] = -1002242048},
  [74] = {["x"] = -80, ["y"] = 2322, ["z"] = 256, ["order"] = 13, ["angle"] = -1224278016},
  [75] = {["x"] = -67, ["y"] = 1437, ["z"] = 256, ["order"] = 13, ["angle"] = -1832452096},
  [76] = {["x"] = -601, ["y"] = 1001, ["z"] = 256, ["order"] = 13, ["angle"] = 1708064768},
  [77] = {["x"] = -1300, ["y"] = 1373, ["z"] = 256, ["order"] = 13, ["angle"] = 1731395584},
  [78] = {["x"] = -1909, ["y"] = 1849, ["z"] = 256, ["order"] = 13, ["angle"] = 2018705408},
  [79] = {["x"] = -2845, ["y"] = 1998, ["z"] = 256, ["order"] = 14, ["angle"] = 1932197888},
  [80] = {["x"] = -4009, ["y"] = 2290, ["z"] = 256, ["order"] = 14, ["angle"] = 1415118848},
  [81] = {["x"] = -4677, ["y"] = 3087, ["z"] = 260, ["order"] = 15, ["angle"] = 1028587520},
  [82] = {["x"] = -4786, ["y"] = 4414, ["z"] = 470, ["order"] = 15, ["angle"] = 1077215232},
  [83] = {["x"] = -4948, ["y"] = 6679, ["z"] = 933, ["order"] = 15, ["angle"] = 707985408},
  [84] = {["x"] = -4343, ["y"] = 8055, ["z"] = 1513, ["order"] = 15, ["angle"] = -260898816},
  [85] = {["x"] = -3165, ["y"] = 7718, ["z"] = 1371, ["order"] = 15, ["angle"] = -863043584},
  [86] = {["x"] = -2825, ["y"] = 6750, ["z"] = 962, ["order"] = 15, ["angle"] = -1038155776},
  [87] = {["x"] = -2804, ["y"] = 5048, ["z"] = 1056, ["order"] = 16, ["angle"] = -1082720256},
  [88] = {["x"] = -2814, ["y"] = 3953, ["z"] = 1056, ["order"] = 16, ["angle"] = -1067778048},
  [89] = {["x"] = -2818, ["y"] = 2814, ["z"] = 1056, ["order"] = 16, ["angle"] = -1067778048},
  [90] = {["x"] = -2831, ["y"] = 1555, ["z"] = 1056, ["order"] = 16, ["angle"] = -1012203520},
  [91] = {["x"] = -2774, ["y"] = 640, ["z"] = 1056, ["order"] = 16, ["angle"] = -902758400},
  [92] = {["x"] = -2686, ["y"] = 30, ["z"] = 800, ["order"] = 16, ["angle"] = -902758400},
  [93] = {["x"] = -2603, ["y"] = -2680, ["z"] = 756, ["order"] = 0, ["angle"] = -1178927104},
  [94] = {["x"] = -2834, ["y"] = -4097, ["z"] = 500, ["order"] = 0, ["angle"] = -1108279296},
  [95] = {["x"] = -2787, ["y"] = -5228, ["z"] = 500, ["order"] = 0, ["angle"] = -42532864},
  [96] = {["x"] = -2164, ["y"] = -5691, ["z"] = 500, ["order"] = 0, ["angle"] = 524353536},
  [97] = {["x"] = 3750, ["y"] = -6403, ["z"] = 0, ["order"] = 3, ["angle"] = -1493499904},
  [98] = {["x"] = 3310, ["y"] = -7152, ["z"] = 0, ["order"] = 3, ["angle"] = -1752498176},
  [99] = {["x"] = 2852, ["y"] = -7534, ["z"] = 0, ["order"] = 3, ["angle"] = -2085683200},
  [100] = {["x"] = 870, ["y"] = -7677, ["z"] = 0, ["order"] = 4, ["angle"] = -1862598656},
  [101] = {["x"] = 6353, ["y"] = -9525, ["z"] = 256, ["order"] = 7, ["angle"] = 1018494976},
  [102] = {["x"] = 6272, ["y"] = -8746, ["z"] = 256, ["order"] = 7, ["angle"] = 960299008},
  [103] = {["x"] = 6296, ["y"] = -7906, ["z"] = 256, ["order"] = 7, ["angle"] = 566558720},
  [104] = {["x"] = 6661, ["y"] = -7412, ["z"] = 256, ["order"] = 8, ["angle"] = 61931520},
  [105] = {["x"] = 7414, ["y"] = -7479, ["z"] = 256, ["order"] = 8, ["angle"] = 313458688},
  [106] = {["x"] = 7843, ["y"] = -7246, ["z"] = 256, ["order"] = 9, ["angle"] = 938278912},
  [107] = {["x"] = 7950, ["y"] = -6502, ["z"] = 256, ["order"] = 10, ["angle"] = 1099628544},
  [108] = {["x"] = 7780, ["y"] = -1715, ["z"] = 256, ["order"] = 10, ["angle"] = 1028718592},
  [109] = {["x"] = 7964, ["y"] = -831, ["z"] = 256, ["order"] = 10, ["angle"] = 1096089600},
  [110] = {["x"] = 7984, ["y"] = -15, ["z"] = 256, ["order"] = 11, ["angle"] = 1107099648},
  [111] = {["x"] = 7998, ["y"] = 676, ["z"] = 256, ["order"] = 11, ["angle"] = 1087963136},
  [112] = {["x"] = 8074, ["y"] = 1430, ["z"] = 256, ["order"] = 11, ["angle"] = 963444736},
  [113] = {["x"] = 8329, ["y"] = 2218, ["z"] = 256, ["order"] = 11, ["angle"] = 1173422080},
  [114] = {["x"] = 8471, ["y"] = 3028, ["z"] = 256, ["order"] = 11, ["angle"] = 1548812288},
  [115] = {["x"] = 8093, ["y"] = 3643, ["z"] = 256, ["order"] = 11, ["angle"] = 1966931968},
  [116] = {["x"] = 6123, ["y"] = 3805, ["z"] = 256, ["order"] = 12, ["angle"] = -1785528320},
  [117] = {["x"] = 5453, ["y"] = 3391, ["z"] = 256, ["order"] = 12, ["angle"] = 2105606144},
  [118] = {["x"] = 4845, ["y"] = 3491, ["z"] = 256, ["order"] = 12, ["angle"] = 1766129664},
  [119] = {["x"] = 4180, ["y"] = 3853, ["z"] = 256, ["order"] = 12, ["angle"] = 2051604480}
}
-- Respawn definitions for VIRTUAL HIGHWAY (MAP39)
krespawn.points[39] = {
  [0] = 89,
  [1] = {["x"] = -1503, ["y"] = -3601, ["z"] = 200, ["order"] = 0, ["angle"] = 1102118912},
  [2] = {["x"] = -1538, ["y"] = -2636, ["z"] = 200, ["order"] = 0, ["angle"] = 1067122688},
  [3] = {["x"] = -1538, ["y"] = -1639, ["z"] = 267, ["order"] = 0, ["angle"] = 1067122688},
  [4] = {["x"] = -1478, ["y"] = -472, ["z"] = 474, ["order"] = 0, ["angle"] = 1112473600},
  [5] = {["x"] = -1382, ["y"] = 718, ["z"] = 600, ["order"] = 0, ["angle"] = 1349189632},
  [6] = {["x"] = -1599, ["y"] = 1496, ["z"] = 600, ["order"] = 1, ["angle"] = 1931673600},
  [7] = {["x"] = -2361, ["y"] = 1839, ["z"] = 600, ["order"] = 1, ["angle"] = -1598619648},
  [8] = {["x"] = -2989, ["y"] = 1387, ["z"] = 600, ["order"] = 1, ["angle"] = -1583153152},
  [9] = {["x"] = -3496, ["y"] = 807, ["z"] = 600, ["order"] = 1, ["angle"] = -1634795520},
  [10] = {["x"] = -4305, ["y"] = -56, ["z"] = 600, ["order"] = 1, ["angle"] = -1861681152},
  [11] = {["x"] = -5179, ["y"] = -607, ["z"] = 600, ["order"] = 1, ["angle"] = -2098003968},
  [12] = {["x"] = -6161, ["y"] = -803, ["z"] = 600, ["order"] = 2, ["angle"] = 1972568064},
  [13] = {["x"] = -7275, ["y"] = -665, ["z"] = 600, ["order"] = 2, ["angle"] = 1514602496},
  [14] = {["x"] = -7834, ["y"] = 35, ["z"] = 705, ["order"] = 2, ["angle"] = 974061568},
  [15] = {["x"] = -7906, ["y"] = 944, ["z"] = 922, ["order"] = 2, ["angle"] = 1006567424},
  [16] = {["x"] = -7945, ["y"] = 1885, ["z"] = 1000, ["order"] = 2, ["angle"] = 740360192},
  [17] = {["x"] = -7532, ["y"] = 2976, ["z"] = 1000, ["order"] = 3, ["angle"] = -23265280},
  [18] = {["x"] = -6507, ["y"] = 2997, ["z"] = 1000, ["order"] = 3, ["angle"] = -23265280},
  [19] = {["x"] = -5406, ["y"] = 2961, ["z"] = 1000, ["order"] = 3, ["angle"] = -82640896},
  [20] = {["x"] = -4181, ["y"] = 2947, ["z"] = 1000, ["order"] = 3, ["angle"] = 46333952},
  [21] = {["x"] = -1558, ["y"] = 3044, ["z"] = 600, ["order"] = 4, ["angle"] = -29949952},
  [22] = {["x"] = -747, ["y"] = 3050, ["z"] = 600, ["order"] = 4, ["angle"] = -29949952},
  [23] = {["x"] = 2711, ["y"] = 3040, ["z"] = 900, ["order"] = 4, ["angle"] = 65536},
  [24] = {["x"] = 3695, ["y"] = 3018, ["z"] = 900, ["order"] = 4, ["angle"] = 7012352},
  [25] = {["x"] = 4881, ["y"] = 2934, ["z"] = 900, ["order"] = 4, ["angle"] = 497483776},
  [26] = {["x"] = 5877, ["y"] = 3416, ["z"] = 900, ["order"] = 5, ["angle"] = 1040121856},
  [27] = {["x"] = 6165, ["y"] = 4596, ["z"] = 1100, ["order"] = 5, ["angle"] = -2084372480},
  [28] = {["x"] = 5404, ["y"] = 5237, ["z"] = 1117, ["order"] = 5, ["angle"] = -2037317632},
  [29] = {["x"] = 4081, ["y"] = 5091, ["z"] = 1300, ["order"] = 6, ["angle"] = -1466368000},
  [30] = {["x"] = 3666, ["y"] = 4234, ["z"] = 1300, ["order"] = 6, ["angle"] = -963837952},
  [31] = {["x"] = 3653, ["y"] = 3008, ["z"] = 1300, ["order"] = 6, ["angle"] = -1044709376},
  [32] = {["x"] = 3633, ["y"] = 1655, ["z"] = 1300, ["order"] = 6, ["angle"] = -995557376},
  [33] = {["x"] = 3527, ["y"] = 231, ["z"] = 1300, ["order"] = 6, ["angle"] = -622788608},
  [34] = {["x"] = 4002, ["y"] = -707, ["z"] = 1300, ["order"] = 6, ["angle"] = -378077184},
  [35] = {["x"] = 5110, ["y"] = -1520, ["z"] = 1300, ["order"] = 6, ["angle"] = -859897856},
  [36] = {["x"] = 5462, ["y"] = -2292, ["z"] = 1300, ["order"] = 7, ["angle"] = -1222311936},
  [37] = {["x"] = 5394, ["y"] = -3706, ["z"] = 1300, ["order"] = 7, ["angle"] = -1721434112},
  [38] = {["x"] = 4672, ["y"] = -4174, ["z"] = 1284, ["order"] = 8, ["angle"] = -2136670208},
  [39] = {["x"] = 3620, ["y"] = -4142, ["z"] = 1107, ["order"] = 8, ["angle"] = -1982922752},
  [40] = {["x"] = 2526, ["y"] = -4152, ["z"] = 938, ["order"] = 8, ["angle"] = -1736638464},
  [41] = {["x"] = 1527, ["y"] = -4545, ["z"] = 900, ["order"] = 9, ["angle"] = -1078525952},
  [42] = {["x"] = 1222, ["y"] = -5266, ["z"] = 900, ["order"] = 9, ["angle"] = -549257216},
  [43] = {["x"] = 1544, ["y"] = -5872, ["z"] = 900, ["order"] = 9, ["angle"] = -116457472},
  [44] = {["x"] = 2358, ["y"] = -5980, ["z"] = 900, ["order"] = 9, ["angle"] = -145162240},
  [45] = {["x"] = 3206, ["y"] = -6281, ["z"] = 900, ["order"] = 9, ["angle"] = -416481280},
  [46] = {["x"] = 3990, ["y"] = -6913, ["z"] = 900, ["order"] = 9, ["angle"] = -469434368},
  [47] = {["x"] = 4765, ["y"] = -7721, ["z"] = 900, ["order"] = 9, ["angle"] = -19333120},
  [48] = {["x"] = 5697, ["y"] = -7766, ["z"] = 900, ["order"] = 10, ["angle"] = 494862336},
  [49] = {["x"] = 6432, ["y"] = -7092, ["z"] = 900, ["order"] = 10, ["angle"] = 310444032},
  [50] = {["x"] = 7309, ["y"] = -6418, ["z"] = 900, ["order"] = 10, ["angle"] = -273481728},
  [51] = {["x"] = 8094, ["y"] = -6417, ["z"] = 900, ["order"] = 11, ["angle"] = -895549440},
  [52] = {["x"] = 8558, ["y"] = -7369, ["z"] = 900, ["order"] = 11, ["angle"] = -1037238272},
  [53] = {["x"] = 8748, ["y"] = -8511, ["z"] = 900, ["order"] = 11, ["angle"] = -1198718976},
  [54] = {["x"] = 8589, ["y"] = -9681, ["z"] = 900, ["order"] = 11, ["angle"] = -1892745216},
  [55] = {["x"] = 7802, ["y"] = -10298, ["z"] = 900, ["order"] = 12, ["angle"] = 2068512768},
  [56] = {["x"] = 6409, ["y"] = -10433, ["z"] = 500, ["order"] = 12, ["angle"] = -2072182784},
  [57] = {["x"] = 5447, ["y"] = -10493, ["z"] = 500, ["order"] = 12, ["angle"] = 2132344832},
  [58] = {["x"] = 4360, ["y"] = -10576, ["z"] = 500, ["order"] = 12, ["angle"] = 2050555904},
  [59] = {["x"] = 3142, ["y"] = -10576, ["z"] = 500, ["order"] = 12, ["angle"] = 1922105344},
  [60] = {["x"] = 2092, ["y"] = -10352, ["z"] = 500, ["order"] = 12, ["angle"] = 1339359232},
  [61] = {["x"] = 1789, ["y"] = -9578, ["z"] = 500, ["order"] = 13, ["angle"] = 1261371392},
  [62] = {["x"] = 1766, ["y"] = -8661, ["z"] = 500, ["order"] = 13, ["angle"] = 1671626752},
  [63] = {["x"] = 1225, ["y"] = -8138, ["z"] = 500, ["order"] = 14, ["angle"] = -2027749376},
  [64] = {["x"] = 521, ["y"] = -8261, ["z"] = 500, ["order"] = 14, ["angle"] = -1670971392},
  [65] = {["x"] = -156, ["y"] = -8910, ["z"] = 500, ["order"] = 14, ["angle"] = -1960247296},
  [66] = {["x"] = -929, ["y"] = -9278, ["z"] = 500, ["order"] = 14, ["angle"] = 1597833216},
  [67] = {["x"] = -1491, ["y"] = -8679, ["z"] = 500, ["order"] = 15, ["angle"] = 1029505024},
  [68] = {["x"] = -1571, ["y"] = -7735, ["z"] = 352, ["order"] = 15, ["angle"] = 1064632320},
  [69] = {["x"] = -1594, ["y"] = -6659, ["z"] = 200, ["order"] = 15, ["angle"] = 1064632320},
  [70] = {["x"] = -1591, ["y"] = -5372, ["z"] = 200, ["order"] = 15, ["angle"] = 1064632320},
  [71] = {["x"] = -1562, ["y"] = -4352, ["z"] = 200, ["order"] = 15, ["angle"] = 1108541440},
  [72] = {["x"] = -5781, ["y"] = -563, ["z"] = 600, ["order"] = 2, ["angle"] = 1360855040},
  [73] = {["x"] = -6128, ["y"] = -55, ["z"] = 600, ["order"] = 2, ["angle"] = 1052311552},
  [74] = {["x"] = -6010, ["y"] = 907, ["z"] = 600, ["order"] = 2, ["angle"] = 1037238272},
  [75] = {["x"] = -6011, ["y"] = 2262, ["z"] = 610, ["order"] = 2, ["angle"] = 1306722304},
  [76] = {["x"] = -6364, ["y"] = 3766, ["z"] = 600, ["order"] = 2, ["angle"] = 762380288},
  [77] = {["x"] = -6318, ["y"] = 4827, ["z"] = 600, ["order"] = 3, ["angle"] = 192086016},
  [78] = {["x"] = -5317, ["y"] = 5085, ["z"] = 600, ["order"] = 3, ["angle"] = -278462464},
  [79] = {["x"] = -4586, ["y"] = 4631, ["z"] = 600, ["order"] = 3, ["angle"] = -650313728},
  [80] = {["x"] = -3728, ["y"] = 3872, ["z"] = 600, ["order"] = 3, ["angle"] = -383057920},
  [81] = {["x"] = -2898, ["y"] = 3302, ["z"] = 600, ["order"] = 3, ["angle"] = -139657216},
  [82] = {["x"] = -2142, ["y"] = 3073, ["z"] = 600, ["order"] = 4, ["angle"] = -2818048},
  [83] = {["x"] = -3318, ["y"] = 3183, ["z"] = 600, ["order"] = 4, ["angle"] = -45809664},
  [84] = {["x"] = 2454, ["y"] = 3040, ["z"] = 900, ["order"] = 4, ["angle"] = 65536},
  [85] = {["x"] = 3822, ["y"] = 2088, ["z"] = 1300, ["order"] = 6, ["angle"] = -707854336},
  [86] = {["x"] = 4381, ["y"] = 1355, ["z"] = 1300, ["order"] = 6, ["angle"] = -534577152},
  [87] = {["x"] = 5326, ["y"] = 784, ["z"] = 1300, ["order"] = 6, ["angle"] = -902889472},
  [88] = {["x"] = 5576, ["y"] = 47, ["z"] = 1300, ["order"] = 6, ["angle"] = -1163067392},
  [89] = {["x"] = 5586, ["y"] = -1084, ["z"] = 1300, ["order"] = 6, ["angle"] = -1088225280}
}
-- Respawn definitions for EGGMAN'S NIGHTCLUB (MAP40)
krespawn.points[40] = {
  [0] = 160,
  [1] = {["x"] = 3240, ["y"] = 6702, ["z"] = 2500, ["order"] = 0, ["angle"] = 33882112},
  [2] = {["x"] = 3999, ["y"] = 6761, ["z"] = 2500, ["order"] = 0, ["angle"] = -12124160},
  [3] = {["x"] = 4779, ["y"] = 6827, ["z"] = 2500, ["order"] = 0, ["angle"] = -92209152},
  [4] = {["x"] = 5920, ["y"] = 6794, ["z"] = 2500, ["order"] = 0, ["angle"] = -478347264},
  [5] = {["x"] = 6695, ["y"] = 6377, ["z"] = 2562, ["order"] = 1, ["angle"] = -1137115136},
  [6] = {["x"] = 6816, ["y"] = 5444, ["z"] = 2500, ["order"] = 1, ["angle"] = -1143406592},
  [7] = {["x"] = 6771, ["y"] = 4416, ["z"] = 2628, ["order"] = 1, ["angle"] = -1070399488},
  [8] = {["x"] = 6765, ["y"] = 4038, ["z"] = 2500, ["order"] = 1, ["angle"] = -1070399488},
  [9] = {["x"] = 6780, ["y"] = 3259, ["z"] = 2500, ["order"] = 1, ["angle"] = -1185087488},
  [10] = {["x"] = 6704, ["y"] = 2230, ["z"] = 2500, ["order"] = 1, ["angle"] = -1312620544},
  [11] = {["x"] = 6484, ["y"] = 1091, ["z"] = 2500, ["order"] = 2, ["angle"] = -1471610880},
  [12] = {["x"] = 5846, ["y"] = 98, ["z"] = 2450, ["order"] = 2, ["angle"] = -1664155648},
  [13] = {["x"] = 4997, ["y"] = -755, ["z"] = 2132, ["order"] = 2, ["angle"] = -1580138496},
  [14] = {["x"] = 4058, ["y"] = -1856, ["z"] = 1850, ["order"] = 2, ["angle"] = -1605959680},
  [15] = {["x"] = 3231, ["y"] = -2831, ["z"] = 1800, ["order"] = 3, ["angle"] = -1786970112},
  [16] = {["x"] = 2292, ["y"] = -3508, ["z"] = 1920, ["order"] = 3, ["angle"] = 1944518656},
  [17] = {["x"] = 1431, ["y"] = -3241, ["z"] = 1843, ["order"] = 4, ["angle"] = 1506476032},
  [18] = {["x"] = 870, ["y"] = -2740, ["z"] = 1800, ["order"] = 4, ["angle"] = 1358233600},
  [19] = {["x"] = 198, ["y"] = -1983, ["z"] = 1849, ["order"] = 4, ["angle"] = 884015104},
  [20] = {["x"] = 367, ["y"] = -1335, ["z"] = 1821, ["order"] = 4, ["angle"] = 195362816},
  [21] = {["x"] = 1090, ["y"] = -1255, ["z"] = 1800, ["order"] = 5, ["angle"] = -7405568},
  [22] = {["x"] = 1950, ["y"] = -1332, ["z"] = 1879, ["order"] = 6, ["angle"] = 183566336},
  [23] = {["x"] = 2909, ["y"] = -1194, ["z"] = 2032, ["order"] = 6, ["angle"] = 891617280},
  [24] = {["x"] = 3253, ["y"] = -631, ["z"] = 2076, ["order"] = 7, ["angle"] = 1300955136},
  [25] = {["x"] = 3104, ["y"] = 52, ["z"] = 2000, ["order"] = 7, ["angle"] = 1442643968},
  [26] = {["x"] = 2569, ["y"] = 825, ["z"] = 2000, ["order"] = 7, ["angle"] = 1190068224},
  [27] = {["x"] = 2301, ["y"] = 1592, ["z"] = 2000, ["order"] = 8, ["angle"] = 903675904},
  [28] = {["x"] = 2271, ["y"] = 2330, ["z"] = 2000, ["order"] = 8, ["angle"] = 637992960},
  [29] = {["x"] = 2570, ["y"] = 3062, ["z"] = 2176, ["order"] = 8, ["angle"] = -174391296},
  [30] = {["x"] = 3078, ["y"] = 2881, ["z"] = 2121, ["order"] = 9, ["angle"] = -442302464},
  [31] = {["x"] = 3550, ["y"] = 2305, ["z"] = 2000, ["order"] = 9, ["angle"] = -309395456},
  [32] = {["x"] = 4125, ["y"] = 1722, ["z"] = 2007, ["order"] = 9, ["angle"] = 231669760},
  [33] = {["x"] = 4712, ["y"] = 1696, ["z"] = 2076, ["order"] = 9, ["angle"] = 850722816},
  [34] = {["x"] = 5002, ["y"] = 2370, ["z"] = 2033, ["order"] = 10, ["angle"] = 1090715648},
  [35] = {["x"] = 5113, ["y"] = 3133, ["z"] = 2000, ["order"] = 10, ["angle"] = 1267531776},
  [36] = {["x"] = 5101, ["y"] = 4058, ["z"] = 2000, ["order"] = 10, ["angle"] = 1786970112},
  [37] = {["x"] = 4690, ["y"] = 4504, ["z"] = 2000, ["order"] = 11, ["angle"] = -1894580224},
  [38] = {["x"] = 3846, ["y"] = 4015, ["z"] = 2000, ["order"] = 11, ["angle"] = 1975975936},
  [39] = {["x"] = 3666, ["y"] = 4569, ["z"] = 2000, ["order"] = 11, ["angle"] = -2069037056},
  [40] = {["x"] = 2824, ["y"] = 4283, ["z"] = 1951, ["order"] = 11, ["angle"] = -2127495168},
  [41] = {["x"] = 1742, ["y"] = 4278, ["z"] = 1800, ["order"] = 12, ["angle"] = 2117140480},
  [42] = {["x"] = 1011, ["y"] = 4161, ["z"] = 1809, ["order"] = 13, ["angle"] = -1856569344},
  [43] = {["x"] = 990, ["y"] = 4330, ["z"] = 1800, ["order"] = 13, ["angle"] = 1859584000},
  [44] = {["x"] = 271, ["y"] = 4748, ["z"] = 1915, ["order"] = 13, ["angle"] = 2096037888},
  [45] = {["x"] = 422, ["y"] = 3847, ["z"] = 1901, ["order"] = 13, ["angle"] = -1945698304},
  [46] = {["x"] = -1505, ["y"] = 4320, ["z"] = 1800, ["order"] = 13, ["angle"] = -2100625408},
  [47] = {["x"] = -2133, ["y"] = 4380, ["z"] = 1800, ["order"] = 14, ["angle"] = -1878851584},
  [48] = {["x"] = -3096, ["y"] = 4399, ["z"] = 1800, ["order"] = 14, ["angle"] = -1603862528},
  [49] = {["x"] = -3826, ["y"] = 3999, ["z"] = 1800, ["order"] = 15, ["angle"] = -1124663296},
  [50] = {["x"] = -4183, ["y"] = 3258, ["z"] = 1800, ["order"] = 15, ["angle"] = -587137024},
  [51] = {["x"] = -3883, ["y"] = 2549, ["z"] = 1800, ["order"] = 16, ["angle"] = -199294976},
  [52] = {["x"] = -3102, ["y"] = 2385, ["z"] = 1830, ["order"] = 16, ["angle"] = -109117440},
  [53] = {["x"] = -2195, ["y"] = 2318, ["z"] = 1945, ["order"] = 16, ["angle"] = -453181440},
  [54] = {["x"] = -1494, ["y"] = 1877, ["z"] = 2034, ["order"] = 17, ["angle"] = -978386944},
  [55] = {["x"] = -1205, ["y"] = 1159, ["z"] = 2070, ["order"] = 17, ["angle"] = -1280376832},
  [56] = {["x"] = -1147, ["y"] = 229, ["z"] = 2078, ["order"] = 17, ["angle"] = -1611333632},
  [57] = {["x"] = -1582, ["y"] = -378, ["z"] = 2023, ["order"] = 18, ["angle"] = -2137718784},
  [58] = {["x"] = -2276, ["y"] = -316, ["z"] = 1935, ["order"] = 18, ["angle"] = -2037317632},
  [59] = {["x"] = -3225, ["y"] = -285, ["z"] = 1815, ["order"] = 18, ["angle"] = -1548550144},
  [60] = {["x"] = -3837, ["y"] = -49, ["z"] = 1800, ["order"] = 19, ["angle"] = -1126105088},
  [61] = {["x"] = -3945, ["y"] = -731, ["z"] = 1800, ["order"] = 19, ["angle"] = -825032704},
  [62] = {["x"] = -3679, ["y"] = -1373, ["z"] = 1800, ["order"] = 19, ["angle"] = -584384512},
  [63] = {["x"] = -3073, ["y"] = -1925, ["z"] = 1800, ["order"] = 19, ["angle"] = -928448512},
  [64] = {["x"] = -2737, ["y"] = -2624, ["z"] = 1800, ["order"] = 19, ["angle"] = -1200947200},
  [65] = {["x"] = -2733, ["y"] = -3649, ["z"] = 1800, ["order"] = 19, ["angle"] = -1821442048},
  [66] = {["x"] = -3246, ["y"] = -4121, ["z"] = 1800, ["order"] = 20, ["angle"] = 1804402688},
  [67] = {["x"] = -3796, ["y"] = -3352, ["z"] = 1800, ["order"] = 20, ["angle"] = 1669005312},
  [68] = {["x"] = -4203, ["y"] = -3607, ["z"] = 1800, ["order"] = 20, ["angle"] = 1626537984},
  [69] = {["x"] = -4621, ["y"] = -3087, ["z"] = 1800, ["order"] = 20, ["angle"] = 1568997376},
  [70] = {["x"] = -5212, ["y"] = -2312, ["z"] = 1800, ["order"] = 20, ["angle"] = 1810563072},
  [71] = {["x"] = -5993, ["y"] = -1814, ["z"] = 1800, ["order"] = 21, ["angle"] = 2125266944},
  [72] = {["x"] = -7096, ["y"] = -1851, ["z"] = 1800, ["order"] = 21, ["angle"] = 2118975488},
  [73] = {["x"] = -8095, ["y"] = -1995, ["z"] = 1800, ["order"] = 22, ["angle"] = 1907425280},
  [74] = {["x"] = -9324, ["y"] = -1912, ["z"] = 1800, ["order"] = 22, ["angle"] = 1438711808},
  [75] = {["x"] = -9838, ["y"] = -1507, ["z"] = 1800, ["order"] = 23, ["angle"] = 902365184},
  [76] = {["x"] = -9950, ["y"] = -702, ["z"] = 1800, ["order"] = 23, ["angle"] = 441122816},
  [77] = {["x"] = -9653, ["y"] = -162, ["z"] = 1800, ["order"] = 24, ["angle"] = 77004800},
  [78] = {["x"] = -8969, ["y"] = -148, ["z"] = 1800, ["order"] = 24, ["angle"] = 40173568},
  [79] = {["x"] = -8208, ["y"] = -288, ["z"] = 1800, ["order"] = 24, ["angle"] = 317390848},
  [80] = {["x"] = -7588, ["y"] = -62, ["z"] = 1800, ["order"] = 24, ["angle"] = 842858496},
  [81] = {["x"] = -7418, ["y"] = 522, ["z"] = 1800, ["order"] = 25, ["angle"] = 1060831232},
  [82] = {["x"] = -7776, ["y"] = 697, ["z"] = 1800, ["order"] = 25, ["angle"] = 943783936},
  [83] = {["x"] = -7509, ["y"] = 1293, ["z"] = 1800, ["order"] = 25, ["angle"] = 954138624},
  [84] = {["x"] = -7608, ["y"] = 2029, ["z"] = 1800, ["order"] = 25, ["angle"] = 851640320},
  [85] = {["x"] = -7442, ["y"] = 2851, ["z"] = 1800, ["order"] = 25, ["angle"] = 616759296},
  [86] = {["x"] = -7223, ["y"] = 3171, ["z"] = 1800, ["order"] = 25, ["angle"] = 416350208},
  [87] = {["x"] = -6659, ["y"] = 3774, ["z"] = 1832, ["order"] = 26, ["angle"] = 722141184},
  [88] = {["x"] = -6152, ["y"] = 4250, ["z"] = 1944, ["order"] = 26, ["angle"] = 644153344},
  [89] = {["x"] = -5502, ["y"] = 4739, ["z"] = 2075, ["order"] = 26, ["angle"] = 783613952},
  [90] = {["x"] = -4942, ["y"] = 5321, ["z"] = 2205, ["order"] = 26, ["angle"] = 1203699712},
  [91] = {["x"] = -4916, ["y"] = 6143, ["z"] = 2302, ["order"] = 26, ["angle"] = 1982791680},
  [92] = {["x"] = -5422, ["y"] = 6484, ["z"] = 2283, ["order"] = 27, ["angle"] = -1493893120},
  [93] = {["x"] = -5915, ["y"] = 6128, ["z"] = 2186, ["order"] = 27, ["angle"] = -1401487360},
  [94] = {["x"] = -6238, ["y"] = 5487, ["z"] = 2076, ["order"] = 27, ["angle"] = -1562050560},
  [95] = {["x"] = -6554, ["y"] = 4952, ["z"] = 2000, ["order"] = 28, ["angle"] = -1787494400},
  [96] = {["x"] = -7173, ["y"] = 4421, ["z"] = 2000, ["order"] = 28, ["angle"] = 2083848192},
  [97] = {["x"] = -7763, ["y"] = 4441, ["z"] = 2000, ["order"] = 28, ["angle"] = 1788280832},
  [98] = {["x"] = -8232, ["y"] = 4861, ["z"] = 2000, ["order"] = 28, ["angle"] = 1867579392},
  [99] = {["x"] = -8735, ["y"] = 5299, ["z"] = 2000, ["order"] = 29, ["angle"] = -2110193664},
  [100] = {["x"] = -9275, ["y"] = 5348, ["z"] = 2000, ["order"] = 29, ["angle"] = -1763115008},
  [101] = {["x"] = -9707, ["y"] = 5049, ["z"] = 2000, ["order"] = 29, ["angle"] = -1731526656},
  [102] = {["x"] = -10201, ["y"] = 4439, ["z"] = 2000, ["order"] = 29, ["angle"] = -1927741440},
  [103] = {["x"] = -11007, ["y"] = 3935, ["z"] = 2035, ["order"] = 29, ["angle"] = 1830223872},
  [104] = {["x"] = -11717, ["y"] = 4074, ["z"] = 2125, ["order"] = 29, ["angle"] = 1105395712},
  [105] = {["x"] = -11872, ["y"] = 4621, ["z"] = 2063, ["order"] = 30, ["angle"] = 582287360},
  [106] = {["x"] = -11793, ["y"] = 5438, ["z"] = 1922, ["order"] = 30, ["angle"] = 491061248},
  [107] = {["x"] = -11259, ["y"] = 5580, ["z"] = 1850, ["order"] = 30, ["angle"] = 607584256},
  [108] = {["x"] = -10826, ["y"] = 6270, ["z"] = 1850, ["order"] = 30, ["angle"] = 475463680},
  [109] = {["x"] = -10276, ["y"] = 6901, ["z"] = 1850, ["order"] = 30, ["angle"] = 446496768},
  [110] = {["x"] = -9867, ["y"] = 6685, ["z"] = 1850, ["order"] = 30, ["angle"] = 545849344},
  [111] = {["x"] = -9538, ["y"] = 7150, ["z"] = 1850, ["order"] = 30, ["angle"] = 644939776},
  [112] = {["x"] = -8980, ["y"] = 7630, ["z"] = 1960, ["order"] = 31, ["angle"] = 641794048},
  [113] = {["x"] = -8447, ["y"] = 8229, ["z"] = 2113, ["order"] = 31, ["angle"] = 990314496},
  [114] = {["x"] = -8395, ["y"] = 8898, ["z"] = 2127, ["order"] = 31, ["angle"] = 1029373952},
  [115] = {["x"] = -8559, ["y"] = 9676, ["z"] = 2100, ["order"] = 31, ["angle"] = 966328320},
  [116] = {["x"] = -8585, ["y"] = 10572, ["z"] = 2100, ["order"] = 31, ["angle"] = 235470848},
  [117] = {["x"] = -8184, ["y"] = 11001, ["z"] = 2100, ["order"] = 31, ["angle"] = -397606912},
  [118] = {["x"] = -7763, ["y"] = 10834, ["z"] = 2100, ["order"] = 32, ["angle"] = -520814592},
  [119] = {["x"] = -7319, ["y"] = 10346, ["z"] = 2100, ["order"] = 32, ["angle"] = -309657600},
  [120] = {["x"] = -6777, ["y"] = 10030, ["z"] = 2100, ["order"] = 32, ["angle"] = 50266112},
  [121] = {["x"] = -5962, ["y"] = 10045, ["z"] = 2100, ["order"] = 32, ["angle"] = 322240512},
  [122] = {["x"] = -5962, ["y"] = 9874, ["z"] = 2100, ["order"] = 32, ["angle"] = -172425216},
  [123] = {["x"] = -5785, ["y"] = 9988, ["z"] = 2100, ["order"] = 32, ["angle"] = -5439488},
  [124] = {["x"] = -4486, ["y"] = 10246, ["z"] = 2134, ["order"] = 32, ["angle"] = 67305472},
  [125] = {["x"] = -4381, ["y"] = 9688, ["z"] = 2135, ["order"] = 32, ["angle"] = 8192000},
  [126] = {["x"] = -3152, ["y"] = 9461, ["z"] = 2100, ["order"] = 33, ["angle"] = 195493888},
  [127] = {["x"] = -2998, ["y"] = 9979, ["z"] = 2100, ["order"] = 33, ["angle"] = -7012352},
  [128] = {["x"] = -2952, ["y"] = 10517, ["z"] = 2100, ["order"] = 33, ["angle"] = -46202880},
  [129] = {["x"] = -1974, ["y"] = 9962, ["z"] = 2100, ["order"] = 33, ["angle"] = 44236800},
  [130] = {["x"] = -1973, ["y"] = 9558, ["z"] = 2100, ["order"] = 33, ["angle"] = 161939456},
  [131] = {["x"] = -2070, ["y"] = 10513, ["z"] = 2100, ["order"] = 33, ["angle"] = -175702016},
  [132] = {["x"] = -1334, ["y"] = 9972, ["z"] = 2100, ["order"] = 33, ["angle"] = 37289984},
  [133] = {["x"] = -542, ["y"] = 10131, ["z"] = 2100, ["order"] = 33, ["angle"] = -165871616},
  [134] = {["x"] = 199, ["y"] = 10262, ["z"] = 2251, ["order"] = 33, ["angle"] = -738000896},
  [135] = {["x"] = 587, ["y"] = 10066, ["z"] = 2285, ["order"] = 33, ["angle"] = -1058734080},
  [136] = {["x"] = 701, ["y"] = 9685, ["z"] = 2237, ["order"] = 34, ["angle"] = -1436090368},
  [137] = {["x"] = 354, ["y"] = 9009, ["z"] = 2100, ["order"] = 34, ["angle"] = -1512636416},
  [138] = {["x"] = -127, ["y"] = 8403, ["z"] = 2100, ["order"] = 34, ["angle"] = -1514340352},
  [139] = {["x"] = -731, ["y"] = 7974, ["z"] = 2100, ["order"] = 34, ["angle"] = -1287192576},
  [140] = {["x"] = -1051, ["y"] = 7557, ["z"] = 2100, ["order"] = 34, ["angle"] = -668663808},
  [141] = {["x"] = -1067, ["y"] = 7060, ["z"] = 2183, ["order"] = 34, ["angle"] = -183566336},
  [142] = {["x"] = -564, ["y"] = 6818, ["z"] = 2140, ["order"] = 35, ["angle"] = 23920640},
  [143] = {["x"] = 631, ["y"] = 6599, ["z"] = 2177, ["order"] = 35, ["angle"] = -12648448},
  [144] = {["x"] = 631, ["y"] = 6807, ["z"] = 2177, ["order"] = 35, ["angle"] = -1376256},
  [145] = {["x"] = 1415, ["y"] = 6736, ["z"] = 2331, ["order"] = 35, ["angle"] = 7012352},
  [146] = {["x"] = 2274, ["y"] = 6736, ["z"] = 2481, ["order"] = 35, ["angle"] = 7012352},
  [147] = {["x"] = 3047, ["y"] = 6740, ["z"] = 2510, ["order"] = 0, ["angle"] = -38338560},
  [148] = {["x"] = 6420, ["y"] = 5639, ["z"] = 2500, ["order"] = 1, ["angle"] = -1322057728},
  [149] = {["x"] = 6115, ["y"] = 4854, ["z"] = 2500, ["order"] = 1, ["angle"] = -1137639424},
  [150] = {["x"] = 6023, ["y"] = 4051, ["z"] = 2500, ["order"] = 1, ["angle"] = -1080754176},
  [151] = {["x"] = 6062, ["y"] = 3120, ["z"] = 2500, ["order"] = 1, ["angle"] = -939589632},
  [152] = {["x"] = 6185, ["y"] = 2333, ["z"] = 2500, ["order"] = 1, ["angle"] = -939589632},
  [153] = {["x"] = 6426, ["y"] = 4238, ["z"] = 2628, ["order"] = 1, ["angle"] = -1056112640},
  [154] = {["x"] = 6441, ["y"] = 3603, ["z"] = 2628, ["order"] = 1, ["angle"] = -1056112640},
  [155] = {["x"] = 6445, ["y"] = 2825, ["z"] = 2628, ["order"] = 1, ["angle"] = -1111031808},
  [156] = {["x"] = 4316, ["y"] = 1005, ["z"] = 2108, ["order"] = 9, ["angle"] = 690421760},
  [157] = {["x"] = 4865, ["y"] = 1166, ["z"] = 2152, ["order"] = 9, ["angle"] = 870514688},
  [158] = {["x"] = -801, ["y"] = 4682, ["z"] = 1881, ["order"] = 13, ["angle"] = -2043215872},
  [159] = {["x"] = -717, ["y"] = 3911, ["z"] = 1875, ["order"] = 13, ["angle"] = 2033647616},
  [160] = {["x"] = -11617, ["y"] = 4439, ["z"] = 2052, ["order"] = 30, ["angle"] = 796721152}
}
-- Respawn definitions for KKR GANBARE (MAP41)
krespawn.points[41] = {
  [0] = 25,
  [1] = {["x"] = 1248, ["y"] = 176, ["z"] = 0, ["order"] = 0, ["angle"] = 1113391104},
  [2] = {["x"] = 1269, ["y"] = 1201, ["z"] = 0, ["order"] = 0, ["angle"] = 1207894016},
  [3] = {["x"] = 1094, ["y"] = 2371, ["z"] = 0, ["order"] = 0, ["angle"] = 1240137728},
  [4] = {["x"] = 919, ["y"] = 3237, ["z"] = 0, ["order"] = 1, ["angle"] = 1088094208},
  [5] = {["x"] = 851, ["y"] = 4488, ["z"] = 0, ["order"] = 1, ["angle"] = 1802436608},
  [6] = {["x"] = 294, ["y"] = 5168, ["z"] = 0, ["order"] = 1, ["angle"] = 2041249792},
  [7] = {["x"] = -591, ["y"] = 5369, ["z"] = 0, ["order"] = 1, ["angle"] = -2046361600},
  [8] = {["x"] = -1546, ["y"] = 5392, ["z"] = 0, ["order"] = 1, ["angle"] = -2091581440},
  [9] = {["x"] = -2640, ["y"] = 5436, ["z"] = 0, ["order"] = 1, ["angle"] = -2066153472},
  [10] = {["x"] = -3859, ["y"] = 5263, ["z"] = 0, ["order"] = 2, ["angle"] = -1820393472},
  [11] = {["x"] = -4859, ["y"] = 4585, ["z"] = 0, ["order"] = 2, ["angle"] = -1239482368},
  [12] = {["x"] = -5474, ["y"] = 3298, ["z"] = 0, ["order"] = 2, ["angle"] = -952958976},
  [13] = {["x"] = -5478, ["y"] = 2023, ["z"] = 0, ["order"] = 2, ["angle"] = -862126080},
  [14] = {["x"] = -5333, ["y"] = 846, ["z"] = 0, ["order"] = 2, ["angle"] = -1043267584},
  [15] = {["x"] = -5339, ["y"] = -716, ["z"] = 0, ["order"] = 2, ["angle"] = -1027014656},
  [16] = {["x"] = -5281, ["y"] = -2539, ["z"] = 0, ["order"] = 3, ["angle"] = -742326272},
  [17] = {["x"] = -4690, ["y"] = -3703, ["z"] = 8, ["order"] = 3, ["angle"] = -241369088},
  [18] = {["x"] = -3488, ["y"] = -4114, ["z"] = 0, ["order"] = 3, ["angle"] = -196608},
  [19] = {["x"] = -2347, ["y"] = -4080, ["z"] = 0, ["order"] = 4, ["angle"] = 86835200},
  [20] = {["x"] = -1076, ["y"] = -4088, ["z"] = 0, ["order"] = 4, ["angle"] = -17760256},
  [21] = {["x"] = 337, ["y"] = -4169, ["z"] = 0, ["order"] = 4, ["angle"] = 428146688},
  [22] = {["x"] = 1289, ["y"] = -3544, ["z"] = 0, ["order"] = 4, ["angle"] = 1166344192},
  [23] = {["x"] = 1326, ["y"] = -2537, ["z"] = 0, ["order"] = 4, ["angle"] = 1101332480},
  [24] = {["x"] = 1241, ["y"] = -1552, ["z"] = 0, ["order"] = 4, ["angle"] = 1085472768},
  [25] = {["x"] = 1210, ["y"] = -286, ["z"] = 0, ["order"] = 4, ["angle"] = 1091108864}
}
-- Respawn definitions for CK CHAO (MAP42)
krespawn.points[42] = {
  [0] = 52,
  [1] = {["x"] = -828, ["y"] = -854, ["z"] = 100, ["order"] = 0, ["angle"] = 1025179648},
  [2] = {["x"] = -887, ["y"] = 58, ["z"] = 100, ["order"] = 0, ["angle"] = 909443072},
  [3] = {["x"] = -894, ["y"] = 1195, ["z"] = 100, ["order"] = 0, ["angle"] = 694616064},
  [4] = {["x"] = -429, ["y"] = 2202, ["z"] = 100, ["order"] = 0, ["angle"] = 107544576},
  [5] = {["x"] = 574, ["y"] = 2459, ["z"] = 100, ["order"] = 1, ["angle"] = -700907520},
  [6] = {["x"] = 1065, ["y"] = 1739, ["z"] = 100, ["order"] = 1, ["angle"] = -1078263808},
  [7] = {["x"] = 1054, ["y"] = 829, ["z"] = 100, ["order"] = 1, ["angle"] = -1114701824},
  [8] = {["x"] = 1019, ["y"] = -192, ["z"] = 100, ["order"] = 1, ["angle"] = -900530176},
  [9] = {["x"] = 1078, ["y"] = -1522, ["z"] = 308, ["order"] = 1, ["angle"] = -472973312},
  [10] = {["x"] = 1959, ["y"] = -2282, ["z"] = 438, ["order"] = 1, ["angle"] = -53149696},
  [11] = {["x"] = 2979, ["y"] = -2551, ["z"] = 484, ["order"] = 1, ["angle"] = 223019008},
  [12] = {["x"] = 4132, ["y"] = -2452, ["z"] = 467, ["order"] = 2, ["angle"] = 598409216},
  [13] = {["x"] = 4824, ["y"] = -1758, ["z"] = 348, ["order"] = 2, ["angle"] = 1011679232},
  [14] = {["x"] = 4980, ["y"] = -575, ["z"] = 146, ["order"] = 2, ["angle"] = 1057423360},
  [15] = {["x"] = 4979, ["y"] = 527, ["z"] = -36, ["order"] = 2, ["angle"] = 1057423360},
  [16] = {["x"] = 4990, ["y"] = 1955, ["z"] = 308, ["order"] = 2, ["angle"] = 1082851328},
  [17] = {["x"] = 4968, ["y"] = 2801, ["z"] = 308, ["order"] = 2, ["angle"] = 1082851328},
  [18] = {["x"] = 4816, ["y"] = 4111, ["z"] = 100, ["order"] = 3, ["angle"] = 836698112},
  [19] = {["x"] = 5292, ["y"] = 5072, ["z"] = 100, ["order"] = 3, ["angle"] = 395640832},
  [20] = {["x"] = 6206, ["y"] = 5648, ["z"] = 100, ["order"] = 3, ["angle"] = 21954560},
  [21] = {["x"] = 7525, ["y"] = 5810, ["z"] = 100, ["order"] = 4, ["angle"] = -56688640},
  [22] = {["x"] = 8868, ["y"] = 5795, ["z"] = -185, ["order"] = 4, ["angle"] = -391184384},
  [23] = {["x"] = 9288, ["y"] = 2778, ["z"] = -200, ["order"] = 5, ["angle"] = -1871773696},
  [24] = {["x"] = 8392, ["y"] = 2793, ["z"] = -200, ["order"] = 5, ["angle"] = -1704394752},
  [25] = {["x"] = 7691, ["y"] = 2414, ["z"] = -200, ["order"] = 5, ["angle"] = -1200685056},
  [26] = {["x"] = 7609, ["y"] = 1587, ["z"] = -93, ["order"] = 6, ["angle"] = -1112342528},
  [27] = {["x"] = 7695, ["y"] = 456, ["z"] = 57, ["order"] = 6, ["angle"] = -1259536384},
  [28] = {["x"] = 7679, ["y"] = -834, ["z"] = 100, ["order"] = 6, ["angle"] = -1339621376},
  [29] = {["x"] = 7315, ["y"] = -1646, ["z"] = 100, ["order"] = 6, ["angle"] = -1768620032},
  [30] = {["x"] = 6225, ["y"] = -2351, ["z"] = 100, ["order"] = 6, ["angle"] = -1366622208},
  [31] = {["x"] = 5848, ["y"] = -3453, ["z"] = 100, ["order"] = 6, ["angle"] = -1288634368},
  [32] = {["x"] = 5416, ["y"] = -4658, ["z"] = 100, ["order"] = 6, ["angle"] = -1932722176},
  [33] = {["x"] = 4268, ["y"] = -4778, ["z"] = 107, ["order"] = 6, ["angle"] = -2132606976},
  [34] = {["x"] = 3050, ["y"] = -4671, ["z"] = 100, ["order"] = 7, ["angle"] = 1913978880},
  [35] = {["x"] = 1754, ["y"] = -4419, ["z"] = 100, ["order"] = 7, ["angle"] = 1839005696},
  [36] = {["x"] = 632, ["y"] = -4053, ["z"] = 100, ["order"] = 7, ["angle"] = 1613037568},
  [37] = {["x"] = -302, ["y"] = -3228, ["z"] = 100, ["order"] = 7, ["angle"] = 1286537216},
  [38] = {["x"] = -757, ["y"] = -2204, ["z"] = 100, ["order"] = 7, ["angle"] = 1099235328},
  [39] = {["x"] = -876, ["y"] = -1405, ["z"] = 100, ["order"] = 7, ["angle"] = 985989120},
  [40] = {["x"] = 5389, ["y"] = 102, ["z"] = 100, ["order"] = 2, ["angle"] = 948764672},
  [41] = {["x"] = 5549, ["y"] = 1476, ["z"] = 100, ["order"] = 2, ["angle"] = 1127022592},
  [42] = {["x"] = 5910, ["y"] = 2513, ["z"] = 113, ["order"] = 2, ["angle"] = 941686784},
  [43] = {["x"] = 5620, ["y"] = 2522, ["z"] = 100, ["order"] = 2, ["angle"] = 1122697216},
  [44] = {["x"] = 5559, ["y"] = 2982, ["z"] = 100, ["order"] = 2, ["angle"] = 1113784320},
  [45] = {["x"] = 4618, ["y"] = -169, ["z"] = 100, ["order"] = 2, ["angle"] = 1293352960},
  [46] = {["x"] = 4566, ["y"] = 468, ["z"] = 100, ["order"] = 2, ["angle"] = 1183776768},
  [47] = {["x"] = 4509, ["y"] = 1707, ["z"] = 100, ["order"] = 2, ["angle"] = 1088618496},
  [48] = {["x"] = 4040, ["y"] = 2454, ["z"] = 124, ["order"] = 2, ["angle"] = 1215496192},
  [49] = {["x"] = 4395, ["y"] = 2861, ["z"] = 100, ["order"] = 2, ["angle"] = 982974464},
  [50] = {["x"] = 4450, ["y"] = 3537, ["z"] = 100, ["order"] = 2, ["angle"] = 650444800},
  [51] = {["x"] = 5590, ["y"] = 3754, ["z"] = 100, ["order"] = 3, ["angle"] = 1125449728},
  [52] = {["x"] = 5647, ["y"] = 4535, ["z"] = 100, ["order"] = 3, ["angle"] = 780337152}
}
-- Respawn definitions for CK CHAO (MAP43)
krespawn.points[43] = {
  [0] = 115,
  [1] = {["x"] = 1520, ["y"] = -902, ["z"] = 0, ["order"] = 0, ["angle"] = 1101594624},
  [2] = {["x"] = 1538, ["y"] = 160, ["z"] = 0, ["order"] = 0, ["angle"] = 1126367232},
  [3] = {["x"] = 1586, ["y"] = 1347, ["z"] = 0, ["order"] = 0, ["angle"] = 1331888128},
  [4] = {["x"] = 1396, ["y"] = 2461, ["z"] = 102, ["order"] = 0, ["angle"] = 1816330240},
  [5] = {["x"] = 612, ["y"] = 3160, ["z"] = 297, ["order"] = 1, ["angle"] = -1492713472},
  [6] = {["x"] = 120, ["y"] = 2816, ["z"] = 201, ["order"] = 1, ["angle"] = -1132658688},
  [7] = {["x"] = 98, ["y"] = 2001, ["z"] = 0, ["order"] = 1, ["angle"] = -1126236160},
  [8] = {["x"] = 69, ["y"] = 1207, ["z"] = 0, ["order"] = 1, ["angle"] = -1126236160},
  [9] = {["x"] = 49, ["y"] = -55, ["z"] = 0, ["order"] = 1, ["angle"] = -1133969408},
  [10] = {["x"] = -48, ["y"] = -1067, ["z"] = 0, ["order"] = 2, ["angle"] = -1639776256},
  [11] = {["x"] = -696, ["y"] = -1473, ["z"] = 0, ["order"] = 2, ["angle"] = 2096300032},
  [12] = {["x"] = -495, ["y"] = -1716, ["z"] = 54, ["order"] = 2, ["angle"] = -1926692864},
  [13] = {["x"] = -1288, ["y"] = -1495, ["z"] = 0, ["order"] = 2, ["angle"] = 2020933632},
  [14] = {["x"] = -2209, ["y"] = -1502, ["z"] = 0, ["order"] = 2, ["angle"] = 1529413632},
  [15] = {["x"] = -2775, ["y"] = -906, ["z"] = 0, ["order"] = 3, ["angle"] = 1164640256},
  [16] = {["x"] = -2976, ["y"] = 119, ["z"] = 0, ["order"] = 3, ["angle"] = 821231616},
  [17] = {["x"] = -2752, ["y"] = 1212, ["z"] = 0, ["order"] = 3, ["angle"] = 437059584},
  [18] = {["x"] = -2120, ["y"] = 1720, ["z"] = 0, ["order"] = 3, ["angle"] = -95748096},
  [19] = {["x"] = -1230, ["y"] = 1725, ["z"] = 0, ["order"] = 3, ["angle"] = -846397440},
  [20] = {["x"] = -749, ["y"] = 1031, ["z"] = 0, ["order"] = 3, ["angle"] = -1524301824},
  [21] = {["x"] = -1341, ["y"] = 91, ["z"] = 0, ["order"] = 3, ["angle"] = 2117140480},
  [22] = {["x"] = -3460, ["y"] = -74, ["z"] = 76, ["order"] = 6, ["angle"] = 2096037888},
  [23] = {["x"] = -4078, ["y"] = -38, ["z"] = 130, ["order"] = 6, ["angle"] = 2096037888},
  [24] = {["x"] = -4918, ["y"] = 53, ["z"] = 203, ["order"] = 6, ["angle"] = 2132213760},
  [25] = {["x"] = -5827, ["y"] = -43, ["z"] = 282, ["order"] = 6, ["angle"] = 2123300864},
  [26] = {["x"] = -6994, ["y"] = -52, ["z"] = 320, ["order"] = 6, ["angle"] = 1684340736},
  [27] = {["x"] = -8069, ["y"] = 599, ["z"] = 320, ["order"] = 6, ["angle"] = 884015104},
  [28] = {["x"] = -8032, ["y"] = 1606, ["z"] = 320, ["order"] = 7, ["angle"] = 204931072},
  [29] = {["x"] = -7340, ["y"] = 2015, ["z"] = 320, ["order"] = 7, ["angle"] = -223150080},
  [30] = {["x"] = -6416, ["y"] = 1448, ["z"] = 282, ["order"] = 8, ["angle"] = 82247680},
  [31] = {["x"] = -5725, ["y"] = 1285, ["z"] = 205, ["order"] = 8, ["angle"] = 329842688},
  [32] = {["x"] = -4847, ["y"] = 1362, ["z"] = 108, ["order"] = 8, ["angle"] = 636157952},
  [33] = {["x"] = -4179, ["y"] = 1906, ["z"] = 35, ["order"] = 8, ["angle"] = 1056243712},
  [34] = {["x"] = -3750, ["y"] = 2796, ["z"] = -12, ["order"] = 8, ["angle"] = 1403584512},
  [35] = {["x"] = -3852, ["y"] = 3759, ["z"] = -90, ["order"] = 9, ["angle"] = 2035744768},
  [36] = {["x"] = -4598, ["y"] = 4143, ["z"] = -135, ["order"] = 9, ["angle"] = -1809383424},
  [37] = {["x"] = -5567, ["y"] = 3769, ["z"] = -195, ["order"] = 9, ["angle"] = -1804009472},
  [38] = {["x"] = -6378, ["y"] = 3528, ["z"] = -200, ["order"] = 10, ["angle"] = -1940586496},
  [39] = {["x"] = -7281, ["y"] = 3387, ["z"] = -200, ["order"] = 10, ["angle"] = -1714225152},
  [40] = {["x"] = -8240, ["y"] = 2966, ["z"] = -200, ["order"] = 10, ["angle"] = -1378811904},
  [41] = {["x"] = -8802, ["y"] = 2225, ["z"] = -200, ["order"] = 10, ["angle"] = -1260191744},
  [42] = {["x"] = -8893, ["y"] = 1050, ["z"] = -200, ["order"] = 10, ["angle"] = -1567162368},
  [43] = {["x"] = -9502, ["y"] = 384, ["z"] = -200, ["order"] = 11, ["angle"] = 1665859584},
  [44] = {["x"] = -10285, ["y"] = 899, ["z"] = -200, ["order"] = 11, ["angle"] = 1169752064},
  [45] = {["x"] = -10342, ["y"] = 1829, ["z"] = -200, ["order"] = 11, ["angle"] = 736690176},
  [46] = {["x"] = -9946, ["y"] = 3102, ["z"] = -200, ["order"] = 11, ["angle"] = 606535680},
  [47] = {["x"] = -9436, ["y"] = 3989, ["z"] = -200, ["order"] = 11, ["angle"] = 466944000},
  [48] = {["x"] = -8525, ["y"] = 4884, ["z"] = -200, ["order"] = 11, ["angle"] = 243728384},
  [49] = {["x"] = -7349, ["y"] = 5199, ["z"] = -200, ["order"] = 11, ["angle"] = 110166016},
  [50] = {["x"] = -6239, ["y"] = 5276, ["z"] = -200, ["order"] = 11, ["angle"] = 591855616},
  [51] = {["x"] = -5625, ["y"] = 5998, ["z"] = -200, ["order"] = 11, ["angle"] = 1121517568},
  [52] = {["x"] = -5627, ["y"] = 9095, ["z"] = 311, ["order"] = 12, ["angle"] = 1079312384},
  [53] = {["x"] = -5598, ["y"] = 9890, ["z"] = 498, ["order"] = 12, ["angle"] = 1217331200},
  [54] = {["x"] = -5554, ["y"] = 10793, ["z"] = 600, ["order"] = 13, ["angle"] = 1381957632},
  [55] = {["x"] = -5731, ["y"] = 11716, ["z"] = 600, ["order"] = 13, ["angle"] = 1711210496},
  [56] = {["x"] = -6421, ["y"] = 12488, ["z"] = 600, ["order"] = 13, ["angle"] = -2079129600},
  [57] = {["x"] = -7401, ["y"] = 12619, ["z"] = 600, ["order"] = 14, ["angle"] = -1468465152},
  [58] = {["x"] = -8034, ["y"] = 11835, ["z"] = 600, ["order"] = 14, ["angle"] = -1192689664},
  [59] = {["x"] = -8332, ["y"] = 10796, ["z"] = 600, ["order"] = 14, ["angle"] = -1194786816},
  [60] = {["x"] = -8726, ["y"] = 10033, ["z"] = 600, ["order"] = 14, ["angle"] = -940900352},
  [61] = {["x"] = -8782, ["y"] = 9239, ["z"] = 600, ["order"] = 15, ["angle"] = -822935552},
  [62] = {["x"] = -8891, ["y"] = 8739, ["z"] = 600, ["order"] = 15, ["angle"] = -646381568},
  [63] = {["x"] = -8332, ["y"] = 8271, ["z"] = 600, ["order"] = 15, ["angle"] = -610467840},
  [64] = {["x"] = -7753, ["y"] = 7557, ["z"] = 600, ["order"] = 15, ["angle"] = -747962368},
  [65] = {["x"] = -7458, ["y"] = 6770, ["z"] = 600, ["order"] = 15, ["angle"] = -1073020928},
  [66] = {["x"] = -7144, ["y"] = 6247, ["z"] = 600, ["order"] = 15, ["angle"] = -999882752},
  [67] = {["x"] = -7336, ["y"] = 5079, ["z"] = 600, ["order"] = 15, ["angle"] = -1122435072},
  [68] = {["x"] = -6933, ["y"] = 4985, ["z"] = 600, ["order"] = 15, ["angle"] = -1035665408},
  [69] = {["x"] = -6916, ["y"] = 4334, ["z"] = 600, ["order"] = 16, ["angle"] = -1085734912},
  [70] = {["x"] = -7440, ["y"] = 4372, ["z"] = 600, ["order"] = 16, ["angle"] = -1042481152},
  [71] = {["x"] = -7099, ["y"] = 3796, ["z"] = 600, ["order"] = 16, ["angle"] = -1216806912},
  [72] = {["x"] = -7139, ["y"] = 3006, ["z"] = 600, ["order"] = 16, ["angle"] = -1136197632},
  [73] = {["x"] = -7173, ["y"] = 2063, ["z"] = 600, ["order"] = 16, ["angle"] = -1043660800},
  [74] = {["x"] = -7193, ["y"] = 950, ["z"] = 600, ["order"] = 16, ["angle"] = -1093074944},
  [75] = {["x"] = -7310, ["y"] = -44, ["z"] = 600, ["order"] = 16, ["angle"] = -744292352},
  [76] = {["x"] = -7070, ["y"] = -893, ["z"] = 499, ["order"] = 16, ["angle"] = -98107392},
  [77] = {["x"] = -6213, ["y"] = -1081, ["z"] = 327, ["order"] = 16, ["angle"] = -43581440},
  [78] = {["x"] = -5326, ["y"] = -1011, ["z"] = 327, ["order"] = 16, ["angle"] = -205324288},
  [79] = {["x"] = -4316, ["y"] = -1170, ["z"] = 348, ["order"] = 16, ["angle"] = -881524736},
  [80] = {["x"] = -3894, ["y"] = -1884, ["z"] = 427, ["order"] = 17, ["angle"] = -1799028736},
  [81] = {["x"] = -4382, ["y"] = -2375, ["z"] = 336, ["order"] = 17, ["angle"] = -2034958336},
  [82] = {["x"] = -5166, ["y"] = -2448, ["z"] = 193, ["order"] = 17, ["angle"] = 2113208320},
  [83] = {["x"] = -6172, ["y"] = -2403, ["z"] = 10, ["order"] = 17, ["angle"] = -1967456256},
  [84] = {["x"] = -7279, ["y"] = -2563, ["z"] = -173, ["order"] = 17, ["angle"] = -1789984768},
  [85] = {["x"] = -8298, ["y"] = -2950, ["z"] = -200, ["order"] = 18, ["angle"] = -1347092480},
  [86] = {["x"] = -8678, ["y"] = -3606, ["z"] = -200, ["order"] = 18, ["angle"] = -624230400},
  [87] = {["x"] = -8333, ["y"] = -4229, ["z"] = -200, ["order"] = 18, ["angle"] = -227737600},
  [88] = {["x"] = -7435, ["y"] = -4622, ["z"] = -200, ["order"] = 18, ["angle"] = -92078080},
  [89] = {["x"] = -6693, ["y"] = -4786, ["z"] = -200, ["order"] = 18, ["angle"] = -92078080},
  [90] = {["x"] = -5819, ["y"] = -4914, ["z"] = -180, ["order"] = 18, ["angle"] = -44630016},
  [91] = {["x"] = -5073, ["y"] = -5071, ["z"] = -83, ["order"] = 18, ["angle"] = 14614528},
  [92] = {["x"] = -4325, ["y"] = -5177, ["z"] = 0, ["order"] = 18, ["angle"] = 400359424},
  [93] = {["x"] = -3385, ["y"] = -4891, ["z"] = 0, ["order"] = 18, ["angle"] = 649789440},
  [94] = {["x"] = -2970, ["y"] = -4214, ["z"] = 0, ["order"] = 18, ["angle"] = 550174720},
  [95] = {["x"] = -2500, ["y"] = -3564, ["z"] = 120, ["order"] = 18, ["angle"] = 206372864},
  [96] = {["x"] = -1705, ["y"] = -3480, ["z"] = 141, ["order"] = 19, ["angle"] = -462749696},
  [97] = {["x"] = -1220, ["y"] = -4091, ["z"] = 0, ["order"] = 19, ["angle"] = -796590080},
  [98] = {["x"] = -870, ["y"] = -5037, ["z"] = 0, ["order"] = 19, ["angle"] = -356450304},
  [99] = {["x"] = -266, ["y"] = -5373, ["z"] = 36, ["order"] = 19, ["angle"] = -36503552},
  [100] = {["x"] = 598, ["y"] = -5410, ["z"] = 48, ["order"] = 19, ["angle"] = -54591488},
  [101] = {["x"] = 1702, ["y"] = -5322, ["z"] = 25, ["order"] = 19, ["angle"] = 1173815296},
  [102] = {["x"] = 1868, ["y"] = -4358, ["z"] = 0, ["order"] = 19, ["angle"] = 1181286400},
  [103] = {["x"] = 1667, ["y"] = -3280, ["z"] = 0, ["order"] = 19, ["angle"] = 1179713536},
  [104] = {["x"] = 1517, ["y"] = -2222, ["z"] = 0, ["order"] = 19, ["angle"] = 1101332480},
  [105] = {["x"] = 1492, ["y"] = -1220, ["z"] = 0, ["order"] = 19, ["angle"] = 1071054848},
  [106] = {["x"] = -1268, ["y"] = -1790, ["z"] = 90, ["order"] = 2, ["angle"] = 1789853696},
  [107] = {["x"] = -2521, ["y"] = 1842, ["z"] = 0, ["order"] = 3, ["angle"] = 816513024},
  [108] = {["x"] = -2269, ["y"] = 2923, ["z"] = 223, ["order"] = 3, ["angle"] = 256704512},
  [109] = {["x"] = -1623, ["y"] = 3444, ["z"] = 354, ["order"] = 3, ["angle"] = -708247552},
  [110] = {["x"] = -1219, ["y"] = 2844, ["z"] = 203, ["order"] = 4, ["angle"] = -856489984},
  [111] = {["x"] = -961, ["y"] = 2065, ["z"] = 8, ["order"] = 4, ["angle"] = -912195584},
  [112] = {["x"] = -815, ["y"] = 166, ["z"] = 0, ["order"] = 4, ["angle"] = 1971912704},
  [113] = {["x"] = -7783, ["y"] = 226, ["z"] = 320, ["order"] = 6, ["angle"] = 1542914048},
  [114] = {["x"] = -8169, ["y"] = 1090, ["z"] = 320, ["order"] = 6, ["angle"] = 794624000},
  [115] = {["x"] = -7805, ["y"] = 1872, ["z"] = 320, ["order"] = 7, ["angle"] = 179634176}
}
-- Respawn definitions for CK CLOUD (MAP44)
krespawn.points[44] = {
  [0] = 42,
  [1] = {["x"] = -3513, ["y"] = -2310, ["z"] = 1000, ["order"] = 0, ["angle"] = 11599872},
  [2] = {["x"] = -2850, ["y"] = -2327, ["z"] = 1000, ["order"] = 0, ["angle"] = 1114112},
  [3] = {["x"] = -1989, ["y"] = -2406, ["z"] = 1000, ["order"] = 0, ["angle"] = 119865344},
  [4] = {["x"] = -1085, ["y"] = -2481, ["z"] = 1000, ["order"] = 0, ["angle"] = 403767296},
  [5] = {["x"] = -169, ["y"] = -2250, ["z"] = 1000, ["order"] = 0, ["angle"] = 763428864},
  [6] = {["x"] = 285, ["y"] = -1680, ["z"] = 1000, ["order"] = 0, ["angle"] = 1342373888},
  [7] = {["x"] = 215, ["y"] = -800, ["z"] = 1000, ["order"] = 1, ["angle"] = 1665728512},
  [8] = {["x"] = -394, ["y"] = -320, ["z"] = 1000, ["order"] = 1, ["angle"] = 1961820160},
  [9] = {["x"] = -1310, ["y"] = -53, ["z"] = 1000, ["order"] = 1, ["angle"] = 1524432896},
  [10] = {["x"] = -1895, ["y"] = 686, ["z"] = 1000, ["order"] = 1, ["angle"] = 1088618496},
  [11] = {["x"] = -2573, ["y"] = 2274, ["z"] = 1000, ["order"] = 2, ["angle"] = 170852352},
  [12] = {["x"] = -2596, ["y"] = 1318, ["z"] = 1000, ["order"] = 2, ["angle"] = 768278528},
  [13] = {["x"] = -82, ["y"] = 2411, ["z"] = 1386, ["order"] = 2, ["angle"] = -62455808},
  [14] = {["x"] = 1746, ["y"] = 2412, ["z"] = 1400, ["order"] = 3, ["angle"] = -447283200},
  [15] = {["x"] = 2414, ["y"] = 1866, ["z"] = 1400, ["order"] = 3, ["angle"] = -881655808},
  [16] = {["x"] = 2811, ["y"] = 974, ["z"] = 1400, ["order"] = 3, ["angle"] = -1236598784},
  [17] = {["x"] = 2867, ["y"] = -100, ["z"] = 1400, ["order"] = 3, ["angle"] = -1765081088},
  [18] = {["x"] = 2291, ["y"] = -849, ["z"] = 1400, ["order"] = 3, ["angle"] = 1848180736},
  [19] = {["x"] = 1503, ["y"] = -696, ["z"] = 1400, ["order"] = 3, ["angle"] = 1345388544},
  [20] = {["x"] = 1200, ["y"] = 270, ["z"] = 1400, ["order"] = 4, ["angle"] = 1698889728},
  [21] = {["x"] = 661, ["y"] = 1003, ["z"] = 1400, ["order"] = 4, ["angle"] = 1970470912},
  [22] = {["x"] = -270, ["y"] = 1320, ["z"] = 1417, ["order"] = 4, ["angle"] = 2015821824},
  [23] = {["x"] = -1032, ["y"] = 1599, ["z"] = 1530, ["order"] = 4, ["angle"] = 2065367040},
  [24] = {["x"] = -1935, ["y"] = 2088, ["z"] = 1530, ["order"] = 4, ["angle"] = -2072576000},
  [25] = {["x"] = -2980, ["y"] = 2274, ["z"] = 1530, ["order"] = 4, ["angle"] = -1753808896},
  [26] = {["x"] = -4068, ["y"] = 1907, ["z"] = 1530, ["order"] = 4, ["angle"] = -1437270016},
  [27] = {["x"] = -4735, ["y"] = 1335, ["z"] = 1530, ["order"] = 4, ["angle"] = -1298726912},
  [28] = {["x"] = -5422, ["y"] = 484, ["z"] = 1400, ["order"] = 6, ["angle"] = -852426752},
  [29] = {["x"] = -5506, ["y"] = -356, ["z"] = 1400, ["order"] = 6, ["angle"] = -550043648},
  [30] = {["x"] = -5047, ["y"] = -1062, ["z"] = 1400, ["order"] = 6, ["angle"] = -179503104},
  [31] = {["x"] = -4288, ["y"] = -1225, ["z"] = 1400, ["order"] = 6, ["angle"] = -155123712},
  [32] = {["x"] = -3373, ["y"] = -1247, ["z"] = 1400, ["order"] = 6, ["angle"] = -363134976},
  [33] = {["x"] = -2510, ["y"] = -1507, ["z"] = 1400, ["order"] = 7, ["angle"] = -1053622272},
  [34] = {["x"] = -2160, ["y"] = -2251, ["z"] = 1400, ["order"] = 7, ["angle"] = -1390739456},
  [35] = {["x"] = -2456, ["y"] = -3184, ["z"] = 1400, ["order"] = 7, ["angle"] = -2047016960},
  [36] = {["x"] = -3515, ["y"] = -3439, ["z"] = 1400, ["order"] = 7, ["angle"] = 2117533696},
  [37] = {["x"] = -4364, ["y"] = -3486, ["z"] = 1400, ["order"] = 7, ["angle"] = 2005598208},
  [38] = {["x"] = -5354, ["y"] = -3544, ["z"] = 1400, ["order"] = 7, ["angle"] = 1771896832},
  [39] = {["x"] = -6084, ["y"] = -3164, ["z"] = 1400, ["order"] = 7, ["angle"] = 764477440},
  [40] = {["x"] = -5943, ["y"] = -2452, ["z"] = 1400, ["order"] = 8, ["angle"] = 129695744},
  [41] = {["x"] = -5183, ["y"] = -2301, ["z"] = 1400, ["order"] = 8, ["angle"] = -6094848},
  [42] = {["x"] = -4003, ["y"] = -2304, ["z"] = 1078, ["order"] = 8, ["angle"] = -6094848}
}
-- Respawn definitions for CK REGAL (MAP45)
krespawn.points[45] = {
  [0] = 140,
  [1] = {["x"] = 4289, ["y"] = 2108, ["z"] = 0, ["order"] = 0, ["angle"] = 1106313216},
  [2] = {["x"] = 4220, ["y"] = 2972, ["z"] = 0, ["order"] = 0, ["angle"] = 1150746624},
  [3] = {["x"] = 4095, ["y"] = 4128, ["z"] = 10, ["order"] = 0, ["angle"] = 1150746624},
  [4] = {["x"] = 3878, ["y"] = 5255, ["z"] = 10, ["order"] = 0, ["angle"] = 987299840},
  [5] = {["x"] = 3903, ["y"] = 6113, ["z"] = 170, ["order"] = 0, ["angle"] = 415694848},
  [6] = {["x"] = 4392, ["y"] = 6550, ["z"] = 220, ["order"] = 1, ["angle"] = -103481344},
  [7] = {["x"] = 5264, ["y"] = 6603, ["z"] = 327, ["order"] = 1, ["angle"] = -419627008},
  [8] = {["x"] = 6242, ["y"] = 6240, ["z"] = 501, ["order"] = 1, ["angle"] = -669843456},
  [9] = {["x"] = 7057, ["y"] = 5550, ["z"] = 645, ["order"] = 1, ["angle"] = -838270976},
  [10] = {["x"] = 7600, ["y"] = 4587, ["z"] = 741, ["order"] = 1, ["angle"] = -1240268800},
  [11] = {["x"] = 7625, ["y"] = 3689, ["z"] = 791, ["order"] = 1, ["angle"] = -1634795520},
  [12] = {["x"] = 6755, ["y"] = 2992, ["z"] = 804, ["order"] = 2, ["angle"] = 2123300864},
  [13] = {["x"] = 2916, ["y"] = 3008, ["z"] = 804, ["order"] = 2, ["angle"] = 2123300864},
  [14] = {["x"] = 2231, ["y"] = 3086, ["z"] = 804, ["order"] = 2, ["angle"] = -2020409344},
  [15] = {["x"] = 1270, ["y"] = 3180, ["z"] = 804, ["order"] = 2, ["angle"] = -1703346176},
  [16] = {["x"] = 486, ["y"] = 2993, ["z"] = 804, ["order"] = 3, ["angle"] = -1239875584},
  [17] = {["x"] = 172, ["y"] = 2124, ["z"] = 753, ["order"] = 3, ["angle"] = -754647040},
  [18] = {["x"] = 298, ["y"] = 1090, ["z"] = 703, ["order"] = 3, ["angle"] = -382009344},
  [19] = {["x"] = 871, ["y"] = 364, ["z"] = 567, ["order"] = 3, ["angle"] = -241106944},
  [20] = {["x"] = 1605, ["y"] = -44, ["z"] = 391, ["order"] = 3, ["angle"] = -225509376},
  [21] = {["x"] = 2581, ["y"] = -175, ["z"] = 159, ["order"] = 3, ["angle"] = -350945280},
  [22] = {["x"] = 3400, ["y"] = -368, ["z"] = 0, ["order"] = 3, ["angle"] = -825032704},
  [23] = {["x"] = 3773, ["y"] = -1061, ["z"] = 0, ["order"] = 3, ["angle"] = -1159397376},
  [24] = {["x"] = 3780, ["y"] = -1977, ["z"] = 0, ["order"] = 4, ["angle"] = -1227292672},
  [25] = {["x"] = 3613, ["y"] = -3088, ["z"] = 0, ["order"] = 4, ["angle"] = -1934295040},
  [26] = {["x"] = 2972, ["y"] = -3535, ["z"] = 0, ["order"] = 4, ["angle"] = 2124349440},
  [27] = {["x"] = 1989, ["y"] = -3535, ["z"] = 0, ["order"] = 4, ["angle"] = 2079653888},
  [28] = {["x"] = 483, ["y"] = -3514, ["z"] = 0, ["order"] = 4, ["angle"] = -2085814272},
  [29] = {["x"] = -248, ["y"] = -3442, ["z"] = 0, ["order"] = 4, ["angle"] = -2010054656},
  [30] = {["x"] = -1216, ["y"] = -3342, ["z"] = 0, ["order"] = 4, ["angle"] = -1751318528},
  [31] = {["x"] = -2091, ["y"] = -3508, ["z"] = 0, ["order"] = 4, ["angle"] = -1336606720},
  [32] = {["x"] = -2393, ["y"] = -4163, ["z"] = 0, ["order"] = 4, ["angle"] = -1189150720},
  [33] = {["x"] = -2433, ["y"] = -5319, ["z"] = 0, ["order"] = 4, ["angle"] = -1410531328},
  [34] = {["x"] = -2835, ["y"] = -6447, ["z"] = 0, ["order"] = 5, ["angle"] = -1621295104},
  [35] = {["x"] = -3530, ["y"] = -7125, ["z"] = 0, ["order"] = 5, ["angle"] = -2001403904},
  [36] = {["x"] = -4521, ["y"] = -7445, ["z"] = 0, ["order"] = 5, ["angle"] = 1995636736},
  [37] = {["x"] = -5649, ["y"] = -7450, ["z"] = 0, ["order"] = 5, ["angle"] = 1926299648},
  [38] = {["x"] = -6525, ["y"] = -7299, ["z"] = 0, ["order"] = 5, ["angle"] = 1565720576},
  [39] = {["x"] = -6983, ["y"] = -6688, ["z"] = 0, ["order"] = 5, ["angle"] = 1099104256},
  [40] = {["x"] = -7105, ["y"] = -5732, ["z"] = 0, ["order"] = 6, ["angle"] = 1099104256},
  [41] = {["x"] = -7172, ["y"] = -4442, ["z"] = 0, ["order"] = 6, ["angle"] = 1099104256},
  [42] = {["x"] = -7214, ["y"] = -3415, ["z"] = 0, ["order"] = 6, ["angle"] = 1099104256},
  [43] = {["x"] = -7353, ["y"] = -2424, ["z"] = 0, ["order"] = 6, ["angle"] = 764739584},
  [44] = {["x"] = -6792, ["y"] = -1487, ["z"] = 0, ["order"] = 6, ["angle"] = 251068416},
  [45] = {["x"] = -5749, ["y"] = -1123, ["z"] = 0, ["order"] = 6, ["angle"] = 350289920},
  [46] = {["x"] = -4763, ["y"] = -683, ["z"] = 0, ["order"] = 7, ["angle"] = 350289920},
  [47] = {["x"] = -3903, ["y"] = -323, ["z"] = -10, ["order"] = 7, ["angle"] = 405733376},
  [48] = {["x"] = -2866, ["y"] = 179, ["z"] = -162, ["order"] = 7, ["angle"] = 548470784},
  [49] = {["x"] = -1952, ["y"] = 913, ["z"] = -200, ["order"] = 7, ["angle"] = 900399104},
  [50] = {["x"] = -1378, ["y"] = 2155, ["z"] = -200, ["order"] = 7, ["angle"] = 1128726528},
  [51] = {["x"] = -1234, ["y"] = 3276, ["z"] = -200, ["order"] = 7, ["angle"] = 1228734464},
  [52] = {["x"] = -1327, ["y"] = 4809, ["z"] = -200, ["order"] = 7, ["angle"] = 1775435776},
  [53] = {["x"] = -2178, ["y"] = 5631, ["z"] = -200, ["order"] = 7, ["angle"] = -1862336512},
  [54] = {["x"] = -2886, ["y"] = 5430, ["z"] = -200, ["order"] = 7, ["angle"] = -2147024896},
  [55] = {["x"] = -2831, ["y"] = 6032, ["z"] = -200, ["order"] = 7, ["angle"] = 2112552960},
  [56] = {["x"] = -4388, ["y"] = 5765, ["z"] = -200, ["order"] = 7, ["angle"] = -2143485952},
  [57] = {["x"] = -5471, ["y"] = 5789, ["z"] = -200, ["order"] = 8, ["angle"] = -2141782016},
  [58] = {["x"] = -6523, ["y"] = 5949, ["z"] = -200, ["order"] = 8, ["angle"] = -1993932800},
  [59] = {["x"] = -7815, ["y"] = 6005, ["z"] = -200, ["order"] = 8, ["angle"] = -1663369216},
  [60] = {["x"] = -8737, ["y"] = 5342, ["z"] = -200, ["order"] = 8, ["angle"] = -1159397376},
  [61] = {["x"] = -9045, ["y"] = 3847, ["z"] = -200, ["order"] = 8, ["angle"] = -1263206400},
  [62] = {["x"] = -9277, ["y"] = 3935, ["z"] = -200, ["order"] = 8, ["angle"] = -913637376},
  [63] = {["x"] = -9480, ["y"] = 2874, ["z"] = -200, ["order"] = 8, ["angle"] = -1067253760},
  [64] = {["x"] = -8888, ["y"] = 2867, ["z"] = -200, ["order"] = 8, ["angle"] = -985989120},
  [65] = {["x"] = -9130, ["y"] = 1811, ["z"] = -200, ["order"] = 8, ["angle"] = -1121255424},
  [66] = {["x"] = -9275, ["y"] = 911, ["z"] = -200, ["order"] = 8, ["angle"] = -975241216},
  [67] = {["x"] = -8529, ["y"] = 647, ["z"] = -200, ["order"] = 8, ["angle"] = -1354825728},
  [68] = {["x"] = -9679, ["y"] = 511, ["z"] = -200, ["order"] = 8, ["angle"] = -1043005440},
  [69] = {["x"] = -9386, ["y"] = -539, ["z"] = -200, ["order"] = 8, ["angle"] = -1229258752},
  [70] = {["x"] = -9692, ["y"] = -1664, ["z"] = -200, ["order"] = 8, ["angle"] = -1108279296},
  [71] = {["x"] = -9759, ["y"] = -2891, ["z"] = -200, ["order"] = 8, ["angle"] = -1040384000},
  [72] = {["x"] = -9723, ["y"] = -4151, ["z"] = -167, ["order"] = 8, ["angle"] = -1040384000},
  [73] = {["x"] = -9682, ["y"] = -5134, ["z"] = 0, ["order"] = 9, ["angle"] = -1040384000},
  [74] = {["x"] = -9611, ["y"] = -6238, ["z"] = 319, ["order"] = 9, ["angle"] = -752812032},
  [75] = {["x"] = -9208, ["y"] = -7153, ["z"] = 610, ["order"] = 9, ["angle"] = 58785792},
  [76] = {["x"] = -8382, ["y"] = -7297, ["z"] = 656, ["order"] = 9, ["angle"] = 38076416},
  [77] = {["x"] = -7554, ["y"] = -7267, ["z"] = 704, ["order"] = 9, ["angle"] = 38076416},
  [78] = {["x"] = -6089, ["y"] = -7361, ["z"] = 704, ["order"] = 9, ["angle"] = -17367040},
  [79] = {["x"] = -4541, ["y"] = -7361, ["z"] = 704, ["order"] = 9, ["angle"] = -46465024},
  [80] = {["x"] = -3105, ["y"] = -7415, ["z"] = 704, ["order"] = 9, ["angle"] = 3080192},
  [81] = {["x"] = -1498, ["y"] = -7319, ["z"] = 704, ["order"] = 9, ["angle"] = 33619968},
  [82] = {["x"] = 268, ["y"] = -7257, ["z"] = 586, ["order"] = 10, ["angle"] = -239796224},
  [83] = {["x"] = 1919, ["y"] = -7093, ["z"] = 538, ["order"] = 10, ["angle"] = -295763968},
  [84] = {["x"] = 3275, ["y"] = -7228, ["z"] = 577, ["order"] = 10, ["angle"] = -39256064},
  [85] = {["x"] = 4442, ["y"] = -7248, ["z"] = 583, ["order"] = 10, ["angle"] = 119996416},
  [86] = {["x"] = 5874, ["y"] = -6892, ["z"] = 480, ["order"] = 10, ["angle"] = 602734592},
  [87] = {["x"] = 6454, ["y"] = -5894, ["z"] = 272, ["order"] = 11, ["angle"] = 1057030144},
  [88] = {["x"] = 6696, ["y"] = -5117, ["z"] = 272, ["order"] = 11, ["angle"] = 561971200},
  [89] = {["x"] = 7369, ["y"] = -4509, ["z"] = 272, ["order"] = 11, ["angle"] = 748093440},
  [90] = {["x"] = 7709, ["y"] = -3788, ["z"] = 272, ["order"] = 11, ["angle"] = 1473314816},
  [91] = {["x"] = 7143, ["y"] = -3114, ["z"] = 272, ["order"] = 11, ["angle"] = 1478295552},
  [92] = {["x"] = 6674, ["y"] = -2287, ["z"] = 244, ["order"] = 12, ["angle"] = 1293746176},
  [93] = {["x"] = 6415, ["y"] = -1258, ["z"] = 86, ["order"] = 12, ["angle"] = 1515126784},
  [94] = {["x"] = 5838, ["y"] = -373, ["z"] = 0, ["order"] = 12, ["angle"] = 1793392640},
  [95] = {["x"] = 4999, ["y"] = 79, ["z"] = 0, ["order"] = 12, ["angle"] = 1497825280},
  [96] = {["x"] = 4398, ["y"] = 733, ["z"] = 0, ["order"] = 12, ["angle"] = 1260978176},
  [97] = {["x"] = 4110, ["y"] = 1527, ["z"] = 0, ["order"] = 12, ["angle"] = 1114963968},
  [98] = {["x"] = 3926, ["y"] = 2788, ["z"] = 0, ["order"] = 0, ["angle"] = 1143930880},
  [99] = {["x"] = 3760, ["y"] = 4025, ["z"] = 10, ["order"] = 0, ["angle"] = 1026097152},
  [100] = {["x"] = 3819, ["y"] = 4889, ["z"] = 10, ["order"] = 0, ["angle"] = 991363072},
  [101] = {["x"] = 4071, ["y"] = -586, ["z"] = 0, ["order"] = 3, ["angle"] = -803799040},
  [102] = {["x"] = 4375, ["y"] = -1273, ["z"] = 0, ["order"] = 4, ["angle"] = -1052573696},
  [103] = {["x"] = 4444, ["y"] = -1989, ["z"] = 83, ["order"] = 4, ["angle"] = -1052573696},
  [104] = {["x"] = 4481, ["y"] = -2680, ["z"] = 191, ["order"] = 4, ["angle"] = -1214971904},
  [105] = {["x"] = 4407, ["y"] = -3701, ["z"] = 330, ["order"] = 4, ["angle"] = -1694826496},
  [106] = {["x"] = 3597, ["y"] = -4471, ["z"] = 330, ["order"] = 4, ["angle"] = 2107834368},
  [107] = {["x"] = 1273, ["y"] = -4579, ["z"] = 397, ["order"] = 4, ["angle"] = 2107834368},
  [108] = {["x"] = 85, ["y"] = -4565, ["z"] = 330, ["order"] = 4, ["angle"] = 2107834368},
  [109] = {["x"] = -3318, ["y"] = -4594, ["z"] = 330, ["order"] = 4, ["angle"] = 2008350720},
  [110] = {["x"] = -3919, ["y"] = -4562, ["z"] = 330, ["order"] = 4, ["angle"] = -1602682880},
  [111] = {["x"] = -4255, ["y"] = -5114, ["z"] = 330, ["order"] = 4, ["angle"] = -1263468544},
  [112] = {["x"] = -4410, ["y"] = -5956, ["z"] = 330, ["order"] = 5, ["angle"] = -1797980160},
  [113] = {["x"] = -5146, ["y"] = -6418, ["z"] = 330, ["order"] = 5, ["angle"] = 1943339008},
  [114] = {["x"] = -6108, ["y"] = -6136, ["z"] = 330, ["order"] = 5, ["angle"] = 1084424192},
  [115] = {["x"] = -6372, ["y"] = -5174, ["z"] = 330, ["order"] = 6, ["angle"] = 947978240},
  [116] = {["x"] = -6163, ["y"] = -3862, ["z"] = 330, ["order"] = 6, ["angle"] = 928317440},
  [117] = {["x"] = -6122, ["y"] = -2926, ["z"] = 330, ["order"] = 6, ["angle"] = 619642880},
  [118] = {["x"] = -5550, ["y"] = -2459, ["z"] = 330, ["order"] = 7, ["angle"] = 189726720},
  [119] = {["x"] = -4595, ["y"] = -2192, ["z"] = 330, ["order"] = 7, ["angle"] = 332070912},
  [120] = {["x"] = -4912, ["y"] = -2833, ["z"] = 330, ["order"] = 7, ["angle"] = -613482496},
  [121] = {["x"] = -4489, ["y"] = -3524, ["z"] = 330, ["order"] = 7, ["angle"] = 103612416},
  [122] = {["x"] = -3876, ["y"] = -3428, ["z"] = 330, ["order"] = 7, ["angle"] = 905379840},
  [123] = {["x"] = -3745, ["y"] = -2502, ["z"] = 330, ["order"] = 7, ["angle"] = 802488320},
  [124] = {["x"] = -3614, ["y"] = -1748, ["z"] = 330, ["order"] = 7, ["angle"] = 546242560},
  [125] = {["x"] = -3073, ["y"] = -1285, ["z"] = 330, ["order"] = 7, ["angle"] = 246480896},
  [126] = {["x"] = -2159, ["y"] = -1096, ["z"] = 330, ["order"] = 7, ["angle"] = 415432704},
  [127] = {["x"] = -1345, ["y"] = -574, ["z"] = 330, ["order"] = 7, ["angle"] = 1104084992},
  [128] = {["x"] = -6196, ["y"] = -7104, ["z"] = 704, ["order"] = 9, ["angle"] = -15269888},
  [129] = {["x"] = -4627, ["y"] = -7135, ["z"] = 704, ["order"] = 9, ["angle"] = -5570560},
  [130] = {["x"] = -3135, ["y"] = -7131, ["z"] = 704, ["order"] = 9, ["angle"] = -5570560},
  [131] = {["x"] = -1499, ["y"] = -7130, ["z"] = 704, ["order"] = 9, ["angle"] = -135987200},
  [132] = {["x"] = 944, ["y"] = -7251, ["z"] = 584, ["order"] = 10, ["angle"] = -240058368},
  [133] = {["x"] = 2595, ["y"] = -7202, ["z"] = 570, ["order"] = 10, ["angle"] = -284360704},
  [134] = {["x"] = 3826, ["y"] = -7234, ["z"] = 579, ["order"] = 10, ["angle"] = -104792064},
  [135] = {["x"] = 5063, ["y"] = -7197, ["z"] = 568, ["order"] = 10, ["angle"] = 165740544},
  [136] = {["x"] = 6492, ["y"] = -5327, ["z"] = 272, ["order"] = 11, ["angle"] = 1579745280},
  [137] = {["x"] = 6017, ["y"] = -4819, ["z"] = 272, ["order"] = 11, ["angle"] = 1376059392},
  [138] = {["x"] = 5462, ["y"] = -4087, ["z"] = 272, ["order"] = 11, ["angle"] = 987824128},
  [139] = {["x"] = 5581, ["y"] = -3354, ["z"] = 272, ["order"] = 11, ["angle"] = 651755520},
  [140] = {["x"] = 6361, ["y"] = -2532, ["z"] = 272, ["order"] = 11, ["angle"] = 1047592960}
}
-- Respawn definitions for SD2 BALLOON (MAP46)
krespawn.points[46] = {
  [0] = 38,
  [1] = {["x"] = -515, ["y"] = -1779, ["z"] = 0, ["order"] = 0, ["angle"] = 2132082688},
  [2] = {["x"] = -1079, ["y"] = -1824, ["z"] = 0, ["order"] = 0, ["angle"] = 2022506496},
  [3] = {["x"] = -1674, ["y"] = -1763, ["z"] = 0, ["order"] = 0, ["angle"] = 1989345280},
  [4] = {["x"] = -2280, ["y"] = -1637, ["z"] = 0, ["order"] = 1, ["angle"] = 1956839424},
  [5] = {["x"] = -2847, ["y"] = -1506, ["z"] = 0, ["order"] = 1, ["angle"] = 1925120000},
  [6] = {["x"] = -3458, ["y"] = -1386, ["z"] = 0, ["order"] = 1, ["angle"] = 1528102912},
  [7] = {["x"] = -3749, ["y"] = -1042, ["z"] = 0, ["order"] = 2, ["angle"] = 1057685504},
  [8] = {["x"] = -3757, ["y"] = -480, ["z"] = 0, ["order"] = 2, ["angle"] = 850460672},
  [9] = {["x"] = -3579, ["y"] = -2, ["z"] = 0, ["order"] = 3, ["angle"] = 744161280},
  [10] = {["x"] = -3494, ["y"] = 466, ["z"] = 0, ["order"] = 3, ["angle"] = 745340928},
  [11] = {["x"] = -3291, ["y"] = 429, ["z"] = 0, ["order"] = 3, ["angle"] = 964886528},
  [12] = {["x"] = -3251, ["y"] = 826, ["z"] = 0, ["order"] = 3, ["angle"] = 696582144},
  [13] = {["x"] = -3077, ["y"] = 1357, ["z"] = 0, ["order"] = 3, ["angle"] = 517406720},
  [14] = {["x"] = -2695, ["y"] = 1892, ["z"] = 0, ["order"] = 4, ["angle"] = -624754688},
  [15] = {["x"] = -2331, ["y"] = 1848, ["z"] = 0, ["order"] = 4, ["angle"] = -929103872},
  [16] = {["x"] = -2226, ["y"] = 1477, ["z"] = 0, ["order"] = 5, ["angle"] = -909049856},
  [17] = {["x"] = -2135, ["y"] = 1026, ["z"] = 0, ["order"] = 5, ["angle"] = -805634048},
  [18] = {["x"] = -1971, ["y"] = 448, ["z"] = 0, ["order"] = 5, ["angle"] = -1007353856},
  [19] = {["x"] = -1944, ["y"] = -159, ["z"] = 0, ["order"] = 5, ["angle"] = -440729600},
  [20] = {["x"] = -1636, ["y"] = -582, ["z"] = 0, ["order"] = 5, ["angle"] = 128385024},
  [21] = {["x"] = -1242, ["y"] = -585, ["z"] = 0, ["order"] = 6, ["angle"] = 561709056},
  [22] = {["x"] = -853, ["y"] = -305, ["z"] = 0, ["order"] = 6, ["angle"] = 487391232},
  [23] = {["x"] = -358, ["y"] = 123, ["z"] = 0, ["order"] = 7, ["angle"] = 780599296},
  [24] = {["x"] = -127, ["y"] = 553, ["z"] = 0, ["order"] = 7, ["angle"] = 784400384},
  [25] = {["x"] = 103, ["y"] = 1288, ["z"] = 0, ["order"] = 7, ["angle"] = 682426368},
  [26] = {["x"] = 372, ["y"] = 1643, ["z"] = 0, ["order"] = 8, ["angle"] = 97189888},
  [27] = {["x"] = 926, ["y"] = 1805, ["z"] = 0, ["order"] = 8, ["angle"] = -261029888},
  [28] = {["x"] = 1410, ["y"] = 1656, ["z"] = 0, ["order"] = 8, ["angle"] = -750321664},
  [29] = {["x"] = 1705, ["y"] = 1153, ["z"] = 0, ["order"] = 9, ["angle"] = -996343808},
  [30] = {["x"] = 1812, ["y"] = 673, ["z"] = 0, ["order"] = 9, ["angle"] = -1145634816},
  [31] = {["x"] = 1757, ["y"] = 218, ["z"] = 0, ["order"] = 9, ["angle"] = -1378549760},
  [32] = {["x"] = 1538, ["y"] = -235, ["z"] = 0, ["order"] = 9, ["angle"] = -1232535552},
  [33] = {["x"] = 1466, ["y"] = -774, ["z"] = 0, ["order"] = 10, ["angle"] = -1317994496},
  [34] = {["x"] = 1380, ["y"] = -1272, ["z"] = 0, ["order"] = 10, ["angle"] = -1562050560},
  [35] = {["x"] = 1118, ["y"] = -1658, ["z"] = 0, ["order"] = 10, ["angle"] = -1998389248},
  [36] = {["x"] = 688, ["y"] = -1821, ["z"] = 0, ["order"] = 11, ["angle"] = 2034171904},
  [37] = {["x"] = 202, ["y"] = -1805, ["z"] = 0, ["order"] = 11, ["angle"] = 2140602368},
  [38] = {["x"] = -313, ["y"] = -1784, ["z"] = 0, ["order"] = 11, ["angle"] = 2140602368}
}
-- Respawn definitions for SM SPECIAL (MAP47)
krespawn.points[47] = {
  [0] = 62,
  [1] = {["x"] = -3440, ["y"] = 1524, ["z"] = 0, ["order"] = 0, ["angle"] = -1074855936},
  [2] = {["x"] = -3505, ["y"] = 766, ["z"] = 0, ["order"] = 0, ["angle"] = -988217344},
  [3] = {["x"] = -3560, ["y"] = -125, ["z"] = 0, ["order"] = 0, ["angle"] = -803405824},
  [4] = {["x"] = -3424, ["y"] = -952, ["z"] = 0, ["order"] = 1, ["angle"] = -297598976},
  [5] = {["x"] = -2730, ["y"] = -1359, ["z"] = 0, ["order"] = 1, ["angle"] = 42270720},
  [6] = {["x"] = -1961, ["y"] = -1973, ["z"] = 0, ["order"] = 1, ["angle"] = 191692800},
  [7] = {["x"] = -1907, ["y"] = -1326, ["z"] = 0, ["order"] = 1, ["angle"] = -186974208},
  [8] = {["x"] = -1311, ["y"] = -1663, ["z"] = 0, ["order"] = 2, ["angle"] = 10944512},
  [9] = {["x"] = -727, ["y"] = -1493, ["z"] = 0, ["order"] = 2, ["angle"] = -57081856},
  [10] = {["x"] = -16, ["y"] = -1419, ["z"] = 0, ["order"] = 2, ["angle"] = -364838912},
  [11] = {["x"] = 700, ["y"] = -1580, ["z"] = 0, ["order"] = 2, ["angle"] = -812318720},
  [12] = {["x"] = 1062, ["y"] = -2087, ["z"] = 0, ["order"] = 2, ["angle"] = -1376714752},
  [13] = {["x"] = 927, ["y"] = -2653, ["z"] = 0, ["order"] = 2, ["angle"] = -1934688256},
  [14] = {["x"] = 382, ["y"] = -2842, ["z"] = 0, ["order"] = 3, ["angle"] = -2053832704},
  [15] = {["x"] = -377, ["y"] = -2796, ["z"] = 0, ["order"] = 3, ["angle"] = -2096693248},
  [16] = {["x"] = -1194, ["y"] = -2720, ["z"] = 0, ["order"] = 3, ["angle"] = -1825505280},
  [17] = {["x"] = -1849, ["y"] = -2856, ["z"] = 0, ["order"] = 3, ["angle"] = -1260060672},
  [18] = {["x"] = -2254, ["y"] = -3419, ["z"] = 0, ["order"] = 3, ["angle"] = -738394112},
  [19] = {["x"] = -2080, ["y"] = -4053, ["z"] = 0, ["order"] = 3, ["angle"] = -222232576},
  [20] = {["x"] = -1319, ["y"] = -4241, ["z"] = 0, ["order"] = 3, ["angle"] = -13565952},
  [21] = {["x"] = -564, ["y"] = -4208, ["z"] = 0, ["order"] = 4, ["angle"] = 40566784},
  [22] = {["x"] = 311, ["y"] = -4166, ["z"] = 0, ["order"] = 4, ["angle"] = 73465856},
  [23] = {["x"] = 1172, ["y"] = -4159, ["z"] = 0, ["order"] = 4, ["angle"] = 239403008},
  [24] = {["x"] = 2081, ["y"] = -4036, ["z"] = 0, ["order"] = 4, ["angle"] = 881786880},
  [25] = {["x"] = 2416, ["y"] = -3361, ["z"] = 0, ["order"] = 4, ["angle"] = 1037762560},
  [26] = {["x"] = 2627, ["y"] = -2648, ["z"] = 0, ["order"] = 5, ["angle"] = 1059258368},
  [27] = {["x"] = 2174, ["y"] = -2632, ["z"] = 0, ["order"] = 5, ["angle"] = 1149566976},
  [28] = {["x"] = 2068, ["y"] = -1693, ["z"] = 0, ["order"] = 5, ["angle"] = 1117454336},
  [29] = {["x"] = 2709, ["y"] = -1509, ["z"] = 0, ["order"] = 5, ["angle"] = 1163984896},
  [30] = {["x"] = 2721, ["y"] = -674, ["z"] = 0, ["order"] = 5, ["angle"] = 1118502912},
  [31] = {["x"] = 1996, ["y"] = -666, ["z"] = 0, ["order"] = 5, ["angle"] = 1055981568},
  [32] = {["x"] = 2583, ["y"] = 256, ["z"] = 0, ["order"] = 5, ["angle"] = 783220736},
  [33] = {["x"] = 3383, ["y"] = 1242, ["z"] = 0, ["order"] = 5, ["angle"] = 993067008},
  [34] = {["x"] = 3808, ["y"] = 2325, ["z"] = 0, ["order"] = 5, ["angle"] = 1614348288},
  [35] = {["x"] = 3793, ["y"] = 3068, ["z"] = 0, ["order"] = 5, ["angle"] = 1939406848},
  [36] = {["x"] = 3591, ["y"] = 3528, ["z"] = 0, ["order"] = 5, ["angle"] = 1907687424},
  [37] = {["x"] = 2919, ["y"] = 3750, ["z"] = 0, ["order"] = 5, ["angle"] = 2036137984},
  [38] = {["x"] = 2861, ["y"] = 4296, ["z"] = 0, ["order"] = 5, ["angle"] = 1997864960},
  [39] = {["x"] = 2181, ["y"] = 4371, ["z"] = 0, ["order"] = 5, ["angle"] = -1884880896},
  [40] = {["x"] = 2200, ["y"] = 3614, ["z"] = 0, ["order"] = 5, ["angle"] = 2031157248},
  [41] = {["x"] = 1474, ["y"] = 3990, ["z"] = 0, ["order"] = 5, ["angle"] = -2079260672},
  [42] = {["x"] = 996, ["y"] = 4509, ["z"] = 0, ["order"] = 5, ["angle"] = -2063269888},
  [43] = {["x"] = 2077, ["y"] = 332, ["z"] = 0, ["order"] = 5, ["angle"] = 1329004544},
  [44] = {["x"] = 1576, ["y"] = 1158, ["z"] = 0, ["order"] = 5, ["angle"] = 1221918720},
  [45] = {["x"] = 2265, ["y"] = 1612, ["z"] = 0, ["order"] = 5, ["angle"] = 1376452608},
  [46] = {["x"] = 2793, ["y"] = 1759, ["z"] = 0, ["order"] = 5, ["angle"] = 1598881792},
  [47] = {["x"] = 2968, ["y"] = 2512, ["z"] = 0, ["order"] = 5, ["angle"] = -2133393408},
  [48] = {["x"] = 2957, ["y"] = 2827, ["z"] = 0, ["order"] = 5, ["angle"] = 1640955904},
  [49] = {["x"] = 2900, ["y"] = 3317, ["z"] = 0, ["order"] = 5, ["angle"] = 1996292096},
  [50] = {["x"] = 1940, ["y"] = 2931, ["z"] = 0, ["order"] = 5, ["angle"] = 2017394688},
  [51] = {["x"] = 1300, ["y"] = 3280, ["z"] = 0, ["order"] = 5, ["angle"] = 1900085248},
  [52] = {["x"] = 286, ["y"] = 3533, ["z"] = 0, ["order"] = 5, ["angle"] = -2065235968},
  [53] = {["x"] = 297, ["y"] = 3792, ["z"] = 0, ["order"] = 5, ["angle"] = -2139553792},
  [54] = {["x"] = 257, ["y"] = 4123, ["z"] = 0, ["order"] = 5, ["angle"] = 2099707904},
  [55] = {["x"] = 253, ["y"] = 2284, ["z"] = 0, ["order"] = 5, ["angle"] = 1618804736},
  [56] = {["x"] = -261, ["y"] = 2844, ["z"] = 0, ["order"] = 5, ["angle"] = 1574371328},
  [57] = {["x"] = -799, ["y"] = 3704, ["z"] = 0, ["order"] = 5, ["angle"] = -2085027840},
  [58] = {["x"] = -1526, ["y"] = 3700, ["z"] = 0, ["order"] = 6, ["angle"] = -1953824768},
  [59] = {["x"] = -2330, ["y"] = 3644, ["z"] = 0, ["order"] = 6, ["angle"] = -1743716352},
  [60] = {["x"] = -3132, ["y"] = 3275, ["z"] = 0, ["order"] = 6, ["angle"] = -1343684608},
  [61] = {["x"] = -3432, ["y"] = 2801, ["z"] = 0, ["order"] = 6, ["angle"] = -1109852160},
  [62] = {["x"] = -3485, ["y"] = 1953, ["z"] = 0, ["order"] = 6, ["angle"] = -1003814912}
}
-- Respawn definitions for MKSC SKY (MAP48)
krespawn.points[48] = {
  [0] = 68,
  [1] = {["x"] = -2863, ["y"] = 10098, ["z"] = 0, ["order"] = 0, ["angle"] = 1114570752},
  [2] = {["x"] = -2827, ["y"] = 10604, ["z"] = 0, ["order"] = 0, ["angle"] = 1128726528},
  [3] = {["x"] = -2767, ["y"] = 11410, ["z"] = 0, ["order"] = 0, ["angle"] = 1153761280},
  [4] = {["x"] = -2714, ["y"] = 12430, ["z"] = 0, ["order"] = 0, ["angle"] = 1625751552},
  [5] = {["x"] = -3053, ["y"] = 13097, ["z"] = 0, ["order"] = 0, ["angle"] = 1986723840},
  [6] = {["x"] = -3700, ["y"] = 13376, ["z"] = 0, ["order"] = 1, ["angle"] = -2006646784},
  [7] = {["x"] = -4539, ["y"] = 13369, ["z"] = 0, ["order"] = 1, ["angle"] = -1935474688},
  [8] = {["x"] = -5425, ["y"] = 13143, ["z"] = 0, ["order"] = 1, ["angle"] = -1814888448},
  [9] = {["x"] = -6172, ["y"] = 12628, ["z"] = 0, ["order"] = 1, ["angle"] = -1774125056},
  [10] = {["x"] = -6805, ["y"] = 12152, ["z"] = 0, ["order"] = 1, ["angle"] = -2055274496},
  [11] = {["x"] = -7559, ["y"] = 11926, ["z"] = 0, ["order"] = 1, ["angle"] = 2052915200},
  [12] = {["x"] = -9052, ["y"] = 11969, ["z"] = 0, ["order"] = 1, ["angle"] = 2014248960},
  [13] = {["x"] = -9674, ["y"] = 12077, ["z"] = 0, ["order"] = 1, ["angle"] = 1648033792},
  [14] = {["x"] = -10212, ["y"] = 12550, ["z"] = 0, ["order"] = 1, ["angle"] = 1685913600},
  [15] = {["x"] = -10742, ["y"] = 13168, ["z"] = 0, ["order"] = 1, ["angle"] = 1836515328},
  [16] = {["x"] = -11577, ["y"] = 13843, ["z"] = 0, ["order"] = 1, ["angle"] = -2030895104},
  [17] = {["x"] = -12389, ["y"] = 14049, ["z"] = 0, ["order"] = 1, ["angle"] = -1438318592},
  [18] = {["x"] = -12933, ["y"] = 13650, ["z"] = 0, ["order"] = 1, ["angle"] = -1112735744},
  [19] = {["x"] = -13061, ["y"] = 12948, ["z"] = 0, ["order"] = 1, ["angle"] = -1057423360},
  [20] = {["x"] = -13141, ["y"] = 11940, ["z"] = 0, ["order"] = 2, ["angle"] = -1071054848},
  [21] = {["x"] = -13259, ["y"] = 10777, ["z"] = 0, ["order"] = 2, ["angle"] = -959643648},
  [22] = {["x"] = -13304, ["y"] = 9656, ["z"] = 0, ["order"] = 2, ["angle"] = -1029111808},
  [23] = {["x"] = -13049, ["y"] = 8794, ["z"] = 0, ["order"] = 3, ["angle"] = -671154176},
  [24] = {["x"] = -12566, ["y"] = 8219, ["z"] = 0, ["order"] = 3, ["angle"] = -726728704},
  [25] = {["x"] = -11820, ["y"] = 7499, ["z"] = 0, ["order"] = 3, ["angle"] = -1149960192},
  [26] = {["x"] = -11581, ["y"] = 6697, ["z"] = 0, ["order"] = 3, ["angle"] = -1819738112},
  [27] = {["x"] = -12020, ["y"] = 6327, ["z"] = 0, ["order"] = 3, ["angle"] = -1977942016},
  [28] = {["x"] = -12773, ["y"] = 6248, ["z"] = 0, ["order"] = 3, ["angle"] = -1900740608},
  [29] = {["x"] = -13699, ["y"] = 6167, ["z"] = 0, ["order"] = 3, ["angle"] = -1410793472},
  [30] = {["x"] = -14272, ["y"] = 5642, ["z"] = 0, ["order"] = 3, ["angle"] = -1113260032},
  [31] = {["x"] = -14536, ["y"] = 4754, ["z"] = 0, ["order"] = 4, ["angle"] = -975503360},
  [32] = {["x"] = -14653, ["y"] = 3806, ["z"] = 0, ["order"] = 4, ["angle"] = -805502976},
  [33] = {["x"] = -14526, ["y"] = 2896, ["z"] = 31, ["order"] = 4, ["angle"] = -298909696},
  [34] = {["x"] = -14042, ["y"] = 2326, ["z"] = 100, ["order"] = 4, ["angle"] = 312147968},
  [35] = {["x"] = -13434, ["y"] = 2380, ["z"] = 93, ["order"] = 4, ["angle"] = 495779840},
  [36] = {["x"] = -12916, ["y"] = 2933, ["z"] = 26, ["order"] = 4, ["angle"] = 388825088},
  [37] = {["x"] = -12144, ["y"] = 3641, ["z"] = 0, ["order"] = 4, ["angle"] = 137691136},
  [38] = {["x"] = -11333, ["y"] = 3828, ["z"] = 0, ["order"] = 4, ["angle"] = 34537472},
  [39] = {["x"] = -10481, ["y"] = 3769, ["z"] = 0, ["order"] = 5, ["angle"] = 352911360},
  [40] = {["x"] = -9558, ["y"] = 3951, ["z"] = 0, ["order"] = 5, ["angle"] = 641007616},
  [41] = {["x"] = -8960, ["y"] = 4505, ["z"] = 0, ["order"] = 5, ["angle"] = 694484992},
  [42] = {["x"] = -8584, ["y"] = 5228, ["z"] = 0, ["order"] = 5, ["angle"] = 722796544},
  [43] = {["x"] = -8263, ["y"] = 6041, ["z"] = 0, ["order"] = 6, ["angle"] = 591462400},
  [44] = {["x"] = -7886, ["y"] = 6792, ["z"] = 123, ["order"] = 6, ["angle"] = 459997184},
  [45] = {["x"] = -7304, ["y"] = 7607, ["z"] = 268, ["order"] = 6, ["angle"] = 121438208},
  [46] = {["x"] = -6462, ["y"] = 8056, ["z"] = 379, ["order"] = 6, ["angle"] = -535232512},
  [47] = {["x"] = -5820, ["y"] = 7857, ["z"] = 392, ["order"] = 6, ["angle"] = -871301120},
  [48] = {["x"] = -5351, ["y"] = 7078, ["z"] = 316, ["order"] = 6, ["angle"] = -1385627648},
  [49] = {["x"] = -5613, ["y"] = 6388, ["z"] = 207, ["order"] = 6, ["angle"] = -1595211776},
  [50] = {["x"] = -6260, ["y"] = 5858, ["z"] = 97, ["order"] = 6, ["angle"] = -1518403584},
  [51] = {["x"] = -6824, ["y"] = 5207, ["z"] = 0, ["order"] = 7, ["angle"] = -1286406144},
  [52] = {["x"] = -7083, ["y"] = 4422, ["z"] = 0, ["order"] = 7, ["angle"] = -1016004608},
  [53] = {["x"] = -7174, ["y"] = 3608, ["z"] = 0, ["order"] = 7, ["angle"] = -849543168},
  [54] = {["x"] = -7136, ["y"] = 2776, ["z"] = 0, ["order"] = 7, ["angle"] = -473497600},
  [55] = {["x"] = -6698, ["y"] = 2155, ["z"] = 0, ["order"] = 7, ["angle"] = 170590208},
  [56] = {["x"] = -6055, ["y"] = 2041, ["z"] = 0, ["order"] = 7, ["angle"] = 464060416},
  [57] = {["x"] = -5478, ["y"] = 2458, ["z"] = 0, ["order"] = 7, ["angle"] = 459866112},
  [58] = {["x"] = -4782, ["y"] = 2905, ["z"] = 0, ["order"] = 7, ["angle"] = 198246400},
  [59] = {["x"] = -3710, ["y"] = 3143, ["z"] = 0, ["order"] = 8, ["angle"] = 326959104},
  [60] = {["x"] = -2775, ["y"] = 3426, ["z"] = 0, ["order"] = 8, ["angle"] = 719650816},
  [61] = {["x"] = -2194, ["y"] = 4095, ["z"] = 0, ["order"] = 8, ["angle"] = 1189675008},
  [62] = {["x"] = -2275, ["y"] = 4832, ["z"] = 0, ["order"] = 8, ["angle"] = 1286930432},
  [63] = {["x"] = -2619, ["y"] = 5614, ["z"] = 0, ["order"] = 8, ["angle"] = 1166082048},
  [64] = {["x"] = -2727, ["y"] = 7035, ["z"] = 0, ["order"] = 9, ["angle"] = 1095827456},
  [65] = {["x"] = -2730, ["y"] = 7518, ["z"] = 0, ["order"] = 9, ["angle"] = 1296891904},
  [66] = {["x"] = -2852, ["y"] = 8122, ["z"] = 0, ["order"] = 9, ["angle"] = 1056505856},
  [67] = {["x"] = -2887, ["y"] = 8800, ["z"] = 0, ["order"] = 9, ["angle"] = 1056505856},
  [68] = {["x"] = -2888, ["y"] = 9668, ["z"] = 0, ["order"] = 9, ["angle"] = 959381504}
}
-- Respawn definitions for MKDS PEACH (MAP49)
krespawn.points[49] = {
  [0] = 59,
  [1] = {["x"] = -9201, ["y"] = 2153, ["z"] = 0, ["order"] = 0, ["angle"] = -1098579968},
  [2] = {["x"] = -9242, ["y"] = 1604, ["z"] = 0, ["order"] = 0, ["angle"] = -1089798144},
  [3] = {["x"] = -9210, ["y"] = 796, ["z"] = 0, ["order"] = 0, ["angle"] = -1069875200},
  [4] = {["x"] = -9073, ["y"] = -317, ["z"] = 28, ["order"] = 1, ["angle"] = -620167168},
  [5] = {["x"] = -8569, ["y"] = -1212, ["z"] = 0, ["order"] = 1, ["angle"] = -463798272},
  [6] = {["x"] = -7840, ["y"] = -1949, ["z"] = 0, ["order"] = 1, ["angle"] = -83165184},
  [7] = {["x"] = -7151, ["y"] = -2116, ["z"] = 64, ["order"] = 1, ["angle"] = 96927744},
  [8] = {["x"] = -6193, ["y"] = -2182, ["z"] = 0, ["order"] = 2, ["angle"] = 90505216},
  [9] = {["x"] = -5072, ["y"] = -2201, ["z"] = 0, ["order"] = 2, ["angle"] = 223805440},
  [10] = {["x"] = -4210, ["y"] = -1804, ["z"] = 0, ["order"] = 2, ["angle"] = 330366976},
  [11] = {["x"] = -3432, ["y"] = -1232, ["z"] = 0, ["order"] = 2, ["angle"] = -255393792},
  [12] = {["x"] = -2648, ["y"] = -1315, ["z"] = 0, ["order"] = 2, ["angle"] = -913506304},
  [13] = {["x"] = -2271, ["y"] = -1923, ["z"] = 0, ["order"] = 2, ["angle"] = -1398079488},
  [14] = {["x"] = -2776, ["y"] = -2619, ["z"] = 0, ["order"] = 2, ["angle"] = -1417609216},
  [15] = {["x"] = -3156, ["y"] = -3546, ["z"] = 16, ["order"] = 3, ["angle"] = -1038024704},
  [16] = {["x"] = -3164, ["y"] = -4448, ["z"] = 120, ["order"] = 3, ["angle"] = -1055326208},
  [17] = {["x"] = -3052, ["y"] = -5703, ["z"] = 256, ["order"] = 3, ["angle"] = -946143232},
  [18] = {["x"] = -2700, ["y"] = -6448, ["z"] = 256, ["order"] = 3, ["angle"] = -1122172928},
  [19] = {["x"] = -2822, ["y"] = -7471, ["z"] = 256, ["order"] = 3, ["angle"] = -1897332736},
  [20] = {["x"] = -3573, ["y"] = -7764, ["z"] = 256, ["order"] = 3, ["angle"] = 2116222976},
  [21] = {["x"] = -4791, ["y"] = -7734, ["z"] = 145, ["order"] = 3, ["angle"] = 2033254400},
  [22] = {["x"] = -6186, ["y"] = -7375, ["z"] = 0, ["order"] = 4, ["angle"] = 1818427392},
  [23] = {["x"] = -7605, ["y"] = -6649, ["z"] = 0, ["order"] = 4, ["angle"] = 1946222592},
  [24] = {["x"] = -8412, ["y"] = -6763, ["z"] = 0, ["order"] = 4, ["angle"] = -1773731840},
  [25] = {["x"] = -9341, ["y"] = -7150, ["z"] = 0, ["order"] = 4, ["angle"] = -1867972608},
  [26] = {["x"] = -10226, ["y"] = -7631, ["z"] = 0, ["order"] = 4, ["angle"] = -1952907264},
  [27] = {["x"] = -11125, ["y"] = -8080, ["z"] = 0, ["order"] = 4, ["angle"] = -1942552576},
  [28] = {["x"] = -12262, ["y"] = -8062, ["z"] = 0, ["order"] = 4, ["angle"] = 1869152256},
  [29] = {["x"] = -6211, ["y"] = -8030, ["z"] = 0, ["order"] = 4, ["angle"] = -1899036672},
  [30] = {["x"] = -7150, ["y"] = -8438, ["z"] = 0, ["order"] = 4, ["angle"] = -1798111232},
  [31] = {["x"] = -8366, ["y"] = -9067, ["z"] = 0, ["order"] = 4, ["angle"] = -1924202496},
  [32] = {["x"] = -10232, ["y"] = -8933, ["z"] = 0, ["order"] = 4, ["angle"] = 1866661888},
  [33] = {["x"] = -7284, ["y"] = -7522, ["z"] = 0, ["order"] = 4, ["angle"] = -1990393856},
  [34] = {["x"] = -7380, ["y"] = -7939, ["z"] = 0, ["order"] = 4, ["angle"] = 1916469248},
  [35] = {["x"] = -8573, ["y"] = -7848, ["z"] = 0, ["order"] = 4, ["angle"] = -1741619200},
  [36] = {["x"] = -8493, ["y"] = -7474, ["z"] = 0, ["order"] = 4, ["angle"] = 1843068928},
  [37] = {["x"] = -10478, ["y"] = -6541, ["z"] = 0, ["order"] = 4, ["angle"] = -1799946240},
  [38] = {["x"] = -11094, ["y"] = -6913, ["z"] = 0, ["order"] = 4, ["angle"] = -1920270336},
  [39] = {["x"] = -12038, ["y"] = -7264, ["z"] = 0, ["order"] = 4, ["angle"] = -1891434496},
  [40] = {["x"] = -12973, ["y"] = -7688, ["z"] = 0, ["order"] = 4, ["angle"] = 2136014848},
  [41] = {["x"] = -14278, ["y"] = -7780, ["z"] = 256, ["order"] = 4, ["angle"] = 1841102848},
  [42] = {["x"] = -15162, ["y"] = -7463, ["z"] = 256, ["order"] = 4, ["angle"] = 1508835328},
  [43] = {["x"] = -15602, ["y"] = -6838, ["z"] = 256, ["order"] = 4, ["angle"] = 901840896},
  [44] = {["x"] = -15280, ["y"] = -5737, ["z"] = 256, ["order"] = 4, ["angle"] = 1047068672},
  [45] = {["x"] = -15254, ["y"] = -4102, ["z"] = 80, ["order"] = 4, ["angle"] = 1265827840},
  [46] = {["x"] = -15738, ["y"] = -2842, ["z"] = 0, ["order"] = 5, ["angle"] = 935002112},
  [47] = {["x"] = -14708, ["y"] = -2860, ["z"] = 0, ["order"] = 5, ["angle"] = 1322582016},
  [48] = {["x"] = -15367, ["y"] = -1547, ["z"] = 0, ["order"] = 5, ["angle"] = 1115226112},
  [49] = {["x"] = -15855, ["y"] = -100, ["z"] = 0, ["order"] = 5, ["angle"] = 736034816},
  [50] = {["x"] = -15435, ["y"] = 777, ["z"] = 0, ["order"] = 5, ["angle"] = 754253824},
  [51] = {["x"] = -14855, ["y"] = 1851, ["z"] = 0, ["order"] = 5, ["angle"] = 1231355904},
  [52] = {["x"] = -15278, ["y"] = 2890, ["z"] = 0, ["order"] = 5, ["angle"] = 1158086656},
  [53] = {["x"] = -15433, ["y"] = 3876, ["z"] = 0, ["order"] = 5, ["angle"] = 131530752},
  [54] = {["x"] = -14569, ["y"] = 4135, ["z"] = 49, ["order"] = 6, ["angle"] = -3866624},
  [55] = {["x"] = -13374, ["y"] = 4158, ["z"] = 160, ["order"] = 6, ["angle"] = -3866624},
  [56] = {["x"] = -12291, ["y"] = 4271, ["z"] = 160, ["order"] = 6, ["angle"] = -2555904},
  [57] = {["x"] = -11005, ["y"] = 4382, ["z"] = 160, ["order"] = 6, ["angle"] = -302841856},
  [58] = {["x"] = -9738, ["y"] = 3928, ["z"] = 160, ["order"] = 6, ["angle"] = -901054464},
  [59] = {["x"] = -9224, ["y"] = 2782, ["z"] = 0, ["order"] = 6, ["angle"] = -1020461056}
}
-- Respawn definitions for MKSC RAINBOW (MAP50)
krespawn.points[50] = {
  [0] = 101,
  [1] = {["x"] = 923, ["y"] = -8928, ["z"] = 64, ["order"] = 0, ["angle"] = 1060700160},
  [2] = {["x"] = 936, ["y"] = -8340, ["z"] = 64, ["order"] = 0, ["angle"] = 1060700160},
  [3] = {["x"] = 899, ["y"] = -7730, ["z"] = 64, ["order"] = 0, ["angle"] = 1002373120},
  [4] = {["x"] = 851, ["y"] = -6918, ["z"] = 64, ["order"] = 0, ["angle"] = 714932224},
  [5] = {["x"] = 993, ["y"] = -6391, ["z"] = 64, ["order"] = 1, ["angle"] = 483852288},
  [6] = {["x"] = 1266, ["y"] = -6049, ["z"] = 64, ["order"] = 1, ["angle"] = 156434432},
  [7] = {["x"] = 1681, ["y"] = -5858, ["z"] = 64, ["order"] = 2, ["angle"] = 27459584},
  [8] = {["x"] = 2407, ["y"] = -5939, ["z"] = 64, ["order"] = 2, ["angle"] = 263258112},
  [9] = {["x"] = 2900, ["y"] = -5853, ["z"] = 64, ["order"] = 2, ["angle"] = 690028544},
  [10] = {["x"] = 3276, ["y"] = -5486, ["z"] = 64, ["order"] = 2, ["angle"] = 913244160},
  [11] = {["x"] = 3420, ["y"] = -5155, ["z"] = 64, ["order"] = 3, ["angle"] = 1027670016},
  [12] = {["x"] = 3468, ["y"] = -4857, ["z"] = 64, ["order"] = 3, ["angle"] = 1187446784},
  [13] = {["x"] = 3469, ["y"] = -4634, ["z"] = 64, ["order"] = 4, ["angle"] = 1116012544},
  [14] = {["x"] = 3450, ["y"] = -4167, ["z"] = 64, ["order"] = 4, ["angle"] = 1116012544},
  [15] = {["x"] = 3440, ["y"] = -3505, ["z"] = 64, ["order"] = 4, ["angle"] = 1057554432},
  [16] = {["x"] = 3423, ["y"] = -2999, ["z"] = 64, ["order"] = 4, ["angle"] = 978255872},
  [17] = {["x"] = 3407, ["y"] = -2452, ["z"] = 64, ["order"] = 4, ["angle"] = 890437632},
  [18] = {["x"] = 3378, ["y"] = -1891, ["z"] = 64, ["order"] = 4, ["angle"] = 740491264},
  [19] = {["x"] = 3617, ["y"] = -1334, ["z"] = 64, ["order"] = 4, ["angle"] = 365232128},
  [20] = {["x"] = 4057, ["y"] = -973, ["z"] = 64, ["order"] = 4, ["angle"] = -144637952},
  [21] = {["x"] = 4769, ["y"] = -982, ["z"] = 64, ["order"] = 5, ["angle"] = -622002176},
  [22] = {["x"] = 5121, ["y"] = -1194, ["z"] = 64, ["order"] = 5, ["angle"] = -734461952},
  [23] = {["x"] = 5516, ["y"] = -1614, ["z"] = 64, ["order"] = 5, ["angle"] = -952303616},
  [24] = {["x"] = 5622, ["y"] = -2053, ["z"] = 64, ["order"] = 6, ["angle"] = -1044447232},
  [25] = {["x"] = 5622, ["y"] = -2467, ["z"] = 64, ["order"] = 7, ["angle"] = -1114832896},
  [26] = {["x"] = 5632, ["y"] = -3011, ["z"] = 64, ["order"] = 7, ["angle"] = -1103822848},
  [27] = {["x"] = 5651, ["y"] = -3637, ["z"] = 64, ["order"] = 7, ["angle"] = -1057161216},
  [28] = {["x"] = 5641, ["y"] = -4247, ["z"] = 64, ["order"] = 7, ["angle"] = -1110507520},
  [29] = {["x"] = 5632, ["y"] = -4820, ["z"] = 64, ["order"] = 7, ["angle"] = -1038286848},
  [30] = {["x"] = 5655, ["y"] = -5576, ["z"] = 64, ["order"] = 7, ["angle"] = -1080098816},
  [31] = {["x"] = 5654, ["y"] = -6145, ["z"] = 64, ["order"] = 7, ["angle"] = -1080098816},
  [32] = {["x"] = 5649, ["y"] = -6719, ["z"] = 64, ["order"] = 7, ["angle"] = -1080098816},
  [33] = {["x"] = 5575, ["y"] = -7360, ["z"] = 64, ["order"] = 7, ["angle"] = -1073807360},
  [34] = {["x"] = 5502, ["y"] = -7968, ["z"] = 64, ["order"] = 7, ["angle"] = -932773888},
  [35] = {["x"] = 5458, ["y"] = -8590, ["z"] = 64, ["order"] = 7, ["angle"] = -902103040},
  [36] = {["x"] = 5377, ["y"] = -9235, ["z"] = 64, ["order"] = 8, ["angle"] = -432603136},
  [37] = {["x"] = 5559, ["y"] = -9672, ["z"] = 64, ["order"] = 8, ["angle"] = 125239296},
  [38] = {["x"] = 5993, ["y"] = -9866, ["z"] = 64, ["order"] = 8, ["angle"] = 413466624},
  [39] = {["x"] = 6465, ["y"] = -9498, ["z"] = 64, ["order"] = 8, ["angle"] = 502202368},
  [40] = {["x"] = 6839, ["y"] = -9009, ["z"] = 64, ["order"] = 8, ["angle"] = 389480448},
  [41] = {["x"] = 7373, ["y"] = -8596, ["z"] = 64, ["order"] = 8, ["angle"] = 197197824},
  [42] = {["x"] = 8071, ["y"] = -8490, ["z"] = 64, ["order"] = 9, ["angle"] = 154861568},
  [43] = {["x"] = 8821, ["y"] = -8411, ["z"] = 64, ["order"] = 9, ["angle"] = 519503872},
  [44] = {["x"] = 9483, ["y"] = -7955, ["z"] = 64, ["order"] = 9, ["angle"] = 603389952},
  [45] = {["x"] = 10076, ["y"] = -7448, ["z"] = 64, ["order"] = 9, ["angle"] = 579141632},
  [46] = {["x"] = 10546, ["y"] = -7042, ["z"] = 64, ["order"] = 9, ["angle"] = 775880704},
  [47] = {["x"] = 10885, ["y"] = -6572, ["z"] = 64, ["order"] = 9, ["angle"] = 996999168},
  [48] = {["x"] = 11078, ["y"] = -6032, ["z"] = 64, ["order"] = 9, ["angle"] = 1067122688},
  [49] = {["x"] = 11062, ["y"] = -5399, ["z"] = 64, ["order"] = 10, ["angle"] = 1448017920},
  [50] = {["x"] = 10722, ["y"] = -4893, ["z"] = 64, ["order"] = 10, ["angle"] = 1552220160},
  [51] = {["x"] = 10260, ["y"] = -4534, ["z"] = 64, ["order"] = 10, ["angle"] = 1739128832},
  [52] = {["x"] = 9590, ["y"] = -4270, ["z"] = 64, ["order"] = 11, ["angle"] = 1586036736},
  [53] = {["x"] = 9065, ["y"] = -3877, ["z"] = 64, ["order"] = 11, ["angle"] = 1160445952},
  [54] = {["x"] = 8804, ["y"] = -3334, ["z"] = 64, ["order"] = 11, ["angle"] = 795279360},
  [55] = {["x"] = 8854, ["y"] = -2779, ["z"] = 64, ["order"] = 12, ["angle"] = 512294912},
  [56] = {["x"] = 9166, ["y"] = -2313, ["z"] = 64, ["order"] = 12, ["angle"] = 30867456},
  [57] = {["x"] = 9605, ["y"] = -2281, ["z"] = 64, ["order"] = 13, ["angle"] = -209387520},
  [58] = {["x"] = 10130, ["y"] = -2618, ["z"] = 64, ["order"] = 13, ["angle"] = -327680},
  [59] = {["x"] = 10713, ["y"] = -2762, ["z"] = 64, ["order"] = 13, ["angle"] = 457768960},
  [60] = {["x"] = 11078, ["y"] = -2428, ["z"] = 64, ["order"] = 13, ["angle"] = 692256768},
  [61] = {["x"] = 11264, ["y"] = -1868, ["z"] = 64, ["order"] = 13, ["angle"] = 601030656},
  [62] = {["x"] = 11411, ["y"] = -1317, ["z"] = 64, ["order"] = 14, ["angle"] = 401276928},
  [63] = {["x"] = 11921, ["y"] = -857, ["z"] = 64, ["order"] = 14, ["angle"] = 12124160},
  [64] = {["x"] = 12530, ["y"] = -731, ["z"] = 64, ["order"] = 15, ["angle"] = -186580992},
  [65] = {["x"] = 13145, ["y"] = -748, ["z"] = 64, ["order"] = 15, ["angle"] = -454230016},
  [66] = {["x"] = 13762, ["y"] = -998, ["z"] = 64, ["order"] = 16, ["angle"] = -767623168},
  [67] = {["x"] = 14085, ["y"] = -1660, ["z"] = 64, ["order"] = 17, ["angle"] = -1088356352},
  [68] = {["x"] = 14108, ["y"] = -2328, ["z"] = 64, ["order"] = 17, ["angle"] = -1235681280},
  [69] = {["x"] = 14011, ["y"] = -3142, ["z"] = 64, ["order"] = 17, ["angle"] = -1259405312},
  [70] = {["x"] = 13702, ["y"] = -3723, ["z"] = 64, ["order"] = 18, ["angle"] = -1465188352},
  [71] = {["x"] = 13280, ["y"] = -4234, ["z"] = 64, ["order"] = 18, ["angle"] = -1247477760},
  [72] = {["x"] = 13156, ["y"] = -4910, ["z"] = 64, ["order"] = 18, ["angle"] = -1008926720},
  [73] = {["x"] = 13225, ["y"] = -5254, ["z"] = 64, ["order"] = 18, ["angle"] = -1066729472},
  [74] = {["x"] = 13173, ["y"] = -6264, ["z"] = 64, ["order"] = 18, ["angle"] = -1070792704},
  [75] = {["x"] = 13259, ["y"] = -7293, ["z"] = 64, ["order"] = 18, ["angle"] = -1076559872},
  [76] = {["x"] = 13189, ["y"] = -8333, ["z"] = 64, ["order"] = 18, ["angle"] = -1002242048},
  [77] = {["x"] = 13107, ["y"] = -9405, ["z"] = 64, ["order"] = 19, ["angle"] = -905641984},
  [78] = {["x"] = 13327, ["y"] = -10213, ["z"] = 64, ["order"] = 19, ["angle"] = -617283584},
  [79] = {["x"] = 14028, ["y"] = -10900, ["z"] = 64, ["order"] = 19, ["angle"] = -568393728},
  [80] = {["x"] = 14634, ["y"] = -11414, ["z"] = 64, ["order"] = 20, ["angle"] = -688717824},
  [81] = {["x"] = 15220, ["y"] = -12031, ["z"] = 64, ["order"] = 20, ["angle"] = -968032256},
  [82] = {["x"] = 15706, ["y"] = -12903, ["z"] = 64, ["order"] = 20, ["angle"] = -1464795136},
  [83] = {["x"] = 15601, ["y"] = -13578, ["z"] = 64, ["order"] = 21, ["angle"] = -1818820608},
  [84] = {["x"] = 14885, ["y"] = -13993, ["z"] = 64, ["order"] = 21, ["angle"] = -2145452032},
  [85] = {["x"] = 14038, ["y"] = -14037, ["z"] = 64, ["order"] = 21, ["angle"] = 2110324736},
  [86] = {["x"] = 12929, ["y"] = -14037, ["z"] = 64, ["order"] = 22, ["angle"] = 2141650944},
  [87] = {["x"] = 10236, ["y"] = -14051, ["z"] = 64, ["order"] = 23, ["angle"] = 2132475904},
  [88] = {["x"] = 9766, ["y"] = -13965, ["z"] = 64, ["order"] = 23, ["angle"] = 2090139648},
  [89] = {["x"] = 8484, ["y"] = -13897, ["z"] = 64, ["order"] = 23, ["angle"] = -2105999360},
  [90] = {["x"] = 7639, ["y"] = -13965, ["z"] = 64, ["order"] = 23, ["angle"] = -2074804224},
  [91] = {["x"] = 6843, ["y"] = -14149, ["z"] = 64, ["order"] = 23, ["angle"] = 2146369536},
  [92] = {["x"] = 3988, ["y"] = -14138, ["z"] = 64, ["order"] = 24, ["angle"] = -2147418112},
  [93] = {["x"] = 3496, ["y"] = -14221, ["z"] = 64, ["order"] = 24, ["angle"] = 1976107008},
  [94] = {["x"] = 2582, ["y"] = -14252, ["z"] = 64, ["order"] = 24, ["angle"] = 1806237696},
  [95] = {["x"] = 1834, ["y"] = -14097, ["z"] = 64, ["order"] = 25, ["angle"] = 1626275840},
  [96] = {["x"] = 1131, ["y"] = -13487, ["z"] = 64, ["order"] = 25, ["angle"] = 929890304},
  [97] = {["x"] = 1010, ["y"] = -12278, ["z"] = 64, ["order"] = 26, ["angle"] = 1101856768},
  [98] = {["x"] = 982, ["y"] = -11162, ["z"] = 64, ["order"] = 27, ["angle"] = 1092288512},
  [99] = {["x"] = 964, ["y"] = -10389, ["z"] = 64, ["order"] = 27, ["angle"] = 1092288512},
  [100] = {["x"] = 942, ["y"] = -9547, ["z"] = 64, ["order"] = 27, ["angle"] = 1092288512},
  [101] = {["x"] = 934, ["y"] = -9195, ["z"] = 64, ["order"] = 27, ["angle"] = 1121517568}
}
-- Respawn definitions for SMK DONUT (MAP51)
krespawn.points[51] = {
  [0] = 31,
  [1] = {["x"] = 927, ["y"] = -3716, ["z"] = 0, ["order"] = 0, ["angle"] = 1041039360},
  [2] = {["x"] = 858, ["y"] = -2650, ["z"] = 0, ["order"] = 0, ["angle"] = 679804928},
  [3] = {["x"] = 1198, ["y"] = -1904, ["z"] = 0, ["order"] = 1, ["angle"] = 475725824},
  [4] = {["x"] = 2005, ["y"] = -1184, ["z"] = 0, ["order"] = 1, ["angle"] = 125108224},
  [5] = {["x"] = 3044, ["y"] = -880, ["z"] = 0, ["order"] = 2, ["angle"] = -28114944},
  [6] = {["x"] = 4369, ["y"] = -795, ["z"] = 0, ["order"] = 2, ["angle"] = 1769472},
  [7] = {["x"] = 5631, ["y"] = -601, ["z"] = 0, ["order"] = 2, ["angle"] = -188416000},
  [8] = {["x"] = 6729, ["y"] = -755, ["z"] = 0, ["order"] = 3, ["angle"] = -733151232},
  [9] = {["x"] = 7326, ["y"] = -1494, ["z"] = 0, ["order"] = 4, ["angle"] = -1246822400},
  [10] = {["x"] = 7247, ["y"] = -2285, ["z"] = 0, ["order"] = 4, ["angle"] = -1456406528},
  [11] = {["x"] = 6547, ["y"] = -3139, ["z"] = 0, ["order"] = 4, ["angle"] = -1291649024},
  [12] = {["x"] = 6202, ["y"] = -3831, ["z"] = 0, ["order"] = 5, ["angle"] = -721879040},
  [13] = {["x"] = 6589, ["y"] = -4435, ["z"] = 0, ["order"] = 5, ["angle"] = -691994624},
  [14] = {["x"] = 7194, ["y"] = -5120, ["z"] = 0, ["order"] = 5, ["angle"] = -1072496640},
  [15] = {["x"] = 7473, ["y"] = -5979, ["z"] = 0, ["order"] = 5, ["angle"] = -1640693760},
  [16] = {["x"] = 7065, ["y"] = -6719, ["z"] = 0, ["order"] = 6, ["angle"] = 2057764864},
  [17] = {["x"] = 6148, ["y"] = -6854, ["z"] = 0, ["order"] = 7, ["angle"] = 1358364672},
  [18] = {["x"] = 5526, ["y"] = -6330, ["z"] = 0, ["order"] = 7, ["angle"] = 1170407424},
  [19] = {["x"] = 5358, ["y"] = -5238, ["z"] = 0, ["order"] = 8, ["angle"] = 1310523392},
  [20] = {["x"] = 5158, ["y"] = -4204, ["z"] = 0, ["order"] = 9, ["angle"] = 1686044672},
  [21] = {["x"] = 4533, ["y"] = -3206, ["z"] = 0, ["order"] = 9, ["angle"] = -2135752704},
  [22] = {["x"] = 3572, ["y"] = -2741, ["z"] = 0, ["order"] = 10, ["angle"] = -1476460544},
  [23] = {["x"] = 3007, ["y"] = -3090, ["z"] = 0, ["order"] = 10, ["angle"] = -976551936},
  [24] = {["x"] = 2934, ["y"] = -3824, ["z"] = 0, ["order"] = 11, ["angle"] = -733806592},
  [25] = {["x"] = 3507, ["y"] = -4634, ["z"] = 0, ["order"] = 11, ["angle"] = -934871040},
  [26] = {["x"] = 3996, ["y"] = -5561, ["z"] = 0, ["order"] = 11, ["angle"] = -1379860480},
  [27] = {["x"] = 3828, ["y"] = -6608, ["z"] = 0, ["order"] = 12, ["angle"] = -2010316800},
  [28] = {["x"] = 2837, ["y"] = -7092, ["z"] = 0, ["order"] = 13, ["angle"] = 1695219712},
  [29] = {["x"] = 1374, ["y"] = -6831, ["z"] = 0, ["order"] = 13, ["angle"] = 1293352960},
  [30] = {["x"] = 970, ["y"] = -5877, ["z"] = 0, ["order"] = 13, ["angle"] = 1075249152},
  [31] = {["x"] = 964, ["y"] = -4802, ["z"] = 0, ["order"] = 14, ["angle"] = 1093074944}
}
-- Respawn definitions for SMK MARIO (MAP52)
krespawn.points[52] = {
  [0] = 52,
  [1] = {["x"] = 7489, ["y"] = -3208, ["z"] = 0, ["order"] = 0, ["angle"] = 1661796352},
  [2] = {["x"] = 7157, ["y"] = -2756, ["z"] = 0, ["order"] = 1, ["angle"] = 1886322688},
  [3] = {["x"] = 6700, ["y"] = -2495, ["z"] = 0, ["order"] = 1, ["angle"] = 2116747264},
  [4] = {["x"] = 5912, ["y"] = -2423, ["z"] = 0, ["order"] = 2, ["angle"] = 1693384704},
  [5] = {["x"] = 5344, ["y"] = -1926, ["z"] = 0, ["order"] = 2, ["angle"] = 1567948800},
  [6] = {["x"] = 5036, ["y"] = -1435, ["z"] = 0, ["order"] = 3, ["angle"] = 1774780416},
  [7] = {["x"] = 4550, ["y"] = -939, ["z"] = 0, ["order"] = 3, ["angle"] = 2000748544},
  [8] = {["x"] = 3992, ["y"] = -652, ["z"] = 0, ["order"] = 3, ["angle"] = 2119499776},
  [9] = {["x"] = 3244, ["y"] = -540, ["z"] = 0, ["order"] = 4, ["angle"] = -2009399296},
  [10] = {["x"] = 2424, ["y"] = -536, ["z"] = 0, ["order"] = 4, ["angle"] = -1916338176},
  [11] = {["x"] = 1490, ["y"] = -593, ["z"] = 0, ["order"] = 5, ["angle"] = -1621295104},
  [12] = {["x"] = 1060, ["y"] = -720, ["z"] = 0, ["order"] = 5, ["angle"] = -1368326144},
  [13] = {["x"] = 776, ["y"] = -1091, ["z"] = 0, ["order"] = 5, ["angle"] = -1057292288},
  [14] = {["x"] = 655, ["y"] = -1666, ["z"] = 0, ["order"] = 5, ["angle"] = -1057292288},
  [15] = {["x"] = 594, ["y"] = -2345, ["z"] = 0, ["order"] = 6, ["angle"] = -911015936},
  [16] = {["x"] = 569, ["y"] = -2921, ["z"] = 0, ["order"] = 6, ["angle"] = -860028928},
  [17] = {["x"] = 590, ["y"] = -3507, ["z"] = 0, ["order"] = 7, ["angle"] = -478740480},
  [18] = {["x"] = 849, ["y"] = -4007, ["z"] = 0, ["order"] = 7, ["angle"] = -112132096},
  [19] = {["x"] = 1289, ["y"] = -4239, ["z"] = 0, ["order"] = 7, ["angle"] = 168886272},
  [20] = {["x"] = 1881, ["y"] = -4195, ["z"] = 0, ["order"] = 8, ["angle"] = 218431488},
  [21] = {["x"] = 2414, ["y"] = -3974, ["z"] = 0, ["order"] = 8, ["angle"] = 204931072},
  [22] = {["x"] = 2856, ["y"] = -3746, ["z"] = 0, ["order"] = 8, ["angle"] = 19070976},
  [23] = {["x"] = 3518, ["y"] = -3441, ["z"] = 0, ["order"] = 8, ["angle"] = -164823040},
  [24] = {["x"] = 4050, ["y"] = -3368, ["z"] = 0, ["order"] = 8, ["angle"] = -474021888},
  [25] = {["x"] = 4644, ["y"] = -3584, ["z"] = 0, ["order"] = 9, ["angle"] = -801177600},
  [26] = {["x"] = 4961, ["y"] = -4010, ["z"] = 0, ["order"] = 9, ["angle"] = -1062666240},
  [27] = {["x"] = 5049, ["y"] = -4514, ["z"] = 0, ["order"] = 10, ["angle"] = -1132134400},
  [28] = {["x"] = 5084, ["y"] = -5114, ["z"] = 0, ["order"] = 10, ["angle"] = -1195573248},
  [29] = {["x"] = 5151, ["y"] = -5860, ["z"] = 0, ["order"] = 10, ["angle"] = -1403453440},
  [30] = {["x"] = 5069, ["y"] = -6488, ["z"] = 0, ["order"] = 11, ["angle"] = -1786970112},
  [31] = {["x"] = 4715, ["y"] = -6929, ["z"] = 0, ["order"] = 11, ["angle"] = -2014380032},
  [32] = {["x"] = 4233, ["y"] = -7140, ["z"] = 0, ["order"] = 12, ["angle"] = -2140078080},
  [33] = {["x"] = 3651, ["y"] = -7209, ["z"] = 0, ["order"] = 12, ["angle"] = 2144796672},
  [34] = {["x"] = 3043, ["y"] = -7331, ["z"] = 0, ["order"] = 12, ["angle"] = 2102591488},
  [35] = {["x"] = 2390, ["y"] = -7480, ["z"] = 0, ["order"] = 12, ["angle"] = 1929838592},
  [36] = {["x"] = 1770, ["y"] = -7520, ["z"] = 0, ["order"] = 12, ["angle"] = 1570308096},
  [37] = {["x"] = 1202, ["y"] = -7277, ["z"] = 0, ["order"] = 13, ["angle"] = 1058340864},
  [38] = {["x"] = 1002, ["y"] = -6908, ["z"] = 0, ["order"] = 13, ["angle"] = 790298624},
  [39] = {["x"] = 1041, ["y"] = -6505, ["z"] = 0, ["order"] = 14, ["angle"] = 494206976},
  [40] = {["x"] = 1299, ["y"] = -6047, ["z"] = 0, ["order"] = 14, ["angle"] = 171507712},
  [41] = {["x"] = 1700, ["y"] = -5735, ["z"] = 0, ["order"] = 14, ["angle"] = -43188224},
  [42] = {["x"] = 2266, ["y"] = -5615, ["z"] = 0, ["order"] = 15, ["angle"] = -42401792},
  [43] = {["x"] = 2912, ["y"] = -5613, ["z"] = 0, ["order"] = 15, ["angle"] = -42401792},
  [44] = {["x"] = 3883, ["y"] = -5619, ["z"] = 0, ["order"] = 15, ["angle"] = 29818880},
  [45] = {["x"] = 5454, ["y"] = -5720, ["z"] = 0, ["order"] = 16, ["angle"] = 48168960},
  [46] = {["x"] = 5839, ["y"] = -5690, ["z"] = 0, ["order"] = 16, ["angle"] = 48168960},
  [47] = {["x"] = 6265, ["y"] = -5685, ["z"] = 0, ["order"] = 17, ["angle"] = 220921856},
  [48] = {["x"] = 6841, ["y"] = -5583, ["z"] = 0, ["order"] = 17, ["angle"] = 541523968},
  [49] = {["x"] = 7171, ["y"] = -5258, ["z"] = 0, ["order"] = 17, ["angle"] = 969342976},
  [50] = {["x"] = 7301, ["y"] = -4752, ["z"] = 0, ["order"] = 17, ["angle"] = 1039859712},
  [51] = {["x"] = 7360, ["y"] = -4104, ["z"] = 0, ["order"] = 18, ["angle"] = 1082589184},
  [52] = {["x"] = 7390, ["y"] = -3480, ["z"] = 0, ["order"] = 18, ["angle"] = 1495990272}
}
-- Respawn definitions for SMK GHOST (MAP53)
krespawn.points[53] = {
  [0] = 41,
  [1] = {["x"] = 961, ["y"] = -3989, ["z"] = 64, ["order"] = 0, ["angle"] = 1049690112},
  [2] = {["x"] = 936, ["y"] = -3566, ["z"] = 64, ["order"] = 0, ["angle"] = 1089798144},
  [3] = {["x"] = 863, ["y"] = -2939, ["z"] = 64, ["order"] = 0, ["angle"] = 898957312},
  [4] = {["x"] = 837, ["y"] = -2105, ["z"] = 64, ["order"] = 0, ["angle"] = 734330880},
  [5] = {["x"] = 978, ["y"] = -1406, ["z"] = 64, ["order"] = 0, ["angle"] = 287506432},
  [6] = {["x"] = 1526, ["y"] = -958, ["z"] = 64, ["order"] = 0, ["angle"] = -91029504},
  [7] = {["x"] = 2222, ["y"] = -880, ["z"] = 64, ["order"] = 1, ["angle"] = -357498880},
  [8] = {["x"] = 2682, ["y"] = -1089, ["z"] = 64, ["order"] = 1, ["angle"] = -609288192},
  [9] = {["x"] = 2979, ["y"] = -1544, ["z"] = 64, ["order"] = 1, ["angle"] = -741933056},
  [10] = {["x"] = 3114, ["y"] = -2094, ["z"] = 64, ["order"] = 2, ["angle"] = -577437696},
  [11] = {["x"] = 3401, ["y"] = -2693, ["z"] = 64, ["order"] = 2, ["angle"] = -214761472},
  [12] = {["x"] = 3952, ["y"] = -3096, ["z"] = 64, ["order"] = 3, ["angle"] = 73728000},
  [13] = {["x"] = 4648, ["y"] = -3221, ["z"] = 64, ["order"] = 3, ["angle"] = 171507712},
  [14] = {["x"] = 5329, ["y"] = -3093, ["z"] = 64, ["order"] = 3, ["angle"] = 377290752},
  [15] = {["x"] = 5687, ["y"] = -2854, ["z"] = 64, ["order"] = 3, ["angle"] = 423821312},
  [16] = {["x"] = 6003, ["y"] = -2468, ["z"] = 64, ["order"] = 4, ["angle"] = 134021120},
  [17] = {["x"] = 6516, ["y"] = -2013, ["z"] = 64, ["order"] = 4, ["angle"] = -291438592},
  [18] = {["x"] = 7222, ["y"] = -1967, ["z"] = 64, ["order"] = 4, ["angle"] = -616103936},
  [19] = {["x"] = 7697, ["y"] = -2278, ["z"] = 64, ["order"] = 4, ["angle"] = -964100096},
  [20] = {["x"] = 7946, ["y"] = -2863, ["z"] = 64, ["order"] = 4, ["angle"] = -1052442624},
  [21] = {["x"] = 8099, ["y"] = -3494, ["z"] = 64, ["order"] = 5, ["angle"] = -1201078272},
  [22] = {["x"] = 8175, ["y"] = -4179, ["z"] = 64, ["order"] = 5, ["angle"] = -1484587008},
  [23] = {["x"] = 7961, ["y"] = -4805, ["z"] = 64, ["order"] = 5, ["angle"] = -1759182848},
  [24] = {["x"] = 7523, ["y"] = -5199, ["z"] = 64, ["order"] = 5, ["angle"] = -1987903488},
  [25] = {["x"] = 7003, ["y"] = -5291, ["z"] = 64, ["order"] = 6, ["angle"] = -2011234304},
  [26] = {["x"] = 6520, ["y"] = -5263, ["z"] = 64, ["order"] = 6, ["angle"] = -1867448320},
  [27] = {["x"] = 5934, ["y"] = -5317, ["z"] = 64, ["order"] = 6, ["angle"] = -1515651072},
  [28] = {["x"] = 5536, ["y"] = -5630, ["z"] = 64, ["order"] = 6, ["angle"] = -1272119296},
  [29] = {["x"] = 5459, ["y"] = -5946, ["z"] = 64, ["order"] = 6, ["angle"] = -1292042240},
  [30] = {["x"] = 5432, ["y"] = -6445, ["z"] = 64, ["order"] = 7, ["angle"] = -1417216000},
  [31] = {["x"] = 5274, ["y"] = -7192, ["z"] = 64, ["order"] = 7, ["angle"] = -1734410240},
  [32] = {["x"] = 4806, ["y"] = -7654, ["z"] = 64, ["order"] = 7, ["angle"] = -2072838144},
  [33] = {["x"] = 4200, ["y"] = -7867, ["z"] = 64, ["order"] = 8, ["angle"] = 2015428608},
  [34] = {["x"] = 3643, ["y"] = -7777, ["z"] = 64, ["order"] = 8, ["angle"] = 1815412736},
  [35] = {["x"] = 3100, ["y"] = -7472, ["z"] = 64, ["order"] = 8, ["angle"] = 1846345728},
  [36] = {["x"] = 2280, ["y"] = -7382, ["z"] = 64, ["order"] = 8, ["angle"] = 1695875072},
  [37] = {["x"] = 1672, ["y"] = -7169, ["z"] = 64, ["order"] = 8, ["angle"] = 1461256192},
  [38] = {["x"] = 1314, ["y"] = -6626, ["z"] = 64, ["order"] = 8, ["angle"] = 1205272576},
  [39] = {["x"] = 1021, ["y"] = -6112, ["z"] = 64, ["order"] = 9, ["angle"] = 1034878976},
  [40] = {["x"] = 993, ["y"] = -5547, ["z"] = 64, ["order"] = 9, ["angle"] = 1030815744},
  [41] = {["x"] = 1006, ["y"] = -4758, ["z"] = 64, ["order"] = 9, ["angle"] = 1139998720}
}
-- Respawn definitions for SMK BOWSER (MAP54)
krespawn.points[54] = {
  [0] = 82,
  [1] = {["x"] = 7605, ["y"] = -2300, ["z"] = 0, ["order"] = 0, ["angle"] = 1147338752},
  [2] = {["x"] = 7612, ["y"] = -1464, ["z"] = 0, ["order"] = 0, ["angle"] = 1329790976},
  [3] = {["x"] = 7550, ["y"] = -684, ["z"] = 0, ["order"] = 1, ["angle"] = 2126446592},
  [4] = {["x"] = 7049, ["y"] = -574, ["z"] = 0, ["order"] = 1, ["angle"] = 2126446592},
  [5] = {["x"] = 6334, ["y"] = -554, ["z"] = 0, ["order"] = 1, ["angle"] = 2130640896},
  [6] = {["x"] = 5386, ["y"] = -551, ["z"] = 0, ["order"] = 1, ["angle"] = -2112290816},
  [7] = {["x"] = 4604, ["y"] = -544, ["z"] = 0, ["order"] = 2, ["angle"] = 2142175232},
  [8] = {["x"] = 4032, ["y"] = -535, ["z"] = 0, ["order"] = 2, ["angle"] = 2142175232},
  [9] = {["x"] = 3584, ["y"] = -529, ["z"] = 0, ["order"] = 2, ["angle"] = 2142175232},
  [10] = {["x"] = 2843, ["y"] = -566, ["z"] = 0, ["order"] = 3, ["angle"] = -2108227584},
  [11] = {["x"] = 2199, ["y"] = -588, ["z"] = 0, ["order"] = 3, ["angle"] = -1999175680},
  [12] = {["x"] = 1648, ["y"] = -586, ["z"] = 0, ["order"] = 4, ["angle"] = -1437007872},
  [13] = {["x"] = 1235, ["y"] = -1213, ["z"] = 0, ["order"] = 4, ["angle"] = -1042219008},
  [14] = {["x"] = 1308, ["y"] = -2187, ["z"] = 0, ["order"] = 4, ["angle"] = -1130561536},
  [15] = {["x"] = 1165, ["y"] = -3173, ["z"] = 0, ["order"] = 6, ["angle"] = -1189675008},
  [16] = {["x"] = 982, ["y"] = -4237, ["z"] = 0, ["order"] = 7, ["angle"] = -1189675008},
  [17] = {["x"] = 848, ["y"] = -5261, ["z"] = 0, ["order"] = 7, ["angle"] = -1159921664},
  [18] = {["x"] = 809, ["y"] = -5933, ["z"] = 0, ["order"] = 8, ["angle"] = -998703104},
  [19] = {["x"] = 865, ["y"] = -6724, ["z"] = 0, ["order"] = 8, ["angle"] = -832503808},
  [20] = {["x"] = 1003, ["y"] = -7372, ["z"] = 0, ["order"] = 8, ["angle"] = -73465856},
  [21] = {["x"] = 1420, ["y"] = -7519, ["z"] = 0, ["order"] = 8, ["angle"] = -102957056},
  [22] = {["x"] = 2103, ["y"] = -7607, ["z"] = 0, ["order"] = 9, ["angle"] = 20250624},
  [23] = {["x"] = 2504, ["y"] = -7655, ["z"] = 0, ["order"] = 9, ["angle"] = 193396736},
  [24] = {["x"] = 3215, ["y"] = -7649, ["z"] = 0, ["order"] = 9, ["angle"] = 487784448},
  [25] = {["x"] = 3603, ["y"] = -7431, ["z"] = 0, ["order"] = 10, ["angle"] = 1100546048},
  [26] = {["x"] = 3544, ["y"] = -7002, ["z"] = 0, ["order"] = 10, ["angle"] = 1155727360},
  [27] = {["x"] = 3514, ["y"] = -6164, ["z"] = 0, ["order"] = 10, ["angle"] = 1077346304},
  [28] = {["x"] = 3502, ["y"] = -4729, ["z"] = 0, ["order"] = 10, ["angle"] = 1077346304},
  [29] = {["x"] = 3494, ["y"] = -3570, ["z"] = 0, ["order"] = 10, ["angle"] = 1077346304},
  [30] = {["x"] = 3499, ["y"] = -2548, ["z"] = 0, ["order"] = 11, ["angle"] = 244383744},
  [31] = {["x"] = 4264, ["y"] = -2116, ["z"] = 0, ["order"] = 11, ["angle"] = -194183168},
  [32] = {["x"] = 4837, ["y"] = -2358, ["z"] = 0, ["order"] = 11, ["angle"] = -836304896},
  [33] = {["x"] = 5082, ["y"] = -3167, ["z"] = 0, ["order"] = 12, ["angle"] = -1085210624},
  [34] = {["x"] = 5089, ["y"] = -4551, ["z"] = 0, ["order"] = 12, ["angle"] = -1081278464},
  [35] = {["x"] = 5065, ["y"] = -5403, ["z"] = 0, ["order"] = 12, ["angle"] = -1081278464},
  [36] = {["x"] = 5054, ["y"] = -6173, ["z"] = 0, ["order"] = 12, ["angle"] = -1081278464},
  [37] = {["x"] = 5013, ["y"] = -6890, ["z"] = 0, ["order"] = 12, ["angle"] = -851640320},
  [38] = {["x"] = 5205, ["y"] = -7496, ["z"] = 0, ["order"] = 13, ["angle"] = -198901760},
  [39] = {["x"] = 5789, ["y"] = -7728, ["z"] = 0, ["order"] = 13, ["angle"] = 115015680},
  [40] = {["x"] = 6516, ["y"] = -7710, ["z"] = 0, ["order"] = 13, ["angle"] = 567869440},
  [41] = {["x"] = 6948, ["y"] = -7305, ["z"] = 0, ["order"] = 14, ["angle"] = 1018363904},
  [42] = {["x"] = 7090, ["y"] = -6608, ["z"] = 0, ["order"] = 14, ["angle"] = 965672960},
  [43] = {["x"] = 7214, ["y"] = -5751, ["z"] = 0, ["order"] = 14, ["angle"] = 923598848},
  [44] = {["x"] = 7370, ["y"] = -4847, ["z"] = 0, ["order"] = 16, ["angle"] = 1014956032},
  [45] = {["x"] = 7458, ["y"] = -3976, ["z"] = 0, ["order"] = 16, ["angle"] = 1014956032},
  [46] = {["x"] = 7524, ["y"] = -3239, ["z"] = 0, ["order"] = 16, ["angle"] = 1087700992},
  [47] = {["x"] = 7548, ["y"] = -2733, ["z"] = 0, ["order"] = 16, ["angle"] = 1087700992},
  [48] = {["x"] = 5877, ["y"] = -553, ["z"] = 0, ["order"] = 1, ["angle"] = -2100101120},
  [49] = {["x"] = 1548, ["y"] = -2190, ["z"] = 0, ["order"] = 4, ["angle"] = -983760896},
  [50] = {["x"] = 1723, ["y"] = -3221, ["z"] = 0, ["order"] = 6, ["angle"] = -895025152},
  [51] = {["x"] = 2024, ["y"] = -3953, ["z"] = 0, ["order"] = 7, ["angle"] = -1339752448},
  [52] = {["x"] = 1678, ["y"] = -4745, ["z"] = 0, ["order"] = 7, ["angle"] = -925564928},
  [53] = {["x"] = 1524, ["y"] = -5631, ["z"] = 0, ["order"] = 7, ["angle"] = -1122828288},
  [54] = {["x"] = 1489, ["y"] = -6762, ["z"] = 0, ["order"] = 8, ["angle"] = -695664640},
  [55] = {["x"] = 1635, ["y"] = -7209, ["z"] = 0, ["order"] = 8, ["angle"] = -208076800},
  [56] = {["x"] = 5364, ["y"] = -2629, ["z"] = 0, ["order"] = 12, ["angle"] = -1204617216},
  [57] = {["x"] = 5390, ["y"] = -3167, ["z"] = 0, ["order"] = 12, ["angle"] = -1079574528},
  [58] = {["x"] = 5374, ["y"] = -4040, ["z"] = 0, ["order"] = 12, ["angle"] = -1055195136},
  [59] = {["x"] = 5358, ["y"] = -4931, ["z"] = 0, ["order"] = 12, ["angle"] = -1123745792},
  [60] = {["x"] = 5402, ["y"] = -5766, ["z"] = 0, ["order"] = 12, ["angle"] = -1112604672},
  [61] = {["x"] = 5396, ["y"] = -6413, ["z"] = 0, ["order"] = 12, ["angle"] = -1048248320},
  [62] = {["x"] = 5410, ["y"] = -7100, ["z"] = 0, ["order"] = 13, ["angle"] = -314245120},
  [63] = {["x"] = 5915, ["y"] = -7377, ["z"] = 0, ["order"] = 13, ["angle"] = 252248064},
  [64] = {["x"] = 6707, ["y"] = -7408, ["z"] = 0, ["order"] = 13, ["angle"] = 993460224},
  [65] = {["x"] = 7380, ["y"] = -4523, ["z"] = 0, ["order"] = 16, ["angle"] = 1038155776},
  [66] = {["x"] = 1596, ["y"] = -2458, ["z"] = 0, ["order"] = 5, ["angle"] = -979566592},
  [67] = {["x"] = 1190, ["y"] = -2462, ["z"] = 0, ["order"] = 5, ["angle"] = -1055719424},
  [68] = {["x"] = 2030, ["y"] = -3822, ["z"] = 0, ["order"] = 7, ["angle"] = -1218904064},
  [69] = {["x"] = 2077, ["y"] = -5537, ["z"] = 0, ["order"] = 8, ["angle"] = -980877312},
  [70] = {["x"] = 2219, ["y"] = -6330, ["z"] = 0, ["order"] = 8, ["angle"] = -911802368},
  [71] = {["x"] = 2270, ["y"] = -6670, ["z"] = 0, ["order"] = 8, ["angle"] = -938409984},
  [72] = {["x"] = 2222, ["y"] = -7161, ["z"] = 0, ["order"] = 9, ["angle"] = -445054976},
  [73] = {["x"] = 5118, ["y"] = -1875, ["z"] = 0, ["order"] = 12, ["angle"] = -330366976},
  [74] = {["x"] = 5713, ["y"] = -2403, ["z"] = 0, ["order"] = 12, ["angle"] = -1019019264},
  [75] = {["x"] = 5701, ["y"] = -3245, ["z"] = 0, ["order"] = 12, ["angle"] = -1060831232},
  [76] = {["x"] = 5699, ["y"] = -4498, ["z"] = 0, ["order"] = 12, ["angle"] = -1086783488},
  [77] = {["x"] = 5705, ["y"] = -5199, ["z"] = 0, ["order"] = 12, ["angle"] = -1022296064},
  [78] = {["x"] = 5670, ["y"] = -6285, ["z"] = 0, ["order"] = 12, ["angle"] = -922025984},
  [79] = {["x"] = 5850, ["y"] = -6883, ["z"] = 0, ["order"] = 13, ["angle"] = -298254336},
  [80] = {["x"] = 6458, ["y"] = -7277, ["z"] = 0, ["order"] = 13, ["angle"] = 543883264},
  [81] = {["x"] = 7133, ["y"] = -6111, ["z"] = 0, ["order"] = 14, ["angle"] = 946012160},
  [82] = {["x"] = 7507, ["y"] = -5357, ["z"] = 0, ["order"] = 16, ["angle"] = 1169227776}
}
-- Respawn definitions for SMK VANILLA (MAP55)
krespawn.points[55] = {
  [0] = 28,
  [1] = {["x"] = 7262, ["y"] = -3823, ["z"] = 0, ["order"] = 0, ["angle"] = 1355481088},
  [2] = {["x"] = 7103, ["y"] = -2823, ["z"] = 0, ["order"] = 0, ["angle"] = 1502937088},
  [3] = {["x"] = 6738, ["y"] = -1911, ["z"] = 0, ["order"] = 1, ["angle"] = 1849229312},
  [4] = {["x"] = 6150, ["y"] = -1312, ["z"] = 0, ["order"] = 1, ["angle"] = -2098266112},
  [5] = {["x"] = 5531, ["y"] = -1043, ["z"] = 0, ["order"] = 1, ["angle"] = -1747779584},
  [6] = {["x"] = 4556, ["y"] = -1436, ["z"] = 0, ["order"] = 1, ["angle"] = -2087780352},
  [7] = {["x"] = 3711, ["y"] = -1381, ["z"] = 0, ["order"] = 2, ["angle"] = -1753153536},
  [8] = {["x"] = 3134, ["y"] = -1499, ["z"] = 0, ["order"] = 2, ["angle"] = -1727856640},
  [9] = {["x"] = 2413, ["y"] = -1497, ["z"] = 0, ["order"] = 2, ["angle"] = -1548812288},
  [10] = {["x"] = 1831, ["y"] = -2062, ["z"] = 0, ["order"] = 2, ["angle"] = -1171456000},
  [11] = {["x"] = 1650, ["y"] = -2648, ["z"] = 0, ["order"] = 3, ["angle"] = -1226637312},
  [12] = {["x"] = 1244, ["y"] = -3253, ["z"] = 0, ["order"] = 3, ["angle"] = -1130692608},
  [13] = {["x"] = 1021, ["y"] = -3771, ["z"] = 0, ["order"] = 3, ["angle"] = -805109760},
  [14] = {["x"] = 1452, ["y"] = -4538, ["z"] = 0, ["order"] = 3, ["angle"] = -1082458112},
  [15] = {["x"] = 1221, ["y"] = -5444, ["z"] = 0, ["order"] = 3, ["angle"] = -1360330752},
  [16] = {["x"] = 636, ["y"] = -6151, ["z"] = 0, ["order"] = 4, ["angle"] = -817299456},
  [17] = {["x"] = 854, ["y"] = -7203, ["z"] = 0, ["order"] = 4, ["angle"] = 458752},
  [18] = {["x"] = 1731, ["y"] = -7343, ["z"] = 0, ["order"] = 4, ["angle"] = 359989248},
  [19] = {["x"] = 2568, ["y"] = -6955, ["z"] = 0, ["order"] = 5, ["angle"] = 421593088},
  [20] = {["x"] = 3560, ["y"] = -7041, ["z"] = 0, ["order"] = 5, ["angle"] = 345440256},
  [21] = {["x"] = 3574, ["y"] = -5883, ["z"] = 0, ["order"] = 5, ["angle"] = -103219200},
  [22] = {["x"] = 4419, ["y"] = -5762, ["z"] = 0, ["order"] = 5, ["angle"] = -687669248},
  [23] = {["x"] = 4678, ["y"] = -6367, ["z"] = 0, ["order"] = 5, ["angle"] = -713490432},
  [24] = {["x"] = 5307, ["y"] = -7396, ["z"] = 0, ["order"] = 5, ["angle"] = 816381952},
  [25] = {["x"] = 5902, ["y"] = -6323, ["z"] = 0, ["order"] = 5, ["angle"] = 833159168},
  [26] = {["x"] = 6572, ["y"] = -5292, ["z"] = 0, ["order"] = 6, ["angle"] = 807075840},
  [27] = {["x"] = 6996, ["y"] = -4874, ["z"] = 0, ["order"] = 6, ["angle"] = 1105264640},
  [28] = {["x"] = 7129, ["y"] = -4269, ["z"] = 0, ["order"] = 6, ["angle"] = 1193476096}
}
-- Respawn definitions for CRYSTAL ABYSS (MAPH0)
krespawn.points[352] = {
  [0] = 200,
  [1] = {["x"] = 49, ["y"] = -1538, ["z"] = 0, ["order"] = 0, ["angle"] = -7798784},
  [2] = {["x"] = 1002, ["y"] = -1516, ["z"] = 0, ["order"] = 0, ["angle"] = -11075584},
  [3] = {["x"] = 1851, ["y"] = -1432, ["z"] = 0, ["order"] = 0, ["angle"] = -94961664},
  [4] = {["x"] = 2700, ["y"] = -1402, ["z"] = 0, ["order"] = 0, ["angle"] = -508100608},
  [5] = {["x"] = 3399, ["y"] = -1997, ["z"] = 0, ["order"] = 1, ["angle"] = -1127153664},
  [6] = {["x"] = 3206, ["y"] = -2957, ["z"] = 0, ["order"] = 1, ["angle"] = -1392181248},
  [7] = {["x"] = 2674, ["y"] = -3795, ["z"] = 0, ["order"] = 1, ["angle"] = -1207894016},
  [8] = {["x"] = 2172, ["y"] = -4677, ["z"] = 0, ["order"] = 1, ["angle"] = -723845120},
  [9] = {["x"] = 2343, ["y"] = -5352, ["z"] = 0, ["order"] = 1, ["angle"] = -25362432},
  [10] = {["x"] = 3021, ["y"] = -5428, ["z"] = 0, ["order"] = 2, ["angle"] = 537591808},
  [11] = {["x"] = 3571, ["y"] = -5050, ["z"] = 89, ["order"] = 2, ["angle"] = 452657152},
  [12] = {["x"] = 4248, ["y"] = -4449, ["z"] = 237, ["order"] = 2, ["angle"] = 119472128},
  [13] = {["x"] = 4964, ["y"] = -4253, ["z"] = 276, ["order"] = 2, ["angle"] = -124715008},
  [14] = {["x"] = 5819, ["y"] = -4584, ["z"] = 300, ["order"] = 2, ["angle"] = 75038720},
  [15] = {["x"] = 6684, ["y"] = -4762, ["z"] = 300, ["order"] = 2, ["angle"] = 564199424},
  [16] = {["x"] = 7530, ["y"] = -4461, ["z"] = 300, ["order"] = 2, ["angle"] = 1196883968},
  [17] = {["x"] = 7409, ["y"] = -3777, ["z"] = 300, ["order"] = 2, ["angle"] = 1599930368},
  [18] = {["x"] = 6727, ["y"] = -3178, ["z"] = 357, ["order"] = 3, ["angle"] = 1547108352},
  [19] = {["x"] = 5987, ["y"] = -2620, ["z"] = 500, ["order"] = 3, ["angle"] = 1162674176},
  [20] = {["x"] = 5748, ["y"] = -1858, ["z"] = 500, ["order"] = 3, ["angle"] = 594083840},
  [21] = {["x"] = 6153, ["y"] = -1130, ["z"] = 500, ["order"] = 4, ["angle"] = -20643840},
  [22] = {["x"] = 6846, ["y"] = -1234, ["z"] = 500, ["order"] = 4, ["angle"] = -318177280},
  [23] = {["x"] = 7645, ["y"] = -1839, ["z"] = 500, ["order"] = 4, ["angle"] = 21954560},
  [24] = {["x"] = 8324, ["y"] = -1900, ["z"] = 500, ["order"] = 4, ["angle"] = 156958720},
  [25] = {["x"] = 9476, ["y"] = -1687, ["z"] = 500, ["order"] = 4, ["angle"] = 760938496},
  [26] = {["x"] = 10099, ["y"] = -1139, ["z"] = 600, ["order"] = 5, ["angle"] = 1255866368},
  [27] = {["x"] = 9999, ["y"] = -500, ["z"] = 700, ["order"] = 5, ["angle"] = 1667432448},
  [28] = {["x"] = 9503, ["y"] = 113, ["z"] = 800, ["order"] = 5, ["angle"] = 1600192512},
  [29] = {["x"] = 8911, ["y"] = 1227, ["z"] = 800, ["order"] = 5, ["angle"] = 1292566528},
  [30] = {["x"] = 8835, ["y"] = 986, ["z"] = 800, ["order"] = 5, ["angle"] = 1916731392},
  [31] = {["x"] = 8880, ["y"] = 2079, ["z"] = 800, ["order"] = 5, ["angle"] = 1347747840},
  [32] = {["x"] = 8758, ["y"] = 2875, ["z"] = 800, ["order"] = 6, ["angle"] = 1829568512},
  [33] = {["x"] = 8118, ["y"] = 3340, ["z"] = 800, ["order"] = 6, ["angle"] = 2076770304},
  [34] = {["x"] = 7384, ["y"] = 3449, ["z"] = 800, ["order"] = 6, ["angle"] = 1721696256},
  [35] = {["x"] = 6996, ["y"] = 3786, ["z"] = 800, ["order"] = 6, ["angle"] = 1757872128},
  [36] = {["x"] = 6564, ["y"] = 4464, ["z"] = 800, ["order"] = 6, ["angle"] = 2107179008},
  [37] = {["x"] = 5852, ["y"] = 4596, ["z"] = 800, ["order"] = 7, ["angle"] = -1322319872},
  [38] = {["x"] = 5609, ["y"] = 3963, ["z"] = 800, ["order"] = 7, ["angle"] = -1135804416},
  [39] = {["x"] = 5582, ["y"] = 2973, ["z"] = 886, ["order"] = 7, ["angle"] = -1523122176},
  [40] = {["x"] = 4911, ["y"] = 1872, ["z"] = 1010, ["order"] = 7, ["angle"] = -2019491840},
  [41] = {["x"] = 3848, ["y"] = 1703, ["z"] = 1100, ["order"] = 7, ["angle"] = 2078998528},
  [42] = {["x"] = 2690, ["y"] = 1740, ["z"] = 1100, ["order"] = 7, ["angle"] = 2121465856},
  [43] = {["x"] = 1594, ["y"] = 1781, ["z"] = 1100, ["order"] = 7, ["angle"] = 2121465856},
  [44] = {["x"] = 506, ["y"] = 1821, ["z"] = 700, ["order"] = 7, ["angle"] = -2138243072},
  [45] = {["x"] = -322, ["y"] = 1754, ["z"] = 700, ["order"] = 9, ["angle"] = 2145714176},
  [46] = {["x"] = -1152, ["y"] = 1652, ["z"] = 700, ["order"] = 9, ["angle"] = 1935474688},
  [47] = {["x"] = -1789, ["y"] = 1780, ["z"] = 700, ["order"] = 9, ["angle"] = 1746468864},
  [48] = {["x"] = -2563, ["y"] = 2260, ["z"] = 700, ["order"] = 9, ["angle"] = 1375666176},
  [49] = {["x"] = -3084, ["y"] = 3060, ["z"] = 700, ["order"] = 9, ["angle"] = 526057472},
  [50] = {["x"] = -2600, ["y"] = 3493, ["z"] = 710, ["order"] = 10, ["angle"] = 80674816},
  [51] = {["x"] = -1771, ["y"] = 3466, ["z"] = 870, ["order"] = 10, ["angle"] = 83034112},
  [52] = {["x"] = -984, ["y"] = 3411, ["z"] = 900, ["order"] = 10, ["angle"] = 381353984},
  [53] = {["x"] = -178, ["y"] = 3672, ["z"] = 900, ["order"] = 10, ["angle"] = 1157824512},
  [54] = {["x"] = -162, ["y"] = 4354, ["z"] = 900, ["order"] = 11, ["angle"] = 1397424128},
  [55] = {["x"] = -601, ["y"] = 5146, ["z"] = 900, ["order"] = 11, ["angle"] = 1250885632},
  [56] = {["x"] = -938, ["y"] = 5912, ["z"] = 900, ["order"] = 11, ["angle"] = 1285095424},
  [57] = {["x"] = -1179, ["y"] = 6715, ["z"] = 900, ["order"] = 11, ["angle"] = 1136066560},
  [58] = {["x"] = -1094, ["y"] = 7485, ["z"] = 900, ["order"] = 11, ["angle"] = 1317470208},
  [59] = {["x"] = -1035, ["y"] = 8267, ["z"] = 900, ["order"] = 12, ["angle"] = 1806237696},
  [60] = {["x"] = -1433, ["y"] = 8848, ["z"] = 900, ["order"] = 12, ["angle"] = -1987510272},
  [61] = {["x"] = -2347, ["y"] = 8960, ["z"] = 900, ["order"] = 13, ["angle"] = -1415512064},
  [62] = {["x"] = -2663, ["y"] = 8437, ["z"] = 889, ["order"] = 13, ["angle"] = -1155858432},
  [63] = {["x"] = -2590, ["y"] = 7426, ["z"] = 800, ["order"] = 13, ["angle"] = -1457848320},
  [64] = {["x"] = -3135, ["y"] = 6330, ["z"] = 800, ["order"] = 13, ["angle"] = -1737031680},
  [65] = {["x"] = -4044, ["y"] = 5867, ["z"] = 800, ["order"] = 13, ["angle"] = -1775566848},
  [66] = {["x"] = -4954, ["y"] = 5551, ["z"] = 810, ["order"] = 13, ["angle"] = -1753677824},
  [67] = {["x"] = -5820, ["y"] = 5240, ["z"] = 810, ["order"] = 13, ["angle"] = -1150484480},
  [68] = {["x"] = -5959, ["y"] = 4452, ["z"] = 810, ["order"] = 14, ["angle"] = -777977856},
  [69] = {["x"] = -5396, ["y"] = 3684, ["z"] = 810, ["order"] = 14, ["angle"] = -1025966080},
  [70] = {["x"] = -5234, ["y"] = 2681, ["z"] = 810, ["order"] = 14, ["angle"] = -1762328576},
  [71] = {["x"] = -6239, ["y"] = 2357, ["z"] = 810, ["order"] = 15, ["angle"] = -2123169792},
  [72] = {["x"] = -7211, ["y"] = 2394, ["z"] = 810, ["order"] = 15, ["angle"] = -2133000192},
  [73] = {["x"] = -8327, ["y"] = 2565, ["z"] = 810, ["order"] = 15, ["angle"] = -1876885504},
  [74] = {["x"] = -8964, ["y"] = 2334, ["z"] = 810, ["order"] = 16, ["angle"] = -1398734848},
  [75] = {["x"] = -9312, ["y"] = -1139, ["z"] = 1800, ["order"] = 16, ["angle"] = -1221787648},
  [76] = {["x"] = -9261, ["y"] = -2086, ["z"] = 1800, ["order"] = 16, ["angle"] = -1457061888},
  [77] = {["x"] = -9497, ["y"] = -2977, ["z"] = 1800, ["order"] = 16, ["angle"] = 2063925248},
  [78] = {["x"] = -10336, ["y"] = -2781, ["z"] = 1800, ["order"] = 17, ["angle"] = 1808990208},
  [79] = {["x"] = -11096, ["y"] = -2258, ["z"] = 1800, ["order"] = 17, ["angle"] = 2063532032},
  [80] = {["x"] = -12020, ["y"] = -2105, ["z"] = 1800, ["order"] = 17, ["angle"] = -1658388480},
  [81] = {["x"] = -12927, ["y"] = -2517, ["z"] = 1800, ["order"] = 17, ["angle"] = -1313931264},
  [82] = {["x"] = -13132, ["y"] = -3215, ["z"] = 1800, ["order"] = 18, ["angle"] = -689897472},
  [83] = {["x"] = -12739, ["y"] = -4367, ["z"] = 1720, ["order"] = 18, ["angle"] = -453312512},
  [84] = {["x"] = -12266, ["y"] = -4793, ["z"] = 1630, ["order"] = 18, ["angle"] = -366542848},
  [85] = {["x"] = -11340, ["y"] = -4979, ["z"] = 1600, ["order"] = 18, ["angle"] = -475594752},
  [86] = {["x"] = -10714, ["y"] = -5513, ["z"] = 1600, ["order"] = 19, ["angle"] = -1044185088},
  [87] = {["x"] = -10479, ["y"] = -6569, ["z"] = 1600, ["order"] = 19, ["angle"] = -1607663616},
  [88] = {["x"] = -11227, ["y"] = -7323, ["z"] = 1584, ["order"] = 20, ["angle"] = -2125135872},
  [89] = {["x"] = -12737, ["y"] = -7537, ["z"] = 1790, ["order"] = 20, ["angle"] = 1773076480},
  [90] = {["x"] = -13446, ["y"] = -7228, ["z"] = 1800, ["order"] = 20, ["angle"] = 1191247872},
  [91] = {["x"] = -13693, ["y"] = -6541, ["z"] = 1844, ["order"] = 20, ["angle"] = 545849344},
  [92] = {["x"] = -13371, ["y"] = -5840, ["z"] = 1912, ["order"] = 21, ["angle"] = -162201600},
  [93] = {["x"] = -12564, ["y"] = -5962, ["z"] = 1963, ["order"] = 21, ["angle"] = -228261888},
  [94] = {["x"] = -12084, ["y"] = -6199, ["z"] = 2007, ["order"] = 21, ["angle"] = -104792064},
  [95] = {["x"] = -11521, ["y"] = -6334, ["z"] = 2100, ["order"] = 21, ["angle"] = 13959168},
  [96] = {["x"] = -10600, ["y"] = -6458, ["z"] = 2100, ["order"] = 21, ["angle"] = 21430272},
  [97] = {["x"] = -10612, ["y"] = -6070, ["z"] = 2100, ["order"] = 21, ["angle"] = 65536},
  [98] = {["x"] = -9521, ["y"] = -6255, ["z"] = 1800, ["order"] = 21, ["angle"] = -25755648},
  [99] = {["x"] = -8963, ["y"] = -6284, ["z"] = 1600, ["order"] = 21, ["angle"] = 88670208},
  [100] = {["x"] = -8134, ["y"] = -6205, ["z"] = 1300, ["order"] = 21, ["angle"] = -13959168},
  [101] = {["x"] = -7606, ["y"] = -6101, ["z"] = 1300, ["order"] = 21, ["angle"] = -113967104},
  [102] = {["x"] = -6691, ["y"] = -6096, ["z"] = 1300, ["order"] = 21, ["angle"] = -556335104},
  [103] = {["x"] = -6280, ["y"] = -6407, ["z"] = 1300, ["order"] = 21, ["angle"] = -1202126848},
  [104] = {["x"] = -6461, ["y"] = -6958, ["z"] = 1320, ["order"] = 22, ["angle"] = -1580924928},
  [105] = {["x"] = -6967, ["y"] = -7479, ["z"] = 1430, ["order"] = 22, ["angle"] = -1510539264},
  [106] = {["x"] = -7522, ["y"] = -8104, ["z"] = 1500, ["order"] = 22, ["angle"] = -1157169152},
  [107] = {["x"] = -7683, ["y"] = -8743, ["z"] = 1460, ["order"] = 22, ["angle"] = -1013645312},
  [108] = {["x"] = -7812, ["y"] = -9573, ["z"] = 1330, ["order"] = 22, ["angle"] = -854654976},
  [109] = {["x"] = -7867, ["y"] = -10377, ["z"] = 1300, ["order"] = 22, ["angle"] = -517275648},
  [110] = {["x"] = -7457, ["y"] = -11083, ["z"] = 1300, ["order"] = 22, ["angle"] = 313458688},
  [111] = {["x"] = -6880, ["y"] = -11087, ["z"] = 1300, ["order"] = 23, ["angle"] = 671416320},
  [112] = {["x"] = -6591, ["y"] = -10549, ["z"] = 1300, ["order"] = 23, ["angle"] = 552271872},
  [113] = {["x"] = -6182, ["y"] = -10033, ["z"] = 1300, ["order"] = 23, ["angle"] = 159055872},
  [114] = {["x"] = -5383, ["y"] = -9813, ["z"] = 1300, ["order"] = 23, ["angle"] = 33882112},
  [115] = {["x"] = -4594, ["y"] = -9783, ["z"] = 1440, ["order"] = 23, ["angle"] = -19988480},
  [116] = {["x"] = -3885, ["y"] = -9789, ["z"] = 1450, ["order"] = 24, ["angle"] = 33357824},
  [117] = {["x"] = -2127, ["y"] = -9792, ["z"] = 1810, ["order"] = 24, ["angle"] = 65536},
  [118] = {["x"] = -1425, ["y"] = -9671, ["z"] = 1800, ["order"] = 24, ["angle"] = -69664768},
  [119] = {["x"] = -583, ["y"] = -9699, ["z"] = 1800, ["order"] = 24, ["angle"] = -423559168},
  [120] = {["x"] = 10, ["y"] = -9989, ["z"] = 1740, ["order"] = 25, ["angle"] = -742326272},
  [121] = {["x"] = 601, ["y"] = -10599, ["z"] = 1545, ["order"] = 25, ["angle"] = -1000275968},
  [122] = {["x"] = 1046, ["y"] = -11284, ["z"] = 1520, ["order"] = 25, ["angle"] = -1281949696},
  [123] = {["x"] = 894, ["y"] = -12147, ["z"] = 1520, ["order"] = 25, ["angle"] = -1805713408},
  [124] = {["x"] = 281, ["y"] = -12486, ["z"] = 1520, ["order"] = 25, ["angle"] = 1762328576},
  [125] = {["x"] = -282, ["y"] = -12162, ["z"] = 1510, ["order"] = 26, ["angle"] = 1452867584},
  [126] = {["x"] = -865, ["y"] = -11392, ["z"] = 1360, ["order"] = 26, ["angle"] = 1519583232},
  [127] = {["x"] = -1514, ["y"] = -10721, ["z"] = 1300, ["order"] = 26, ["angle"] = 1229914112},
  [128] = {["x"] = -1871, ["y"] = -9898, ["z"] = 1300, ["order"] = 26, ["angle"] = 1053753344},
  [129] = {["x"] = -1701, ["y"] = -8891, ["z"] = 1300, ["order"] = 26, ["angle"] = 1167130624},
  [130] = {["x"] = -2154, ["y"] = -8807, ["z"] = 1300, ["order"] = 26, ["angle"] = 992804864},
  [131] = {["x"] = -1808, ["y"] = -8078, ["z"] = 1300, ["order"] = 26, ["angle"] = 1444478976},
  [132] = {["x"] = -2261, ["y"] = -7274, ["z"] = 1300, ["order"] = 27, ["angle"] = 1715929088},
  [133] = {["x"] = -3293, ["y"] = -6514, ["z"] = 1000, ["order"] = 27, ["angle"] = 1788411904},
  [134] = {["x"] = -4127, ["y"] = -6160, ["z"] = 1000, ["order"] = 27, ["angle"] = 1535705088},
  [135] = {["x"] = -4806, ["y"] = -5547, ["z"] = 1000, ["order"] = 27, ["angle"] = 1127153664},
  [136] = {["x"] = -5008, ["y"] = -4668, ["z"] = 1000, ["order"] = 28, ["angle"] = 1039728640},
  [137] = {["x"] = -5043, ["y"] = -3609, ["z"] = 1000, ["order"] = 28, ["angle"] = 1031864320},
  [138] = {["x"] = -5066, ["y"] = -2523, ["z"] = 1000, ["order"] = 28, ["angle"] = 1171849216},
  [139] = {["x"] = -5344, ["y"] = -1573, ["z"] = 1000, ["order"] = 30, ["angle"] = 1327169536},
  [140] = {["x"] = -5962, ["y"] = -704, ["z"] = 970, ["order"] = 30, ["angle"] = 949551104},
  [141] = {["x"] = -6092, ["y"] = 80, ["z"] = 910, ["order"] = 30, ["angle"] = 492634112},
  [142] = {["x"] = -5649, ["y"] = 771, ["z"] = 827, ["order"] = 30, ["angle"] = -90505216},
  [143] = {["x"] = -4939, ["y"] = 1008, ["z"] = 777, ["order"] = 30, ["angle"] = -533528576},
  [144] = {["x"] = -4267, ["y"] = 684, ["z"] = 752, ["order"] = 30, ["angle"] = -628817920},
  [145] = {["x"] = -3450, ["y"] = 32, ["z"] = 565, ["order"] = 30, ["angle"] = -890175488},
  [146] = {["x"] = -3095, ["y"] = -797, ["z"] = 460, ["order"] = 30, ["angle"] = -1547108352},
  [147] = {["x"] = -3695, ["y"] = -1435, ["z"] = 418, ["order"] = 30, ["angle"] = 2094071808},
  [148] = {["x"] = -4765, ["y"] = -1343, ["z"] = 301, ["order"] = 30, ["angle"] = -1878982656},
  [149] = {["x"] = -6097, ["y"] = -1617, ["z"] = 301, ["order"] = 30, ["angle"] = -1311178752},
  [150] = {["x"] = -6623, ["y"] = -2377, ["z"] = 301, ["order"] = 30, ["angle"] = -615448576},
  [151] = {["x"] = -6258, ["y"] = -3170, ["z"] = 301, ["order"] = 31, ["angle"] = -113311744},
  [152] = {["x"] = -5420, ["y"] = -3351, ["z"] = 301, ["order"] = 31, ["angle"] = -67698688},
  [153] = {["x"] = -4535, ["y"] = -3455, ["z"] = 301, ["order"] = 31, ["angle"] = -502202368},
  [154] = {["x"] = -3937, ["y"] = -4122, ["z"] = 300, ["order"] = 31, ["angle"] = -478085120},
  [155] = {["x"] = -3241, ["y"] = -4786, ["z"] = 300, ["order"] = 31, ["angle"] = 57344000},
  [156] = {["x"] = -2414, ["y"] = -4894, ["z"] = 300, ["order"] = 31, ["angle"] = 781123584},
  [157] = {["x"] = -2098, ["y"] = -4309, ["z"] = 300, ["order"] = 32, ["angle"] = 911933440},
  [158] = {["x"] = -2091, ["y"] = -3560, ["z"] = 300, ["order"] = 32, ["angle"] = 897384448},
  [159] = {["x"] = -1984, ["y"] = -2587, ["z"] = 187, ["order"] = 32, ["angle"] = 566427648},
  [160] = {["x"] = -1525, ["y"] = -1746, ["z"] = 0, ["order"] = 32, ["angle"] = 52232192},
  [161] = {["x"] = -643, ["y"] = -1515, ["z"] = 0, ["order"] = 32, ["angle"] = 24051712},
  [162] = {["x"] = 8467, ["y"] = 1058, ["z"] = 800, ["order"] = 5, ["angle"] = 1946877952},
  [163] = {["x"] = 7736, ["y"] = 1157, ["z"] = 800, ["order"] = 5, ["angle"] = 1897463808},
  [164] = {["x"] = 7118, ["y"] = 1423, ["z"] = 800, ["order"] = 6, ["angle"] = 1550254080},
  [165] = {["x"] = 6752, ["y"] = 1860, ["z"] = 800, ["order"] = 6, ["angle"] = 1148125184},
  [166] = {["x"] = 6695, ["y"] = 2466, ["z"] = 800, ["order"] = 6, ["angle"] = 1051394048},
  [167] = {["x"] = 6758, ["y"] = 3546, ["z"] = 800, ["order"] = 6, ["angle"] = 1282605056},
  [168] = {["x"] = 2961, ["y"] = 1823, ["z"] = 1100, ["order"] = 7, ["angle"] = 1420361728},
  [169] = {["x"] = 2648, ["y"] = 2631, ["z"] = 1180, ["order"] = 7, ["angle"] = 1178664960},
  [170] = {["x"] = 2275, ["y"] = 3616, ["z"] = 1200, ["order"] = 7, ["angle"] = 1270415360},
  [171] = {["x"] = 1908, ["y"] = 4400, ["z"] = 1200, ["order"] = 7, ["angle"] = 703922176},
  [172] = {["x"] = 2204, ["y"] = 5054, ["z"] = 1200, ["order"] = 9, ["angle"] = 48824320},
  [173] = {["x"] = 3074, ["y"] = 5055, ["z"] = 1200, ["order"] = 9, ["angle"] = 319225856},
  [174] = {["x"] = 4188, ["y"] = 5241, ["z"] = 1200, ["order"] = 10, ["angle"] = 1002897408},
  [175] = {["x"] = 4383, ["y"] = 5906, ["z"] = 1200, ["order"] = 11, ["angle"] = 1946877952},
  [176] = {["x"] = 3712, ["y"] = 6098, ["z"] = 1140, ["order"] = 11, ["angle"] = 2076901376},
  [177] = {["x"] = 2821, ["y"] = 6184, ["z"] = 1000, ["order"] = 11, ["angle"] = 2111373312},
  [178] = {["x"] = 1935, ["y"] = 6078, ["z"] = 1000, ["order"] = 11, ["angle"] = 1965096960},
  [179] = {["x"] = 1004, ["y"] = 6211, ["z"] = 1000, ["order"] = 11, ["angle"] = 1748434944},
  [180] = {["x"] = 237, ["y"] = 6527, ["z"] = 1000, ["order"] = 11, ["angle"] = 1576730624},
  [181] = {["x"] = -297, ["y"] = 7224, ["z"] = 1000, ["order"] = 11, ["angle"] = 1576468480},
  [182] = {["x"] = -895, ["y"] = 7667, ["z"] = 900, ["order"] = 12, ["angle"] = 1501102080},
  [183] = {["x"] = -4093, ["y"] = -10192, ["z"] = 1450, ["order"] = 24, ["angle"] = -832897024},
  [184] = {["x"] = -3900, ["y"] = -10869, ["z"] = 1450, ["order"] = 24, ["angle"] = -1072627712},
  [185] = {["x"] = -3829, ["y"] = -11675, ["z"] = 1460, ["order"] = 24, ["angle"] = -1086783488},
  [186] = {["x"] = -3970, ["y"] = -12393, ["z"] = 1490, ["order"] = 24, ["angle"] = -917307392},
  [187] = {["x"] = -3959, ["y"] = -13059, ["z"] = 1490, ["order"] = 24, ["angle"] = -240975872},
  [188] = {["x"] = -3503, ["y"] = -13337, ["z"] = 1490, ["order"] = 25, ["angle"] = 311230464},
  [189] = {["x"] = -2895, ["y"] = -13124, ["z"] = 1490, ["order"] = 25, ["angle"] = 270860288},
  [190] = {["x"] = -2183, ["y"] = -12943, ["z"] = 1490, ["order"] = 25, ["angle"] = 462356480},
  [191] = {["x"] = -1719, ["y"] = -12604, ["z"] = 1490, ["order"] = 25, ["angle"] = 1108017152},
  [192] = {["x"] = -1742, ["y"] = -12077, ["z"] = 1490, ["order"] = 26, ["angle"] = 1355350016},
  [193] = {["x"] = -2065, ["y"] = -11545, ["z"] = 1403, ["order"] = 26, ["angle"] = 1262682112},
  [194] = {["x"] = -2376, ["y"] = -10998, ["z"] = 1300, ["order"] = 26, ["angle"] = 868941824},
  [195] = {["x"] = -2154, ["y"] = -10360, ["z"] = 1300, ["order"] = 26, ["angle"] = 894763008},
  [196] = {["x"] = -4563, ["y"] = -1773, ["z"] = 301, ["order"] = 30, ["angle"] = -2051604480},
  [197] = {["x"] = -5604, ["y"] = -1790, ["z"] = 301, ["order"] = 30, ["angle"] = -1732706304},
  [198] = {["x"] = -6434, ["y"] = -2624, ["z"] = 301, ["order"] = 30, ["angle"] = -562495488},
  [199] = {["x"] = -1242, ["y"] = -1533, ["z"] = 0, ["order"] = 32, ["angle"] = -98107392},
  [200] = {["x"] = -879, ["y"] = -1485, ["z"] = 0, ["order"] = 32, ["angle"] = -27197440}
}
-- Respawn definitions for PEACH'S CASTLE (MAPH1)
krespawn.points[353] = {
  [0] = 56,
  [1] = {["x"] = 287, ["y"] = -5987, ["z"] = 0, ["order"] = 0, ["angle"] = 1128333312},
  [2] = {["x"] = 249, ["y"] = -5320, ["z"] = 0, ["order"] = 0, ["angle"] = 1055719424},
  [3] = {["x"] = 220, ["y"] = -4506, ["z"] = 0, ["order"] = 0, ["angle"] = 1076035584},
  [4] = {["x"] = 170, ["y"] = -3700, ["z"] = 0, ["order"] = 0, ["angle"] = 1010630656},
  [5] = {["x"] = 108, ["y"] = -2936, ["z"] = 160, ["order"] = 0, ["angle"] = 893059072},
  [6] = {["x"] = 69, ["y"] = -2117, ["z"] = 192, ["order"] = 0, ["angle"] = 816381952},
  [7] = {["x"] = 306, ["y"] = -1254, ["z"] = 257, ["order"] = 0, ["angle"] = 51970048},
  [8] = {["x"] = 927, ["y"] = -1103, ["z"] = 348, ["order"] = 0, ["angle"] = -846135296},
  [9] = {["x"] = 1314, ["y"] = -1618, ["z"] = 439, ["order"] = 0, ["angle"] = -1561919488},
  [10] = {["x"] = 1015, ["y"] = -2143, ["z"] = 517, ["order"] = 0, ["angle"] = 1883570176},
  [11] = {["x"] = 484, ["y"] = -2070, ["z"] = 608, ["order"] = 0, ["angle"] = 1223098368},
  [12] = {["x"] = 262, ["y"] = -1373, ["z"] = 673, ["order"] = 0, ["angle"] = 1021902848},
  [13] = {["x"] = 230, ["y"] = -531, ["z"] = 673, ["order"] = 0, ["angle"] = 1031733248},
  [14] = {["x"] = 234, ["y"] = 164, ["z"] = 600, ["order"] = 1, ["angle"] = 1090846720},
  [15] = {["x"] = 236, ["y"] = 1419, ["z"] = 0, ["order"] = 1, ["angle"] = 1093468160},
  [16] = {["x"] = 243, ["y"] = 1890, ["z"] = 0, ["order"] = 1, ["angle"] = 1068302336},
  [17] = {["x"] = 323, ["y"] = 2653, ["z"] = 0, ["order"] = 1, ["angle"] = 1178009600},
  [18] = {["x"] = 276, ["y"] = 3702, ["z"] = 0, ["order"] = 1, ["angle"] = 1445134336},
  [19] = {["x"] = -205, ["y"] = 4729, ["z"] = 0, ["order"] = 1, ["angle"] = 1747517440},
  [20] = {["x"] = -1005, ["y"] = 5238, ["z"] = 0, ["order"] = 1, ["angle"] = -2112159744},
  [21] = {["x"] = -2084, ["y"] = 5361, ["z"] = 0, ["order"] = 1, ["angle"] = -2112159744},
  [22] = {["x"] = -2807, ["y"] = 5327, ["z"] = 0, ["order"] = 1, ["angle"] = -1861287936},
  [23] = {["x"] = -3570, ["y"] = 4826, ["z"] = 0, ["order"] = 1, ["angle"] = -2079391744},
  [24] = {["x"] = -4110, ["y"] = 4728, ["z"] = 0, ["order"] = 1, ["angle"] = 2007957504},
  [25] = {["x"] = -4953, ["y"] = 5122, ["z"] = 0, ["order"] = 1, ["angle"] = 2068905984},
  [26] = {["x"] = -5672, ["y"] = 5377, ["z"] = 0, ["order"] = 2, ["angle"] = -1831534592},
  [27] = {["x"] = -6162, ["y"] = 5272, ["z"] = 0, ["order"] = 2, ["angle"] = -1581318144},
  [28] = {["x"] = -6671, ["y"] = 4564, ["z"] = 0, ["order"] = 2, ["angle"] = -1573847040},
  [29] = {["x"] = -7082, ["y"] = 4028, ["z"] = 0, ["order"] = 2, ["angle"] = -1711865856},
  [30] = {["x"] = -8208, ["y"] = 3344, ["z"] = 0, ["order"] = 2, ["angle"] = -1616314368},
  [31] = {["x"] = -9153, ["y"] = 2669, ["z"] = 0, ["order"] = 2, ["angle"] = -1443692544},
  [32] = {["x"] = -9892, ["y"] = 1793, ["z"] = 0, ["order"] = 2, ["angle"] = -1110245376},
  [33] = {["x"] = -10255, ["y"] = 819, ["z"] = 0, ["order"] = 3, ["angle"] = -994377728},
  [34] = {["x"] = -10328, ["y"] = -134, ["z"] = -480, ["order"] = 3, ["angle"] = -759365632},
  [35] = {["x"] = -10146, ["y"] = -820, ["z"] = -512, ["order"] = 3, ["angle"] = -552009728},
  [36] = {["x"] = -9377, ["y"] = -1793, ["z"] = -512, ["order"] = 3, ["angle"] = -361955328},
  [37] = {["x"] = -8209, ["y"] = -2391, ["z"] = -512, ["order"] = 3, ["angle"] = -285147136},
  [38] = {["x"] = -6883, ["y"] = -2688, ["z"] = -512, ["order"] = 3, ["angle"] = -390135808},
  [39] = {["x"] = -5642, ["y"] = -3285, ["z"] = -512, ["order"] = 3, ["angle"] = -817430528},
  [40] = {["x"] = -5012, ["y"] = -4258, ["z"] = -512, ["order"] = 4, ["angle"] = -1232273408},
  [41] = {["x"] = -4975, ["y"] = -5558, ["z"] = -512, ["order"] = 4, ["angle"] = -1740832768},
  [42] = {["x"] = -5783, ["y"] = -6504, ["z"] = -512, ["order"] = 4, ["angle"] = -2053963776},
  [43] = {["x"] = -6868, ["y"] = -6769, ["z"] = -480, ["order"] = 4, ["angle"] = 2118451200},
  [44] = {["x"] = -7652, ["y"] = -6804, ["z"] = -96, ["order"] = 4, ["angle"] = -2097086464},
  [45] = {["x"] = -8626, ["y"] = -6765, ["z"] = 130, ["order"] = 4, ["angle"] = -2024734720},
  [46] = {["x"] = -9881, ["y"] = -6745, ["z"] = 316, ["order"] = 4, ["angle"] = -1695875072},
  [47] = {["x"] = -10747, ["y"] = -7178, ["z"] = 384, ["order"] = 4, ["angle"] = -1106837504},
  [48] = {["x"] = -10820, ["y"] = -7980, ["z"] = 384, ["order"] = 5, ["angle"] = -257884160},
  [49] = {["x"] = -9469, ["y"] = -8489, ["z"] = 529, ["order"] = 5, ["angle"] = 24313856},
  [50] = {["x"] = -8502, ["y"] = -8506, ["z"] = 679, ["order"] = 5, ["angle"] = 24313856},
  [51] = {["x"] = -7449, ["y"] = -8483, ["z"] = 844, ["order"] = 5, ["angle"] = 24313856},
  [52] = {["x"] = -6309, ["y"] = -8458, ["z"] = 1008, ["order"] = 5, ["angle"] = 37027840},
  [53] = {["x"] = -243, ["y"] = -8432, ["z"] = 0, ["order"] = 5, ["angle"] = 1005649920},
  [54] = {["x"] = -7, ["y"] = -7909, ["z"] = 0, ["order"] = 5, ["angle"] = 938672128},
  [55] = {["x"] = 134, ["y"] = -7178, ["z"] = 0, ["order"] = 5, ["angle"] = 943783936},
  [56] = {["x"] = 240, ["y"] = -6502, ["z"] = 0, ["order"] = 5, ["angle"] = 983367680}
}
-- Respawn definitions for ARID SANDS (MAPH2)
krespawn.points[354] = {
  [0] = 68,
  [1] = {["x"] = 6134, ["y"] = -3044, ["z"] = 0, ["order"] = 0, ["angle"] = 1052442624},
  [2] = {["x"] = 6143, ["y"] = -2413, ["z"] = 0, ["order"] = 0, ["angle"] = 1052442624},
  [3] = {["x"] = 6194, ["y"] = -1664, ["z"] = 0, ["order"] = 0, ["angle"] = 1082720256},
  [4] = {["x"] = 6284, ["y"] = -625, ["z"] = 0, ["order"] = 0, ["angle"] = 1326252032},
  [5] = {["x"] = 6040, ["y"] = 342, ["z"] = 0, ["order"] = 1, ["angle"] = 1851195392},
  [6] = {["x"] = 5297, ["y"] = 766, ["z"] = 0, ["order"] = 1, ["angle"] = 2085683200},
  [7] = {["x"] = 4280, ["y"] = 961, ["z"] = 0, ["order"] = 2, ["angle"] = 2014773248},
  [8] = {["x"] = 3349, ["y"] = 1099, ["z"] = 0, ["order"] = 2, ["angle"] = -2012545024},
  [9] = {["x"] = 2415, ["y"] = 918, ["z"] = 0, ["order"] = 2, ["angle"] = 1953038336},
  [10] = {["x"] = 1225, ["y"] = 985, ["z"] = 0, ["order"] = 3, ["angle"] = 1265958912},
  [11] = {["x"] = 856, ["y"] = 1254, ["z"] = 0, ["order"] = 3, ["angle"] = 1124007936},
  [12] = {["x"] = 758, ["y"] = 1852, ["z"] = 0, ["order"] = 3, ["angle"] = 1150877696},
  [13] = {["x"] = 716, ["y"] = 2602, ["z"] = 7, ["order"] = 3, ["angle"] = 1273298944},
  [14] = {["x"] = 435, ["y"] = 3953, ["z"] = 16, ["order"] = 3, ["angle"] = 1019019264},
  [15] = {["x"] = 757, ["y"] = 4906, ["z"] = 0, ["order"] = 3, ["angle"] = 1177747456},
  [16] = {["x"] = 516, ["y"] = 5901, ["z"] = 0, ["order"] = 3, ["angle"] = 948764672},
  [17] = {["x"] = 617, ["y"] = 6745, ["z"] = 0, ["order"] = 4, ["angle"] = 127467520},
  [18] = {["x"] = 1392, ["y"] = 7014, ["z"] = 0, ["order"] = 4, ["angle"] = -16318464},
  [19] = {["x"] = 2319, ["y"] = 7045, ["z"] = 327, ["order"] = 4, ["angle"] = -16318464},
  [20] = {["x"] = 3576, ["y"] = 7117, ["z"] = 770, ["order"] = 4, ["angle"] = -551485440},
  [21] = {["x"] = 4187, ["y"] = 6392, ["z"] = 957, ["order"] = 5, ["angle"] = -1538064384},
  [22] = {["x"] = 3530, ["y"] = 5622, ["z"] = 756, ["order"] = 5, ["angle"] = -1623785472},
  [23] = {["x"] = 2971, ["y"] = 5007, ["z"] = 585, ["order"] = 5, ["angle"] = -1262288896},
  [24] = {["x"] = 2661, ["y"] = 4102, ["z"] = 491, ["order"] = 5, ["angle"] = -1090453504},
  [25] = {["x"] = 2555, ["y"] = 3031, ["z"] = 451, ["order"] = 5, ["angle"] = -1090453504},
  [26] = {["x"] = 2517, ["y"] = 2133, ["z"] = 343, ["order"] = 5, ["angle"] = -1090453504},
  [27] = {["x"] = 3171, ["y"] = -2078, ["z"] = 500, ["order"] = 6, ["angle"] = -1231093760},
  [28] = {["x"] = 3182, ["y"] = -2960, ["z"] = 500, ["order"] = 6, ["angle"] = -1752498176},
  [29] = {["x"] = 2830, ["y"] = -3414, ["z"] = 500, ["order"] = 6, ["angle"] = -1866530816},
  [30] = {["x"] = 2092, ["y"] = -3850, ["z"] = 500, ["order"] = 6, ["angle"] = -2085421056},
  [31] = {["x"] = 1237, ["y"] = -3997, ["z"] = 500, ["order"] = 6, ["angle"] = 2144796672},
  [32] = {["x"] = 168, ["y"] = -4023, ["z"] = 500, ["order"] = 6, ["angle"] = 2144796672},
  [33] = {["x"] = -840, ["y"] = -4010, ["z"] = 500, ["order"] = 6, ["angle"] = 2015428608},
  [34] = {["x"] = -2291, ["y"] = -3842, ["z"] = 500, ["order"] = 7, ["angle"] = -2022637568},
  [35] = {["x"] = -3458, ["y"] = -4020, ["z"] = 710, ["order"] = 7, ["angle"] = -1472397312},
  [36] = {["x"] = -4221, ["y"] = -4740, ["z"] = 885, ["order"] = 7, ["angle"] = -701693952},
  [37] = {["x"] = -4000, ["y"] = -5448, ["z"] = 834, ["order"] = 7, ["angle"] = -260636672},
  [38] = {["x"] = -3448, ["y"] = -5768, ["z"] = 707, ["order"] = 7, ["angle"] = -24051712},
  [39] = {["x"] = -2469, ["y"] = -6015, ["z"] = 500, ["order"] = 8, ["angle"] = -193265664},
  [40] = {["x"] = -1235, ["y"] = -6465, ["z"] = 500, ["order"] = 8, ["angle"] = 209125376},
  [41] = {["x"] = -1584, ["y"] = -5950, ["z"] = 500, ["order"] = 8, ["angle"] = -84869120},
  [42] = {["x"] = 115, ["y"] = -6364, ["z"] = 500, ["order"] = 8, ["angle"] = -720896},
  [43] = {["x"] = 76, ["y"] = -5614, ["z"] = 500, ["order"] = 8, ["angle"] = -195756032},
  [44] = {["x"] = 1458, ["y"] = -6042, ["z"] = 500, ["order"] = 8, ["angle"] = -52756480},
  [45] = {["x"] = 1918, ["y"] = -6623, ["z"] = 500, ["order"] = 8, ["angle"] = 42139648},
  [46] = {["x"] = 3101, ["y"] = -6287, ["z"] = 500, ["order"] = 8, ["angle"] = 2424832},
  [47] = {["x"] = 4287, ["y"] = -6375, ["z"] = 485, ["order"] = 9, ["angle"] = 114622464},
  [48] = {["x"] = 5806, ["y"] = -6312, ["z"] = 186, ["order"] = 9, ["angle"] = 999358464},
  [49] = {["x"] = 6153, ["y"] = -5769, ["z"] = 102, ["order"] = 9, ["angle"] = 1110245376},
  [50] = {["x"] = 6199, ["y"] = -4968, ["z"] = 0, ["order"] = 9, ["angle"] = 1073938432},
  [51] = {["x"] = 6169, ["y"] = -4240, ["z"] = 0, ["order"] = 9, ["angle"] = 1111162880},
  [52] = {["x"] = 6142, ["y"] = -3493, ["z"] = 0, ["order"] = 9, ["angle"] = 1042087936},
  [53] = {["x"] = 4105, ["y"] = 569, ["z"] = 0, ["order"] = 2, ["angle"] = -1948188672},
  [54] = {["x"] = 2822, ["y"] = 256, ["z"] = 372, ["order"] = 2, ["angle"] = -2147418112},
  [55] = {["x"] = 2021, ["y"] = 256, ["z"] = 372, ["order"] = 2, ["angle"] = -2147418112},
  [56] = {["x"] = -77, ["y"] = 923, ["z"] = 0, ["order"] = 3, ["angle"] = 1043398656},
  [57] = {["x"] = 4433, ["y"] = 5575, ["z"] = 1100, ["order"] = 5, ["angle"] = -1826684928},
  [58] = {["x"] = 3109, ["y"] = 4709, ["z"] = 1100, ["order"] = 5, ["angle"] = -1052311552},
  [59] = {["x"] = 2302, ["y"] = -1299, ["z"] = 500, ["order"] = 5, ["angle"] = -1470562304},
  [60] = {["x"] = 1814, ["y"] = -2081, ["z"] = 500, ["order"] = 6, ["angle"] = -1538064384},
  [61] = {["x"] = 965, ["y"] = -2798, ["z"] = 500, ["order"] = 6, ["angle"] = -2050555904},
  [62] = {["x"] = -1132, ["y"] = -3010, ["z"] = 500, ["order"] = 6, ["angle"] = -1944256512},
  [63] = {["x"] = -2392, ["y"] = -3286, ["z"] = 500, ["order"] = 7, ["angle"] = -1786445824},
  [64] = {["x"] = 5331, ["y"] = -7342, ["z"] = 500, ["order"] = 9, ["angle"] = -21954560},
  [65] = {["x"] = 2494, ["y"] = 5377, ["z"] = 768, ["order"] = 5, ["angle"] = -1343815680},
  [66] = {["x"] = 2020, ["y"] = 3032, ["z"] = 768, ["order"] = 5, ["angle"] = -1050345472},
  [67] = {["x"] = 3066, ["y"] = 3762, ["z"] = 1100, ["order"] = 5, ["angle"] = -1063583744},
  [68] = {["x"] = 2016, ["y"] = -1503, ["z"] = 500, ["order"] = 6, ["angle"] = -1489174528}
}
-- Respawn definitions for SEPTIC QUARRY (MAPH3)
krespawn.points[355] = {
  [0] = 85,
  [1] = {["x"] = 2528, ["y"] = -9046, ["z"] = 256, ["order"] = 0, ["angle"] = 1148125184},
  [2] = {["x"] = 2457, ["y"] = -8115, ["z"] = 256, ["order"] = 0, ["angle"] = 1352466432},
  [3] = {["x"] = 2012, ["y"] = -7240, ["z"] = 256, ["order"] = 0, ["angle"] = 1510146048},
  [4] = {["x"] = 1304, ["y"] = -6418, ["z"] = 256, ["order"] = 0, ["angle"] = 1276837888},
  [5] = {["x"] = 948, ["y"] = -5564, ["z"] = 256, ["order"] = 0, ["angle"] = 686489600},
  [6] = {["x"] = 1235, ["y"] = -4986, ["z"] = 256, ["order"] = 1, ["angle"] = 273612800},
  [7] = {["x"] = 2174, ["y"] = -4656, ["z"] = 256, ["order"] = 1, ["angle"] = 573374464},
  [8] = {["x"] = 2652, ["y"] = -4162, ["z"] = 256, ["order"] = 1, ["angle"] = 1104084992},
  [9] = {["x"] = 2658, ["y"] = -3255, ["z"] = 256, ["order"] = 1, ["angle"] = 743768064},
  [10] = {["x"] = 3048, ["y"] = -2432, ["z"] = 256, ["order"] = 1, ["angle"] = 53411840},
  [11] = {["x"] = 3824, ["y"] = -2220, ["z"] = 256, ["order"] = 1, ["angle"] = -156172288},
  [12] = {["x"] = 4929, ["y"] = -2172, ["z"] = 256, ["order"] = 2, ["angle"] = -425918464},
  [13] = {["x"] = 5859, ["y"] = -2636, ["z"] = 256, ["order"] = 2, ["angle"] = -1049821184},
  [14] = {["x"] = 6117, ["y"] = -3563, ["z"] = 256, ["order"] = 2, ["angle"] = -1723138048},
  [15] = {["x"] = 5464, ["y"] = -3860, ["z"] = 256, ["order"] = 2, ["angle"] = -1883176960},
  [16] = {["x"] = 4272, ["y"] = -4206, ["z"] = 256, ["order"] = 2, ["angle"] = -1214971904},
  [17] = {["x"] = 3944, ["y"] = -5068, ["z"] = 256, ["order"] = 3, ["angle"] = -520945664},
  [18] = {["x"] = 4461, ["y"] = -5794, ["z"] = 256, ["order"] = 3, ["angle"] = -653590528},
  [19] = {["x"] = 5083, ["y"] = -6939, ["z"] = 256, ["order"] = 3, ["angle"] = 458752},
  [20] = {["x"] = 6030, ["y"] = -7215, ["z"] = 256, ["order"] = 3, ["angle"] = 176881664},
  [21] = {["x"] = 7164, ["y"] = -6945, ["z"] = 256, ["order"] = 3, ["angle"] = 617414656},
  [22] = {["x"] = 7943, ["y"] = -6055, ["z"] = 256, ["order"] = 3, ["angle"] = 691077120},
  [23] = {["x"] = 8598, ["y"] = -4931, ["z"] = 256, ["order"] = 4, ["angle"] = 816644096},
  [24] = {["x"] = 9039, ["y"] = -3867, ["z"] = 256, ["order"] = 4, ["angle"] = 989659136},
  [25] = {["x"] = 9068, ["y"] = -2359, ["z"] = 256, ["order"] = 4, ["angle"] = 637468672},
  [26] = {["x"] = 9484, ["y"] = -1086, ["z"] = 256, ["order"] = 5, ["angle"] = -119603200},
  [27] = {["x"] = 10624, ["y"] = -980, ["z"] = 256, ["order"] = 6, ["angle"] = -665518080},
  [28] = {["x"] = 11034, ["y"] = -1698, ["z"] = 256, ["order"] = 6, ["angle"] = -906690560},
  [29] = {["x"] = 11135, ["y"] = -2979, ["z"] = 256, ["order"] = 6, ["angle"] = -508755968},
  [30] = {["x"] = 11732, ["y"] = -3625, ["z"] = 256, ["order"] = 6, ["angle"] = -461701120},
  [31] = {["x"] = 12721, ["y"] = -4198, ["z"] = 256, ["order"] = 7, ["angle"] = -715718656},
  [32] = {["x"] = 13533, ["y"] = -5005, ["z"] = 256, ["order"] = 7, ["angle"] = -1185480704},
  [33] = {["x"] = 13358, ["y"] = -6004, ["z"] = 256, ["order"] = 7, ["angle"] = -1695612928},
  [34] = {["x"] = 12635, ["y"] = -6586, ["z"] = 256, ["order"] = 7, ["angle"] = -1834942464},
  [35] = {["x"] = 11462, ["y"] = -7114, ["z"] = 256, ["order"] = 8, ["angle"] = -1329266688},
  [36] = {["x"] = 10786, ["y"] = -7942, ["z"] = 256, ["order"] = 8, ["angle"] = -658309120},
  [37] = {["x"] = 11248, ["y"] = -9248, ["z"] = 256, ["order"] = 9, ["angle"] = -176095232},
  [38] = {["x"] = 12067, ["y"] = -9674, ["z"] = 256, ["order"] = 9, ["angle"] = 167444480},
  [39] = {["x"] = 13166, ["y"] = -9396, ["z"] = 256, ["order"] = 9, ["angle"] = 150798336},
  [40] = {["x"] = 14359, ["y"] = -8913, ["z"] = 256, ["order"] = 9, ["angle"] = -392888320},
  [41] = {["x"] = 15291, ["y"] = -9261, ["z"] = 256, ["order"] = 9, ["angle"] = -608370688},
  [42] = {["x"] = 16217, ["y"] = -10396, ["z"] = 256, ["order"] = 10, ["angle"] = -371785728},
  [43] = {["x"] = 16890, ["y"] = -11184, ["z"] = 256, ["order"] = 10, ["angle"] = -442433536},
  [44] = {["x"] = 17814, ["y"] = -11975, ["z"] = 283, ["order"] = 10, ["angle"] = 601686016},
  [45] = {["x"] = 18950, ["y"] = -11220, ["z"] = 384, ["order"] = 11, ["angle"] = 1637679104},
  [46] = {["x"] = 18096, ["y"] = -10065, ["z"] = 466, ["order"] = 11, ["angle"] = -1638334464},
  [47] = {["x"] = 17406, ["y"] = -10517, ["z"] = 512, ["order"] = 11, ["angle"] = -1600585728},
  [48] = {["x"] = 16853, ["y"] = -11021, ["z"] = 512, ["order"] = 12, ["angle"] = -1555234816},
  [49] = {["x"] = 16222, ["y"] = -11655, ["z"] = 512, ["order"] = 12, ["angle"] = -1438449664},
  [50] = {["x"] = 15396, ["y"] = -12367, ["z"] = 512, ["order"] = 12, ["angle"] = -631963648},
  [51] = {["x"] = 15783, ["y"] = -12883, ["z"] = 512, ["order"] = 13, ["angle"] = -447414272},
  [52] = {["x"] = 16567, ["y"] = -13215, ["z"] = 512, ["order"] = 13, ["angle"] = -558432256},
  [53] = {["x"] = 17348, ["y"] = -13620, ["z"] = 512, ["order"] = 13, ["angle"] = -1234501632},
  [54] = {["x"] = 17361, ["y"] = -14352, ["z"] = 512, ["order"] = 14, ["angle"] = -1755250688},
  [55] = {["x"] = 16953, ["y"] = -14838, ["z"] = 512, ["order"] = 14, ["angle"] = -1781727232},
  [56] = {["x"] = 16343, ["y"] = -15425, ["z"] = 512, ["order"] = 14, ["angle"] = -1978597376},
  [57] = {["x"] = 15708, ["y"] = -15887, ["z"] = 512, ["order"] = 14, ["angle"] = 1798897664},
  [58] = {["x"] = 15125, ["y"] = -15702, ["z"] = 512, ["order"] = 15, ["angle"] = 1405026304},
  [59] = {["x"] = 14761, ["y"] = -15211, ["z"] = 512, ["order"] = 15, ["angle"] = 1228734464},
  [60] = {["x"] = 14675, ["y"] = -14467, ["z"] = 512, ["order"] = 15, ["angle"] = 1146421248},
  [61] = {["x"] = 14698, ["y"] = -13613, ["z"] = 512, ["order"] = 15, ["angle"] = 1337262080},
  [62] = {["x"] = 14472, ["y"] = -12771, ["z"] = 512, ["order"] = 15, ["angle"] = 1744764928},
  [63] = {["x"] = 13946, ["y"] = -12123, ["z"] = 512, ["order"] = 15, ["angle"] = -2087911424},
  [64] = {["x"] = 13394, ["y"] = -12060, ["z"] = 512, ["order"] = 16, ["angle"] = -1673723904},
  [65] = {["x"] = 11390, ["y"] = -13918, ["z"] = 256, ["order"] = 16, ["angle"] = -1601241088},
  [66] = {["x"] = 10722, ["y"] = -14701, ["z"] = 256, ["order"] = 16, ["angle"] = -1932460032},
  [67] = {["x"] = 9876, ["y"] = -15076, ["z"] = 256, ["order"] = 17, ["angle"] = 1783037952},
  [68] = {["x"] = 9116, ["y"] = -14669, ["z"] = 256, ["order"] = 17, ["angle"] = 1660223488},
  [69] = {["x"] = 8284, ["y"] = -13749, ["z"] = 256, ["order"] = 17, ["angle"] = 1934032896},
  [70] = {["x"] = 7402, ["y"] = -13327, ["z"] = 256, ["order"] = 17, ["angle"] = -1842675712},
  [71] = {["x"] = 6695, ["y"] = -13685, ["z"] = 256, ["order"] = 17, ["angle"] = -1665597440},
  [72] = {["x"] = 6028, ["y"] = -14505, ["z"] = 256, ["order"] = 17, ["angle"] = -1945042944},
  [73] = {["x"] = 5170, ["y"] = -15152, ["z"] = 256, ["order"] = 17, ["angle"] = 1703870464},
  [74] = {["x"] = 4597, ["y"] = -14761, ["z"] = 256, ["order"] = 18, ["angle"] = 1188757504},
  [75] = {["x"] = 4568, ["y"] = -13864, ["z"] = 256, ["order"] = 18, ["angle"] = 1046151168},
  [76] = {["x"] = 4876, ["y"] = -12440, ["z"] = 256, ["order"] = 18, ["angle"] = 1609367552},
  [77] = {["x"] = 4162, ["y"] = -11872, ["z"] = 256, ["order"] = 18, ["angle"] = 1864302592},
  [78] = {["x"] = 3265, ["y"] = -11703, ["z"] = 256, ["order"] = 18, ["angle"] = 1555628032},
  [79] = {["x"] = 2520, ["y"] = -11217, ["z"] = 256, ["order"] = 18, ["angle"] = 1006436352},
  [80] = {["x"] = 2454, ["y"] = -10485, ["z"] = 256, ["order"] = 18, ["angle"] = 1015218176},
  [81] = {["x"] = 2508, ["y"] = -9591, ["z"] = 256, ["order"] = 18, ["angle"] = 1116798976},
  [82] = {["x"] = 15701, ["y"] = -9624, ["z"] = 256, ["order"] = 9, ["angle"] = -578355200},
  [83] = {["x"] = 9977, ["y"] = -13005, ["z"] = 256, ["order"] = 16, ["angle"] = 2099576832},
  [84] = {["x"] = 8738, ["y"] = -12794, ["z"] = 256, ["order"] = 17, ["angle"] = -1990918144},
  [85] = {["x"] = 7613, ["y"] = -13006, ["z"] = 256, ["order"] = 17, ["angle"] = -1748566016}
}
-- Respawn definitions for CHEMICAL FACILITY (MAPH4)
krespawn.points[356] = {
  [0] = 122,
  [1] = {["x"] = 13241, ["y"] = -4386, ["z"] = 0, ["order"] = 0, ["angle"] = -23265280},
  [2] = {["x"] = 14108, ["y"] = -4371, ["z"] = 0, ["order"] = 0, ["angle"] = 24444928},
  [3] = {["x"] = 15173, ["y"] = -4306, ["z"] = 16, ["order"] = 0, ["angle"] = 39256064},
  [4] = {["x"] = 16397, ["y"] = -4186, ["z"] = 0, ["order"] = 0, ["angle"] = -71106560},
  [5] = {["x"] = 17621, ["y"] = -4118, ["z"] = 0, ["order"] = 0, ["angle"] = -451608576},
  [6] = {["x"] = 18410, ["y"] = -4246, ["z"] = -16, ["order"] = 0, ["angle"] = -1076690944},
  [7] = {["x"] = 18613, ["y"] = -4914, ["z"] = 0, ["order"] = 0, ["angle"] = -1066205184},
  [8] = {["x"] = 18579, ["y"] = -5903, ["z"] = 0, ["order"] = 0, ["angle"] = -1375535104},
  [9] = {["x"] = 18472, ["y"] = -6863, ["z"] = 0, ["order"] = 0, ["angle"] = -1375535104},
  [10] = {["x"] = 18174, ["y"] = -7881, ["z"] = 0, ["order"] = 0, ["angle"] = -1304756224},
  [11] = {["x"] = 17912, ["y"] = -8656, ["z"] = -64, ["order"] = 0, ["angle"] = -1424162816},
  [12] = {["x"] = 17693, ["y"] = -9890, ["z"] = -64, ["order"] = 0, ["angle"] = -1984102400},
  [13] = {["x"] = 17172, ["y"] = -10252, ["z"] = -64, ["order"] = 0, ["angle"] = 2095644672},
  [14] = {["x"] = 16268, ["y"] = -10368, ["z"] = -64, ["order"] = 0, ["angle"] = 1889468416},
  [15] = {["x"] = 15105, ["y"] = -10151, ["z"] = -64, ["order"] = 1, ["angle"] = 1281687552},
  [16] = {["x"] = 14823, ["y"] = -9599, ["z"] = -96, ["order"] = 1, ["angle"] = 687013888},
  [17] = {["x"] = 15184, ["y"] = -8923, ["z"] = -128, ["order"] = 1, ["angle"] = 651624448},
  [18] = {["x"] = 15844, ["y"] = -8232, ["z"] = -128, ["order"] = 1, ["angle"] = 712441856},
  [19] = {["x"] = 16758, ["y"] = -7047, ["z"] = -112, ["order"] = 1, ["angle"] = 750714880},
  [20] = {["x"] = 17367, ["y"] = -6329, ["z"] = -96, ["order"] = 1, ["angle"] = 1369374720},
  [21] = {["x"] = 17164, ["y"] = -5621, ["z"] = -80, ["order"] = 2, ["angle"] = -2063400960},
  [22] = {["x"] = 16394, ["y"] = -5567, ["z"] = -80, ["order"] = 2, ["angle"] = -1589313536},
  [23] = {["x"] = 15469, ["y"] = -6322, ["z"] = -48, ["order"] = 2, ["angle"] = -1981612032},
  [24] = {["x"] = 12373, ["y"] = -7146, ["z"] = 320, ["order"] = 2, ["angle"] = -1872560128},
  [25] = {["x"] = 11381, ["y"] = -7238, ["z"] = 320, ["order"] = 2, ["angle"] = -1735065600},
  [26] = {["x"] = 10383, ["y"] = -7710, ["z"] = 320, ["order"] = 2, ["angle"] = -1217986560},
  [27] = {["x"] = 10040, ["y"] = -8718, ["z"] = 320, ["order"] = 3, ["angle"] = -947060736},
  [28] = {["x"] = 9866, ["y"] = -10170, ["z"] = 320, ["order"] = 3, ["angle"] = -322109440},
  [29] = {["x"] = 10652, ["y"] = -10662, ["z"] = 320, ["order"] = 3, ["angle"] = -43843584},
  [30] = {["x"] = 11501, ["y"] = -10618, ["z"] = 320, ["order"] = 3, ["angle"] = -31391744},
  [31] = {["x"] = 12532, ["y"] = -10469, ["z"] = 320, ["order"] = 3, ["angle"] = -355139584},
  [32] = {["x"] = 13275, ["y"] = -10733, ["z"] = 320, ["order"] = 3, ["angle"] = -761593856},
  [33] = {["x"] = 13512, ["y"] = -11505, ["z"] = 320, ["order"] = 3, ["angle"] = -674299904},
  [34] = {["x"] = 13869, ["y"] = -12162, ["z"] = 320, ["order"] = 3, ["angle"] = -23920640},
  [35] = {["x"] = 14655, ["y"] = -12266, ["z"] = 320, ["order"] = 3, ["angle"] = -47120384},
  [36] = {["x"] = 15559, ["y"] = -12189, ["z"] = 320, ["order"] = 3, ["angle"] = -355532800},
  [37] = {["x"] = 16288, ["y"] = -12423, ["z"] = 320, ["order"] = 3, ["angle"] = -907083776},
  [38] = {["x"] = 16515, ["y"] = -13152, ["z"] = 320, ["order"] = 3, ["angle"] = -1097400320},
  [39] = {["x"] = 16536, ["y"] = -14243, ["z"] = 320, ["order"] = 4, ["angle"] = -1097400320},
  [40] = {["x"] = 16726, ["y"] = -18795, ["z"] = 512, ["order"] = 5, ["angle"] = 2110980096},
  [41] = {["x"] = 15510, ["y"] = -18788, ["z"] = 512, ["order"] = 5, ["angle"] = -2074542080},
  [42] = {["x"] = 12734, ["y"] = -18928, ["z"] = 520, ["order"] = 5, ["angle"] = 1711079424},
  [43] = {["x"] = 13470, ["y"] = -18923, ["z"] = 512, ["order"] = 5, ["angle"] = 2059599872},
  [44] = {["x"] = 14236, ["y"] = -18925, ["z"] = 512, ["order"] = 5, ["angle"] = 2082275328},
  [45] = {["x"] = 14914, ["y"] = -18928, ["z"] = 512, ["order"] = 5, ["angle"] = 2091450368},
  [46] = {["x"] = 12296, ["y"] = -18618, ["z"] = 512, ["order"] = 5, ["angle"] = 1664155648},
  [47] = {["x"] = 11587, ["y"] = -17863, ["z"] = 512, ["order"] = 5, ["angle"] = 1974140928},
  [48] = {["x"] = 11073, ["y"] = -17642, ["z"] = 512, ["order"] = 5, ["angle"] = -1462304768},
  [49] = {["x"] = 10811, ["y"] = -18125, ["z"] = 528, ["order"] = 5, ["angle"] = -1218117632},
  [50] = {["x"] = 10861, ["y"] = -18627, ["z"] = 704, ["order"] = 5, ["angle"] = -1555759104},
  [51] = {["x"] = 10623, ["y"] = -19342, ["z"] = 768, ["order"] = 5, ["angle"] = -1916076032},
  [52] = {["x"] = 10078, ["y"] = -19561, ["z"] = 768, ["order"] = 5, ["angle"] = 1561001984},
  [53] = {["x"] = 9708, ["y"] = -19054, ["z"] = 800, ["order"] = 5, ["angle"] = 1393360896},
  [54] = {["x"] = 9548, ["y"] = -18269, ["z"] = 832, ["order"] = 6, ["angle"] = 1632567296},
  [55] = {["x"] = 9150, ["y"] = -17660, ["z"] = 832, ["order"] = 6, ["angle"] = -2097217536},
  [56] = {["x"] = 8357, ["y"] = -17530, ["z"] = 832, ["order"] = 6, ["angle"] = -1605435392},
  [57] = {["x"] = 8011, ["y"] = -17756, ["z"] = 832, ["order"] = 6, ["angle"] = -1360068608},
  [58] = {["x"] = 7900, ["y"] = -18337, ["z"] = 896, ["order"] = 6, ["angle"] = -1245773824},
  [59] = {["x"] = 7890, ["y"] = -18954, ["z"] = 912, ["order"] = 7, ["angle"] = -1497432064},
  [60] = {["x"] = 7641, ["y"] = -19773, ["z"] = 960, ["order"] = 7, ["angle"] = -1848573952},
  [61] = {["x"] = 7021, ["y"] = -20180, ["z"] = 960, ["order"] = 7, ["angle"] = 1957756928},
  [62] = {["x"] = 6440, ["y"] = -19961, ["z"] = 1024, ["order"] = 7, ["angle"] = 1856831488},
  [63] = {["x"] = 5901, ["y"] = -19932, ["z"] = 1024, ["order"] = 7, ["angle"] = 2044919808},
  [64] = {["x"] = 4882, ["y"] = -19966, ["z"] = 1024, ["order"] = 8, ["angle"] = 1543569408},
  [65] = {["x"] = 4471, ["y"] = -19489, ["z"] = 1024, ["order"] = 8, ["angle"] = 1458503680},
  [66] = {["x"] = 4042, ["y"] = -18914, ["z"] = 960, ["order"] = 8, ["angle"] = 1685651456},
  [67] = {["x"] = 3486, ["y"] = -18489, ["z"] = 960, ["order"] = 8, ["angle"] = 1762459648},
  [68] = {["x"] = 2593, ["y"] = -18175, ["z"] = 960, ["order"] = 8, ["angle"] = 1675165696},
  [69] = {["x"] = 1919, ["y"] = -17762, ["z"] = 960, ["order"] = 9, ["angle"] = 1131216896},
  [70] = {["x"] = 1624, ["y"] = -16798, ["z"] = 960, ["order"] = 9, ["angle"] = 503513088},
  [71] = {["x"] = 2124, ["y"] = -16273, ["z"] = 960, ["order"] = 9, ["angle"] = 451870720},
  [72] = {["x"] = 3024, ["y"] = -15859, ["z"] = 960, ["order"] = 9, ["angle"] = 453050368},
  [73] = {["x"] = 3958, ["y"] = -15306, ["z"] = 960, ["order"] = 9, ["angle"] = 549912576},
  [74] = {["x"] = 4958, ["y"] = -14622, ["z"] = 960, ["order"] = 10, ["angle"] = 619511808},
  [75] = {["x"] = 5794, ["y"] = -13758, ["z"] = 960, ["order"] = 10, ["angle"] = 675741696},
  [76] = {["x"] = 6615, ["y"] = -12879, ["z"] = 960, ["order"] = 10, ["angle"] = 668008448},
  [77] = {["x"] = 7460, ["y"] = -11968, ["z"] = 960, ["order"] = 10, ["angle"] = 777322496},
  [78] = {["x"] = 8116, ["y"] = -11103, ["z"] = 976, ["order"] = 10, ["angle"] = 965410816},
  [79] = {["x"] = 8523, ["y"] = -10128, ["z"] = 1008, ["order"] = 11, ["angle"] = 1533607936},
  [80] = {["x"] = 8242, ["y"] = -9467, ["z"] = 1024, ["order"] = 11, ["angle"] = 1991442432},
  [81] = {["x"] = 7534, ["y"] = -9096, ["z"] = 1040, ["order"] = 11, ["angle"] = -1796276224},
  [82] = {["x"] = 6796, ["y"] = -9252, ["z"] = 1056, ["order"] = 12, ["angle"] = -1626931200},
  [83] = {["x"] = 5962, ["y"] = -9824, ["z"] = 1072, ["order"] = 12, ["angle"] = -1453391872},
  [84] = {["x"] = 5441, ["y"] = -10573, ["z"] = 1088, ["order"] = 12, ["angle"] = -1394802688},
  [85] = {["x"] = 5167, ["y"] = -11647, ["z"] = 1136, ["order"] = 13, ["angle"] = -1392050176},
  [86] = {["x"] = 4831, ["y"] = -12555, ["z"] = 1152, ["order"] = 13, ["angle"] = -1745027072},
  [87] = {["x"] = 4139, ["y"] = -13216, ["z"] = 1152, ["order"] = 13, ["angle"] = -1943732224},
  [88] = {["x"] = 3227, ["y"] = -13753, ["z"] = 1152, ["order"] = 13, ["angle"] = -2108227584},
  [89] = {["x"] = 2448, ["y"] = -13712, ["z"] = 1152, ["order"] = 13, ["angle"] = 1653014528},
  [90] = {["x"] = 1694, ["y"] = -13308, ["z"] = 1152, ["order"] = 13, ["angle"] = 1336606720},
  [91] = {["x"] = 1426, ["y"] = -12616, ["z"] = 1152, ["order"] = 13, ["angle"] = 1061093376},
  [92] = {["x"] = 1300, ["y"] = -11648, ["z"] = 1152, ["order"] = 13, ["angle"] = 1098186752},
  [93] = {["x"] = 1166, ["y"] = -10634, ["z"] = 1152, ["order"] = 13, ["angle"] = 917569536},
  [94] = {["x"] = 1359, ["y"] = -9705, ["z"] = 1152, ["order"] = 13, ["angle"] = 967770112},
  [95] = {["x"] = 1461, ["y"] = -8862, ["z"] = 1152, ["order"] = 13, ["angle"] = 1109721088},
  [96] = {["x"] = 1450, ["y"] = -7985, ["z"] = 1232, ["order"] = 14, ["angle"] = 1056505856},
  [97] = {["x"] = 1459, ["y"] = -7124, ["z"] = 1216, ["order"] = 14, ["angle"] = 1056505856},
  [98] = {["x"] = 1482, ["y"] = -6076, ["z"] = 1216, ["order"] = 14, ["angle"] = 1056505856},
  [99] = {["x"] = 1513, ["y"] = -4819, ["z"] = -32, ["order"] = 14, ["angle"] = 1056505856},
  [100] = {["x"] = 1524, ["y"] = -4028, ["z"] = 0, ["order"] = 14, ["angle"] = 1079967744},
  [101] = {["x"] = 1465, ["y"] = -3015, ["z"] = -32, ["order"] = 14, ["angle"] = 1056899072},
  [102] = {["x"] = 1286, ["y"] = -2114, ["z"] = 0, ["order"] = 15, ["angle"] = 730529792},
  [103] = {["x"] = 1555, ["y"] = -1244, ["z"] = -8, ["order"] = 15, ["angle"] = 183697408},
  [104] = {["x"] = 2456, ["y"] = -1063, ["z"] = 0, ["order"] = 15, ["angle"] = -8585216},
  [105] = {["x"] = 3180, ["y"] = -998, ["z"] = 0, ["order"] = 15, ["angle"] = -180944896},
  [106] = {["x"] = 4137, ["y"] = -992, ["z"] = 0, ["order"] = 15, ["angle"] = -594870272},
  [107] = {["x"] = 4665, ["y"] = -1446, ["z"] = 0, ["order"] = 15, ["angle"] = -1117192192},
  [108] = {["x"] = 4757, ["y"] = -2823, ["z"] = 0, ["order"] = 15, ["angle"] = -1088749568},
  [109] = {["x"] = 4742, ["y"] = -3747, ["z"] = 0, ["order"] = 15, ["angle"] = -1047330816},
  [110] = {["x"] = 4721, ["y"] = -4822, ["z"] = 0, ["order"] = 15, ["angle"] = -1060175872},
  [111] = {["x"] = 4673, ["y"] = -5845, ["z"] = 0, ["order"] = 15, ["angle"] = -58785792},
  [112] = {["x"] = 5273, ["y"] = -6023, ["z"] = 0, ["order"] = 15, ["angle"] = 22872064},
  [113] = {["x"] = 6050, ["y"] = -6022, ["z"] = 0, ["order"] = 15, ["angle"] = -3342336},
  [114] = {["x"] = 6821, ["y"] = -6018, ["z"] = 0, ["order"] = 15, ["angle"] = -3342336},
  [115] = {["x"] = 7443, ["y"] = -6031, ["z"] = 0, ["order"] = 15, ["angle"] = 900923392},
  [116] = {["x"] = 7610, ["y"] = -5318, ["z"] = 0, ["order"] = 15, ["angle"] = 692912128},
  [117] = {["x"] = 7993, ["y"] = -4650, ["z"] = 0, ["order"] = 15, ["angle"] = 337444864},
  [118] = {["x"] = 8641, ["y"] = -4321, ["z"] = 0, ["order"] = 15, ["angle"] = 15663104},
  [119] = {["x"] = 9704, ["y"] = -4208, ["z"] = 0, ["order"] = 16, ["angle"] = -42401792},
  [120] = {["x"] = 10725, ["y"] = -4249, ["z"] = 0, ["order"] = 16, ["angle"] = -49217536},
  [121] = {["x"] = 11876, ["y"] = -4310, ["z"] = 0, ["order"] = 16, ["angle"] = -11862016},
  [122] = {["x"] = 12847, ["y"] = -4330, ["z"] = 0, ["order"] = 16, ["angle"] = 8978432}
}
-- Respawn definitions for 3 COLOR DRIVE (MAPH5)
krespawn.points[357] = {
  [0] = 103,
  [1] = {["x"] = 1331, ["y"] = 2395, ["z"] = 0, ["order"] = 0, ["angle"] = 1060962304},
  [2] = {["x"] = 1379, ["y"] = 3144, ["z"] = 0, ["order"] = 0, ["angle"] = 1060962304},
  [3] = {["x"] = 1422, ["y"] = 4050, ["z"] = 46, ["order"] = 0, ["angle"] = 1103429632},
  [4] = {["x"] = 1455, ["y"] = 5115, ["z"] = 128, ["order"] = 0, ["angle"] = 1146028032},
  [5] = {["x"] = 1548, ["y"] = 6371, ["z"] = 106, ["order"] = 0, ["angle"] = 1160445952},
  [6] = {["x"] = 1633, ["y"] = 7454, ["z"] = 0, ["order"] = 0, ["angle"] = 1284440064},
  [7] = {["x"] = 1558, ["y"] = 8485, ["z"] = 0, ["order"] = 1, ["angle"] = 1531641856},
  [8] = {["x"] = 1219, ["y"] = 9416, ["z"] = 0, ["order"] = 1, ["angle"] = 1779367936},
  [9] = {["x"] = 488, ["y"] = 9998, ["z"] = 0, ["order"] = 1, ["angle"] = 2057109504},
  [10] = {["x"] = -348, ["y"] = 10140, ["z"] = 0, ["order"] = 2, ["angle"] = -2099314688},
  [11] = {["x"] = -1299, ["y"] = 10118, ["z"] = 0, ["order"] = 2, ["angle"] = -2127888384},
  [12] = {["x"] = -2173, ["y"] = 10131, ["z"] = 0, ["order"] = 2, ["angle"] = -2124087296},
  [13] = {["x"] = -3005, ["y"] = 10148, ["z"] = -125, ["order"] = 2, ["angle"] = 2122252288},
  [14] = {["x"] = -3671, ["y"] = 10159, ["z"] = 0, ["order"] = 2, ["angle"] = -2133655552},
  [15] = {["x"] = -4940, ["y"] = 10139, ["z"] = -127, ["order"] = 2, ["angle"] = -2133655552},
  [16] = {["x"] = -5626, ["y"] = 10149, ["z"] = 0, ["order"] = 2, ["angle"] = 2143223808},
  [17] = {["x"] = -6647, ["y"] = 9932, ["z"] = 0, ["order"] = 3, ["angle"] = 2145452032},
  [18] = {["x"] = -7540, ["y"] = 9700, ["z"] = 0, ["order"] = 3, ["angle"] = 1925906432},
  [19] = {["x"] = -8586, ["y"] = 9972, ["z"] = 0, ["order"] = 3, ["angle"] = 1419313152},
  [20] = {["x"] = -9234, ["y"] = 10668, ["z"] = 0, ["order"] = 3, ["angle"] = 1056636928},
  [21] = {["x"] = -9455, ["y"] = 11651, ["z"] = 0, ["order"] = 3, ["angle"] = 898826240},
  [22] = {["x"] = -9408, ["y"] = 12682, ["z"] = 0, ["order"] = 3, ["angle"] = 837353472},
  [23] = {["x"] = -9238, ["y"] = 13720, ["z"] = 0, ["order"] = 4, ["angle"] = 500891648},
  [24] = {["x"] = -8582, ["y"] = 14583, ["z"] = 0, ["order"] = 4, ["angle"] = 92340224},
  [25] = {["x"] = -7731, ["y"] = 14915, ["z"] = 0, ["order"] = 4, ["angle"] = 8192000},
  [26] = {["x"] = -6715, ["y"] = 14939, ["z"] = 0, ["order"] = 5, ["angle"] = 7405568},
  [27] = {["x"] = -5834, ["y"] = 14922, ["z"] = 0, ["order"] = 5, ["angle"] = 7405568},
  [28] = {["x"] = -5183, ["y"] = 14923, ["z"] = 0, ["order"] = 5, ["angle"] = 7405568},
  [29] = {["x"] = -4478, ["y"] = 14945, ["z"] = 0, ["order"] = 5, ["angle"] = 15269888},
  [30] = {["x"] = -3711, ["y"] = 14881, ["z"] = 0, ["order"] = 5, ["angle"] = 68747264},
  [31] = {["x"] = -2985, ["y"] = 14855, ["z"] = 0, ["order"] = 5, ["angle"] = 224591872},
  [32] = {["x"] = -2132, ["y"] = 14872, ["z"] = 0, ["order"] = 5, ["angle"] = 267321344},
  [33] = {["x"] = -1313, ["y"] = 15001, ["z"] = 0, ["order"] = 5, ["angle"] = 493289472},
  [34] = {["x"] = -546, ["y"] = 15355, ["z"] = 0, ["order"] = 5, ["angle"] = 647430144},
  [35] = {["x"] = -7, ["y"] = 15822, ["z"] = 0, ["order"] = 5, ["angle"] = 817299456},
  [36] = {["x"] = 350, ["y"] = 16476, ["z"] = 0, ["order"] = 5, ["angle"] = 868679680},
  [37] = {["x"] = 483, ["y"] = 17213, ["z"] = 0, ["order"] = 6, ["angle"] = 740884480},
  [38] = {["x"] = 629, ["y"] = 17961, ["z"] = 0, ["order"] = 6, ["angle"] = 427884544},
  [39] = {["x"] = 840, ["y"] = 18451, ["z"] = 0, ["order"] = 6, ["angle"] = 180289536},
  [40] = {["x"] = 1338, ["y"] = 18904, ["z"] = 0, ["order"] = 6, ["angle"] = -28114944},
  [41] = {["x"] = 2159, ["y"] = 19155, ["z"] = 0, ["order"] = 6, ["angle"] = -174260224},
  [42] = {["x"] = 3095, ["y"] = 19092, ["z"] = 0, ["order"] = 7, ["angle"] = -436797440},
  [43] = {["x"] = 3730, ["y"] = 18798, ["z"] = 0, ["order"] = 7, ["angle"] = -744030208},
  [44] = {["x"] = 4154, ["y"] = 18268, ["z"] = 0, ["order"] = 7, ["angle"] = -971177984},
  [45] = {["x"] = 4288, ["y"] = 17445, ["z"] = 0, ["order"] = 7, ["angle"] = -919797760},
  [46] = {["x"] = 4413, ["y"] = 16723, ["z"] = 0, ["order"] = 8, ["angle"] = -919797760},
  [47] = {["x"] = 4670, ["y"] = 15904, ["z"] = 0, ["order"] = 8, ["angle"] = -881655808},
  [48] = {["x"] = 5099, ["y"] = 14888, ["z"] = 0, ["order"] = 8, ["angle"] = -977338368},
  [49] = {["x"] = 5358, ["y"] = 13860, ["z"] = 0, ["order"] = 8, ["angle"] = -1075904512},
  [50] = {["x"] = 5436, ["y"] = 13103, ["z"] = 74, ["order"] = 8, ["angle"] = -1070399488},
  [51] = {["x"] = 5453, ["y"] = 12220, ["z"] = 128, ["order"] = 8, ["angle"] = -1070399488},
  [52] = {["x"] = 5404, ["y"] = 11182, ["z"] = 117, ["order"] = 8, ["angle"] = -1093992448},
  [53] = {["x"] = 5256, ["y"] = 9956, ["z"] = 4, ["order"] = 8, ["angle"] = -1119289344},
  [54] = {["x"] = 5117, ["y"] = 8675, ["z"] = 0, ["order"] = 8, ["angle"] = -734461952},
  [55] = {["x"] = 5273, ["y"] = 7630, ["z"] = 0, ["order"] = 9, ["angle"] = -292356096},
  [56] = {["x"] = 5962, ["y"] = 7169, ["z"] = 0, ["order"] = 9, ["angle"] = -11730944},
  [57] = {["x"] = 6860, ["y"] = 7196, ["z"] = 0, ["order"] = 9, ["angle"] = -24182784},
  [58] = {["x"] = 7764, ["y"] = 7323, ["z"] = 0, ["order"] = 9, ["angle"] = -22216704},
  [59] = {["x"] = 8734, ["y"] = 7436, ["z"] = 0, ["order"] = 10, ["angle"] = -176750592},
  [60] = {["x"] = 9503, ["y"] = 7374, ["z"] = 0, ["order"] = 10, ["angle"] = -573636608},
  [61] = {["x"] = 10024, ["y"] = 6831, ["z"] = 0, ["order"] = 10, ["angle"] = -929628160},
  [62] = {["x"] = 10229, ["y"] = 6035, ["z"] = 0, ["order"] = 10, ["angle"] = -1064501248},
  [63] = {["x"] = 10252, ["y"] = 4924, ["z"] = 0, ["order"] = 10, ["angle"] = -1273298944},
  [64] = {["x"] = 10021, ["y"] = 4071, ["z"] = 0, ["order"] = 10, ["angle"] = -1458372608},
  [65] = {["x"] = 9606, ["y"] = 3417, ["z"] = 0, ["order"] = 10, ["angle"] = -1334509568},
  [66] = {["x"] = 9442, ["y"] = 2898, ["z"] = 0, ["order"] = 10, ["angle"] = -776142848},
  [67] = {["x"] = 9740, ["y"] = 2287, ["z"] = 0, ["order"] = 10, ["angle"] = -649658368},
  [68] = {["x"] = 10212, ["y"] = 1322, ["z"] = 0, ["order"] = 10, ["angle"] = -789250048},
  [69] = {["x"] = 10609, ["y"] = 676, ["z"] = 0, ["order"] = 10, ["angle"] = -745865216},
  [70] = {["x"] = 10998, ["y"] = 24, ["z"] = 0, ["order"] = 10, ["angle"] = -1107886080},
  [71] = {["x"] = 10840, ["y"] = -698, ["z"] = 0, ["order"] = 10, ["angle"] = -1444741120},
  [72] = {["x"] = 10456, ["y"] = -1460, ["z"] = 0, ["order"] = 10, ["angle"] = -1163591680},
  [73] = {["x"] = 10299, ["y"] = -2398, ["z"] = 0, ["order"] = 10, ["angle"] = -1129119744},
  [74] = {["x"] = 10367, ["y"] = -3138, ["z"] = 0, ["order"] = 10, ["angle"] = -1228079104},
  [75] = {["x"] = 10409, ["y"] = -3906, ["z"] = 0, ["order"] = 11, ["angle"] = -1379336192},
  [76] = {["x"] = 10081, ["y"] = -4831, ["z"] = 0, ["order"] = 11, ["angle"] = -1969553408},
  [77] = {["x"] = 9286, ["y"] = -5129, ["z"] = 0, ["order"] = 11, ["angle"] = -2083848192},
  [78] = {["x"] = 8227, ["y"] = -5276, ["z"] = 0, ["order"] = 11, ["angle"] = -2044002304},
  [79] = {["x"] = 7048, ["y"] = -5449, ["z"] = 0, ["order"] = 11, ["angle"] = -2044002304},
  [80] = {["x"] = 5938, ["y"] = -5610, ["z"] = 0, ["order"] = 11, ["angle"] = -2145714176},
  [81] = {["x"] = 4674, ["y"] = -5724, ["z"] = 0, ["order"] = 11, ["angle"] = -2127888384},
  [82] = {["x"] = 3322, ["y"] = -5968, ["z"] = 0, ["order"] = 11, ["angle"] = 1881079808},
  [83] = {["x"] = 2305, ["y"] = -5845, ["z"] = 0, ["order"] = 12, ["angle"] = 1407254528},
  [84] = {["x"] = 1545, ["y"] = -5209, ["z"] = 0, ["order"] = 12, ["angle"] = 1057947648},
  [85] = {["x"] = 1345, ["y"] = -4324, ["z"] = 0, ["order"] = 12, ["angle"] = 995819520},
  [86] = {["x"] = 1363, ["y"] = -3249, ["z"] = 0, ["order"] = 12, ["angle"] = 1078657024},
  [87] = {["x"] = 1363, ["y"] = -1958, ["z"] = -128, ["order"] = 12, ["angle"] = 1078657024},
  [88] = {["x"] = 1355, ["y"] = -528, ["z"] = 0, ["order"] = 12, ["angle"] = 1078657024},
  [89] = {["x"] = 1348, ["y"] = 473, ["z"] = 0, ["order"] = 12, ["angle"] = 1078657024},
  [90] = {["x"] = 1342, ["y"] = 1457, ["z"] = 0, ["order"] = 12, ["angle"] = 1078657024},
  [91] = {["x"] = 1339, ["y"] = 1891, ["z"] = 0, ["order"] = 12, ["angle"] = 1078657024},
  [92] = {["x"] = 10356, ["y"] = 4213, ["z"] = 0, ["order"] = 10, ["angle"] = -780992512},
  [93] = {["x"] = 10734, ["y"] = 3622, ["z"] = 0, ["order"] = 10, ["angle"] = -747831296},
  [94] = {["x"] = 11168, ["y"] = 2956, ["z"] = 0, ["order"] = 10, ["angle"] = -1363345408},
  [95] = {["x"] = 10885, ["y"] = 2352, ["z"] = 0, ["order"] = 10, ["angle"] = -1543438336},
  [96] = {["x"] = 10424, ["y"] = 1384, ["z"] = 0, ["order"] = 10, ["angle"] = -1385103360},
  [97] = {["x"] = 9958, ["y"] = 786, ["z"] = 0, ["order"] = 10, ["angle"] = -1462829056},
  [98] = {["x"] = 9787, ["y"] = 1440, ["z"] = 0, ["order"] = 10, ["angle"] = -1170145280},
  [99] = {["x"] = 10666, ["y"] = 1370, ["z"] = 0, ["order"] = 10, ["angle"] = -1048248320},
  [100] = {["x"] = 9519, ["y"] = 242, ["z"] = 0, ["order"] = 10, ["angle"] = -1101594624},
  [101] = {["x"] = 9485, ["y"] = -495, ["z"] = 0, ["order"] = 10, ["angle"] = -509411328},
  [102] = {["x"] = 10004, ["y"] = -1359, ["z"] = 0, ["order"] = 10, ["angle"] = -955449344},
  [103] = {["x"] = 10152, ["y"] = -2190, ["z"] = 0, ["order"] = 10, ["angle"] = -1037369344}
}
-- Respawn definitions for CK CLOUD (MAPH6)
krespawn.points[358] = {
  [0] = 56,
  [1] = {["x"] = -3007, ["y"] = -1609, ["z"] = 1000, ["order"] = 0, ["angle"] = 41746432},
  [2] = {["x"] = -2341, ["y"] = -1667, ["z"] = 1000, ["order"] = 0, ["angle"] = 172425216},
  [3] = {["x"] = -1381, ["y"] = -1738, ["z"] = 1000, ["order"] = 0, ["angle"] = 405995520},
  [4] = {["x"] = -576, ["y"] = -1394, ["z"] = 1000, ["order"] = 0, ["angle"] = 884539392},
  [5] = {["x"] = -211, ["y"] = -681, ["z"] = 1000, ["order"] = 0, ["angle"] = 1134886912},
  [6] = {["x"] = -207, ["y"] = 2004, ["z"] = 1000, ["order"] = 1, ["angle"] = 1070399488},
  [7] = {["x"] = -188, ["y"] = 2572, ["z"] = 1000, ["order"] = 1, ["angle"] = 1070399488},
  [8] = {["x"] = -173, ["y"] = 3405, ["z"] = 1000, ["order"] = 1, ["angle"] = 1059258368},
  [9] = {["x"] = -159, ["y"] = 4041, ["z"] = 1000, ["order"] = 1, ["angle"] = 1059258368},
  [10] = {["x"] = -93, ["y"] = 6209, ["z"] = 1000, ["order"] = 1, ["angle"] = 976289792},
  [11] = {["x"] = -332, ["y"] = 6669, ["z"] = 1000, ["order"] = 1, ["angle"] = 930283520},
  [12] = {["x"] = -118, ["y"] = 7197, ["z"] = 1000, ["order"] = 1, ["angle"] = 328138752},
  [13] = {["x"] = 514, ["y"] = 7788, ["z"] = 1000, ["order"] = 1, ["angle"] = -234684416},
  [14] = {["x"] = 1103, ["y"] = 7757, ["z"] = 1000, ["order"] = 2, ["angle"] = -603783168},
  [15] = {["x"] = 1568, ["y"] = 7350, ["z"] = 1000, ["order"] = 2, ["angle"] = -774438912},
  [16] = {["x"] = 1888, ["y"] = 6719, ["z"] = 1000, ["order"] = 2, ["angle"] = -787546112},
  [17] = {["x"] = 2042, ["y"] = 5964, ["z"] = 1000, ["order"] = 2, ["angle"] = -731709440},
  [18] = {["x"] = 2181, ["y"] = 4975, ["z"] = 1000, ["order"] = 3, ["angle"] = -596312064},
  [19] = {["x"] = 2645, ["y"] = 4139, ["z"] = 1000, ["order"] = 3, ["angle"] = -212533248},
  [20] = {["x"] = 3354, ["y"] = 3744, ["z"] = 1000, ["order"] = 3, ["angle"] = 238354432},
  [21] = {["x"] = 4206, ["y"] = 3731, ["z"] = 1000, ["order"] = 3, ["angle"] = 729612288},
  [22] = {["x"] = 4829, ["y"] = 4224, ["z"] = 1000, ["order"] = 3, ["angle"] = 1143537664},
  [23] = {["x"] = 5070, ["y"] = 4957, ["z"] = 1000, ["order"] = 3, ["angle"] = 1724579840},
  [24] = {["x"] = 4615, ["y"] = 5380, ["z"] = 1000, ["order"] = 4, ["angle"] = -2087518208},
  [25] = {["x"] = -123, ["y"] = 5472, ["z"] = 1000, ["order"] = 4, ["angle"] = -2147418112},
  [26] = {["x"] = -729, ["y"] = 5463, ["z"] = 1000, ["order"] = 4, ["angle"] = -1955266560},
  [27] = {["x"] = -2245, ["y"] = 5741, ["z"] = 600, ["order"] = 4, ["angle"] = -1022951424},
  [28] = {["x"] = -2760, ["y"] = 5765, ["z"] = 600, ["order"] = 4, ["angle"] = -966066176},
  [29] = {["x"] = -3254, ["y"] = 5736, ["z"] = 600, ["order"] = 4, ["angle"] = -1106051072},
  [30] = {["x"] = -3809, ["y"] = 5744, ["z"] = 600, ["order"] = 4, ["angle"] = -875626496},
  [31] = {["x"] = -3535, ["y"] = 4830, ["z"] = 600, ["order"] = 4, ["angle"] = -890568704},
  [32] = {["x"] = -2893, ["y"] = 4788, ["z"] = 600, ["order"] = 4, ["angle"] = -929890304},
  [33] = {["x"] = -2418, ["y"] = 4796, ["z"] = 600, ["order"] = 4, ["angle"] = -949288960},
  [34] = {["x"] = -1989, ["y"] = 4499, ["z"] = 600, ["order"] = 4, ["angle"] = -1065418752},
  [35] = {["x"] = -2630, ["y"] = 4220, ["z"] = 600, ["order"] = 4, ["angle"] = -1176305664},
  [36] = {["x"] = -3296, ["y"] = 4293, ["z"] = 600, ["order"] = 4, ["angle"] = -865665024},
  [37] = {["x"] = -2819, ["y"] = 3551, ["z"] = 600, ["order"] = 4, ["angle"] = -872611840},
  [38] = {["x"] = -2739, ["y"] = 2595, ["z"] = 600, ["order"] = 4, ["angle"] = -753205248},
  [39] = {["x"] = -2520, ["y"] = 1803, ["z"] = 600, ["order"] = 5, ["angle"] = -255655936},
  [40] = {["x"] = -1902, ["y"] = 1223, ["z"] = 600, ["order"] = 5, ["angle"] = -122093568},
  [41] = {["x"] = -1172, ["y"] = 1107, ["z"] = 600, ["order"] = 5, ["angle"] = -137297920},
  [42] = {["x"] = -456, ["y"] = 1065, ["z"] = 600, ["order"] = 5, ["angle"] = -65339392},
  [43] = {["x"] = 277, ["y"] = 1157, ["z"] = 600, ["order"] = 5, ["angle"] = -175964160},
  [44] = {["x"] = 1120, ["y"] = 1272, ["z"] = 600, ["order"] = 6, ["angle"] = -363003904},
  [45] = {["x"] = 1983, ["y"] = 1188, ["z"] = 600, ["order"] = 6, ["angle"] = -658309120},
  [46] = {["x"] = 2573, ["y"] = 882, ["z"] = 600, ["order"] = 6, ["angle"] = -1072627712},
  [47] = {["x"] = 2955, ["y"] = 136, ["z"] = 600, ["order"] = 6, ["angle"] = -1551826944},
  [48] = {["x"] = 2806, ["y"] = -641, ["z"] = 600, ["order"] = 7, ["angle"] = -2014248960},
  [49] = {["x"] = 1848, ["y"] = -1238, ["z"] = 600, ["order"] = 7, ["angle"] = 2120941568},
  [50] = {["x"] = -1009, ["y"] = -1408, ["z"] = 600, ["order"] = 7, ["angle"] = -2138243072},
  [51] = {["x"] = -2151, ["y"] = -1437, ["z"] = 600, ["order"] = 7, ["angle"] = -2138243072},
  [52] = {["x"] = -3242, ["y"] = -1456, ["z"] = 600, ["order"] = 7, ["angle"] = -2138243072},
  [53] = {["x"] = -4239, ["y"] = -1471, ["z"] = 600, ["order"] = 7, ["angle"] = -2138243072},
  [54] = {["x"] = -3849, ["y"] = -1472, ["z"] = 1000, ["order"] = 8, ["angle"] = 65536},
  [55] = {["x"] = -3294, ["y"] = -1472, ["z"] = 1000, ["order"] = 8, ["angle"] = 65536},
  [56] = {["x"] = -237, ["y"] = 1665, ["z"] = 1004, ["order"] = 0, ["angle"] = 1027538944}
}
-- Respawn definitions for CK DUNGEON (MAPH7)
krespawn.points[359] = {
  [0] = 61,
  [1] = {["x"] = -4390, ["y"] = -4673, ["z"] = 0, ["order"] = 0, ["angle"] = 79757312},
  [2] = {["x"] = -3066, ["y"] = -4831, ["z"] = 0, ["order"] = 0, ["angle"] = 132579328},
  [3] = {["x"] = -2956, ["y"] = -4612, ["z"] = 0, ["order"] = 0, ["angle"] = 193265664},
  [4] = {["x"] = -1744, ["y"] = -4960, ["z"] = 0, ["order"] = 0, ["angle"] = 305856512},
  [5] = {["x"] = -608, ["y"] = -4684, ["z"] = 0, ["order"] = 1, ["angle"] = 852819968},
  [6] = {["x"] = -74, ["y"] = -3821, ["z"] = 0, ["order"] = 1, ["angle"] = 1518141440},
  [7] = {["x"] = -326, ["y"] = -2860, ["z"] = 0, ["order"] = 1, ["angle"] = -2078343168},
  [8] = {["x"] = -1159, ["y"] = -2874, ["z"] = 0, ["order"] = 1, ["angle"] = 2080702464},
  [9] = {["x"] = -2082, ["y"] = -3071, ["z"] = 0, ["order"] = 2, ["angle"] = 1857355776},
  [10] = {["x"] = -3174, ["y"] = -2846, ["z"] = 0, ["order"] = 2, ["angle"] = 1058734080},
  [11] = {["x"] = -3454, ["y"] = -2021, ["z"] = 0, ["order"] = 2, ["angle"] = 473235456},
  [12] = {["x"] = -2938, ["y"] = -1535, ["z"] = 0, ["order"] = 2, ["angle"] = 64684032},
  [13] = {["x"] = -1881, ["y"] = -1513, ["z"] = 82, ["order"] = 3, ["angle"] = 221052928},
  [14] = {["x"] = -923, ["y"] = -1574, ["z"] = 110, ["order"] = 3, ["angle"] = 594608128},
  [15] = {["x"] = -173, ["y"] = -1132, ["z"] = 110, ["order"] = 3, ["angle"] = 1143799808},
  [16] = {["x"] = 19, ["y"] = -307, ["z"] = 110, ["order"] = 3, ["angle"] = 1103822848},
  [17] = {["x"] = -408, ["y"] = 698, ["z"] = 0, ["order"] = 4, ["angle"] = 1102249984},
  [18] = {["x"] = 19, ["y"] = 743, ["z"] = 0, ["order"] = 4, ["angle"] = 1106051072},
  [19] = {["x"] = -70, ["y"] = 1725, ["z"] = 0, ["order"] = 4, ["angle"] = 1283653632},
  [20] = {["x"] = 44, ["y"] = 2724, ["z"] = 0, ["order"] = 5, ["angle"] = 1536884736},
  [21] = {["x"] = -356, ["y"] = 3595, ["z"] = 0, ["order"] = 5, ["angle"] = 1868103680},
  [22] = {["x"] = -1009, ["y"] = 4010, ["z"] = 0, ["order"] = 5, ["angle"] = -2111635456},
  [23] = {["x"] = -1799, ["y"] = 4088, ["z"] = 0, ["order"] = 5, ["angle"] = -1763246080},
  [24] = {["x"] = -2598, ["y"] = 3821, ["z"] = 0, ["order"] = 6, ["angle"] = -1286668288},
  [25] = {["x"] = -2794, ["y"] = 3203, ["z"] = 0, ["order"] = 6, ["angle"] = -1293615104},
  [26] = {["x"] = -2852, ["y"] = 2465, ["z"] = 0, ["order"] = 6, ["angle"] = -1632043008},
  [27] = {["x"] = -3179, ["y"] = 1850, ["z"] = 0, ["order"] = 6, ["angle"] = -1867448320},
  [28] = {["x"] = -3769, ["y"] = 1618, ["z"] = 0, ["order"] = 7, ["angle"] = -2103508992},
  [29] = {["x"] = -4611, ["y"] = 1701, ["z"] = 0, ["order"] = 7, ["angle"] = -2078474240},
  [30] = {["x"] = -5188, ["y"] = 1671, ["z"] = 0, ["order"] = 7, ["angle"] = -1738473472},
  [31] = {["x"] = -5955, ["y"] = 1600, ["z"] = 0, ["order"] = 7, ["angle"] = -1415380992},
  [32] = {["x"] = -6541, ["y"] = 1316, ["z"] = 0, ["order"] = 7, ["angle"] = -1153236992},
  [33] = {["x"] = -6868, ["y"] = 886, ["z"] = 0, ["order"] = 7, ["angle"] = -1062928384},
  [34] = {["x"] = -6944, ["y"] = 2728, ["z"] = 884, ["order"] = 8, ["angle"] = 796983296},
  [35] = {["x"] = -6722, ["y"] = 2566, ["z"] = 850, ["order"] = 8, ["angle"] = 905904128},
  [36] = {["x"] = -6388, ["y"] = 2444, ["z"] = 850, ["order"] = 8, ["angle"] = 864223232},
  [37] = {["x"] = -5916, ["y"] = 2421, ["z"] = 850, ["order"] = 8, ["angle"] = 990183424},
  [38] = {["x"] = -6140, ["y"] = 294, ["z"] = 0, ["order"] = 8, ["angle"] = -1060831232},
  [39] = {["x"] = -6955, ["y"] = 337, ["z"] = 0, ["order"] = 8, ["angle"] = -937885696},
  [40] = {["x"] = -6847, ["y"] = 3225, ["z"] = 850, ["order"] = 8, ["angle"] = 335478784},
  [41] = {["x"] = -6289, ["y"] = 3636, ["z"] = 850, ["order"] = 8, ["angle"] = -138477568},
  [42] = {["x"] = -5668, ["y"] = 3725, ["z"] = 850, ["order"] = 8, ["angle"] = -585302016},
  [43] = {["x"] = -5007, ["y"] = 3452, ["z"] = 850, ["order"] = 8, ["angle"] = -1005256704},
  [44] = {["x"] = -4799, ["y"] = 3002, ["z"] = 850, ["order"] = 8, ["angle"] = -1005256704},
  [45] = {["x"] = -4743, ["y"] = 2177, ["z"] = 850, ["order"] = 9, ["angle"] = -1078788096},
  [46] = {["x"] = -4564, ["y"] = 1348, ["z"] = 850, ["order"] = 9, ["angle"] = -1238695936},
  [47] = {["x"] = -4619, ["y"] = 539, ["z"] = 850, ["order"] = 9, ["angle"] = -1405288448},
  [48] = {["x"] = -4830, ["y"] = 4, ["z"] = 850, ["order"] = 9, ["angle"] = -1492189184},
  [49] = {["x"] = -5327, ["y"] = -925, ["z"] = 0, ["order"] = 10, ["angle"] = -1398472704},
  [50] = {["x"] = -5538, ["y"] = -1326, ["z"] = 0, ["order"] = 10, ["angle"] = -1102905344},
  [51] = {["x"] = -6239, ["y"] = -945, ["z"] = 0, ["order"] = 10, ["angle"] = -1208156160},
  [52] = {["x"] = -6255, ["y"] = -1508, ["z"] = 0, ["order"] = 10, ["angle"] = -1019936768},
  [53] = {["x"] = -5845, ["y"] = -1949, ["z"] = 0, ["order"] = 10, ["angle"] = -1313275904},
  [54] = {["x"] = -6531, ["y"] = -2502, ["z"] = 0, ["order"] = 10, ["angle"] = -869203968},
  [55] = {["x"] = -7271, ["y"] = -2649, ["z"] = 0, ["order"] = 10, ["angle"] = -734593024},
  [56] = {["x"] = -7399, ["y"] = -3472, ["z"] = 0, ["order"] = 10, ["angle"] = -327221248},
  [57] = {["x"] = -7094, ["y"] = -3832, ["z"] = 0, ["order"] = 11, ["angle"] = -424869888},
  [58] = {["x"] = -6580, ["y"] = -4274, ["z"] = 0, ["order"] = 11, ["angle"] = 11862016},
  [59] = {["x"] = -6049, ["y"] = -4374, ["z"] = 0, ["order"] = 11, ["angle"] = 37027840},
  [60] = {["x"] = -5534, ["y"] = -4499, ["z"] = 0, ["order"] = 11, ["angle"] = -151584768},
  [61] = {["x"] = -4734, ["y"] = -4648, ["z"] = 0, ["order"] = 11, ["angle"] = 28508160}
}
-- Respawn definitions for CK DEATH (MAPH8)
krespawn.points[360] = {
  [0] = 91,
  [1] = {["x"] = -1962, ["y"] = -1834, ["z"] = 2000, ["order"] = 0, ["angle"] = 1076953088},
  [2] = {["x"] = -2056, ["y"] = -1044, ["z"] = 2024, ["order"] = 0, ["angle"] = 990445568},
  [3] = {["x"] = -2199, ["y"] = -212, ["z"] = 2135, ["order"] = 0, ["angle"] = 924516352},
  [4] = {["x"] = -2223, ["y"] = 787, ["z"] = 2200, ["order"] = 1, ["angle"] = 572719104},
  [5] = {["x"] = -1734, ["y"] = 1539, ["z"] = 2200, ["order"] = 1, ["angle"] = -42139648},
  [6] = {["x"] = -1092, ["y"] = 1657, ["z"] = 2200, ["order"] = 1, ["angle"] = -99155968},
  [7] = {["x"] = -316, ["y"] = 1625, ["z"] = 2200, ["order"] = 1, ["angle"] = -71368704},
  [8] = {["x"] = 509, ["y"] = 1710, ["z"] = 2200, ["order"] = 2, ["angle"] = -246874112},
  [9] = {["x"] = 1029, ["y"] = 1642, ["z"] = 2200, ["order"] = 2, ["angle"] = -976158720},
  [10] = {["x"] = 1251, ["y"] = 1074, ["z"] = 2200, ["order"] = 2, ["angle"] = -1160183808},
  [11] = {["x"] = 1251, ["y"] = 526, ["z"] = 2200, ["order"] = 2, ["angle"] = -1113260032},
  [12] = {["x"] = 1285, ["y"] = -7, ["z"] = 2150, ["order"] = 3, ["angle"] = -1311571968},
  [13] = {["x"] = 1299, ["y"] = -864, ["z"] = 2150, ["order"] = 3, ["angle"] = -1450770432},
  [14] = {["x"] = 1147, ["y"] = -1629, ["z"] = 2150, ["order"] = 3, ["angle"] = -1782120448},
  [15] = {["x"] = 627, ["y"] = -2225, ["z"] = 2150, ["order"] = 3, ["angle"] = -2105212928},
  [16] = {["x"] = -174, ["y"] = -2456, ["z"] = 2191, ["order"] = 4, ["angle"] = -2144796672},
  [17] = {["x"] = -898, ["y"] = -2543, ["z"] = 2298, ["order"] = 4, ["angle"] = 1988820992},
  [18] = {["x"] = -1705, ["y"] = -2583, ["z"] = 2300, ["order"] = 4, ["angle"] = 1918042112},
  [19] = {["x"] = -2733, ["y"] = -2398, ["z"] = 2318, ["order"] = 5, ["angle"] = 1540030464},
  [20] = {["x"] = -3141, ["y"] = -1985, ["z"] = 2378, ["order"] = 5, ["angle"] = 1400963072},
  [21] = {["x"] = -3241, ["y"] = -1003, ["z"] = 2470, ["order"] = 5, ["angle"] = 1759576064},
  [22] = {["x"] = -3680, ["y"] = -529, ["z"] = 2470, ["order"] = 5, ["angle"] = 1991704576},
  [23] = {["x"] = -4499, ["y"] = -271, ["z"] = 2470, ["order"] = 5, ["angle"] = 2065760256},
  [24] = {["x"] = -5291, ["y"] = -327, ["z"] = 2470, ["order"] = 6, ["angle"] = 1906638848},
  [25] = {["x"] = -6388, ["y"] = -323, ["z"] = 2470, ["order"] = 6, ["angle"] = 1101070336},
  [26] = {["x"] = -6675, ["y"] = 649, ["z"] = 2470, ["order"] = 7, ["angle"] = 545325056},
  [27] = {["x"] = -6308, ["y"] = 1369, ["z"] = 2470, ["order"] = 7, ["angle"] = 37814272},
  [28] = {["x"] = -5512, ["y"] = 1339, ["z"] = 2470, ["order"] = 7, ["angle"] = 21823488},
  [29] = {["x"] = -4576, ["y"] = 1333, ["z"] = 2470, ["order"] = 8, ["angle"] = 21823488},
  [30] = {["x"] = -3717, ["y"] = 1355, ["z"] = 2470, ["order"] = 8, ["angle"] = 94044160},
  [31] = {["x"] = -2699, ["y"] = 1441, ["z"] = 2470, ["order"] = 8, ["angle"] = 48431104},
  [32] = {["x"] = -1683, ["y"] = 1560, ["z"] = 2470, ["order"] = 8, ["angle"] = 17367040},
  [33] = {["x"] = -732, ["y"] = 1624, ["z"] = 2470, ["order"] = 8, ["angle"] = -50790400},
  [34] = {["x"] = 125, ["y"] = 1593, ["z"] = 2470, ["order"] = 9, ["angle"] = -6619136},
  [35] = {["x"] = 1104, ["y"] = 1559, ["z"] = 2470, ["order"] = 9, ["angle"] = 35323904},
  [36] = {["x"] = 2046, ["y"] = 1584, ["z"] = 2470, ["order"] = 9, ["angle"] = 35323904},
  [37] = {["x"] = 2563, ["y"] = 1429, ["z"] = 2470, ["order"] = 9, ["angle"] = 213843968},
  [38] = {["x"] = 3302, ["y"] = 1380, ["z"] = 2470, ["order"] = 9, ["angle"] = 591986688},
  [39] = {["x"] = 3936, ["y"] = 1993, ["z"] = 2470, ["order"] = 9, ["angle"] = 933560320},
  [40] = {["x"] = 4195, ["y"] = 2791, ["z"] = 2470, ["order"] = 10, ["angle"] = 1147863040},
  [41] = {["x"] = 4263, ["y"] = 3530, ["z"] = 2470, ["order"] = 10, ["angle"] = 1431240704},
  [42] = {["x"] = 4179, ["y"] = 4313, ["z"] = 2470, ["order"] = 10, ["angle"] = 1718943744},
  [43] = {["x"] = 3588, ["y"] = 4813, ["z"] = 2470, ["order"] = 10, ["angle"] = -2113732608},
  [44] = {["x"] = 2959, ["y"] = 4881, ["z"] = 2470, ["order"] = 11, ["angle"] = -2113732608},
  [45] = {["x"] = 2209, ["y"] = 4864, ["z"] = 2470, ["order"] = 11, ["angle"] = -2113732608},
  [46] = {["x"] = 1510, ["y"] = 4876, ["z"] = 2470, ["order"] = 11, ["angle"] = 2140864512},
  [47] = {["x"] = 638, ["y"] = 4867, ["z"] = 2470, ["order"] = 11, ["angle"] = -1834418176},
  [48] = {["x"] = -89, ["y"] = 4629, ["z"] = 2470, ["order"] = 11, ["angle"] = -1577385984},
  [49] = {["x"] = -588, ["y"] = 4065, ["z"] = 2470, ["order"] = 11, ["angle"] = -1967587328},
  [50] = {["x"] = -1163, ["y"] = 3367, ["z"] = 2470, ["order"] = 11, ["angle"] = 1987772416},
  [51] = {["x"] = -1903, ["y"] = 3319, ["z"] = 2470, ["order"] = 12, ["angle"] = 1559298048},
  [52] = {["x"] = -2360, ["y"] = 3676, ["z"] = 2470, ["order"] = 12, ["angle"] = 1229258752},
  [53] = {["x"] = -2767, ["y"] = 4809, ["z"] = 2470, ["order"] = 12, ["angle"] = 1047592960},
  [54] = {["x"] = -2807, ["y"] = 6189, ["z"] = 1470, ["order"] = 12, ["angle"] = 1047592960},
  [55] = {["x"] = -2794, ["y"] = 6885, ["z"] = 1470, ["order"] = 12, ["angle"] = 976551936},
  [56] = {["x"] = -2975, ["y"] = 7851, ["z"] = 1470, ["order"] = 12, ["angle"] = 833945600},
  [57] = {["x"] = -2939, ["y"] = 8835, ["z"] = 1470, ["order"] = 13, ["angle"] = 494338048},
  [58] = {["x"] = -2600, ["y"] = 9586, ["z"] = 1470, ["order"] = 13, ["angle"] = 669712384},
  [59] = {["x"] = -1834, ["y"] = 10168, ["z"] = 1470, ["order"] = 13, ["angle"] = 3080192},
  [60] = {["x"] = 2498, ["y"] = 10442, ["z"] = 2200, ["order"] = 14, ["angle"] = -48037888},
  [61] = {["x"] = 3022, ["y"] = 10617, ["z"] = 2200, ["order"] = 14, ["angle"] = -135069696},
  [62] = {["x"] = 3951, ["y"] = 10794, ["z"] = 2200, ["order"] = 14, ["angle"] = -438501376},
  [63] = {["x"] = 4578, ["y"] = 10375, ["z"] = 2200, ["order"] = 14, ["angle"] = -826998784},
  [64] = {["x"] = 5045, ["y"] = 9590, ["z"] = 2200, ["order"] = 15, ["angle"] = -956628992},
  [65] = {["x"] = 5267, ["y"] = 8756, ["z"] = 2200, ["order"] = 15, ["angle"] = -1113653248},
  [66] = {["x"] = 5451, ["y"] = 7741, ["z"] = 2200, ["order"] = 15, ["angle"] = -1388642304},
  [67] = {["x"] = 5269, ["y"] = 6799, ["z"] = 2259, ["order"] = 15, ["angle"] = -1883045888},
  [68] = {["x"] = 4515, ["y"] = 6176, ["z"] = 2468, ["order"] = 15, ["angle"] = 1806761984},
  [69] = {["x"] = 3681, ["y"] = 6359, ["z"] = 2517, ["order"] = 15, ["angle"] = 1317076992},
  [70] = {["x"] = 3451, ["y"] = 6876, ["z"] = 2433, ["order"] = 15, ["angle"] = 1268449280},
  [71] = {["x"] = 3456, ["y"] = 7648, ["z"] = 2400, ["order"] = 17, ["angle"] = 1282080768},
  [72] = {["x"] = 3247, ["y"] = 8589, ["z"] = 2400, ["order"] = 17, ["angle"] = 1806237696},
  [73] = {["x"] = 2739, ["y"] = 9010, ["z"] = 2400, ["order"] = 17, ["angle"] = -1979121664},
  [74] = {["x"] = 2086, ["y"] = 8838, ["z"] = 2400, ["order"] = 17, ["angle"] = -1863778304},
  [75] = {["x"] = 1538, ["y"] = 8724, ["z"] = 2400, ["order"] = 17, ["angle"] = 2060648448},
  [76] = {["x"] = 624, ["y"] = 9064, ["z"] = 2400, ["order"] = 18, ["angle"] = -1797455872},
  [77] = {["x"] = -111, ["y"] = 8770, ["z"] = 2400, ["order"] = 18, ["angle"] = -1166475264},
  [78] = {["x"] = -173, ["y"] = 8197, ["z"] = 2400, ["order"] = 18, ["angle"] = -1003814912},
  [79] = {["x"] = -174, ["y"] = 7537, ["z"] = 2400, ["order"] = 18, ["angle"] = -1060438016},
  [80] = {["x"] = -174, ["y"] = 6680, ["z"] = 2400, ["order"] = 19, ["angle"] = -1060438016},
  [81] = {["x"] = -160, ["y"] = 753, ["z"] = 2470, ["order"] = 20, ["angle"] = -1060438016},
  [82] = {["x"] = -156, ["y"] = 142, ["z"] = 2470, ["order"] = 20, ["angle"] = -1060438016},
  [83] = {["x"] = -137, ["y"] = -899, ["z"] = 2470, ["order"] = 20, ["angle"] = -1060438016},
  [84] = {["x"] = -151, ["y"] = -3027, ["z"] = 2000, ["order"] = 20, ["angle"] = -1247608832},
  [85] = {["x"] = 170, ["y"] = -3606, ["z"] = 2000, ["order"] = 20, ["angle"] = -1694695424},
  [86] = {["x"] = -222, ["y"] = -4332, ["z"] = 2000, ["order"] = 21, ["angle"] = -1995898880},
  [87] = {["x"] = -960, ["y"] = -4666, ["z"] = 2000, ["order"] = 21, ["angle"] = 1693253632},
  [88] = {["x"] = -1656, ["y"] = -4404, ["z"] = 2000, ["order"] = 21, ["angle"] = 1168703488},
  [89] = {["x"] = -1794, ["y"] = -3626, ["z"] = 2000, ["order"] = 21, ["angle"] = 1070530560},
  [90] = {["x"] = -1810, ["y"] = -2988, ["z"] = 2000, ["order"] = 21, ["angle"] = 1161887744},
  [91] = {["x"] = -1861, ["y"] = -2343, ["z"] = 2000, ["order"] = 21, ["angle"] = 1161887744}
}
-- Respawn definitions for PWR RETRO (MAPH9)
krespawn.points[361] = {
  [0] = 106,
  [1] = {["x"] = 3064, ["y"] = -5130, ["z"] = 0, ["order"] = 0, ["angle"] = 43974656},
  [2] = {["x"] = 3931, ["y"] = -5295, ["z"] = 0, ["order"] = 0, ["angle"] = 284360704},
  [3] = {["x"] = 4717, ["y"] = -4960, ["z"] = 0, ["order"] = 0, ["angle"] = 1019150336},
  [4] = {["x"] = 4860, ["y"] = -4100, ["z"] = 0, ["order"] = 1, ["angle"] = 1912274944},
  [5] = {["x"] = 4304, ["y"] = -3955, ["z"] = 0, ["order"] = 1, ["angle"] = 1982660608},
  [6] = {["x"] = 3409, ["y"] = -3914, ["z"] = 0, ["order"] = 1, ["angle"] = 1307770880},
  [7] = {["x"] = 3144, ["y"] = -3418, ["z"] = 0, ["order"] = 1, ["angle"] = 1023213568},
  [8] = {["x"] = 3112, ["y"] = -2663, ["z"] = 0, ["order"] = 1, ["angle"] = 1073676288},
  [9] = {["x"] = 3113, ["y"] = -1764, ["z"] = 0, ["order"] = 1, ["angle"] = 1075249152},
  [10] = {["x"] = 3115, ["y"] = -777, ["z"] = 0, ["order"] = 1, ["angle"] = 1075249152},
  [11] = {["x"] = 3115, ["y"] = 123, ["z"] = 0, ["order"] = 2, ["angle"] = 1075249152},
  [12] = {["x"] = 3114, ["y"] = 1215, ["z"] = 0, ["order"] = 2, ["angle"] = 1075249152},
  [13] = {["x"] = 3114, ["y"] = 2087, ["z"] = 0, ["order"] = 2, ["angle"] = 980484096},
  [14] = {["x"] = 3099, ["y"] = 3014, ["z"] = 0, ["order"] = 2, ["angle"] = 440991744},
  [15] = {["x"] = 3829, ["y"] = 3330, ["z"] = 0, ["order"] = 2, ["angle"] = 472842240},
  [16] = {["x"] = 4387, ["y"] = 3705, ["z"] = 0, ["order"] = 2, ["angle"] = 943390720},
  [17] = {["x"] = 4596, ["y"] = 4436, ["z"] = 0, ["order"] = 3, ["angle"] = 1961164800},
  [18] = {["x"] = 4176, ["y"] = 4762, ["z"] = 0, ["order"] = 3, ["angle"] = 2114650112},
  [19] = {["x"] = 3452, ["y"] = 4864, ["z"] = 0, ["order"] = 3, ["angle"] = -2106785792},
  [20] = {["x"] = 2675, ["y"] = 4911, ["z"] = 0, ["order"] = 3, ["angle"] = 2097479680},
  [21] = {["x"] = 1508, ["y"] = 5012, ["z"] = 0, ["order"] = 3, ["angle"] = -1756561408},
  [22] = {["x"] = 845, ["y"] = 4307, ["z"] = 0, ["order"] = 3, ["angle"] = -1720516608},
  [23] = {["x"] = -6, ["y"] = 3736, ["z"] = 0, ["order"] = 4, ["angle"] = 1638596608},
  [24] = {["x"] = -476, ["y"] = 4434, ["z"] = 0, ["order"] = 4, ["angle"] = 1910177792},
  [25] = {["x"] = -996, ["y"] = 4742, ["z"] = 0, ["order"] = 4, ["angle"] = 2019885056},
  [26] = {["x"] = -1795, ["y"] = 4899, ["z"] = 0, ["order"] = 4, ["angle"] = -2074935296},
  [27] = {["x"] = -2767, ["y"] = 5004, ["z"] = 0, ["order"] = 4, ["angle"] = -1998520320},
  [28] = {["x"] = -3728, ["y"] = 5030, ["z"] = 0, ["order"] = 4, ["angle"] = -1683161088},
  [29] = {["x"] = -4375, ["y"] = 4688, ["z"] = 0, ["order"] = 4, ["angle"] = -1179975680},
  [30] = {["x"] = -4551, ["y"] = 3930, ["z"] = 0, ["order"] = 5, ["angle"] = -468647936},
  [31] = {["x"] = -4135, ["y"] = 3670, ["z"] = 0, ["order"] = 5, ["angle"] = -112263168},
  [32] = {["x"] = -3260, ["y"] = 3578, ["z"] = 0, ["order"] = 5, ["angle"] = -984023040},
  [33] = {["x"] = -3158, ["y"] = 3044, ["z"] = 0, ["order"] = 5, ["angle"] = -1113522176},
  [34] = {["x"] = -3171, ["y"] = 2410, ["z"] = 0, ["order"] = 5, ["angle"] = -1113522176},
  [35] = {["x"] = -3190, ["y"] = 1560, ["z"] = 0, ["order"] = 5, ["angle"] = -1111293952},
  [36] = {["x"] = -3205, ["y"] = 750, ["z"] = 0, ["order"] = 5, ["angle"] = -1073545216},
  [37] = {["x"] = -3213, ["y"] = -94, ["z"] = 0, ["order"] = 5, ["angle"] = -1073545216},
  [38] = {["x"] = -3216, ["y"] = -1057, ["z"] = 0, ["order"] = 6, ["angle"] = -1073545216},
  [39] = {["x"] = -3195, ["y"] = -1954, ["z"] = 0, ["order"] = 6, ["angle"] = -1001193472},
  [40] = {["x"] = -3073, ["y"] = -2813, ["z"] = 0, ["order"] = 6, ["angle"] = -1257439232},
  [41] = {["x"] = -3154, ["y"] = -3557, ["z"] = 0, ["order"] = 6, ["angle"] = -1948188672},
  [42] = {["x"] = -3797, ["y"] = -3663, ["z"] = 0, ["order"] = 6, ["angle"] = -1935736832},
  [43] = {["x"] = -4537, ["y"] = -3715, ["z"] = 0, ["order"] = 6, ["angle"] = -1273298944},
  [44] = {["x"] = -4930, ["y"] = -4288, ["z"] = 0, ["order"] = 6, ["angle"] = -839974912},
  [45] = {["x"] = -4771, ["y"] = -4947, ["z"] = 0, ["order"] = 6, ["angle"] = -12386304},
  [46] = {["x"] = -4113, ["y"] = -5157, ["z"] = 0, ["order"] = 6, ["angle"] = 32702464},
  [47] = {["x"] = -3349, ["y"] = -5201, ["z"] = 0, ["order"] = 6, ["angle"] = 32702464},
  [48] = {["x"] = -2445, ["y"] = -5296, ["z"] = 0, ["order"] = 7, ["angle"] = 204144640},
  [49] = {["x"] = -1575, ["y"] = -5270, ["z"] = 0, ["order"] = 7, ["angle"] = 855048192},
  [50] = {["x"] = -1265, ["y"] = -4603, ["z"] = 0, ["order"] = 7, ["angle"] = 1434124288},
  [51] = {["x"] = -1968, ["y"] = -3736, ["z"] = 0, ["order"] = 7, ["angle"] = 1167654912},
  [52] = {["x"] = -1966, ["y"] = -3195, ["z"] = 0, ["order"] = 7, ["angle"] = 700252160},
  [53] = {["x"] = -1518, ["y"] = -2628, ["z"] = 0, ["order"] = 7, ["angle"] = 922681344},
  [54] = {["x"] = -1273, ["y"] = -2021, ["z"] = 0, ["order"] = 7, ["angle"] = 1473445888},
  [55] = {["x"] = -1767, ["y"] = -1311, ["z"] = 0, ["order"] = 7, ["angle"] = 1109590016},
  [56] = {["x"] = -1995, ["y"] = -695, ["z"] = 0, ["order"] = 7, ["angle"] = 765657088},
  [57] = {["x"] = -1811, ["y"] = -54, ["z"] = 0, ["order"] = 7, ["angle"] = -22347776},
  [58] = {["x"] = -1347, ["y"] = 18, ["z"] = 0, ["order"] = 7, ["angle"] = 30212096},
  [59] = {["x"] = -623, ["y"] = 65, ["z"] = 0, ["order"] = 7, ["angle"] = 30212096},
  [60] = {["x"] = -48, ["y"] = 117, ["z"] = 0, ["order"] = 8, ["angle"] = 132710400},
  [61] = {["x"] = 863, ["y"] = 236, ["z"] = 0, ["order"] = 8, ["angle"] = -271384576},
  [62] = {["x"] = 1627, ["y"] = 91, ["z"] = 0, ["order"] = 8, ["angle"] = -849281024},
  [63] = {["x"] = 1862, ["y"] = -281, ["z"] = 0, ["order"] = 8, ["angle"] = -1153499136},
  [64] = {["x"] = 1792, ["y"] = -863, ["z"] = 0, ["order"] = 8, ["angle"] = -1634140160},
  [65] = {["x"] = 1372, ["y"] = -1406, ["z"] = 0, ["order"] = 8, ["angle"] = -1138032640},
  [66] = {["x"] = 1371, ["y"] = -2007, ["z"] = 0, ["order"] = 8, ["angle"] = -730267648},
  [67] = {["x"] = 1796, ["y"] = -2627, ["z"] = 0, ["order"] = 8, ["angle"] = -1051000832},
  [68] = {["x"] = 1879, ["y"] = -3300, ["z"] = 0, ["order"] = 8, ["angle"] = -1555234816},
  [69] = {["x"] = 1404, ["y"] = -3964, ["z"] = 0, ["order"] = 8, ["angle"] = -1181679616},
  [70] = {["x"] = 1230, ["y"] = -4637, ["z"] = 0, ["order"] = 8, ["angle"] = -458555392},
  [71] = {["x"] = 1701, ["y"] = -5071, ["z"] = 0, ["order"] = 8, ["angle"] = 28114944},
  [72] = {["x"] = 2447, ["y"] = -5162, ["z"] = 0, ["order"] = 8, ["angle"] = 2686976},
  [73] = {["x"] = 4425, ["y"] = -3764, ["z"] = 0, ["order"] = 1, ["angle"] = 1344602112},
  [74] = {["x"] = 4157, ["y"] = -3238, ["z"] = 0, ["order"] = 1, ["angle"] = 775618560},
  [75] = {["x"] = 4574, ["y"] = -2385, ["z"] = 0, ["order"] = 1, ["angle"] = 1401880576},
  [76] = {["x"] = 4436, ["y"] = -1917, ["z"] = 0, ["order"] = 1, ["angle"] = 1889861632},
  [77] = {["x"] = 3786, ["y"] = -1763, ["z"] = 0, ["order"] = 1, ["angle"] = 1625358336},
  [78] = {["x"] = 3313, ["y"] = -1330, ["z"] = 0, ["order"] = 1, ["angle"] = 1098448896},
  [79] = {["x"] = 2878, ["y"] = 1113, ["z"] = 0, ["order"] = 2, ["angle"] = 1863122944},
  [80] = {["x"] = 2093, ["y"] = 1301, ["z"] = 0, ["order"] = 2, ["angle"] = 1440284672},
  [81] = {["x"] = 1216, ["y"] = 1800, ["z"] = 0, ["order"] = 2, ["angle"] = 781123584},
  [82] = {["x"] = 1151, ["y"] = 2428, ["z"] = 0, ["order"] = 2, ["angle"] = 515309568},
  [83] = {["x"] = 1573, ["y"] = 3087, ["z"] = 0, ["order"] = 2, ["angle"] = 169410560},
  [84] = {["x"] = 2249, ["y"] = 3290, ["z"] = 0, ["order"] = 2, ["angle"] = 119209984},
  [85] = {["x"] = 3315, ["y"] = 3479, ["z"] = 0, ["order"] = 2, ["angle"] = 140312576},
  [86] = {["x"] = 3390, ["y"] = 2440, ["z"] = 0, ["order"] = 2, ["angle"] = 164823040},
  [87] = {["x"] = 4068, ["y"] = 2412, ["z"] = 0, ["order"] = 2, ["angle"] = 677445632},
  [88] = {["x"] = 4430, ["y"] = 2807, ["z"] = 0, ["order"] = 2, ["angle"] = 995557376},
  [89] = {["x"] = 4567, ["y"] = 3279, ["z"] = 0, ["order"] = 2, ["angle"] = 1200947200},
  [90] = {["x"] = 981, ["y"] = 4848, ["z"] = 0, ["order"] = 3, ["angle"] = -1458896896},
  [91] = {["x"] = -4619, ["y"] = 3385, ["z"] = 0, ["order"] = 5, ["angle"] = -744816640},
  [92] = {["x"] = -4496, ["y"] = 2844, ["z"] = 0, ["order"] = 5, ["angle"] = -412811264},
  [93] = {["x"] = -4167, ["y"] = 2634, ["z"] = 0, ["order"] = 5, ["angle"] = -116326400},
  [94] = {["x"] = -3650, ["y"] = 2689, ["z"] = 0, ["order"] = 5, ["angle"] = -354746368},
  [95] = {["x"] = -2376, ["y"] = 3577, ["z"] = 0, ["order"] = 5, ["angle"] = -649920512},
  [96] = {["x"] = -1761, ["y"] = 3197, ["z"] = 0, ["order"] = 5, ["angle"] = -971702272},
  [97] = {["x"] = -1262, ["y"] = 2715, ["z"] = 0, ["order"] = 5, ["angle"] = -1437270016},
  [98] = {["x"] = -1038, ["y"] = 2188, ["z"] = 0, ["order"] = 5, ["angle"] = -1884749824},
  [99] = {["x"] = -1745, ["y"] = 1787, ["z"] = 0, ["order"] = 5, ["angle"] = -1668874240},
  [100] = {["x"] = -2441, ["y"] = 1689, ["z"] = 0, ["order"] = 5, ["angle"] = -1498349568},
  [101] = {["x"] = -2788, ["y"] = 1310, ["z"] = 0, ["order"] = 5, ["angle"] = -1403453440},
  [102] = {["x"] = -3213, ["y"] = -1472, ["z"] = 0, ["order"] = 6, ["angle"] = -2036662272},
  [103] = {["x"] = -3844, ["y"] = -1368, ["z"] = 0, ["order"] = 6, ["angle"] = -1689190400},
  [104] = {["x"] = -4472, ["y"] = -1750, ["z"] = 0, ["order"] = 6, ["angle"] = -922025984},
  [105] = {["x"] = -4170, ["y"] = -2419, ["z"] = 0, ["order"] = 6, ["angle"] = -1100546048},
  [106] = {["x"] = -4077, ["y"] = -3108, ["z"] = 0, ["order"] = 6, ["angle"] = -1375141888}
}
-- Respawn definitions for SRB2 FROZEN (MAPHA)
krespawn.points[362] = {
  [0] = 46,
  [1] = {["x"] = 628, ["y"] = -712, ["z"] = 64, ["order"] = 0, ["angle"] = -1345126400},
  [2] = {["x"] = 510, ["y"] = -1571, ["z"] = 64, ["order"] = 0, ["angle"] = -2086338560},
  [3] = {["x"] = -244, ["y"] = -2038, ["z"] = 19, ["order"] = 0, ["angle"] = 1420754944},
  [4] = {["x"] = -609, ["y"] = -1463, ["z"] = 0, ["order"] = 0, ["angle"] = 1289682944},
  [5] = {["x"] = -891, ["y"] = -783, ["z"] = 0, ["order"] = 0, ["angle"] = 1422983168},
  [6] = {["x"] = -1266, ["y"] = -123, ["z"] = 0, ["order"] = 0, ["angle"] = 1429405696},
  [7] = {["x"] = -1921, ["y"] = 922, ["z"] = 0, ["order"] = 0, ["angle"] = 1150353408},
  [8] = {["x"] = -2334, ["y"] = 1939, ["z"] = 0, ["order"] = 0, ["angle"] = 868417536},
  [9] = {["x"] = -2330, ["y"] = 2682, ["z"] = 0, ["order"] = 0, ["angle"] = 763297792},
  [10] = {["x"] = -2006, ["y"] = 3421, ["z"] = 0, ["order"] = 1, ["angle"] = 756219904},
  [11] = {["x"] = -1602, ["y"] = 4356, ["z"] = 0, ["order"] = 1, ["angle"] = 792002560},
  [12] = {["x"] = -1249, ["y"] = 5490, ["z"] = 0, ["order"] = 1, ["angle"] = 418971648},
  [13] = {["x"] = -503, ["y"] = 6467, ["z"] = 128, ["order"] = 1, ["angle"] = 92340224},
  [14] = {["x"] = 43, ["y"] = 6816, ["z"] = 128, ["order"] = 1, ["angle"] = -97058816},
  [15] = {["x"] = 1005, ["y"] = 7061, ["z"] = 128, ["order"] = 2, ["angle"] = -376373248},
  [16] = {["x"] = 1897, ["y"] = 6907, ["z"] = 128, ["order"] = 2, ["angle"] = -644284416},
  [17] = {["x"] = 2811, ["y"] = 6392, ["z"] = 128, ["order"] = 2, ["angle"] = -788463616},
  [18] = {["x"] = 3391, ["y"] = 5782, ["z"] = 64, ["order"] = 2, ["angle"] = -797638656},
  [19] = {["x"] = 3877, ["y"] = 5207, ["z"] = 64, ["order"] = 2, ["angle"] = -1070661632},
  [20] = {["x"] = 3568, ["y"] = 3785, ["z"] = 64, ["order"] = 2, ["angle"] = -813629440},
  [21] = {["x"] = 3670, ["y"] = 2947, ["z"] = 64, ["order"] = 2, ["angle"] = -911802368},
  [22] = {["x"] = 3950, ["y"] = 2198, ["z"] = 64, ["order"] = 2, ["angle"] = -874708992},
  [23] = {["x"] = 4253, ["y"] = 1259, ["z"] = 64, ["order"] = 2, ["angle"] = -953221120},
  [24] = {["x"] = 4530, ["y"] = 313, ["z"] = 64, ["order"] = 2, ["angle"] = -948240384},
  [25] = {["x"] = 4847, ["y"] = -631, ["z"] = 144, ["order"] = 3, ["angle"] = -1183645696},
  [26] = {["x"] = 5194, ["y"] = -1544, ["z"] = 192, ["order"] = 3, ["angle"] = -1571094528},
  [27] = {["x"] = 5091, ["y"] = -2292, ["z"] = 146, ["order"] = 3, ["angle"] = -1958281216},
  [28] = {["x"] = 4536, ["y"] = -3148, ["z"] = 128, ["order"] = 3, ["angle"] = 1830354944},
  [29] = {["x"] = 3607, ["y"] = -3457, ["z"] = 128, ["order"] = 3, ["angle"] = 1524957184},
  [30] = {["x"] = 3610, ["y"] = -2598, ["z"] = 128, ["order"] = 3, ["angle"] = 1573847040},
  [31] = {["x"] = 3008, ["y"] = -2789, ["z"] = 128, ["order"] = 3, ["angle"] = 1700986880},
  [32] = {["x"] = 2245, ["y"] = -2384, ["z"] = 128, ["order"] = 3, ["angle"] = 1289420800},
  [33] = {["x"] = 1934, ["y"] = -1599, ["z"] = 128, ["order"] = 3, ["angle"] = 1067122688},
  [34] = {["x"] = 1868, ["y"] = -639, ["z"] = 142, ["order"] = 3, ["angle"] = 1156907008},
  [35] = {["x"] = 1845, ["y"] = 347, ["z"] = 161, ["order"] = 4, ["angle"] = 1243152384},
  [36] = {["x"] = 1840, ["y"] = 1201, ["z"] = 17, ["order"] = 4, ["angle"] = 1270153216},
  [37] = {["x"] = 1794, ["y"] = 2200, ["z"] = 0, ["order"] = 4, ["angle"] = 1483276288},
  [38] = {["x"] = 1633, ["y"] = 3419, ["z"] = 0, ["order"] = 4, ["angle"] = 2015035392},
  [39] = {["x"] = 1209, ["y"] = 3964, ["z"] = 0, ["order"] = 4, ["angle"] = -1920008192},
  [40] = {["x"] = 200, ["y"] = 4096, ["z"] = 0, ["order"] = 5, ["angle"] = -1484587008},
  [41] = {["x"] = -450, ["y"] = 3603, ["z"] = 0, ["order"] = 5, ["angle"] = -976420864},
  [42] = {["x"] = -507, ["y"] = 2662, ["z"] = 16, ["order"] = 5, ["angle"] = -763166720},
  [43] = {["x"] = -180, ["y"] = 1925, ["z"] = 64, ["order"] = 5, ["angle"] = -777322496},
  [44] = {["x"] = 208, ["y"] = 1155, ["z"] = 64, ["order"] = 5, ["angle"] = -886505472},
  [45] = {["x"] = 609, ["y"] = 206, ["z"] = 64, ["order"] = 5, ["angle"] = -1097793536},
  [46] = {["x"] = 670, ["y"] = -273, ["z"] = 64, ["order"] = 5, ["angle"] = -1214447616}
}
-- Respawn definitions for CNR TOWNSVILLE (MAPHB)
krespawn.points[363] = {
  [0] = 99,
  [1] = {["x"] = 4423, ["y"] = -499, ["z"] = 0, ["order"] = 0, ["angle"] = -16318464},
  [2] = {["x"] = 5472, ["y"] = -299, ["z"] = 0, ["order"] = 0, ["angle"] = -318046208},
  [3] = {["x"] = 6293, ["y"] = -407, ["z"] = 0, ["order"] = 0, ["angle"] = -818610176},
  [4] = {["x"] = 6853, ["y"] = -1019, ["z"] = 0, ["order"] = 0, ["angle"] = -1066074112},
  [5] = {["x"] = 6995, ["y"] = -1852, ["z"] = 0, ["order"] = 0, ["angle"] = -1545142272},
  [6] = {["x"] = 6640, ["y"] = -2662, ["z"] = 0, ["order"] = 0, ["angle"] = -1851195392},
  [7] = {["x"] = 6128, ["y"] = -2424, ["z"] = 0, ["order"] = 0, ["angle"] = -1970470912},
  [8] = {["x"] = 5447, ["y"] = -2557, ["z"] = 0, ["order"] = 0, ["angle"] = -2081882112},
  [9] = {["x"] = 4193, ["y"] = -2539, ["z"] = 0, ["order"] = 0, ["angle"] = -1822490624},
  [10] = {["x"] = 3481, ["y"] = -2660, ["z"] = 0, ["order"] = 0, ["angle"] = -1765343232},
  [11] = {["x"] = 2812, ["y"] = -3079, ["z"] = 0, ["order"] = 0, ["angle"] = -1657470976},
  [12] = {["x"] = 2172, ["y"] = -3520, ["z"] = 0, ["order"] = 0, ["angle"] = -1554579456},
  [13] = {["x"] = 1184, ["y"] = -4254, ["z"] = 0, ["order"] = 0, ["angle"] = -1468071936},
  [14] = {["x"] = 491, ["y"] = -4939, ["z"] = 0, ["order"] = 0, ["angle"] = -1227554816},
  [15] = {["x"] = 285, ["y"] = -5667, ["z"] = 0, ["order"] = 0, ["angle"] = -1055981568},
  [16] = {["x"] = 210, ["y"] = -6809, ["z"] = 0, ["order"] = 0, ["angle"] = -1196359680},
  [17] = {["x"] = 5805, ["y"] = -3184, ["z"] = 0, ["order"] = 0, ["angle"] = -1999437824},
  [18] = {["x"] = 5236, ["y"] = -3375, ["z"] = 0, ["order"] = 0, ["angle"] = -1525743616},
  [19] = {["x"] = 4788, ["y"] = -3960, ["z"] = 0, ["order"] = 0, ["angle"] = -1291780096},
  [20] = {["x"] = 4623, ["y"] = -5007, ["z"] = 0, ["order"] = 0, ["angle"] = -1480261632},
  [21] = {["x"] = 4443, ["y"] = -5953, ["z"] = 0, ["order"] = 0, ["angle"] = -1967718400},
  [22] = {["x"] = 3556, ["y"] = -6106, ["z"] = 0, ["order"] = 0, ["angle"] = -1962606592},
  [23] = {["x"] = 2885, ["y"] = -5311, ["z"] = 0, ["order"] = 0, ["angle"] = -1521942528},
  [24] = {["x"] = 2686, ["y"] = -6133, ["z"] = 0, ["order"] = 0, ["angle"] = -1854603264},
  [25] = {["x"] = 1871, ["y"] = -6022, ["z"] = 0, ["order"] = 0, ["angle"] = -1662582784},
  [26] = {["x"] = 1226, ["y"] = -6413, ["z"] = 0, ["order"] = 0, ["angle"] = -1576468480},
  [27] = {["x"] = 526, ["y"] = -7056, ["z"] = 0, ["order"] = 0, ["angle"] = -1452343296},
  [28] = {["x"] = -149, ["y"] = -7961, ["z"] = -26, ["order"] = 1, ["angle"] = -1511849984},
  [29] = {["x"] = -500, ["y"] = -8690, ["z"] = -150, ["order"] = 1, ["angle"] = -1913061376},
  [30] = {["x"] = -1077, ["y"] = -9233, ["z"] = -150, ["order"] = 1, ["angle"] = -2068512768},
  [31] = {["x"] = -1724, ["y"] = -9782, ["z"] = -150, ["order"] = 1, ["angle"] = 1975058432},
  [32] = {["x"] = -2315, ["y"] = -9819, ["z"] = -150, ["order"] = 1, ["angle"] = 1813839872},
  [33] = {["x"] = -3118, ["y"] = -9699, ["z"] = -150, ["order"] = 1, ["angle"] = 1725628416},
  [34] = {["x"] = -3682, ["y"] = -9315, ["z"] = -150, ["order"] = 1, ["angle"] = 1108803584},
  [35] = {["x"] = -3389, ["y"] = -8494, ["z"] = -160, ["order"] = 1, ["angle"] = 665780224},
  [36] = {["x"] = -2912, ["y"] = -7852, ["z"] = -41, ["order"] = 1, ["angle"] = 739049472},
  [37] = {["x"] = -2563, ["y"] = -7356, ["z"] = 0, ["order"] = 1, ["angle"] = 949026816},
  [38] = {["x"] = -2197, ["y"] = -6538, ["z"] = 0, ["order"] = 1, ["angle"] = 1017577472},
  [39] = {["x"] = -2071, ["y"] = -5686, ["z"] = 0, ["order"] = 1, ["angle"] = 1055195136},
  [40] = {["x"] = -2042, ["y"] = -4642, ["z"] = 114, ["order"] = 1, ["angle"] = 1034092544},
  [41] = {["x"] = -4046, ["y"] = -9096, ["z"] = -150, ["order"] = 1, ["angle"] = 1643708416},
  [42] = {["x"] = -4889, ["y"] = -8176, ["z"] = -150, ["order"] = 1, ["angle"] = 1442643968},
  [43] = {["x"] = -5527, ["y"] = -7333, ["z"] = 0, ["order"] = 1, ["angle"] = 1370947584},
  [44] = {["x"] = -6187, ["y"] = -6520, ["z"] = 0, ["order"] = 1, ["angle"] = 703397888},
  [45] = {["x"] = -6064, ["y"] = -5503, ["z"] = 0, ["order"] = 1, ["angle"] = 307691520},
  [46] = {["x"] = -5498, ["y"] = -5171, ["z"] = 0, ["order"] = 1, ["angle"] = 45285376},
  [47] = {["x"] = -4549, ["y"] = -5260, ["z"] = 0, ["order"] = 1, ["angle"] = 490274816},
  [48] = {["x"] = -3763, ["y"] = -5544, ["z"] = 0, ["order"] = 1, ["angle"] = 55115776},
  [49] = {["x"] = -2795, ["y"] = -5801, ["z"] = 0, ["order"] = 1, ["angle"] = 537591808},
  [50] = {["x"] = -2302, ["y"] = -4428, ["z"] = 150, ["order"] = 1, ["angle"] = 921632768},
  [51] = {["x"] = -2057, ["y"] = -3677, ["z"] = 150, ["order"] = 1, ["angle"] = 1044185088},
  [52] = {["x"] = -2138, ["y"] = -2820, ["z"] = 150, ["order"] = 1, ["angle"] = 1084162048},
  [53] = {["x"] = -2099, ["y"] = -1816, ["z"] = 150, ["order"] = 1, ["angle"] = 1026097152},
  [54] = {["x"] = -2056, ["y"] = -1047, ["z"] = 150, ["order"] = 1, ["angle"] = 1115095040},
  [55] = {["x"] = -2144, ["y"] = -351, ["z"] = 88, ["order"] = 1, ["angle"] = 861601792},
  [56] = {["x"] = -2107, ["y"] = 259, ["z"] = 0, ["order"] = 1, ["angle"] = 673906688},
  [57] = {["x"] = -3945, ["y"] = -4572, ["z"] = 15, ["order"] = 1, ["angle"] = 802488320},
  [58] = {["x"] = -3524, ["y"] = -3392, ["z"] = 150, ["order"] = 1, ["angle"] = 1104740352},
  [59] = {["x"] = -3569, ["y"] = -2306, ["z"] = 150, ["order"] = 1, ["angle"] = 1095565312},
  [60] = {["x"] = -3667, ["y"] = -1482, ["z"] = 150, ["order"] = 1, ["angle"] = 856227840},
  [61] = {["x"] = -3648, ["y"] = -329, ["z"] = 0, ["order"] = 1, ["angle"] = 495910912},
  [62] = {["x"] = -3066, ["y"] = 111, ["z"] = 0, ["order"] = 1, ["angle"] = 484245504},
  [63] = {["x"] = -1874, ["y"] = 833, ["z"] = 0, ["order"] = 2, ["angle"] = 661192704},
  [64] = {["x"] = -1479, ["y"] = 1632, ["z"] = 0, ["order"] = 2, ["angle"] = 575078400},
  [65] = {["x"] = -1001, ["y"] = 2091, ["z"] = 0, ["order"] = 2, ["angle"] = 97845248},
  [66] = {["x"] = 229, ["y"] = 2246, ["z"] = 51, ["order"] = 2, ["angle"] = 233373696},
  [67] = {["x"] = 1350, ["y"] = 2816, ["z"] = 119, ["order"] = 2, ["angle"] = 702611456},
  [68] = {["x"] = 2173, ["y"] = 3766, ["z"] = 148, ["order"] = 2, ["angle"] = 851247104},
  [69] = {["x"] = 2473, ["y"] = 4648, ["z"] = 172, ["order"] = 2, ["angle"] = 1203699712},
  [70] = {["x"] = 2236, ["y"] = 5697, ["z"] = 79, ["order"] = 2, ["angle"] = 1691549696},
  [71] = {["x"] = 1675, ["y"] = 6285, ["z"] = 128, ["order"] = 3, ["angle"] = -2040070144},
  [72] = {["x"] = 885, ["y"] = 6388, ["z"] = 101, ["order"] = 3, ["angle"] = -1897988096},
  [73] = {["x"] = -71, ["y"] = 6200, ["z"] = 96, ["order"] = 3, ["angle"] = -1706360832},
  [74] = {["x"] = -792, ["y"] = 5834, ["z"] = 100, ["order"] = 3, ["angle"] = -1587478528},
  [75] = {["x"] = -855, ["y"] = 4929, ["z"] = 0, ["order"] = 3, ["angle"] = -1255342080},
  [76] = {["x"] = -867, ["y"] = 4462, ["z"] = 0, ["order"] = 3, ["angle"] = -1299251200},
  [77] = {["x"] = -1044, ["y"] = 3781, ["z"] = 0, ["order"] = 3, ["angle"] = -1467416576},
  [78] = {["x"] = -1332, ["y"] = 3062, ["z"] = 0, ["order"] = 4, ["angle"] = -1553661952},
  [79] = {["x"] = -1635, ["y"] = 2515, ["z"] = 0, ["order"] = 4, ["angle"] = -1556545536},
  [80] = {["x"] = -2063, ["y"] = 1977, ["z"] = 0, ["order"] = 4, ["angle"] = -1600847872},
  [81] = {["x"] = -2549, ["y"] = 1390, ["z"] = 0, ["order"] = 4, ["angle"] = -1957756928},
  [82] = {["x"] = -3239, ["y"] = 1096, ["z"] = 0, ["order"] = 4, ["angle"] = 2147024896},
  [83] = {["x"] = -4129, ["y"] = 1172, ["z"] = 0, ["order"] = 4, ["angle"] = 2115960832},
  [84] = {["x"] = -5142, ["y"] = 1067, ["z"] = 0, ["order"] = 5, ["angle"] = -1619066880},
  [85] = {["x"] = -5803, ["y"] = 354, ["z"] = 0, ["order"] = 5, ["angle"] = -768671744},
  [86] = {["x"] = -5700, ["y"] = -335, ["z"] = 0, ["order"] = 5, ["angle"] = -439287808},
  [87] = {["x"] = -5297, ["y"] = -841, ["z"] = 0, ["order"] = 5, ["angle"] = -529989632},
  [88] = {["x"] = -4372, ["y"] = -1800, ["z"] = -230, ["order"] = 5, ["angle"] = -563019776},
  [89] = {["x"] = -3693, ["y"] = -2553, ["z"] = -230, ["order"] = 5, ["angle"] = -565772288},
  [90] = {["x"] = -2868, ["y"] = -3514, ["z"] = -230, ["order"] = 5, ["angle"] = -161284096},
  [91] = {["x"] = -2242, ["y"] = -4073, ["z"] = -230, ["order"] = 5, ["angle"] = 386859008},
  [92] = {["x"] = -1101, ["y"] = -3956, ["z"] = -230, ["order"] = 5, ["angle"] = 726597632},
  [93] = {["x"] = -279, ["y"] = -2941, ["z"] = -230, ["order"] = 5, ["angle"] = 747700224},
  [94] = {["x"] = 147, ["y"] = -2258, ["z"] = -87, ["order"] = 5, ["angle"] = 672989184},
  [95] = {["x"] = 602, ["y"] = -1527, ["z"] = 0, ["order"] = 5, ["angle"] = 320929792},
  [96] = {["x"] = 1535, ["y"] = -809, ["z"] = 0, ["order"] = 5, ["angle"] = 97320960},
  [97] = {["x"] = 2498, ["y"] = -589, ["z"] = 0, ["order"] = 5, ["angle"] = 79757312},
  [98] = {["x"] = 3477, ["y"] = -494, ["z"] = 0, ["order"] = 5, ["angle"] = 327680},
  [99] = {["x"] = 3994, ["y"] = -475, ["z"] = 0, ["order"] = 5, ["angle"] = -41353216}
}
-- Respawn definitions for FZ SILENCE (MAPHC)
krespawn.points[364] = {
  [0] = 108,
  [1] = {["x"] = 2850, ["y"] = -2165, ["z"] = 0, ["order"] = 0, ["angle"] = 976814080},
  [2] = {["x"] = 2927, ["y"] = -1723, ["z"] = 0, ["order"] = 0, ["angle"] = 970915840},
  [3] = {["x"] = 2962, ["y"] = -1526, ["z"] = 0, ["order"] = 0, ["angle"] = 1045626880},
  [4] = {["x"] = 2980, ["y"] = -1280, ["z"] = 0, ["order"] = 0, ["angle"] = 1081409536},
  [5] = {["x"] = 2988, ["y"] = -1042, ["z"] = 0, ["order"] = 0, ["angle"] = 1103691776},
  [6] = {["x"] = 2990, ["y"] = -815, ["z"] = 0, ["order"] = 0, ["angle"] = 1103691776},
  [7] = {["x"] = 2987, ["y"] = -510, ["z"] = 0, ["order"] = 0, ["angle"] = 1103691776},
  [8] = {["x"] = 2981, ["y"] = -203, ["z"] = 0, ["order"] = 0, ["angle"] = 1103691776},
  [9] = {["x"] = 2974, ["y"] = 70, ["z"] = 0, ["order"] = 0, ["angle"] = 1103691776},
  [10] = {["x"] = 2962, ["y"] = 377, ["z"] = 0, ["order"] = 0, ["angle"] = 1240662016},
  [11] = {["x"] = 2935, ["y"] = 649, ["z"] = 0, ["order"] = 0, ["angle"] = 1259143168},
  [12] = {["x"] = 2889, ["y"] = 952, ["z"] = 0, ["order"] = 0, ["angle"] = 1259143168},
  [13] = {["x"] = 2838, ["y"] = 1221, ["z"] = 0, ["order"] = 0, ["angle"] = 1259143168},
  [14] = {["x"] = 2780, ["y"] = 1488, ["z"] = 0, ["order"] = 0, ["angle"] = 1259143168},
  [15] = {["x"] = 2734, ["y"] = 1688, ["z"] = 0, ["order"] = 0, ["angle"] = 1259143168},
  [16] = {["x"] = 2781, ["y"] = 2024, ["z"] = 0, ["order"] = 2, ["angle"] = -2034958336},
  [17] = {["x"] = 2547, ["y"] = 1914, ["z"] = 0, ["order"] = 2, ["angle"] = 2103115776},
  [18] = {["x"] = 2201, ["y"] = 1873, ["z"] = 0, ["order"] = 2, ["angle"] = 2025521152},
  [19] = {["x"] = 1760, ["y"] = 1862, ["z"] = 0, ["order"] = 2, ["angle"] = 1919090688},
  [20] = {["x"] = 1494, ["y"] = 1924, ["z"] = 0, ["order"] = 3, ["angle"] = 1637679104},
  [21] = {["x"] = 1287, ["y"] = 2073, ["z"] = 0, ["order"] = 3, ["angle"] = 1530593280},
  [22] = {["x"] = 1076, ["y"] = 2274, ["z"] = 0, ["order"] = 3, ["angle"] = 1519845376},
  [23] = {["x"] = 833, ["y"] = 2523, ["z"] = 0, ["order"] = 3, ["angle"] = 1585119232},
  [24] = {["x"] = 589, ["y"] = 2773, ["z"] = 0, ["order"] = 3, ["angle"] = 1615134720},
  [25] = {["x"] = 272, ["y"] = 3092, ["z"] = 0, ["order"] = 3, ["angle"] = 1615134720},
  [26] = {["x"] = -38, ["y"] = 3401, ["z"] = 0, ["order"] = 3, ["angle"] = 1615134720},
  [27] = {["x"] = -339, ["y"] = 3718, ["z"] = 0, ["order"] = 3, ["angle"] = 1585905664},
  [28] = {["x"] = -564, ["y"] = 3973, ["z"] = 0, ["order"] = 3, ["angle"] = 1672937472},
  [29] = {["x"] = -797, ["y"] = 4268, ["z"] = 0, ["order"] = 3, ["angle"] = 1732706304},
  [30] = {["x"] = -1105, ["y"] = 4588, ["z"] = 0, ["order"] = 3, ["angle"] = 1777795072},
  [31] = {["x"] = -1389, ["y"] = 4868, ["z"] = 0, ["order"] = 3, ["angle"] = 1951072256},
  [32] = {["x"] = -1707, ["y"] = 5078, ["z"] = 0, ["order"] = 3, ["angle"] = -1939144704},
  [33] = {["x"] = -2111, ["y"] = 5119, ["z"] = 0, ["order"] = 4, ["angle"] = -1693253632},
  [34] = {["x"] = -2436, ["y"] = 5013, ["z"] = 0, ["order"] = 4, ["angle"] = -1402142720},
  [35] = {["x"] = -2670, ["y"] = 4798, ["z"] = 0, ["order"] = 4, ["angle"] = -1234501632},
  [36] = {["x"] = -2812, ["y"] = 4526, ["z"] = 0, ["order"] = 4, ["angle"] = -1185480704},
  [37] = {["x"] = -2793, ["y"] = 4055, ["z"] = 0, ["order"] = 4, ["angle"] = -1000669184},
  [38] = {["x"] = -2864, ["y"] = 2966, ["z"] = 0, ["order"] = 6, ["angle"] = -1012203520},
  [39] = {["x"] = -2909, ["y"] = 2678, ["z"] = 0, ["order"] = 6, ["angle"] = -895811584},
  [40] = {["x"] = -2933, ["y"] = 2374, ["z"] = 0, ["order"] = 6, ["angle"] = -766705664},
  [41] = {["x"] = -2894, ["y"] = 2036, ["z"] = 0, ["order"] = 6, ["angle"] = -157614080},
  [42] = {["x"] = -2679, ["y"] = 1920, ["z"] = 0, ["order"] = 7, ["angle"] = 47775744},
  [43] = {["x"] = -2374, ["y"] = 1989, ["z"] = 0, ["order"] = 7, ["angle"] = -37158912},
  [44] = {["x"] = -2126, ["y"] = 2042, ["z"] = 0, ["order"] = 7, ["angle"] = -192086016},
  [45] = {["x"] = -1749, ["y"] = 2068, ["z"] = 0, ["order"] = 7, ["angle"] = -141361152},
  [46] = {["x"] = -1397, ["y"] = 2055, ["z"] = 0, ["order"] = 7, ["angle"] = -438370304},
  [47] = {["x"] = -1224, ["y"] = 1972, ["z"] = 0, ["order"] = 8, ["angle"] = -635764736},
  [48] = {["x"] = -1147, ["y"] = 1736, ["z"] = 0, ["order"] = 8, ["angle"] = -596312064},
  [49] = {["x"] = -1006, ["y"] = 1443, ["z"] = 0, ["order"] = 9, ["angle"] = -138477568},
  [50] = {["x"] = -732, ["y"] = 1389, ["z"] = 0, ["order"] = 10, ["angle"] = 80543744},
  [51] = {["x"] = -357, ["y"] = 1493, ["z"] = 0, ["order"] = 10, ["angle"] = -196804608},
  [52] = {["x"] = -26, ["y"] = 1502, ["z"] = 0, ["order"] = 10, ["angle"] = -527106048},
  [53] = {["x"] = 221, ["y"] = 1380, ["z"] = 0, ["order"] = 11, ["angle"] = -779681792},
  [54] = {["x"] = 342, ["y"] = 1116, ["z"] = 0, ["order"] = 12, ["angle"] = -1078788096},
  [55] = {["x"] = 385, ["y"] = 652, ["z"] = 0, ["order"] = 12, ["angle"] = -1173946368},
  [56] = {["x"] = 327, ["y"] = 248, ["z"] = 0, ["order"] = 12, ["angle"] = -1094254592},
  [57] = {["x"] = 295, ["y"] = -6, ["z"] = 0, ["order"] = 12, ["angle"] = -1094254592},
  [58] = {["x"] = 272, ["y"] = -312, ["z"] = 0, ["order"] = 12, ["angle"] = -1045626880},
  [59] = {["x"] = 257, ["y"] = -714, ["z"] = 0, ["order"] = 12, ["angle"] = -630259712},
  [60] = {["x"] = 445, ["y"] = -1044, ["z"] = 0, ["order"] = 13, ["angle"] = 413335552},
  [61] = {["x"] = 635, ["y"] = -1015, ["z"] = 0, ["order"] = 14, ["angle"] = 529072128},
  [62] = {["x"] = 790, ["y"] = -709, ["z"] = 0, ["order"] = 15, ["angle"] = 285802496},
  [63] = {["x"] = 1118, ["y"] = -548, ["z"] = 0, ["order"] = 16, ["angle"] = -511246336},
  [64] = {["x"] = 1232, ["y"] = -602, ["z"] = 0, ["order"] = 16, ["angle"] = -861732864},
  [65] = {["x"] = 1341, ["y"] = -912, ["z"] = 0, ["order"] = 17, ["angle"] = -1108410368},
  [66] = {["x"] = 1334, ["y"] = -1311, ["z"] = 0, ["order"] = 17, ["angle"] = -1108410368},
  [67] = {["x"] = 1329, ["y"] = -1705, ["z"] = 0, ["order"] = 17, ["angle"] = -1057554432},
  [68] = {["x"] = 1336, ["y"] = -2229, ["z"] = 0, ["order"] = 17, ["angle"] = -1057554432},
  [69] = {["x"] = 1345, ["y"] = -2661, ["z"] = 0, ["order"] = 17, ["angle"] = -1057554432},
  [70] = {["x"] = 1355, ["y"] = -3139, ["z"] = 0, ["order"] = 17, ["angle"] = -1057554432},
  [71] = {["x"] = 1364, ["y"] = -3578, ["z"] = 0, ["order"] = 17, ["angle"] = -1092026368},
  [72] = {["x"] = 1353, ["y"] = -4032, ["z"] = 0, ["order"] = 17, ["angle"] = -1191903232},
  [73] = {["x"] = 1307, ["y"] = -4471, ["z"] = 0, ["order"] = 17, ["angle"] = -1061486592},
  [74] = {["x"] = 1266, ["y"] = -4864, ["z"] = 0, ["order"] = 18, ["angle"] = -181862400},
  [75] = {["x"] = 1531, ["y"] = -4954, ["z"] = 0, ["order"] = 18, ["angle"] = -268500992},
  [76] = {["x"] = 1657, ["y"] = -4996, ["z"] = 0, ["order"] = 19, ["angle"] = -268500992},
  [77] = {["x"] = 1767, ["y"] = -5091, ["z"] = 0, ["order"] = 19, ["angle"] = -545587200},
  [78] = {["x"] = 1999, ["y"] = -5317, ["z"] = 0, ["order"] = 19, ["angle"] = 323289088},
  [79] = {["x"] = 2237, ["y"] = -5263, ["z"] = 0, ["order"] = 20, ["angle"] = 905773056},
  [80] = {["x"] = 2321, ["y"] = -5051, ["z"] = 0, ["order"] = 21, ["angle"] = 890175488},
  [81] = {["x"] = 2265, ["y"] = -4653, ["z"] = 0, ["order"] = 21, ["angle"] = 832897024},
  [82] = {["x"] = 2331, ["y"] = -4378, ["z"] = 0, ["order"] = 22, ["angle"] = 501940224},
  [83] = {["x"] = 2374, ["y"] = -4284, ["z"] = 0, ["order"] = 22, ["angle"] = 433651712},
  [84] = {["x"] = 2568, ["y"] = -4281, ["z"] = 0, ["order"] = 23, ["angle"] = 569442304},
  [85] = {["x"] = 2714, ["y"] = -4235, ["z"] = 0, ["order"] = 23, ["angle"] = 1012334592},
  [86] = {["x"] = 2762, ["y"] = -3985, ["z"] = 0, ["order"] = 24, ["angle"] = 1051131904},
  [87] = {["x"] = 2775, ["y"] = -3753, ["z"] = 0, ["order"] = 24, ["angle"] = 1126629376},
  [88] = {["x"] = 2768, ["y"] = -3421, ["z"] = 0, ["order"] = 24, ["angle"] = 1088225280},
  [89] = {["x"] = 2762, ["y"] = -3116, ["z"] = 0, ["order"] = 24, ["angle"] = 1057161216},
  [90] = {["x"] = 2768, ["y"] = -2795, ["z"] = 0, ["order"] = 24, ["angle"] = 1011810304},
  [91] = {["x"] = 2803, ["y"] = -2398, ["z"] = 0, ["order"] = 24, ["angle"] = 1008533504},
  [92] = {["x"] = 2837, ["y"] = 1743, ["z"] = 0, ["order"] = 1, ["angle"] = 1735327744},
  [93] = {["x"] = -910, ["y"] = 4026, ["z"] = 0, ["order"] = 3, ["angle"] = 2137718784},
  [94] = {["x"] = -1185, ["y"] = 4033, ["z"] = 0, ["order"] = 3, ["angle"] = -2122121216},
  [95] = {["x"] = -1527, ["y"] = 4025, ["z"] = 0, ["order"] = 3, ["angle"] = -2141650944},
  [96] = {["x"] = -1792, ["y"] = 4021, ["z"] = 0, ["order"] = 3, ["angle"] = -2141650944},
  [97] = {["x"] = -2086, ["y"] = 4020, ["z"] = 0, ["order"] = 4, ["angle"] = 2123825152},
  [98] = {["x"] = -2272, ["y"] = 4025, ["z"] = 0, ["order"] = 4, ["angle"] = 2123825152},
  [99] = {["x"] = -2497, ["y"] = 4034, ["z"] = 0, ["order"] = 4, ["angle"] = 2133000192},
  [100] = {["x"] = -2777, ["y"] = 3907, ["z"] = 0, ["order"] = 4, ["angle"] = -1194393600},
  [101] = {["x"] = -612, ["y"] = 1445, ["z"] = 0, ["order"] = 10, ["angle"] = -71892992},
  [102] = {["x"] = 340, ["y"] = 1311, ["z"] = 0, ["order"] = 11, ["angle"] = -1017315328},
  [103] = {["x"] = 342, ["y"] = 917, ["z"] = 0, ["order"] = 12, ["angle"] = -1087569920},
  [104] = {["x"] = 225, ["y"] = -862, ["z"] = 0, ["order"] = 13, ["angle"] = -298123264},
  [105] = {["x"] = 213, ["y"] = -1167, ["z"] = 0, ["order"] = 13, ["angle"] = 433782784},
  [106] = {["x"] = 621, ["y"] = -1199, ["z"] = 0, ["order"] = 14, ["angle"] = 634322944},
  [107] = {["x"] = 753, ["y"] = -833, ["z"] = 0, ["order"] = 15, ["angle"] = 481886208},
  [108] = {["x"] = 1067, ["y"] = -552, ["z"] = 0, ["order"] = 16, ["angle"] = -495648768}
}
-- Respawn definitions for SMK RAINBOW (MAPHD)
krespawn.points[365] = {
  [0] = 55,
  [1] = {["x"] = 327, ["y"] = -3329, ["z"] = 44, ["order"] = 0, ["angle"] = 1054539776},
  [2] = {["x"] = 312, ["y"] = -2481, ["z"] = 44, ["order"] = 0, ["angle"] = 944439296},
  [3] = {["x"] = 261, ["y"] = -1461, ["z"] = 48, ["order"] = 1, ["angle"] = 609157120},
  [4] = {["x"] = 277, ["y"] = -813, ["z"] = 44, ["order"] = 1, ["angle"] = 222494720},
  [5] = {["x"] = 690, ["y"] = -318, ["z"] = 44, ["order"] = 1, ["angle"] = -91291648},
  [6] = {["x"] = 1423, ["y"] = -278, ["z"] = 44, ["order"] = 1, ["angle"] = -3604480},
  [7] = {["x"] = 2308, ["y"] = -279, ["z"] = 44, ["order"] = 2, ["angle"] = -81854464},
  [8] = {["x"] = 3197, ["y"] = -312, ["z"] = 44, ["order"] = 2, ["angle"] = 17235968},
  [9] = {["x"] = 3860, ["y"] = -314, ["z"] = 44, ["order"] = 2, ["angle"] = -3211264},
  [10] = {["x"] = 4418, ["y"] = -318, ["z"] = 44, ["order"] = 3, ["angle"] = -3211264},
  [11] = {["x"] = 5007, ["y"] = -79, ["z"] = 48, ["order"] = 3, ["angle"] = -459735040},
  [12] = {["x"] = 5655, ["y"] = -136, ["z"] = 44, ["order"] = 3, ["angle"] = -873398272},
  [13] = {["x"] = 6114, ["y"] = -552, ["z"] = 44, ["order"] = 3, ["angle"] = -1100939264},
  [14] = {["x"] = 6176, ["y"] = -1447, ["z"] = 44, ["order"] = 4, ["angle"] = -1221132288},
  [15] = {["x"] = 6258, ["y"] = -2104, ["z"] = 44, ["order"] = 4, ["angle"] = -1408434176},
  [16] = {["x"] = 6404, ["y"] = -2659, ["z"] = 48, ["order"] = 4, ["angle"] = -1782644736},
  [17] = {["x"] = 6246, ["y"] = -3316, ["z"] = 44, ["order"] = 4, ["angle"] = -2106130432},
  [18] = {["x"] = 5320, ["y"] = -3304, ["z"] = 44, ["order"] = 4, ["angle"] = 2072838144},
  [19] = {["x"] = 4363, ["y"] = -3199, ["z"] = 44, ["order"] = 5, ["angle"] = -1960378368},
  [20] = {["x"] = 3580, ["y"] = -3128, ["z"] = 48, ["order"] = 5, ["angle"] = -1635844096},
  [21] = {["x"] = 2769, ["y"] = -3168, ["z"] = 44, ["order"] = 5, ["angle"] = -1093337088},
  [22] = {["x"] = 2300, ["y"] = -3644, ["z"] = 44, ["order"] = 5, ["angle"] = -859504640},
  [23] = {["x"] = 2213, ["y"] = -4286, ["z"] = 44, ["order"] = 5, ["angle"] = -573767680},
  [24] = {["x"] = 2288, ["y"] = -4927, ["z"] = 44, ["order"] = 5, ["angle"] = -304414720},
  [25] = {["x"] = 2393, ["y"] = -5321, ["z"] = 44, ["order"] = 5, ["angle"] = -24444928},
  [26] = {["x"] = 3145, ["y"] = -5271, ["z"] = 44, ["order"] = 5, ["angle"] = 65732608},
  [27] = {["x"] = 3932, ["y"] = -5227, ["z"] = 44, ["order"] = 6, ["angle"] = -50528256},
  [28] = {["x"] = 4757, ["y"] = -5255, ["z"] = 44, ["order"] = 6, ["angle"] = -8454144},
  [29] = {["x"] = 5473, ["y"] = -5360, ["z"] = 44, ["order"] = 6, ["angle"] = 93257728},
  [30] = {["x"] = 6186, ["y"] = -5085, ["z"] = 44, ["order"] = 6, ["angle"] = -85131264},
  [31] = {["x"] = 6912, ["y"] = -5047, ["z"] = 48, ["order"] = 6, ["angle"] = -452132864},
  [32] = {["x"] = 7550, ["y"] = -5150, ["z"] = 44, ["order"] = 6, ["angle"] = -994639872},
  [33] = {["x"] = 7813, ["y"] = -5669, ["z"] = 44, ["order"] = 6, ["angle"] = -1226768384},
  [34] = {["x"] = 7945, ["y"] = -6360, ["z"] = 48, ["order"] = 7, ["angle"] = -1439891456},
  [35] = {["x"] = 7939, ["y"] = -6888, ["z"] = 48, ["order"] = 7, ["angle"] = -1829699584},
  [36] = {["x"] = 7877, ["y"] = -7396, ["z"] = 44, ["order"] = 7, ["angle"] = -2112815104},
  [37] = {["x"] = 7313, ["y"] = -7468, ["z"] = 44, ["order"] = 7, ["angle"] = 2120679424},
  [38] = {["x"] = 6494, ["y"] = -7486, ["z"] = 44, ["order"] = 8, ["angle"] = 2103640064},
  [39] = {["x"] = 5514, ["y"] = -7455, ["z"] = 44, ["order"] = 8, ["angle"] = 2103640064},
  [40] = {["x"] = 4351, ["y"] = -7410, ["z"] = 44, ["order"] = 9, ["angle"] = 1983184896},
  [41] = {["x"] = 4354, ["y"] = -7492, ["z"] = 44, ["order"] = 9, ["angle"] = -2021064704},
  [42] = {["x"] = 3472, ["y"] = -7151, ["z"] = 44, ["order"] = 9, ["angle"] = -2087649280},
  [43] = {["x"] = 2616, ["y"] = -7258, ["z"] = 44, ["order"] = 9, ["angle"] = -2066415616},
  [44] = {["x"] = 1880, ["y"] = -7534, ["z"] = 44, ["order"] = 9, ["angle"] = 1961426944},
  [45] = {["x"] = 1304, ["y"] = -7640, ["z"] = 44, ["order"] = 9, ["angle"] = 1534525440},
  [46] = {["x"] = 573, ["y"] = -7255, ["z"] = 44, ["order"] = 9, ["angle"] = 1227423744},
  [47] = {["x"] = 740, ["y"] = -6211, ["z"] = 44, ["order"] = 10, ["angle"] = 1127677952},
  [48] = {["x"] = 290, ["y"] = -6206, ["z"] = 44, ["order"] = 10, ["angle"] = 1060175872},
  [49] = {["x"] = 502, ["y"] = -5425, ["z"] = 44, ["order"] = 10, ["angle"] = 1106313216},
  [50] = {["x"] = 505, ["y"] = -4959, ["z"] = 44, ["order"] = 10, ["angle"] = 1065549824},
  [51] = {["x"] = 505, ["y"] = -3858, ["z"] = 44, ["order"] = 10, ["angle"] = 1202520064},
  [52] = {["x"] = 6245, ["y"] = -2010, ["z"] = 44, ["order"] = 4, ["angle"] = -1119027200},
  [53] = {["x"] = 6242, ["y"] = -2064, ["z"] = 44, ["order"] = 4, ["angle"] = -1119027200},
  [54] = {["x"] = 3664, ["y"] = -7687, ["z"] = 44, ["order"] = 9, ["angle"] = -2086338560},
  [55] = {["x"] = 2409, ["y"] = -7828, ["z"] = 48, ["order"] = 9, ["angle"] = 1843462144}
}
--No more levels! Finally, woo!